<!DOCTYPE html>
<html>
    <head>
        <title>calibre</title>
        <meta charset="utf-8">
        <meta name="robots" content="noindex">
        <meta name="viewport" content="width=device-width, initial-scale=1">
        <meta name="apple-mobile-web-app-capable" content="yes">
        <meta name="mobile-web-app-capable" content="yes">
        <meta http-equiv="X-UA-Compatible" content="IE=edge">
        <link rel="icon" sizes="256x256" type="image/png" href="favicon.png">
        <link rel="apple-touch-icon" sizes="256x256" href="apple-touch-icon.png">
    </head>
    <body>
        <div id="page_load_progress">
            <progress>
            </progress>
            <div>Loading, please wait&hellip;</div>
            <div>You can also use a simple, non-JavaScript version of this site: <a href="mobile">here</a></div>
            <style type="text/css">
#page_load_progress {
    position:relative;
    top: 50px;
    margin-left: auto; margin-right: auto;
    text-align: center;
}
            </style>
        </div>
<!-- reset styles  {{{ -->
        <style type="text/css">
RESET_STYLES
        </style>
<!-- }}} -->

<!-- SVG icon definitions {{{ -->
ICONS
<!-- }}} -->

<!-- main.js {{{ -->
    <script id="main_js">MAIN_JS</script>
<!-- }}} -->
    </body>
</html>
