# Form implementation generated from reading ui file '/home/kovid/work/calibre/src/calibre/gui2/preferences/look_feel_tabs/tb_hierarchy.ui'
#
# Created by: PyQt6 UI code generator 6.8.1
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        self.gridlayout = QtWidgets.QGridLayout(Form)
        self.gridlayout.setObjectName("gridlayout")
        self.label = QtWidgets.QLabel(parent=Form)
        self.label.setObjectName("label")
        self.gridlayout.addWidget(self.label, 0, 0, 1, 1)
        self.tb_hierarchical_cats = QtWidgets.QListView(parent=Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(1)
        sizePolicy.setHeightForWidth(self.tb_hierarchical_cats.sizePolicy().hasHeightForWidth())
        self.tb_hierarchical_cats.setSizePolicy(sizePolicy)
        self.tb_hierarchical_cats.setMinimumSize(QtCore.QSize(0, 200))
        self.tb_hierarchical_cats.setAlternatingRowColors(True)
        self.tb_hierarchical_cats.setObjectName("tb_hierarchical_cats")
        self.gridlayout.addWidget(self.tb_hierarchical_cats, 1, 0, 3, 1)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.tb_hierarchy_reset_layout_button = QtWidgets.QPushButton(parent=Form)
        self.tb_hierarchy_reset_layout_button.setObjectName("tb_hierarchy_reset_layout_button")
        self.horizontalLayout_4.addWidget(self.tb_hierarchy_reset_layout_button)
        self.tb_hierarchy_import_layout_button = QtWidgets.QPushButton(parent=Form)
        self.tb_hierarchy_import_layout_button.setObjectName("tb_hierarchy_import_layout_button")
        self.horizontalLayout_4.addWidget(self.tb_hierarchy_import_layout_button)
        self.tb_hierarchy_export_layout_button = QtWidgets.QPushButton(parent=Form)
        self.tb_hierarchy_export_layout_button.setObjectName("tb_hierarchy_export_layout_button")
        self.horizontalLayout_4.addWidget(self.tb_hierarchy_export_layout_button)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem)
        self.gridlayout.addLayout(self.horizontalLayout_4, 4, 0, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(0, 20, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.gridlayout.addItem(spacerItem1, 10, 0, 1, 1)
        self.label1 = QtWidgets.QLabel(parent=Form)
        self.label1.setObjectName("label1")
        self.gridlayout.addWidget(self.label1, 11, 0, 1, 1)
        self.tb_search_order = ListWidgetWithMoveByKeyPress(parent=Form)
        self.tb_search_order.setSizeAdjustPolicy(QtWidgets.QAbstractScrollArea.SizeAdjustPolicy.AdjustToContents)
        self.tb_search_order.setObjectName("tb_search_order")
        self.gridlayout.addWidget(self.tb_search_order, 12, 0, 3, 1)
        self.tb_search_order_up_button = QtWidgets.QToolButton(parent=Form)
        icon = QtGui.QIcon.ic("arrow-up.png")
        self.tb_search_order_up_button.setIcon(icon)
        self.tb_search_order_up_button.setObjectName("tb_search_order_up_button")
        self.gridlayout.addWidget(self.tb_search_order_up_button, 12, 1, 1, 1)
        spacerItem2 = QtWidgets.QSpacerItem(1, 1, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.gridlayout.addItem(spacerItem2, 13, 1, 1, 1)
        self.tb_search_order_down_button = QtWidgets.QToolButton(parent=Form)
        icon1 = QtGui.QIcon.ic("arrow-down.png")
        self.tb_search_order_down_button.setIcon(icon1)
        self.tb_search_order_down_button.setObjectName("tb_search_order_down_button")
        self.gridlayout.addWidget(self.tb_search_order_down_button, 14, 1, 1, 1)
        self.horizontalLayout_41 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_41.setObjectName("horizontalLayout_41")
        self.tb_search_order_reset_button = QtWidgets.QPushButton(parent=Form)
        self.tb_search_order_reset_button.setObjectName("tb_search_order_reset_button")
        self.horizontalLayout_41.addWidget(self.tb_search_order_reset_button)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout_41.addItem(spacerItem3)
        self.gridlayout.addLayout(self.horizontalLayout_41, 15, 0, 1, 1)
        self.label.setBuddy(self.tb_hierarchical_cats)
        self.label1.setBuddy(self.tb_search_order)

        self.retranslateUi(Form)

    def retranslateUi(self, Form):

        self.label.setToolTip(_("<p>Check the box for an item if it is to be displayed as a\n"
"hierarchical tree in the Tag browser. For example, if you check\n"
"\'tags\' then tags of the form \'Mystery.English\'\n"
"and \'Mystery.Thriller\' will be displayed with English and Thriller\n"
"both under \'Mystery\'. If \'tags\' is not checked\n"
"then the tags will be displayed each on their own line.</p>\n"
"<p>The categories \'authors\', \'publisher\', \'news\', \'formats\', and \'rating\'\n"
"cannot be hierarchical.</p>\n"
"<p>User categories are always hierarchical and so do not appear in this list.</p>"))
        self.label.setText(_("Select categories with &hierarchical items:"))
        self.tb_hierarchy_reset_layout_button.setToolTip(_("Click this button to reset the list to its default order."))
        self.tb_hierarchy_reset_layout_button.setText(_("Reset list"))
        self.tb_hierarchy_import_layout_button.setToolTip(_("<p>Click this button to set the list to one\n"
"previously exported. This could be useful if you have several libraries with\n"
"similar structure and you want to use the same for each one.</p>"))
        self.tb_hierarchy_import_layout_button.setText(_("Import list"))
        self.tb_hierarchy_export_layout_button.setToolTip(_("<p>Click this button to write the current display\n"
"settings to a file. This could be useful if you have several libraries with similar\n"
"structure and you want to use the same for each one.</p>"))
        self.tb_hierarchy_export_layout_button.setText(_("Export list"))
        self.label1.setToolTip(_("<p>Set the order of the searches when clicking on an item in\n"
"the Tag browser. The \'or its children\' options are ignored when clicking on\n"
"top-level categories, items that aren\'t in a hierarchical category, and items\n"
"that don\'t have children.</p>"))
        self.label1.setText(_("Set the &order of searches when clicking on items"))
        self.tb_search_order_up_button.setToolTip(_("Move up. Keyboard shortcut: Ctrl-Up arrow"))
        self.tb_search_order_down_button.setToolTip(_("Move down. Keyboard shortcut: Ctrl-Down arrow"))
        self.tb_search_order_reset_button.setToolTip(_("Click this button to reset the list to its default order."))
        self.tb_search_order_reset_button.setText(_("Reset list"))
from calibre.gui2.preferences import ListWidgetWithMoveByKeyPress
