# Form implementation generated from reading ui file '/home/kovid/work/calibre/src/calibre/gui2/preferences/look_feel_tabs/tb_quickview.ui'
#
# Created by: PyQt6 UI code generator 6.8.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_Form(object):
    def setupUi(self, cover_browser_tab):
        cover_browser_tab.setObjectName("cover_browser_tab")
        self.gridLayout_122 = QtWidgets.QGridLayout(cover_browser_tab)
        self.gridLayout_122.setObjectName("gridLayout_122")
        self.gridLayout_1221 = QtWidgets.QGridLayout()
        self.gridLayout_1221.setObjectName("gridLayout_1221")
        self.opt_qv_respects_vls = QtWidgets.QCheckBox(parent=cover_browser_tab)
        self.opt_qv_respects_vls.setObjectName("opt_qv_respects_vls")
        self.gridLayout_1221.addWidget(self.opt_qv_respects_vls, 2, 0, 1, 1)
        self.opt_qv_follows_column = QtWidgets.QCheckBox(parent=cover_browser_tab)
        self.opt_qv_follows_column.setObjectName("opt_qv_follows_column")
        self.gridLayout_1221.addWidget(self.opt_qv_follows_column, 2, 1, 1, 1)
        self.opt_qv_retkey_changes_column = QtWidgets.QCheckBox(parent=cover_browser_tab)
        self.opt_qv_retkey_changes_column.setObjectName("opt_qv_retkey_changes_column")
        self.gridLayout_1221.addWidget(self.opt_qv_retkey_changes_column, 1, 0, 1, 1)
        self.opt_qv_dclick_changes_column = QtWidgets.QCheckBox(parent=cover_browser_tab)
        self.opt_qv_dclick_changes_column.setObjectName("opt_qv_dclick_changes_column")
        self.gridLayout_1221.addWidget(self.opt_qv_dclick_changes_column, 1, 1, 1, 1)
        self.gridLayout_122.addLayout(self.gridLayout_1221, 1, 0, 1, 2)
        self.groupBox = QtWidgets.QGroupBox(parent=cover_browser_tab)
        self.groupBox.setObjectName("groupBox")
        self.gridLayout_32 = QtWidgets.QGridLayout(self.groupBox)
        self.gridLayout_32.setObjectName("gridLayout_32")
        self.qv_up_button = QtWidgets.QToolButton(parent=self.groupBox)
        icon = QtGui.QIcon.ic("arrow-up.png")
        self.qv_up_button.setIcon(icon)
        self.qv_up_button.setObjectName("qv_up_button")
        self.gridLayout_32.addWidget(self.qv_up_button, 0, 1, 1, 1)
        self.qv_down_button = QtWidgets.QToolButton(parent=self.groupBox)
        icon1 = QtGui.QIcon.ic("arrow-down.png")
        self.qv_down_button.setIcon(icon1)
        self.qv_down_button.setObjectName("qv_down_button")
        self.gridLayout_32.addWidget(self.qv_down_button, 2, 1, 1, 1)
        self.qv_display_order = ListViewWithMoveByKeyPress(parent=self.groupBox)
        self.qv_display_order.setAlternatingRowColors(True)
        self.qv_display_order.setObjectName("qv_display_order")
        self.gridLayout_32.addWidget(self.qv_display_order, 0, 0, 3, 1)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.gridLayout_32.addItem(spacerItem, 1, 1, 1, 1)
        self.gridLayout_122.addWidget(self.groupBox, 3, 0, 2, 1)

        self.retranslateUi(cover_browser_tab)

    def retranslateUi(self, cover_browser_tab):

        self.opt_qv_respects_vls.setToolTip(_("<p>Check this box to make Quickview show books only in the\n"
"current Virtual library. If unchecked, Quickview ignores Virtual libraries. If\n"
"unchecked then only row changes are taken into account.</p>"))
        self.opt_qv_respects_vls.setText(_("&Apply Virtual libraries"))
        self.opt_qv_follows_column.setToolTip(_("<p>Check this box to make Quickview change the column being examined\n"
"when the column in the book list is changed using the cursor arrow keys</p>"))
        self.opt_qv_follows_column.setText(_("&Change Quickview item when book list column changes"))
        self.opt_qv_retkey_changes_column.setToolTip(_("<p>Pressing Enter in a cell changes both the book and the\n"
"column being examined (the left-hand panel)</p>"))
        self.opt_qv_retkey_changes_column.setText(_("&Pressing Enter changes the examined column"))
        self.opt_qv_dclick_changes_column.setToolTip(_("<p>Double-clicking in a cell changes both the book and the\n"
"column being examined (the left-hand panel)</p>"))
        self.opt_qv_dclick_changes_column.setText(_("&Double click changes examined column"))
        self.groupBox.setTitle(_("Select columns to display"))
        self.qv_up_button.setToolTip(_("Move up. Keyboard shortcut: Ctrl-Up arrow"))
        self.qv_down_button.setToolTip(_("Move down. Keyboard shortcut: Ctrl-Down arrow"))
from calibre.gui2.preferences import ListViewWithMoveByKeyPress
