#!/usr/local/bin/python3.13


__license__ = 'GPL v3'
__copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'

from calibre.ebooks.conversion.config import OPTIONS
from calibre.gui2.convert import Widget
from calibre.gui2.convert.rtf_input_ui import Ui_Form


class PluginWidget(Widget, Ui_Form):

    TITLE = _('RTF input')
    HELP = _('Options specific to')+' RTF '+_('input')
    COMMIT_NAME = 'rtf_input'
    ICON = 'mimetypes/rtf.png'

    def __init__(self, parent, get_option, get_help, db=None, book_id=None):
        Widget.__init__(self, parent, OPTIONS['input']['rtf'])
        self.initialize_options(get_option, get_help, db, book_id)
