__license__ = 'GPL v3'
__copyright__ = '2022, Vaso Peras-Likodric <vaso at vipl.in.rs>'
__docformat__ = 'restructuredtext en'

import enum


class PageNumberTypes(enum.Enum):
    Arabic = 'a'
    Roman = 'r'
    Custom = 'c'
