# Form implementation generated from reading ui file '/home/kovid/work/calibre/src/calibre/gui2/preferences/look_feel_tabs/book_details.ui'
#
# Created by: PyQt6 UI code generator 6.8.1
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_Form(object):
    def setupUi(self, book_details_tab):
        book_details_tab.setObjectName("book_details_tab")
        self.gridLayout_6 = QtWidgets.QGridLayout(book_details_tab)
        self.gridLayout_6.setObjectName("gridLayout_6")
        self.groupBox_5 = QtWidgets.QGroupBox(parent=book_details_tab)
        self.groupBox_5.setObjectName("groupBox_5")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.groupBox_5)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.opt_book_details_css = QtWidgets.QPlainTextEdit(parent=self.groupBox_5)
        self.opt_book_details_css.setObjectName("opt_book_details_css")
        self.verticalLayout_3.addWidget(self.opt_book_details_css)
        self.gridLayout_6.addWidget(self.groupBox_5, 6, 0, 1, 1)
        self.id_links_button = QtWidgets.QPushButton(parent=book_details_tab)
        self.id_links_button.setObjectName("id_links_button")
        self.gridLayout_6.addWidget(self.id_links_button, 5, 0, 1, 3)
        self.default_author_link = DefaultAuthorLink(parent=book_details_tab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Preferred, QtWidgets.QSizePolicy.Policy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.default_author_link.sizePolicy().hasHeightForWidth())
        self.default_author_link.setSizePolicy(sizePolicy)
        self.default_author_link.setObjectName("default_author_link")
        self.gridLayout_6.addWidget(self.default_author_link, 3, 0, 1, 3)
        self.categories_on_separate_lines_group_box = QtWidgets.QGroupBox(parent=book_details_tab)
        self.categories_on_separate_lines_group_box.setObjectName("categories_on_separate_lines_group_box")
        self.vblayout_3 = QtWidgets.QVBoxLayout(self.categories_on_separate_lines_group_box)
        self.vblayout_3.setObjectName("vblayout_3")
        self.bd_vertical_cats = QtWidgets.QListView(parent=self.categories_on_separate_lines_group_box)
        self.bd_vertical_cats.setAlternatingRowColors(True)
        self.bd_vertical_cats.setObjectName("bd_vertical_cats")
        self.vblayout_3.addWidget(self.bd_vertical_cats)
        self.gridLayout_6.addWidget(self.categories_on_separate_lines_group_box, 6, 2, 1, 1)
        self.groupBox = QtWidgets.QGroupBox(parent=book_details_tab)
        self.groupBox.setObjectName("groupBox")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.groupBox)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.df_down_button = QtWidgets.QToolButton(parent=self.groupBox)
        icon = QtGui.QIcon.ic("arrow-down.png")
        self.df_down_button.setIcon(icon)
        self.df_down_button.setObjectName("df_down_button")
        self.gridLayout_3.addWidget(self.df_down_button, 3, 1, 1, 1)
        self.df_up_button = QtWidgets.QToolButton(parent=self.groupBox)
        icon1 = QtGui.QIcon.ic("arrow-up.png")
        self.df_up_button.setIcon(icon1)
        self.df_up_button.setObjectName("df_up_button")
        self.gridLayout_3.addWidget(self.df_up_button, 1, 1, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.gridLayout_3.addItem(spacerItem, 2, 1, 1, 1)
        self.field_display_order = ListViewWithMoveByKeyPress(parent=self.groupBox)
        self.field_display_order.setAlternatingRowColors(True)
        self.field_display_order.setObjectName("field_display_order")
        self.gridLayout_3.addWidget(self.field_display_order, 1, 0, 3, 1)
        self.label_3 = QtWidgets.QLabel(parent=self.groupBox)
        self.label_3.setWordWrap(True)
        self.label_3.setObjectName("label_3")
        self.gridLayout_3.addWidget(self.label_3, 4, 0, 1, 2)
        self.gridLayout_6.addWidget(self.groupBox, 6, 1, 1, 1)
        self.hboxlayout = QtWidgets.QHBoxLayout()
        self.hboxlayout.setObjectName("hboxlayout")
        self.opt_bd_show_cover = QtWidgets.QCheckBox(parent=book_details_tab)
        self.opt_bd_show_cover.setObjectName("opt_bd_show_cover")
        self.hboxlayout.addWidget(self.opt_bd_show_cover)
        self.opt_bd_overlay_cover_size = QtWidgets.QCheckBox(parent=book_details_tab)
        self.opt_bd_overlay_cover_size.setObjectName("opt_bd_overlay_cover_size")
        self.hboxlayout.addWidget(self.opt_bd_overlay_cover_size)
        self.opt_use_roman_numerals_for_series_number = QtWidgets.QCheckBox(parent=book_details_tab)
        self.opt_use_roman_numerals_for_series_number.setChecked(True)
        self.opt_use_roman_numerals_for_series_number.setObjectName("opt_use_roman_numerals_for_series_number")
        self.hboxlayout.addWidget(self.opt_use_roman_numerals_for_series_number)
        spacerItem1 = QtWidgets.QSpacerItem(20, 0, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.hboxlayout.addItem(spacerItem1)
        self.gridLayout_6.addLayout(self.hboxlayout, 0, 0, 1, 3)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_25 = QtWidgets.QLabel(parent=book_details_tab)
        self.label_25.setObjectName("label_25")
        self.horizontalLayout_2.addWidget(self.label_25)
        self.opt_book_details_comments_heading_pos = QtWidgets.QComboBox(parent=book_details_tab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.opt_book_details_comments_heading_pos.sizePolicy().hasHeightForWidth())
        self.opt_book_details_comments_heading_pos.setSizePolicy(sizePolicy)
        self.opt_book_details_comments_heading_pos.setObjectName("opt_book_details_comments_heading_pos")
        self.horizontalLayout_2.addWidget(self.opt_book_details_comments_heading_pos)
        spacerItem2 = QtWidgets.QSpacerItem(10, 0, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem2)
        self.label_1022 = QtWidgets.QLabel(parent=book_details_tab)
        self.label_1022.setObjectName("label_1022")
        self.horizontalLayout_2.addWidget(self.label_1022)
        self.opt_book_details_note_link_icon_width = QtWidgets.QDoubleSpinBox(parent=book_details_tab)
        self.opt_book_details_note_link_icon_width.setDecimals(1)
        self.opt_book_details_note_link_icon_width.setMinimum(0.5)
        self.opt_book_details_note_link_icon_width.setMaximum(4.0)
        self.opt_book_details_note_link_icon_width.setSingleStep(0.1)
        self.opt_book_details_note_link_icon_width.setObjectName("opt_book_details_note_link_icon_width")
        self.horizontalLayout_2.addWidget(self.opt_book_details_note_link_icon_width)
        spacerItem3 = QtWidgets.QSpacerItem(30, 0, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem3)
        self.gridLayout_6.addLayout(self.horizontalLayout_2, 1, 0, 1, 3)
        self.label_25.setBuddy(self.opt_book_details_comments_heading_pos)
        self.label_1022.setBuddy(self.opt_book_details_note_link_icon_width)

        self.retranslateUi(book_details_tab)

    def retranslateUi(self, book_details_tab):

        self.groupBox_5.setTitle(_("Text styling"))
        self.id_links_button.setText(_("Create rules to convert &identifiers into links"))
        self.categories_on_separate_lines_group_box.setToolTip(_("<p>Check the box if you want the category\'s values displayed on separate lines instead of separated by commas</p>"))
        self.categories_on_separate_lines_group_box.setTitle(_("Categories on separate lines"))
        self.groupBox.setTitle(_("Select displayed metadata"))
        self.df_down_button.setToolTip(_("Move down. Keyboard shortcut: Ctrl-Down arrow"))
        self.df_up_button.setToolTip(_("Move up. Keyboard shortcut: Ctrl-Up arrow"))
        self.label_3.setText(_("<p>Note: <b>comments</b>-like columns will always\n"
"                 be displayed at the end unless their \"Heading position\" is\n"
"                 \"Show heading to the side\".</p>"))
        self.opt_bd_show_cover.setText(_("Show &cover"))
        self.opt_bd_overlay_cover_size.setToolTip(_("Show the size of the book\'s cover in pixels"))
        self.opt_bd_overlay_cover_size.setText(_("Show cover &size"))
        self.opt_use_roman_numerals_for_series_number.setText(_("Use &Roman numerals for series"))
        self.label_25.setText(_("Show comments &heading:"))
        self.label_1022.setText(_("Link and note icon si&ze:"))
        self.opt_book_details_note_link_icon_width.setToolTip(_("Increase or decrease the size of the links and notes icons by this number. Larger\n"
"than one increases the icon size while smaller than one decreases it."))
from calibre.gui2.preferences import ListViewWithMoveByKeyPress
from calibre.gui2.preferences.look_feel_tabs import DefaultAuthorLink
