# Form implementation generated from reading ui file '/home/kovid/work/calibre/src/calibre/gui2/dialogs/tag_categories.ui'
#
# Created by: PyQt6 UI code generator 6.4.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_TagCategories(object):
    def setupUi(self, TagCategories):
        TagCategories.setObjectName("TagCategories")
        TagCategories.resize(628, 482)
        icon = QtGui.QIcon.ic("chapters.png")
        TagCategories.setWindowIcon(icon)
        self.vboxlayout = QtWidgets.QVBoxLayout(TagCategories)
        self.vboxlayout.setObjectName("vboxlayout")
        self.gridLayout_4 = QtWidgets.QGridLayout()
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.formLayout = QtWidgets.QGridLayout()
        self.formLayout.setObjectName("formLayout")
        self.label_3 = QtWidgets.QLabel(TagCategories)
        self.label_3.setAlignment(QtCore.Qt.AlignmentFlag.AlignLeading|QtCore.Qt.AlignmentFlag.AlignLeft|QtCore.Qt.AlignmentFlag.AlignVCenter)
        self.label_3.setObjectName("label_3")
        self.formLayout.addWidget(self.label_3, 0, 0, 1, 1)
        self.category_box = QtWidgets.QComboBox(TagCategories)
        self.category_box.setEditable(False)
        self.category_box.setMinimumSize(QtCore.QSize(150, 0))
        self.category_box.setObjectName("category_box")
        self.formLayout.addWidget(self.category_box, 0, 1, 1, 1)
        self.copy_category_name_to_clipboard = QtWidgets.QToolButton(TagCategories)
        icon1 = QtGui.QIcon.ic("edit-copy.png")
        self.copy_category_name_to_clipboard.setIcon(icon1)
        self.copy_category_name_to_clipboard.setObjectName("copy_category_name_to_clipboard")
        self.formLayout.addWidget(self.copy_category_name_to_clipboard, 0, 2, 1, 1)
        self.label_5 = QtWidgets.QLabel(TagCategories)
        self.label_5.setAlignment(QtCore.Qt.AlignmentFlag.AlignLeading|QtCore.Qt.AlignmentFlag.AlignLeft|QtCore.Qt.AlignmentFlag.AlignVCenter)
        self.label_5.setObjectName("label_5")
        self.formLayout.addWidget(self.label_5, 1, 0, 1, 1)
        self.category_filter_box = QtWidgets.QComboBox(TagCategories)
        self.category_filter_box.setMinimumSize(QtCore.QSize(150, 0))
        self.category_filter_box.setObjectName("category_filter_box")
        self.formLayout.addWidget(self.category_filter_box, 1, 1, 1, 1)
        self.label_filt = QtWidgets.QLabel(TagCategories)
        self.label_filt.setAlignment(QtCore.Qt.AlignmentFlag.AlignLeading|QtCore.Qt.AlignmentFlag.AlignLeft|QtCore.Qt.AlignmentFlag.AlignVCenter)
        self.label_filt.setObjectName("label_filt")
        self.formLayout.addWidget(self.label_filt, 2, 0, 1, 1)
        self.item_filter_box = QtWidgets.QLineEdit(TagCategories)
        self.item_filter_box.setMinimumSize(QtCore.QSize(150, 0))
        self.item_filter_box.setObjectName("item_filter_box")
        self.formLayout.addWidget(self.item_filter_box, 2, 1, 1, 1)
        self.gridLayout_4.addLayout(self.formLayout, 0, 0, 1, 1)
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.delete_category_button = QtWidgets.QToolButton(TagCategories)
        icon2 = QtGui.QIcon.ic("trash.png")
        self.delete_category_button.setIcon(icon2)
        self.delete_category_button.setObjectName("delete_category_button")
        self.gridLayout.addWidget(self.delete_category_button, 0, 0, 1, 1)
        self.hboxlayout = QtWidgets.QHBoxLayout()
        self.hboxlayout.setObjectName("hboxlayout")
        self.input_box = QtWidgets.QLineEdit(TagCategories)
        self.input_box.setClearButtonEnabled(True)
        self.input_box.setObjectName("input_box")
        self.hboxlayout.addWidget(self.input_box)
        self.add_category_button = QtWidgets.QToolButton(TagCategories)
        icon3 = QtGui.QIcon.ic("plus.png")
        self.add_category_button.setIcon(icon3)
        self.add_category_button.setObjectName("add_category_button")
        self.hboxlayout.addWidget(self.add_category_button)
        self.gridLayout.addLayout(self.hboxlayout, 0, 1, 1, 1)
        self.rename_category_button = QtWidgets.QToolButton(TagCategories)
        icon4 = QtGui.QIcon.ic("edit-undo.png")
        self.rename_category_button.setIcon(icon4)
        self.rename_category_button.setObjectName("rename_category_button")
        self.gridLayout.addWidget(self.rename_category_button, 0, 2, 1, 1)
        self.apply_vl_checkbox = QtWidgets.QCheckBox(TagCategories)
        self.apply_vl_checkbox.setObjectName("apply_vl_checkbox")
        self.gridLayout.addWidget(self.apply_vl_checkbox, 1, 0, 1, 3)
        self.hide_hidden_categories_checkbox = QtWidgets.QCheckBox(TagCategories)
        self.hide_hidden_categories_checkbox.setObjectName("hide_hidden_categories_checkbox")
        self.gridLayout.addWidget(self.hide_hidden_categories_checkbox, 2, 0, 1, 3)
        self.gridLayout_4.addLayout(self.gridLayout, 0, 1, 1, 3)
        self.vboxlayout.addLayout(self.gridLayout_4)
        self.gridLayout_2 = QtWidgets.QGridLayout()
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.label = QtWidgets.QLabel(TagCategories)
        self.label.setObjectName("label")
        self.gridLayout_2.addWidget(self.label, 1, 0, 1, 1)
        self.available_items_box = QtWidgets.QListWidget(TagCategories)
        self.available_items_box.setAlternatingRowColors(True)
        self.available_items_box.setSelectionMode(QtWidgets.QAbstractItemView.SelectionMode.MultiSelection)
        self.available_items_box.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectionBehavior.SelectRows)
        self.available_items_box.setObjectName("available_items_box")
        self.gridLayout_2.addWidget(self.available_items_box, 3, 0, 1, 1)
        self.apply_button = QtWidgets.QToolButton(TagCategories)
        icon5 = QtGui.QIcon.ic("forward.png")
        self.apply_button.setIcon(icon5)
        self.apply_button.setObjectName("apply_button")
        self.gridLayout_2.addWidget(self.apply_button, 3, 1, 1, 1)
        self.label_2 = QtWidgets.QLabel(TagCategories)
        self.label_2.setObjectName("label_2")
        self.gridLayout_2.addWidget(self.label_2, 1, 2, 1, 1)
        self.applied_items_box = QtWidgets.QListWidget(TagCategories)
        self.applied_items_box.setAlternatingRowColors(True)
        self.applied_items_box.setSelectionMode(QtWidgets.QAbstractItemView.SelectionMode.MultiSelection)
        self.applied_items_box.setObjectName("applied_items_box")
        self.gridLayout_2.addWidget(self.applied_items_box, 3, 2, 1, 1)
        self.unapply_button = QtWidgets.QToolButton(TagCategories)
        icon6 = QtGui.QIcon.ic("minus.png")
        self.unapply_button.setIcon(icon6)
        self.unapply_button.setObjectName("unapply_button")
        self.gridLayout_2.addWidget(self.unapply_button, 3, 3, 1, 1)
        self.buttonBox = QtWidgets.QDialogButtonBox(TagCategories)
        self.buttonBox.setOrientation(QtCore.Qt.Orientation.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.StandardButton.Cancel|QtWidgets.QDialogButtonBox.StandardButton.Ok)
        self.buttonBox.setCenterButtons(False)
        self.buttonBox.setObjectName("buttonBox")
        self.gridLayout_2.addWidget(self.buttonBox, 4, 0, 1, 4)
        self.vboxlayout.addLayout(self.gridLayout_2)
        self.label_3.setBuddy(self.category_box)
        self.label_5.setBuddy(self.category_filter_box)
        self.label_filt.setBuddy(self.item_filter_box)
        self.label.setBuddy(self.available_items_box)
        self.label_2.setBuddy(self.applied_items_box)

        self.retranslateUi(TagCategories)
        self.buttonBox.accepted.connect(TagCategories.accept) # type: ignore
        self.buttonBox.rejected.connect(TagCategories.reject) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(TagCategories)

    def retranslateUi(self, TagCategories):

        TagCategories.setWindowTitle(_("User categories editor"))
        self.label_3.setText(_("&User category name: "))
        self.category_box.setToolTip(_("Select a category to edit"))
        self.copy_category_name_to_clipboard.setToolTip(_("Copy the category name to the clipboard"))
        self.label_5.setText(_("&Category filter: "))
        self.category_filter_box.setToolTip(_("Select the content kind of the new category"))
        self.label_filt.setText(_("Item &filter: "))
        self.item_filter_box.setToolTip(_("Enter text to filter the available items. Case and accents are ignored."))
        self.delete_category_button.setToolTip(_("Delete the current User category"))
        self.delete_category_button.setText(_("..."))
        self.input_box.setToolTip(_("Enter a category name, then use the Add button or the Rename button"))
        self.add_category_button.setToolTip(_("Add a new category"))
        self.add_category_button.setText(_("..."))
        self.rename_category_button.setToolTip(_("Rename the current category to what is in the box"))
        self.rename_category_button.setText(_("..."))
        self.apply_vl_checkbox.setToolTip(_("<p>Only show items that are visible in current Virtual\n"
"            library. Applied items not shown in the Virtual library will be\n"
"            marked \"Hidden by Virtual library\".</p>"))
        self.apply_vl_checkbox.setText(_("&Only show items visible in current Virtual library"))
        self.hide_hidden_categories_checkbox.setToolTip(_("<p>Don\'t show items that are hidden in the Tag browser.\n"
"          Applied items not shown in the Tag browser will be marked\n"
"          \"Hidden in Tag browser\".</p>"))
        self.hide_hidden_categories_checkbox.setText(_("&Don\'t show items hidden in the Tag browser"))
        self.label.setText(_("&Available items"))
        self.apply_button.setToolTip(_("Apply items to current category"))
        self.apply_button.setText(_("..."))
        self.label_2.setText(_("Applied &items"))
        self.unapply_button.setToolTip(_("Unapply (remove) selected items from current category"))
        self.unapply_button.setText(_("..."))
