__license__   = 'GPL v3'
__copyright__ = '2008, Marshall T. Vandegrift <llasram@gmail.com>'

'''
Microsoft LIT tag and attribute tables.
'''

from calibre.ebooks.lit.maps.html import MAP as HTML_MAP
from calibre.ebooks.lit.maps.opf import MAP as OPF_MAP

OPF_MAP, HTML_MAP
