# Form implementation generated from reading ui file '/home/kovid/work/calibre/src/calibre/gui2/preferences/look_feel_tabs/main_interface.ui'
#
# Created by: PyQt6 UI code generator 6.8.1
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_main_interface_tab(object):
    def setupUi(self, main_interface_tab):
        main_interface_tab.setObjectName("main_interface_tab")
        main_interface_tab.resize(617, 621)
        self.gridLayout_main_interface = QtWidgets.QGridLayout(main_interface_tab)
        self.gridLayout_main_interface.setObjectName("gridLayout_main_interface")
        self.opt_booklist_grid = QtWidgets.QCheckBox(parent=main_interface_tab)
        self.opt_booklist_grid.setObjectName("opt_booklist_grid")
        self.gridLayout_main_interface.addWidget(self.opt_booklist_grid, 13, 0, 1, 1)
        self.opt_row_numbers_in_book_list = QtWidgets.QCheckBox(parent=main_interface_tab)
        self.opt_row_numbers_in_book_list.setObjectName("opt_row_numbers_in_book_list")
        self.gridLayout_main_interface.addWidget(self.opt_row_numbers_in_book_list, 12, 0, 1, 1)
        self.label_17 = QtWidgets.QLabel(parent=main_interface_tab)
        self.label_17.setObjectName("label_17")
        self.gridLayout_main_interface.addWidget(self.label_17, 3, 0, 1, 1)
        self.opt_color_palette_label = QtWidgets.QLabel(parent=main_interface_tab)
        self.opt_color_palette_label.setObjectName("opt_color_palette_label")
        self.gridLayout_main_interface.addWidget(self.opt_color_palette_label, 2, 0, 1, 1)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.opt_cover_corner_radius = QtWidgets.QSpinBox(parent=main_interface_tab)
        self.opt_cover_corner_radius.setObjectName("opt_cover_corner_radius")
        self.horizontalLayout_3.addWidget(self.opt_cover_corner_radius)
        self.opt_cover_corner_radius_unit = QtWidgets.QComboBox(parent=main_interface_tab)
        self.opt_cover_corner_radius_unit.setObjectName("opt_cover_corner_radius_unit")
        self.horizontalLayout_3.addWidget(self.opt_cover_corner_radius_unit)
        self.gridLayout_main_interface.addLayout(self.horizontalLayout_3, 5, 1, 1, 1)
        self.opt_dnd_merge = QtWidgets.QCheckBox(parent=main_interface_tab)
        self.opt_dnd_merge.setObjectName("opt_dnd_merge")
        self.gridLayout_main_interface.addWidget(self.opt_dnd_merge, 14, 0, 1, 1)
        self.opt_disable_animations = QtWidgets.QCheckBox(parent=main_interface_tab)
        self.opt_disable_animations.setObjectName("opt_disable_animations")
        self.gridLayout_main_interface.addWidget(self.opt_disable_animations, 6, 1, 1, 1)
        self.opt_book_list_tooltips = QtWidgets.QCheckBox(parent=main_interface_tab)
        self.opt_book_list_tooltips.setObjectName("opt_book_list_tooltips")
        self.gridLayout_main_interface.addWidget(self.opt_book_list_tooltips, 8, 0, 1, 1)
        self.label_7 = QtWidgets.QLabel(parent=main_interface_tab)
        self.label_7.setObjectName("label_7")
        self.gridLayout_main_interface.addWidget(self.label_7, 4, 0, 1, 1)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label_2 = QtWidgets.QLabel(parent=main_interface_tab)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout.addWidget(self.label_2)
        self.font_display = QtWidgets.QLineEdit(parent=main_interface_tab)
        self.font_display.setReadOnly(True)
        self.font_display.setObjectName("font_display")
        self.horizontalLayout.addWidget(self.font_display)
        self.gridLayout_main_interface.addLayout(self.horizontalLayout, 9, 0, 1, 1)
        self.opt_book_list_extra_row_spacing = QtWidgets.QSpinBox(parent=main_interface_tab)
        self.opt_book_list_extra_row_spacing.setMinimum(-20)
        self.opt_book_list_extra_row_spacing.setObjectName("opt_book_list_extra_row_spacing")
        self.gridLayout_main_interface.addWidget(self.opt_book_list_extra_row_spacing, 11, 1, 1, 1)
        self.opt_show_splash_screen = QtWidgets.QCheckBox(parent=main_interface_tab)
        self.opt_show_splash_screen.setObjectName("opt_show_splash_screen")
        self.gridLayout_main_interface.addWidget(self.opt_show_splash_screen, 7, 1, 1, 1)
        self.icon_theme = QtWidgets.QLabel(parent=main_interface_tab)
        self.icon_theme.setText("")
        self.icon_theme.setObjectName("icon_theme")
        self.gridLayout_main_interface.addWidget(self.icon_theme, 1, 0, 1, 1)
        self.label_widget_style = QtWidgets.QLabel(parent=main_interface_tab)
        self.label_widget_style.setObjectName("label_widget_style")
        self.gridLayout_main_interface.addWidget(self.label_widget_style, 0, 0, 1, 1)
        self.groupBox = QtWidgets.QGroupBox(parent=main_interface_tab)
        self.groupBox.setObjectName("groupBox")
        self.vboxlayout = QtWidgets.QVBoxLayout(self.groupBox)
        self.vboxlayout.setObjectName("vboxlayout")
        self.opt_show_layout_buttons = QtWidgets.QCheckBox(parent=self.groupBox)
        self.opt_show_layout_buttons.setObjectName("opt_show_layout_buttons")
        self.vboxlayout.addWidget(self.opt_show_layout_buttons)
        self.opt_show_sb_all_actions_button = QtWidgets.QCheckBox(parent=self.groupBox)
        self.opt_show_sb_all_actions_button.setObjectName("opt_show_sb_all_actions_button")
        self.vboxlayout.addWidget(self.opt_show_sb_all_actions_button)
        spacerItem = QtWidgets.QSpacerItem(10, 20, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.vboxlayout.addItem(spacerItem)
        self.gridLayout_main_interface.addWidget(self.groupBox, 10, 1, 1, 1)
        self.change_font_button = QtWidgets.QPushButton(parent=main_interface_tab)
        self.change_font_button.setObjectName("change_font_button")
        self.gridLayout_main_interface.addWidget(self.change_font_button, 9, 1, 1, 1)
        self.label_6 = QtWidgets.QLabel(parent=main_interface_tab)
        self.label_6.setObjectName("label_6")
        self.gridLayout_main_interface.addWidget(self.label_6, 5, 0, 1, 1)
        self.icon_theme_button = QtWidgets.QPushButton(parent=main_interface_tab)
        self.icon_theme_button.setObjectName("icon_theme_button")
        self.gridLayout_main_interface.addWidget(self.icon_theme_button, 1, 1, 1, 1)
        self.groupBox_2 = QtWidgets.QGroupBox(parent=main_interface_tab)
        self.groupBox_2.setObjectName("groupBox_2")
        self.gridLayout_8 = QtWidgets.QGridLayout(self.groupBox_2)
        self.gridLayout_8.setObjectName("gridLayout_8")
        self.opt_toolbar_icon_size = QtWidgets.QComboBox(parent=self.groupBox_2)
        self.opt_toolbar_icon_size.setObjectName("opt_toolbar_icon_size")
        self.gridLayout_8.addWidget(self.opt_toolbar_icon_size, 0, 1, 1, 1)
        self.opt_toolbar_text = QtWidgets.QComboBox(parent=self.groupBox_2)
        self.opt_toolbar_text.setObjectName("opt_toolbar_text")
        self.gridLayout_8.addWidget(self.opt_toolbar_text, 1, 1, 1, 1)
        self.label_8 = QtWidgets.QLabel(parent=self.groupBox_2)
        self.label_8.setObjectName("label_8")
        self.gridLayout_8.addWidget(self.label_8, 1, 0, 1, 1)
        self.opt_wrap_toolbar_text = QtWidgets.QCheckBox(parent=self.groupBox_2)
        self.opt_wrap_toolbar_text.setObjectName("opt_wrap_toolbar_text")
        self.gridLayout_8.addWidget(self.opt_wrap_toolbar_text, 2, 0, 1, 2)
        self.label_5 = QtWidgets.QLabel(parent=self.groupBox_2)
        self.label_5.setObjectName("label_5")
        self.gridLayout_8.addWidget(self.label_5, 0, 0, 1, 1)
        self.gridLayout_main_interface.addWidget(self.groupBox_2, 10, 0, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.gridLayout_main_interface.addItem(spacerItem1, 16, 0, 1, 1)
        self.label_4 = QtWidgets.QLabel(parent=main_interface_tab)
        self.label_4.setObjectName("label_4")
        self.gridLayout_main_interface.addWidget(self.label_4, 11, 0, 1, 1)
        self.opt_disable_tray_notification = QtWidgets.QCheckBox(parent=main_interface_tab)
        self.opt_disable_tray_notification.setObjectName("opt_disable_tray_notification")
        self.gridLayout_main_interface.addWidget(self.opt_disable_tray_notification, 7, 0, 1, 1)
        self.button_adjust_colors = QtWidgets.QPushButton(parent=main_interface_tab)
        self.button_adjust_colors.setObjectName("button_adjust_colors")
        self.gridLayout_main_interface.addWidget(self.button_adjust_colors, 2, 1, 1, 1)
        self.opt_language = QtWidgets.QComboBox(parent=main_interface_tab)
        self.opt_language.setSizeAdjustPolicy(QtWidgets.QComboBox.SizeAdjustPolicy.AdjustToMinimumContentsLengthWithIcon)
        self.opt_language.setMinimumContentsLength(20)
        self.opt_language.setObjectName("opt_language")
        self.gridLayout_main_interface.addWidget(self.opt_language, 4, 1, 1, 1)
        self.opt_systray_icon = QtWidgets.QCheckBox(parent=main_interface_tab)
        self.opt_systray_icon.setObjectName("opt_systray_icon")
        self.gridLayout_main_interface.addWidget(self.opt_systray_icon, 6, 0, 1, 1)
        self.opt_ui_style = QtWidgets.QComboBox(parent=main_interface_tab)
        self.opt_ui_style.setObjectName("opt_ui_style")
        self.gridLayout_main_interface.addWidget(self.opt_ui_style, 0, 1, 1, 1)
        self.opt_gui_layout = QtWidgets.QComboBox(parent=main_interface_tab)
        self.opt_gui_layout.setMaximumSize(QtCore.QSize(250, 16777215))
        self.opt_gui_layout.setSizeAdjustPolicy(QtWidgets.QComboBox.SizeAdjustPolicy.AdjustToMinimumContentsLengthWithIcon)
        self.opt_gui_layout.setMinimumContentsLength(20)
        self.opt_gui_layout.setObjectName("opt_gui_layout")
        self.gridLayout_main_interface.addWidget(self.opt_gui_layout, 3, 1, 1, 1)
        spacerItem2 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.gridLayout_main_interface.addItem(spacerItem2, 15, 0, 1, 1)
        self.label_17.setBuddy(self.opt_gui_layout)
        self.opt_color_palette_label.setBuddy(self.button_adjust_colors)
        self.label_7.setBuddy(self.opt_language)
        self.label_2.setBuddy(self.font_display)
        self.label_widget_style.setBuddy(self.opt_ui_style)
        self.label_6.setBuddy(self.opt_cover_corner_radius)
        self.label_8.setBuddy(self.opt_toolbar_text)
        self.label_5.setBuddy(self.opt_toolbar_icon_size)
        self.label_4.setBuddy(self.opt_book_list_extra_row_spacing)

        self.retranslateUi(main_interface_tab)
        QtCore.QMetaObject.connectSlotsByName(main_interface_tab)

    def retranslateUi(self, main_interface_tab):

        self.opt_booklist_grid.setText(_("Draw a &grid in the book list"))
        self.opt_row_numbers_in_book_list.setText(_("Show &row numbers in the book list"))
        self.label_17.setText(_("&User interface layout:"))
        self.opt_color_palette_label.setText(_("User interface &colors:"))
        self.opt_cover_corner_radius.setToolTip(_("Round the corners of covers in many places they are displayed such as the Cover grid, Book details panel, etc. Adjust the amount of rounding with this setting. A value of between 5 and 10 looks good with most covers sizes. Zero disables rounding and is the default."))
        self.opt_cover_corner_radius.setSpecialValueText(_("no rounding"))
        self.opt_cover_corner_radius_unit.setToolTip(_("The unit for the cover corner rounding. Either pixel values or as a percentage of the cover size."))
        self.opt_dnd_merge.setText(_("Allow using &drag and drop to merge books"))
        self.opt_disable_animations.setToolTip(_("Disable all animations. Useful if you have a slow/old computer."))
        self.opt_disable_animations.setText(_("Disable &animations"))
        self.opt_book_list_tooltips.setText(_("Show &tooltips in the book list"))
        self.label_7.setText(_("Choose &language (needs restart):"))
        self.label_2.setText(_("&Interface font:"))
        self.opt_book_list_extra_row_spacing.setSuffix(_(" px"))
        self.opt_show_splash_screen.setText(_("Show the &splash screen at startup"))
        self.label_widget_style.setText(_("User interface style (&needs restart):"))
        self.groupBox.setTitle(_("Status bar buttons"))
        self.opt_show_layout_buttons.setToolTip(_("Show individual buttons to control the layout of the interface rather than a single button with a popup."))
        self.opt_show_layout_buttons.setText(_("Show &layout buttons"))
        self.opt_show_sb_all_actions_button.setToolTip(_("Show a button that gives quick access to all available actions with a single click."))
        self.opt_show_sb_all_actions_button.setText(_("S&how actions button"))
        self.change_font_button.setText(_("Change &font (needs restart)"))
        self.label_6.setText(_("&Round the corners of covers:"))
        self.icon_theme_button.setText(_("Change &icon theme"))
        self.groupBox_2.setTitle(_("Toolbar"))
        self.label_8.setText(_("Show &text under icons:"))
        self.opt_wrap_toolbar_text.setText(_("Use t&wo lines for the text under the icons (needs restart)"))
        self.label_5.setText(_("Icon si&ze:"))
        self.label_4.setText(_("E&xtra spacing to add between rows in the book list (can be negative):"))
        self.opt_disable_tray_notification.setToolTip(_("Disable popup notifications when calibre completes jobs such a conversion, sending to device etc. The notifications are sent via the operating system notification facility, if available."))
        self.opt_disable_tray_notification.setText(_("Disable n&otifications on job completion"))
        self.button_adjust_colors.setText(_("Adjust &colors"))
        self.opt_systray_icon.setText(_("Enable s&ystem tray icon (needs restart)"))
