# Form implementation generated from reading ui file '/home/kovid/work/calibre/src/calibre/gui2/dialogs/tag_editor.ui'
#
# Created by: PyQt6 UI code generator 6.6.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_TagEditor(object):
    def setupUi(self, TagEditor):
        TagEditor.setObjectName("TagEditor")
        TagEditor.resize(640, 666)
        self.verticalLayout = QtWidgets.QVBoxLayout(TagEditor)
        self.verticalLayout.setContentsMargins(11, 11, 11, 11)
        self.verticalLayout.setSpacing(6)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setContentsMargins(0, -1, 0, -1)
        self.horizontalLayout_4.setSpacing(6)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem)
        self.label_3 = QtWidgets.QLabel(parent=TagEditor)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout_4.addWidget(self.label_3)
        self.add_tag_input = EnLineEdit(parent=TagEditor)
        self.add_tag_input.setObjectName("add_tag_input")
        self.horizontalLayout_4.addWidget(self.add_tag_input)
        self.add_tag_button = QtWidgets.QToolButton(parent=TagEditor)
        icon = QtGui.QIcon.ic("plus.png")
        self.add_tag_button.setIcon(icon)
        self.add_tag_button.setObjectName("add_tag_button")
        self.horizontalLayout_4.addWidget(self.add_tag_button)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem1)
        self.verticalLayout.addLayout(self.horizontalLayout_4)
        self.middleBoxLayout = QtWidgets.QGridLayout()
        self.middleBoxLayout.setSpacing(6)
        self.middleBoxLayout.setObjectName("middleBoxLayout")
        self.available_filter_input = QtWidgets.QLineEdit(parent=TagEditor)
        self.available_filter_input.setClearButtonEnabled(True)
        self.available_filter_input.setObjectName("available_filter_input")
        self.middleBoxLayout.addWidget(self.available_filter_input, 1, 1, 1, 1)
        self.available_tags = QtWidgets.QListView(parent=TagEditor)
        self.available_tags.setAlternatingRowColors(True)
        self.available_tags.setSelectionMode(QtWidgets.QAbstractItemView.SelectionMode.MultiSelection)
        self.available_tags.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectionBehavior.SelectRows)
        self.available_tags.setObjectName("available_tags")
        self.middleBoxLayout.addWidget(self.available_tags, 2, 1, 1, 1)
        self.buttonverticalLayout = QtWidgets.QVBoxLayout()
        self.buttonverticalLayout.setSpacing(6)
        self.buttonverticalLayout.setObjectName("buttonverticalLayout")
        spacerItem2 = QtWidgets.QSpacerItem(10, 20, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.buttonverticalLayout.addItem(spacerItem2)
        self.delete_button = QtWidgets.QToolButton(parent=TagEditor)
        icon1 = QtGui.QIcon.ic("trash.png")
        self.delete_button.setIcon(icon1)
        self.delete_button.setObjectName("delete_button")
        self.buttonverticalLayout.addWidget(self.delete_button)
        spacerItem3 = QtWidgets.QSpacerItem(10, 20, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.buttonverticalLayout.addItem(spacerItem3)
        self.middleBoxLayout.addLayout(self.buttonverticalLayout, 2, 0, 1, 1)
        self.label = QtWidgets.QLabel(parent=TagEditor)
        self.label.setObjectName("label")
        self.middleBoxLayout.addWidget(self.label, 0, 1, 1, 2)
        self.buttonverticalLayout1 = QtWidgets.QVBoxLayout()
        self.buttonverticalLayout1.setSpacing(6)
        self.buttonverticalLayout1.setObjectName("buttonverticalLayout1")
        spacerItem4 = QtWidgets.QSpacerItem(10, 20, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.buttonverticalLayout1.addItem(spacerItem4)
        self.apply_button = QtWidgets.QToolButton(parent=TagEditor)
        icon2 = QtGui.QIcon.ic("forward.png")
        self.apply_button.setIcon(icon2)
        self.apply_button.setObjectName("apply_button")
        self.buttonverticalLayout1.addWidget(self.apply_button)
        self.unapply_button = QtWidgets.QToolButton(parent=TagEditor)
        icon3 = QtGui.QIcon.ic("back.png")
        self.unapply_button.setIcon(icon3)
        self.unapply_button.setObjectName("unapply_button")
        self.buttonverticalLayout1.addWidget(self.unapply_button)
        spacerItem5 = QtWidgets.QSpacerItem(10, 20, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.buttonverticalLayout1.addItem(spacerItem5)
        self.middleBoxLayout.addLayout(self.buttonverticalLayout1, 2, 2, 1, 1)
        self.applied_filter_input = QtWidgets.QLineEdit(parent=TagEditor)
        self.applied_filter_input.setClearButtonEnabled(True)
        self.applied_filter_input.setObjectName("applied_filter_input")
        self.middleBoxLayout.addWidget(self.applied_filter_input, 1, 3, 1, 1)
        self.applied_tags = QtWidgets.QListView(parent=TagEditor)
        self.applied_tags.setAlternatingRowColors(True)
        self.applied_tags.setSelectionMode(QtWidgets.QAbstractItemView.SelectionMode.MultiSelection)
        self.applied_tags.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectionBehavior.SelectRows)
        self.applied_tags.setObjectName("applied_tags")
        self.middleBoxLayout.addWidget(self.applied_tags, 2, 3, 1, 1)
        self.label_2 = QtWidgets.QLabel(parent=TagEditor)
        self.label_2.setObjectName("label_2")
        self.middleBoxLayout.addWidget(self.label_2, 0, 3, 1, 1)
        self.verticalLayout.addLayout(self.middleBoxLayout)
        self.buttonBox = QtWidgets.QDialogButtonBox(parent=TagEditor)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.StandardButton.Cancel|QtWidgets.QDialogButtonBox.StandardButton.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)
        self.label_3.setBuddy(self.add_tag_input)
        self.label.setBuddy(self.available_tags)
        self.label_2.setBuddy(self.applied_tags)

        self.retranslateUi(TagEditor)
        self.buttonBox.accepted.connect(TagEditor.accept) # type: ignore
        self.buttonBox.rejected.connect(TagEditor.reject) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(TagEditor)

    def retranslateUi(self, TagEditor):

        TagEditor.setWindowTitle(_("Manage items"))
        self.label_3.setText(_("&Apply item:"))
        self.add_tag_button.setToolTip(_("Apply item(s) to the current book"))
        self.available_filter_input.setPlaceholderText(_("Filter the available items"))
        self.delete_button.setToolTip(_("Delete the selected items from database. This will unapply the items from all books and then remove them from the database."))
        self.label.setText(_("Ava&ilable items"))
        self.apply_button.setToolTip(_("Apply item to current book"))
        self.unapply_button.setToolTip(_("Unapply (remove) the item from current book"))
        self.applied_filter_input.setPlaceholderText(_("Filter the applied items"))
        self.label_2.setText(_("Applied &items"))
from calibre.gui2.widgets import EnLineEdit
