# Form implementation generated from reading ui file '/home/kovid/work/calibre/src/calibre/gui2/wizard/send_email.ui'
#
# Created by: PyQt6 UI code generator 6.7.1
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(585, 242)
        self.gridLayout = QtWidgets.QGridLayout(Form)
        self.gridLayout.setObjectName("gridLayout")
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        self.label_15 = QtWidgets.QLabel(parent=Form)
        self.label_15.setObjectName("label_15")
        self.horizontalLayout_9.addWidget(self.label_15)
        self.email_from = QtWidgets.QLineEdit(parent=Form)
        self.email_from.setObjectName("email_from")
        self.horizontalLayout_9.addWidget(self.email_from)
        self.gridLayout.addLayout(self.horizontalLayout_9, 0, 0, 1, 2)
        self.groupBox_5 = QtWidgets.QGroupBox(parent=Form)
        self.groupBox_5.setObjectName("groupBox_5")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.groupBox_5)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.label_16 = QtWidgets.QLabel(parent=self.groupBox_5)
        self.label_16.setWordWrap(True)
        self.label_16.setObjectName("label_16")
        self.gridLayout_3.addWidget(self.label_16, 0, 0, 1, 4)
        self.label_17 = QtWidgets.QLabel(parent=self.groupBox_5)
        self.label_17.setObjectName("label_17")
        self.gridLayout_3.addWidget(self.label_17, 1, 0, 1, 1)
        self.relay_host = QtWidgets.QLineEdit(parent=self.groupBox_5)
        self.relay_host.setObjectName("relay_host")
        self.gridLayout_3.addWidget(self.relay_host, 1, 1, 1, 2)
        self.horizontalLayout_11 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_11.setObjectName("horizontalLayout_11")
        self.label_18 = QtWidgets.QLabel(parent=self.groupBox_5)
        self.label_18.setObjectName("label_18")
        self.horizontalLayout_11.addWidget(self.label_18)
        self.relay_port = QtWidgets.QSpinBox(parent=self.groupBox_5)
        self.relay_port.setMinimum(1)
        self.relay_port.setMaximum(65555)
        self.relay_port.setProperty("value", 25)
        self.relay_port.setObjectName("relay_port")
        self.horizontalLayout_11.addWidget(self.relay_port)
        self.gridLayout_3.addLayout(self.horizontalLayout_11, 1, 3, 1, 1)
        self.label_19 = QtWidgets.QLabel(parent=self.groupBox_5)
        self.label_19.setObjectName("label_19")
        self.gridLayout_3.addWidget(self.label_19, 2, 0, 1, 1)
        self.relay_username = QtWidgets.QLineEdit(parent=self.groupBox_5)
        self.relay_username.setObjectName("relay_username")
        self.gridLayout_3.addWidget(self.relay_username, 2, 1, 1, 2)
        self.label_20 = QtWidgets.QLabel(parent=self.groupBox_5)
        self.label_20.setObjectName("label_20")
        self.gridLayout_3.addWidget(self.label_20, 3, 0, 1, 1)
        self.relay_password = QtWidgets.QLineEdit(parent=self.groupBox_5)
        self.relay_password.setEchoMode(QtWidgets.QLineEdit.EchoMode.Password)
        self.relay_password.setObjectName("relay_password")
        self.gridLayout_3.addWidget(self.relay_password, 3, 1, 1, 2)
        self.relay_show_password = QtWidgets.QCheckBox(parent=self.groupBox_5)
        self.relay_show_password.setObjectName("relay_show_password")
        self.gridLayout_3.addWidget(self.relay_show_password, 3, 3, 1, 1)
        self.label_21 = QtWidgets.QLabel(parent=self.groupBox_5)
        self.label_21.setObjectName("label_21")
        self.gridLayout_3.addWidget(self.label_21, 4, 0, 1, 1)
        self.relay_tls = QtWidgets.QRadioButton(parent=self.groupBox_5)
        self.relay_tls.setChecked(True)
        self.relay_tls.setObjectName("relay_tls")
        self.gridLayout_3.addWidget(self.relay_tls, 4, 1, 1, 1)
        self.relay_ssl = QtWidgets.QRadioButton(parent=self.groupBox_5)
        self.relay_ssl.setObjectName("relay_ssl")
        self.gridLayout_3.addWidget(self.relay_ssl, 4, 2, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.gridLayout_3.addItem(spacerItem, 3, 4, 1, 1)
        self.relay_none = QtWidgets.QRadioButton(parent=self.groupBox_5)
        self.relay_none.setObjectName("relay_none")
        self.gridLayout_3.addWidget(self.relay_none, 4, 3, 1, 1)
        self.gridLayout.addWidget(self.groupBox_5, 1, 0, 1, 1)
        self.verticalLayout_9 = QtWidgets.QVBoxLayout()
        self.verticalLayout_9.setObjectName("verticalLayout_9")
        self.relay_use_gmx = QtWidgets.QToolButton(parent=Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.relay_use_gmx.sizePolicy().hasHeightForWidth())
        self.relay_use_gmx.setSizePolicy(sizePolicy)
        icon = QtGui.QIcon.ic("gmail_logo.png")
        self.relay_use_gmx.setIcon(icon)
        self.relay_use_gmx.setIconSize(QtCore.QSize(48, 48))
        self.relay_use_gmx.setToolButtonStyle(QtCore.Qt.ToolButtonStyle.ToolButtonTextUnderIcon)
        self.relay_use_gmx.setObjectName("relay_use_gmx")
        self.verticalLayout_9.addWidget(self.relay_use_gmx)
        self.test_email_button = QtWidgets.QPushButton(parent=Form)
        self.test_email_button.setObjectName("test_email_button")
        self.verticalLayout_9.addWidget(self.test_email_button)
        self.gridLayout.addLayout(self.verticalLayout_9, 1, 1, 1, 1)
        self.label_15.setBuddy(self.email_from)
        self.label_17.setBuddy(self.relay_host)
        self.label_18.setBuddy(self.relay_port)
        self.label_19.setBuddy(self.relay_username)
        self.label_20.setBuddy(self.relay_password)
        self.label_21.setBuddy(self.relay_tls)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):

        Form.setWindowTitle(_("Form"))
        self.label_15.setText(_("Send email &from:"))
        self.email_from.setToolTip(_("<p>This is what will be present in the From: field of emails sent by calibre.<br> Set it to your email address"))
        self.groupBox_5.setToolTip(_("<p>A mail server is useful if the service you are sending mail to only accepts email from well know mail services."))
        self.groupBox_5.setTitle(_("Email server"))
        self.label_16.setText(_("calibre can <b>optionally</b> use a server to send mail"))
        self.label_17.setText(_("&Hostname:"))
        self.relay_host.setToolTip(_("The hostname of your mail server. For example, smtp.gmail.com"))
        self.label_18.setText(_("&Port:"))
        self.relay_port.setToolTip(_("The port your mail server listens for connections on. The default is 25"))
        self.label_19.setText(_("&Username:"))
        self.relay_username.setToolTip(_("Your username on the mail server"))
        self.label_20.setText(_("&Password:"))
        self.relay_password.setToolTip(_("Your password on the mail server"))
        self.relay_show_password.setText(_("&Show"))
        self.label_21.setText(_("Encr&yption:"))
        self.relay_tls.setToolTip(_("Use TLS encryption when connecting to the mail server. This is the most common."))
        self.relay_tls.setText(_("&TLS"))
        self.relay_ssl.setToolTip(_("Use SSL encryption when connecting to the mail server."))
        self.relay_ssl.setText(_("SS&L"))
        self.relay_none.setToolTip(_("WARNING: Using no encryption is highly insecure"))
        self.relay_none.setText(_("&None"))
        self.relay_use_gmx.setText(_("Use &GMX"))
        self.test_email_button.setText(_("&Test email"))
