# Form implementation generated from reading ui file '/home/kovid/work/calibre/src/calibre/gui2/preferences/columns.ui'
#
# Created by: PyQt6 UI code generator 6.8.0.dev2410211537
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(611, 482)
        self.gridLayout = QtWidgets.QGridLayout(Form)
        self.gridLayout.setObjectName("gridLayout")
        self.label = QtWidgets.QLabel(parent=Form)
        self.label.setWordWrap(True)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 0, 0, 1, 2)
        self.opt_columns = TableWidgetWithMoveByKeyPress(parent=Form)
        self.opt_columns.setAlternatingRowColors(True)
        self.opt_columns.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectionBehavior.SelectRows)
        self.opt_columns.setObjectName("opt_columns")
        self.opt_columns.setColumnCount(0)
        self.opt_columns.setRowCount(0)
        self.gridLayout.addWidget(self.opt_columns, 1, 0, 1, 1)
        self.verticalLayout_3 = QtWidgets.QVBoxLayout()
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.column_up = QtWidgets.QToolButton(parent=Form)
        icon = QtGui.QIcon.ic("arrow-up.png")
        self.column_up.setIcon(icon)
        self.column_up.setIconSize(QtCore.QSize(32, 32))
        self.column_up.setObjectName("column_up")
        self.verticalLayout_3.addWidget(self.column_up)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.verticalLayout_3.addItem(spacerItem)
        self.del_custcol_button = QtWidgets.QToolButton(parent=Form)
        icon1 = QtGui.QIcon.ic("minus.png")
        self.del_custcol_button.setIcon(icon1)
        self.del_custcol_button.setIconSize(QtCore.QSize(32, 32))
        self.del_custcol_button.setObjectName("del_custcol_button")
        self.verticalLayout_3.addWidget(self.del_custcol_button)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.verticalLayout_3.addItem(spacerItem1)
        self.add_custcol_button = QtWidgets.QToolButton(parent=Form)
        icon2 = QtGui.QIcon.ic("plus.png")
        self.add_custcol_button.setIcon(icon2)
        self.add_custcol_button.setIconSize(QtCore.QSize(32, 32))
        self.add_custcol_button.setObjectName("add_custcol_button")
        self.verticalLayout_3.addWidget(self.add_custcol_button)
        spacerItem2 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.verticalLayout_3.addItem(spacerItem2)
        self.edit_custcol_button = QtWidgets.QToolButton(parent=Form)
        icon3 = QtGui.QIcon.ic("edit_input.png")
        self.edit_custcol_button.setIcon(icon3)
        self.edit_custcol_button.setIconSize(QtCore.QSize(32, 32))
        self.edit_custcol_button.setObjectName("edit_custcol_button")
        self.verticalLayout_3.addWidget(self.edit_custcol_button)
        spacerItem3 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.verticalLayout_3.addItem(spacerItem3)
        self.column_down = QtWidgets.QToolButton(parent=Form)
        icon4 = QtGui.QIcon.ic("arrow-down.png")
        self.column_down.setIcon(icon4)
        self.column_down.setIconSize(QtCore.QSize(32, 32))
        self.column_down.setObjectName("column_down")
        self.verticalLayout_3.addWidget(self.column_down)
        self.gridLayout.addLayout(self.verticalLayout_3, 1, 1, 1, 1)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.add_col_button = QtWidgets.QPushButton(parent=Form)
        self.add_col_button.setIcon(icon2)
        self.add_col_button.setObjectName("add_col_button")
        self.horizontalLayout.addWidget(self.add_col_button)
        spacerItem4 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout.addItem(spacerItem4)
        self.show_all_button = QtWidgets.QPushButton(parent=Form)
        self.show_all_button.setObjectName("show_all_button")
        self.horizontalLayout.addWidget(self.show_all_button)
        self.hide_all_button = QtWidgets.QPushButton(parent=Form)
        self.hide_all_button.setObjectName("hide_all_button")
        self.horizontalLayout.addWidget(self.hide_all_button)
        self.gridLayout.addLayout(self.horizontalLayout, 2, 0, 1, 2)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):

        Form.setWindowTitle(_("Form"))
        self.label.setText(_("Here you can re-arrange the layout of the columns in the calibre library book list. You can hide columns by unchecking them. You can also create your own, custom columns."))
        self.column_up.setToolTip(_("<p>Click to move the row up 1. Shift+click to move the row up 5.\n"
"Ctrl+click to move the row up 10. Ctrl+Shift+click to move the row to the top.</p>"))
        self.column_up.setText(_("..."))
        self.del_custcol_button.setToolTip(_("Remove a user-defined column"))
        self.del_custcol_button.setText(_("..."))
        self.add_custcol_button.setToolTip(_("Add a user-defined column"))
        self.add_custcol_button.setText(_("..."))
        self.edit_custcol_button.setToolTip(_("Edit settings of a user-defined column"))
        self.edit_custcol_button.setText(_("..."))
        self.column_down.setToolTip(_("<p>Click to move the row down 1. Shift+click to move the row down 5.\n"
"Ctrl+click to move the row down 10. Ctrl+Shift+click to move the row to the bottom.</p>"))
        self.column_down.setText(_("..."))
        self.add_col_button.setText(_("Add &custom column"))
        self.show_all_button.setText(_("&Show all"))
        self.hide_all_button.setText(_("&Hide all"))
from calibre.gui2.preferences import TableWidgetWithMoveByKeyPress
