# Form implementation generated from reading ui file '/home/kovid/work/calibre/src/calibre/gui2/preferences/behavior.ui'
#
# Created by: PyQt6 UI code generator 6.6.1
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(941, 563)
        self.horizontalLayout = QtWidgets.QHBoxLayout(Form)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.formLayout = QtWidgets.QFormLayout()
        self.formLayout.setObjectName("formLayout")
        self.opt_new_version_notification = QtWidgets.QCheckBox(parent=Form)
        self.opt_new_version_notification.setObjectName("opt_new_version_notification")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.ItemRole.SpanningRole, self.opt_new_version_notification)
        self.opt_output_format = QtWidgets.QComboBox(parent=Form)
        self.opt_output_format.setSizeAdjustPolicy(QtWidgets.QComboBox.SizeAdjustPolicy.AdjustToMinimumContentsLengthWithIcon)
        self.opt_output_format.setMinimumContentsLength(10)
        self.opt_output_format.setObjectName("opt_output_format")
        self.formLayout.setWidget(7, QtWidgets.QFormLayout.ItemRole.FieldRole, self.opt_output_format)
        self.priority_label = QtWidgets.QLabel(parent=Form)
        self.priority_label.setObjectName("priority_label")
        self.formLayout.setWidget(9, QtWidgets.QFormLayout.ItemRole.LabelRole, self.priority_label)
        self.opt_worker_process_priority = QtWidgets.QComboBox(parent=Form)
        self.opt_worker_process_priority.setSizeAdjustPolicy(QtWidgets.QComboBox.SizeAdjustPolicy.AdjustToMinimumContentsLengthWithIcon)
        self.opt_worker_process_priority.setMinimumContentsLength(20)
        self.opt_worker_process_priority.setObjectName("opt_worker_process_priority")
        self.opt_worker_process_priority.addItem("")
        self.opt_worker_process_priority.addItem("")
        self.opt_worker_process_priority.addItem("")
        self.formLayout.setWidget(9, QtWidgets.QFormLayout.ItemRole.FieldRole, self.opt_worker_process_priority)
        self.groupBox_5 = QtWidgets.QGroupBox(parent=Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Preferred, QtWidgets.QSizePolicy.Policy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(2)
        sizePolicy.setHeightForWidth(self.groupBox_5.sizePolicy().hasHeightForWidth())
        self.groupBox_5.setSizePolicy(sizePolicy)
        self.groupBox_5.setObjectName("groupBox_5")
        self.verticalLayout_11 = QtWidgets.QVBoxLayout(self.groupBox_5)
        self.verticalLayout_11.setObjectName("verticalLayout_11")
        self.horizontalLayout_10 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_10.setObjectName("horizontalLayout_10")
        self.opt_input_order = ListWidgetWithMoveByKeyPress(parent=self.groupBox_5)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.opt_input_order.sizePolicy().hasHeightForWidth())
        self.opt_input_order.setSizePolicy(sizePolicy)
        self.opt_input_order.setDragEnabled(True)
        self.opt_input_order.setDragDropMode(QtWidgets.QAbstractItemView.DragDropMode.InternalMove)
        self.opt_input_order.setDefaultDropAction(QtCore.Qt.DropAction.MoveAction)
        self.opt_input_order.setAlternatingRowColors(True)
        self.opt_input_order.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectionBehavior.SelectRows)
        self.opt_input_order.setObjectName("opt_input_order")
        self.horizontalLayout_10.addWidget(self.opt_input_order)
        self.verticalLayout_10 = QtWidgets.QVBoxLayout()
        self.verticalLayout_10.setObjectName("verticalLayout_10")
        self.input_up_button = QtWidgets.QToolButton(parent=self.groupBox_5)
        icon = QtGui.QIcon.ic("arrow-up.png")
        self.input_up_button.setIcon(icon)
        self.input_up_button.setObjectName("input_up_button")
        self.verticalLayout_10.addWidget(self.input_up_button)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.verticalLayout_10.addItem(spacerItem)
        self.input_down_button = QtWidgets.QToolButton(parent=self.groupBox_5)
        icon1 = QtGui.QIcon.ic("arrow-down.png")
        self.input_down_button.setIcon(icon1)
        self.input_down_button.setObjectName("input_down_button")
        self.verticalLayout_10.addWidget(self.input_down_button)
        self.horizontalLayout_10.addLayout(self.verticalLayout_10)
        self.verticalLayout_11.addLayout(self.horizontalLayout_10)
        self.formLayout.setWidget(15, QtWidgets.QFormLayout.ItemRole.SpanningRole, self.groupBox_5)
        self.label_23 = QtWidgets.QLabel(parent=Form)
        self.label_23.setObjectName("label_23")
        self.formLayout.setWidget(7, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_23)
        self.opt_numeric_collation = QtWidgets.QCheckBox(parent=Form)
        self.opt_numeric_collation.setObjectName("opt_numeric_collation")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.ItemRole.SpanningRole, self.opt_numeric_collation)
        self.horizontalLayout.addLayout(self.formLayout)
        self.line = QtWidgets.QFrame(parent=Form)
        self.line.setFrameShape(QtWidgets.QFrame.Shape.VLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Shadow.Sunken)
        self.line.setObjectName("line")
        self.horizontalLayout.addWidget(self.line)
        self.formLayout_2 = QtWidgets.QFormLayout()
        self.formLayout_2.setObjectName("formLayout_2")
        self.opt_bools_are_tristate = QtWidgets.QCheckBox(parent=Form)
        self.opt_bools_are_tristate.setObjectName("opt_bools_are_tristate")
        self.formLayout_2.setWidget(3, QtWidgets.QFormLayout.ItemRole.SpanningRole, self.opt_bools_are_tristate)
        self.label_2 = QtWidgets.QLabel(parent=Form)
        self.label_2.setObjectName("label_2")
        self.formLayout_2.setWidget(7, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_2)
        self.opt_network_timeout = QtWidgets.QSpinBox(parent=Form)
        self.opt_network_timeout.setMinimum(2)
        self.opt_network_timeout.setMaximum(120)
        self.opt_network_timeout.setProperty("value", 5)
        self.opt_network_timeout.setObjectName("opt_network_timeout")
        self.formLayout_2.setWidget(7, QtWidgets.QFormLayout.ItemRole.FieldRole, self.opt_network_timeout)
        self.opt_virtual_lib_on_startup = QtWidgets.QComboBox(parent=Form)
        self.opt_virtual_lib_on_startup.setMaximumSize(QtCore.QSize(250, 16777215))
        self.opt_virtual_lib_on_startup.setSizeAdjustPolicy(QtWidgets.QComboBox.SizeAdjustPolicy.AdjustToMinimumContentsLengthWithIcon)
        self.opt_virtual_lib_on_startup.setMinimumContentsLength(15)
        self.opt_virtual_lib_on_startup.setObjectName("opt_virtual_lib_on_startup")
        self.formLayout_2.setWidget(9, QtWidgets.QFormLayout.ItemRole.FieldRole, self.opt_virtual_lib_on_startup)
        self.reset_confirmation_button = QtWidgets.QPushButton(parent=Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Maximum, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.reset_confirmation_button.sizePolicy().hasHeightForWidth())
        self.reset_confirmation_button.setSizePolicy(sizePolicy)
        self.reset_confirmation_button.setObjectName("reset_confirmation_button")
        self.formLayout_2.setWidget(10, QtWidgets.QFormLayout.ItemRole.SpanningRole, self.reset_confirmation_button)
        self.groupBox_3 = QtWidgets.QGroupBox(parent=Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Preferred, QtWidgets.QSizePolicy.Policy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(3)
        sizePolicy.setHeightForWidth(self.groupBox_3.sizePolicy().hasHeightForWidth())
        self.groupBox_3.setSizePolicy(sizePolicy)
        self.groupBox_3.setObjectName("groupBox_3")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.groupBox_3)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.opt_internally_viewed_formats = QtWidgets.QListWidget(parent=self.groupBox_3)
        self.opt_internally_viewed_formats.setAlternatingRowColors(True)
        self.opt_internally_viewed_formats.setSelectionMode(QtWidgets.QAbstractItemView.SelectionMode.NoSelection)
        self.opt_internally_viewed_formats.setObjectName("opt_internally_viewed_formats")
        self.gridLayout_4.addWidget(self.opt_internally_viewed_formats, 0, 0, 1, 1)
        self.formLayout_2.setWidget(11, QtWidgets.QFormLayout.ItemRole.SpanningRole, self.groupBox_3)
        self.label_170 = QtWidgets.QLabel(parent=Form)
        self.label_170.setObjectName("label_170")
        self.formLayout_2.setWidget(8, QtWidgets.QFormLayout.ItemRole.SpanningRole, self.label_170)
        self.opt_upload_news_to_device = QtWidgets.QCheckBox(parent=Form)
        self.opt_upload_news_to_device.setObjectName("opt_upload_news_to_device")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.ItemRole.SpanningRole, self.opt_upload_news_to_device)
        self.opt_delete_news_from_library_on_upload = QtWidgets.QCheckBox(parent=Form)
        self.opt_delete_news_from_library_on_upload.setObjectName("opt_delete_news_from_library_on_upload")
        self.formLayout_2.setWidget(2, QtWidgets.QFormLayout.ItemRole.SpanningRole, self.opt_delete_news_from_library_on_upload)
        self.horizontalLayout.addLayout(self.formLayout_2)
        self.priority_label.setBuddy(self.opt_worker_process_priority)
        self.label_23.setBuddy(self.opt_output_format)
        self.label_2.setBuddy(self.opt_network_timeout)
        self.label_170.setBuddy(self.opt_virtual_lib_on_startup)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):

        Form.setWindowTitle(_("Form"))
        self.opt_new_version_notification.setText(_("Notify when a &new version is available"))
        self.priority_label.setText(_("Job &priority:"))
        self.opt_worker_process_priority.setItemText(0, _("Normal"))
        self.opt_worker_process_priority.setItemText(1, _("High"))
        self.opt_worker_process_priority.setItemText(2, _("Low"))
        self.groupBox_5.setTitle(_("Preferred &input format order:"))
        self.opt_input_order.setToolTip(_("When multiple formats are present in a book being converted, this list determines which format will be used as the \"source\" for the conversion."))
        self.input_up_button.setText(_("..."))
        self.input_down_button.setText(_("..."))
        self.label_23.setText(_("Preferred &output format:"))
        self.opt_numeric_collation.setToolTip(_("<p>Setting this means that when sorting on text fields\n"
"like title the text \"Book 2\" will sort before the text \"Book 100\".\n"
"Note that setting this can cause problems with text that starts\n"
"with numbers and is a little slower. This setting is per user, not\n"
"per library.</p>"))
        self.opt_numeric_collation.setText(_("Recognize numbers inside text when sorting (needs restart)"))
        self.opt_bools_are_tristate.setToolTip(_("If checked, Yes/No custom columns values can be Yes, No, or Unknown.\n"
"If not checked, the values can be Yes or No."))
        self.opt_bools_are_tristate.setText(_("&Yes/No columns have three values (needs restart)"))
        self.label_2.setText(_("Default network &timeout:"))
        self.opt_network_timeout.setToolTip(_("Set the default timeout for network fetches (i.e. anytime we go out to the internet to get information)"))
        self.opt_network_timeout.setSuffix(_(" seconds"))
        self.opt_virtual_lib_on_startup.setToolTip(_("Use this Virtual library on calibre startup if the current library is being used. Also applied when switching to this library. Note that this setting is per library. "))
        self.reset_confirmation_button.setText(_("Reset all disabled &confirmation dialogs"))
        self.groupBox_3.setTitle(_("Use internal &viewer for:"))
        self.opt_internally_viewed_formats.setToolTip(_("File types that are unchecked in this list will be opened with the default program configured in the operating system, for that file type."))
        self.label_170.setText(_("Virtual library to apply when the current &library is opened:"))
        self.opt_upload_news_to_device.setText(_("Automatically send downloaded &news to e-book reader"))
        self.opt_delete_news_from_library_on_upload.setText(_("&Delete news from library when it is automatically sent to reader"))
from calibre.gui2.preferences import ListWidgetWithMoveByKeyPress
