# Form implementation generated from reading ui file '/home/kovid/work/calibre/src/calibre/gui2/preferences/look_feel.ui'
#
# Created by: PyQt6 UI code generator 6.8.1
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(1096, 791)
        self.gridLayout_2 = QtWidgets.QGridLayout(Form)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.tabWidget = ScrollingTabWidget(parent=Form)
        self.tabWidget.setObjectName("tabWidget")
        self.main_interface_tab = MainInterfaceTab()
        self.main_interface_tab.setObjectName("main_interface_tab")
        icon = QtGui.QIcon.ic("lt.png")
        self.tabWidget.addTab(self.main_interface_tab, icon, "")
        self.cover_grid_tab = CoverGridTab()
        self.cover_grid_tab.setObjectName("cover_grid_tab")
        icon1 = QtGui.QIcon.ic("grid.png")
        self.tabWidget.addTab(self.cover_grid_tab, icon1, "")
        self.book_details_tab = BookDetailsTab()
        self.book_details_tab.setObjectName("book_details_tab")
        icon2 = QtGui.QIcon.ic("book.png")
        self.tabWidget.addTab(self.book_details_tab, icon2, "")
        self.edit_metadata_tab = EditMetadataTab()
        self.edit_metadata_tab.setObjectName("edit_metadata_tab")
        icon3 = QtGui.QIcon.ic("edit_input.png")
        self.tabWidget.addTab(self.edit_metadata_tab, icon3, "")
        self.tag_browser_tab = QtWidgets.QTabWidget()
        self.tag_browser_tab.setObjectName("tag_browser_tab")
        self.tb_display_tab = TbDisplayTab()
        self.tb_display_tab.setObjectName("tb_display_tab")
        self.tag_browser_tab.addTab(self.tb_display_tab, "")
        self.tb_partioning_tab = TbPartitioningTab()
        self.tb_partioning_tab.setObjectName("tb_partioning_tab")
        self.tag_browser_tab.addTab(self.tb_partioning_tab, "")
        self.tb_hierarchy_tab = TbHierarchyTab()
        self.tb_hierarchy_tab.setObjectName("tb_hierarchy_tab")
        self.tag_browser_tab.addTab(self.tb_hierarchy_tab, "")
        self.tb_icon_browser_tab = TbIconRulesTab()
        self.tb_icon_browser_tab.setObjectName("tb_icon_browser_tab")
        self.tag_browser_tab.addTab(self.tb_icon_browser_tab, "")
        icon4 = QtGui.QIcon.ic("tags.png")
        self.tabWidget.addTab(self.tag_browser_tab, icon4, "")
        self.cover_view = CoverView()
        self.cover_view.setObjectName("cover_view")
        icon5 = QtGui.QIcon.ic("cover_flow.png")
        self.tabWidget.addTab(self.cover_view, icon5, "")
        self.quickview_tab = QuickviewTab()
        self.quickview_tab.setObjectName("quickview_tab")
        icon6 = QtGui.QIcon.ic("quickview.png")
        self.tabWidget.addTab(self.quickview_tab, icon6, "")
        self.edit_rules = ColumnColorRules()
        self.edit_rules.setObjectName("edit_rules")
        icon7 = QtGui.QIcon.ic("format-fill-color.png")
        self.tabWidget.addTab(self.edit_rules, icon7, "")
        self.icon_rules = ColumnIconRules()
        self.icon_rules.setObjectName("icon_rules")
        icon8 = QtGui.QIcon.ic("icon_choose.png")
        self.tabWidget.addTab(self.icon_rules, icon8, "")
        self.gridLayout_2.addWidget(self.tabWidget, 0, 1, 1, 1)
        self.sections_view = QtWidgets.QListWidget(parent=Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.sections_view.sizePolicy().hasHeightForWidth())
        self.sections_view.setSizePolicy(sizePolicy)
        self.sections_view.setObjectName("sections_view")
        self.gridLayout_2.addWidget(self.sections_view, 0, 0, 1, 1)

        self.retranslateUi(Form)
        self.tabWidget.setCurrentIndex(0)

    def retranslateUi(self, Form):

        Form.setWindowTitle(_("Form"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.main_interface_tab), _("&Main interface"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.cover_grid_tab), _("Cover &grid"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.book_details_tab), _("&Book details"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.edit_metadata_tab), _("Edit &metadata"))
        self.tag_browser_tab.setTabText(self.tag_browser_tab.indexOf(self.tb_display_tab), _("Di&splay"))
        self.tag_browser_tab.setTabText(self.tag_browser_tab.indexOf(self.tb_partioning_tab), _("&Partitioning and collapsing"))
        self.tag_browser_tab.setTabText(self.tag_browser_tab.indexOf(self.tb_hierarchy_tab), _("&Hierarchy and searching"))
        self.tag_browser_tab.setTabText(self.tag_browser_tab.indexOf(self.tb_icon_browser_tab), _("Val&ue icon rules"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tag_browser_tab), _("&Tag browser"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.cover_view), _("Cover &browser"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.quickview_tab), _("&Quickview"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.edit_rules), _("Column &coloring"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.icon_rules), _("Column &icons"))
from calibre.gui2.preferences.look_feel_tabs import ColumnColorRules, ColumnIconRules
from calibre.gui2.preferences.look_feel_tabs.book_details import BookDetailsTab
from calibre.gui2.preferences.look_feel_tabs.cover_grid import CoverGridTab
from calibre.gui2.preferences.look_feel_tabs.cover_view import CoverView
from calibre.gui2.preferences.look_feel_tabs.edit_metadata import EditMetadataTab
from calibre.gui2.preferences.look_feel_tabs.main_interface import MainInterfaceTab
from calibre.gui2.preferences.look_feel_tabs.quickview import QuickviewTab
from calibre.gui2.preferences.look_feel_tabs.tb_display import TbDisplayTab
from calibre.gui2.preferences.look_feel_tabs.tb_hierarchy import TbHierarchyTab
from calibre.gui2.preferences.look_feel_tabs.tb_icon_rules import TbIconRulesTab
from calibre.gui2.preferences.look_feel_tabs.tb_partitioning import TbPartitioningTab
from calibre.gui2.widgets2 import ScrollingTabWidget
