# Form implementation generated from reading ui file '/home/kovid/work/calibre/src/calibre/gui2/preferences/look_feel_tabs/cover_view.ui'
#
# Created by: PyQt6 UI code generator 6.8.1
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_Form(object):
    def setupUi(self, cover_browser_tab):
        cover_browser_tab.setObjectName("cover_browser_tab")
        self.gridLayout_cover_browser = QtWidgets.QGridLayout(cover_browser_tab)
        self.gridLayout_cover_browser.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_cover_browser.setObjectName("gridLayout_cover_browser")
        self.opt_cb_fullscreen = QtWidgets.QCheckBox(parent=cover_browser_tab)
        self.opt_cb_fullscreen.setObjectName("opt_cb_fullscreen")
        self.gridLayout_cover_browser.addWidget(self.opt_cb_fullscreen, 0, 1, 1, 2)
        self.opt_separate_cover_flow = QtWidgets.QCheckBox(parent=cover_browser_tab)
        self.opt_separate_cover_flow.setObjectName("opt_separate_cover_flow")
        self.gridLayout_cover_browser.addWidget(self.opt_separate_cover_flow, 0, 0, 1, 3)
        self.opt_cover_flow_queue_length = QtWidgets.QSpinBox(parent=cover_browser_tab)
        self.opt_cover_flow_queue_length.setObjectName("opt_cover_flow_queue_length")
        self.gridLayout_cover_browser.addWidget(self.opt_cover_flow_queue_length, 7, 1, 1, 2)
        self.label_61 = QtWidgets.QLabel(parent=cover_browser_tab)
        self.label_61.setObjectName("label_61")
        self.gridLayout_cover_browser.addWidget(self.label_61, 7, 0, 1, 1)
        self.cover_browser_title_template_button = QtWidgets.QPushButton(parent=cover_browser_tab)
        self.cover_browser_title_template_button.setObjectName("cover_browser_title_template_button")
        self.gridLayout_cover_browser.addWidget(self.cover_browser_title_template_button, 8, 2, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(690, 20, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.gridLayout_cover_browser.addItem(spacerItem, 11, 0, 1, 2)
        self.opt_cover_browser_title_template = QtWidgets.QLineEdit(parent=cover_browser_tab)
        self.opt_cover_browser_title_template.setObjectName("opt_cover_browser_title_template")
        self.gridLayout_cover_browser.addWidget(self.opt_cover_browser_title_template, 8, 1, 1, 1)
        self.opt_cover_browser_reflections = QtWidgets.QCheckBox(parent=cover_browser_tab)
        self.opt_cover_browser_reflections.setObjectName("opt_cover_browser_reflections")
        self.gridLayout_cover_browser.addWidget(self.opt_cover_browser_reflections, 4, 0, 1, 1)
        self.opt_cb_preserve_aspect_ratio = QtWidgets.QCheckBox(parent=cover_browser_tab)
        self.opt_cb_preserve_aspect_ratio.setObjectName("opt_cb_preserve_aspect_ratio")
        self.gridLayout_cover_browser.addWidget(self.opt_cb_preserve_aspect_ratio, 4, 1, 1, 2)
        self.label_987 = QtWidgets.QLabel(parent=cover_browser_tab)
        self.label_987.setObjectName("label_987")
        self.gridLayout_cover_browser.addWidget(self.label_987, 6, 0, 1, 1)
        self.opt_cover_browser_narrow_view_position = QtWidgets.QComboBox(parent=cover_browser_tab)
        self.opt_cover_browser_narrow_view_position.setObjectName("opt_cover_browser_narrow_view_position")
        self.gridLayout_cover_browser.addWidget(self.opt_cover_browser_narrow_view_position, 6, 1, 1, 2)
        self.opt_cover_browser_subtitle_field = QtWidgets.QComboBox(parent=cover_browser_tab)
        self.opt_cover_browser_subtitle_field.setObjectName("opt_cover_browser_subtitle_field")
        self.gridLayout_cover_browser.addWidget(self.opt_cover_browser_subtitle_field, 9, 1, 1, 2)
        self.label_23 = QtWidgets.QLabel(parent=cover_browser_tab)
        self.label_23.setObjectName("label_23")
        self.gridLayout_cover_browser.addWidget(self.label_23, 8, 0, 1, 1)
        self.label_24 = QtWidgets.QLabel(parent=cover_browser_tab)
        self.label_24.setObjectName("label_24")
        self.gridLayout_cover_browser.addWidget(self.label_24, 9, 0, 1, 1)
        self.fs_help_msg = QtWidgets.QLabel(parent=cover_browser_tab)
        self.fs_help_msg.setStyleSheet("margin-left: 1.5em")
        self.fs_help_msg.setWordWrap(True)
        self.fs_help_msg.setObjectName("fs_help_msg")
        self.gridLayout_cover_browser.addWidget(self.fs_help_msg, 1, 1, 1, 2)
        self.label_26 = QtWidgets.QLabel(parent=cover_browser_tab)
        self.label_26.setObjectName("label_26")
        self.gridLayout_cover_browser.addWidget(self.label_26, 10, 0, 1, 1)
        self.opt_books_autoscroll_time = QtWidgets.QDoubleSpinBox(parent=cover_browser_tab)
        self.opt_books_autoscroll_time.setDecimals(1)
        self.opt_books_autoscroll_time.setMinimum(1.0)
        self.opt_books_autoscroll_time.setMaximum(100000.0)
        self.opt_books_autoscroll_time.setObjectName("opt_books_autoscroll_time")
        self.gridLayout_cover_browser.addWidget(self.opt_books_autoscroll_time, 10, 1, 1, 1)
        self.opt_cb_double_click_to_activate = QtWidgets.QCheckBox(parent=cover_browser_tab)
        self.opt_cb_double_click_to_activate.setObjectName("opt_cb_double_click_to_activate")
        self.gridLayout_cover_browser.addWidget(self.opt_cb_double_click_to_activate, 5, 0, 1, 2)
        self.label_61.setBuddy(self.opt_cover_flow_queue_length)
        self.label_987.setBuddy(self.opt_cover_browser_narrow_view_position)
        self.label_23.setBuddy(self.opt_cover_browser_title_template)
        self.label_24.setBuddy(self.opt_cover_browser_subtitle_field)
        self.label_26.setBuddy(self.opt_books_autoscroll_time)

        self.retranslateUi(cover_browser_tab)

    def retranslateUi(self, cover_browser_tab):

        self.opt_cb_fullscreen.setText(_("When showing in a separate window, show it &fullscreen"))
        self.opt_separate_cover_flow.setText(_("Show in a &separate window (needs restart)"))
        self.label_61.setText(_("&Number of covers to show in browse mode (needs restart):"))
        self.cover_browser_title_template_button.setText(_("Template &editor"))
        self.opt_cover_browser_title_template.setToolTip(_("<p>The template used to generate the text below the covers. This template uses\n"
"the same syntax as save templates. Defaults to just the book title.\n"
"Note that this setting is per-library, which means that you have to\n"
"set it again for every different calibre library you use. Use an\n"
"empty template for no text.</p>"))
        self.opt_cover_browser_reflections.setText(_("Show &reflections"))
        self.opt_cb_preserve_aspect_ratio.setToolTip(_("Show covers in their original aspect ratio instead of resizing\n"
"them to all have the same width and height"))
        self.opt_cb_preserve_aspect_ratio.setText(_("Preserve &aspect ratio of covers"))
        self.label_987.setText(_("Cover browser &position in the narrow layout:"))
        self.label_23.setText(_("&Template for caption:"))
        self.label_24.setText(_("Fie&ld for sub-title:"))
        self.fs_help_msg.setText(_("You can press the %s key to toggle full screen mode."))
        self.label_26.setText(_("Show ne&xt cover during auto scroll after:"))
        self.opt_books_autoscroll_time.setSuffix(_(" seconds"))
        self.opt_cb_double_click_to_activate.setText(_("&Double click to view the central book, instead of single click"))
