# autogenerated by __main__.py do not edit
name_map = {'PYQT_VERSION': 'PyQt6.QtCore',
 'PYQT_VERSION_STR': 'PyQt6.QtCore',
 'QAbstractAnimation': 'PyQt6.QtCore',
 'QAbstractButton': 'PyQt6.QtWidgets',
 'QAbstractEventDispatcher': 'PyQt6.QtCore',
 'QAbstractFileIconProvider': 'PyQt6.QtGui',
 'QAbstractGraphicsShapeItem': 'PyQt6.QtWidgets',
 'QAbstractItemDelegate': 'PyQt6.QtWidgets',
 'QAbstractItemModel': 'PyQt6.QtCore',
 'QAbstractItemView': 'PyQt6.QtWidgets',
 'QAbstractListModel': 'PyQt6.QtCore',
 'QAbstractNativeEventFilter': 'PyQt6.QtCore',
 'QAbstractNetworkCache': 'PyQt6.QtNetwork',
 'QAbstractOpenGLFunctions': 'PyQt6.QtOpenGL',
 'QAbstractPrintDialog': 'PyQt6.QtPrintSupport',
 'QAbstractProxyModel': 'PyQt6.QtCore',
 'QAbstractScrollArea': 'PyQt6.QtWidgets',
 'QAbstractSlider': 'PyQt6.QtWidgets',
 'QAbstractSocket': 'PyQt6.QtNetwork',
 'QAbstractSpinBox': 'PyQt6.QtWidgets',
 'QAbstractTableModel': 'PyQt6.QtCore',
 'QAbstractTextDocumentLayout': 'PyQt6.QtGui',
 'QAction': 'PyQt6.QtGui',
 'QActionEvent': 'PyQt6.QtGui',
 'QActionGroup': 'PyQt6.QtGui',
 'QAnimationGroup': 'PyQt6.QtCore',
 'QApplication': 'PyQt6.QtWidgets',
 'QAudio': 'PyQt6.QtMultimedia',
 'QAudioBuffer': 'PyQt6.QtMultimedia',
 'QAudioDecoder': 'PyQt6.QtMultimedia',
 'QAudioDevice': 'PyQt6.QtMultimedia',
 'QAudioFormat': 'PyQt6.QtMultimedia',
 'QAudioInput': 'PyQt6.QtMultimedia',
 'QAudioOutput': 'PyQt6.QtMultimedia',
 'QAudioSink': 'PyQt6.QtMultimedia',
 'QAudioSource': 'PyQt6.QtMultimedia',
 'QAuthenticator': 'PyQt6.QtNetwork',
 'QBackingStore': 'PyQt6.QtGui',
 'QBasicTimer': 'PyQt6.QtCore',
 'QBitArray': 'PyQt6.QtCore',
 'QBitmap': 'PyQt6.QtGui',
 'QBluetoothPermission': 'PyQt6.QtCore',
 'QBoxLayout': 'PyQt6.QtWidgets',
 'QBrush': 'PyQt6.QtGui',
 'QBuffer': 'PyQt6.QtCore',
 'QButtonGroup': 'PyQt6.QtWidgets',
 'QByteArray': 'PyQt6.QtCore',
 'QByteArrayMatcher': 'PyQt6.QtCore',
 'QCalendar': 'PyQt6.QtCore',
 'QCalendarPermission': 'PyQt6.QtCore',
 'QCalendarWidget': 'PyQt6.QtWidgets',
 'QCamera': 'PyQt6.QtMultimedia',
 'QCameraDevice': 'PyQt6.QtMultimedia',
 'QCameraFormat': 'PyQt6.QtMultimedia',
 'QCameraPermission': 'PyQt6.QtCore',
 'QCapturableWindow': 'PyQt6.QtMultimedia',
 'QCborError': 'PyQt6.QtCore',
 'QCborKnownTags': 'PyQt6.QtCore',
 'QCborSimpleType': 'PyQt6.QtCore',
 'QCborStreamReader': 'PyQt6.QtCore',
 'QCborStreamWriter': 'PyQt6.QtCore',
 'QCheckBox': 'PyQt6.QtWidgets',
 'QChildEvent': 'PyQt6.QtCore',
 'QChildWindowEvent': 'PyQt6.QtGui',
 'QClipboard': 'PyQt6.QtGui',
 'QCloseEvent': 'PyQt6.QtGui',
 'QCollator': 'PyQt6.QtCore',
 'QCollatorSortKey': 'PyQt6.QtCore',
 'QColor': 'PyQt6.QtGui',
 'QColorConstants': 'PyQt6.QtGui',
 'QColorDialog': 'PyQt6.QtWidgets',
 'QColorSpace': 'PyQt6.QtGui',
 'QColorTransform': 'PyQt6.QtGui',
 'QColumnView': 'PyQt6.QtWidgets',
 'QComboBox': 'PyQt6.QtWidgets',
 'QCommandLineOption': 'PyQt6.QtCore',
 'QCommandLineParser': 'PyQt6.QtCore',
 'QCommandLinkButton': 'PyQt6.QtWidgets',
 'QCommonStyle': 'PyQt6.QtWidgets',
 'QCompleter': 'PyQt6.QtWidgets',
 'QConcatenateTablesProxyModel': 'PyQt6.QtCore',
 'QConicalGradient': 'PyQt6.QtGui',
 'QContactsPermission': 'PyQt6.QtCore',
 'QContextMenuEvent': 'PyQt6.QtGui',
 'QCoreApplication': 'PyQt6.QtCore',
 'QCryptographicHash': 'PyQt6.QtCore',
 'QCursor': 'PyQt6.QtGui',
 'QDataStream': 'PyQt6.QtCore',
 'QDataWidgetMapper': 'PyQt6.QtWidgets',
 'QDate': 'PyQt6.QtCore',
 'QDateEdit': 'PyQt6.QtWidgets',
 'QDateTime': 'PyQt6.QtCore',
 'QDateTimeEdit': 'PyQt6.QtWidgets',
 'QDeadlineTimer': 'PyQt6.QtCore',
 'QDesktopServices': 'PyQt6.QtGui',
 'QDial': 'PyQt6.QtWidgets',
 'QDialog': 'PyQt6.QtWidgets',
 'QDialogButtonBox': 'PyQt6.QtWidgets',
 'QDir': 'PyQt6.QtCore',
 'QDirIterator': 'PyQt6.QtCore',
 'QDnsDomainNameRecord': 'PyQt6.QtNetwork',
 'QDnsHostAddressRecord': 'PyQt6.QtNetwork',
 'QDnsLookup': 'PyQt6.QtNetwork',
 'QDnsMailExchangeRecord': 'PyQt6.QtNetwork',
 'QDnsServiceRecord': 'PyQt6.QtNetwork',
 'QDnsTextRecord': 'PyQt6.QtNetwork',
 'QDockWidget': 'PyQt6.QtWidgets',
 'QDoubleSpinBox': 'PyQt6.QtWidgets',
 'QDoubleValidator': 'PyQt6.QtGui',
 'QDrag': 'PyQt6.QtGui',
 'QDragEnterEvent': 'PyQt6.QtGui',
 'QDragLeaveEvent': 'PyQt6.QtGui',
 'QDragMoveEvent': 'PyQt6.QtGui',
 'QDropEvent': 'PyQt6.QtGui',
 'QDynamicPropertyChangeEvent': 'PyQt6.QtCore',
 'QEasingCurve': 'PyQt6.QtCore',
 'QElapsedTimer': 'PyQt6.QtCore',
 'QEnterEvent': 'PyQt6.QtGui',
 'QErrorMessage': 'PyQt6.QtWidgets',
 'QEvent': 'PyQt6.QtCore',
 'QEventLoop': 'PyQt6.QtCore',
 'QEventLoopLocker': 'PyQt6.QtCore',
 'QEventPoint': 'PyQt6.QtGui',
 'QExposeEvent': 'PyQt6.QtGui',
 'QFile': 'PyQt6.QtCore',
 'QFileDevice': 'PyQt6.QtCore',
 'QFileDialog': 'PyQt6.QtWidgets',
 'QFileIconProvider': 'PyQt6.QtWidgets',
 'QFileInfo': 'PyQt6.QtCore',
 'QFileOpenEvent': 'PyQt6.QtGui',
 'QFileSelector': 'PyQt6.QtCore',
 'QFileSystemModel': 'PyQt6.QtGui',
 'QFileSystemWatcher': 'PyQt6.QtCore',
 'QFocusEvent': 'PyQt6.QtGui',
 'QFocusFrame': 'PyQt6.QtWidgets',
 'QFont': 'PyQt6.QtGui',
 'QFontComboBox': 'PyQt6.QtWidgets',
 'QFontDatabase': 'PyQt6.QtGui',
 'QFontDialog': 'PyQt6.QtWidgets',
 'QFontInfo': 'PyQt6.QtGui',
 'QFontMetrics': 'PyQt6.QtGui',
 'QFontMetricsF': 'PyQt6.QtGui',
 'QFormLayout': 'PyQt6.QtWidgets',
 'QFrame': 'PyQt6.QtWidgets',
 'QGenericArgument': 'PyQt6.QtCore',
 'QGenericReturnArgument': 'PyQt6.QtCore',
 'QGesture': 'PyQt6.QtWidgets',
 'QGestureEvent': 'PyQt6.QtWidgets',
 'QGestureRecognizer': 'PyQt6.QtWidgets',
 'QGlyphRun': 'PyQt6.QtGui',
 'QGradient': 'PyQt6.QtGui',
 'QGraphicsAnchor': 'PyQt6.QtWidgets',
 'QGraphicsAnchorLayout': 'PyQt6.QtWidgets',
 'QGraphicsBlurEffect': 'PyQt6.QtWidgets',
 'QGraphicsColorizeEffect': 'PyQt6.QtWidgets',
 'QGraphicsDropShadowEffect': 'PyQt6.QtWidgets',
 'QGraphicsEffect': 'PyQt6.QtWidgets',
 'QGraphicsEllipseItem': 'PyQt6.QtWidgets',
 'QGraphicsGridLayout': 'PyQt6.QtWidgets',
 'QGraphicsItem': 'PyQt6.QtWidgets',
 'QGraphicsItemGroup': 'PyQt6.QtWidgets',
 'QGraphicsLayout': 'PyQt6.QtWidgets',
 'QGraphicsLayoutItem': 'PyQt6.QtWidgets',
 'QGraphicsLineItem': 'PyQt6.QtWidgets',
 'QGraphicsLinearLayout': 'PyQt6.QtWidgets',
 'QGraphicsObject': 'PyQt6.QtWidgets',
 'QGraphicsOpacityEffect': 'PyQt6.QtWidgets',
 'QGraphicsPathItem': 'PyQt6.QtWidgets',
 'QGraphicsPixmapItem': 'PyQt6.QtWidgets',
 'QGraphicsPolygonItem': 'PyQt6.QtWidgets',
 'QGraphicsProxyWidget': 'PyQt6.QtWidgets',
 'QGraphicsRectItem': 'PyQt6.QtWidgets',
 'QGraphicsRotation': 'PyQt6.QtWidgets',
 'QGraphicsScale': 'PyQt6.QtWidgets',
 'QGraphicsScene': 'PyQt6.QtWidgets',
 'QGraphicsSceneContextMenuEvent': 'PyQt6.QtWidgets',
 'QGraphicsSceneDragDropEvent': 'PyQt6.QtWidgets',
 'QGraphicsSceneEvent': 'PyQt6.QtWidgets',
 'QGraphicsSceneHelpEvent': 'PyQt6.QtWidgets',
 'QGraphicsSceneHoverEvent': 'PyQt6.QtWidgets',
 'QGraphicsSceneMouseEvent': 'PyQt6.QtWidgets',
 'QGraphicsSceneMoveEvent': 'PyQt6.QtWidgets',
 'QGraphicsSceneResizeEvent': 'PyQt6.QtWidgets',
 'QGraphicsSceneWheelEvent': 'PyQt6.QtWidgets',
 'QGraphicsSimpleTextItem': 'PyQt6.QtWidgets',
 'QGraphicsTextItem': 'PyQt6.QtWidgets',
 'QGraphicsTransform': 'PyQt6.QtWidgets',
 'QGraphicsVideoItem': 'PyQt6.QtMultimediaWidgets',
 'QGraphicsView': 'PyQt6.QtWidgets',
 'QGraphicsWidget': 'PyQt6.QtWidgets',
 'QGridLayout': 'PyQt6.QtWidgets',
 'QGroupBox': 'PyQt6.QtWidgets',
 'QGuiApplication': 'PyQt6.QtGui',
 'QHBoxLayout': 'PyQt6.QtWidgets',
 'QHeaderView': 'PyQt6.QtWidgets',
 'QHelpEvent': 'PyQt6.QtGui',
 'QHideEvent': 'PyQt6.QtGui',
 'QHostAddress': 'PyQt6.QtNetwork',
 'QHostInfo': 'PyQt6.QtNetwork',
 'QHoverEvent': 'PyQt6.QtGui',
 'QHstsPolicy': 'PyQt6.QtNetwork',
 'QHttp1Configuration': 'PyQt6.QtNetwork',
 'QHttp2Configuration': 'PyQt6.QtNetwork',
 'QHttpHeaders': 'PyQt6.QtNetwork',
 'QHttpMultiPart': 'PyQt6.QtNetwork',
 'QHttpPart': 'PyQt6.QtNetwork',
 'QIODevice': 'PyQt6.QtCore',
 'QIODeviceBase': 'PyQt6.QtCore',
 'QIcon': 'PyQt6.QtGui',
 'QIconDragEvent': 'PyQt6.QtGui',
 'QIconEngine': 'PyQt6.QtGui',
 'QIdentityProxyModel': 'PyQt6.QtCore',
 'QImage': 'PyQt6.QtGui',
 'QImageCapture': 'PyQt6.QtMultimedia',
 'QImageIOHandler': 'PyQt6.QtGui',
 'QImageReader': 'PyQt6.QtGui',
 'QImageWriter': 'PyQt6.QtGui',
 'QInputDevice': 'PyQt6.QtGui',
 'QInputDialog': 'PyQt6.QtWidgets',
 'QInputEvent': 'PyQt6.QtGui',
 'QInputMethod': 'PyQt6.QtGui',
 'QInputMethodEvent': 'PyQt6.QtGui',
 'QInputMethodQueryEvent': 'PyQt6.QtGui',
 'QIntValidator': 'PyQt6.QtGui',
 'QItemDelegate': 'PyQt6.QtWidgets',
 'QItemEditorCreatorBase': 'PyQt6.QtWidgets',
 'QItemEditorFactory': 'PyQt6.QtWidgets',
 'QItemSelection': 'PyQt6.QtCore',
 'QItemSelectionModel': 'PyQt6.QtCore',
 'QItemSelectionRange': 'PyQt6.QtCore',
 'QJsonDocument': 'PyQt6.QtCore',
 'QJsonParseError': 'PyQt6.QtCore',
 'QJsonValue': 'PyQt6.QtCore',
 'QKeyCombination': 'PyQt6.QtCore',
 'QKeyEvent': 'PyQt6.QtGui',
 'QKeySequence': 'PyQt6.QtGui',
 'QKeySequenceEdit': 'PyQt6.QtWidgets',
 'QLCDNumber': 'PyQt6.QtWidgets',
 'QLabel': 'PyQt6.QtWidgets',
 'QLayout': 'PyQt6.QtWidgets',
 'QLayoutItem': 'PyQt6.QtWidgets',
 'QLibrary': 'PyQt6.QtCore',
 'QLibraryInfo': 'PyQt6.QtCore',
 'QLine': 'PyQt6.QtCore',
 'QLineEdit': 'PyQt6.QtWidgets',
 'QLineF': 'PyQt6.QtCore',
 'QLinearGradient': 'PyQt6.QtGui',
 'QListView': 'PyQt6.QtWidgets',
 'QListWidget': 'PyQt6.QtWidgets',
 'QListWidgetItem': 'PyQt6.QtWidgets',
 'QLocalServer': 'PyQt6.QtNetwork',
 'QLocalSocket': 'PyQt6.QtNetwork',
 'QLocale': 'PyQt6.QtCore',
 'QLocationPermission': 'PyQt6.QtCore',
 'QLockFile': 'PyQt6.QtCore',
 'QLoggingCategory': 'PyQt6.QtCore',
 'QMainWindow': 'PyQt6.QtWidgets',
 'QMargins': 'PyQt6.QtCore',
 'QMarginsF': 'PyQt6.QtCore',
 'QMatrix2x2': 'PyQt6.QtGui',
 'QMatrix2x3': 'PyQt6.QtGui',
 'QMatrix2x4': 'PyQt6.QtGui',
 'QMatrix3x2': 'PyQt6.QtGui',
 'QMatrix3x3': 'PyQt6.QtGui',
 'QMatrix3x4': 'PyQt6.QtGui',
 'QMatrix4x2': 'PyQt6.QtGui',
 'QMatrix4x3': 'PyQt6.QtGui',
 'QMatrix4x4': 'PyQt6.QtGui',
 'QMdiArea': 'PyQt6.QtWidgets',
 'QMdiSubWindow': 'PyQt6.QtWidgets',
 'QMediaCaptureSession': 'PyQt6.QtMultimedia',
 'QMediaDevices': 'PyQt6.QtMultimedia',
 'QMediaFormat': 'PyQt6.QtMultimedia',
 'QMediaMetaData': 'PyQt6.QtMultimedia',
 'QMediaPlayer': 'PyQt6.QtMultimedia',
 'QMediaRecorder': 'PyQt6.QtMultimedia',
 'QMediaTimeRange': 'PyQt6.QtMultimedia',
 'QMenu': 'PyQt6.QtWidgets',
 'QMenuBar': 'PyQt6.QtWidgets',
 'QMessageAuthenticationCode': 'PyQt6.QtCore',
 'QMessageBox': 'PyQt6.QtWidgets',
 'QMessageLogContext': 'PyQt6.QtCore',
 'QMessageLogger': 'PyQt6.QtCore',
 'QMetaClassInfo': 'PyQt6.QtCore',
 'QMetaEnum': 'PyQt6.QtCore',
 'QMetaMethod': 'PyQt6.QtCore',
 'QMetaObject': 'PyQt6.QtCore',
 'QMetaProperty': 'PyQt6.QtCore',
 'QMetaType': 'PyQt6.QtCore',
 'QMicrophonePermission': 'PyQt6.QtCore',
 'QMimeData': 'PyQt6.QtCore',
 'QMimeDatabase': 'PyQt6.QtCore',
 'QMimeType': 'PyQt6.QtCore',
 'QModelIndex': 'PyQt6.QtCore',
 'QModelRoleData': 'PyQt6.QtCore',
 'QModelRoleDataSpan': 'PyQt6.QtCore',
 'QMouseEvent': 'PyQt6.QtGui',
 'QMoveEvent': 'PyQt6.QtGui',
 'QMovie': 'PyQt6.QtGui',
 'QMutex': 'PyQt6.QtCore',
 'QMutexLocker': 'PyQt6.QtCore',
 'QNativeGestureEvent': 'PyQt6.QtGui',
 'QNativeInterface': 'PyQt6.QtQuick',
 'QNativeIpcKey': 'PyQt6.QtCore',
 'QNetworkAccessManager': 'PyQt6.QtNetwork',
 'QNetworkAddressEntry': 'PyQt6.QtNetwork',
 'QNetworkCacheMetaData': 'PyQt6.QtNetwork',
 'QNetworkCookie': 'PyQt6.QtNetwork',
 'QNetworkCookieJar': 'PyQt6.QtNetwork',
 'QNetworkDatagram': 'PyQt6.QtNetwork',
 'QNetworkDiskCache': 'PyQt6.QtNetwork',
 'QNetworkInformation': 'PyQt6.QtNetwork',
 'QNetworkInterface': 'PyQt6.QtNetwork',
 'QNetworkProxy': 'PyQt6.QtNetwork',
 'QNetworkProxyFactory': 'PyQt6.QtNetwork',
 'QNetworkProxyQuery': 'PyQt6.QtNetwork',
 'QNetworkReply': 'PyQt6.QtNetwork',
 'QNetworkRequest': 'PyQt6.QtNetwork',
 'QObject': 'PyQt6.QtCore',
 'QObjectCleanupHandler': 'PyQt6.QtCore',
 'QOcspCertificateStatus': 'PyQt6.QtNetwork',
 'QOcspResponse': 'PyQt6.QtNetwork',
 'QOcspRevocationReason': 'PyQt6.QtNetwork',
 'QOffscreenSurface': 'PyQt6.QtGui',
 'QOpenGLBuffer': 'PyQt6.QtOpenGL',
 'QOpenGLContext': 'PyQt6.QtGui',
 'QOpenGLContextGroup': 'PyQt6.QtGui',
 'QOpenGLDebugLogger': 'PyQt6.QtOpenGL',
 'QOpenGLDebugMessage': 'PyQt6.QtOpenGL',
 'QOpenGLFramebufferObject': 'PyQt6.QtOpenGL',
 'QOpenGLFramebufferObjectFormat': 'PyQt6.QtOpenGL',
 'QOpenGLFunctions_2_0': 'PyQt6.QtOpenGL',
 'QOpenGLFunctions_2_1': 'PyQt6.QtOpenGL',
 'QOpenGLFunctions_4_1_Core': 'PyQt6.QtOpenGL',
 'QOpenGLPaintDevice': 'PyQt6.QtOpenGL',
 'QOpenGLPixelTransferOptions': 'PyQt6.QtOpenGL',
 'QOpenGLShader': 'PyQt6.QtOpenGL',
 'QOpenGLShaderProgram': 'PyQt6.QtOpenGL',
 'QOpenGLTexture': 'PyQt6.QtOpenGL',
 'QOpenGLTextureBlitter': 'PyQt6.QtOpenGL',
 'QOpenGLTimeMonitor': 'PyQt6.QtOpenGL',
 'QOpenGLTimerQuery': 'PyQt6.QtOpenGL',
 'QOpenGLVersionFunctionsFactory': 'PyQt6.QtOpenGL',
 'QOpenGLVersionProfile': 'PyQt6.QtOpenGL',
 'QOpenGLVertexArrayObject': 'PyQt6.QtOpenGL',
 'QOpenGLWidget': 'PyQt6.QtOpenGLWidgets',
 'QOpenGLWindow': 'PyQt6.QtOpenGL',
 'QOperatingSystemVersion': 'PyQt6.QtCore',
 'QOperatingSystemVersionBase': 'PyQt6.QtCore',
 'QPageLayout': 'PyQt6.QtGui',
 'QPageRanges': 'PyQt6.QtGui',
 'QPageSetupDialog': 'PyQt6.QtPrintSupport',
 'QPageSize': 'PyQt6.QtGui',
 'QPagedPaintDevice': 'PyQt6.QtGui',
 'QPaintDevice': 'PyQt6.QtGui',
 'QPaintDeviceWindow': 'PyQt6.QtGui',
 'QPaintEngine': 'PyQt6.QtGui',
 'QPaintEngineState': 'PyQt6.QtGui',
 'QPaintEvent': 'PyQt6.QtGui',
 'QPainter': 'PyQt6.QtGui',
 'QPainterPath': 'PyQt6.QtGui',
 'QPainterPathStroker': 'PyQt6.QtGui',
 'QPalette': 'PyQt6.QtGui',
 'QPanGesture': 'PyQt6.QtWidgets',
 'QParallelAnimationGroup': 'PyQt6.QtCore',
 'QPasswordDigestor': 'PyQt6.QtNetwork',
 'QPauseAnimation': 'PyQt6.QtCore',
 'QPdfWriter': 'PyQt6.QtGui',
 'QPen': 'PyQt6.QtGui',
 'QPermission': 'PyQt6.QtCore',
 'QPersistentModelIndex': 'PyQt6.QtCore',
 'QPicture': 'PyQt6.QtGui',
 'QPinchGesture': 'PyQt6.QtWidgets',
 'QPixelFormat': 'PyQt6.QtGui',
 'QPixmap': 'PyQt6.QtGui',
 'QPixmapCache': 'PyQt6.QtGui',
 'QPlainTextDocumentLayout': 'PyQt6.QtWidgets',
 'QPlainTextEdit': 'PyQt6.QtWidgets',
 'QPlatformSurfaceEvent': 'PyQt6.QtGui',
 'QPluginLoader': 'PyQt6.QtCore',
 'QPoint': 'PyQt6.QtCore',
 'QPointF': 'PyQt6.QtCore',
 'QPointerEvent': 'PyQt6.QtGui',
 'QPointingDevice': 'PyQt6.QtGui',
 'QPointingDeviceUniqueId': 'PyQt6.QtGui',
 'QPolygon': 'PyQt6.QtGui',
 'QPolygonF': 'PyQt6.QtGui',
 'QPrintDialog': 'PyQt6.QtPrintSupport',
 'QPrintEngine': 'PyQt6.QtPrintSupport',
 'QPrintPreviewDialog': 'PyQt6.QtPrintSupport',
 'QPrintPreviewWidget': 'PyQt6.QtPrintSupport',
 'QPrinter': 'PyQt6.QtPrintSupport',
 'QPrinterInfo': 'PyQt6.QtPrintSupport',
 'QProcess': 'PyQt6.QtCore',
 'QProcessEnvironment': 'PyQt6.QtCore',
 'QProgressBar': 'PyQt6.QtWidgets',
 'QProgressDialog': 'PyQt6.QtWidgets',
 'QPropertyAnimation': 'PyQt6.QtCore',
 'QProxyStyle': 'PyQt6.QtWidgets',
 'QPushButton': 'PyQt6.QtWidgets',
 'QQuaternion': 'PyQt6.QtGui',
 'QQuickAsyncImageProvider': 'PyQt6.QtQuick',
 'QQuickCloseEvent': 'PyQt6.QtQuick',
 'QQuickFramebufferObject': 'PyQt6.QtQuick',
 'QQuickGraphicsConfiguration': 'PyQt6.QtQuick',
 'QQuickGraphicsDevice': 'PyQt6.QtQuick',
 'QQuickImageProvider': 'PyQt6.QtQuick',
 'QQuickImageResponse': 'PyQt6.QtQuick',
 'QQuickItem': 'PyQt6.QtQuick',
 'QQuickItemGrabResult': 'PyQt6.QtQuick',
 'QQuickPaintedItem': 'PyQt6.QtQuick',
 'QQuickRenderControl': 'PyQt6.QtQuick',
 'QQuickRenderTarget': 'PyQt6.QtQuick',
 'QQuickTextDocument': 'PyQt6.QtQuick',
 'QQuickTextureFactory': 'PyQt6.QtQuick',
 'QQuickView': 'PyQt6.QtQuick',
 'QQuickWindow': 'PyQt6.QtQuick',
 'QRadialGradient': 'PyQt6.QtGui',
 'QRadioButton': 'PyQt6.QtWidgets',
 'QRandomGenerator': 'PyQt6.QtCore',
 'QRasterWindow': 'PyQt6.QtGui',
 'QRawFont': 'PyQt6.QtGui',
 'QReadLocker': 'PyQt6.QtCore',
 'QReadWriteLock': 'PyQt6.QtCore',
 'QRect': 'PyQt6.QtCore',
 'QRectF': 'PyQt6.QtCore',
 'QRecursiveMutex': 'PyQt6.QtCore',
 'QRegion': 'PyQt6.QtGui',
 'QRegularExpression': 'PyQt6.QtCore',
 'QRegularExpressionMatch': 'PyQt6.QtCore',
 'QRegularExpressionMatchIterator': 'PyQt6.QtCore',
 'QRegularExpressionValidator': 'PyQt6.QtGui',
 'QResizeEvent': 'PyQt6.QtGui',
 'QResource': 'PyQt6.QtCore',
 'QRgba64': 'PyQt6.QtGui',
 'QRubberBand': 'PyQt6.QtWidgets',
 'QRunnable': 'PyQt6.QtCore',
 'QSGBasicGeometryNode': 'PyQt6.QtQuick',
 'QSGClipNode': 'PyQt6.QtQuick',
 'QSGDynamicTexture': 'PyQt6.QtQuick',
 'QSGFlatColorMaterial': 'PyQt6.QtQuick',
 'QSGGeometry': 'PyQt6.QtQuick',
 'QSGGeometryNode': 'PyQt6.QtQuick',
 'QSGImageNode': 'PyQt6.QtQuick',
 'QSGMaterial': 'PyQt6.QtQuick',
 'QSGMaterialShader': 'PyQt6.QtQuick',
 'QSGMaterialType': 'PyQt6.QtQuick',
 'QSGNode': 'PyQt6.QtQuick',
 'QSGOpacityNode': 'PyQt6.QtQuick',
 'QSGOpaqueTextureMaterial': 'PyQt6.QtQuick',
 'QSGRectangleNode': 'PyQt6.QtQuick',
 'QSGRenderNode': 'PyQt6.QtQuick',
 'QSGRendererInterface': 'PyQt6.QtQuick',
 'QSGSimpleRectNode': 'PyQt6.QtQuick',
 'QSGSimpleTextureNode': 'PyQt6.QtQuick',
 'QSGTextNode': 'PyQt6.QtQuick',
 'QSGTexture': 'PyQt6.QtQuick',
 'QSGTextureMaterial': 'PyQt6.QtQuick',
 'QSGTextureProvider': 'PyQt6.QtQuick',
 'QSGTransformNode': 'PyQt6.QtQuick',
 'QSGVertexColorMaterial': 'PyQt6.QtQuick',
 'QSaveFile': 'PyQt6.QtCore',
 'QScreen': 'PyQt6.QtGui',
 'QScreenCapture': 'PyQt6.QtMultimedia',
 'QScrollArea': 'PyQt6.QtWidgets',
 'QScrollBar': 'PyQt6.QtWidgets',
 'QScrollEvent': 'PyQt6.QtGui',
 'QScrollPrepareEvent': 'PyQt6.QtGui',
 'QScroller': 'PyQt6.QtWidgets',
 'QScrollerProperties': 'PyQt6.QtWidgets',
 'QSemaphore': 'PyQt6.QtCore',
 'QSemaphoreReleaser': 'PyQt6.QtCore',
 'QSequentialAnimationGroup': 'PyQt6.QtCore',
 'QSessionManager': 'PyQt6.QtGui',
 'QSettings': 'PyQt6.QtCore',
 'QSharedMemory': 'PyQt6.QtCore',
 'QShortcut': 'PyQt6.QtGui',
 'QShortcutEvent': 'PyQt6.QtGui',
 'QShowEvent': 'PyQt6.QtGui',
 'QSignalBlocker': 'PyQt6.QtCore',
 'QSignalMapper': 'PyQt6.QtCore',
 'QSinglePointEvent': 'PyQt6.QtGui',
 'QSize': 'PyQt6.QtCore',
 'QSizeF': 'PyQt6.QtCore',
 'QSizeGrip': 'PyQt6.QtWidgets',
 'QSizePolicy': 'PyQt6.QtWidgets',
 'QSlider': 'PyQt6.QtWidgets',
 'QSocketNotifier': 'PyQt6.QtCore',
 'QSortFilterProxyModel': 'PyQt6.QtCore',
 'QSoundEffect': 'PyQt6.QtMultimedia',
 'QSpacerItem': 'PyQt6.QtWidgets',
 'QSpinBox': 'PyQt6.QtWidgets',
 'QSplashScreen': 'PyQt6.QtWidgets',
 'QSplitter': 'PyQt6.QtWidgets',
 'QSplitterHandle': 'PyQt6.QtWidgets',
 'QSsl': 'PyQt6.QtNetwork',
 'QSslCertificate': 'PyQt6.QtNetwork',
 'QSslCertificateExtension': 'PyQt6.QtNetwork',
 'QSslCipher': 'PyQt6.QtNetwork',
 'QSslConfiguration': 'PyQt6.QtNetwork',
 'QSslDiffieHellmanParameters': 'PyQt6.QtNetwork',
 'QSslEllipticCurve': 'PyQt6.QtNetwork',
 'QSslError': 'PyQt6.QtNetwork',
 'QSslKey': 'PyQt6.QtNetwork',
 'QSslPreSharedKeyAuthenticator': 'PyQt6.QtNetwork',
 'QSslServer': 'PyQt6.QtNetwork',
 'QSslSocket': 'PyQt6.QtNetwork',
 'QStackedLayout': 'PyQt6.QtWidgets',
 'QStackedWidget': 'PyQt6.QtWidgets',
 'QStandardItem': 'PyQt6.QtGui',
 'QStandardItemModel': 'PyQt6.QtGui',
 'QStandardPaths': 'PyQt6.QtCore',
 'QStaticText': 'PyQt6.QtGui',
 'QStatusBar': 'PyQt6.QtWidgets',
 'QStatusTipEvent': 'PyQt6.QtGui',
 'QStorageInfo': 'PyQt6.QtCore',
 'QStringConverter': 'PyQt6.QtCore',
 'QStringConverterBase': 'PyQt6.QtCore',
 'QStringDecoder': 'PyQt6.QtCore',
 'QStringEncoder': 'PyQt6.QtCore',
 'QStringListModel': 'PyQt6.QtCore',
 'QStyle': 'PyQt6.QtWidgets',
 'QStyleFactory': 'PyQt6.QtWidgets',
 'QStyleHintReturn': 'PyQt6.QtWidgets',
 'QStyleHintReturnMask': 'PyQt6.QtWidgets',
 'QStyleHintReturnVariant': 'PyQt6.QtWidgets',
 'QStyleHints': 'PyQt6.QtGui',
 'QStyleOption': 'PyQt6.QtWidgets',
 'QStyleOptionButton': 'PyQt6.QtWidgets',
 'QStyleOptionComboBox': 'PyQt6.QtWidgets',
 'QStyleOptionComplex': 'PyQt6.QtWidgets',
 'QStyleOptionDockWidget': 'PyQt6.QtWidgets',
 'QStyleOptionFocusRect': 'PyQt6.QtWidgets',
 'QStyleOptionFrame': 'PyQt6.QtWidgets',
 'QStyleOptionGraphicsItem': 'PyQt6.QtWidgets',
 'QStyleOptionGroupBox': 'PyQt6.QtWidgets',
 'QStyleOptionHeader': 'PyQt6.QtWidgets',
 'QStyleOptionHeaderV2': 'PyQt6.QtWidgets',
 'QStyleOptionMenuItem': 'PyQt6.QtWidgets',
 'QStyleOptionProgressBar': 'PyQt6.QtWidgets',
 'QStyleOptionRubberBand': 'PyQt6.QtWidgets',
 'QStyleOptionSizeGrip': 'PyQt6.QtWidgets',
 'QStyleOptionSlider': 'PyQt6.QtWidgets',
 'QStyleOptionSpinBox': 'PyQt6.QtWidgets',
 'QStyleOptionTab': 'PyQt6.QtWidgets',
 'QStyleOptionTabBarBase': 'PyQt6.QtWidgets',
 'QStyleOptionTabWidgetFrame': 'PyQt6.QtWidgets',
 'QStyleOptionTitleBar': 'PyQt6.QtWidgets',
 'QStyleOptionToolBar': 'PyQt6.QtWidgets',
 'QStyleOptionToolBox': 'PyQt6.QtWidgets',
 'QStyleOptionToolButton': 'PyQt6.QtWidgets',
 'QStyleOptionViewItem': 'PyQt6.QtWidgets',
 'QStylePainter': 'PyQt6.QtWidgets',
 'QStyledItemDelegate': 'PyQt6.QtWidgets',
 'QSurface': 'PyQt6.QtGui',
 'QSurfaceFormat': 'PyQt6.QtGui',
 'QSvgGenerator': 'PyQt6.QtSvg',
 'QSvgRenderer': 'PyQt6.QtSvg',
 'QSwipeGesture': 'PyQt6.QtWidgets',
 'QSyntaxHighlighter': 'PyQt6.QtGui',
 'QSysInfo': 'PyQt6.QtCore',
 'QSystemSemaphore': 'PyQt6.QtCore',
 'QSystemTrayIcon': 'PyQt6.QtWidgets',
 'QT_TRANSLATE_NOOP': 'PyQt6.QtCore',
 'QT_TR_NOOP': 'PyQt6.QtCore',
 'QT_VERSION': 'PyQt6.QtCore',
 'QT_VERSION_STR': 'PyQt6.QtCore',
 'QTabBar': 'PyQt6.QtWidgets',
 'QTabWidget': 'PyQt6.QtWidgets',
 'QTableView': 'PyQt6.QtWidgets',
 'QTableWidget': 'PyQt6.QtWidgets',
 'QTableWidgetItem': 'PyQt6.QtWidgets',
 'QTableWidgetSelectionRange': 'PyQt6.QtWidgets',
 'QTabletEvent': 'PyQt6.QtGui',
 'QTapAndHoldGesture': 'PyQt6.QtWidgets',
 'QTapGesture': 'PyQt6.QtWidgets',
 'QTcpServer': 'PyQt6.QtNetwork',
 'QTcpSocket': 'PyQt6.QtNetwork',
 'QTemporaryDir': 'PyQt6.QtCore',
 'QTemporaryFile': 'PyQt6.QtCore',
 'QTextBlock': 'PyQt6.QtGui',
 'QTextBlockFormat': 'PyQt6.QtGui',
 'QTextBlockGroup': 'PyQt6.QtGui',
 'QTextBlockUserData': 'PyQt6.QtGui',
 'QTextBoundaryFinder': 'PyQt6.QtCore',
 'QTextBrowser': 'PyQt6.QtWidgets',
 'QTextCharFormat': 'PyQt6.QtGui',
 'QTextCursor': 'PyQt6.QtGui',
 'QTextDocument': 'PyQt6.QtGui',
 'QTextDocumentFragment': 'PyQt6.QtGui',
 'QTextDocumentWriter': 'PyQt6.QtGui',
 'QTextEdit': 'PyQt6.QtWidgets',
 'QTextFormat': 'PyQt6.QtGui',
 'QTextFragment': 'PyQt6.QtGui',
 'QTextFrame': 'PyQt6.QtGui',
 'QTextFrameFormat': 'PyQt6.QtGui',
 'QTextImageFormat': 'PyQt6.QtGui',
 'QTextInlineObject': 'PyQt6.QtGui',
 'QTextItem': 'PyQt6.QtGui',
 'QTextLayout': 'PyQt6.QtGui',
 'QTextLength': 'PyQt6.QtGui',
 'QTextLine': 'PyQt6.QtGui',
 'QTextList': 'PyQt6.QtGui',
 'QTextListFormat': 'PyQt6.QtGui',
 'QTextObject': 'PyQt6.QtGui',
 'QTextObjectInterface': 'PyQt6.QtGui',
 'QTextOption': 'PyQt6.QtGui',
 'QTextStream': 'PyQt6.QtCore',
 'QTextStreamManipulator': 'PyQt6.QtCore',
 'QTextTable': 'PyQt6.QtGui',
 'QTextTableCell': 'PyQt6.QtGui',
 'QTextTableCellFormat': 'PyQt6.QtGui',
 'QTextTableFormat': 'PyQt6.QtGui',
 'QTextToSpeech': 'PyQt6.QtTextToSpeech',
 'QThread': 'PyQt6.QtCore',
 'QThreadPool': 'PyQt6.QtCore',
 'QTime': 'PyQt6.QtCore',
 'QTimeEdit': 'PyQt6.QtWidgets',
 'QTimeLine': 'PyQt6.QtCore',
 'QTimeZone': 'PyQt6.QtCore',
 'QTimer': 'PyQt6.QtCore',
 'QTimerEvent': 'PyQt6.QtCore',
 'QToolBar': 'PyQt6.QtWidgets',
 'QToolBox': 'PyQt6.QtWidgets',
 'QToolButton': 'PyQt6.QtWidgets',
 'QToolTip': 'PyQt6.QtWidgets',
 'QTouchEvent': 'PyQt6.QtGui',
 'QTransform': 'PyQt6.QtGui',
 'QTranslator': 'PyQt6.QtCore',
 'QTransposeProxyModel': 'PyQt6.QtCore',
 'QTreeView': 'PyQt6.QtWidgets',
 'QTreeWidget': 'PyQt6.QtWidgets',
 'QTreeWidgetItem': 'PyQt6.QtWidgets',
 'QTreeWidgetItemIterator': 'PyQt6.QtWidgets',
 'QTypeRevision': 'PyQt6.QtCore',
 'QUdpSocket': 'PyQt6.QtNetwork',
 'QUndoCommand': 'PyQt6.QtGui',
 'QUndoGroup': 'PyQt6.QtGui',
 'QUndoStack': 'PyQt6.QtGui',
 'QUndoView': 'PyQt6.QtWidgets',
 'QUrl': 'PyQt6.QtCore',
 'QUrlQuery': 'PyQt6.QtCore',
 'QUuid': 'PyQt6.QtCore',
 'QVBoxLayout': 'PyQt6.QtWidgets',
 'QValidator': 'PyQt6.QtGui',
 'QVariant': 'PyQt6.QtCore',
 'QVariantAnimation': 'PyQt6.QtCore',
 'QVector2D': 'PyQt6.QtGui',
 'QVector3D': 'PyQt6.QtGui',
 'QVector4D': 'PyQt6.QtGui',
 'QVersionNumber': 'PyQt6.QtCore',
 'QVideoFrame': 'PyQt6.QtMultimedia',
 'QVideoFrameFormat': 'PyQt6.QtMultimedia',
 'QVideoSink': 'PyQt6.QtMultimedia',
 'QVideoWidget': 'PyQt6.QtMultimediaWidgets',
 'QVoice': 'PyQt6.QtTextToSpeech',
 'QWIDGETSIZE_MAX': 'PyQt6.QtWidgets',
 'QWaitCondition': 'PyQt6.QtCore',
 'QWhatsThis': 'PyQt6.QtWidgets',
 'QWhatsThisClickedEvent': 'PyQt6.QtGui',
 'QWheelEvent': 'PyQt6.QtGui',
 'QWidget': 'PyQt6.QtWidgets',
 'QWidgetAction': 'PyQt6.QtWidgets',
 'QWidgetItem': 'PyQt6.QtWidgets',
 'QWindow': 'PyQt6.QtGui',
 'QWindowCapture': 'PyQt6.QtMultimedia',
 'QWindowStateChangeEvent': 'PyQt6.QtGui',
 'QWizard': 'PyQt6.QtWidgets',
 'QWizardPage': 'PyQt6.QtWidgets',
 'QWriteLocker': 'PyQt6.QtCore',
 'QXmlStreamAttribute': 'PyQt6.QtCore',
 'QXmlStreamAttributes': 'PyQt6.QtCore',
 'QXmlStreamEntityDeclaration': 'PyQt6.QtCore',
 'QXmlStreamEntityResolver': 'PyQt6.QtCore',
 'QXmlStreamNamespaceDeclaration': 'PyQt6.QtCore',
 'QXmlStreamNotationDeclaration': 'PyQt6.QtCore',
 'QXmlStreamReader': 'PyQt6.QtCore',
 'QXmlStreamWriter': 'PyQt6.QtCore',
 'Q_ARG': 'PyQt6.QtCore',
 'Q_RETURN_ARG': 'PyQt6.QtCore',
 'Qt': 'PyQt6.QtCore',
 'QtCore': 'PyQt6.QtCore',
 'QtGui': 'PyQt6.QtGui',
 'QtMsgType': 'PyQt6.QtCore',
 'QtMultimedia': 'PyQt6.QtMultimedia',
 'QtMultimediaWidgets': 'PyQt6.QtMultimediaWidgets',
 'QtNetwork': 'PyQt6.QtNetwork',
 'QtOpenGL': 'PyQt6.QtOpenGL',
 'QtOpenGLWidgets': 'PyQt6.QtOpenGLWidgets',
 'QtPrintSupport': 'PyQt6.QtPrintSupport',
 'QtQuick': 'PyQt6.QtQuick',
 'QtSvg': 'PyQt6.QtSvg',
 'QtTextToSpeech': 'PyQt6.QtTextToSpeech',
 'QtVideo': 'PyQt6.QtMultimedia',
 'QtWidgets': 'PyQt6.QtWidgets',
 'pyqtBoundSignal': 'PyQt6.QtCore',
 'pyqtClassInfo': 'PyQt6.QtCore',
 'pyqtEnum': 'PyQt6.QtCore',
 'pyqtPickleProtocol': 'PyQt6.QtCore',
 'pyqtProperty': 'PyQt6.QtCore',
 'pyqtRemoveInputHook': 'PyQt6.QtCore',
 'pyqtRestoreInputHook': 'PyQt6.QtCore',
 'pyqtSetPickleProtocol': 'PyQt6.QtCore',
 'pyqtSignal': 'PyQt6.QtCore',
 'pyqtSlot': 'PyQt6.QtCore',
 'qAbs': 'PyQt6.QtCore',
 'qAddPostRoutine': 'PyQt6.QtCore',
 'qAddPreRoutine': 'PyQt6.QtCore',
 'qAlpha': 'PyQt6.QtGui',
 'qBlue': 'PyQt6.QtGui',
 'qChecksum': 'PyQt6.QtCore',
 'qCompress': 'PyQt6.QtCore',
 'qCritical': 'PyQt6.QtCore',
 'qDebug': 'PyQt6.QtCore',
 'qDrawBorderPixmap': 'PyQt6.QtWidgets',
 'qDrawPlainRect': 'PyQt6.QtWidgets',
 'qDrawPlainRoundedRect': 'PyQt6.QtWidgets',
 'qDrawShadeLine': 'PyQt6.QtWidgets',
 'qDrawShadePanel': 'PyQt6.QtWidgets',
 'qDrawShadeRect': 'PyQt6.QtWidgets',
 'qDrawWinButton': 'PyQt6.QtWidgets',
 'qDrawWinPanel': 'PyQt6.QtWidgets',
 'qEnvironmentVariable': 'PyQt6.QtCore',
 'qEnvironmentVariableIntValue': 'PyQt6.QtCore',
 'qEnvironmentVariableIsEmpty': 'PyQt6.QtCore',
 'qEnvironmentVariableIsSet': 'PyQt6.QtCore',
 'qFatal': 'PyQt6.QtCore',
 'qFloatDistance': 'PyQt6.QtCore',
 'qFormatLogMessage': 'PyQt6.QtCore',
 'qFuzzyCompare': 'PyQt6.QtCore',
 'qFuzzyIsNull': 'PyQt6.QtCore',
 'qGray': 'PyQt6.QtGui',
 'qGreen': 'PyQt6.QtGui',
 'qInf': 'PyQt6.QtCore',
 'qInfo': 'PyQt6.QtCore',
 'qInstallMessageHandler': 'PyQt6.QtCore',
 'qIsFinite': 'PyQt6.QtCore',
 'qIsInf': 'PyQt6.QtCore',
 'qIsNaN': 'PyQt6.QtCore',
 'qPixelFormatAlpha': 'PyQt6.QtGui',
 'qPixelFormatCmyk': 'PyQt6.QtGui',
 'qPixelFormatGrayscale': 'PyQt6.QtGui',
 'qPixelFormatHsl': 'PyQt6.QtGui',
 'qPixelFormatHsv': 'PyQt6.QtGui',
 'qPixelFormatRgba': 'PyQt6.QtGui',
 'qPixelFormatYuv': 'PyQt6.QtGui',
 'qPremultiply': 'PyQt6.QtGui',
 'qQNaN': 'PyQt6.QtCore',
 'qRed': 'PyQt6.QtGui',
 'qRegisterResourceData': 'PyQt6.QtCore',
 'qRemovePostRoutine': 'PyQt6.QtCore',
 'qRgb': 'PyQt6.QtGui',
 'qRgba': 'PyQt6.QtGui',
 'qRgba64': 'PyQt6.QtGui',
 'qRound': 'PyQt6.QtCore',
 'qRound64': 'PyQt6.QtCore',
 'qSNaN': 'PyQt6.QtCore',
 'qSetFieldWidth': 'PyQt6.QtCore',
 'qSetMessagePattern': 'PyQt6.QtCore',
 'qSetPadChar': 'PyQt6.QtCore',
 'qSetRealNumberPrecision': 'PyQt6.QtCore',
 'qUncompress': 'PyQt6.QtCore',
 'qUnpremultiply': 'PyQt6.QtGui',
 'qUnregisterResourceData': 'PyQt6.QtCore',
 'qVersion': 'PyQt6.QtCore',
 'qWarning': 'PyQt6.QtCore',
 'qYieldCpu': 'PyQt6.QtCore',
 'qt_set_sequence_auto_mnemonic': 'PyQt6.QtGui',
 'sip': 'PyQt6.sip'}
module_names = frozenset(('QtCore',
 'QtGui',
 'QtWidgets',
 'QtNetwork',
 'QtSvg',
 'QtPrintSupport',
 'QtOpenGL',
 'QtOpenGLWidgets',
 'QtQuick',
 'QtMultimedia',
 'QtMultimediaWidgets',
 'QtTextToSpeech',
 'sip')
)
