# Form implementation generated from reading ui file '/home/kovid/work/calibre/src/calibre/gui2/preferences/look_feel_tabs/tb_partitioning.ui'
#
# Created by: PyQt6 UI code generator 6.8.1
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(Form)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.gridLayout_10 = QtWidgets.QGridLayout()
        self.gridLayout_10.setObjectName("gridLayout_10")
        self.formLayout = QtWidgets.QFormLayout()
        self.formLayout.setFieldGrowthPolicy(QtWidgets.QFormLayout.FieldGrowthPolicy.ExpandingFieldsGrow)
        self.formLayout.setObjectName("formLayout")
        self.label_9 = QtWidgets.QLabel(parent=Form)
        self.label_9.setObjectName("label_9")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_9)
        self.opt_tags_browser_partition_method = QtWidgets.QComboBox(parent=Form)
        self.opt_tags_browser_partition_method.setObjectName("opt_tags_browser_partition_method")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.ItemRole.FieldRole, self.opt_tags_browser_partition_method)
        self.label_10 = QtWidgets.QLabel(parent=Form)
        self.label_10.setObjectName("label_10")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_10)
        self.opt_tags_browser_collapse_fl_at = QtWidgets.QSpinBox(parent=Form)
        self.opt_tags_browser_collapse_fl_at.setMaximum(10000)
        self.opt_tags_browser_collapse_fl_at.setObjectName("opt_tags_browser_collapse_fl_at")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.ItemRole.FieldRole, self.opt_tags_browser_collapse_fl_at)
        self.label_101 = QtWidgets.QLabel(parent=Form)
        self.label_101.setObjectName("label_101")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_101)
        self.opt_tags_browser_collapse_at = QtWidgets.QSpinBox(parent=Form)
        self.opt_tags_browser_collapse_at.setMaximum(10000)
        self.opt_tags_browser_collapse_at.setObjectName("opt_tags_browser_collapse_at")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.ItemRole.FieldRole, self.opt_tags_browser_collapse_at)
        self.gridLayout_10.addLayout(self.formLayout, 0, 0, 1, 1)
        self.label = QtWidgets.QLabel(parent=Form)
        self.label.setObjectName("label")
        self.gridLayout_10.addWidget(self.label, 1, 0, 1, 1)
        self.tb_cats_to_partition = QtWidgets.QListView(parent=Form)
        self.tb_cats_to_partition.setMinimumSize(QtCore.QSize(0, 200))
        self.tb_cats_to_partition.setAlternatingRowColors(True)
        self.tb_cats_to_partition.setObjectName("tb_cats_to_partition")
        self.gridLayout_10.addWidget(self.tb_cats_to_partition, 2, 0, 1, 1)
        self.horizontalLayout_41 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_41.setObjectName("horizontalLayout_41")
        self.tb_partition_reset_button = QtWidgets.QPushButton(parent=Form)
        self.tb_partition_reset_button.setObjectName("tb_partition_reset_button")
        self.horizontalLayout_41.addWidget(self.tb_partition_reset_button)
        self.tb_partition_import_layout_button = QtWidgets.QPushButton(parent=Form)
        self.tb_partition_import_layout_button.setObjectName("tb_partition_import_layout_button")
        self.horizontalLayout_41.addWidget(self.tb_partition_import_layout_button)
        self.tb_partition_export_layout_button = QtWidgets.QPushButton(parent=Form)
        self.tb_partition_export_layout_button.setObjectName("tb_partition_export_layout_button")
        self.horizontalLayout_41.addWidget(self.tb_partition_export_layout_button)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout_41.addItem(spacerItem)
        self.gridLayout_10.addLayout(self.horizontalLayout_41, 3, 0, 1, 1)
        self.verticalLayout_4.addLayout(self.gridLayout_10)
        self.label_9.setBuddy(self.opt_tags_browser_partition_method)
        self.label_10.setBuddy(self.opt_tags_browser_collapse_fl_at)
        self.label_101.setBuddy(self.opt_tags_browser_collapse_at)
        self.label.setBuddy(self.tb_cats_to_partition)

        self.retranslateUi(Form)

    def retranslateUi(self, Form):

        self.label_9.setText(_("&Category partitioning method:"))
        self.opt_tags_browser_partition_method.setToolTip(_("<p>Choose how Tag browser subcategories are displayed when\n"
"there are more items than the limit. Select by first\n"
"letter to see an A, B, C list. Choose partitioned to\n"
"have a list of fixed-sized groups. Set to disabled\n"
"if you never want subcategories</p>"))
        self.label_10.setText(_("Combine letters &when fewer items than:"))
        self.opt_tags_browser_collapse_fl_at.setToolTip(_("<p>If collapsing by first letter, combine adjacent letters together if\n"
"there are fewer items under a letter than specified here. If the partition method is\n"
"not set to first letter, this value is ignored. Set to zero to disable.</p>"))
        self.label_101.setText(_("Co&llapse when more items than:"))
        self.opt_tags_browser_collapse_at.setToolTip(_("<p>If a Tag browser category has more than this number of items, it is divided\n"
"up into subcategories. If the partition method is set to disable, this value is ignored.</p>"))
        self.label.setToolTip(_("<p>Check the box for categories that are to\n"
"be partitioned using the criteria above. Uncheck the box if you don\'t want to\n"
"partition a category even if the number of items is larger than\n"
"the value shown above. This option can be used to\n"
"avoid collapsing hierarchical categories that have only\n"
"a few top-level elements.</p>"))
        self.label.setText(_("Select categories to &partition:"))
        self.tb_partition_reset_button.setToolTip(_("Click this button to reset the list to its default order."))
        self.tb_partition_reset_button.setText(_("Reset list"))
        self.tb_partition_import_layout_button.setToolTip(_("<p>Click this button to set the list to one\n"
"previously exported. This could be useful if you have several libraries with\n"
"similar structure and you want to use the same for each one.</p>"))
        self.tb_partition_import_layout_button.setText(_("Import list"))
        self.tb_partition_export_layout_button.setToolTip(_("<p>Click this button to write the current display\n"
"settings to a file. This could be useful if you have several libraries with similar\n"
"structure and you want to use the same for each one.</p>"))
        self.tb_partition_export_layout_button.setText(_("Export list"))
