# Form implementation generated from reading ui file '/home/kovid/work/calibre/src/calibre/gui2/preferences/look_feel_tabs/tb_icon_rules.ui'
#
# Created by: PyQt6 UI code generator 6.8.1
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        self.vboxlayout = QtWidgets.QVBoxLayout(Form)
        self.vboxlayout.setObjectName("vboxlayout")
        self.hboxlayout = QtWidgets.QHBoxLayout()
        self.hboxlayout.setObjectName("hboxlayout")
        self.opt_tag_browser_show_category_icons = QtWidgets.QCheckBox(parent=Form)
        self.opt_tag_browser_show_category_icons.setObjectName("opt_tag_browser_show_category_icons")
        self.hboxlayout.addWidget(self.opt_tag_browser_show_category_icons)
        self.opt_tag_browser_show_value_icons = QtWidgets.QCheckBox(parent=Form)
        self.opt_tag_browser_show_value_icons.setObjectName("opt_tag_browser_show_value_icons")
        self.hboxlayout.addWidget(self.opt_tag_browser_show_value_icons)
        self.vboxlayout.addLayout(self.hboxlayout)
        self.tb_icon_rules_groupbox = QtWidgets.QGroupBox(parent=Form)
        self.tb_icon_rules_groupbox.setObjectName("tb_icon_rules_groupbox")
        self.tb_icon_rules_gridlayout = QtWidgets.QGridLayout(self.tb_icon_rules_groupbox)
        self.tb_icon_rules_gridlayout.setObjectName("tb_icon_rules_gridlayout")
        self.l12345 = QtWidgets.QLabel(parent=self.tb_icon_rules_groupbox)
        self.l12345.setWordWrap(True)
        self.l12345.setObjectName("l12345")
        self.tb_icon_rules_gridlayout.addWidget(self.l12345, 0, 0, 1, 2)
        self.show_only_current_library = QtWidgets.QCheckBox(parent=self.tb_icon_rules_groupbox)
        self.show_only_current_library.setObjectName("show_only_current_library")
        self.tb_icon_rules_gridlayout.addWidget(self.show_only_current_library, 1, 0, 1, 2)
        self.rules_table = QtWidgets.QTableWidget(parent=self.tb_icon_rules_groupbox)
        self.rules_table.setObjectName("rules_table")
        self.rules_table.setColumnCount(0)
        self.rules_table.setRowCount(0)
        self.tb_icon_rules_gridlayout.addWidget(self.rules_table, 2, 1, 1, 1)
        self.vboxlayout1 = QtWidgets.QVBoxLayout()
        self.vboxlayout1.setObjectName("vboxlayout1")
        self.delete_button = QtWidgets.QToolButton(parent=self.tb_icon_rules_groupbox)
        icon = QtGui.QIcon.ic("trash.png")
        self.delete_button.setIcon(icon)
        self.delete_button.setObjectName("delete_button")
        self.vboxlayout1.addWidget(self.delete_button)
        self.edit_button = QtWidgets.QToolButton(parent=self.tb_icon_rules_groupbox)
        icon1 = QtGui.QIcon.ic("edit_input.png")
        self.edit_button.setIcon(icon1)
        self.edit_button.setObjectName("edit_button")
        self.vboxlayout1.addWidget(self.edit_button)
        self.undo_button = QtWidgets.QToolButton(parent=self.tb_icon_rules_groupbox)
        icon2 = QtGui.QIcon.ic("edit-undo.png")
        self.undo_button.setIcon(icon2)
        self.undo_button.setObjectName("undo_button")
        self.vboxlayout1.addWidget(self.undo_button)
        self.add_button = QtWidgets.QToolButton(parent=self.tb_icon_rules_groupbox)
        icon3 = QtGui.QIcon.ic("plus.png")
        self.add_button.setIcon(icon3)
        self.add_button.setObjectName("add_button")
        self.vboxlayout1.addWidget(self.add_button)
        self.tb_icon_rules_gridlayout.addLayout(self.vboxlayout1, 2, 0, 1, 1)
        self.vboxlayout.addWidget(self.tb_icon_rules_groupbox)

        self.retranslateUi(Form)

    def retranslateUi(self, Form):

        self.opt_tag_browser_show_category_icons.setText(_("Show icons on &categories in the Tag browser"))
        self.opt_tag_browser_show_value_icons.setText(_("Show icons on &values in the Tag browser"))
        self.tb_icon_rules_groupbox.setTitle(_("Value icon rules"))
        self.l12345.setText(_("<p>View all the defined value icon rules, including template rules.\n"
"Rules are defined and edited in the Tag browser context menus. Rules can be deleted in\n"
"this dialog by selecting the Category cell then using the button, the delete key,\n"
"or the context menu. The value icon rules are defined per-user, not per-library.</p>"))
        self.show_only_current_library.setText(_("Show only categories and values available in the current library"))
        self.show_only_current_library.setToolTip(_("This check box cannot be changed after any other changes have been made.\n"
"Otherwise all edits would be lost."))
        self.delete_button.setToolTip(_("Delete the selected rule. The Category cell for the rule must be selected"))
        self.edit_button.setToolTip(_("Edit the selected cell"))
        self.undo_button.setToolTip(_("Undo changes in the selected cell, if any."))
        self.add_button.setToolTip(_("Add a new rule. The rule is added immediately. If you decide you don\'t\n"
"want it then you must delete it."))
