# Form implementation generated from reading ui file '/home/kovid/work/calibre/src/calibre/gui2/convert/kepub_output.ui'
#
# Created by: PyQt6 UI code generator 6.8.1
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(644, 353)
        self.formLayout = QtWidgets.QFormLayout(Form)
        self.formLayout.setObjectName("formLayout")
        self.opt_dont_split_on_page_breaks = QtWidgets.QCheckBox(parent=Form)
        self.opt_dont_split_on_page_breaks.setObjectName("opt_dont_split_on_page_breaks")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.ItemRole.LabelRole, self.opt_dont_split_on_page_breaks)
        self.label = QtWidgets.QLabel(parent=Form)
        self.label.setObjectName("label")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label)
        self.opt_flow_size = QtWidgets.QSpinBox(parent=Form)
        self.opt_flow_size.setMinimum(0)
        self.opt_flow_size.setMaximum(1000000)
        self.opt_flow_size.setSingleStep(20)
        self.opt_flow_size.setObjectName("opt_flow_size")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.ItemRole.FieldRole, self.opt_flow_size)
        self.label_4 = QtWidgets.QLabel(parent=Form)
        self.label_4.setObjectName("label_4")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_4)
        self.opt_kepub_max_image_size = QtWidgets.QLineEdit(parent=Form)
        self.opt_kepub_max_image_size.setObjectName("opt_kepub_max_image_size")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.ItemRole.FieldRole, self.opt_kepub_max_image_size)
        self.opt_kepub_affect_hyphenation = QtWidgets.QCheckBox(parent=Form)
        self.opt_kepub_affect_hyphenation.setObjectName("opt_kepub_affect_hyphenation")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.ItemRole.SpanningRole, self.opt_kepub_affect_hyphenation)
        self.opt_kepub_disable_hyphenation = QtWidgets.QCheckBox(parent=Form)
        self.opt_kepub_disable_hyphenation.setObjectName("opt_kepub_disable_hyphenation")
        self.formLayout.setWidget(4, QtWidgets.QFormLayout.ItemRole.LabelRole, self.opt_kepub_disable_hyphenation)
        self.label_2 = QtWidgets.QLabel(parent=Form)
        self.label_2.setObjectName("label_2")
        self.formLayout.setWidget(5, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_2)
        self.opt_kepub_hyphenation_min_chars = QtWidgets.QSpinBox(parent=Form)
        self.opt_kepub_hyphenation_min_chars.setObjectName("opt_kepub_hyphenation_min_chars")
        self.formLayout.setWidget(5, QtWidgets.QFormLayout.ItemRole.FieldRole, self.opt_kepub_hyphenation_min_chars)
        self.label_3 = QtWidgets.QLabel(parent=Form)
        self.label_3.setObjectName("label_3")
        self.formLayout.setWidget(6, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_3)
        self.opt_kepub_hyphenation_min_chars_before = QtWidgets.QSpinBox(parent=Form)
        self.opt_kepub_hyphenation_min_chars_before.setObjectName("opt_kepub_hyphenation_min_chars_before")
        self.formLayout.setWidget(6, QtWidgets.QFormLayout.ItemRole.FieldRole, self.opt_kepub_hyphenation_min_chars_before)
        self.label_5 = QtWidgets.QLabel(parent=Form)
        self.label_5.setObjectName("label_5")
        self.formLayout.setWidget(7, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_5)
        self.opt_kepub_hyphenation_min_chars_after = QtWidgets.QSpinBox(parent=Form)
        self.opt_kepub_hyphenation_min_chars_after.setObjectName("opt_kepub_hyphenation_min_chars_after")
        self.formLayout.setWidget(7, QtWidgets.QFormLayout.ItemRole.FieldRole, self.opt_kepub_hyphenation_min_chars_after)
        self.label_6 = QtWidgets.QLabel(parent=Form)
        self.label_6.setObjectName("label_6")
        self.formLayout.setWidget(8, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_6)
        self.opt_kepub_hyphenation_limit_lines = QtWidgets.QSpinBox(parent=Form)
        self.opt_kepub_hyphenation_limit_lines.setObjectName("opt_kepub_hyphenation_limit_lines")
        self.formLayout.setWidget(8, QtWidgets.QFormLayout.ItemRole.FieldRole, self.opt_kepub_hyphenation_limit_lines)
        spacerItem = QtWidgets.QSpacerItem(20, 262, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.formLayout.setItem(9, QtWidgets.QFormLayout.ItemRole.LabelRole, spacerItem)
        self.label.setBuddy(self.opt_flow_size)
        self.label_4.setBuddy(self.opt_kepub_max_image_size)
        self.label_2.setBuddy(self.opt_kepub_hyphenation_min_chars)
        self.label_3.setBuddy(self.opt_kepub_hyphenation_min_chars)
        self.label_5.setBuddy(self.opt_kepub_hyphenation_min_chars)
        self.label_6.setBuddy(self.opt_kepub_hyphenation_min_chars)

        self.retranslateUi(Form)
        self.opt_kepub_affect_hyphenation.toggled['bool'].connect(self.opt_kepub_disable_hyphenation.setEnabled) # type: ignore
        self.opt_kepub_affect_hyphenation.toggled['bool'].connect(self.opt_kepub_hyphenation_min_chars.setEnabled) # type: ignore
        self.opt_kepub_affect_hyphenation.toggled['bool'].connect(self.opt_kepub_hyphenation_min_chars_before.setEnabled) # type: ignore
        self.opt_kepub_affect_hyphenation.toggled['bool'].connect(self.opt_kepub_hyphenation_min_chars_after.setEnabled) # type: ignore
        self.opt_kepub_affect_hyphenation.toggled['bool'].connect(self.opt_kepub_hyphenation_limit_lines.setEnabled) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):

        Form.setWindowTitle(_("Form"))
        self.opt_dont_split_on_page_breaks.setText(_("Do not &split on page breaks"))
        self.label.setText(_("Split files &larger than:"))
        self.opt_flow_size.setSpecialValueText(_("Disabled"))
        self.opt_flow_size.setSuffix(_(" KB"))
        self.label_4.setText(_("Shrink &images larger than:"))
        self.opt_kepub_affect_hyphenation.setText(_("Enable/disable &hyphenation for this book"))
        self.opt_kepub_disable_hyphenation.setText(_("&Prevent all hyphenation"))
        self.label_2.setText(_("Hyphenation: &minimum word length:"))
        self.opt_kepub_hyphenation_min_chars.setSpecialValueText(_("Disabled"))
        self.opt_kepub_hyphenation_min_chars.setSuffix(_(" characters"))
        self.label_3.setText(_("Hyphenation: minimum characters &before:"))
        self.opt_kepub_hyphenation_min_chars_before.setSuffix(_(" characters"))
        self.label_5.setText(_("Hyphenation: minimum characters &after:"))
        self.opt_kepub_hyphenation_min_chars_after.setSuffix(_(" characters"))
        self.label_6.setText(_("Hyphenation: &limit lines:"))
        self.opt_kepub_hyphenation_limit_lines.setSuffix(_(" lines"))
