# Form implementation generated from reading ui file '/home/kovid/work/calibre/src/calibre/gui2/preferences/look_feel_tabs/edit_metadata.ui'
#
# Created by: PyQt6 UI code generator 6.8.1
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_Form(object):
    def setupUi(self, edit_metadata_tab):
        edit_metadata_tab.setObjectName("edit_metadata_tab")
        self.vbox_layout_61 = QtWidgets.QVBoxLayout(edit_metadata_tab)
        self.vbox_layout_61.setObjectName("vbox_layout_61")
        self.formlayout = QtWidgets.QFormLayout()
        self.formlayout.setObjectName("formlayout")
        self.edit_metadata_single_label = QtWidgets.QLabel(parent=edit_metadata_tab)
        self.edit_metadata_single_label.setObjectName("edit_metadata_single_label")
        self.formlayout.setWidget(0, QtWidgets.QFormLayout.ItemRole.LabelRole, self.edit_metadata_single_label)
        self.opt_edit_metadata_single_layout = QtWidgets.QComboBox(parent=edit_metadata_tab)
        self.opt_edit_metadata_single_layout.setObjectName("opt_edit_metadata_single_layout")
        self.formlayout.setWidget(0, QtWidgets.QFormLayout.ItemRole.FieldRole, self.opt_edit_metadata_single_layout)
        self.vbox_layout_61.addLayout(self.formlayout)
        self.groupBox = QtWidgets.QGroupBox(parent=edit_metadata_tab)
        self.groupBox.setObjectName("groupBox")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.groupBox)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.em_down_button = QtWidgets.QToolButton(parent=self.groupBox)
        icon = QtGui.QIcon.ic("arrow-down.png")
        self.em_down_button.setIcon(icon)
        self.em_down_button.setObjectName("em_down_button")
        self.gridLayout_3.addWidget(self.em_down_button, 3, 1, 1, 1)
        self.em_up_button = QtWidgets.QToolButton(parent=self.groupBox)
        icon1 = QtGui.QIcon.ic("arrow-up.png")
        self.em_up_button.setIcon(icon1)
        self.em_up_button.setObjectName("em_up_button")
        self.gridLayout_3.addWidget(self.em_up_button, 1, 1, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.gridLayout_3.addItem(spacerItem, 2, 1, 1, 1)
        self.em_display_order = ListViewWithMoveByKeyPress(parent=self.groupBox)
        self.em_display_order.setAlternatingRowColors(True)
        self.em_display_order.setObjectName("em_display_order")
        self.gridLayout_3.addWidget(self.em_display_order, 1, 0, 3, 1)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.em_reset_layout_button = QtWidgets.QPushButton(parent=self.groupBox)
        self.em_reset_layout_button.setObjectName("em_reset_layout_button")
        self.horizontalLayout_4.addWidget(self.em_reset_layout_button)
        self.em_import_layout_button = QtWidgets.QPushButton(parent=self.groupBox)
        self.em_import_layout_button.setObjectName("em_import_layout_button")
        self.horizontalLayout_4.addWidget(self.em_import_layout_button)
        self.em_export_layout_button = QtWidgets.QPushButton(parent=self.groupBox)
        self.em_export_layout_button.setObjectName("em_export_layout_button")
        self.horizontalLayout_4.addWidget(self.em_export_layout_button)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem1)
        self.gridLayout_3.addLayout(self.horizontalLayout_4, 5, 0, 1, 1)
        self.vbox_layout_61.addWidget(self.groupBox)
        self.hboxlayout = QtWidgets.QHBoxLayout()
        self.hboxlayout.setObjectName("hboxlayout")
        self.formlayout1 = QtWidgets.QFormLayout()
        self.formlayout1.setObjectName("formlayout1")
        self.opt_edit_metadata_ignore_display_order = QtWidgets.QCheckBox(parent=edit_metadata_tab)
        self.opt_edit_metadata_ignore_display_order.setObjectName("opt_edit_metadata_ignore_display_order")
        self.formlayout1.setWidget(0, QtWidgets.QFormLayout.ItemRole.SpanningRole, self.opt_edit_metadata_ignore_display_order)
        self.opt_edit_metadata_elide_labels = QtWidgets.QCheckBox(parent=edit_metadata_tab)
        self.opt_edit_metadata_elide_labels.setObjectName("opt_edit_metadata_elide_labels")
        self.formlayout1.setWidget(1, QtWidgets.QFormLayout.ItemRole.SpanningRole, self.opt_edit_metadata_elide_labels)
        self.label_1023 = QtWidgets.QLabel(parent=edit_metadata_tab)
        self.label_1023.setText("")
        self.label_1023.setObjectName("label_1023")
        self.formlayout1.setWidget(2, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_1023)
        self.hboxlayout1 = QtWidgets.QHBoxLayout()
        self.hboxlayout1.setObjectName("hboxlayout1")
        self.label_102 = QtWidgets.QLabel(parent=edit_metadata_tab)
        self.label_102.setObjectName("label_102")
        self.hboxlayout1.addWidget(self.label_102)
        self.opt_edit_metadata_elision_point = QtWidgets.QComboBox(parent=edit_metadata_tab)
        self.opt_edit_metadata_elision_point.setObjectName("opt_edit_metadata_elision_point")
        self.hboxlayout1.addWidget(self.opt_edit_metadata_elision_point)
        self.formlayout1.setLayout(2, QtWidgets.QFormLayout.ItemRole.FieldRole, self.hboxlayout1)
        self.opt_edit_metadata_templates_only_F2_on_booklist = QtWidgets.QCheckBox(parent=edit_metadata_tab)
        self.opt_edit_metadata_templates_only_F2_on_booklist.setObjectName("opt_edit_metadata_templates_only_F2_on_booklist")
        self.formlayout1.setWidget(3, QtWidgets.QFormLayout.ItemRole.SpanningRole, self.opt_edit_metadata_templates_only_F2_on_booklist)
        self.hboxlayout.addLayout(self.formlayout1)
        self.formlayout2 = QtWidgets.QFormLayout()
        self.formlayout2.setObjectName("formlayout2")
        self.opt_edit_metadata_single_use_2_cols_for_custom_fields = QtWidgets.QCheckBox(parent=edit_metadata_tab)
        self.opt_edit_metadata_single_use_2_cols_for_custom_fields.setObjectName("opt_edit_metadata_single_use_2_cols_for_custom_fields")
        self.formlayout2.setWidget(0, QtWidgets.QFormLayout.ItemRole.SpanningRole, self.opt_edit_metadata_single_use_2_cols_for_custom_fields)
        self.label_1021 = QtWidgets.QLabel(parent=edit_metadata_tab)
        self.label_1021.setObjectName("label_1021")
        self.formlayout2.setWidget(1, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_1021)
        self.opt_edit_metadata_bulk_cc_label_length = QtWidgets.QSpinBox(parent=edit_metadata_tab)
        self.opt_edit_metadata_bulk_cc_label_length.setObjectName("opt_edit_metadata_bulk_cc_label_length")
        self.formlayout2.setWidget(1, QtWidgets.QFormLayout.ItemRole.FieldRole, self.opt_edit_metadata_bulk_cc_label_length)
        self.label_1022 = QtWidgets.QLabel(parent=edit_metadata_tab)
        self.label_1022.setObjectName("label_1022")
        self.formlayout2.setWidget(2, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_1022)
        self.opt_edit_metadata_single_cc_label_length = QtWidgets.QSpinBox(parent=edit_metadata_tab)
        self.opt_edit_metadata_single_cc_label_length.setObjectName("opt_edit_metadata_single_cc_label_length")
        self.formlayout2.setWidget(2, QtWidgets.QFormLayout.ItemRole.FieldRole, self.opt_edit_metadata_single_cc_label_length)
        self.hboxlayout.addLayout(self.formlayout2)
        self.vbox_layout_61.addLayout(self.hboxlayout)
        self.edit_metadata_single_label.setBuddy(self.opt_edit_metadata_single_layout)
        self.label_102.setBuddy(self.opt_edit_metadata_elision_point)
        self.label_1021.setBuddy(self.opt_edit_metadata_bulk_cc_label_length)
        self.label_1022.setBuddy(self.opt_edit_metadata_single_cc_label_length)

        self.retranslateUi(edit_metadata_tab)

    def retranslateUi(self, edit_metadata_tab):

        self.edit_metadata_single_label.setText(_("Edit metadata (single) &layout:"))
        self.opt_edit_metadata_single_layout.setToolTip(_("Choose a different layout for the Edit metadata dialog. The compact metadata layout favors editing custom metadata over changing covers and formats."))
        self.groupBox.setTitle(_("Select the custom columns (for this library) to display in the edit metadata dialogs and their order:"))
        self.em_down_button.setToolTip(_("Move down. Keyboard shortcut: Ctrl-Down arrow"))
        self.em_up_button.setToolTip(_("Move up. Keyboard shortcut: Ctrl-Up arrow"))
        self.em_reset_layout_button.setToolTip(_("Click this button to reset the list to its default order."))
        self.em_reset_layout_button.setText(_("Reset list"))
        self.em_import_layout_button.setToolTip(_("<p>Click this button to set the list to one\n"
"previously exported. This could be useful if you have several libraries with\n"
"similar structure and you want to use the same column order for each one. Columns\n"
"in the imported list that aren\'t in the current library are ignored. Columns in\n"
"the library that are not in the imported list are put at the end and marked\n"
"as displayable.</p>"))
        self.em_import_layout_button.setText(_("Import list"))
        self.em_export_layout_button.setToolTip(_("<p>Click this button to write the current display\n"
"settings to a file. This could be useful if you have several libraries with similar\n"
"structure and you want to use the same column order for each one.</p>"))
        self.em_export_layout_button.setText(_("Export list"))
        self.opt_edit_metadata_ignore_display_order.setToolTip(_("<p>Check this box to make the edit metadata dialogs ignore the\n"
"above specifications, showing all the columns in the default order. This is\n"
"useful for temporarily seeing all your columns in the dialogs without losing\n"
"the display and order specifications.</p>"))
        self.opt_edit_metadata_ignore_display_order.setText(_("Show &all columns in default order when editing metadata"))
        self.opt_edit_metadata_elide_labels.setToolTip(_("If checked then labels wider than the label width\n"
"will be elided, otherwise they will be word wrapped."))
        self.opt_edit_metadata_elide_labels.setText(_("&Elide labels when editing custom columns"))
        self.label_102.setText(_("Elision point:"))
        self.opt_edit_metadata_elision_point.setToolTip(_("Choose where in the label to put the..."))
        self.opt_edit_metadata_templates_only_F2_on_booklist.setToolTip(_("<p>Check this box to allow only the F2 (Edit) key to\n"
"open the template editor in the book list for a \"Column built from other\n"
"columns\". Editing with mouse clicks and the Tab key will be disabled.</p>"))
        self.opt_edit_metadata_templates_only_F2_on_booklist.setText(_("Only the &F2 (Edit) key edits column templates in the book list"))
        self.opt_edit_metadata_single_use_2_cols_for_custom_fields.setText(_("Use &two columns for custom columns in the Default layout"))
        self.label_1021.setText(_("&Bulk edit custom column label length:"))
        self.opt_edit_metadata_bulk_cc_label_length.setToolTip(_("The maximum width of a custom column label for the bulk metadata edit dialog in average characters."))
        self.opt_edit_metadata_bulk_cc_label_length.setSuffix(_(" chars"))
        self.label_1022.setText(_("&Single edit custom column label length:"))
        self.opt_edit_metadata_single_cc_label_length.setToolTip(_("The maximum width of a custom column label for the single metadata edit dialog in average characters."))
        self.opt_edit_metadata_single_cc_label_length.setSuffix(_(" chars"))
from calibre.gui2.preferences import ListViewWithMoveByKeyPress
