# Form implementation generated from reading ui file '/home/kovid/work/calibre/src/calibre/gui2/convert/pdf_input.ui'
#
# Created by: PyQt6 UI code generator 6.8.0.dev2410141303
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(416, 300)
        self.gridLayout = QtWidgets.QGridLayout(Form)
        self.gridLayout.setObjectName("gridLayout")
        self.label_t = QtWidgets.QLabel(parent=Form)
        self.label_t.setObjectName("label_t")
        self.gridLayout.addWidget(self.label_t, 3, 0, 1, 1)
        self.opt_pdf_footer_skip = QtWidgets.QSpinBox(parent=Form)
        self.opt_pdf_footer_skip.setMinimum(-1)
        self.opt_pdf_footer_skip.setMaximum(99999)
        self.opt_pdf_footer_skip.setProperty("value", -1)
        self.opt_pdf_footer_skip.setObjectName("opt_pdf_footer_skip")
        self.gridLayout.addWidget(self.opt_pdf_footer_skip, 4, 1, 1, 1)
        self.opt_pdf_engine = QtWidgets.QComboBox(parent=Form)
        self.opt_pdf_engine.setObjectName("opt_pdf_engine")
        self.gridLayout.addWidget(self.opt_pdf_engine, 2, 1, 1, 1)
        self.opt_pdf_header_regex = QtWidgets.QLineEdit(parent=Form)
        self.opt_pdf_header_regex.setClearButtonEnabled(True)
        self.opt_pdf_header_regex.setObjectName("opt_pdf_header_regex")
        self.gridLayout.addWidget(self.opt_pdf_header_regex, 5, 1, 1, 1)
        self.opt_pdf_footer_regex = QtWidgets.QLineEdit(parent=Form)
        self.opt_pdf_footer_regex.setClearButtonEnabled(True)
        self.opt_pdf_footer_regex.setObjectName("opt_pdf_footer_regex")
        self.gridLayout.addWidget(self.opt_pdf_footer_regex, 6, 1, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(20, 20, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.gridLayout.addItem(spacerItem, 7, 0, 1, 1)
        self.label_rt = QtWidgets.QLabel(parent=Form)
        self.label_rt.setObjectName("label_rt")
        self.gridLayout.addWidget(self.label_rt, 5, 0, 1, 1)
        self.opt_no_images = QtWidgets.QCheckBox(parent=Form)
        self.opt_no_images.setObjectName("opt_no_images")
        self.gridLayout.addWidget(self.opt_no_images, 1, 0, 1, 1)
        self.label_rb = QtWidgets.QLabel(parent=Form)
        self.label_rb.setObjectName("label_rb")
        self.gridLayout.addWidget(self.label_rb, 6, 0, 1, 1)
        self.opt_unwrap_factor = QtWidgets.QDoubleSpinBox(parent=Form)
        self.opt_unwrap_factor.setMaximum(1.0)
        self.opt_unwrap_factor.setSingleStep(0.01)
        self.opt_unwrap_factor.setProperty("value", 0.45)
        self.opt_unwrap_factor.setObjectName("opt_unwrap_factor")
        self.gridLayout.addWidget(self.opt_unwrap_factor, 0, 1, 1, 1)
        self.label_b = QtWidgets.QLabel(parent=Form)
        self.label_b.setObjectName("label_b")
        self.gridLayout.addWidget(self.label_b, 4, 0, 1, 1)
        self.label = QtWidgets.QLabel(parent=Form)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 2, 0, 1, 1)
        self.label_2 = QtWidgets.QLabel(parent=Form)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 0, 0, 1, 1)
        self.opt_pdf_header_skip = QtWidgets.QSpinBox(parent=Form)
        self.opt_pdf_header_skip.setMinimum(-1)
        self.opt_pdf_header_skip.setMaximum(99999)
        self.opt_pdf_header_skip.setProperty("value", -1)
        self.opt_pdf_header_skip.setObjectName("opt_pdf_header_skip")
        self.gridLayout.addWidget(self.opt_pdf_header_skip, 3, 1, 1, 1)
        self.label_t.setBuddy(self.opt_pdf_header_skip)
        self.label_rt.setBuddy(self.opt_pdf_header_regex)
        self.label_rb.setBuddy(self.opt_pdf_footer_regex)
        self.label_b.setBuddy(self.opt_pdf_footer_skip)
        self.label.setBuddy(self.opt_pdf_engine)
        self.label_2.setBuddy(self.opt_unwrap_factor)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):

        Form.setWindowTitle(_("Form"))
        self.label_t.setText(_("Remove headers at &top of page by:"))
        self.opt_pdf_footer_skip.setSpecialValueText(_("Automatically"))
        self.opt_pdf_footer_skip.setSuffix(_(" px"))
        self.label_rt.setText(_("Regular expression to remove &header at top of page:"))
        self.opt_no_images.setText(_("No &images"))
        self.label_rb.setText(_("Regular expression to remove &footer at bottom of page:"))
        self.label_b.setText(_("Remove footers at &bottom of page by:"))
        self.label.setText(_("PDF &engine:"))
        self.label_2.setText(_("Line &un-wrapping factor:"))
        self.opt_pdf_header_skip.setSpecialValueText(_("Automatically"))
        self.opt_pdf_header_skip.setSuffix(_(" px"))
