__license__   = 'GPL v3'
__copyright__ = '2008, Kovid Goyal <kovid at kovidgoyal.net>'
'''
This package contains code to convert HTML ebooks to LRF ebooks.
'''

__docformat__ = 'epytext'
__author__    = 'Kovid Goyal <kovid@kovidgoyal.net>'
