<?xml version='1.0' encoding='utf-8'?>
<html lang="_LANGUAGE_" xmlns="http://www.w3.org/1999/xhtml">

    <head>
        <title>_TITLE_</title>
    </head>

    <body>
        <h1>_TITLE_</h1>
    </body>
</html>

