char_set = '''
<ms_standard>
NON-BREAKING HYPEHN:_:8290:&#x2011;
LEFT DOUBLE QUOTATION MARK:ldblquote:8220:&#x201C;
RIGHT DOUBLE QUOTATION MARK:rdblquote:8221:&#x201D;
LEFT SINGLE QUOTATION MARK:lquote:8216:&#x2018;
RIGHT SINGLE QUOTATION MARK:rquote:8217:&#x2019;
EN DASH:endash:8211:&#x2013;
EM DASH:emdash:8212:&#x2014;
MIDDLE DOT:bullet:183:&#x00B7;
<control>:tab:9:&#x009;
NO-BREAK SPACE:~:160:&#x00A0;
SOFT-HYPHEN:-:173:&#x00AD;
</ms_standard>
<ms_symbol>
REGISTERED SIGN:ldblquote:174:&#x00AE;
COPYRIGHT SIGN:rdblquote:169:&#x00A9;
N-ARY PRODUCT:rquote:8719:&#x220F;
TRADE MARK SIGN:lquote:8482:&#x2122;
ANGLE:emdash:8736:&#x2220;
WHITE DOWN-POINTING TRIANGLE:endash:9661:&#x25BD;
INFINITY:bullet:8734:&#x221E;
<control>:tab:9:&#x009;
NO-BREAK SPACE:~:160:&#x00A0;
NON-BREAKING HYPEHN:_:8209:&#x2011;
</ms_symbol>
<ms_dingbats>
DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT NINE:10130:ldblquote:&#x2792;
DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT TEN:rdblquote:10131:&#x2793;
HEAVY WIDE-HEADED RIGHTWARDS ARROW:lquote:10132:&#x2794;
RIGHTWARDS ARROW:rquote:8594:&#x2192;
DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT SEVEN:endash:10128:&#x2790;
DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT EIGHT:emdash:10129:&#x2791;
ROTATED HEAVY BLACK HEART BULLET:bullet:10085:&#x2765;
DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT ONE:~:10122:&#x278A;
WRITING HAND:_:9997:&#x270D;
</ms_dingbats>
<ms_wingdings>
<control>:tab:9:&#x009;
PROPOSE "HEDERA UPPER RIGHT":ldblquote:none:<udef_symbol num="?" description="hedera_upper_right"/>
PROPOSE "HEDERA LOWER LEFT":rdblquote:none:<udef_symbol num="?" description="hedera_lower_left"/>
PROPOSE "HEDERA LOWER RIGHT":lquote:none:<udef_symbol num="?" description="hedera_lower_right"/>
ERASE TO THE LEFT:rquote:9003:&#x232B;
AMPERSAND:endash:38:&amp;
PROPOSE "HEDERA UPPER LEFT":emdash:none:<udef_symbol num="?" description="hedera_upper_left"/>
SUN:bullet:9737:&#x2609;
PROPOSE "NOTCHED RIGHTWARDS DOUBLE ARROW WITH TIP DOWNWARDS":~:none:<udef_symbol description="notched_rightwards_double_arrow_with_tip_downwards"/>
PROPOSE "MAIL FLAG UP":_:none:<udef_symbol num="?" description="mail_flag_up"/>
</ms_wingdings>
<not_unicode>
NULL (NUL):'00:0:&#x0000;
START OF HEADING (SOH):'01:1:&#x0001;
START OF TEXT (STX):'02:2:&#x0002;
END OF TEXT (ETX):'03:3:&#x0003;
END OF TRANSMISSION (EOT):'04:4:&#x0004;
ENQUIRY (ENQ):'05:5:&#x0005;
ACKNOWLEDGE (ACK):'06:6:&#x0006;
BELL (BEL):'07:7:&#x0007;
BACKSPACE (BS):'08:8:&#x0008;
LINE TABULATION (VT):'0B:11:&#x000B;
FORM FEED (FF):'0C:12:&#x000C;
SHIFT OUT (SO):'0E:14:&#x000E;
SHIFT IN (SI):'0F:15:&#x000F;
DATALINK ESCAPE (DLE):'10:16:&#x0010;
DEVICE CONTROL ONE (DC1):'11:17:&#x0011;
DEVICE CONTROL TWO (DC2):'12:18:&#x0012;
DEVICE CONTROL THREE (DC3):'13:19:&#x0013;
DEVICE CONTROL FOUR (DC4):'14:20:&#x0014;
NEGATIVE ACKNOWLEDGE (NAK):'15:21:&#x0015;
SYNCHRONOUS IDLE (SYN):'16:22:&#x0016;
END OF TRANSMISSION BLOCK (ETB):'17:23:&#x0017;
CANCEL (CAN):'18:24:&#x0018;
END OF MEDIUM (EM):'19:25:&#x0019;
SUBSTITUTE (SUB):'1A:26:&#x001A;
ESCAPE (ESC):'1B:27:&#x001B;
FILE SEPARATOR (IS4):'1C:28:&#x001C;
GROUP SEPARATOR (IS3):'1D:29:&#x001D;
RECORD SEPARATOR (IS2):'1E:30:&#x001E;
UNIT SEPARATOR (IS1):'1F:31:&#x001F;
</not_unicode>
<bottom_128>
CHARACTER TABULATION (HT):'09:9:&#x0009;
LINE FEED (LF):'0A:10:&#x000A;
CARRIAGE RETURN (CR):'0D:13:&#x000D;
SPACE:'20:32:
EXCLAMATION MARK:'21:33:!
QUOTATION MARK:'22:34:"
NUMBER SIGN:'23:35:#
DOLLAR SIGN:'24:36:$
PERCENT SIGN:'25:37:%
AMPERSAND:'26:38:&amp;
APOSTROPHE:'27:39:'
LEFT PARENTHESIS:'28:40:(
RIGHT PARENTHESIS:'29:41:)
ASTERISK:'2A:42:*
PLUS SIGN:'2B:43:+
COMMA:'2C:44:,
HYPHEN-MINUS:'2D:45:-
FULL STOP:'2E:46:.
SOLIDUS:'2F:47:/
DIGIT ZERO:'30:48:0
DIGIT ONE:'31:49:1
DIGIT TWO:'32:50:2
DIGIT THREE:'33:51:3
DIGIT FOUR:'34:52:4
DIGIT FIVE:'35:53:5
DIGIT SIX:'36:54:6
DIGIT SEVEN:'37:55:7
DIGIT EIGHT:'38:56:8
DIGIT NINE:'39:57:9
COLON:'3A:58:\\colon
SEMICOLON:'3B:59:;
LESS-THAN SIGN:'3C:60:&lt;
EQUALS SIGN:'3D:61:=
GREATER-THAN SIGN:'3E:62:&gt;
QUESTION MARK:'3F:63:?
COMMERCIAL AT:'40:64:&#x0040;
LATIN CAPITAL LETTER A:'41:65:A
LATIN CAPITAL LETTER B:'42:66:B
LATIN CAPITAL LETTER C:'43:67:C
LATIN CAPITAL LETTER D:'44:68:D
LATIN CAPITAL LETTER E:'45:69:E
LATIN CAPITAL LETTER F:'46:70:F
LATIN CAPITAL LETTER G:'47:71:G
LATIN CAPITAL LETTER H:'48:72:H
LATIN CAPITAL LETTER I:'49:73:I
LATIN CAPITAL LETTER J:'4A:74:J
LATIN CAPITAL LETTER K:'4B:75:K
LATIN CAPITAL LETTER L:'4C:76:L
LATIN CAPITAL LETTER M:'4D:77:M
LATIN CAPITAL LETTER N:'4E:78:N
LATIN CAPITAL LETTER O:'4F:79:O
LATIN CAPITAL LETTER P:'50:80:P
LATIN CAPITAL LETTER Q:'51:81:Q
LATIN CAPITAL LETTER R:'52:82:R
LATIN CAPITAL LETTER S:'53:83:S
LATIN CAPITAL LETTER T:'54:84:T
LATIN CAPITAL LETTER U:'55:85:U
LATIN CAPITAL LETTER V:'56:86:V
LATIN CAPITAL LETTER W:'57:87:W
LATIN CAPITAL LETTER X:'58:88:X
LATIN CAPITAL LETTER Y:'59:89:Y
LATIN CAPITAL LETTER Z:'5A:90:Z
LEFT SQUARE BRACKET:'5B:91:[
REVERSE SOLIDUS:'5C:92:\
RIGHT SQUARE BRACKET:'5D:93:]
CIRCUMFLEX ACCENT:'5E:94:&#x005E;
LOW LINE:'5F:95:&#x005F;
GRAVE ACCENT:'60:96:&#x0060;
LATIN SMALL LETTER A:'61:97:a
LATIN SMALL LETTER B:'62:98:b
LATIN SMALL LETTER C:'63:99:c
LATIN SMALL LETTER D:'64:100:d
LATIN SMALL LETTER E:'65:101:e
LATIN SMALL LETTER F:'66:102:f
LATIN SMALL LETTER G:'67:103:g
LATIN SMALL LETTER H:'68:104:h
LATIN SMALL LETTER I:'69:105:i
LATIN SMALL LETTER J:'6A:106:j
LATIN SMALL LETTER K:'6B:107:k
LATIN SMALL LETTER L:'6C:108:l
LATIN SMALL LETTER M:'6D:109:m
LATIN SMALL LETTER N:'6E:110:n
LATIN SMALL LETTER O:'6F:111:o
LATIN SMALL LETTER P:'70:112:p
LATIN SMALL LETTER Q:'71:113:q
LATIN SMALL LETTER R:'72:114:r
LATIN SMALL LETTER S:'73:115:s
LATIN SMALL LETTER T:'74:116:t
LATIN SMALL LETTER U:'75:117:u
LATIN SMALL LETTER V:'76:118:v
LATIN SMALL LETTER W:'77:119:w
LATIN SMALL LETTER X:'78:120:x
LATIN SMALL LETTER Y:'79:121:y
LATIN SMALL LETTER Z:'7A:122:z
LEFT CURLY BRACKET:'7B:123:{
VERTICAL LINE:'7C:124:|
RIGHT CURLY BRACKET:'7D:125:}
TILDE:'7E:126:~
DELETE (DEL):'7F:127:&#x007F;
</bottom_128>
<bottom_128_old>
NULL (NUL):'00:0:&#x0000;
START OF HEADING (SOH):'01:1:&#x0001;
START OF TEXT (STX):'02:2:&#x0002;
END OF TEXT (ETX):'03:3:&#x0003;
END OF TRANSMISSION (EOT):'04:4:&#x0004;
ENQUIRY (ENQ):'05:5:&#x0005;
ACKNOWLEDGE (ACK):'06:6:&#x0006;
BELL (BEL):'07:7:&#x0007;
BACKSPACE (BS):'08:8:&#x0008;
CHARACTER TABULATION (HT):'09:9:&#x0009;
LINE FEED (LF):'0A:10:&#x000A;
LINE TABULATION (VT):'0B:11:&#x000B;
FORM FEED (FF):'0C:12:&#x000C;
CARRIAGE RETURN (CR):'0D:13:&#x000D;
SHIFT OUT (SO):'0E:14:&#x000E;
SHIFT IN (SI):'0F:15:&#x000F;
DATALINK ESCAPE (DLE):'10:16:&#x0010;
DEVICE CONTROL ONE (DC1):'11:17:&#x0011;
DEVICE CONTROL TWO (DC2):'12:18:&#x0012;
DEVICE CONTROL THREE (DC3):'13:19:&#x0013;
DEVICE CONTROL FOUR (DC4):'14:20:&#x0014;
NEGATIVE ACKNOWLEDGE (NAK):'15:21:&#x0015;
SYNCHRONOUS IDLE (SYN):'16:22:&#x0016;
END OF TRANSMISSION BLOCK (ETB):'17:23:&#x0017;
CANCEL (CAN):'18:24:&#x0018;
END OF MEDIUM (EM):'19:25:&#x0019;
SUBSTITUTE (SUB):'1A:26:&#x001A;
ESCAPE (ESC):'1B:27:&#x001B;
FILE SEPARATOR (IS4):'1C:28:&#x001C;
GROUP SEPARATOR (IS3):'1D:29:&#x001D;
RECORD SEPARATOR (IS2):'1E:30:&#x001E;
UNIT SEPARATOR (IS1):'1F:31:&#x001F;
SPACE:'20:32:&#x0020;
EXCLAMATION MARK:'21:33:&#x0021;
QUOTATION MARK:'22:34:&#x0022;
NUMBER SIGN:'23:35:&#x0023;
DOLLAR SIGN:'24:36:&#x0024;
PERCENT SIGN:'25:37:&#x0025;
AMPERSAND:'26:38:&#x0026;
APOSTROPHE:'27:39:&#x0027;
LEFT PARENTHESIS:'28:40:&#x0028;
RIGHT PARENTHESIS:'29:41:&#x0029;
ASTERISK:'2A:42:&#x002A;
PLUS SIGN:'2B:43:&#x002B;
COMMA:'2C:44:&#x002C;
HYPHEN-MINUS:'2D:45:&#x002D;
FULL STOP:'2E:46:&#x002E;
SOLIDUS:'2F:47:&#x002F;
DIGIT ZERO:'30:48:&#x0030;
DIGIT ONE:'31:49:&#x0031;
DIGIT TWO:'32:50:&#x0032;
DIGIT THREE:'33:51:&#x0033;
DIGIT FOUR:'34:52:&#x0034;
DIGIT FIVE:'35:53:&#x0035;
DIGIT SIX:'36:54:&#x0036;
DIGIT SEVEN:'37:55:&#x0037;
DIGIT EIGHT:'38:56:&#x0038;
DIGIT NINE:'39:57:&#x0039;
COLON:'3A:58:&#x003A;
SEMICOLON:'3B:59:&#x003B;
LESS-THAN SIGN:'3C:60:&#x003C;
EQUALS SIGN:'3D:61:&#x003D;
GREATER-THAN SIGN:'3E:62:&#x003E;
QUESTION MARK:'3F:63:&#x003F;
COMMERCIAL AT:'40:64:&#x0040;
LATIN CAPITAL LETTER A:'41:65:&#x0041;
LATIN CAPITAL LETTER B:'42:66:&#x0042;
LATIN CAPITAL LETTER C:'43:67:&#x0043;
LATIN CAPITAL LETTER D:'44:68:&#x0044;
LATIN CAPITAL LETTER E:'45:69:&#x0045;
LATIN CAPITAL LETTER F:'46:70:&#x0046;
LATIN CAPITAL LETTER G:'47:71:&#x0047;
LATIN CAPITAL LETTER H:'48:72:&#x0048;
LATIN CAPITAL LETTER I:'49:73:&#x0049;
LATIN CAPITAL LETTER J:'4A:74:&#x004A;
LATIN CAPITAL LETTER K:'4B:75:&#x004B;
LATIN CAPITAL LETTER L:'4C:76:&#x004C;
LATIN CAPITAL LETTER M:'4D:77:&#x004D;
LATIN CAPITAL LETTER N:'4E:78:&#x004E;
LATIN CAPITAL LETTER O:'4F:79:&#x004F;
LATIN CAPITAL LETTER P:'50:80:&#x0050;
LATIN CAPITAL LETTER Q:'51:81:&#x0051;
LATIN CAPITAL LETTER R:'52:82:&#x0052;
LATIN CAPITAL LETTER S:'53:83:&#x0053;
LATIN CAPITAL LETTER T:'54:84:&#x0054;
LATIN CAPITAL LETTER U:'55:85:&#x0055;
LATIN CAPITAL LETTER V:'56:86:&#x0056;
LATIN CAPITAL LETTER W:'57:87:&#x0057;
LATIN CAPITAL LETTER X:'58:88:&#x0058;
LATIN CAPITAL LETTER Y:'59:89:&#x0059;
LATIN CAPITAL LETTER Z:'5A:90:&#x005A;
LEFT SQUARE BRACKET:'5B:91:&#x005B;
REVERSE SOLIDUS:'5C:92:&#x005C;
RIGHT SQUARE BRACKET:'5D:93:&#x005D;
CIRCUMFLEX ACCENT:'5E:94:&#x005E;
LOW LINE:'5F:95:&#x005F;
GRAVE ACCENT:'60:96:&#x0060;
LATIN SMALL LETTER A:'61:97:&#x0061;
LATIN SMALL LETTER B:'62:98:&#x0062;
LATIN SMALL LETTER C:'63:99:&#x0063;
LATIN SMALL LETTER D:'64:100:&#x0064;
LATIN SMALL LETTER E:'65:101:&#x0065;
LATIN SMALL LETTER F:'66:102:&#x0066;
LATIN SMALL LETTER G:'67:103:&#x0067;
LATIN SMALL LETTER H:'68:104:&#x0068;
LATIN SMALL LETTER I:'69:105:&#x0069;
LATIN SMALL LETTER J:'6A:106:&#x006A;
LATIN SMALL LETTER K:'6B:107:&#x006B;
LATIN SMALL LETTER L:'6C:108:&#x006C;
LATIN SMALL LETTER M:'6D:109:&#x006D;
LATIN SMALL LETTER N:'6E:110:&#x006E;
LATIN SMALL LETTER O:'6F:111:&#x006F;
LATIN SMALL LETTER P:'70:112:&#x0070;
LATIN SMALL LETTER Q:'71:113:&#x0071;
LATIN SMALL LETTER R:'72:114:&#x0072;
LATIN SMALL LETTER S:'73:115:&#x0073;
LATIN SMALL LETTER T:'74:116:&#x0074;
LATIN SMALL LETTER U:'75:117:&#x0075;
LATIN SMALL LETTER V:'76:118:&#x0076;
LATIN SMALL LETTER W:'77:119:&#x0077;
LATIN SMALL LETTER X:'78:120:&#x0078;
LATIN SMALL LETTER Y:'79:121:&#x0079;
LATIN SMALL LETTER Z:'7A:122:&#x007A;
</bottom_128_old>
<ansicpg950>
DBCS LEAD BYTE:'81:129:<udef_symbol num="129"/>
DBCS LEAD BYTE:'82:130:<udef_symbol num="130"/>
DBCS LEAD BYTE:'83:131:<udef_symbol num="131"/>
DBCS LEAD BYTE:'84:132:<udef_symbol num="132"/>
DBCS LEAD BYTE:'85:133:<udef_symbol num="133"/>
DBCS LEAD BYTE:'86:134:<udef_symbol num="134"/>
DBCS LEAD BYTE:'87:135:<udef_symbol num="135"/>
DBCS LEAD BYTE:'88:136:<udef_symbol num="136"/>
DBCS LEAD BYTE:'89:137:<udef_symbol num="137"/>
DBCS LEAD BYTE:'8A:138:<udef_symbol num="138"/>
DBCS LEAD BYTE:'8B:139:<udef_symbol num="139"/>
DBCS LEAD BYTE:'8C:140:<udef_symbol num="140"/>
DBCS LEAD BYTE:'8D:141:<udef_symbol num="141"/>
DBCS LEAD BYTE:'8E:142:<udef_symbol num="142"/>
DBCS LEAD BYTE:'8F:143:<udef_symbol num="143"/>
DBCS LEAD BYTE:'90:144:<udef_symbol num="144"/>
DBCS LEAD BYTE:'91:145:<udef_symbol num="145"/>
DBCS LEAD BYTE:'92:146:<udef_symbol num="146"/>
DBCS LEAD BYTE:'93:147:<udef_symbol num="147"/>
DBCS LEAD BYTE:'94:148:<udef_symbol num="148"/>
DBCS LEAD BYTE:'95:149:<udef_symbol num="149"/>
DBCS LEAD BYTE:'96:150:<udef_symbol num="150"/>
DBCS LEAD BYTE:'97:151:<udef_symbol num="151"/>
DBCS LEAD BYTE:'98:152:<udef_symbol num="152"/>
DBCS LEAD BYTE:'99:153:<udef_symbol num="153"/>
DBCS LEAD BYTE:'9A:154:<udef_symbol num="154"/>
DBCS LEAD BYTE:'9B:155:<udef_symbol num="155"/>
DBCS LEAD BYTE:'9C:156:<udef_symbol num="156"/>
DBCS LEAD BYTE:'9D:157:<udef_symbol num="157"/>
DBCS LEAD BYTE:'9E:158:<udef_symbol num="158"/>
DBCS LEAD BYTE:'9F:159:<udef_symbol num="159"/>
DBCS LEAD BYTE:'A0:160:<udef_symbol num="160"/>
DBCS LEAD BYTE:'A1:161:<udef_symbol num="161"/>
DBCS LEAD BYTE:'A2:162:<udef_symbol num="162"/>
DBCS LEAD BYTE:'A3:163:<udef_symbol num="163"/>
DBCS LEAD BYTE:'A4:164:<udef_symbol num="164"/>
DBCS LEAD BYTE:'A5:165:<udef_symbol num="165"/>
DBCS LEAD BYTE:'A6:166:<udef_symbol num="166"/>
DBCS LEAD BYTE:'A7:167:<udef_symbol num="167"/>
DBCS LEAD BYTE:'A8:168:<udef_symbol num="168"/>
DBCS LEAD BYTE:'A9:169:<udef_symbol num="169"/>
DBCS LEAD BYTE:'AA:170:<udef_symbol num="170"/>
DBCS LEAD BYTE:'AB:171:<udef_symbol num="171"/>
DBCS LEAD BYTE:'AC:172:<udef_symbol num="172"/>
DBCS LEAD BYTE:'AD:173:<udef_symbol num="173"/>
DBCS LEAD BYTE:'AE:174:<udef_symbol num="174"/>
DBCS LEAD BYTE:'AF:175:<udef_symbol num="175"/>
DBCS LEAD BYTE:'B0:176:<udef_symbol num="176"/>
DBCS LEAD BYTE:'B1:177:<udef_symbol num="177"/>
DBCS LEAD BYTE:'B2:178:<udef_symbol num="178"/>
DBCS LEAD BYTE:'B3:179:<udef_symbol num="179"/>
DBCS LEAD BYTE:'B4:180:<udef_symbol num="180"/>
DBCS LEAD BYTE:'B5:181:<udef_symbol num="181"/>
DBCS LEAD BYTE:'B6:182:<udef_symbol num="182"/>
DBCS LEAD BYTE:'B7:183:<udef_symbol num="183"/>
DBCS LEAD BYTE:'B8:184:<udef_symbol num="184"/>
DBCS LEAD BYTE:'B9:185:<udef_symbol num="185"/>
DBCS LEAD BYTE:'BA:186:<udef_symbol num="186"/>
DBCS LEAD BYTE:'BB:187:<udef_symbol num="187"/>
DBCS LEAD BYTE:'BC:188:<udef_symbol num="188"/>
DBCS LEAD BYTE:'BD:189:<udef_symbol num="189"/>
DBCS LEAD BYTE:'BE:190:<udef_symbol num="190"/>
DBCS LEAD BYTE:'BF:191:<udef_symbol num="191"/>
DBCS LEAD BYTE:'C0:192:<udef_symbol num="192"/>
DBCS LEAD BYTE:'C1:193:<udef_symbol num="193"/>
DBCS LEAD BYTE:'C2:194:<udef_symbol num="194"/>
DBCS LEAD BYTE:'C3:195:<udef_symbol num="195"/>
DBCS LEAD BYTE:'C4:196:<udef_symbol num="196"/>
DBCS LEAD BYTE:'C5:197:<udef_symbol num="197"/>
DBCS LEAD BYTE:'C6:198:<udef_symbol num="198"/>
DBCS LEAD BYTE:'C7:199:<udef_symbol num="199"/>
DBCS LEAD BYTE:'C8:200:<udef_symbol num="200"/>
DBCS LEAD BYTE:'C9:201:<udef_symbol num="201"/>
DBCS LEAD BYTE:'CA:202:<udef_symbol num="202"/>
DBCS LEAD BYTE:'CB:203:<udef_symbol num="203"/>
DBCS LEAD BYTE:'CC:204:<udef_symbol num="204"/>
DBCS LEAD BYTE:'CD:205:<udef_symbol num="205"/>
DBCS LEAD BYTE:'CE:206:<udef_symbol num="206"/>
DBCS LEAD BYTE:'CF:207:<udef_symbol num="207"/>
DBCS LEAD BYTE:'D0:208:<udef_symbol num="208"/>
DBCS LEAD BYTE:'D1:209:<udef_symbol num="209"/>
DBCS LEAD BYTE:'D2:210:<udef_symbol num="210"/>
DBCS LEAD BYTE:'D3:211:<udef_symbol num="211"/>
DBCS LEAD BYTE:'D4:212:<udef_symbol num="212"/>
DBCS LEAD BYTE:'D5:213:<udef_symbol num="213"/>
DBCS LEAD BYTE:'D6:214:<udef_symbol num="214"/>
DBCS LEAD BYTE:'D7:215:<udef_symbol num="215"/>
DBCS LEAD BYTE:'D8:216:<udef_symbol num="216"/>
DBCS LEAD BYTE:'D9:217:<udef_symbol num="217"/>
DBCS LEAD BYTE:'DA:218:<udef_symbol num="218"/>
DBCS LEAD BYTE:'DB:219:<udef_symbol num="219"/>
DBCS LEAD BYTE:'DC:220:<udef_symbol num="220"/>
DBCS LEAD BYTE:'DD:221:<udef_symbol num="221"/>
DBCS LEAD BYTE:'DE:222:<udef_symbol num="222"/>
DBCS LEAD BYTE:'DF:223:<udef_symbol num="223"/>
DBCS LEAD BYTE:'E0:224:<udef_symbol num="224"/>
DBCS LEAD BYTE:'E1:225:<udef_symbol num="225"/>
DBCS LEAD BYTE:'E2:226:<udef_symbol num="226"/>
DBCS LEAD BYTE:'E3:227:<udef_symbol num="227"/>
DBCS LEAD BYTE:'E4:228:<udef_symbol num="228"/>
DBCS LEAD BYTE:'E5:229:<udef_symbol num="229"/>
DBCS LEAD BYTE:'E6:230:<udef_symbol num="230"/>
DBCS LEAD BYTE:'E7:231:<udef_symbol num="231"/>
DBCS LEAD BYTE:'E8:232:<udef_symbol num="232"/>
DBCS LEAD BYTE:'E9:233:<udef_symbol num="233"/>
DBCS LEAD BYTE:'EA:234:<udef_symbol num="234"/>
DBCS LEAD BYTE:'EB:235:<udef_symbol num="235"/>
DBCS LEAD BYTE:'EC:236:<udef_symbol num="236"/>
DBCS LEAD BYTE:'ED:237:<udef_symbol num="237"/>
DBCS LEAD BYTE:'EE:238:<udef_symbol num="238"/>
DBCS LEAD BYTE:'EF:239:<udef_symbol num="239"/>
DBCS LEAD BYTE:'F0:240:<udef_symbol num="240"/>
DBCS LEAD BYTE:'F1:241:<udef_symbol num="241"/>
DBCS LEAD BYTE:'F2:242:<udef_symbol num="242"/>
DBCS LEAD BYTE:'F3:243:<udef_symbol num="243"/>
DBCS LEAD BYTE:'F4:244:<udef_symbol num="244"/>
DBCS LEAD BYTE:'F5:245:<udef_symbol num="245"/>
DBCS LEAD BYTE:'F6:246:<udef_symbol num="246"/>
DBCS LEAD BYTE:'F7:247:<udef_symbol num="247"/>
DBCS LEAD BYTE:'F8:248:<udef_symbol num="248"/>
DBCS LEAD BYTE:'F9:249:<udef_symbol num="249"/>
DBCS LEAD BYTE:'FA:250:<udef_symbol num="250"/>
DBCS LEAD BYTE:'FB:251:<udef_symbol num="251"/>
DBCS LEAD BYTE:'FC:252:<udef_symbol num="252"/>
DBCS LEAD BYTE:'FD:253:<udef_symbol num="253"/>
DBCS LEAD BYTE:'FE:254:<udef_symbol num="254"/>
IDEOGRAPHIC SPACE:'A140:41280:&#x3000
FULLWIDTH COMMA:'A141:41281:&#xFF0C
IDEOGRAPHIC COMMA:'A142:41282:&#x3001
IDEOGRAPHIC FULL STOP:'A143:41283:&#x3002
FULLWIDTH FULL STOP:'A144:41284:&#xFF0E
HYPHENATION POINT:'A145:41285:&#x2027
FULLWIDTH SEMICOLON:'A146:41286:&#xFF1B
FULLWIDTH COLON:'A147:41287:&#xFF1A
FULLWIDTH QUESTION MARK:'A148:41288:&#xFF1F
FULLWIDTH EXCLAMATION MARK:'A149:41289:&#xFF01
PRESENTATION FORM FOR VERTICAL TWO DOT LEADER:'A14A:41290:&#xFE30
HORIZONTAL ELLIPSIS:'A14B:41291:&#x2026
TWO DOT LEADER:'A14C:41292:&#x2025
SMALL COMMA:'A14D:41293:&#xFE50
SMALL IDEOGRAPHIC COMMA:'A14E:41294:&#xFE51
SMALL FULL STOP:'A14F:41295:&#xFE52
MIDDLE DOT:'A150:41296:&#x00B7
SMALL SEMICOLON:'A151:41297:&#xFE54
SMALL COLON:'A152:41298:&#xFE55
SMALL QUESTION MARK:'A153:41299:&#xFE56
SMALL EXCLAMATION MARK:'A154:41300:&#xFE57
FULLWIDTH VERTICAL LINE:'A155:41301:&#xFF5C
EN DASH:'A156:41302:&#x2013
PRESENTATION FORM FOR VERTICAL EM DASH:'A157:41303:&#xFE31
EM DASH:'A158:41304:&#x2014
PRESENTATION FORM FOR VERTICAL LOW LINE:'A159:41305:&#xFE33
BOX DRAWINGS LIGHT LEFT:'A15A:41306:&#x2574
PRESENTATION FORM FOR VERTICAL WAVY LOW LINE:'A15B:41307:&#xFE34
WAVY LOW LINE:'A15C:41308:&#xFE4F
FULLWIDTH LEFT PARENTHESIS:'A15D:41309:&#xFF08
FULLWIDTH RIGHT PARENTHESIS:'A15E:41310:&#xFF09
PRESENTATION FORM FOR VERTICAL LEFT PARENTHESIS:'A15F:41311:&#xFE35
PRESENTATION FORM FOR VERTICAL RIGHT PARENTHESIS:'A160:41312:&#xFE36
FULLWIDTH LEFT CURLY BRACKET:'A161:41313:&#xFF5B
FULLWIDTH RIGHT CURLY BRACKET:'A162:41314:&#xFF5D
PRESENTATION FORM FOR VERTICAL LEFT CURLY BRACKET:'A163:41315:&#xFE37
PRESENTATION FORM FOR VERTICAL RIGHT CURLY BRACKET:'A164:41316:&#xFE38
LEFT TORTOISE SHELL BRACKET:'A165:41317:&#x3014
RIGHT TORTOISE SHELL BRACKET:'A166:41318:&#x3015
PRESENTATION FORM FOR VERTICAL LEFT TORTOISE SHELL BRACKET:'A167:41319:&#xFE39
PRESENTATION FORM FOR VERTICAL RIGHT TORTOISE SHELL BRACKET:'A168:41320:&#xFE3A
LEFT BLACK LENTICULAR BRACKET:'A169:41321:&#x3010
RIGHT BLACK LENTICULAR BRACKET:'A16A:41322:&#x3011
PRESENTATION FORM FOR VERTICAL LEFT BLACK LENTICULAR BRACKET:'A16B:41323:&#xFE3B
PRESENTATION FORM FOR VERTICAL RIGHT BLACK LENTICULAR BRACKET:'A16C:41324:&#xFE3C
LEFT DOUBLE ANGLE BRACKET:'A16D:41325:&#x300A
RIGHT DOUBLE ANGLE BRACKET:'A16E:41326:&#x300B
PRESENTATION FORM FOR VERTICAL LEFT DOUBLE ANGLE BRACKET:'A16F:41327:&#xFE3D
PRESENTATION FORM FOR VERTICAL RIGHT DOUBLE ANGLE BRACKET:'A170:41328:&#xFE3E
LEFT ANGLE BRACKET:'A171:41329:&#x3008
RIGHT ANGLE BRACKET:'A172:41330:&#x3009
PRESENTATION FORM FOR VERTICAL LEFT ANGLE BRACKET:'A173:41331:&#xFE3F
PRESENTATION FORM FOR VERTICAL RIGHT ANGLE BRACKET:'A174:41332:&#xFE40
LEFT CORNER BRACKET:'A175:41333:&#x300C
RIGHT CORNER BRACKET:'A176:41334:&#x300D
PRESENTATION FORM FOR VERTICAL LEFT CORNER BRACKET:'A177:41335:&#xFE41
PRESENTATION FORM FOR VERTICAL RIGHT CORNER BRACKET:'A178:41336:&#xFE42
LEFT WHITE CORNER BRACKET:'A179:41337:&#x300E
RIGHT WHITE CORNER BRACKET:'A17A:41338:&#x300F
PRESENTATION FORM FOR VERTICAL LEFT WHITE CORNER BRACKET:'A17B:41339:&#xFE43
PRESENTATION FORM FOR VERTICAL RIGHT WHITE CORNER BRACKET:'A17C:41340:&#xFE44
SMALL LEFT PARENTHESIS:'A17D:41341:&#xFE59
SMALL RIGHT PARENTHESIS:'A17E:41342:&#xFE5A
SMALL LEFT CURLY BRACKET:'A1A1:41377:&#xFE5B
SMALL RIGHT CURLY BRACKET:'A1A2:41378:&#xFE5C
SMALL LEFT TORTOISE SHELL BRACKET:'A1A3:41379:&#xFE5D
SMALL RIGHT TORTOISE SHELL BRACKET:'A1A4:41380:&#xFE5E
LEFT SINGLE QUOTATION MARK:'A1A5:41381:&#x2018
RIGHT SINGLE QUOTATION MARK:'A1A6:41382:&#x2019
LEFT DOUBLE QUOTATION MARK:'A1A7:41383:&#x201C
RIGHT DOUBLE QUOTATION MARK:'A1A8:41384:&#x201D
REVERSED DOUBLE PRIME QUOTATION MARK:'A1A9:41385:&#x301D
DOUBLE PRIME QUOTATION MARK:'A1AA:41386:&#x301E
REVERSED PRIME:'A1AB:41387:&#x2035
PRIME:'A1AC:41388:&#x2032
FULLWIDTH NUMBER SIGN:'A1AD:41389:&#xFF03
FULLWIDTH AMPERSAND:'A1AE:41390:&#xFF06
FULLWIDTH ASTERISK:'A1AF:41391:&#xFF0A
REFERENCE MARK:'A1B0:41392:&#x203B
SECTION SIGN:'A1B1:41393:&#x00A7
DITTO MARK:'A1B2:41394:&#x3003
WHITE CIRCLE:'A1B3:41395:&#x25CB
BLACK CIRCLE:'A1B4:41396:&#x25CF
WHITE UP-POINTING TRIANGLE:'A1B5:41397:&#x25B3
BLACK UP-POINTING TRIANGLE:'A1B6:41398:&#x25B2
BULLSEYE:'A1B7:41399:&#x25CE
WHITE STAR:'A1B8:41400:&#x2606
BLACK STAR:'A1B9:41401:&#x2605
WHITE DIAMOND:'A1BA:41402:&#x25C7
BLACK DIAMOND:'A1BB:41403:&#x25C6
WHITE SQUARE:'A1BC:41404:&#x25A1
BLACK SQUARE:'A1BD:41405:&#x25A0
WHITE DOWN-POINTING TRIANGLE:'A1BE:41406:&#x25BD
BLACK DOWN-POINTING TRIANGLE:'A1BF:41407:&#x25BC
CIRCLED IDEOGRAPH CORRECT:'A1C0:41408:&#x32A3
CARE OF:'A1C1:41409:&#x2105
MACRON:'A1C2:41410:&#x00AF
FULLWIDTH MACRON:'A1C3:41411:&#xFFE3
FULLWIDTH LOW LINE:'A1C4:41412:&#xFF3F
MODIFIER LETTER LOW MACRON:'A1C5:41413:&#x02CD
DASHED OVERLINE:'A1C6:41414:&#xFE49
CENTRELINE OVERLINE:'A1C7:41415:&#xFE4A
DASHED LOW LINE:'A1C8:41416:&#xFE4D
CENTRELINE LOW LINE:'A1C9:41417:&#xFE4E
WAVY OVERLINE:'A1CA:41418:&#xFE4B
DOUBLE WAVY OVERLINE:'A1CB:41419:&#xFE4C
SMALL NUMBER SIGN:'A1CC:41420:&#xFE5F
SMALL AMPERSAND:'A1CD:41421:&#xFE60
SMALL ASTERISK:'A1CE:41422:&#xFE61
FULLWIDTH PLUS SIGN:'A1CF:41423:&#xFF0B
FULLWIDTH HYPHEN-MINUS:'A1D0:41424:&#xFF0D
MULTIPLICATION SIGN:'A1D1:41425:&#x00D7
DIVISION SIGN:'A1D2:41426:&#x00F7
PLUS-MINUS SIGN:'A1D3:41427:&#x00B1
SQUARE ROOT:'A1D4:41428:&#x221A
FULLWIDTH LESS-THAN SIGN:'A1D5:41429:&#xFF1C
FULLWIDTH GREATER-THAN SIGN:'A1D6:41430:&#xFF1E
FULLWIDTH EQUALS SIGN:'A1D7:41431:&#xFF1D
LESS-THAN OVER EQUAL TO:'A1D8:41432:&#x2266
GREATER-THAN OVER EQUAL TO:'A1D9:41433:&#x2267
NOT EQUAL TO:'A1DA:41434:&#x2260
INFINITY:'A1DB:41435:&#x221E
APPROXIMATELY EQUAL TO OR THE IMAGE OF:'A1DC:41436:&#x2252
IDENTICAL TO:'A1DD:41437:&#x2261
SMALL PLUS SIGN:'A1DE:41438:&#xFE62
SMALL HYPHEN-MINUS:'A1DF:41439:&#xFE63
SMALL LESS-THAN SIGN:'A1E0:41440:&#xFE64
SMALL GREATER-THAN SIGN:'A1E1:41441:&#xFE65
SMALL EQUALS SIGN:'A1E2:41442:&#xFE66
FULLWIDTH TILDE:'A1E3:41443:&#xFF5E
INTERSECTION:'A1E4:41444:&#x2229
UNION:'A1E5:41445:&#x222A
UP TACK:'A1E6:41446:&#x22A5
ANGLE:'A1E7:41447:&#x2220
RIGHT ANGLE:'A1E8:41448:&#x221F
RIGHT TRIANGLE:'A1E9:41449:&#x22BF
SQUARE LOG:'A1EA:41450:&#x33D2
SQUARE LN:'A1EB:41451:&#x33D1
INTEGRAL:'A1EC:41452:&#x222B
CONTOUR INTEGRAL:'A1ED:41453:&#x222E
BECAUSE:'A1EE:41454:&#x2235
THEREFORE:'A1EF:41455:&#x2234
FEMALE SIGN:'A1F0:41456:&#x2640
MALE SIGN:'A1F1:41457:&#x2642
CIRCLED PLUS:'A1F2:41458:&#x2295
CIRCLED DOT OPERATOR:'A1F3:41459:&#x2299
UPWARDS ARROW:'A1F4:41460:&#x2191
DOWNWARDS ARROW:'A1F5:41461:&#x2193
LEFTWARDS ARROW:'A1F6:41462:&#x2190
RIGHTWARDS ARROW:'A1F7:41463:&#x2192
NORTH WEST ARROW:'A1F8:41464:&#x2196
NORTH EAST ARROW:'A1F9:41465:&#x2197
SOUTH WEST ARROW:'A1FA:41466:&#x2199
SOUTH EAST ARROW:'A1FB:41467:&#x2198
PARALLEL TO:'A1FC:41468:&#x2225
DIVIDES:'A1FD:41469:&#x2223
FULLWIDTH SOLIDUS:'A1FE:41470:&#xFF0F
FULLWIDTH REVERSE SOLIDUS:'A240:41536:&#xFF3C
DIVISION SLASH:'A241:41537:&#x2215
SMALL REVERSE SOLIDUS:'A242:41538:&#xFE68
FULLWIDTH DOLLAR SIGN:'A243:41539:&#xFF04
FULLWIDTH YEN SIGN:'A244:41540:&#xFFE5
POSTAL MARK:'A245:41541:&#x3012
FULLWIDTH CENT SIGN:'A246:41542:&#xFFE0
FULLWIDTH POUND SIGN:'A247:41543:&#xFFE1
FULLWIDTH PERCENT SIGN:'A248:41544:&#xFF05
FULLWIDTH COMMERCIAL AT:'A249:41545:&#xFF20
DEGREE CELSIUS:'A24A:41546:&#x2103
DEGREE FAHRENHEIT:'A24B:41547:&#x2109
SMALL DOLLAR SIGN:'A24C:41548:&#xFE69
SMALL PERCENT SIGN:'A24D:41549:&#xFE6A
SMALL COMMERCIAL AT:'A24E:41550:&#xFE6B
SQUARE MIL:'A24F:41551:&#x33D5
SQUARE MM:'A250:41552:&#x339C
SQUARE CM:'A251:41553:&#x339D
SQUARE KM:'A252:41554:&#x339E
SQUARE KM CAPITAL:'A253:41555:&#x33CE
SQUARE M SQUARED:'A254:41556:&#x33A1
SQUARE MG:'A255:41557:&#x338E
SQUARE KG:'A256:41558:&#x338F
SQUARE CC:'A257:41559:&#x33C4
DEGREE SIGN:'A258:41560:&#x00B0
CJK UNIFIED IDEOGRAPH:'A259:41561:&#x5159
CJK UNIFIED IDEOGRAPH:'A25A:41562:&#x515B
CJK UNIFIED IDEOGRAPH:'A25B:41563:&#x515E
CJK UNIFIED IDEOGRAPH:'A25C:41564:&#x515D
CJK UNIFIED IDEOGRAPH:'A25D:41565:&#x5161
CJK UNIFIED IDEOGRAPH:'A25E:41566:&#x5163
CJK UNIFIED IDEOGRAPH:'A25F:41567:&#x55E7
CJK UNIFIED IDEOGRAPH:'A260:41568:&#x74E9
CJK UNIFIED IDEOGRAPH:'A261:41569:&#x7CCE
LOWER ONE EIGHTH BLOCK:'A262:41570:&#x2581
LOWER ONE QUARTER BLOCK:'A263:41571:&#x2582
LOWER THREE EIGHTHS BLOCK:'A264:41572:&#x2583
LOWER HALF BLOCK:'A265:41573:&#x2584
LOWER FIVE EIGHTHS BLOCK:'A266:41574:&#x2585
LOWER THREE QUARTERS BLOCK:'A267:41575:&#x2586
LOWER SEVEN EIGHTHS BLOCK:'A268:41576:&#x2587
FULL BLOCK:'A269:41577:&#x2588
LEFT ONE EIGHTH BLOCK:'A26A:41578:&#x258F
LEFT ONE QUARTER BLOCK:'A26B:41579:&#x258E
LEFT THREE EIGHTHS BLOCK:'A26C:41580:&#x258D
LEFT HALF BLOCK:'A26D:41581:&#x258C
LEFT FIVE EIGHTHS BLOCK:'A26E:41582:&#x258B
LEFT THREE QUARTERS BLOCK:'A26F:41583:&#x258A
LEFT SEVEN EIGHTHS BLOCK:'A270:41584:&#x2589
BOX DRAWINGS LIGHT VERTICAL AND HORIZONTAL:'A271:41585:&#x253C
BOX DRAWINGS LIGHT UP AND HORIZONTAL:'A272:41586:&#x2534
BOX DRAWINGS LIGHT DOWN AND HORIZONTAL:'A273:41587:&#x252C
BOX DRAWINGS LIGHT VERTICAL AND LEFT:'A274:41588:&#x2524
BOX DRAWINGS LIGHT VERTICAL AND RIGHT:'A275:41589:&#x251C
UPPER ONE EIGHTH BLOCK:'A276:41590:&#x2594
BOX DRAWINGS LIGHT HORIZONTAL:'A277:41591:&#x2500
BOX DRAWINGS LIGHT VERTICAL:'A278:41592:&#x2502
RIGHT ONE EIGHTH BLOCK:'A279:41593:&#x2595
BOX DRAWINGS LIGHT DOWN AND RIGHT:'A27A:41594:&#x250C
BOX DRAWINGS LIGHT DOWN AND LEFT:'A27B:41595:&#x2510
BOX DRAWINGS LIGHT UP AND RIGHT:'A27C:41596:&#x2514
BOX DRAWINGS LIGHT UP AND LEFT:'A27D:41597:&#x2518
BOX DRAWINGS LIGHT ARC DOWN AND RIGHT:'A27E:41598:&#x256D
BOX DRAWINGS LIGHT ARC DOWN AND LEFT:'A2A1:41633:&#x256E
BOX DRAWINGS LIGHT ARC UP AND RIGHT:'A2A2:41634:&#x2570
BOX DRAWINGS LIGHT ARC UP AND LEFT:'A2A3:41635:&#x256F
BOX DRAWINGS DOUBLE HORIZONTAL:'A2A4:41636:&#x2550
BOX DRAWINGS VERTICAL SINGLE AND RIGHT DOUBLE:'A2A5:41637:&#x255E
BOX DRAWINGS VERTICAL SINGLE AND HORIZONTAL DOUBLE:'A2A6:41638:&#x256A
BOX DRAWINGS VERTICAL SINGLE AND LEFT DOUBLE:'A2A7:41639:&#x2561
BLACK LOWER RIGHT TRIANGLE:'A2A8:41640:&#x25E2
BLACK LOWER LEFT TRIANGLE:'A2A9:41641:&#x25E3
BLACK UPPER RIGHT TRIANGLE:'A2AA:41642:&#x25E5
BLACK UPPER LEFT TRIANGLE:'A2AB:41643:&#x25E4
BOX DRAWINGS LIGHT DIAGONAL UPPER RIGHT TO LOWER LEFT:'A2AC:41644:&#x2571
BOX DRAWINGS LIGHT DIAGONAL UPPER LEFT TO LOWER RIGHT:'A2AD:41645:&#x2572
BOX DRAWINGS LIGHT DIAGONAL CROSS:'A2AE:41646:&#x2573
FULLWIDTH DIGIT ZERO:'A2AF:41647:&#xFF10
FULLWIDTH DIGIT ONE:'A2B0:41648:&#xFF11
FULLWIDTH DIGIT TWO:'A2B1:41649:&#xFF12
FULLWIDTH DIGIT THREE:'A2B2:41650:&#xFF13
FULLWIDTH DIGIT FOUR:'A2B3:41651:&#xFF14
FULLWIDTH DIGIT FIVE:'A2B4:41652:&#xFF15
FULLWIDTH DIGIT SIX:'A2B5:41653:&#xFF16
FULLWIDTH DIGIT SEVEN:'A2B6:41654:&#xFF17
FULLWIDTH DIGIT EIGHT:'A2B7:41655:&#xFF18
FULLWIDTH DIGIT NINE:'A2B8:41656:&#xFF19
ROMAN NUMERAL ONE:'A2B9:41657:&#x2160
ROMAN NUMERAL TWO:'A2BA:41658:&#x2161
ROMAN NUMERAL THREE:'A2BB:41659:&#x2162
ROMAN NUMERAL FOUR:'A2BC:41660:&#x2163
ROMAN NUMERAL FIVE:'A2BD:41661:&#x2164
ROMAN NUMERAL SIX:'A2BE:41662:&#x2165
ROMAN NUMERAL SEVEN:'A2BF:41663:&#x2166
ROMAN NUMERAL EIGHT:'A2C0:41664:&#x2167
ROMAN NUMERAL NINE:'A2C1:41665:&#x2168
ROMAN NUMERAL TEN:'A2C2:41666:&#x2169
HANGZHOU NUMERAL ONE:'A2C3:41667:&#x3021
HANGZHOU NUMERAL TWO:'A2C4:41668:&#x3022
HANGZHOU NUMERAL THREE:'A2C5:41669:&#x3023
HANGZHOU NUMERAL FOUR:'A2C6:41670:&#x3024
HANGZHOU NUMERAL FIVE:'A2C7:41671:&#x3025
HANGZHOU NUMERAL SIX:'A2C8:41672:&#x3026
HANGZHOU NUMERAL SEVEN:'A2C9:41673:&#x3027
HANGZHOU NUMERAL EIGHT:'A2CA:41674:&#x3028
HANGZHOU NUMERAL NINE:'A2CB:41675:&#x3029
CJK UNIFIED IDEOGRAPH:'A2CC:41676:&#x5341
CJK UNIFIED IDEOGRAPH:'A2CD:41677:&#x5344
CJK UNIFIED IDEOGRAPH:'A2CE:41678:&#x5345
FULLWIDTH LATIN CAPITAL LETTER A:'A2CF:41679:&#xFF21
FULLWIDTH LATIN CAPITAL LETTER B:'A2D0:41680:&#xFF22
FULLWIDTH LATIN CAPITAL LETTER C:'A2D1:41681:&#xFF23
FULLWIDTH LATIN CAPITAL LETTER D:'A2D2:41682:&#xFF24
FULLWIDTH LATIN CAPITAL LETTER E:'A2D3:41683:&#xFF25
FULLWIDTH LATIN CAPITAL LETTER F:'A2D4:41684:&#xFF26
FULLWIDTH LATIN CAPITAL LETTER G:'A2D5:41685:&#xFF27
FULLWIDTH LATIN CAPITAL LETTER H:'A2D6:41686:&#xFF28
FULLWIDTH LATIN CAPITAL LETTER I:'A2D7:41687:&#xFF29
FULLWIDTH LATIN CAPITAL LETTER J:'A2D8:41688:&#xFF2A
FULLWIDTH LATIN CAPITAL LETTER K:'A2D9:41689:&#xFF2B
FULLWIDTH LATIN CAPITAL LETTER L:'A2DA:41690:&#xFF2C
FULLWIDTH LATIN CAPITAL LETTER M:'A2DB:41691:&#xFF2D
FULLWIDTH LATIN CAPITAL LETTER N:'A2DC:41692:&#xFF2E
FULLWIDTH LATIN CAPITAL LETTER O:'A2DD:41693:&#xFF2F
FULLWIDTH LATIN CAPITAL LETTER P:'A2DE:41694:&#xFF30
FULLWIDTH LATIN CAPITAL LETTER Q:'A2DF:41695:&#xFF31
FULLWIDTH LATIN CAPITAL LETTER R:'A2E0:41696:&#xFF32
FULLWIDTH LATIN CAPITAL LETTER S:'A2E1:41697:&#xFF33
FULLWIDTH LATIN CAPITAL LETTER T:'A2E2:41698:&#xFF34
FULLWIDTH LATIN CAPITAL LETTER U:'A2E3:41699:&#xFF35
FULLWIDTH LATIN CAPITAL LETTER V:'A2E4:41700:&#xFF36
FULLWIDTH LATIN CAPITAL LETTER W:'A2E5:41701:&#xFF37
FULLWIDTH LATIN CAPITAL LETTER X:'A2E6:41702:&#xFF38
FULLWIDTH LATIN CAPITAL LETTER Y:'A2E7:41703:&#xFF39
FULLWIDTH LATIN CAPITAL LETTER Z:'A2E8:41704:&#xFF3A
FULLWIDTH LATIN SMALL LETTER A:'A2E9:41705:&#xFF41
FULLWIDTH LATIN SMALL LETTER B:'A2EA:41706:&#xFF42
FULLWIDTH LATIN SMALL LETTER C:'A2EB:41707:&#xFF43
FULLWIDTH LATIN SMALL LETTER D:'A2EC:41708:&#xFF44
FULLWIDTH LATIN SMALL LETTER E:'A2ED:41709:&#xFF45
FULLWIDTH LATIN SMALL LETTER F:'A2EE:41710:&#xFF46
FULLWIDTH LATIN SMALL LETTER G:'A2EF:41711:&#xFF47
FULLWIDTH LATIN SMALL LETTER H:'A2F0:41712:&#xFF48
FULLWIDTH LATIN SMALL LETTER I:'A2F1:41713:&#xFF49
FULLWIDTH LATIN SMALL LETTER J:'A2F2:41714:&#xFF4A
FULLWIDTH LATIN SMALL LETTER K:'A2F3:41715:&#xFF4B
FULLWIDTH LATIN SMALL LETTER L:'A2F4:41716:&#xFF4C
FULLWIDTH LATIN SMALL LETTER M:'A2F5:41717:&#xFF4D
FULLWIDTH LATIN SMALL LETTER N:'A2F6:41718:&#xFF4E
FULLWIDTH LATIN SMALL LETTER O:'A2F7:41719:&#xFF4F
FULLWIDTH LATIN SMALL LETTER P:'A2F8:41720:&#xFF50
FULLWIDTH LATIN SMALL LETTER Q:'A2F9:41721:&#xFF51
FULLWIDTH LATIN SMALL LETTER R:'A2FA:41722:&#xFF52
FULLWIDTH LATIN SMALL LETTER S:'A2FB:41723:&#xFF53
FULLWIDTH LATIN SMALL LETTER T:'A2FC:41724:&#xFF54
FULLWIDTH LATIN SMALL LETTER U:'A2FD:41725:&#xFF55
FULLWIDTH LATIN SMALL LETTER V:'A2FE:41726:&#xFF56
FULLWIDTH LATIN SMALL LETTER W:'A340:41792:&#xFF57
FULLWIDTH LATIN SMALL LETTER X:'A341:41793:&#xFF58
FULLWIDTH LATIN SMALL LETTER Y:'A342:41794:&#xFF59
FULLWIDTH LATIN SMALL LETTER Z:'A343:41795:&#xFF5A
GREEK CAPITAL LETTER ALPHA:'A344:41796:&#x0391
GREEK CAPITAL LETTER BETA:'A345:41797:&#x0392
GREEK CAPITAL LETTER GAMMA:'A346:41798:&#x0393
GREEK CAPITAL LETTER DELTA:'A347:41799:&#x0394
GREEK CAPITAL LETTER EPSILON:'A348:41800:&#x0395
GREEK CAPITAL LETTER ZETA:'A349:41801:&#x0396
GREEK CAPITAL LETTER ETA:'A34A:41802:&#x0397
GREEK CAPITAL LETTER THETA:'A34B:41803:&#x0398
GREEK CAPITAL LETTER IOTA:'A34C:41804:&#x0399
GREEK CAPITAL LETTER KAPPA:'A34D:41805:&#x039A
GREEK CAPITAL LETTER LAMDA:'A34E:41806:&#x039B
GREEK CAPITAL LETTER MU:'A34F:41807:&#x039C
GREEK CAPITAL LETTER NU:'A350:41808:&#x039D
GREEK CAPITAL LETTER XI:'A351:41809:&#x039E
GREEK CAPITAL LETTER OMICRON:'A352:41810:&#x039F
GREEK CAPITAL LETTER PI:'A353:41811:&#x03A0
GREEK CAPITAL LETTER RHO:'A354:41812:&#x03A1
GREEK CAPITAL LETTER SIGMA:'A355:41813:&#x03A3
GREEK CAPITAL LETTER TAU:'A356:41814:&#x03A4
GREEK CAPITAL LETTER UPSILON:'A357:41815:&#x03A5
GREEK CAPITAL LETTER PHI:'A358:41816:&#x03A6
GREEK CAPITAL LETTER CHI:'A359:41817:&#x03A7
GREEK CAPITAL LETTER PSI:'A35A:41818:&#x03A8
GREEK CAPITAL LETTER OMEGA:'A35B:41819:&#x03A9
GREEK SMALL LETTER ALPHA:'A35C:41820:&#x03B1
GREEK SMALL LETTER BETA:'A35D:41821:&#x03B2
GREEK SMALL LETTER GAMMA:'A35E:41822:&#x03B3
GREEK SMALL LETTER DELTA:'A35F:41823:&#x03B4
GREEK SMALL LETTER EPSILON:'A360:41824:&#x03B5
GREEK SMALL LETTER ZETA:'A361:41825:&#x03B6
GREEK SMALL LETTER ETA:'A362:41826:&#x03B7
GREEK SMALL LETTER THETA:'A363:41827:&#x03B8
GREEK SMALL LETTER IOTA:'A364:41828:&#x03B9
GREEK SMALL LETTER KAPPA:'A365:41829:&#x03BA
GREEK SMALL LETTER LAMDA:'A366:41830:&#x03BB
GREEK SMALL LETTER MU:'A367:41831:&#x03BC
GREEK SMALL LETTER NU:'A368:41832:&#x03BD
GREEK SMALL LETTER XI:'A369:41833:&#x03BE
GREEK SMALL LETTER OMICRON:'A36A:41834:&#x03BF
GREEK SMALL LETTER PI:'A36B:41835:&#x03C0
GREEK SMALL LETTER RHO:'A36C:41836:&#x03C1
GREEK SMALL LETTER SIGMA:'A36D:41837:&#x03C3
GREEK SMALL LETTER TAU:'A36E:41838:&#x03C4
GREEK SMALL LETTER UPSILON:'A36F:41839:&#x03C5
GREEK SMALL LETTER PHI:'A370:41840:&#x03C6
GREEK SMALL LETTER CHI:'A371:41841:&#x03C7
GREEK SMALL LETTER PSI:'A372:41842:&#x03C8
GREEK SMALL LETTER OMEGA:'A373:41843:&#x03C9
BOPOMOFO LETTER B:'A374:41844:&#x3105
BOPOMOFO LETTER P:'A375:41845:&#x3106
BOPOMOFO LETTER M:'A376:41846:&#x3107
BOPOMOFO LETTER F:'A377:41847:&#x3108
BOPOMOFO LETTER D:'A378:41848:&#x3109
BOPOMOFO LETTER T:'A379:41849:&#x310A
BOPOMOFO LETTER N:'A37A:41850:&#x310B
BOPOMOFO LETTER L:'A37B:41851:&#x310C
BOPOMOFO LETTER G:'A37C:41852:&#x310D
BOPOMOFO LETTER K:'A37D:41853:&#x310E
BOPOMOFO LETTER H:'A37E:41854:&#x310F
BOPOMOFO LETTER J:'A3A1:41889:&#x3110
BOPOMOFO LETTER Q:'A3A2:41890:&#x3111
BOPOMOFO LETTER X:'A3A3:41891:&#x3112
BOPOMOFO LETTER ZH:'A3A4:41892:&#x3113
BOPOMOFO LETTER CH:'A3A5:41893:&#x3114
BOPOMOFO LETTER SH:'A3A6:41894:&#x3115
BOPOMOFO LETTER R:'A3A7:41895:&#x3116
BOPOMOFO LETTER Z:'A3A8:41896:&#x3117
BOPOMOFO LETTER C:'A3A9:41897:&#x3118
BOPOMOFO LETTER S:'A3AA:41898:&#x3119
BOPOMOFO LETTER A:'A3AB:41899:&#x311A
BOPOMOFO LETTER O:'A3AC:41900:&#x311B
BOPOMOFO LETTER E:'A3AD:41901:&#x311C
BOPOMOFO LETTER EH:'A3AE:41902:&#x311D
BOPOMOFO LETTER AI:'A3AF:41903:&#x311E
BOPOMOFO LETTER EI:'A3B0:41904:&#x311F
BOPOMOFO LETTER AU:'A3B1:41905:&#x3120
BOPOMOFO LETTER OU:'A3B2:41906:&#x3121
BOPOMOFO LETTER AN:'A3B3:41907:&#x3122
BOPOMOFO LETTER EN:'A3B4:41908:&#x3123
BOPOMOFO LETTER ANG:'A3B5:41909:&#x3124
BOPOMOFO LETTER ENG:'A3B6:41910:&#x3125
BOPOMOFO LETTER ER:'A3B7:41911:&#x3126
BOPOMOFO LETTER I:'A3B8:41912:&#x3127
BOPOMOFO LETTER U:'A3B9:41913:&#x3128
BOPOMOFO LETTER IU:'A3BA:41914:&#x3129
DOT ABOVE:'A3BB:41915:&#x02D9
MODIFIER LETTER MACRON:'A3BC:41916:&#x02C9
MODIFIER LETTER ACUTE ACCENT:'A3BD:41917:&#x02CA
CARON:'A3BE:41918:&#x02C7
MODIFIER LETTER GRAVE ACCENT:'A3BF:41919:&#x02CB
EURO SIGN:'A3E1:41953:&#x20AC
CJK UNIFIED IDEOGRAPH:'A440:42048:&#x4E00
CJK UNIFIED IDEOGRAPH:'A441:42049:&#x4E59
CJK UNIFIED IDEOGRAPH:'A442:42050:&#x4E01
CJK UNIFIED IDEOGRAPH:'A443:42051:&#x4E03
CJK UNIFIED IDEOGRAPH:'A444:42052:&#x4E43
CJK UNIFIED IDEOGRAPH:'A445:42053:&#x4E5D
CJK UNIFIED IDEOGRAPH:'A446:42054:&#x4E86
CJK UNIFIED IDEOGRAPH:'A447:42055:&#x4E8C
CJK UNIFIED IDEOGRAPH:'A448:42056:&#x4EBA
CJK UNIFIED IDEOGRAPH:'A449:42057:&#x513F
CJK UNIFIED IDEOGRAPH:'A44A:42058:&#x5165
CJK UNIFIED IDEOGRAPH:'A44B:42059:&#x516B
CJK UNIFIED IDEOGRAPH:'A44C:42060:&#x51E0
CJK UNIFIED IDEOGRAPH:'A44D:42061:&#x5200
CJK UNIFIED IDEOGRAPH:'A44E:42062:&#x5201
CJK UNIFIED IDEOGRAPH:'A44F:42063:&#x529B
CJK UNIFIED IDEOGRAPH:'A450:42064:&#x5315
CJK UNIFIED IDEOGRAPH:'A451:42065:&#x5341
CJK UNIFIED IDEOGRAPH:'A452:42066:&#x535C
CJK UNIFIED IDEOGRAPH:'A453:42067:&#x53C8
CJK UNIFIED IDEOGRAPH:'A454:42068:&#x4E09
CJK UNIFIED IDEOGRAPH:'A455:42069:&#x4E0B
CJK UNIFIED IDEOGRAPH:'A456:42070:&#x4E08
CJK UNIFIED IDEOGRAPH:'A457:42071:&#x4E0A
CJK UNIFIED IDEOGRAPH:'A458:42072:&#x4E2B
CJK UNIFIED IDEOGRAPH:'A459:42073:&#x4E38
CJK UNIFIED IDEOGRAPH:'A45A:42074:&#x51E1
CJK UNIFIED IDEOGRAPH:'A45B:42075:&#x4E45
CJK UNIFIED IDEOGRAPH:'A45C:42076:&#x4E48
CJK UNIFIED IDEOGRAPH:'A45D:42077:&#x4E5F
CJK UNIFIED IDEOGRAPH:'A45E:42078:&#x4E5E
CJK UNIFIED IDEOGRAPH:'A45F:42079:&#x4E8E
CJK UNIFIED IDEOGRAPH:'A460:42080:&#x4EA1
CJK UNIFIED IDEOGRAPH:'A461:42081:&#x5140
CJK UNIFIED IDEOGRAPH:'A462:42082:&#x5203
CJK UNIFIED IDEOGRAPH:'A463:42083:&#x52FA
CJK UNIFIED IDEOGRAPH:'A464:42084:&#x5343
CJK UNIFIED IDEOGRAPH:'A465:42085:&#x53C9
CJK UNIFIED IDEOGRAPH:'A466:42086:&#x53E3
CJK UNIFIED IDEOGRAPH:'A467:42087:&#x571F
CJK UNIFIED IDEOGRAPH:'A468:42088:&#x58EB
CJK UNIFIED IDEOGRAPH:'A469:42089:&#x5915
CJK UNIFIED IDEOGRAPH:'A46A:42090:&#x5927
CJK UNIFIED IDEOGRAPH:'A46B:42091:&#x5973
CJK UNIFIED IDEOGRAPH:'A46C:42092:&#x5B50
CJK UNIFIED IDEOGRAPH:'A46D:42093:&#x5B51
CJK UNIFIED IDEOGRAPH:'A46E:42094:&#x5B53
CJK UNIFIED IDEOGRAPH:'A46F:42095:&#x5BF8
CJK UNIFIED IDEOGRAPH:'A470:42096:&#x5C0F
CJK UNIFIED IDEOGRAPH:'A471:42097:&#x5C22
CJK UNIFIED IDEOGRAPH:'A472:42098:&#x5C38
CJK UNIFIED IDEOGRAPH:'A473:42099:&#x5C71
CJK UNIFIED IDEOGRAPH:'A474:42100:&#x5DDD
CJK UNIFIED IDEOGRAPH:'A475:42101:&#x5DE5
CJK UNIFIED IDEOGRAPH:'A476:42102:&#x5DF1
CJK UNIFIED IDEOGRAPH:'A477:42103:&#x5DF2
CJK UNIFIED IDEOGRAPH:'A478:42104:&#x5DF3
CJK UNIFIED IDEOGRAPH:'A479:42105:&#x5DFE
CJK UNIFIED IDEOGRAPH:'A47A:42106:&#x5E72
CJK UNIFIED IDEOGRAPH:'A47B:42107:&#x5EFE
CJK UNIFIED IDEOGRAPH:'A47C:42108:&#x5F0B
CJK UNIFIED IDEOGRAPH:'A47D:42109:&#x5F13
CJK UNIFIED IDEOGRAPH:'A47E:42110:&#x624D
CJK UNIFIED IDEOGRAPH:'A4A1:42145:&#x4E11
CJK UNIFIED IDEOGRAPH:'A4A2:42146:&#x4E10
CJK UNIFIED IDEOGRAPH:'A4A3:42147:&#x4E0D
CJK UNIFIED IDEOGRAPH:'A4A4:42148:&#x4E2D
CJK UNIFIED IDEOGRAPH:'A4A5:42149:&#x4E30
CJK UNIFIED IDEOGRAPH:'A4A6:42150:&#x4E39
CJK UNIFIED IDEOGRAPH:'A4A7:42151:&#x4E4B
CJK UNIFIED IDEOGRAPH:'A4A8:42152:&#x5C39
CJK UNIFIED IDEOGRAPH:'A4A9:42153:&#x4E88
CJK UNIFIED IDEOGRAPH:'A4AA:42154:&#x4E91
CJK UNIFIED IDEOGRAPH:'A4AB:42155:&#x4E95
CJK UNIFIED IDEOGRAPH:'A4AC:42156:&#x4E92
CJK UNIFIED IDEOGRAPH:'A4AD:42157:&#x4E94
CJK UNIFIED IDEOGRAPH:'A4AE:42158:&#x4EA2
CJK UNIFIED IDEOGRAPH:'A4AF:42159:&#x4EC1
CJK UNIFIED IDEOGRAPH:'A4B0:42160:&#x4EC0
CJK UNIFIED IDEOGRAPH:'A4B1:42161:&#x4EC3
CJK UNIFIED IDEOGRAPH:'A4B2:42162:&#x4EC6
CJK UNIFIED IDEOGRAPH:'A4B3:42163:&#x4EC7
CJK UNIFIED IDEOGRAPH:'A4B4:42164:&#x4ECD
CJK UNIFIED IDEOGRAPH:'A4B5:42165:&#x4ECA
CJK UNIFIED IDEOGRAPH:'A4B6:42166:&#x4ECB
CJK UNIFIED IDEOGRAPH:'A4B7:42167:&#x4EC4
CJK UNIFIED IDEOGRAPH:'A4B8:42168:&#x5143
CJK UNIFIED IDEOGRAPH:'A4B9:42169:&#x5141
CJK UNIFIED IDEOGRAPH:'A4BA:42170:&#x5167
CJK UNIFIED IDEOGRAPH:'A4BB:42171:&#x516D
CJK UNIFIED IDEOGRAPH:'A4BC:42172:&#x516E
CJK UNIFIED IDEOGRAPH:'A4BD:42173:&#x516C
CJK UNIFIED IDEOGRAPH:'A4BE:42174:&#x5197
CJK UNIFIED IDEOGRAPH:'A4BF:42175:&#x51F6
CJK UNIFIED IDEOGRAPH:'A4C0:42176:&#x5206
CJK UNIFIED IDEOGRAPH:'A4C1:42177:&#x5207
CJK UNIFIED IDEOGRAPH:'A4C2:42178:&#x5208
CJK UNIFIED IDEOGRAPH:'A4C3:42179:&#x52FB
CJK UNIFIED IDEOGRAPH:'A4C4:42180:&#x52FE
CJK UNIFIED IDEOGRAPH:'A4C5:42181:&#x52FF
CJK UNIFIED IDEOGRAPH:'A4C6:42182:&#x5316
CJK UNIFIED IDEOGRAPH:'A4C7:42183:&#x5339
CJK UNIFIED IDEOGRAPH:'A4C8:42184:&#x5348
CJK UNIFIED IDEOGRAPH:'A4C9:42185:&#x5347
CJK UNIFIED IDEOGRAPH:'A4CA:42186:&#x5345
CJK UNIFIED IDEOGRAPH:'A4CB:42187:&#x535E
CJK UNIFIED IDEOGRAPH:'A4CC:42188:&#x5384
CJK UNIFIED IDEOGRAPH:'A4CD:42189:&#x53CB
CJK UNIFIED IDEOGRAPH:'A4CE:42190:&#x53CA
CJK UNIFIED IDEOGRAPH:'A4CF:42191:&#x53CD
CJK UNIFIED IDEOGRAPH:'A4D0:42192:&#x58EC
CJK UNIFIED IDEOGRAPH:'A4D1:42193:&#x5929
CJK UNIFIED IDEOGRAPH:'A4D2:42194:&#x592B
CJK UNIFIED IDEOGRAPH:'A4D3:42195:&#x592A
CJK UNIFIED IDEOGRAPH:'A4D4:42196:&#x592D
CJK UNIFIED IDEOGRAPH:'A4D5:42197:&#x5B54
CJK UNIFIED IDEOGRAPH:'A4D6:42198:&#x5C11
CJK UNIFIED IDEOGRAPH:'A4D7:42199:&#x5C24
CJK UNIFIED IDEOGRAPH:'A4D8:42200:&#x5C3A
CJK UNIFIED IDEOGRAPH:'A4D9:42201:&#x5C6F
CJK UNIFIED IDEOGRAPH:'A4DA:42202:&#x5DF4
CJK UNIFIED IDEOGRAPH:'A4DB:42203:&#x5E7B
CJK UNIFIED IDEOGRAPH:'A4DC:42204:&#x5EFF
CJK UNIFIED IDEOGRAPH:'A4DD:42205:&#x5F14
CJK UNIFIED IDEOGRAPH:'A4DE:42206:&#x5F15
CJK UNIFIED IDEOGRAPH:'A4DF:42207:&#x5FC3
CJK UNIFIED IDEOGRAPH:'A4E0:42208:&#x6208
CJK UNIFIED IDEOGRAPH:'A4E1:42209:&#x6236
CJK UNIFIED IDEOGRAPH:'A4E2:42210:&#x624B
CJK UNIFIED IDEOGRAPH:'A4E3:42211:&#x624E
CJK UNIFIED IDEOGRAPH:'A4E4:42212:&#x652F
CJK UNIFIED IDEOGRAPH:'A4E5:42213:&#x6587
CJK UNIFIED IDEOGRAPH:'A4E6:42214:&#x6597
CJK UNIFIED IDEOGRAPH:'A4E7:42215:&#x65A4
CJK UNIFIED IDEOGRAPH:'A4E8:42216:&#x65B9
CJK UNIFIED IDEOGRAPH:'A4E9:42217:&#x65E5
CJK UNIFIED IDEOGRAPH:'A4EA:42218:&#x66F0
CJK UNIFIED IDEOGRAPH:'A4EB:42219:&#x6708
CJK UNIFIED IDEOGRAPH:'A4EC:42220:&#x6728
CJK UNIFIED IDEOGRAPH:'A4ED:42221:&#x6B20
CJK UNIFIED IDEOGRAPH:'A4EE:42222:&#x6B62
CJK UNIFIED IDEOGRAPH:'A4EF:42223:&#x6B79
CJK UNIFIED IDEOGRAPH:'A4F0:42224:&#x6BCB
CJK UNIFIED IDEOGRAPH:'A4F1:42225:&#x6BD4
CJK UNIFIED IDEOGRAPH:'A4F2:42226:&#x6BDB
CJK UNIFIED IDEOGRAPH:'A4F3:42227:&#x6C0F
CJK UNIFIED IDEOGRAPH:'A4F4:42228:&#x6C34
CJK UNIFIED IDEOGRAPH:'A4F5:42229:&#x706B
CJK UNIFIED IDEOGRAPH:'A4F6:42230:&#x722A
CJK UNIFIED IDEOGRAPH:'A4F7:42231:&#x7236
CJK UNIFIED IDEOGRAPH:'A4F8:42232:&#x723B
CJK UNIFIED IDEOGRAPH:'A4F9:42233:&#x7247
CJK UNIFIED IDEOGRAPH:'A4FA:42234:&#x7259
CJK UNIFIED IDEOGRAPH:'A4FB:42235:&#x725B
CJK UNIFIED IDEOGRAPH:'A4FC:42236:&#x72AC
CJK UNIFIED IDEOGRAPH:'A4FD:42237:&#x738B
CJK UNIFIED IDEOGRAPH:'A4FE:42238:&#x4E19
CJK UNIFIED IDEOGRAPH:'A540:42304:&#x4E16
CJK UNIFIED IDEOGRAPH:'A541:42305:&#x4E15
CJK UNIFIED IDEOGRAPH:'A542:42306:&#x4E14
CJK UNIFIED IDEOGRAPH:'A543:42307:&#x4E18
CJK UNIFIED IDEOGRAPH:'A544:42308:&#x4E3B
CJK UNIFIED IDEOGRAPH:'A545:42309:&#x4E4D
CJK UNIFIED IDEOGRAPH:'A546:42310:&#x4E4F
CJK UNIFIED IDEOGRAPH:'A547:42311:&#x4E4E
CJK UNIFIED IDEOGRAPH:'A548:42312:&#x4EE5
CJK UNIFIED IDEOGRAPH:'A549:42313:&#x4ED8
CJK UNIFIED IDEOGRAPH:'A54A:42314:&#x4ED4
CJK UNIFIED IDEOGRAPH:'A54B:42315:&#x4ED5
CJK UNIFIED IDEOGRAPH:'A54C:42316:&#x4ED6
CJK UNIFIED IDEOGRAPH:'A54D:42317:&#x4ED7
CJK UNIFIED IDEOGRAPH:'A54E:42318:&#x4EE3
CJK UNIFIED IDEOGRAPH:'A54F:42319:&#x4EE4
CJK UNIFIED IDEOGRAPH:'A550:42320:&#x4ED9
CJK UNIFIED IDEOGRAPH:'A551:42321:&#x4EDE
CJK UNIFIED IDEOGRAPH:'A552:42322:&#x5145
CJK UNIFIED IDEOGRAPH:'A553:42323:&#x5144
CJK UNIFIED IDEOGRAPH:'A554:42324:&#x5189
CJK UNIFIED IDEOGRAPH:'A555:42325:&#x518A
CJK UNIFIED IDEOGRAPH:'A556:42326:&#x51AC
CJK UNIFIED IDEOGRAPH:'A557:42327:&#x51F9
CJK UNIFIED IDEOGRAPH:'A558:42328:&#x51FA
CJK UNIFIED IDEOGRAPH:'A559:42329:&#x51F8
CJK UNIFIED IDEOGRAPH:'A55A:42330:&#x520A
CJK UNIFIED IDEOGRAPH:'A55B:42331:&#x52A0
CJK UNIFIED IDEOGRAPH:'A55C:42332:&#x529F
CJK UNIFIED IDEOGRAPH:'A55D:42333:&#x5305
CJK UNIFIED IDEOGRAPH:'A55E:42334:&#x5306
CJK UNIFIED IDEOGRAPH:'A55F:42335:&#x5317
CJK UNIFIED IDEOGRAPH:'A560:42336:&#x531D
CJK UNIFIED IDEOGRAPH:'A561:42337:&#x4EDF
CJK UNIFIED IDEOGRAPH:'A562:42338:&#x534A
CJK UNIFIED IDEOGRAPH:'A563:42339:&#x5349
CJK UNIFIED IDEOGRAPH:'A564:42340:&#x5361
CJK UNIFIED IDEOGRAPH:'A565:42341:&#x5360
CJK UNIFIED IDEOGRAPH:'A566:42342:&#x536F
CJK UNIFIED IDEOGRAPH:'A567:42343:&#x536E
CJK UNIFIED IDEOGRAPH:'A568:42344:&#x53BB
CJK UNIFIED IDEOGRAPH:'A569:42345:&#x53EF
CJK UNIFIED IDEOGRAPH:'A56A:42346:&#x53E4
CJK UNIFIED IDEOGRAPH:'A56B:42347:&#x53F3
CJK UNIFIED IDEOGRAPH:'A56C:42348:&#x53EC
CJK UNIFIED IDEOGRAPH:'A56D:42349:&#x53EE
CJK UNIFIED IDEOGRAPH:'A56E:42350:&#x53E9
CJK UNIFIED IDEOGRAPH:'A56F:42351:&#x53E8
CJK UNIFIED IDEOGRAPH:'A570:42352:&#x53FC
CJK UNIFIED IDEOGRAPH:'A571:42353:&#x53F8
CJK UNIFIED IDEOGRAPH:'A572:42354:&#x53F5
CJK UNIFIED IDEOGRAPH:'A573:42355:&#x53EB
CJK UNIFIED IDEOGRAPH:'A574:42356:&#x53E6
CJK UNIFIED IDEOGRAPH:'A575:42357:&#x53EA
CJK UNIFIED IDEOGRAPH:'A576:42358:&#x53F2
CJK UNIFIED IDEOGRAPH:'A577:42359:&#x53F1
CJK UNIFIED IDEOGRAPH:'A578:42360:&#x53F0
CJK UNIFIED IDEOGRAPH:'A579:42361:&#x53E5
CJK UNIFIED IDEOGRAPH:'A57A:42362:&#x53ED
CJK UNIFIED IDEOGRAPH:'A57B:42363:&#x53FB
CJK UNIFIED IDEOGRAPH:'A57C:42364:&#x56DB
CJK UNIFIED IDEOGRAPH:'A57D:42365:&#x56DA
CJK UNIFIED IDEOGRAPH:'A57E:42366:&#x5916
CJK UNIFIED IDEOGRAPH:'A5A1:42401:&#x592E
CJK UNIFIED IDEOGRAPH:'A5A2:42402:&#x5931
CJK UNIFIED IDEOGRAPH:'A5A3:42403:&#x5974
CJK UNIFIED IDEOGRAPH:'A5A4:42404:&#x5976
CJK UNIFIED IDEOGRAPH:'A5A5:42405:&#x5B55
CJK UNIFIED IDEOGRAPH:'A5A6:42406:&#x5B83
CJK UNIFIED IDEOGRAPH:'A5A7:42407:&#x5C3C
CJK UNIFIED IDEOGRAPH:'A5A8:42408:&#x5DE8
CJK UNIFIED IDEOGRAPH:'A5A9:42409:&#x5DE7
CJK UNIFIED IDEOGRAPH:'A5AA:42410:&#x5DE6
CJK UNIFIED IDEOGRAPH:'A5AB:42411:&#x5E02
CJK UNIFIED IDEOGRAPH:'A5AC:42412:&#x5E03
CJK UNIFIED IDEOGRAPH:'A5AD:42413:&#x5E73
CJK UNIFIED IDEOGRAPH:'A5AE:42414:&#x5E7C
CJK UNIFIED IDEOGRAPH:'A5AF:42415:&#x5F01
CJK UNIFIED IDEOGRAPH:'A5B0:42416:&#x5F18
CJK UNIFIED IDEOGRAPH:'A5B1:42417:&#x5F17
CJK UNIFIED IDEOGRAPH:'A5B2:42418:&#x5FC5
CJK UNIFIED IDEOGRAPH:'A5B3:42419:&#x620A
CJK UNIFIED IDEOGRAPH:'A5B4:42420:&#x6253
CJK UNIFIED IDEOGRAPH:'A5B5:42421:&#x6254
CJK UNIFIED IDEOGRAPH:'A5B6:42422:&#x6252
CJK UNIFIED IDEOGRAPH:'A5B7:42423:&#x6251
CJK UNIFIED IDEOGRAPH:'A5B8:42424:&#x65A5
CJK UNIFIED IDEOGRAPH:'A5B9:42425:&#x65E6
CJK UNIFIED IDEOGRAPH:'A5BA:42426:&#x672E
CJK UNIFIED IDEOGRAPH:'A5BB:42427:&#x672C
CJK UNIFIED IDEOGRAPH:'A5BC:42428:&#x672A
CJK UNIFIED IDEOGRAPH:'A5BD:42429:&#x672B
CJK UNIFIED IDEOGRAPH:'A5BE:42430:&#x672D
CJK UNIFIED IDEOGRAPH:'A5BF:42431:&#x6B63
CJK UNIFIED IDEOGRAPH:'A5C0:42432:&#x6BCD
CJK UNIFIED IDEOGRAPH:'A5C1:42433:&#x6C11
CJK UNIFIED IDEOGRAPH:'A5C2:42434:&#x6C10
CJK UNIFIED IDEOGRAPH:'A5C3:42435:&#x6C38
CJK UNIFIED IDEOGRAPH:'A5C4:42436:&#x6C41
CJK UNIFIED IDEOGRAPH:'A5C5:42437:&#x6C40
CJK UNIFIED IDEOGRAPH:'A5C6:42438:&#x6C3E
CJK UNIFIED IDEOGRAPH:'A5C7:42439:&#x72AF
CJK UNIFIED IDEOGRAPH:'A5C8:42440:&#x7384
CJK UNIFIED IDEOGRAPH:'A5C9:42441:&#x7389
CJK UNIFIED IDEOGRAPH:'A5CA:42442:&#x74DC
CJK UNIFIED IDEOGRAPH:'A5CB:42443:&#x74E6
CJK UNIFIED IDEOGRAPH:'A5CC:42444:&#x7518
CJK UNIFIED IDEOGRAPH:'A5CD:42445:&#x751F
CJK UNIFIED IDEOGRAPH:'A5CE:42446:&#x7528
CJK UNIFIED IDEOGRAPH:'A5CF:42447:&#x7529
CJK UNIFIED IDEOGRAPH:'A5D0:42448:&#x7530
CJK UNIFIED IDEOGRAPH:'A5D1:42449:&#x7531
CJK UNIFIED IDEOGRAPH:'A5D2:42450:&#x7532
CJK UNIFIED IDEOGRAPH:'A5D3:42451:&#x7533
CJK UNIFIED IDEOGRAPH:'A5D4:42452:&#x758B
CJK UNIFIED IDEOGRAPH:'A5D5:42453:&#x767D
CJK UNIFIED IDEOGRAPH:'A5D6:42454:&#x76AE
CJK UNIFIED IDEOGRAPH:'A5D7:42455:&#x76BF
CJK UNIFIED IDEOGRAPH:'A5D8:42456:&#x76EE
CJK UNIFIED IDEOGRAPH:'A5D9:42457:&#x77DB
CJK UNIFIED IDEOGRAPH:'A5DA:42458:&#x77E2
CJK UNIFIED IDEOGRAPH:'A5DB:42459:&#x77F3
CJK UNIFIED IDEOGRAPH:'A5DC:42460:&#x793A
CJK UNIFIED IDEOGRAPH:'A5DD:42461:&#x79BE
CJK UNIFIED IDEOGRAPH:'A5DE:42462:&#x7A74
CJK UNIFIED IDEOGRAPH:'A5DF:42463:&#x7ACB
CJK UNIFIED IDEOGRAPH:'A5E0:42464:&#x4E1E
CJK UNIFIED IDEOGRAPH:'A5E1:42465:&#x4E1F
CJK UNIFIED IDEOGRAPH:'A5E2:42466:&#x4E52
CJK UNIFIED IDEOGRAPH:'A5E3:42467:&#x4E53
CJK UNIFIED IDEOGRAPH:'A5E4:42468:&#x4E69
CJK UNIFIED IDEOGRAPH:'A5E5:42469:&#x4E99
CJK UNIFIED IDEOGRAPH:'A5E6:42470:&#x4EA4
CJK UNIFIED IDEOGRAPH:'A5E7:42471:&#x4EA6
CJK UNIFIED IDEOGRAPH:'A5E8:42472:&#x4EA5
CJK UNIFIED IDEOGRAPH:'A5E9:42473:&#x4EFF
CJK UNIFIED IDEOGRAPH:'A5EA:42474:&#x4F09
CJK UNIFIED IDEOGRAPH:'A5EB:42475:&#x4F19
CJK UNIFIED IDEOGRAPH:'A5EC:42476:&#x4F0A
CJK UNIFIED IDEOGRAPH:'A5ED:42477:&#x4F15
CJK UNIFIED IDEOGRAPH:'A5EE:42478:&#x4F0D
CJK UNIFIED IDEOGRAPH:'A5EF:42479:&#x4F10
CJK UNIFIED IDEOGRAPH:'A5F0:42480:&#x4F11
CJK UNIFIED IDEOGRAPH:'A5F1:42481:&#x4F0F
CJK UNIFIED IDEOGRAPH:'A5F2:42482:&#x4EF2
CJK UNIFIED IDEOGRAPH:'A5F3:42483:&#x4EF6
CJK UNIFIED IDEOGRAPH:'A5F4:42484:&#x4EFB
CJK UNIFIED IDEOGRAPH:'A5F5:42485:&#x4EF0
CJK UNIFIED IDEOGRAPH:'A5F6:42486:&#x4EF3
CJK UNIFIED IDEOGRAPH:'A5F7:42487:&#x4EFD
CJK UNIFIED IDEOGRAPH:'A5F8:42488:&#x4F01
CJK UNIFIED IDEOGRAPH:'A5F9:42489:&#x4F0B
CJK UNIFIED IDEOGRAPH:'A5FA:42490:&#x5149
CJK UNIFIED IDEOGRAPH:'A5FB:42491:&#x5147
CJK UNIFIED IDEOGRAPH:'A5FC:42492:&#x5146
CJK UNIFIED IDEOGRAPH:'A5FD:42493:&#x5148
CJK UNIFIED IDEOGRAPH:'A5FE:42494:&#x5168
CJK UNIFIED IDEOGRAPH:'A640:42560:&#x5171
CJK UNIFIED IDEOGRAPH:'A641:42561:&#x518D
CJK UNIFIED IDEOGRAPH:'A642:42562:&#x51B0
CJK UNIFIED IDEOGRAPH:'A643:42563:&#x5217
CJK UNIFIED IDEOGRAPH:'A644:42564:&#x5211
CJK UNIFIED IDEOGRAPH:'A645:42565:&#x5212
CJK UNIFIED IDEOGRAPH:'A646:42566:&#x520E
CJK UNIFIED IDEOGRAPH:'A647:42567:&#x5216
CJK UNIFIED IDEOGRAPH:'A648:42568:&#x52A3
CJK UNIFIED IDEOGRAPH:'A649:42569:&#x5308
CJK UNIFIED IDEOGRAPH:'A64A:42570:&#x5321
CJK UNIFIED IDEOGRAPH:'A64B:42571:&#x5320
CJK UNIFIED IDEOGRAPH:'A64C:42572:&#x5370
CJK UNIFIED IDEOGRAPH:'A64D:42573:&#x5371
CJK UNIFIED IDEOGRAPH:'A64E:42574:&#x5409
CJK UNIFIED IDEOGRAPH:'A64F:42575:&#x540F
CJK UNIFIED IDEOGRAPH:'A650:42576:&#x540C
CJK UNIFIED IDEOGRAPH:'A651:42577:&#x540A
CJK UNIFIED IDEOGRAPH:'A652:42578:&#x5410
CJK UNIFIED IDEOGRAPH:'A653:42579:&#x5401
CJK UNIFIED IDEOGRAPH:'A654:42580:&#x540B
CJK UNIFIED IDEOGRAPH:'A655:42581:&#x5404
CJK UNIFIED IDEOGRAPH:'A656:42582:&#x5411
CJK UNIFIED IDEOGRAPH:'A657:42583:&#x540D
CJK UNIFIED IDEOGRAPH:'A658:42584:&#x5408
CJK UNIFIED IDEOGRAPH:'A659:42585:&#x5403
CJK UNIFIED IDEOGRAPH:'A65A:42586:&#x540E
CJK UNIFIED IDEOGRAPH:'A65B:42587:&#x5406
CJK UNIFIED IDEOGRAPH:'A65C:42588:&#x5412
CJK UNIFIED IDEOGRAPH:'A65D:42589:&#x56E0
CJK UNIFIED IDEOGRAPH:'A65E:42590:&#x56DE
CJK UNIFIED IDEOGRAPH:'A65F:42591:&#x56DD
CJK UNIFIED IDEOGRAPH:'A660:42592:&#x5733
CJK UNIFIED IDEOGRAPH:'A661:42593:&#x5730
CJK UNIFIED IDEOGRAPH:'A662:42594:&#x5728
CJK UNIFIED IDEOGRAPH:'A663:42595:&#x572D
CJK UNIFIED IDEOGRAPH:'A664:42596:&#x572C
CJK UNIFIED IDEOGRAPH:'A665:42597:&#x572F
CJK UNIFIED IDEOGRAPH:'A666:42598:&#x5729
CJK UNIFIED IDEOGRAPH:'A667:42599:&#x5919
CJK UNIFIED IDEOGRAPH:'A668:42600:&#x591A
CJK UNIFIED IDEOGRAPH:'A669:42601:&#x5937
CJK UNIFIED IDEOGRAPH:'A66A:42602:&#x5938
CJK UNIFIED IDEOGRAPH:'A66B:42603:&#x5984
CJK UNIFIED IDEOGRAPH:'A66C:42604:&#x5978
CJK UNIFIED IDEOGRAPH:'A66D:42605:&#x5983
CJK UNIFIED IDEOGRAPH:'A66E:42606:&#x597D
CJK UNIFIED IDEOGRAPH:'A66F:42607:&#x5979
CJK UNIFIED IDEOGRAPH:'A670:42608:&#x5982
CJK UNIFIED IDEOGRAPH:'A671:42609:&#x5981
CJK UNIFIED IDEOGRAPH:'A672:42610:&#x5B57
CJK UNIFIED IDEOGRAPH:'A673:42611:&#x5B58
CJK UNIFIED IDEOGRAPH:'A674:42612:&#x5B87
CJK UNIFIED IDEOGRAPH:'A675:42613:&#x5B88
CJK UNIFIED IDEOGRAPH:'A676:42614:&#x5B85
CJK UNIFIED IDEOGRAPH:'A677:42615:&#x5B89
CJK UNIFIED IDEOGRAPH:'A678:42616:&#x5BFA
CJK UNIFIED IDEOGRAPH:'A679:42617:&#x5C16
CJK UNIFIED IDEOGRAPH:'A67A:42618:&#x5C79
CJK UNIFIED IDEOGRAPH:'A67B:42619:&#x5DDE
CJK UNIFIED IDEOGRAPH:'A67C:42620:&#x5E06
CJK UNIFIED IDEOGRAPH:'A67D:42621:&#x5E76
CJK UNIFIED IDEOGRAPH:'A67E:42622:&#x5E74
CJK UNIFIED IDEOGRAPH:'A6A1:42657:&#x5F0F
CJK UNIFIED IDEOGRAPH:'A6A2:42658:&#x5F1B
CJK UNIFIED IDEOGRAPH:'A6A3:42659:&#x5FD9
CJK UNIFIED IDEOGRAPH:'A6A4:42660:&#x5FD6
CJK UNIFIED IDEOGRAPH:'A6A5:42661:&#x620E
CJK UNIFIED IDEOGRAPH:'A6A6:42662:&#x620C
CJK UNIFIED IDEOGRAPH:'A6A7:42663:&#x620D
CJK UNIFIED IDEOGRAPH:'A6A8:42664:&#x6210
CJK UNIFIED IDEOGRAPH:'A6A9:42665:&#x6263
CJK UNIFIED IDEOGRAPH:'A6AA:42666:&#x625B
CJK UNIFIED IDEOGRAPH:'A6AB:42667:&#x6258
CJK UNIFIED IDEOGRAPH:'A6AC:42668:&#x6536
CJK UNIFIED IDEOGRAPH:'A6AD:42669:&#x65E9
CJK UNIFIED IDEOGRAPH:'A6AE:42670:&#x65E8
CJK UNIFIED IDEOGRAPH:'A6AF:42671:&#x65EC
CJK UNIFIED IDEOGRAPH:'A6B0:42672:&#x65ED
CJK UNIFIED IDEOGRAPH:'A6B1:42673:&#x66F2
CJK UNIFIED IDEOGRAPH:'A6B2:42674:&#x66F3
CJK UNIFIED IDEOGRAPH:'A6B3:42675:&#x6709
CJK UNIFIED IDEOGRAPH:'A6B4:42676:&#x673D
CJK UNIFIED IDEOGRAPH:'A6B5:42677:&#x6734
CJK UNIFIED IDEOGRAPH:'A6B6:42678:&#x6731
CJK UNIFIED IDEOGRAPH:'A6B7:42679:&#x6735
CJK UNIFIED IDEOGRAPH:'A6B8:42680:&#x6B21
CJK UNIFIED IDEOGRAPH:'A6B9:42681:&#x6B64
CJK UNIFIED IDEOGRAPH:'A6BA:42682:&#x6B7B
CJK UNIFIED IDEOGRAPH:'A6BB:42683:&#x6C16
CJK UNIFIED IDEOGRAPH:'A6BC:42684:&#x6C5D
CJK UNIFIED IDEOGRAPH:'A6BD:42685:&#x6C57
CJK UNIFIED IDEOGRAPH:'A6BE:42686:&#x6C59
CJK UNIFIED IDEOGRAPH:'A6BF:42687:&#x6C5F
CJK UNIFIED IDEOGRAPH:'A6C0:42688:&#x6C60
CJK UNIFIED IDEOGRAPH:'A6C1:42689:&#x6C50
CJK UNIFIED IDEOGRAPH:'A6C2:42690:&#x6C55
CJK UNIFIED IDEOGRAPH:'A6C3:42691:&#x6C61
CJK UNIFIED IDEOGRAPH:'A6C4:42692:&#x6C5B
CJK UNIFIED IDEOGRAPH:'A6C5:42693:&#x6C4D
CJK UNIFIED IDEOGRAPH:'A6C6:42694:&#x6C4E
CJK UNIFIED IDEOGRAPH:'A6C7:42695:&#x7070
CJK UNIFIED IDEOGRAPH:'A6C8:42696:&#x725F
CJK UNIFIED IDEOGRAPH:'A6C9:42697:&#x725D
CJK UNIFIED IDEOGRAPH:'A6CA:42698:&#x767E
CJK UNIFIED IDEOGRAPH:'A6CB:42699:&#x7AF9
CJK UNIFIED IDEOGRAPH:'A6CC:42700:&#x7C73
CJK UNIFIED IDEOGRAPH:'A6CD:42701:&#x7CF8
CJK UNIFIED IDEOGRAPH:'A6CE:42702:&#x7F36
CJK UNIFIED IDEOGRAPH:'A6CF:42703:&#x7F8A
CJK UNIFIED IDEOGRAPH:'A6D0:42704:&#x7FBD
CJK UNIFIED IDEOGRAPH:'A6D1:42705:&#x8001
CJK UNIFIED IDEOGRAPH:'A6D2:42706:&#x8003
CJK UNIFIED IDEOGRAPH:'A6D3:42707:&#x800C
CJK UNIFIED IDEOGRAPH:'A6D4:42708:&#x8012
CJK UNIFIED IDEOGRAPH:'A6D5:42709:&#x8033
CJK UNIFIED IDEOGRAPH:'A6D6:42710:&#x807F
CJK UNIFIED IDEOGRAPH:'A6D7:42711:&#x8089
CJK UNIFIED IDEOGRAPH:'A6D8:42712:&#x808B
CJK UNIFIED IDEOGRAPH:'A6D9:42713:&#x808C
CJK UNIFIED IDEOGRAPH:'A6DA:42714:&#x81E3
CJK UNIFIED IDEOGRAPH:'A6DB:42715:&#x81EA
CJK UNIFIED IDEOGRAPH:'A6DC:42716:&#x81F3
CJK UNIFIED IDEOGRAPH:'A6DD:42717:&#x81FC
CJK UNIFIED IDEOGRAPH:'A6DE:42718:&#x820C
CJK UNIFIED IDEOGRAPH:'A6DF:42719:&#x821B
CJK UNIFIED IDEOGRAPH:'A6E0:42720:&#x821F
CJK UNIFIED IDEOGRAPH:'A6E1:42721:&#x826E
CJK UNIFIED IDEOGRAPH:'A6E2:42722:&#x8272
CJK UNIFIED IDEOGRAPH:'A6E3:42723:&#x827E
CJK UNIFIED IDEOGRAPH:'A6E4:42724:&#x866B
CJK UNIFIED IDEOGRAPH:'A6E5:42725:&#x8840
CJK UNIFIED IDEOGRAPH:'A6E6:42726:&#x884C
CJK UNIFIED IDEOGRAPH:'A6E7:42727:&#x8863
CJK UNIFIED IDEOGRAPH:'A6E8:42728:&#x897F
CJK UNIFIED IDEOGRAPH:'A6E9:42729:&#x9621
CJK UNIFIED IDEOGRAPH:'A6EA:42730:&#x4E32
CJK UNIFIED IDEOGRAPH:'A6EB:42731:&#x4EA8
CJK UNIFIED IDEOGRAPH:'A6EC:42732:&#x4F4D
CJK UNIFIED IDEOGRAPH:'A6ED:42733:&#x4F4F
CJK UNIFIED IDEOGRAPH:'A6EE:42734:&#x4F47
CJK UNIFIED IDEOGRAPH:'A6EF:42735:&#x4F57
CJK UNIFIED IDEOGRAPH:'A6F0:42736:&#x4F5E
CJK UNIFIED IDEOGRAPH:'A6F1:42737:&#x4F34
CJK UNIFIED IDEOGRAPH:'A6F2:42738:&#x4F5B
CJK UNIFIED IDEOGRAPH:'A6F3:42739:&#x4F55
CJK UNIFIED IDEOGRAPH:'A6F4:42740:&#x4F30
CJK UNIFIED IDEOGRAPH:'A6F5:42741:&#x4F50
CJK UNIFIED IDEOGRAPH:'A6F6:42742:&#x4F51
CJK UNIFIED IDEOGRAPH:'A6F7:42743:&#x4F3D
CJK UNIFIED IDEOGRAPH:'A6F8:42744:&#x4F3A
CJK UNIFIED IDEOGRAPH:'A6F9:42745:&#x4F38
CJK UNIFIED IDEOGRAPH:'A6FA:42746:&#x4F43
CJK UNIFIED IDEOGRAPH:'A6FB:42747:&#x4F54
CJK UNIFIED IDEOGRAPH:'A6FC:42748:&#x4F3C
CJK UNIFIED IDEOGRAPH:'A6FD:42749:&#x4F46
CJK UNIFIED IDEOGRAPH:'A6FE:42750:&#x4F63
CJK UNIFIED IDEOGRAPH:'A740:42816:&#x4F5C
CJK UNIFIED IDEOGRAPH:'A741:42817:&#x4F60
CJK UNIFIED IDEOGRAPH:'A742:42818:&#x4F2F
CJK UNIFIED IDEOGRAPH:'A743:42819:&#x4F4E
CJK UNIFIED IDEOGRAPH:'A744:42820:&#x4F36
CJK UNIFIED IDEOGRAPH:'A745:42821:&#x4F59
CJK UNIFIED IDEOGRAPH:'A746:42822:&#x4F5D
CJK UNIFIED IDEOGRAPH:'A747:42823:&#x4F48
CJK UNIFIED IDEOGRAPH:'A748:42824:&#x4F5A
CJK UNIFIED IDEOGRAPH:'A749:42825:&#x514C
CJK UNIFIED IDEOGRAPH:'A74A:42826:&#x514B
CJK UNIFIED IDEOGRAPH:'A74B:42827:&#x514D
CJK UNIFIED IDEOGRAPH:'A74C:42828:&#x5175
CJK UNIFIED IDEOGRAPH:'A74D:42829:&#x51B6
CJK UNIFIED IDEOGRAPH:'A74E:42830:&#x51B7
CJK UNIFIED IDEOGRAPH:'A74F:42831:&#x5225
CJK UNIFIED IDEOGRAPH:'A750:42832:&#x5224
CJK UNIFIED IDEOGRAPH:'A751:42833:&#x5229
CJK UNIFIED IDEOGRAPH:'A752:42834:&#x522A
CJK UNIFIED IDEOGRAPH:'A753:42835:&#x5228
CJK UNIFIED IDEOGRAPH:'A754:42836:&#x52AB
CJK UNIFIED IDEOGRAPH:'A755:42837:&#x52A9
CJK UNIFIED IDEOGRAPH:'A756:42838:&#x52AA
CJK UNIFIED IDEOGRAPH:'A757:42839:&#x52AC
CJK UNIFIED IDEOGRAPH:'A758:42840:&#x5323
CJK UNIFIED IDEOGRAPH:'A759:42841:&#x5373
CJK UNIFIED IDEOGRAPH:'A75A:42842:&#x5375
CJK UNIFIED IDEOGRAPH:'A75B:42843:&#x541D
CJK UNIFIED IDEOGRAPH:'A75C:42844:&#x542D
CJK UNIFIED IDEOGRAPH:'A75D:42845:&#x541E
CJK UNIFIED IDEOGRAPH:'A75E:42846:&#x543E
CJK UNIFIED IDEOGRAPH:'A75F:42847:&#x5426
CJK UNIFIED IDEOGRAPH:'A760:42848:&#x544E
CJK UNIFIED IDEOGRAPH:'A761:42849:&#x5427
CJK UNIFIED IDEOGRAPH:'A762:42850:&#x5446
CJK UNIFIED IDEOGRAPH:'A763:42851:&#x5443
CJK UNIFIED IDEOGRAPH:'A764:42852:&#x5433
CJK UNIFIED IDEOGRAPH:'A765:42853:&#x5448
CJK UNIFIED IDEOGRAPH:'A766:42854:&#x5442
CJK UNIFIED IDEOGRAPH:'A767:42855:&#x541B
CJK UNIFIED IDEOGRAPH:'A768:42856:&#x5429
CJK UNIFIED IDEOGRAPH:'A769:42857:&#x544A
CJK UNIFIED IDEOGRAPH:'A76A:42858:&#x5439
CJK UNIFIED IDEOGRAPH:'A76B:42859:&#x543B
CJK UNIFIED IDEOGRAPH:'A76C:42860:&#x5438
CJK UNIFIED IDEOGRAPH:'A76D:42861:&#x542E
CJK UNIFIED IDEOGRAPH:'A76E:42862:&#x5435
CJK UNIFIED IDEOGRAPH:'A76F:42863:&#x5436
CJK UNIFIED IDEOGRAPH:'A770:42864:&#x5420
CJK UNIFIED IDEOGRAPH:'A771:42865:&#x543C
CJK UNIFIED IDEOGRAPH:'A772:42866:&#x5440
CJK UNIFIED IDEOGRAPH:'A773:42867:&#x5431
CJK UNIFIED IDEOGRAPH:'A774:42868:&#x542B
CJK UNIFIED IDEOGRAPH:'A775:42869:&#x541F
CJK UNIFIED IDEOGRAPH:'A776:42870:&#x542C
CJK UNIFIED IDEOGRAPH:'A777:42871:&#x56EA
CJK UNIFIED IDEOGRAPH:'A778:42872:&#x56F0
CJK UNIFIED IDEOGRAPH:'A779:42873:&#x56E4
CJK UNIFIED IDEOGRAPH:'A77A:42874:&#x56EB
CJK UNIFIED IDEOGRAPH:'A77B:42875:&#x574A
CJK UNIFIED IDEOGRAPH:'A77C:42876:&#x5751
CJK UNIFIED IDEOGRAPH:'A77D:42877:&#x5740
CJK UNIFIED IDEOGRAPH:'A77E:42878:&#x574D
CJK UNIFIED IDEOGRAPH:'A7A1:42913:&#x5747
CJK UNIFIED IDEOGRAPH:'A7A2:42914:&#x574E
CJK UNIFIED IDEOGRAPH:'A7A3:42915:&#x573E
CJK UNIFIED IDEOGRAPH:'A7A4:42916:&#x5750
CJK UNIFIED IDEOGRAPH:'A7A5:42917:&#x574F
CJK UNIFIED IDEOGRAPH:'A7A6:42918:&#x573B
CJK UNIFIED IDEOGRAPH:'A7A7:42919:&#x58EF
CJK UNIFIED IDEOGRAPH:'A7A8:42920:&#x593E
CJK UNIFIED IDEOGRAPH:'A7A9:42921:&#x599D
CJK UNIFIED IDEOGRAPH:'A7AA:42922:&#x5992
CJK UNIFIED IDEOGRAPH:'A7AB:42923:&#x59A8
CJK UNIFIED IDEOGRAPH:'A7AC:42924:&#x599E
CJK UNIFIED IDEOGRAPH:'A7AD:42925:&#x59A3
CJK UNIFIED IDEOGRAPH:'A7AE:42926:&#x5999
CJK UNIFIED IDEOGRAPH:'A7AF:42927:&#x5996
CJK UNIFIED IDEOGRAPH:'A7B0:42928:&#x598D
CJK UNIFIED IDEOGRAPH:'A7B1:42929:&#x59A4
CJK UNIFIED IDEOGRAPH:'A7B2:42930:&#x5993
CJK UNIFIED IDEOGRAPH:'A7B3:42931:&#x598A
CJK UNIFIED IDEOGRAPH:'A7B4:42932:&#x59A5
CJK UNIFIED IDEOGRAPH:'A7B5:42933:&#x5B5D
CJK UNIFIED IDEOGRAPH:'A7B6:42934:&#x5B5C
CJK UNIFIED IDEOGRAPH:'A7B7:42935:&#x5B5A
CJK UNIFIED IDEOGRAPH:'A7B8:42936:&#x5B5B
CJK UNIFIED IDEOGRAPH:'A7B9:42937:&#x5B8C
CJK UNIFIED IDEOGRAPH:'A7BA:42938:&#x5B8B
CJK UNIFIED IDEOGRAPH:'A7BB:42939:&#x5B8F
CJK UNIFIED IDEOGRAPH:'A7BC:42940:&#x5C2C
CJK UNIFIED IDEOGRAPH:'A7BD:42941:&#x5C40
CJK UNIFIED IDEOGRAPH:'A7BE:42942:&#x5C41
CJK UNIFIED IDEOGRAPH:'A7BF:42943:&#x5C3F
CJK UNIFIED IDEOGRAPH:'A7C0:42944:&#x5C3E
CJK UNIFIED IDEOGRAPH:'A7C1:42945:&#x5C90
CJK UNIFIED IDEOGRAPH:'A7C2:42946:&#x5C91
CJK UNIFIED IDEOGRAPH:'A7C3:42947:&#x5C94
CJK UNIFIED IDEOGRAPH:'A7C4:42948:&#x5C8C
CJK UNIFIED IDEOGRAPH:'A7C5:42949:&#x5DEB
CJK UNIFIED IDEOGRAPH:'A7C6:42950:&#x5E0C
CJK UNIFIED IDEOGRAPH:'A7C7:42951:&#x5E8F
CJK UNIFIED IDEOGRAPH:'A7C8:42952:&#x5E87
CJK UNIFIED IDEOGRAPH:'A7C9:42953:&#x5E8A
CJK UNIFIED IDEOGRAPH:'A7CA:42954:&#x5EF7
CJK UNIFIED IDEOGRAPH:'A7CB:42955:&#x5F04
CJK UNIFIED IDEOGRAPH:'A7CC:42956:&#x5F1F
CJK UNIFIED IDEOGRAPH:'A7CD:42957:&#x5F64
CJK UNIFIED IDEOGRAPH:'A7CE:42958:&#x5F62
CJK UNIFIED IDEOGRAPH:'A7CF:42959:&#x5F77
CJK UNIFIED IDEOGRAPH:'A7D0:42960:&#x5F79
CJK UNIFIED IDEOGRAPH:'A7D1:42961:&#x5FD8
CJK UNIFIED IDEOGRAPH:'A7D2:42962:&#x5FCC
CJK UNIFIED IDEOGRAPH:'A7D3:42963:&#x5FD7
CJK UNIFIED IDEOGRAPH:'A7D4:42964:&#x5FCD
CJK UNIFIED IDEOGRAPH:'A7D5:42965:&#x5FF1
CJK UNIFIED IDEOGRAPH:'A7D6:42966:&#x5FEB
CJK UNIFIED IDEOGRAPH:'A7D7:42967:&#x5FF8
CJK UNIFIED IDEOGRAPH:'A7D8:42968:&#x5FEA
CJK UNIFIED IDEOGRAPH:'A7D9:42969:&#x6212
CJK UNIFIED IDEOGRAPH:'A7DA:42970:&#x6211
CJK UNIFIED IDEOGRAPH:'A7DB:42971:&#x6284
CJK UNIFIED IDEOGRAPH:'A7DC:42972:&#x6297
CJK UNIFIED IDEOGRAPH:'A7DD:42973:&#x6296
CJK UNIFIED IDEOGRAPH:'A7DE:42974:&#x6280
CJK UNIFIED IDEOGRAPH:'A7DF:42975:&#x6276
CJK UNIFIED IDEOGRAPH:'A7E0:42976:&#x6289
CJK UNIFIED IDEOGRAPH:'A7E1:42977:&#x626D
CJK UNIFIED IDEOGRAPH:'A7E2:42978:&#x628A
CJK UNIFIED IDEOGRAPH:'A7E3:42979:&#x627C
CJK UNIFIED IDEOGRAPH:'A7E4:42980:&#x627E
CJK UNIFIED IDEOGRAPH:'A7E5:42981:&#x6279
CJK UNIFIED IDEOGRAPH:'A7E6:42982:&#x6273
CJK UNIFIED IDEOGRAPH:'A7E7:42983:&#x6292
CJK UNIFIED IDEOGRAPH:'A7E8:42984:&#x626F
CJK UNIFIED IDEOGRAPH:'A7E9:42985:&#x6298
CJK UNIFIED IDEOGRAPH:'A7EA:42986:&#x626E
CJK UNIFIED IDEOGRAPH:'A7EB:42987:&#x6295
CJK UNIFIED IDEOGRAPH:'A7EC:42988:&#x6293
CJK UNIFIED IDEOGRAPH:'A7ED:42989:&#x6291
CJK UNIFIED IDEOGRAPH:'A7EE:42990:&#x6286
CJK UNIFIED IDEOGRAPH:'A7EF:42991:&#x6539
CJK UNIFIED IDEOGRAPH:'A7F0:42992:&#x653B
CJK UNIFIED IDEOGRAPH:'A7F1:42993:&#x6538
CJK UNIFIED IDEOGRAPH:'A7F2:42994:&#x65F1
CJK UNIFIED IDEOGRAPH:'A7F3:42995:&#x66F4
CJK UNIFIED IDEOGRAPH:'A7F4:42996:&#x675F
CJK UNIFIED IDEOGRAPH:'A7F5:42997:&#x674E
CJK UNIFIED IDEOGRAPH:'A7F6:42998:&#x674F
CJK UNIFIED IDEOGRAPH:'A7F7:42999:&#x6750
CJK UNIFIED IDEOGRAPH:'A7F8:43000:&#x6751
CJK UNIFIED IDEOGRAPH:'A7F9:43001:&#x675C
CJK UNIFIED IDEOGRAPH:'A7FA:43002:&#x6756
CJK UNIFIED IDEOGRAPH:'A7FB:43003:&#x675E
CJK UNIFIED IDEOGRAPH:'A7FC:43004:&#x6749
CJK UNIFIED IDEOGRAPH:'A7FD:43005:&#x6746
CJK UNIFIED IDEOGRAPH:'A7FE:43006:&#x6760
CJK UNIFIED IDEOGRAPH:'A840:43072:&#x6753
CJK UNIFIED IDEOGRAPH:'A841:43073:&#x6757
CJK UNIFIED IDEOGRAPH:'A842:43074:&#x6B65
CJK UNIFIED IDEOGRAPH:'A843:43075:&#x6BCF
CJK UNIFIED IDEOGRAPH:'A844:43076:&#x6C42
CJK UNIFIED IDEOGRAPH:'A845:43077:&#x6C5E
CJK UNIFIED IDEOGRAPH:'A846:43078:&#x6C99
CJK UNIFIED IDEOGRAPH:'A847:43079:&#x6C81
CJK UNIFIED IDEOGRAPH:'A848:43080:&#x6C88
CJK UNIFIED IDEOGRAPH:'A849:43081:&#x6C89
CJK UNIFIED IDEOGRAPH:'A84A:43082:&#x6C85
CJK UNIFIED IDEOGRAPH:'A84B:43083:&#x6C9B
CJK UNIFIED IDEOGRAPH:'A84C:43084:&#x6C6A
CJK UNIFIED IDEOGRAPH:'A84D:43085:&#x6C7A
CJK UNIFIED IDEOGRAPH:'A84E:43086:&#x6C90
CJK UNIFIED IDEOGRAPH:'A84F:43087:&#x6C70
CJK UNIFIED IDEOGRAPH:'A850:43088:&#x6C8C
CJK UNIFIED IDEOGRAPH:'A851:43089:&#x6C68
CJK UNIFIED IDEOGRAPH:'A852:43090:&#x6C96
CJK UNIFIED IDEOGRAPH:'A853:43091:&#x6C92
CJK UNIFIED IDEOGRAPH:'A854:43092:&#x6C7D
CJK UNIFIED IDEOGRAPH:'A855:43093:&#x6C83
CJK UNIFIED IDEOGRAPH:'A856:43094:&#x6C72
CJK UNIFIED IDEOGRAPH:'A857:43095:&#x6C7E
CJK UNIFIED IDEOGRAPH:'A858:43096:&#x6C74
CJK UNIFIED IDEOGRAPH:'A859:43097:&#x6C86
CJK UNIFIED IDEOGRAPH:'A85A:43098:&#x6C76
CJK UNIFIED IDEOGRAPH:'A85B:43099:&#x6C8D
CJK UNIFIED IDEOGRAPH:'A85C:43100:&#x6C94
CJK UNIFIED IDEOGRAPH:'A85D:43101:&#x6C98
CJK UNIFIED IDEOGRAPH:'A85E:43102:&#x6C82
CJK UNIFIED IDEOGRAPH:'A85F:43103:&#x7076
CJK UNIFIED IDEOGRAPH:'A860:43104:&#x707C
CJK UNIFIED IDEOGRAPH:'A861:43105:&#x707D
CJK UNIFIED IDEOGRAPH:'A862:43106:&#x7078
CJK UNIFIED IDEOGRAPH:'A863:43107:&#x7262
CJK UNIFIED IDEOGRAPH:'A864:43108:&#x7261
CJK UNIFIED IDEOGRAPH:'A865:43109:&#x7260
CJK UNIFIED IDEOGRAPH:'A866:43110:&#x72C4
CJK UNIFIED IDEOGRAPH:'A867:43111:&#x72C2
CJK UNIFIED IDEOGRAPH:'A868:43112:&#x7396
CJK UNIFIED IDEOGRAPH:'A869:43113:&#x752C
CJK UNIFIED IDEOGRAPH:'A86A:43114:&#x752B
CJK UNIFIED IDEOGRAPH:'A86B:43115:&#x7537
CJK UNIFIED IDEOGRAPH:'A86C:43116:&#x7538
CJK UNIFIED IDEOGRAPH:'A86D:43117:&#x7682
CJK UNIFIED IDEOGRAPH:'A86E:43118:&#x76EF
CJK UNIFIED IDEOGRAPH:'A86F:43119:&#x77E3
CJK UNIFIED IDEOGRAPH:'A870:43120:&#x79C1
CJK UNIFIED IDEOGRAPH:'A871:43121:&#x79C0
CJK UNIFIED IDEOGRAPH:'A872:43122:&#x79BF
CJK UNIFIED IDEOGRAPH:'A873:43123:&#x7A76
CJK UNIFIED IDEOGRAPH:'A874:43124:&#x7CFB
CJK UNIFIED IDEOGRAPH:'A875:43125:&#x7F55
CJK UNIFIED IDEOGRAPH:'A876:43126:&#x8096
CJK UNIFIED IDEOGRAPH:'A877:43127:&#x8093
CJK UNIFIED IDEOGRAPH:'A878:43128:&#x809D
CJK UNIFIED IDEOGRAPH:'A879:43129:&#x8098
CJK UNIFIED IDEOGRAPH:'A87A:43130:&#x809B
CJK UNIFIED IDEOGRAPH:'A87B:43131:&#x809A
CJK UNIFIED IDEOGRAPH:'A87C:43132:&#x80B2
CJK UNIFIED IDEOGRAPH:'A87D:43133:&#x826F
CJK UNIFIED IDEOGRAPH:'A87E:43134:&#x8292
CJK UNIFIED IDEOGRAPH:'A8A1:43169:&#x828B
CJK UNIFIED IDEOGRAPH:'A8A2:43170:&#x828D
CJK UNIFIED IDEOGRAPH:'A8A3:43171:&#x898B
CJK UNIFIED IDEOGRAPH:'A8A4:43172:&#x89D2
CJK UNIFIED IDEOGRAPH:'A8A5:43173:&#x8A00
CJK UNIFIED IDEOGRAPH:'A8A6:43174:&#x8C37
CJK UNIFIED IDEOGRAPH:'A8A7:43175:&#x8C46
CJK UNIFIED IDEOGRAPH:'A8A8:43176:&#x8C55
CJK UNIFIED IDEOGRAPH:'A8A9:43177:&#x8C9D
CJK UNIFIED IDEOGRAPH:'A8AA:43178:&#x8D64
CJK UNIFIED IDEOGRAPH:'A8AB:43179:&#x8D70
CJK UNIFIED IDEOGRAPH:'A8AC:43180:&#x8DB3
CJK UNIFIED IDEOGRAPH:'A8AD:43181:&#x8EAB
CJK UNIFIED IDEOGRAPH:'A8AE:43182:&#x8ECA
CJK UNIFIED IDEOGRAPH:'A8AF:43183:&#x8F9B
CJK UNIFIED IDEOGRAPH:'A8B0:43184:&#x8FB0
CJK UNIFIED IDEOGRAPH:'A8B1:43185:&#x8FC2
CJK UNIFIED IDEOGRAPH:'A8B2:43186:&#x8FC6
CJK UNIFIED IDEOGRAPH:'A8B3:43187:&#x8FC5
CJK UNIFIED IDEOGRAPH:'A8B4:43188:&#x8FC4
CJK UNIFIED IDEOGRAPH:'A8B5:43189:&#x5DE1
CJK UNIFIED IDEOGRAPH:'A8B6:43190:&#x9091
CJK UNIFIED IDEOGRAPH:'A8B7:43191:&#x90A2
CJK UNIFIED IDEOGRAPH:'A8B8:43192:&#x90AA
CJK UNIFIED IDEOGRAPH:'A8B9:43193:&#x90A6
CJK UNIFIED IDEOGRAPH:'A8BA:43194:&#x90A3
CJK UNIFIED IDEOGRAPH:'A8BB:43195:&#x9149
CJK UNIFIED IDEOGRAPH:'A8BC:43196:&#x91C6
CJK UNIFIED IDEOGRAPH:'A8BD:43197:&#x91CC
CJK UNIFIED IDEOGRAPH:'A8BE:43198:&#x9632
CJK UNIFIED IDEOGRAPH:'A8BF:43199:&#x962E
CJK UNIFIED IDEOGRAPH:'A8C0:43200:&#x9631
CJK UNIFIED IDEOGRAPH:'A8C1:43201:&#x962A
CJK UNIFIED IDEOGRAPH:'A8C2:43202:&#x962C
CJK UNIFIED IDEOGRAPH:'A8C3:43203:&#x4E26
CJK UNIFIED IDEOGRAPH:'A8C4:43204:&#x4E56
CJK UNIFIED IDEOGRAPH:'A8C5:43205:&#x4E73
CJK UNIFIED IDEOGRAPH:'A8C6:43206:&#x4E8B
CJK UNIFIED IDEOGRAPH:'A8C7:43207:&#x4E9B
CJK UNIFIED IDEOGRAPH:'A8C8:43208:&#x4E9E
CJK UNIFIED IDEOGRAPH:'A8C9:43209:&#x4EAB
CJK UNIFIED IDEOGRAPH:'A8CA:43210:&#x4EAC
CJK UNIFIED IDEOGRAPH:'A8CB:43211:&#x4F6F
CJK UNIFIED IDEOGRAPH:'A8CC:43212:&#x4F9D
CJK UNIFIED IDEOGRAPH:'A8CD:43213:&#x4F8D
CJK UNIFIED IDEOGRAPH:'A8CE:43214:&#x4F73
CJK UNIFIED IDEOGRAPH:'A8CF:43215:&#x4F7F
CJK UNIFIED IDEOGRAPH:'A8D0:43216:&#x4F6C
CJK UNIFIED IDEOGRAPH:'A8D1:43217:&#x4F9B
CJK UNIFIED IDEOGRAPH:'A8D2:43218:&#x4F8B
CJK UNIFIED IDEOGRAPH:'A8D3:43219:&#x4F86
CJK UNIFIED IDEOGRAPH:'A8D4:43220:&#x4F83
CJK UNIFIED IDEOGRAPH:'A8D5:43221:&#x4F70
CJK UNIFIED IDEOGRAPH:'A8D6:43222:&#x4F75
CJK UNIFIED IDEOGRAPH:'A8D7:43223:&#x4F88
CJK UNIFIED IDEOGRAPH:'A8D8:43224:&#x4F69
CJK UNIFIED IDEOGRAPH:'A8D9:43225:&#x4F7B
CJK UNIFIED IDEOGRAPH:'A8DA:43226:&#x4F96
CJK UNIFIED IDEOGRAPH:'A8DB:43227:&#x4F7E
CJK UNIFIED IDEOGRAPH:'A8DC:43228:&#x4F8F
CJK UNIFIED IDEOGRAPH:'A8DD:43229:&#x4F91
CJK UNIFIED IDEOGRAPH:'A8DE:43230:&#x4F7A
CJK UNIFIED IDEOGRAPH:'A8DF:43231:&#x5154
CJK UNIFIED IDEOGRAPH:'A8E0:43232:&#x5152
CJK UNIFIED IDEOGRAPH:'A8E1:43233:&#x5155
CJK UNIFIED IDEOGRAPH:'A8E2:43234:&#x5169
CJK UNIFIED IDEOGRAPH:'A8E3:43235:&#x5177
CJK UNIFIED IDEOGRAPH:'A8E4:43236:&#x5176
CJK UNIFIED IDEOGRAPH:'A8E5:43237:&#x5178
CJK UNIFIED IDEOGRAPH:'A8E6:43238:&#x51BD
CJK UNIFIED IDEOGRAPH:'A8E7:43239:&#x51FD
CJK UNIFIED IDEOGRAPH:'A8E8:43240:&#x523B
CJK UNIFIED IDEOGRAPH:'A8E9:43241:&#x5238
CJK UNIFIED IDEOGRAPH:'A8EA:43242:&#x5237
CJK UNIFIED IDEOGRAPH:'A8EB:43243:&#x523A
CJK UNIFIED IDEOGRAPH:'A8EC:43244:&#x5230
CJK UNIFIED IDEOGRAPH:'A8ED:43245:&#x522E
CJK UNIFIED IDEOGRAPH:'A8EE:43246:&#x5236
CJK UNIFIED IDEOGRAPH:'A8EF:43247:&#x5241
CJK UNIFIED IDEOGRAPH:'A8F0:43248:&#x52BE
CJK UNIFIED IDEOGRAPH:'A8F1:43249:&#x52BB
CJK UNIFIED IDEOGRAPH:'A8F2:43250:&#x5352
CJK UNIFIED IDEOGRAPH:'A8F3:43251:&#x5354
CJK UNIFIED IDEOGRAPH:'A8F4:43252:&#x5353
CJK UNIFIED IDEOGRAPH:'A8F5:43253:&#x5351
CJK UNIFIED IDEOGRAPH:'A8F6:43254:&#x5366
CJK UNIFIED IDEOGRAPH:'A8F7:43255:&#x5377
CJK UNIFIED IDEOGRAPH:'A8F8:43256:&#x5378
CJK UNIFIED IDEOGRAPH:'A8F9:43257:&#x5379
CJK UNIFIED IDEOGRAPH:'A8FA:43258:&#x53D6
CJK UNIFIED IDEOGRAPH:'A8FB:43259:&#x53D4
CJK UNIFIED IDEOGRAPH:'A8FC:43260:&#x53D7
CJK UNIFIED IDEOGRAPH:'A8FD:43261:&#x5473
CJK UNIFIED IDEOGRAPH:'A8FE:43262:&#x5475
CJK UNIFIED IDEOGRAPH:'A940:43328:&#x5496
CJK UNIFIED IDEOGRAPH:'A941:43329:&#x5478
CJK UNIFIED IDEOGRAPH:'A942:43330:&#x5495
CJK UNIFIED IDEOGRAPH:'A943:43331:&#x5480
CJK UNIFIED IDEOGRAPH:'A944:43332:&#x547B
CJK UNIFIED IDEOGRAPH:'A945:43333:&#x5477
CJK UNIFIED IDEOGRAPH:'A946:43334:&#x5484
CJK UNIFIED IDEOGRAPH:'A947:43335:&#x5492
CJK UNIFIED IDEOGRAPH:'A948:43336:&#x5486
CJK UNIFIED IDEOGRAPH:'A949:43337:&#x547C
CJK UNIFIED IDEOGRAPH:'A94A:43338:&#x5490
CJK UNIFIED IDEOGRAPH:'A94B:43339:&#x5471
CJK UNIFIED IDEOGRAPH:'A94C:43340:&#x5476
CJK UNIFIED IDEOGRAPH:'A94D:43341:&#x548C
CJK UNIFIED IDEOGRAPH:'A94E:43342:&#x549A
CJK UNIFIED IDEOGRAPH:'A94F:43343:&#x5462
CJK UNIFIED IDEOGRAPH:'A950:43344:&#x5468
CJK UNIFIED IDEOGRAPH:'A951:43345:&#x548B
CJK UNIFIED IDEOGRAPH:'A952:43346:&#x547D
CJK UNIFIED IDEOGRAPH:'A953:43347:&#x548E
CJK UNIFIED IDEOGRAPH:'A954:43348:&#x56FA
CJK UNIFIED IDEOGRAPH:'A955:43349:&#x5783
CJK UNIFIED IDEOGRAPH:'A956:43350:&#x5777
CJK UNIFIED IDEOGRAPH:'A957:43351:&#x576A
CJK UNIFIED IDEOGRAPH:'A958:43352:&#x5769
CJK UNIFIED IDEOGRAPH:'A959:43353:&#x5761
CJK UNIFIED IDEOGRAPH:'A95A:43354:&#x5766
CJK UNIFIED IDEOGRAPH:'A95B:43355:&#x5764
CJK UNIFIED IDEOGRAPH:'A95C:43356:&#x577C
CJK UNIFIED IDEOGRAPH:'A95D:43357:&#x591C
CJK UNIFIED IDEOGRAPH:'A95E:43358:&#x5949
CJK UNIFIED IDEOGRAPH:'A95F:43359:&#x5947
CJK UNIFIED IDEOGRAPH:'A960:43360:&#x5948
CJK UNIFIED IDEOGRAPH:'A961:43361:&#x5944
CJK UNIFIED IDEOGRAPH:'A962:43362:&#x5954
CJK UNIFIED IDEOGRAPH:'A963:43363:&#x59BE
CJK UNIFIED IDEOGRAPH:'A964:43364:&#x59BB
CJK UNIFIED IDEOGRAPH:'A965:43365:&#x59D4
CJK UNIFIED IDEOGRAPH:'A966:43366:&#x59B9
CJK UNIFIED IDEOGRAPH:'A967:43367:&#x59AE
CJK UNIFIED IDEOGRAPH:'A968:43368:&#x59D1
CJK UNIFIED IDEOGRAPH:'A969:43369:&#x59C6
CJK UNIFIED IDEOGRAPH:'A96A:43370:&#x59D0
CJK UNIFIED IDEOGRAPH:'A96B:43371:&#x59CD
CJK UNIFIED IDEOGRAPH:'A96C:43372:&#x59CB
CJK UNIFIED IDEOGRAPH:'A96D:43373:&#x59D3
CJK UNIFIED IDEOGRAPH:'A96E:43374:&#x59CA
CJK UNIFIED IDEOGRAPH:'A96F:43375:&#x59AF
CJK UNIFIED IDEOGRAPH:'A970:43376:&#x59B3
CJK UNIFIED IDEOGRAPH:'A971:43377:&#x59D2
CJK UNIFIED IDEOGRAPH:'A972:43378:&#x59C5
CJK UNIFIED IDEOGRAPH:'A973:43379:&#x5B5F
CJK UNIFIED IDEOGRAPH:'A974:43380:&#x5B64
CJK UNIFIED IDEOGRAPH:'A975:43381:&#x5B63
CJK UNIFIED IDEOGRAPH:'A976:43382:&#x5B97
CJK UNIFIED IDEOGRAPH:'A977:43383:&#x5B9A
CJK UNIFIED IDEOGRAPH:'A978:43384:&#x5B98
CJK UNIFIED IDEOGRAPH:'A979:43385:&#x5B9C
CJK UNIFIED IDEOGRAPH:'A97A:43386:&#x5B99
CJK UNIFIED IDEOGRAPH:'A97B:43387:&#x5B9B
CJK UNIFIED IDEOGRAPH:'A97C:43388:&#x5C1A
CJK UNIFIED IDEOGRAPH:'A97D:43389:&#x5C48
CJK UNIFIED IDEOGRAPH:'A97E:43390:&#x5C45
CJK UNIFIED IDEOGRAPH:'A9A1:43425:&#x5C46
CJK UNIFIED IDEOGRAPH:'A9A2:43426:&#x5CB7
CJK UNIFIED IDEOGRAPH:'A9A3:43427:&#x5CA1
CJK UNIFIED IDEOGRAPH:'A9A4:43428:&#x5CB8
CJK UNIFIED IDEOGRAPH:'A9A5:43429:&#x5CA9
CJK UNIFIED IDEOGRAPH:'A9A6:43430:&#x5CAB
CJK UNIFIED IDEOGRAPH:'A9A7:43431:&#x5CB1
CJK UNIFIED IDEOGRAPH:'A9A8:43432:&#x5CB3
CJK UNIFIED IDEOGRAPH:'A9A9:43433:&#x5E18
CJK UNIFIED IDEOGRAPH:'A9AA:43434:&#x5E1A
CJK UNIFIED IDEOGRAPH:'A9AB:43435:&#x5E16
CJK UNIFIED IDEOGRAPH:'A9AC:43436:&#x5E15
CJK UNIFIED IDEOGRAPH:'A9AD:43437:&#x5E1B
CJK UNIFIED IDEOGRAPH:'A9AE:43438:&#x5E11
CJK UNIFIED IDEOGRAPH:'A9AF:43439:&#x5E78
CJK UNIFIED IDEOGRAPH:'A9B0:43440:&#x5E9A
CJK UNIFIED IDEOGRAPH:'A9B1:43441:&#x5E97
CJK UNIFIED IDEOGRAPH:'A9B2:43442:&#x5E9C
CJK UNIFIED IDEOGRAPH:'A9B3:43443:&#x5E95
CJK UNIFIED IDEOGRAPH:'A9B4:43444:&#x5E96
CJK UNIFIED IDEOGRAPH:'A9B5:43445:&#x5EF6
CJK UNIFIED IDEOGRAPH:'A9B6:43446:&#x5F26
CJK UNIFIED IDEOGRAPH:'A9B7:43447:&#x5F27
CJK UNIFIED IDEOGRAPH:'A9B8:43448:&#x5F29
CJK UNIFIED IDEOGRAPH:'A9B9:43449:&#x5F80
CJK UNIFIED IDEOGRAPH:'A9BA:43450:&#x5F81
CJK UNIFIED IDEOGRAPH:'A9BB:43451:&#x5F7F
CJK UNIFIED IDEOGRAPH:'A9BC:43452:&#x5F7C
CJK UNIFIED IDEOGRAPH:'A9BD:43453:&#x5FDD
CJK UNIFIED IDEOGRAPH:'A9BE:43454:&#x5FE0
CJK UNIFIED IDEOGRAPH:'A9BF:43455:&#x5FFD
CJK UNIFIED IDEOGRAPH:'A9C0:43456:&#x5FF5
CJK UNIFIED IDEOGRAPH:'A9C1:43457:&#x5FFF
CJK UNIFIED IDEOGRAPH:'A9C2:43458:&#x600F
CJK UNIFIED IDEOGRAPH:'A9C3:43459:&#x6014
CJK UNIFIED IDEOGRAPH:'A9C4:43460:&#x602F
CJK UNIFIED IDEOGRAPH:'A9C5:43461:&#x6035
CJK UNIFIED IDEOGRAPH:'A9C6:43462:&#x6016
CJK UNIFIED IDEOGRAPH:'A9C7:43463:&#x602A
CJK UNIFIED IDEOGRAPH:'A9C8:43464:&#x6015
CJK UNIFIED IDEOGRAPH:'A9C9:43465:&#x6021
CJK UNIFIED IDEOGRAPH:'A9CA:43466:&#x6027
CJK UNIFIED IDEOGRAPH:'A9CB:43467:&#x6029
CJK UNIFIED IDEOGRAPH:'A9CC:43468:&#x602B
CJK UNIFIED IDEOGRAPH:'A9CD:43469:&#x601B
CJK UNIFIED IDEOGRAPH:'A9CE:43470:&#x6216
CJK UNIFIED IDEOGRAPH:'A9CF:43471:&#x6215
CJK UNIFIED IDEOGRAPH:'A9D0:43472:&#x623F
CJK UNIFIED IDEOGRAPH:'A9D1:43473:&#x623E
CJK UNIFIED IDEOGRAPH:'A9D2:43474:&#x6240
CJK UNIFIED IDEOGRAPH:'A9D3:43475:&#x627F
CJK UNIFIED IDEOGRAPH:'A9D4:43476:&#x62C9
CJK UNIFIED IDEOGRAPH:'A9D5:43477:&#x62CC
CJK UNIFIED IDEOGRAPH:'A9D6:43478:&#x62C4
CJK UNIFIED IDEOGRAPH:'A9D7:43479:&#x62BF
CJK UNIFIED IDEOGRAPH:'A9D8:43480:&#x62C2
CJK UNIFIED IDEOGRAPH:'A9D9:43481:&#x62B9
CJK UNIFIED IDEOGRAPH:'A9DA:43482:&#x62D2
CJK UNIFIED IDEOGRAPH:'A9DB:43483:&#x62DB
CJK UNIFIED IDEOGRAPH:'A9DC:43484:&#x62AB
CJK UNIFIED IDEOGRAPH:'A9DD:43485:&#x62D3
CJK UNIFIED IDEOGRAPH:'A9DE:43486:&#x62D4
CJK UNIFIED IDEOGRAPH:'A9DF:43487:&#x62CB
CJK UNIFIED IDEOGRAPH:'A9E0:43488:&#x62C8
CJK UNIFIED IDEOGRAPH:'A9E1:43489:&#x62A8
CJK UNIFIED IDEOGRAPH:'A9E2:43490:&#x62BD
CJK UNIFIED IDEOGRAPH:'A9E3:43491:&#x62BC
CJK UNIFIED IDEOGRAPH:'A9E4:43492:&#x62D0
CJK UNIFIED IDEOGRAPH:'A9E5:43493:&#x62D9
CJK UNIFIED IDEOGRAPH:'A9E6:43494:&#x62C7
CJK UNIFIED IDEOGRAPH:'A9E7:43495:&#x62CD
CJK UNIFIED IDEOGRAPH:'A9E8:43496:&#x62B5
CJK UNIFIED IDEOGRAPH:'A9E9:43497:&#x62DA
CJK UNIFIED IDEOGRAPH:'A9EA:43498:&#x62B1
CJK UNIFIED IDEOGRAPH:'A9EB:43499:&#x62D8
CJK UNIFIED IDEOGRAPH:'A9EC:43500:&#x62D6
CJK UNIFIED IDEOGRAPH:'A9ED:43501:&#x62D7
CJK UNIFIED IDEOGRAPH:'A9EE:43502:&#x62C6
CJK UNIFIED IDEOGRAPH:'A9EF:43503:&#x62AC
CJK UNIFIED IDEOGRAPH:'A9F0:43504:&#x62CE
CJK UNIFIED IDEOGRAPH:'A9F1:43505:&#x653E
CJK UNIFIED IDEOGRAPH:'A9F2:43506:&#x65A7
CJK UNIFIED IDEOGRAPH:'A9F3:43507:&#x65BC
CJK UNIFIED IDEOGRAPH:'A9F4:43508:&#x65FA
CJK UNIFIED IDEOGRAPH:'A9F5:43509:&#x6614
CJK UNIFIED IDEOGRAPH:'A9F6:43510:&#x6613
CJK UNIFIED IDEOGRAPH:'A9F7:43511:&#x660C
CJK UNIFIED IDEOGRAPH:'A9F8:43512:&#x6606
CJK UNIFIED IDEOGRAPH:'A9F9:43513:&#x6602
CJK UNIFIED IDEOGRAPH:'A9FA:43514:&#x660E
CJK UNIFIED IDEOGRAPH:'A9FB:43515:&#x6600
CJK UNIFIED IDEOGRAPH:'A9FC:43516:&#x660F
CJK UNIFIED IDEOGRAPH:'A9FD:43517:&#x6615
CJK UNIFIED IDEOGRAPH:'A9FE:43518:&#x660A
CJK UNIFIED IDEOGRAPH:'AA40:43584:&#x6607
CJK UNIFIED IDEOGRAPH:'AA41:43585:&#x670D
CJK UNIFIED IDEOGRAPH:'AA42:43586:&#x670B
CJK UNIFIED IDEOGRAPH:'AA43:43587:&#x676D
CJK UNIFIED IDEOGRAPH:'AA44:43588:&#x678B
CJK UNIFIED IDEOGRAPH:'AA45:43589:&#x6795
CJK UNIFIED IDEOGRAPH:'AA46:43590:&#x6771
CJK UNIFIED IDEOGRAPH:'AA47:43591:&#x679C
CJK UNIFIED IDEOGRAPH:'AA48:43592:&#x6773
CJK UNIFIED IDEOGRAPH:'AA49:43593:&#x6777
CJK UNIFIED IDEOGRAPH:'AA4A:43594:&#x6787
CJK UNIFIED IDEOGRAPH:'AA4B:43595:&#x679D
CJK UNIFIED IDEOGRAPH:'AA4C:43596:&#x6797
CJK UNIFIED IDEOGRAPH:'AA4D:43597:&#x676F
CJK UNIFIED IDEOGRAPH:'AA4E:43598:&#x6770
CJK UNIFIED IDEOGRAPH:'AA4F:43599:&#x677F
CJK UNIFIED IDEOGRAPH:'AA50:43600:&#x6789
CJK UNIFIED IDEOGRAPH:'AA51:43601:&#x677E
CJK UNIFIED IDEOGRAPH:'AA52:43602:&#x6790
CJK UNIFIED IDEOGRAPH:'AA53:43603:&#x6775
CJK UNIFIED IDEOGRAPH:'AA54:43604:&#x679A
CJK UNIFIED IDEOGRAPH:'AA55:43605:&#x6793
CJK UNIFIED IDEOGRAPH:'AA56:43606:&#x677C
CJK UNIFIED IDEOGRAPH:'AA57:43607:&#x676A
CJK UNIFIED IDEOGRAPH:'AA58:43608:&#x6772
CJK UNIFIED IDEOGRAPH:'AA59:43609:&#x6B23
CJK UNIFIED IDEOGRAPH:'AA5A:43610:&#x6B66
CJK UNIFIED IDEOGRAPH:'AA5B:43611:&#x6B67
CJK UNIFIED IDEOGRAPH:'AA5C:43612:&#x6B7F
CJK UNIFIED IDEOGRAPH:'AA5D:43613:&#x6C13
CJK UNIFIED IDEOGRAPH:'AA5E:43614:&#x6C1B
CJK UNIFIED IDEOGRAPH:'AA5F:43615:&#x6CE3
CJK UNIFIED IDEOGRAPH:'AA60:43616:&#x6CE8
CJK UNIFIED IDEOGRAPH:'AA61:43617:&#x6CF3
CJK UNIFIED IDEOGRAPH:'AA62:43618:&#x6CB1
CJK UNIFIED IDEOGRAPH:'AA63:43619:&#x6CCC
CJK UNIFIED IDEOGRAPH:'AA64:43620:&#x6CE5
CJK UNIFIED IDEOGRAPH:'AA65:43621:&#x6CB3
CJK UNIFIED IDEOGRAPH:'AA66:43622:&#x6CBD
CJK UNIFIED IDEOGRAPH:'AA67:43623:&#x6CBE
CJK UNIFIED IDEOGRAPH:'AA68:43624:&#x6CBC
CJK UNIFIED IDEOGRAPH:'AA69:43625:&#x6CE2
CJK UNIFIED IDEOGRAPH:'AA6A:43626:&#x6CAB
CJK UNIFIED IDEOGRAPH:'AA6B:43627:&#x6CD5
CJK UNIFIED IDEOGRAPH:'AA6C:43628:&#x6CD3
CJK UNIFIED IDEOGRAPH:'AA6D:43629:&#x6CB8
CJK UNIFIED IDEOGRAPH:'AA6E:43630:&#x6CC4
CJK UNIFIED IDEOGRAPH:'AA6F:43631:&#x6CB9
CJK UNIFIED IDEOGRAPH:'AA70:43632:&#x6CC1
CJK UNIFIED IDEOGRAPH:'AA71:43633:&#x6CAE
CJK UNIFIED IDEOGRAPH:'AA72:43634:&#x6CD7
CJK UNIFIED IDEOGRAPH:'AA73:43635:&#x6CC5
CJK UNIFIED IDEOGRAPH:'AA74:43636:&#x6CF1
CJK UNIFIED IDEOGRAPH:'AA75:43637:&#x6CBF
CJK UNIFIED IDEOGRAPH:'AA76:43638:&#x6CBB
CJK UNIFIED IDEOGRAPH:'AA77:43639:&#x6CE1
CJK UNIFIED IDEOGRAPH:'AA78:43640:&#x6CDB
CJK UNIFIED IDEOGRAPH:'AA79:43641:&#x6CCA
CJK UNIFIED IDEOGRAPH:'AA7A:43642:&#x6CAC
CJK UNIFIED IDEOGRAPH:'AA7B:43643:&#x6CEF
CJK UNIFIED IDEOGRAPH:'AA7C:43644:&#x6CDC
CJK UNIFIED IDEOGRAPH:'AA7D:43645:&#x6CD6
CJK UNIFIED IDEOGRAPH:'AA7E:43646:&#x6CE0
CJK UNIFIED IDEOGRAPH:'AAA1:43681:&#x7095
CJK UNIFIED IDEOGRAPH:'AAA2:43682:&#x708E
CJK UNIFIED IDEOGRAPH:'AAA3:43683:&#x7092
CJK UNIFIED IDEOGRAPH:'AAA4:43684:&#x708A
CJK UNIFIED IDEOGRAPH:'AAA5:43685:&#x7099
CJK UNIFIED IDEOGRAPH:'AAA6:43686:&#x722C
CJK UNIFIED IDEOGRAPH:'AAA7:43687:&#x722D
CJK UNIFIED IDEOGRAPH:'AAA8:43688:&#x7238
CJK UNIFIED IDEOGRAPH:'AAA9:43689:&#x7248
CJK UNIFIED IDEOGRAPH:'AAAA:43690:&#x7267
CJK UNIFIED IDEOGRAPH:'AAAB:43691:&#x7269
CJK UNIFIED IDEOGRAPH:'AAAC:43692:&#x72C0
CJK UNIFIED IDEOGRAPH:'AAAD:43693:&#x72CE
CJK UNIFIED IDEOGRAPH:'AAAE:43694:&#x72D9
CJK UNIFIED IDEOGRAPH:'AAAF:43695:&#x72D7
CJK UNIFIED IDEOGRAPH:'AAB0:43696:&#x72D0
CJK UNIFIED IDEOGRAPH:'AAB1:43697:&#x73A9
CJK UNIFIED IDEOGRAPH:'AAB2:43698:&#x73A8
CJK UNIFIED IDEOGRAPH:'AAB3:43699:&#x739F
CJK UNIFIED IDEOGRAPH:'AAB4:43700:&#x73AB
CJK UNIFIED IDEOGRAPH:'AAB5:43701:&#x73A5
CJK UNIFIED IDEOGRAPH:'AAB6:43702:&#x753D
CJK UNIFIED IDEOGRAPH:'AAB7:43703:&#x759D
CJK UNIFIED IDEOGRAPH:'AAB8:43704:&#x7599
CJK UNIFIED IDEOGRAPH:'AAB9:43705:&#x759A
CJK UNIFIED IDEOGRAPH:'AABA:43706:&#x7684
CJK UNIFIED IDEOGRAPH:'AABB:43707:&#x76C2
CJK UNIFIED IDEOGRAPH:'AABC:43708:&#x76F2
CJK UNIFIED IDEOGRAPH:'AABD:43709:&#x76F4
CJK UNIFIED IDEOGRAPH:'AABE:43710:&#x77E5
CJK UNIFIED IDEOGRAPH:'AABF:43711:&#x77FD
CJK UNIFIED IDEOGRAPH:'AAC0:43712:&#x793E
CJK UNIFIED IDEOGRAPH:'AAC1:43713:&#x7940
CJK UNIFIED IDEOGRAPH:'AAC2:43714:&#x7941
CJK UNIFIED IDEOGRAPH:'AAC3:43715:&#x79C9
CJK UNIFIED IDEOGRAPH:'AAC4:43716:&#x79C8
CJK UNIFIED IDEOGRAPH:'AAC5:43717:&#x7A7A
CJK UNIFIED IDEOGRAPH:'AAC6:43718:&#x7A79
CJK UNIFIED IDEOGRAPH:'AAC7:43719:&#x7AFA
CJK UNIFIED IDEOGRAPH:'AAC8:43720:&#x7CFE
CJK UNIFIED IDEOGRAPH:'AAC9:43721:&#x7F54
CJK UNIFIED IDEOGRAPH:'AACA:43722:&#x7F8C
CJK UNIFIED IDEOGRAPH:'AACB:43723:&#x7F8B
CJK UNIFIED IDEOGRAPH:'AACC:43724:&#x8005
CJK UNIFIED IDEOGRAPH:'AACD:43725:&#x80BA
CJK UNIFIED IDEOGRAPH:'AACE:43726:&#x80A5
CJK UNIFIED IDEOGRAPH:'AACF:43727:&#x80A2
CJK UNIFIED IDEOGRAPH:'AAD0:43728:&#x80B1
CJK UNIFIED IDEOGRAPH:'AAD1:43729:&#x80A1
CJK UNIFIED IDEOGRAPH:'AAD2:43730:&#x80AB
CJK UNIFIED IDEOGRAPH:'AAD3:43731:&#x80A9
CJK UNIFIED IDEOGRAPH:'AAD4:43732:&#x80B4
CJK UNIFIED IDEOGRAPH:'AAD5:43733:&#x80AA
CJK UNIFIED IDEOGRAPH:'AAD6:43734:&#x80AF
CJK UNIFIED IDEOGRAPH:'AAD7:43735:&#x81E5
CJK UNIFIED IDEOGRAPH:'AAD8:43736:&#x81FE
CJK UNIFIED IDEOGRAPH:'AAD9:43737:&#x820D
CJK UNIFIED IDEOGRAPH:'AADA:43738:&#x82B3
CJK UNIFIED IDEOGRAPH:'AADB:43739:&#x829D
CJK UNIFIED IDEOGRAPH:'AADC:43740:&#x8299
CJK UNIFIED IDEOGRAPH:'AADD:43741:&#x82AD
CJK UNIFIED IDEOGRAPH:'AADE:43742:&#x82BD
CJK UNIFIED IDEOGRAPH:'AADF:43743:&#x829F
CJK UNIFIED IDEOGRAPH:'AAE0:43744:&#x82B9
CJK UNIFIED IDEOGRAPH:'AAE1:43745:&#x82B1
CJK UNIFIED IDEOGRAPH:'AAE2:43746:&#x82AC
CJK UNIFIED IDEOGRAPH:'AAE3:43747:&#x82A5
CJK UNIFIED IDEOGRAPH:'AAE4:43748:&#x82AF
CJK UNIFIED IDEOGRAPH:'AAE5:43749:&#x82B8
CJK UNIFIED IDEOGRAPH:'AAE6:43750:&#x82A3
CJK UNIFIED IDEOGRAPH:'AAE7:43751:&#x82B0
CJK UNIFIED IDEOGRAPH:'AAE8:43752:&#x82BE
CJK UNIFIED IDEOGRAPH:'AAE9:43753:&#x82B7
CJK UNIFIED IDEOGRAPH:'AAEA:43754:&#x864E
CJK UNIFIED IDEOGRAPH:'AAEB:43755:&#x8671
CJK UNIFIED IDEOGRAPH:'AAEC:43756:&#x521D
CJK UNIFIED IDEOGRAPH:'AAED:43757:&#x8868
CJK UNIFIED IDEOGRAPH:'AAEE:43758:&#x8ECB
CJK UNIFIED IDEOGRAPH:'AAEF:43759:&#x8FCE
CJK UNIFIED IDEOGRAPH:'AAF0:43760:&#x8FD4
CJK UNIFIED IDEOGRAPH:'AAF1:43761:&#x8FD1
CJK UNIFIED IDEOGRAPH:'AAF2:43762:&#x90B5
CJK UNIFIED IDEOGRAPH:'AAF3:43763:&#x90B8
CJK UNIFIED IDEOGRAPH:'AAF4:43764:&#x90B1
CJK UNIFIED IDEOGRAPH:'AAF5:43765:&#x90B6
CJK UNIFIED IDEOGRAPH:'AAF6:43766:&#x91C7
CJK UNIFIED IDEOGRAPH:'AAF7:43767:&#x91D1
CJK UNIFIED IDEOGRAPH:'AAF8:43768:&#x9577
CJK UNIFIED IDEOGRAPH:'AAF9:43769:&#x9580
CJK UNIFIED IDEOGRAPH:'AAFA:43770:&#x961C
CJK UNIFIED IDEOGRAPH:'AAFB:43771:&#x9640
CJK UNIFIED IDEOGRAPH:'AAFC:43772:&#x963F
CJK UNIFIED IDEOGRAPH:'AAFD:43773:&#x963B
CJK UNIFIED IDEOGRAPH:'AAFE:43774:&#x9644
CJK UNIFIED IDEOGRAPH:'AB40:43840:&#x9642
CJK UNIFIED IDEOGRAPH:'AB41:43841:&#x96B9
CJK UNIFIED IDEOGRAPH:'AB42:43842:&#x96E8
CJK UNIFIED IDEOGRAPH:'AB43:43843:&#x9752
CJK UNIFIED IDEOGRAPH:'AB44:43844:&#x975E
CJK UNIFIED IDEOGRAPH:'AB45:43845:&#x4E9F
CJK UNIFIED IDEOGRAPH:'AB46:43846:&#x4EAD
CJK UNIFIED IDEOGRAPH:'AB47:43847:&#x4EAE
CJK UNIFIED IDEOGRAPH:'AB48:43848:&#x4FE1
CJK UNIFIED IDEOGRAPH:'AB49:43849:&#x4FB5
CJK UNIFIED IDEOGRAPH:'AB4A:43850:&#x4FAF
CJK UNIFIED IDEOGRAPH:'AB4B:43851:&#x4FBF
CJK UNIFIED IDEOGRAPH:'AB4C:43852:&#x4FE0
CJK UNIFIED IDEOGRAPH:'AB4D:43853:&#x4FD1
CJK UNIFIED IDEOGRAPH:'AB4E:43854:&#x4FCF
CJK UNIFIED IDEOGRAPH:'AB4F:43855:&#x4FDD
CJK UNIFIED IDEOGRAPH:'AB50:43856:&#x4FC3
CJK UNIFIED IDEOGRAPH:'AB51:43857:&#x4FB6
CJK UNIFIED IDEOGRAPH:'AB52:43858:&#x4FD8
CJK UNIFIED IDEOGRAPH:'AB53:43859:&#x4FDF
CJK UNIFIED IDEOGRAPH:'AB54:43860:&#x4FCA
CJK UNIFIED IDEOGRAPH:'AB55:43861:&#x4FD7
CJK UNIFIED IDEOGRAPH:'AB56:43862:&#x4FAE
CJK UNIFIED IDEOGRAPH:'AB57:43863:&#x4FD0
CJK UNIFIED IDEOGRAPH:'AB58:43864:&#x4FC4
CJK UNIFIED IDEOGRAPH:'AB59:43865:&#x4FC2
CJK UNIFIED IDEOGRAPH:'AB5A:43866:&#x4FDA
CJK UNIFIED IDEOGRAPH:'AB5B:43867:&#x4FCE
CJK UNIFIED IDEOGRAPH:'AB5C:43868:&#x4FDE
CJK UNIFIED IDEOGRAPH:'AB5D:43869:&#x4FB7
CJK UNIFIED IDEOGRAPH:'AB5E:43870:&#x5157
CJK UNIFIED IDEOGRAPH:'AB5F:43871:&#x5192
CJK UNIFIED IDEOGRAPH:'AB60:43872:&#x5191
CJK UNIFIED IDEOGRAPH:'AB61:43873:&#x51A0
CJK UNIFIED IDEOGRAPH:'AB62:43874:&#x524E
CJK UNIFIED IDEOGRAPH:'AB63:43875:&#x5243
CJK UNIFIED IDEOGRAPH:'AB64:43876:&#x524A
CJK UNIFIED IDEOGRAPH:'AB65:43877:&#x524D
CJK UNIFIED IDEOGRAPH:'AB66:43878:&#x524C
CJK UNIFIED IDEOGRAPH:'AB67:43879:&#x524B
CJK UNIFIED IDEOGRAPH:'AB68:43880:&#x5247
CJK UNIFIED IDEOGRAPH:'AB69:43881:&#x52C7
CJK UNIFIED IDEOGRAPH:'AB6A:43882:&#x52C9
CJK UNIFIED IDEOGRAPH:'AB6B:43883:&#x52C3
CJK UNIFIED IDEOGRAPH:'AB6C:43884:&#x52C1
CJK UNIFIED IDEOGRAPH:'AB6D:43885:&#x530D
CJK UNIFIED IDEOGRAPH:'AB6E:43886:&#x5357
CJK UNIFIED IDEOGRAPH:'AB6F:43887:&#x537B
CJK UNIFIED IDEOGRAPH:'AB70:43888:&#x539A
CJK UNIFIED IDEOGRAPH:'AB71:43889:&#x53DB
CJK UNIFIED IDEOGRAPH:'AB72:43890:&#x54AC
CJK UNIFIED IDEOGRAPH:'AB73:43891:&#x54C0
CJK UNIFIED IDEOGRAPH:'AB74:43892:&#x54A8
CJK UNIFIED IDEOGRAPH:'AB75:43893:&#x54CE
CJK UNIFIED IDEOGRAPH:'AB76:43894:&#x54C9
CJK UNIFIED IDEOGRAPH:'AB77:43895:&#x54B8
CJK UNIFIED IDEOGRAPH:'AB78:43896:&#x54A6
CJK UNIFIED IDEOGRAPH:'AB79:43897:&#x54B3
CJK UNIFIED IDEOGRAPH:'AB7A:43898:&#x54C7
CJK UNIFIED IDEOGRAPH:'AB7B:43899:&#x54C2
CJK UNIFIED IDEOGRAPH:'AB7C:43900:&#x54BD
CJK UNIFIED IDEOGRAPH:'AB7D:43901:&#x54AA
CJK UNIFIED IDEOGRAPH:'AB7E:43902:&#x54C1
CJK UNIFIED IDEOGRAPH:'ABA1:43937:&#x54C4
CJK UNIFIED IDEOGRAPH:'ABA2:43938:&#x54C8
CJK UNIFIED IDEOGRAPH:'ABA3:43939:&#x54AF
CJK UNIFIED IDEOGRAPH:'ABA4:43940:&#x54AB
CJK UNIFIED IDEOGRAPH:'ABA5:43941:&#x54B1
CJK UNIFIED IDEOGRAPH:'ABA6:43942:&#x54BB
CJK UNIFIED IDEOGRAPH:'ABA7:43943:&#x54A9
CJK UNIFIED IDEOGRAPH:'ABA8:43944:&#x54A7
CJK UNIFIED IDEOGRAPH:'ABA9:43945:&#x54BF
CJK UNIFIED IDEOGRAPH:'ABAA:43946:&#x56FF
CJK UNIFIED IDEOGRAPH:'ABAB:43947:&#x5782
CJK UNIFIED IDEOGRAPH:'ABAC:43948:&#x578B
CJK UNIFIED IDEOGRAPH:'ABAD:43949:&#x57A0
CJK UNIFIED IDEOGRAPH:'ABAE:43950:&#x57A3
CJK UNIFIED IDEOGRAPH:'ABAF:43951:&#x57A2
CJK UNIFIED IDEOGRAPH:'ABB0:43952:&#x57CE
CJK UNIFIED IDEOGRAPH:'ABB1:43953:&#x57AE
CJK UNIFIED IDEOGRAPH:'ABB2:43954:&#x5793
CJK UNIFIED IDEOGRAPH:'ABB3:43955:&#x5955
CJK UNIFIED IDEOGRAPH:'ABB4:43956:&#x5951
CJK UNIFIED IDEOGRAPH:'ABB5:43957:&#x594F
CJK UNIFIED IDEOGRAPH:'ABB6:43958:&#x594E
CJK UNIFIED IDEOGRAPH:'ABB7:43959:&#x5950
CJK UNIFIED IDEOGRAPH:'ABB8:43960:&#x59DC
CJK UNIFIED IDEOGRAPH:'ABB9:43961:&#x59D8
CJK UNIFIED IDEOGRAPH:'ABBA:43962:&#x59FF
CJK UNIFIED IDEOGRAPH:'ABBB:43963:&#x59E3
CJK UNIFIED IDEOGRAPH:'ABBC:43964:&#x59E8
CJK UNIFIED IDEOGRAPH:'ABBD:43965:&#x5A03
CJK UNIFIED IDEOGRAPH:'ABBE:43966:&#x59E5
CJK UNIFIED IDEOGRAPH:'ABBF:43967:&#x59EA
CJK UNIFIED IDEOGRAPH:'ABC0:43968:&#x59DA
CJK UNIFIED IDEOGRAPH:'ABC1:43969:&#x59E6
CJK UNIFIED IDEOGRAPH:'ABC2:43970:&#x5A01
CJK UNIFIED IDEOGRAPH:'ABC3:43971:&#x59FB
CJK UNIFIED IDEOGRAPH:'ABC4:43972:&#x5B69
CJK UNIFIED IDEOGRAPH:'ABC5:43973:&#x5BA3
CJK UNIFIED IDEOGRAPH:'ABC6:43974:&#x5BA6
CJK UNIFIED IDEOGRAPH:'ABC7:43975:&#x5BA4
CJK UNIFIED IDEOGRAPH:'ABC8:43976:&#x5BA2
CJK UNIFIED IDEOGRAPH:'ABC9:43977:&#x5BA5
CJK UNIFIED IDEOGRAPH:'ABCA:43978:&#x5C01
CJK UNIFIED IDEOGRAPH:'ABCB:43979:&#x5C4E
CJK UNIFIED IDEOGRAPH:'ABCC:43980:&#x5C4F
CJK UNIFIED IDEOGRAPH:'ABCD:43981:&#x5C4D
CJK UNIFIED IDEOGRAPH:'ABCE:43982:&#x5C4B
CJK UNIFIED IDEOGRAPH:'ABCF:43983:&#x5CD9
CJK UNIFIED IDEOGRAPH:'ABD0:43984:&#x5CD2
CJK UNIFIED IDEOGRAPH:'ABD1:43985:&#x5DF7
CJK UNIFIED IDEOGRAPH:'ABD2:43986:&#x5E1D
CJK UNIFIED IDEOGRAPH:'ABD3:43987:&#x5E25
CJK UNIFIED IDEOGRAPH:'ABD4:43988:&#x5E1F
CJK UNIFIED IDEOGRAPH:'ABD5:43989:&#x5E7D
CJK UNIFIED IDEOGRAPH:'ABD6:43990:&#x5EA0
CJK UNIFIED IDEOGRAPH:'ABD7:43991:&#x5EA6
CJK UNIFIED IDEOGRAPH:'ABD8:43992:&#x5EFA
CJK UNIFIED IDEOGRAPH:'ABD9:43993:&#x5F08
CJK UNIFIED IDEOGRAPH:'ABDA:43994:&#x5F2D
CJK UNIFIED IDEOGRAPH:'ABDB:43995:&#x5F65
CJK UNIFIED IDEOGRAPH:'ABDC:43996:&#x5F88
CJK UNIFIED IDEOGRAPH:'ABDD:43997:&#x5F85
CJK UNIFIED IDEOGRAPH:'ABDE:43998:&#x5F8A
CJK UNIFIED IDEOGRAPH:'ABDF:43999:&#x5F8B
CJK UNIFIED IDEOGRAPH:'ABE0:44000:&#x5F87
CJK UNIFIED IDEOGRAPH:'ABE1:44001:&#x5F8C
CJK UNIFIED IDEOGRAPH:'ABE2:44002:&#x5F89
CJK UNIFIED IDEOGRAPH:'ABE3:44003:&#x6012
CJK UNIFIED IDEOGRAPH:'ABE4:44004:&#x601D
CJK UNIFIED IDEOGRAPH:'ABE5:44005:&#x6020
CJK UNIFIED IDEOGRAPH:'ABE6:44006:&#x6025
CJK UNIFIED IDEOGRAPH:'ABE7:44007:&#x600E
CJK UNIFIED IDEOGRAPH:'ABE8:44008:&#x6028
CJK UNIFIED IDEOGRAPH:'ABE9:44009:&#x604D
CJK UNIFIED IDEOGRAPH:'ABEA:44010:&#x6070
CJK UNIFIED IDEOGRAPH:'ABEB:44011:&#x6068
CJK UNIFIED IDEOGRAPH:'ABEC:44012:&#x6062
CJK UNIFIED IDEOGRAPH:'ABED:44013:&#x6046
CJK UNIFIED IDEOGRAPH:'ABEE:44014:&#x6043
CJK UNIFIED IDEOGRAPH:'ABEF:44015:&#x606C
CJK UNIFIED IDEOGRAPH:'ABF0:44016:&#x606B
CJK UNIFIED IDEOGRAPH:'ABF1:44017:&#x606A
CJK UNIFIED IDEOGRAPH:'ABF2:44018:&#x6064
CJK UNIFIED IDEOGRAPH:'ABF3:44019:&#x6241
CJK UNIFIED IDEOGRAPH:'ABF4:44020:&#x62DC
CJK UNIFIED IDEOGRAPH:'ABF5:44021:&#x6316
CJK UNIFIED IDEOGRAPH:'ABF6:44022:&#x6309
CJK UNIFIED IDEOGRAPH:'ABF7:44023:&#x62FC
CJK UNIFIED IDEOGRAPH:'ABF8:44024:&#x62ED
CJK UNIFIED IDEOGRAPH:'ABF9:44025:&#x6301
CJK UNIFIED IDEOGRAPH:'ABFA:44026:&#x62EE
CJK UNIFIED IDEOGRAPH:'ABFB:44027:&#x62FD
CJK UNIFIED IDEOGRAPH:'ABFC:44028:&#x6307
CJK UNIFIED IDEOGRAPH:'ABFD:44029:&#x62F1
CJK UNIFIED IDEOGRAPH:'ABFE:44030:&#x62F7
CJK UNIFIED IDEOGRAPH:'AC40:44096:&#x62EF
CJK UNIFIED IDEOGRAPH:'AC41:44097:&#x62EC
CJK UNIFIED IDEOGRAPH:'AC42:44098:&#x62FE
CJK UNIFIED IDEOGRAPH:'AC43:44099:&#x62F4
CJK UNIFIED IDEOGRAPH:'AC44:44100:&#x6311
CJK UNIFIED IDEOGRAPH:'AC45:44101:&#x6302
CJK UNIFIED IDEOGRAPH:'AC46:44102:&#x653F
CJK UNIFIED IDEOGRAPH:'AC47:44103:&#x6545
CJK UNIFIED IDEOGRAPH:'AC48:44104:&#x65AB
CJK UNIFIED IDEOGRAPH:'AC49:44105:&#x65BD
CJK UNIFIED IDEOGRAPH:'AC4A:44106:&#x65E2
CJK UNIFIED IDEOGRAPH:'AC4B:44107:&#x6625
CJK UNIFIED IDEOGRAPH:'AC4C:44108:&#x662D
CJK UNIFIED IDEOGRAPH:'AC4D:44109:&#x6620
CJK UNIFIED IDEOGRAPH:'AC4E:44110:&#x6627
CJK UNIFIED IDEOGRAPH:'AC4F:44111:&#x662F
CJK UNIFIED IDEOGRAPH:'AC50:44112:&#x661F
CJK UNIFIED IDEOGRAPH:'AC51:44113:&#x6628
CJK UNIFIED IDEOGRAPH:'AC52:44114:&#x6631
CJK UNIFIED IDEOGRAPH:'AC53:44115:&#x6624
CJK UNIFIED IDEOGRAPH:'AC54:44116:&#x66F7
CJK UNIFIED IDEOGRAPH:'AC55:44117:&#x67FF
CJK UNIFIED IDEOGRAPH:'AC56:44118:&#x67D3
CJK UNIFIED IDEOGRAPH:'AC57:44119:&#x67F1
CJK UNIFIED IDEOGRAPH:'AC58:44120:&#x67D4
CJK UNIFIED IDEOGRAPH:'AC59:44121:&#x67D0
CJK UNIFIED IDEOGRAPH:'AC5A:44122:&#x67EC
CJK UNIFIED IDEOGRAPH:'AC5B:44123:&#x67B6
CJK UNIFIED IDEOGRAPH:'AC5C:44124:&#x67AF
CJK UNIFIED IDEOGRAPH:'AC5D:44125:&#x67F5
CJK UNIFIED IDEOGRAPH:'AC5E:44126:&#x67E9
CJK UNIFIED IDEOGRAPH:'AC5F:44127:&#x67EF
CJK UNIFIED IDEOGRAPH:'AC60:44128:&#x67C4
CJK UNIFIED IDEOGRAPH:'AC61:44129:&#x67D1
CJK UNIFIED IDEOGRAPH:'AC62:44130:&#x67B4
CJK UNIFIED IDEOGRAPH:'AC63:44131:&#x67DA
CJK UNIFIED IDEOGRAPH:'AC64:44132:&#x67E5
CJK UNIFIED IDEOGRAPH:'AC65:44133:&#x67B8
CJK UNIFIED IDEOGRAPH:'AC66:44134:&#x67CF
CJK UNIFIED IDEOGRAPH:'AC67:44135:&#x67DE
CJK UNIFIED IDEOGRAPH:'AC68:44136:&#x67F3
CJK UNIFIED IDEOGRAPH:'AC69:44137:&#x67B0
CJK UNIFIED IDEOGRAPH:'AC6A:44138:&#x67D9
CJK UNIFIED IDEOGRAPH:'AC6B:44139:&#x67E2
CJK UNIFIED IDEOGRAPH:'AC6C:44140:&#x67DD
CJK UNIFIED IDEOGRAPH:'AC6D:44141:&#x67D2
CJK UNIFIED IDEOGRAPH:'AC6E:44142:&#x6B6A
CJK UNIFIED IDEOGRAPH:'AC6F:44143:&#x6B83
CJK UNIFIED IDEOGRAPH:'AC70:44144:&#x6B86
CJK UNIFIED IDEOGRAPH:'AC71:44145:&#x6BB5
CJK UNIFIED IDEOGRAPH:'AC72:44146:&#x6BD2
CJK UNIFIED IDEOGRAPH:'AC73:44147:&#x6BD7
CJK UNIFIED IDEOGRAPH:'AC74:44148:&#x6C1F
CJK UNIFIED IDEOGRAPH:'AC75:44149:&#x6CC9
CJK UNIFIED IDEOGRAPH:'AC76:44150:&#x6D0B
CJK UNIFIED IDEOGRAPH:'AC77:44151:&#x6D32
CJK UNIFIED IDEOGRAPH:'AC78:44152:&#x6D2A
CJK UNIFIED IDEOGRAPH:'AC79:44153:&#x6D41
CJK UNIFIED IDEOGRAPH:'AC7A:44154:&#x6D25
CJK UNIFIED IDEOGRAPH:'AC7B:44155:&#x6D0C
CJK UNIFIED IDEOGRAPH:'AC7C:44156:&#x6D31
CJK UNIFIED IDEOGRAPH:'AC7D:44157:&#x6D1E
CJK UNIFIED IDEOGRAPH:'AC7E:44158:&#x6D17
CJK UNIFIED IDEOGRAPH:'ACA1:44193:&#x6D3B
CJK UNIFIED IDEOGRAPH:'ACA2:44194:&#x6D3D
CJK UNIFIED IDEOGRAPH:'ACA3:44195:&#x6D3E
CJK UNIFIED IDEOGRAPH:'ACA4:44196:&#x6D36
CJK UNIFIED IDEOGRAPH:'ACA5:44197:&#x6D1B
CJK UNIFIED IDEOGRAPH:'ACA6:44198:&#x6CF5
CJK UNIFIED IDEOGRAPH:'ACA7:44199:&#x6D39
CJK UNIFIED IDEOGRAPH:'ACA8:44200:&#x6D27
CJK UNIFIED IDEOGRAPH:'ACA9:44201:&#x6D38
CJK UNIFIED IDEOGRAPH:'ACAA:44202:&#x6D29
CJK UNIFIED IDEOGRAPH:'ACAB:44203:&#x6D2E
CJK UNIFIED IDEOGRAPH:'ACAC:44204:&#x6D35
CJK UNIFIED IDEOGRAPH:'ACAD:44205:&#x6D0E
CJK UNIFIED IDEOGRAPH:'ACAE:44206:&#x6D2B
CJK UNIFIED IDEOGRAPH:'ACAF:44207:&#x70AB
CJK UNIFIED IDEOGRAPH:'ACB0:44208:&#x70BA
CJK UNIFIED IDEOGRAPH:'ACB1:44209:&#x70B3
CJK UNIFIED IDEOGRAPH:'ACB2:44210:&#x70AC
CJK UNIFIED IDEOGRAPH:'ACB3:44211:&#x70AF
CJK UNIFIED IDEOGRAPH:'ACB4:44212:&#x70AD
CJK UNIFIED IDEOGRAPH:'ACB5:44213:&#x70B8
CJK UNIFIED IDEOGRAPH:'ACB6:44214:&#x70AE
CJK UNIFIED IDEOGRAPH:'ACB7:44215:&#x70A4
CJK UNIFIED IDEOGRAPH:'ACB8:44216:&#x7230
CJK UNIFIED IDEOGRAPH:'ACB9:44217:&#x7272
CJK UNIFIED IDEOGRAPH:'ACBA:44218:&#x726F
CJK UNIFIED IDEOGRAPH:'ACBB:44219:&#x7274
CJK UNIFIED IDEOGRAPH:'ACBC:44220:&#x72E9
CJK UNIFIED IDEOGRAPH:'ACBD:44221:&#x72E0
CJK UNIFIED IDEOGRAPH:'ACBE:44222:&#x72E1
CJK UNIFIED IDEOGRAPH:'ACBF:44223:&#x73B7
CJK UNIFIED IDEOGRAPH:'ACC0:44224:&#x73CA
CJK UNIFIED IDEOGRAPH:'ACC1:44225:&#x73BB
CJK UNIFIED IDEOGRAPH:'ACC2:44226:&#x73B2
CJK UNIFIED IDEOGRAPH:'ACC3:44227:&#x73CD
CJK UNIFIED IDEOGRAPH:'ACC4:44228:&#x73C0
CJK UNIFIED IDEOGRAPH:'ACC5:44229:&#x73B3
CJK UNIFIED IDEOGRAPH:'ACC6:44230:&#x751A
CJK UNIFIED IDEOGRAPH:'ACC7:44231:&#x752D
CJK UNIFIED IDEOGRAPH:'ACC8:44232:&#x754F
CJK UNIFIED IDEOGRAPH:'ACC9:44233:&#x754C
CJK UNIFIED IDEOGRAPH:'ACCA:44234:&#x754E
CJK UNIFIED IDEOGRAPH:'ACCB:44235:&#x754B
CJK UNIFIED IDEOGRAPH:'ACCC:44236:&#x75AB
CJK UNIFIED IDEOGRAPH:'ACCD:44237:&#x75A4
CJK UNIFIED IDEOGRAPH:'ACCE:44238:&#x75A5
CJK UNIFIED IDEOGRAPH:'ACCF:44239:&#x75A2
CJK UNIFIED IDEOGRAPH:'ACD0:44240:&#x75A3
CJK UNIFIED IDEOGRAPH:'ACD1:44241:&#x7678
CJK UNIFIED IDEOGRAPH:'ACD2:44242:&#x7686
CJK UNIFIED IDEOGRAPH:'ACD3:44243:&#x7687
CJK UNIFIED IDEOGRAPH:'ACD4:44244:&#x7688
CJK UNIFIED IDEOGRAPH:'ACD5:44245:&#x76C8
CJK UNIFIED IDEOGRAPH:'ACD6:44246:&#x76C6
CJK UNIFIED IDEOGRAPH:'ACD7:44247:&#x76C3
CJK UNIFIED IDEOGRAPH:'ACD8:44248:&#x76C5
CJK UNIFIED IDEOGRAPH:'ACD9:44249:&#x7701
CJK UNIFIED IDEOGRAPH:'ACDA:44250:&#x76F9
CJK UNIFIED IDEOGRAPH:'ACDB:44251:&#x76F8
CJK UNIFIED IDEOGRAPH:'ACDC:44252:&#x7709
CJK UNIFIED IDEOGRAPH:'ACDD:44253:&#x770B
CJK UNIFIED IDEOGRAPH:'ACDE:44254:&#x76FE
CJK UNIFIED IDEOGRAPH:'ACDF:44255:&#x76FC
CJK UNIFIED IDEOGRAPH:'ACE0:44256:&#x7707
CJK UNIFIED IDEOGRAPH:'ACE1:44257:&#x77DC
CJK UNIFIED IDEOGRAPH:'ACE2:44258:&#x7802
CJK UNIFIED IDEOGRAPH:'ACE3:44259:&#x7814
CJK UNIFIED IDEOGRAPH:'ACE4:44260:&#x780C
CJK UNIFIED IDEOGRAPH:'ACE5:44261:&#x780D
CJK UNIFIED IDEOGRAPH:'ACE6:44262:&#x7946
CJK UNIFIED IDEOGRAPH:'ACE7:44263:&#x7949
CJK UNIFIED IDEOGRAPH:'ACE8:44264:&#x7948
CJK UNIFIED IDEOGRAPH:'ACE9:44265:&#x7947
CJK UNIFIED IDEOGRAPH:'ACEA:44266:&#x79B9
CJK UNIFIED IDEOGRAPH:'ACEB:44267:&#x79BA
CJK UNIFIED IDEOGRAPH:'ACEC:44268:&#x79D1
CJK UNIFIED IDEOGRAPH:'ACED:44269:&#x79D2
CJK UNIFIED IDEOGRAPH:'ACEE:44270:&#x79CB
CJK UNIFIED IDEOGRAPH:'ACEF:44271:&#x7A7F
CJK UNIFIED IDEOGRAPH:'ACF0:44272:&#x7A81
CJK UNIFIED IDEOGRAPH:'ACF1:44273:&#x7AFF
CJK UNIFIED IDEOGRAPH:'ACF2:44274:&#x7AFD
CJK UNIFIED IDEOGRAPH:'ACF3:44275:&#x7C7D
CJK UNIFIED IDEOGRAPH:'ACF4:44276:&#x7D02
CJK UNIFIED IDEOGRAPH:'ACF5:44277:&#x7D05
CJK UNIFIED IDEOGRAPH:'ACF6:44278:&#x7D00
CJK UNIFIED IDEOGRAPH:'ACF7:44279:&#x7D09
CJK UNIFIED IDEOGRAPH:'ACF8:44280:&#x7D07
CJK UNIFIED IDEOGRAPH:'ACF9:44281:&#x7D04
CJK UNIFIED IDEOGRAPH:'ACFA:44282:&#x7D06
CJK UNIFIED IDEOGRAPH:'ACFB:44283:&#x7F38
CJK UNIFIED IDEOGRAPH:'ACFC:44284:&#x7F8E
CJK UNIFIED IDEOGRAPH:'ACFD:44285:&#x7FBF
CJK UNIFIED IDEOGRAPH:'ACFE:44286:&#x8004
CJK UNIFIED IDEOGRAPH:'AD40:44352:&#x8010
CJK UNIFIED IDEOGRAPH:'AD41:44353:&#x800D
CJK UNIFIED IDEOGRAPH:'AD42:44354:&#x8011
CJK UNIFIED IDEOGRAPH:'AD43:44355:&#x8036
CJK UNIFIED IDEOGRAPH:'AD44:44356:&#x80D6
CJK UNIFIED IDEOGRAPH:'AD45:44357:&#x80E5
CJK UNIFIED IDEOGRAPH:'AD46:44358:&#x80DA
CJK UNIFIED IDEOGRAPH:'AD47:44359:&#x80C3
CJK UNIFIED IDEOGRAPH:'AD48:44360:&#x80C4
CJK UNIFIED IDEOGRAPH:'AD49:44361:&#x80CC
CJK UNIFIED IDEOGRAPH:'AD4A:44362:&#x80E1
CJK UNIFIED IDEOGRAPH:'AD4B:44363:&#x80DB
CJK UNIFIED IDEOGRAPH:'AD4C:44364:&#x80CE
CJK UNIFIED IDEOGRAPH:'AD4D:44365:&#x80DE
CJK UNIFIED IDEOGRAPH:'AD4E:44366:&#x80E4
CJK UNIFIED IDEOGRAPH:'AD4F:44367:&#x80DD
CJK UNIFIED IDEOGRAPH:'AD50:44368:&#x81F4
CJK UNIFIED IDEOGRAPH:'AD51:44369:&#x8222
CJK UNIFIED IDEOGRAPH:'AD52:44370:&#x82E7
CJK UNIFIED IDEOGRAPH:'AD53:44371:&#x8303
CJK UNIFIED IDEOGRAPH:'AD54:44372:&#x8305
CJK UNIFIED IDEOGRAPH:'AD55:44373:&#x82E3
CJK UNIFIED IDEOGRAPH:'AD56:44374:&#x82DB
CJK UNIFIED IDEOGRAPH:'AD57:44375:&#x82E6
CJK UNIFIED IDEOGRAPH:'AD58:44376:&#x8304
CJK UNIFIED IDEOGRAPH:'AD59:44377:&#x82E5
CJK UNIFIED IDEOGRAPH:'AD5A:44378:&#x8302
CJK UNIFIED IDEOGRAPH:'AD5B:44379:&#x8309
CJK UNIFIED IDEOGRAPH:'AD5C:44380:&#x82D2
CJK UNIFIED IDEOGRAPH:'AD5D:44381:&#x82D7
CJK UNIFIED IDEOGRAPH:'AD5E:44382:&#x82F1
CJK UNIFIED IDEOGRAPH:'AD5F:44383:&#x8301
CJK UNIFIED IDEOGRAPH:'AD60:44384:&#x82DC
CJK UNIFIED IDEOGRAPH:'AD61:44385:&#x82D4
CJK UNIFIED IDEOGRAPH:'AD62:44386:&#x82D1
CJK UNIFIED IDEOGRAPH:'AD63:44387:&#x82DE
CJK UNIFIED IDEOGRAPH:'AD64:44388:&#x82D3
CJK UNIFIED IDEOGRAPH:'AD65:44389:&#x82DF
CJK UNIFIED IDEOGRAPH:'AD66:44390:&#x82EF
CJK UNIFIED IDEOGRAPH:'AD67:44391:&#x8306
CJK UNIFIED IDEOGRAPH:'AD68:44392:&#x8650
CJK UNIFIED IDEOGRAPH:'AD69:44393:&#x8679
CJK UNIFIED IDEOGRAPH:'AD6A:44394:&#x867B
CJK UNIFIED IDEOGRAPH:'AD6B:44395:&#x867A
CJK UNIFIED IDEOGRAPH:'AD6C:44396:&#x884D
CJK UNIFIED IDEOGRAPH:'AD6D:44397:&#x886B
CJK UNIFIED IDEOGRAPH:'AD6E:44398:&#x8981
CJK UNIFIED IDEOGRAPH:'AD6F:44399:&#x89D4
CJK UNIFIED IDEOGRAPH:'AD70:44400:&#x8A08
CJK UNIFIED IDEOGRAPH:'AD71:44401:&#x8A02
CJK UNIFIED IDEOGRAPH:'AD72:44402:&#x8A03
CJK UNIFIED IDEOGRAPH:'AD73:44403:&#x8C9E
CJK UNIFIED IDEOGRAPH:'AD74:44404:&#x8CA0
CJK UNIFIED IDEOGRAPH:'AD75:44405:&#x8D74
CJK UNIFIED IDEOGRAPH:'AD76:44406:&#x8D73
CJK UNIFIED IDEOGRAPH:'AD77:44407:&#x8DB4
CJK UNIFIED IDEOGRAPH:'AD78:44408:&#x8ECD
CJK UNIFIED IDEOGRAPH:'AD79:44409:&#x8ECC
CJK UNIFIED IDEOGRAPH:'AD7A:44410:&#x8FF0
CJK UNIFIED IDEOGRAPH:'AD7B:44411:&#x8FE6
CJK UNIFIED IDEOGRAPH:'AD7C:44412:&#x8FE2
CJK UNIFIED IDEOGRAPH:'AD7D:44413:&#x8FEA
CJK UNIFIED IDEOGRAPH:'AD7E:44414:&#x8FE5
CJK UNIFIED IDEOGRAPH:'ADA1:44449:&#x8FED
CJK UNIFIED IDEOGRAPH:'ADA2:44450:&#x8FEB
CJK UNIFIED IDEOGRAPH:'ADA3:44451:&#x8FE4
CJK UNIFIED IDEOGRAPH:'ADA4:44452:&#x8FE8
CJK UNIFIED IDEOGRAPH:'ADA5:44453:&#x90CA
CJK UNIFIED IDEOGRAPH:'ADA6:44454:&#x90CE
CJK UNIFIED IDEOGRAPH:'ADA7:44455:&#x90C1
CJK UNIFIED IDEOGRAPH:'ADA8:44456:&#x90C3
CJK UNIFIED IDEOGRAPH:'ADA9:44457:&#x914B
CJK UNIFIED IDEOGRAPH:'ADAA:44458:&#x914A
CJK UNIFIED IDEOGRAPH:'ADAB:44459:&#x91CD
CJK UNIFIED IDEOGRAPH:'ADAC:44460:&#x9582
CJK UNIFIED IDEOGRAPH:'ADAD:44461:&#x9650
CJK UNIFIED IDEOGRAPH:'ADAE:44462:&#x964B
CJK UNIFIED IDEOGRAPH:'ADAF:44463:&#x964C
CJK UNIFIED IDEOGRAPH:'ADB0:44464:&#x964D
CJK UNIFIED IDEOGRAPH:'ADB1:44465:&#x9762
CJK UNIFIED IDEOGRAPH:'ADB2:44466:&#x9769
CJK UNIFIED IDEOGRAPH:'ADB3:44467:&#x97CB
CJK UNIFIED IDEOGRAPH:'ADB4:44468:&#x97ED
CJK UNIFIED IDEOGRAPH:'ADB5:44469:&#x97F3
CJK UNIFIED IDEOGRAPH:'ADB6:44470:&#x9801
CJK UNIFIED IDEOGRAPH:'ADB7:44471:&#x98A8
CJK UNIFIED IDEOGRAPH:'ADB8:44472:&#x98DB
CJK UNIFIED IDEOGRAPH:'ADB9:44473:&#x98DF
CJK UNIFIED IDEOGRAPH:'ADBA:44474:&#x9996
CJK UNIFIED IDEOGRAPH:'ADBB:44475:&#x9999
CJK UNIFIED IDEOGRAPH:'ADBC:44476:&#x4E58
CJK UNIFIED IDEOGRAPH:'ADBD:44477:&#x4EB3
CJK UNIFIED IDEOGRAPH:'ADBE:44478:&#x500C
CJK UNIFIED IDEOGRAPH:'ADBF:44479:&#x500D
CJK UNIFIED IDEOGRAPH:'ADC0:44480:&#x5023
CJK UNIFIED IDEOGRAPH:'ADC1:44481:&#x4FEF
CJK UNIFIED IDEOGRAPH:'ADC2:44482:&#x5026
CJK UNIFIED IDEOGRAPH:'ADC3:44483:&#x5025
CJK UNIFIED IDEOGRAPH:'ADC4:44484:&#x4FF8
CJK UNIFIED IDEOGRAPH:'ADC5:44485:&#x5029
CJK UNIFIED IDEOGRAPH:'ADC6:44486:&#x5016
CJK UNIFIED IDEOGRAPH:'ADC7:44487:&#x5006
CJK UNIFIED IDEOGRAPH:'ADC8:44488:&#x503C
CJK UNIFIED IDEOGRAPH:'ADC9:44489:&#x501F
CJK UNIFIED IDEOGRAPH:'ADCA:44490:&#x501A
CJK UNIFIED IDEOGRAPH:'ADCB:44491:&#x5012
CJK UNIFIED IDEOGRAPH:'ADCC:44492:&#x5011
CJK UNIFIED IDEOGRAPH:'ADCD:44493:&#x4FFA
CJK UNIFIED IDEOGRAPH:'ADCE:44494:&#x5000
CJK UNIFIED IDEOGRAPH:'ADCF:44495:&#x5014
CJK UNIFIED IDEOGRAPH:'ADD0:44496:&#x5028
CJK UNIFIED IDEOGRAPH:'ADD1:44497:&#x4FF1
CJK UNIFIED IDEOGRAPH:'ADD2:44498:&#x5021
CJK UNIFIED IDEOGRAPH:'ADD3:44499:&#x500B
CJK UNIFIED IDEOGRAPH:'ADD4:44500:&#x5019
CJK UNIFIED IDEOGRAPH:'ADD5:44501:&#x5018
CJK UNIFIED IDEOGRAPH:'ADD6:44502:&#x4FF3
CJK UNIFIED IDEOGRAPH:'ADD7:44503:&#x4FEE
CJK UNIFIED IDEOGRAPH:'ADD8:44504:&#x502D
CJK UNIFIED IDEOGRAPH:'ADD9:44505:&#x502A
CJK UNIFIED IDEOGRAPH:'ADDA:44506:&#x4FFE
CJK UNIFIED IDEOGRAPH:'ADDB:44507:&#x502B
CJK UNIFIED IDEOGRAPH:'ADDC:44508:&#x5009
CJK UNIFIED IDEOGRAPH:'ADDD:44509:&#x517C
CJK UNIFIED IDEOGRAPH:'ADDE:44510:&#x51A4
CJK UNIFIED IDEOGRAPH:'ADDF:44511:&#x51A5
CJK UNIFIED IDEOGRAPH:'ADE0:44512:&#x51A2
CJK UNIFIED IDEOGRAPH:'ADE1:44513:&#x51CD
CJK UNIFIED IDEOGRAPH:'ADE2:44514:&#x51CC
CJK UNIFIED IDEOGRAPH:'ADE3:44515:&#x51C6
CJK UNIFIED IDEOGRAPH:'ADE4:44516:&#x51CB
CJK UNIFIED IDEOGRAPH:'ADE5:44517:&#x5256
CJK UNIFIED IDEOGRAPH:'ADE6:44518:&#x525C
CJK UNIFIED IDEOGRAPH:'ADE7:44519:&#x5254
CJK UNIFIED IDEOGRAPH:'ADE8:44520:&#x525B
CJK UNIFIED IDEOGRAPH:'ADE9:44521:&#x525D
CJK UNIFIED IDEOGRAPH:'ADEA:44522:&#x532A
CJK UNIFIED IDEOGRAPH:'ADEB:44523:&#x537F
CJK UNIFIED IDEOGRAPH:'ADEC:44524:&#x539F
CJK UNIFIED IDEOGRAPH:'ADED:44525:&#x539D
CJK UNIFIED IDEOGRAPH:'ADEE:44526:&#x53DF
CJK UNIFIED IDEOGRAPH:'ADEF:44527:&#x54E8
CJK UNIFIED IDEOGRAPH:'ADF0:44528:&#x5510
CJK UNIFIED IDEOGRAPH:'ADF1:44529:&#x5501
CJK UNIFIED IDEOGRAPH:'ADF2:44530:&#x5537
CJK UNIFIED IDEOGRAPH:'ADF3:44531:&#x54FC
CJK UNIFIED IDEOGRAPH:'ADF4:44532:&#x54E5
CJK UNIFIED IDEOGRAPH:'ADF5:44533:&#x54F2
CJK UNIFIED IDEOGRAPH:'ADF6:44534:&#x5506
CJK UNIFIED IDEOGRAPH:'ADF7:44535:&#x54FA
CJK UNIFIED IDEOGRAPH:'ADF8:44536:&#x5514
CJK UNIFIED IDEOGRAPH:'ADF9:44537:&#x54E9
CJK UNIFIED IDEOGRAPH:'ADFA:44538:&#x54ED
CJK UNIFIED IDEOGRAPH:'ADFB:44539:&#x54E1
CJK UNIFIED IDEOGRAPH:'ADFC:44540:&#x5509
CJK UNIFIED IDEOGRAPH:'ADFD:44541:&#x54EE
CJK UNIFIED IDEOGRAPH:'ADFE:44542:&#x54EA
CJK UNIFIED IDEOGRAPH:'AE40:44608:&#x54E6
CJK UNIFIED IDEOGRAPH:'AE41:44609:&#x5527
CJK UNIFIED IDEOGRAPH:'AE42:44610:&#x5507
CJK UNIFIED IDEOGRAPH:'AE43:44611:&#x54FD
CJK UNIFIED IDEOGRAPH:'AE44:44612:&#x550F
CJK UNIFIED IDEOGRAPH:'AE45:44613:&#x5703
CJK UNIFIED IDEOGRAPH:'AE46:44614:&#x5704
CJK UNIFIED IDEOGRAPH:'AE47:44615:&#x57C2
CJK UNIFIED IDEOGRAPH:'AE48:44616:&#x57D4
CJK UNIFIED IDEOGRAPH:'AE49:44617:&#x57CB
CJK UNIFIED IDEOGRAPH:'AE4A:44618:&#x57C3
CJK UNIFIED IDEOGRAPH:'AE4B:44619:&#x5809
CJK UNIFIED IDEOGRAPH:'AE4C:44620:&#x590F
CJK UNIFIED IDEOGRAPH:'AE4D:44621:&#x5957
CJK UNIFIED IDEOGRAPH:'AE4E:44622:&#x5958
CJK UNIFIED IDEOGRAPH:'AE4F:44623:&#x595A
CJK UNIFIED IDEOGRAPH:'AE50:44624:&#x5A11
CJK UNIFIED IDEOGRAPH:'AE51:44625:&#x5A18
CJK UNIFIED IDEOGRAPH:'AE52:44626:&#x5A1C
CJK UNIFIED IDEOGRAPH:'AE53:44627:&#x5A1F
CJK UNIFIED IDEOGRAPH:'AE54:44628:&#x5A1B
CJK UNIFIED IDEOGRAPH:'AE55:44629:&#x5A13
CJK UNIFIED IDEOGRAPH:'AE56:44630:&#x59EC
CJK UNIFIED IDEOGRAPH:'AE57:44631:&#x5A20
CJK UNIFIED IDEOGRAPH:'AE58:44632:&#x5A23
CJK UNIFIED IDEOGRAPH:'AE59:44633:&#x5A29
CJK UNIFIED IDEOGRAPH:'AE5A:44634:&#x5A25
CJK UNIFIED IDEOGRAPH:'AE5B:44635:&#x5A0C
CJK UNIFIED IDEOGRAPH:'AE5C:44636:&#x5A09
CJK UNIFIED IDEOGRAPH:'AE5D:44637:&#x5B6B
CJK UNIFIED IDEOGRAPH:'AE5E:44638:&#x5C58
CJK UNIFIED IDEOGRAPH:'AE5F:44639:&#x5BB0
CJK UNIFIED IDEOGRAPH:'AE60:44640:&#x5BB3
CJK UNIFIED IDEOGRAPH:'AE61:44641:&#x5BB6
CJK UNIFIED IDEOGRAPH:'AE62:44642:&#x5BB4
CJK UNIFIED IDEOGRAPH:'AE63:44643:&#x5BAE
CJK UNIFIED IDEOGRAPH:'AE64:44644:&#x5BB5
CJK UNIFIED IDEOGRAPH:'AE65:44645:&#x5BB9
CJK UNIFIED IDEOGRAPH:'AE66:44646:&#x5BB8
CJK UNIFIED IDEOGRAPH:'AE67:44647:&#x5C04
CJK UNIFIED IDEOGRAPH:'AE68:44648:&#x5C51
CJK UNIFIED IDEOGRAPH:'AE69:44649:&#x5C55
CJK UNIFIED IDEOGRAPH:'AE6A:44650:&#x5C50
CJK UNIFIED IDEOGRAPH:'AE6B:44651:&#x5CED
CJK UNIFIED IDEOGRAPH:'AE6C:44652:&#x5CFD
CJK UNIFIED IDEOGRAPH:'AE6D:44653:&#x5CFB
CJK UNIFIED IDEOGRAPH:'AE6E:44654:&#x5CEA
CJK UNIFIED IDEOGRAPH:'AE6F:44655:&#x5CE8
CJK UNIFIED IDEOGRAPH:'AE70:44656:&#x5CF0
CJK UNIFIED IDEOGRAPH:'AE71:44657:&#x5CF6
CJK UNIFIED IDEOGRAPH:'AE72:44658:&#x5D01
CJK UNIFIED IDEOGRAPH:'AE73:44659:&#x5CF4
CJK UNIFIED IDEOGRAPH:'AE74:44660:&#x5DEE
CJK UNIFIED IDEOGRAPH:'AE75:44661:&#x5E2D
CJK UNIFIED IDEOGRAPH:'AE76:44662:&#x5E2B
CJK UNIFIED IDEOGRAPH:'AE77:44663:&#x5EAB
CJK UNIFIED IDEOGRAPH:'AE78:44664:&#x5EAD
CJK UNIFIED IDEOGRAPH:'AE79:44665:&#x5EA7
CJK UNIFIED IDEOGRAPH:'AE7A:44666:&#x5F31
CJK UNIFIED IDEOGRAPH:'AE7B:44667:&#x5F92
CJK UNIFIED IDEOGRAPH:'AE7C:44668:&#x5F91
CJK UNIFIED IDEOGRAPH:'AE7D:44669:&#x5F90
CJK UNIFIED IDEOGRAPH:'AE7E:44670:&#x6059
CJK UNIFIED IDEOGRAPH:'AEA1:44705:&#x6063
CJK UNIFIED IDEOGRAPH:'AEA2:44706:&#x6065
CJK UNIFIED IDEOGRAPH:'AEA3:44707:&#x6050
CJK UNIFIED IDEOGRAPH:'AEA4:44708:&#x6055
CJK UNIFIED IDEOGRAPH:'AEA5:44709:&#x606D
CJK UNIFIED IDEOGRAPH:'AEA6:44710:&#x6069
CJK UNIFIED IDEOGRAPH:'AEA7:44711:&#x606F
CJK UNIFIED IDEOGRAPH:'AEA8:44712:&#x6084
CJK UNIFIED IDEOGRAPH:'AEA9:44713:&#x609F
CJK UNIFIED IDEOGRAPH:'AEAA:44714:&#x609A
CJK UNIFIED IDEOGRAPH:'AEAB:44715:&#x608D
CJK UNIFIED IDEOGRAPH:'AEAC:44716:&#x6094
CJK UNIFIED IDEOGRAPH:'AEAD:44717:&#x608C
CJK UNIFIED IDEOGRAPH:'AEAE:44718:&#x6085
CJK UNIFIED IDEOGRAPH:'AEAF:44719:&#x6096
CJK UNIFIED IDEOGRAPH:'AEB0:44720:&#x6247
CJK UNIFIED IDEOGRAPH:'AEB1:44721:&#x62F3
CJK UNIFIED IDEOGRAPH:'AEB2:44722:&#x6308
CJK UNIFIED IDEOGRAPH:'AEB3:44723:&#x62FF
CJK UNIFIED IDEOGRAPH:'AEB4:44724:&#x634E
CJK UNIFIED IDEOGRAPH:'AEB5:44725:&#x633E
CJK UNIFIED IDEOGRAPH:'AEB6:44726:&#x632F
CJK UNIFIED IDEOGRAPH:'AEB7:44727:&#x6355
CJK UNIFIED IDEOGRAPH:'AEB8:44728:&#x6342
CJK UNIFIED IDEOGRAPH:'AEB9:44729:&#x6346
CJK UNIFIED IDEOGRAPH:'AEBA:44730:&#x634F
CJK UNIFIED IDEOGRAPH:'AEBB:44731:&#x6349
CJK UNIFIED IDEOGRAPH:'AEBC:44732:&#x633A
CJK UNIFIED IDEOGRAPH:'AEBD:44733:&#x6350
CJK UNIFIED IDEOGRAPH:'AEBE:44734:&#x633D
CJK UNIFIED IDEOGRAPH:'AEBF:44735:&#x632A
CJK UNIFIED IDEOGRAPH:'AEC0:44736:&#x632B
CJK UNIFIED IDEOGRAPH:'AEC1:44737:&#x6328
CJK UNIFIED IDEOGRAPH:'AEC2:44738:&#x634D
CJK UNIFIED IDEOGRAPH:'AEC3:44739:&#x634C
CJK UNIFIED IDEOGRAPH:'AEC4:44740:&#x6548
CJK UNIFIED IDEOGRAPH:'AEC5:44741:&#x6549
CJK UNIFIED IDEOGRAPH:'AEC6:44742:&#x6599
CJK UNIFIED IDEOGRAPH:'AEC7:44743:&#x65C1
CJK UNIFIED IDEOGRAPH:'AEC8:44744:&#x65C5
CJK UNIFIED IDEOGRAPH:'AEC9:44745:&#x6642
CJK UNIFIED IDEOGRAPH:'AECA:44746:&#x6649
CJK UNIFIED IDEOGRAPH:'AECB:44747:&#x664F
CJK UNIFIED IDEOGRAPH:'AECC:44748:&#x6643
CJK UNIFIED IDEOGRAPH:'AECD:44749:&#x6652
CJK UNIFIED IDEOGRAPH:'AECE:44750:&#x664C
CJK UNIFIED IDEOGRAPH:'AECF:44751:&#x6645
CJK UNIFIED IDEOGRAPH:'AED0:44752:&#x6641
CJK UNIFIED IDEOGRAPH:'AED1:44753:&#x66F8
CJK UNIFIED IDEOGRAPH:'AED2:44754:&#x6714
CJK UNIFIED IDEOGRAPH:'AED3:44755:&#x6715
CJK UNIFIED IDEOGRAPH:'AED4:44756:&#x6717
CJK UNIFIED IDEOGRAPH:'AED5:44757:&#x6821
CJK UNIFIED IDEOGRAPH:'AED6:44758:&#x6838
CJK UNIFIED IDEOGRAPH:'AED7:44759:&#x6848
CJK UNIFIED IDEOGRAPH:'AED8:44760:&#x6846
CJK UNIFIED IDEOGRAPH:'AED9:44761:&#x6853
CJK UNIFIED IDEOGRAPH:'AEDA:44762:&#x6839
CJK UNIFIED IDEOGRAPH:'AEDB:44763:&#x6842
CJK UNIFIED IDEOGRAPH:'AEDC:44764:&#x6854
CJK UNIFIED IDEOGRAPH:'AEDD:44765:&#x6829
CJK UNIFIED IDEOGRAPH:'AEDE:44766:&#x68B3
CJK UNIFIED IDEOGRAPH:'AEDF:44767:&#x6817
CJK UNIFIED IDEOGRAPH:'AEE0:44768:&#x684C
CJK UNIFIED IDEOGRAPH:'AEE1:44769:&#x6851
CJK UNIFIED IDEOGRAPH:'AEE2:44770:&#x683D
CJK UNIFIED IDEOGRAPH:'AEE3:44771:&#x67F4
CJK UNIFIED IDEOGRAPH:'AEE4:44772:&#x6850
CJK UNIFIED IDEOGRAPH:'AEE5:44773:&#x6840
CJK UNIFIED IDEOGRAPH:'AEE6:44774:&#x683C
CJK UNIFIED IDEOGRAPH:'AEE7:44775:&#x6843
CJK UNIFIED IDEOGRAPH:'AEE8:44776:&#x682A
CJK UNIFIED IDEOGRAPH:'AEE9:44777:&#x6845
CJK UNIFIED IDEOGRAPH:'AEEA:44778:&#x6813
CJK UNIFIED IDEOGRAPH:'AEEB:44779:&#x6818
CJK UNIFIED IDEOGRAPH:'AEEC:44780:&#x6841
CJK UNIFIED IDEOGRAPH:'AEED:44781:&#x6B8A
CJK UNIFIED IDEOGRAPH:'AEEE:44782:&#x6B89
CJK UNIFIED IDEOGRAPH:'AEEF:44783:&#x6BB7
CJK UNIFIED IDEOGRAPH:'AEF0:44784:&#x6C23
CJK UNIFIED IDEOGRAPH:'AEF1:44785:&#x6C27
CJK UNIFIED IDEOGRAPH:'AEF2:44786:&#x6C28
CJK UNIFIED IDEOGRAPH:'AEF3:44787:&#x6C26
CJK UNIFIED IDEOGRAPH:'AEF4:44788:&#x6C24
CJK UNIFIED IDEOGRAPH:'AEF5:44789:&#x6CF0
CJK UNIFIED IDEOGRAPH:'AEF6:44790:&#x6D6A
CJK UNIFIED IDEOGRAPH:'AEF7:44791:&#x6D95
CJK UNIFIED IDEOGRAPH:'AEF8:44792:&#x6D88
CJK UNIFIED IDEOGRAPH:'AEF9:44793:&#x6D87
CJK UNIFIED IDEOGRAPH:'AEFA:44794:&#x6D66
CJK UNIFIED IDEOGRAPH:'AEFB:44795:&#x6D78
CJK UNIFIED IDEOGRAPH:'AEFC:44796:&#x6D77
CJK UNIFIED IDEOGRAPH:'AEFD:44797:&#x6D59
CJK UNIFIED IDEOGRAPH:'AEFE:44798:&#x6D93
CJK UNIFIED IDEOGRAPH:'AF40:44864:&#x6D6C
CJK UNIFIED IDEOGRAPH:'AF41:44865:&#x6D89
CJK UNIFIED IDEOGRAPH:'AF42:44866:&#x6D6E
CJK UNIFIED IDEOGRAPH:'AF43:44867:&#x6D5A
CJK UNIFIED IDEOGRAPH:'AF44:44868:&#x6D74
CJK UNIFIED IDEOGRAPH:'AF45:44869:&#x6D69
CJK UNIFIED IDEOGRAPH:'AF46:44870:&#x6D8C
CJK UNIFIED IDEOGRAPH:'AF47:44871:&#x6D8A
CJK UNIFIED IDEOGRAPH:'AF48:44872:&#x6D79
CJK UNIFIED IDEOGRAPH:'AF49:44873:&#x6D85
CJK UNIFIED IDEOGRAPH:'AF4A:44874:&#x6D65
CJK UNIFIED IDEOGRAPH:'AF4B:44875:&#x6D94
CJK UNIFIED IDEOGRAPH:'AF4C:44876:&#x70CA
CJK UNIFIED IDEOGRAPH:'AF4D:44877:&#x70D8
CJK UNIFIED IDEOGRAPH:'AF4E:44878:&#x70E4
CJK UNIFIED IDEOGRAPH:'AF4F:44879:&#x70D9
CJK UNIFIED IDEOGRAPH:'AF50:44880:&#x70C8
CJK UNIFIED IDEOGRAPH:'AF51:44881:&#x70CF
CJK UNIFIED IDEOGRAPH:'AF52:44882:&#x7239
CJK UNIFIED IDEOGRAPH:'AF53:44883:&#x7279
CJK UNIFIED IDEOGRAPH:'AF54:44884:&#x72FC
CJK UNIFIED IDEOGRAPH:'AF55:44885:&#x72F9
CJK UNIFIED IDEOGRAPH:'AF56:44886:&#x72FD
CJK UNIFIED IDEOGRAPH:'AF57:44887:&#x72F8
CJK UNIFIED IDEOGRAPH:'AF58:44888:&#x72F7
CJK UNIFIED IDEOGRAPH:'AF59:44889:&#x7386
CJK UNIFIED IDEOGRAPH:'AF5A:44890:&#x73ED
CJK UNIFIED IDEOGRAPH:'AF5B:44891:&#x7409
CJK UNIFIED IDEOGRAPH:'AF5C:44892:&#x73EE
CJK UNIFIED IDEOGRAPH:'AF5D:44893:&#x73E0
CJK UNIFIED IDEOGRAPH:'AF5E:44894:&#x73EA
CJK UNIFIED IDEOGRAPH:'AF5F:44895:&#x73DE
CJK UNIFIED IDEOGRAPH:'AF60:44896:&#x7554
CJK UNIFIED IDEOGRAPH:'AF61:44897:&#x755D
CJK UNIFIED IDEOGRAPH:'AF62:44898:&#x755C
CJK UNIFIED IDEOGRAPH:'AF63:44899:&#x755A
CJK UNIFIED IDEOGRAPH:'AF64:44900:&#x7559
CJK UNIFIED IDEOGRAPH:'AF65:44901:&#x75BE
CJK UNIFIED IDEOGRAPH:'AF66:44902:&#x75C5
CJK UNIFIED IDEOGRAPH:'AF67:44903:&#x75C7
CJK UNIFIED IDEOGRAPH:'AF68:44904:&#x75B2
CJK UNIFIED IDEOGRAPH:'AF69:44905:&#x75B3
CJK UNIFIED IDEOGRAPH:'AF6A:44906:&#x75BD
CJK UNIFIED IDEOGRAPH:'AF6B:44907:&#x75BC
CJK UNIFIED IDEOGRAPH:'AF6C:44908:&#x75B9
CJK UNIFIED IDEOGRAPH:'AF6D:44909:&#x75C2
CJK UNIFIED IDEOGRAPH:'AF6E:44910:&#x75B8
CJK UNIFIED IDEOGRAPH:'AF6F:44911:&#x768B
CJK UNIFIED IDEOGRAPH:'AF70:44912:&#x76B0
CJK UNIFIED IDEOGRAPH:'AF71:44913:&#x76CA
CJK UNIFIED IDEOGRAPH:'AF72:44914:&#x76CD
CJK UNIFIED IDEOGRAPH:'AF73:44915:&#x76CE
CJK UNIFIED IDEOGRAPH:'AF74:44916:&#x7729
CJK UNIFIED IDEOGRAPH:'AF75:44917:&#x771F
CJK UNIFIED IDEOGRAPH:'AF76:44918:&#x7720
CJK UNIFIED IDEOGRAPH:'AF77:44919:&#x7728
CJK UNIFIED IDEOGRAPH:'AF78:44920:&#x77E9
CJK UNIFIED IDEOGRAPH:'AF79:44921:&#x7830
CJK UNIFIED IDEOGRAPH:'AF7A:44922:&#x7827
CJK UNIFIED IDEOGRAPH:'AF7B:44923:&#x7838
CJK UNIFIED IDEOGRAPH:'AF7C:44924:&#x781D
CJK UNIFIED IDEOGRAPH:'AF7D:44925:&#x7834
CJK UNIFIED IDEOGRAPH:'AF7E:44926:&#x7837
CJK UNIFIED IDEOGRAPH:'AFA1:44961:&#x7825
CJK UNIFIED IDEOGRAPH:'AFA2:44962:&#x782D
CJK UNIFIED IDEOGRAPH:'AFA3:44963:&#x7820
CJK UNIFIED IDEOGRAPH:'AFA4:44964:&#x781F
CJK UNIFIED IDEOGRAPH:'AFA5:44965:&#x7832
CJK UNIFIED IDEOGRAPH:'AFA6:44966:&#x7955
CJK UNIFIED IDEOGRAPH:'AFA7:44967:&#x7950
CJK UNIFIED IDEOGRAPH:'AFA8:44968:&#x7960
CJK UNIFIED IDEOGRAPH:'AFA9:44969:&#x795F
CJK UNIFIED IDEOGRAPH:'AFAA:44970:&#x7956
CJK UNIFIED IDEOGRAPH:'AFAB:44971:&#x795E
CJK UNIFIED IDEOGRAPH:'AFAC:44972:&#x795D
CJK UNIFIED IDEOGRAPH:'AFAD:44973:&#x7957
CJK UNIFIED IDEOGRAPH:'AFAE:44974:&#x795A
CJK UNIFIED IDEOGRAPH:'AFAF:44975:&#x79E4
CJK UNIFIED IDEOGRAPH:'AFB0:44976:&#x79E3
CJK UNIFIED IDEOGRAPH:'AFB1:44977:&#x79E7
CJK UNIFIED IDEOGRAPH:'AFB2:44978:&#x79DF
CJK UNIFIED IDEOGRAPH:'AFB3:44979:&#x79E6
CJK UNIFIED IDEOGRAPH:'AFB4:44980:&#x79E9
CJK UNIFIED IDEOGRAPH:'AFB5:44981:&#x79D8
CJK UNIFIED IDEOGRAPH:'AFB6:44982:&#x7A84
CJK UNIFIED IDEOGRAPH:'AFB7:44983:&#x7A88
CJK UNIFIED IDEOGRAPH:'AFB8:44984:&#x7AD9
CJK UNIFIED IDEOGRAPH:'AFB9:44985:&#x7B06
CJK UNIFIED IDEOGRAPH:'AFBA:44986:&#x7B11
CJK UNIFIED IDEOGRAPH:'AFBB:44987:&#x7C89
CJK UNIFIED IDEOGRAPH:'AFBC:44988:&#x7D21
CJK UNIFIED IDEOGRAPH:'AFBD:44989:&#x7D17
CJK UNIFIED IDEOGRAPH:'AFBE:44990:&#x7D0B
CJK UNIFIED IDEOGRAPH:'AFBF:44991:&#x7D0A
CJK UNIFIED IDEOGRAPH:'AFC0:44992:&#x7D20
CJK UNIFIED IDEOGRAPH:'AFC1:44993:&#x7D22
CJK UNIFIED IDEOGRAPH:'AFC2:44994:&#x7D14
CJK UNIFIED IDEOGRAPH:'AFC3:44995:&#x7D10
CJK UNIFIED IDEOGRAPH:'AFC4:44996:&#x7D15
CJK UNIFIED IDEOGRAPH:'AFC5:44997:&#x7D1A
CJK UNIFIED IDEOGRAPH:'AFC6:44998:&#x7D1C
CJK UNIFIED IDEOGRAPH:'AFC7:44999:&#x7D0D
CJK UNIFIED IDEOGRAPH:'AFC8:45000:&#x7D19
CJK UNIFIED IDEOGRAPH:'AFC9:45001:&#x7D1B
CJK UNIFIED IDEOGRAPH:'AFCA:45002:&#x7F3A
CJK UNIFIED IDEOGRAPH:'AFCB:45003:&#x7F5F
CJK UNIFIED IDEOGRAPH:'AFCC:45004:&#x7F94
CJK UNIFIED IDEOGRAPH:'AFCD:45005:&#x7FC5
CJK UNIFIED IDEOGRAPH:'AFCE:45006:&#x7FC1
CJK UNIFIED IDEOGRAPH:'AFCF:45007:&#x8006
CJK UNIFIED IDEOGRAPH:'AFD0:45008:&#x8018
CJK UNIFIED IDEOGRAPH:'AFD1:45009:&#x8015
CJK UNIFIED IDEOGRAPH:'AFD2:45010:&#x8019
CJK UNIFIED IDEOGRAPH:'AFD3:45011:&#x8017
CJK UNIFIED IDEOGRAPH:'AFD4:45012:&#x803D
CJK UNIFIED IDEOGRAPH:'AFD5:45013:&#x803F
CJK UNIFIED IDEOGRAPH:'AFD6:45014:&#x80F1
CJK UNIFIED IDEOGRAPH:'AFD7:45015:&#x8102
CJK UNIFIED IDEOGRAPH:'AFD8:45016:&#x80F0
CJK UNIFIED IDEOGRAPH:'AFD9:45017:&#x8105
CJK UNIFIED IDEOGRAPH:'AFDA:45018:&#x80ED
CJK UNIFIED IDEOGRAPH:'AFDB:45019:&#x80F4
CJK UNIFIED IDEOGRAPH:'AFDC:45020:&#x8106
CJK UNIFIED IDEOGRAPH:'AFDD:45021:&#x80F8
CJK UNIFIED IDEOGRAPH:'AFDE:45022:&#x80F3
CJK UNIFIED IDEOGRAPH:'AFDF:45023:&#x8108
CJK UNIFIED IDEOGRAPH:'AFE0:45024:&#x80FD
CJK UNIFIED IDEOGRAPH:'AFE1:45025:&#x810A
CJK UNIFIED IDEOGRAPH:'AFE2:45026:&#x80FC
CJK UNIFIED IDEOGRAPH:'AFE3:45027:&#x80EF
CJK UNIFIED IDEOGRAPH:'AFE4:45028:&#x81ED
CJK UNIFIED IDEOGRAPH:'AFE5:45029:&#x81EC
CJK UNIFIED IDEOGRAPH:'AFE6:45030:&#x8200
CJK UNIFIED IDEOGRAPH:'AFE7:45031:&#x8210
CJK UNIFIED IDEOGRAPH:'AFE8:45032:&#x822A
CJK UNIFIED IDEOGRAPH:'AFE9:45033:&#x822B
CJK UNIFIED IDEOGRAPH:'AFEA:45034:&#x8228
CJK UNIFIED IDEOGRAPH:'AFEB:45035:&#x822C
CJK UNIFIED IDEOGRAPH:'AFEC:45036:&#x82BB
CJK UNIFIED IDEOGRAPH:'AFED:45037:&#x832B
CJK UNIFIED IDEOGRAPH:'AFEE:45038:&#x8352
CJK UNIFIED IDEOGRAPH:'AFEF:45039:&#x8354
CJK UNIFIED IDEOGRAPH:'AFF0:45040:&#x834A
CJK UNIFIED IDEOGRAPH:'AFF1:45041:&#x8338
CJK UNIFIED IDEOGRAPH:'AFF2:45042:&#x8350
CJK UNIFIED IDEOGRAPH:'AFF3:45043:&#x8349
CJK UNIFIED IDEOGRAPH:'AFF4:45044:&#x8335
CJK UNIFIED IDEOGRAPH:'AFF5:45045:&#x8334
CJK UNIFIED IDEOGRAPH:'AFF6:45046:&#x834F
CJK UNIFIED IDEOGRAPH:'AFF7:45047:&#x8332
CJK UNIFIED IDEOGRAPH:'AFF8:45048:&#x8339
CJK UNIFIED IDEOGRAPH:'AFF9:45049:&#x8336
CJK UNIFIED IDEOGRAPH:'AFFA:45050:&#x8317
CJK UNIFIED IDEOGRAPH:'AFFB:45051:&#x8340
CJK UNIFIED IDEOGRAPH:'AFFC:45052:&#x8331
CJK UNIFIED IDEOGRAPH:'AFFD:45053:&#x8328
CJK UNIFIED IDEOGRAPH:'AFFE:45054:&#x8343
CJK UNIFIED IDEOGRAPH:'B040:45120:&#x8654
CJK UNIFIED IDEOGRAPH:'B041:45121:&#x868A
CJK UNIFIED IDEOGRAPH:'B042:45122:&#x86AA
CJK UNIFIED IDEOGRAPH:'B043:45123:&#x8693
CJK UNIFIED IDEOGRAPH:'B044:45124:&#x86A4
CJK UNIFIED IDEOGRAPH:'B045:45125:&#x86A9
CJK UNIFIED IDEOGRAPH:'B046:45126:&#x868C
CJK UNIFIED IDEOGRAPH:'B047:45127:&#x86A3
CJK UNIFIED IDEOGRAPH:'B048:45128:&#x869C
CJK UNIFIED IDEOGRAPH:'B049:45129:&#x8870
CJK UNIFIED IDEOGRAPH:'B04A:45130:&#x8877
CJK UNIFIED IDEOGRAPH:'B04B:45131:&#x8881
CJK UNIFIED IDEOGRAPH:'B04C:45132:&#x8882
CJK UNIFIED IDEOGRAPH:'B04D:45133:&#x887D
CJK UNIFIED IDEOGRAPH:'B04E:45134:&#x8879
CJK UNIFIED IDEOGRAPH:'B04F:45135:&#x8A18
CJK UNIFIED IDEOGRAPH:'B050:45136:&#x8A10
CJK UNIFIED IDEOGRAPH:'B051:45137:&#x8A0E
CJK UNIFIED IDEOGRAPH:'B052:45138:&#x8A0C
CJK UNIFIED IDEOGRAPH:'B053:45139:&#x8A15
CJK UNIFIED IDEOGRAPH:'B054:45140:&#x8A0A
CJK UNIFIED IDEOGRAPH:'B055:45141:&#x8A17
CJK UNIFIED IDEOGRAPH:'B056:45142:&#x8A13
CJK UNIFIED IDEOGRAPH:'B057:45143:&#x8A16
CJK UNIFIED IDEOGRAPH:'B058:45144:&#x8A0F
CJK UNIFIED IDEOGRAPH:'B059:45145:&#x8A11
CJK UNIFIED IDEOGRAPH:'B05A:45146:&#x8C48
CJK UNIFIED IDEOGRAPH:'B05B:45147:&#x8C7A
CJK UNIFIED IDEOGRAPH:'B05C:45148:&#x8C79
CJK UNIFIED IDEOGRAPH:'B05D:45149:&#x8CA1
CJK UNIFIED IDEOGRAPH:'B05E:45150:&#x8CA2
CJK UNIFIED IDEOGRAPH:'B05F:45151:&#x8D77
CJK UNIFIED IDEOGRAPH:'B060:45152:&#x8EAC
CJK UNIFIED IDEOGRAPH:'B061:45153:&#x8ED2
CJK UNIFIED IDEOGRAPH:'B062:45154:&#x8ED4
CJK UNIFIED IDEOGRAPH:'B063:45155:&#x8ECF
CJK UNIFIED IDEOGRAPH:'B064:45156:&#x8FB1
CJK UNIFIED IDEOGRAPH:'B065:45157:&#x9001
CJK UNIFIED IDEOGRAPH:'B066:45158:&#x9006
CJK UNIFIED IDEOGRAPH:'B067:45159:&#x8FF7
CJK UNIFIED IDEOGRAPH:'B068:45160:&#x9000
CJK UNIFIED IDEOGRAPH:'B069:45161:&#x8FFA
CJK UNIFIED IDEOGRAPH:'B06A:45162:&#x8FF4
CJK UNIFIED IDEOGRAPH:'B06B:45163:&#x9003
CJK UNIFIED IDEOGRAPH:'B06C:45164:&#x8FFD
CJK UNIFIED IDEOGRAPH:'B06D:45165:&#x9005
CJK UNIFIED IDEOGRAPH:'B06E:45166:&#x8FF8
CJK UNIFIED IDEOGRAPH:'B06F:45167:&#x9095
CJK UNIFIED IDEOGRAPH:'B070:45168:&#x90E1
CJK UNIFIED IDEOGRAPH:'B071:45169:&#x90DD
CJK UNIFIED IDEOGRAPH:'B072:45170:&#x90E2
CJK UNIFIED IDEOGRAPH:'B073:45171:&#x9152
CJK UNIFIED IDEOGRAPH:'B074:45172:&#x914D
CJK UNIFIED IDEOGRAPH:'B075:45173:&#x914C
CJK UNIFIED IDEOGRAPH:'B076:45174:&#x91D8
CJK UNIFIED IDEOGRAPH:'B077:45175:&#x91DD
CJK UNIFIED IDEOGRAPH:'B078:45176:&#x91D7
CJK UNIFIED IDEOGRAPH:'B079:45177:&#x91DC
CJK UNIFIED IDEOGRAPH:'B07A:45178:&#x91D9
CJK UNIFIED IDEOGRAPH:'B07B:45179:&#x9583
CJK UNIFIED IDEOGRAPH:'B07C:45180:&#x9662
CJK UNIFIED IDEOGRAPH:'B07D:45181:&#x9663
CJK UNIFIED IDEOGRAPH:'B07E:45182:&#x9661
CJK UNIFIED IDEOGRAPH:'B0A1:45217:&#x965B
CJK UNIFIED IDEOGRAPH:'B0A2:45218:&#x965D
CJK UNIFIED IDEOGRAPH:'B0A3:45219:&#x9664
CJK UNIFIED IDEOGRAPH:'B0A4:45220:&#x9658
CJK UNIFIED IDEOGRAPH:'B0A5:45221:&#x965E
CJK UNIFIED IDEOGRAPH:'B0A6:45222:&#x96BB
CJK UNIFIED IDEOGRAPH:'B0A7:45223:&#x98E2
CJK UNIFIED IDEOGRAPH:'B0A8:45224:&#x99AC
CJK UNIFIED IDEOGRAPH:'B0A9:45225:&#x9AA8
CJK UNIFIED IDEOGRAPH:'B0AA:45226:&#x9AD8
CJK UNIFIED IDEOGRAPH:'B0AB:45227:&#x9B25
CJK UNIFIED IDEOGRAPH:'B0AC:45228:&#x9B32
CJK UNIFIED IDEOGRAPH:'B0AD:45229:&#x9B3C
CJK UNIFIED IDEOGRAPH:'B0AE:45230:&#x4E7E
CJK UNIFIED IDEOGRAPH:'B0AF:45231:&#x507A
CJK UNIFIED IDEOGRAPH:'B0B0:45232:&#x507D
CJK UNIFIED IDEOGRAPH:'B0B1:45233:&#x505C
CJK UNIFIED IDEOGRAPH:'B0B2:45234:&#x5047
CJK UNIFIED IDEOGRAPH:'B0B3:45235:&#x5043
CJK UNIFIED IDEOGRAPH:'B0B4:45236:&#x504C
CJK UNIFIED IDEOGRAPH:'B0B5:45237:&#x505A
CJK UNIFIED IDEOGRAPH:'B0B6:45238:&#x5049
CJK UNIFIED IDEOGRAPH:'B0B7:45239:&#x5065
CJK UNIFIED IDEOGRAPH:'B0B8:45240:&#x5076
CJK UNIFIED IDEOGRAPH:'B0B9:45241:&#x504E
CJK UNIFIED IDEOGRAPH:'B0BA:45242:&#x5055
CJK UNIFIED IDEOGRAPH:'B0BB:45243:&#x5075
CJK UNIFIED IDEOGRAPH:'B0BC:45244:&#x5074
CJK UNIFIED IDEOGRAPH:'B0BD:45245:&#x5077
CJK UNIFIED IDEOGRAPH:'B0BE:45246:&#x504F
CJK UNIFIED IDEOGRAPH:'B0BF:45247:&#x500F
CJK UNIFIED IDEOGRAPH:'B0C0:45248:&#x506F
CJK UNIFIED IDEOGRAPH:'B0C1:45249:&#x506D
CJK UNIFIED IDEOGRAPH:'B0C2:45250:&#x515C
CJK UNIFIED IDEOGRAPH:'B0C3:45251:&#x5195
CJK UNIFIED IDEOGRAPH:'B0C4:45252:&#x51F0
CJK UNIFIED IDEOGRAPH:'B0C5:45253:&#x526A
CJK UNIFIED IDEOGRAPH:'B0C6:45254:&#x526F
CJK UNIFIED IDEOGRAPH:'B0C7:45255:&#x52D2
CJK UNIFIED IDEOGRAPH:'B0C8:45256:&#x52D9
CJK UNIFIED IDEOGRAPH:'B0C9:45257:&#x52D8
CJK UNIFIED IDEOGRAPH:'B0CA:45258:&#x52D5
CJK UNIFIED IDEOGRAPH:'B0CB:45259:&#x5310
CJK UNIFIED IDEOGRAPH:'B0CC:45260:&#x530F
CJK UNIFIED IDEOGRAPH:'B0CD:45261:&#x5319
CJK UNIFIED IDEOGRAPH:'B0CE:45262:&#x533F
CJK UNIFIED IDEOGRAPH:'B0CF:45263:&#x5340
CJK UNIFIED IDEOGRAPH:'B0D0:45264:&#x533E
CJK UNIFIED IDEOGRAPH:'B0D1:45265:&#x53C3
CJK UNIFIED IDEOGRAPH:'B0D2:45266:&#x66FC
CJK UNIFIED IDEOGRAPH:'B0D3:45267:&#x5546
CJK UNIFIED IDEOGRAPH:'B0D4:45268:&#x556A
CJK UNIFIED IDEOGRAPH:'B0D5:45269:&#x5566
CJK UNIFIED IDEOGRAPH:'B0D6:45270:&#x5544
CJK UNIFIED IDEOGRAPH:'B0D7:45271:&#x555E
CJK UNIFIED IDEOGRAPH:'B0D8:45272:&#x5561
CJK UNIFIED IDEOGRAPH:'B0D9:45273:&#x5543
CJK UNIFIED IDEOGRAPH:'B0DA:45274:&#x554A
CJK UNIFIED IDEOGRAPH:'B0DB:45275:&#x5531
CJK UNIFIED IDEOGRAPH:'B0DC:45276:&#x5556
CJK UNIFIED IDEOGRAPH:'B0DD:45277:&#x554F
CJK UNIFIED IDEOGRAPH:'B0DE:45278:&#x5555
CJK UNIFIED IDEOGRAPH:'B0DF:45279:&#x552F
CJK UNIFIED IDEOGRAPH:'B0E0:45280:&#x5564
CJK UNIFIED IDEOGRAPH:'B0E1:45281:&#x5538
CJK UNIFIED IDEOGRAPH:'B0E2:45282:&#x552E
CJK UNIFIED IDEOGRAPH:'B0E3:45283:&#x555C
CJK UNIFIED IDEOGRAPH:'B0E4:45284:&#x552C
CJK UNIFIED IDEOGRAPH:'B0E5:45285:&#x5563
CJK UNIFIED IDEOGRAPH:'B0E6:45286:&#x5533
CJK UNIFIED IDEOGRAPH:'B0E7:45287:&#x5541
CJK UNIFIED IDEOGRAPH:'B0E8:45288:&#x5557
CJK UNIFIED IDEOGRAPH:'B0E9:45289:&#x5708
CJK UNIFIED IDEOGRAPH:'B0EA:45290:&#x570B
CJK UNIFIED IDEOGRAPH:'B0EB:45291:&#x5709
CJK UNIFIED IDEOGRAPH:'B0EC:45292:&#x57DF
CJK UNIFIED IDEOGRAPH:'B0ED:45293:&#x5805
CJK UNIFIED IDEOGRAPH:'B0EE:45294:&#x580A
CJK UNIFIED IDEOGRAPH:'B0EF:45295:&#x5806
CJK UNIFIED IDEOGRAPH:'B0F0:45296:&#x57E0
CJK UNIFIED IDEOGRAPH:'B0F1:45297:&#x57E4
CJK UNIFIED IDEOGRAPH:'B0F2:45298:&#x57FA
CJK UNIFIED IDEOGRAPH:'B0F3:45299:&#x5802
CJK UNIFIED IDEOGRAPH:'B0F4:45300:&#x5835
CJK UNIFIED IDEOGRAPH:'B0F5:45301:&#x57F7
CJK UNIFIED IDEOGRAPH:'B0F6:45302:&#x57F9
CJK UNIFIED IDEOGRAPH:'B0F7:45303:&#x5920
CJK UNIFIED IDEOGRAPH:'B0F8:45304:&#x5962
CJK UNIFIED IDEOGRAPH:'B0F9:45305:&#x5A36
CJK UNIFIED IDEOGRAPH:'B0FA:45306:&#x5A41
CJK UNIFIED IDEOGRAPH:'B0FB:45307:&#x5A49
CJK UNIFIED IDEOGRAPH:'B0FC:45308:&#x5A66
CJK UNIFIED IDEOGRAPH:'B0FD:45309:&#x5A6A
CJK UNIFIED IDEOGRAPH:'B0FE:45310:&#x5A40
CJK UNIFIED IDEOGRAPH:'B140:45376:&#x5A3C
CJK UNIFIED IDEOGRAPH:'B141:45377:&#x5A62
CJK UNIFIED IDEOGRAPH:'B142:45378:&#x5A5A
CJK UNIFIED IDEOGRAPH:'B143:45379:&#x5A46
CJK UNIFIED IDEOGRAPH:'B144:45380:&#x5A4A
CJK UNIFIED IDEOGRAPH:'B145:45381:&#x5B70
CJK UNIFIED IDEOGRAPH:'B146:45382:&#x5BC7
CJK UNIFIED IDEOGRAPH:'B147:45383:&#x5BC5
CJK UNIFIED IDEOGRAPH:'B148:45384:&#x5BC4
CJK UNIFIED IDEOGRAPH:'B149:45385:&#x5BC2
CJK UNIFIED IDEOGRAPH:'B14A:45386:&#x5BBF
CJK UNIFIED IDEOGRAPH:'B14B:45387:&#x5BC6
CJK UNIFIED IDEOGRAPH:'B14C:45388:&#x5C09
CJK UNIFIED IDEOGRAPH:'B14D:45389:&#x5C08
CJK UNIFIED IDEOGRAPH:'B14E:45390:&#x5C07
CJK UNIFIED IDEOGRAPH:'B14F:45391:&#x5C60
CJK UNIFIED IDEOGRAPH:'B150:45392:&#x5C5C
CJK UNIFIED IDEOGRAPH:'B151:45393:&#x5C5D
CJK UNIFIED IDEOGRAPH:'B152:45394:&#x5D07
CJK UNIFIED IDEOGRAPH:'B153:45395:&#x5D06
CJK UNIFIED IDEOGRAPH:'B154:45396:&#x5D0E
CJK UNIFIED IDEOGRAPH:'B155:45397:&#x5D1B
CJK UNIFIED IDEOGRAPH:'B156:45398:&#x5D16
CJK UNIFIED IDEOGRAPH:'B157:45399:&#x5D22
CJK UNIFIED IDEOGRAPH:'B158:45400:&#x5D11
CJK UNIFIED IDEOGRAPH:'B159:45401:&#x5D29
CJK UNIFIED IDEOGRAPH:'B15A:45402:&#x5D14
CJK UNIFIED IDEOGRAPH:'B15B:45403:&#x5D19
CJK UNIFIED IDEOGRAPH:'B15C:45404:&#x5D24
CJK UNIFIED IDEOGRAPH:'B15D:45405:&#x5D27
CJK UNIFIED IDEOGRAPH:'B15E:45406:&#x5D17
CJK UNIFIED IDEOGRAPH:'B15F:45407:&#x5DE2
CJK UNIFIED IDEOGRAPH:'B160:45408:&#x5E38
CJK UNIFIED IDEOGRAPH:'B161:45409:&#x5E36
CJK UNIFIED IDEOGRAPH:'B162:45410:&#x5E33
CJK UNIFIED IDEOGRAPH:'B163:45411:&#x5E37
CJK UNIFIED IDEOGRAPH:'B164:45412:&#x5EB7
CJK UNIFIED IDEOGRAPH:'B165:45413:&#x5EB8
CJK UNIFIED IDEOGRAPH:'B166:45414:&#x5EB6
CJK UNIFIED IDEOGRAPH:'B167:45415:&#x5EB5
CJK UNIFIED IDEOGRAPH:'B168:45416:&#x5EBE
CJK UNIFIED IDEOGRAPH:'B169:45417:&#x5F35
CJK UNIFIED IDEOGRAPH:'B16A:45418:&#x5F37
CJK UNIFIED IDEOGRAPH:'B16B:45419:&#x5F57
CJK UNIFIED IDEOGRAPH:'B16C:45420:&#x5F6C
CJK UNIFIED IDEOGRAPH:'B16D:45421:&#x5F69
CJK UNIFIED IDEOGRAPH:'B16E:45422:&#x5F6B
CJK UNIFIED IDEOGRAPH:'B16F:45423:&#x5F97
CJK UNIFIED IDEOGRAPH:'B170:45424:&#x5F99
CJK UNIFIED IDEOGRAPH:'B171:45425:&#x5F9E
CJK UNIFIED IDEOGRAPH:'B172:45426:&#x5F98
CJK UNIFIED IDEOGRAPH:'B173:45427:&#x5FA1
CJK UNIFIED IDEOGRAPH:'B174:45428:&#x5FA0
CJK UNIFIED IDEOGRAPH:'B175:45429:&#x5F9C
CJK UNIFIED IDEOGRAPH:'B176:45430:&#x607F
CJK UNIFIED IDEOGRAPH:'B177:45431:&#x60A3
CJK UNIFIED IDEOGRAPH:'B178:45432:&#x6089
CJK UNIFIED IDEOGRAPH:'B179:45433:&#x60A0
CJK UNIFIED IDEOGRAPH:'B17A:45434:&#x60A8
CJK UNIFIED IDEOGRAPH:'B17B:45435:&#x60CB
CJK UNIFIED IDEOGRAPH:'B17C:45436:&#x60B4
CJK UNIFIED IDEOGRAPH:'B17D:45437:&#x60E6
CJK UNIFIED IDEOGRAPH:'B17E:45438:&#x60BD
CJK UNIFIED IDEOGRAPH:'B1A1:45473:&#x60C5
CJK UNIFIED IDEOGRAPH:'B1A2:45474:&#x60BB
CJK UNIFIED IDEOGRAPH:'B1A3:45475:&#x60B5
CJK UNIFIED IDEOGRAPH:'B1A4:45476:&#x60DC
CJK UNIFIED IDEOGRAPH:'B1A5:45477:&#x60BC
CJK UNIFIED IDEOGRAPH:'B1A6:45478:&#x60D8
CJK UNIFIED IDEOGRAPH:'B1A7:45479:&#x60D5
CJK UNIFIED IDEOGRAPH:'B1A8:45480:&#x60C6
CJK UNIFIED IDEOGRAPH:'B1A9:45481:&#x60DF
CJK UNIFIED IDEOGRAPH:'B1AA:45482:&#x60B8
CJK UNIFIED IDEOGRAPH:'B1AB:45483:&#x60DA
CJK UNIFIED IDEOGRAPH:'B1AC:45484:&#x60C7
CJK UNIFIED IDEOGRAPH:'B1AD:45485:&#x621A
CJK UNIFIED IDEOGRAPH:'B1AE:45486:&#x621B
CJK UNIFIED IDEOGRAPH:'B1AF:45487:&#x6248
CJK UNIFIED IDEOGRAPH:'B1B0:45488:&#x63A0
CJK UNIFIED IDEOGRAPH:'B1B1:45489:&#x63A7
CJK UNIFIED IDEOGRAPH:'B1B2:45490:&#x6372
CJK UNIFIED IDEOGRAPH:'B1B3:45491:&#x6396
CJK UNIFIED IDEOGRAPH:'B1B4:45492:&#x63A2
CJK UNIFIED IDEOGRAPH:'B1B5:45493:&#x63A5
CJK UNIFIED IDEOGRAPH:'B1B6:45494:&#x6377
CJK UNIFIED IDEOGRAPH:'B1B7:45495:&#x6367
CJK UNIFIED IDEOGRAPH:'B1B8:45496:&#x6398
CJK UNIFIED IDEOGRAPH:'B1B9:45497:&#x63AA
CJK UNIFIED IDEOGRAPH:'B1BA:45498:&#x6371
CJK UNIFIED IDEOGRAPH:'B1BB:45499:&#x63A9
CJK UNIFIED IDEOGRAPH:'B1BC:45500:&#x6389
CJK UNIFIED IDEOGRAPH:'B1BD:45501:&#x6383
CJK UNIFIED IDEOGRAPH:'B1BE:45502:&#x639B
CJK UNIFIED IDEOGRAPH:'B1BF:45503:&#x636B
CJK UNIFIED IDEOGRAPH:'B1C0:45504:&#x63A8
CJK UNIFIED IDEOGRAPH:'B1C1:45505:&#x6384
CJK UNIFIED IDEOGRAPH:'B1C2:45506:&#x6388
CJK UNIFIED IDEOGRAPH:'B1C3:45507:&#x6399
CJK UNIFIED IDEOGRAPH:'B1C4:45508:&#x63A1
CJK UNIFIED IDEOGRAPH:'B1C5:45509:&#x63AC
CJK UNIFIED IDEOGRAPH:'B1C6:45510:&#x6392
CJK UNIFIED IDEOGRAPH:'B1C7:45511:&#x638F
CJK UNIFIED IDEOGRAPH:'B1C8:45512:&#x6380
CJK UNIFIED IDEOGRAPH:'B1C9:45513:&#x637B
CJK UNIFIED IDEOGRAPH:'B1CA:45514:&#x6369
CJK UNIFIED IDEOGRAPH:'B1CB:45515:&#x6368
CJK UNIFIED IDEOGRAPH:'B1CC:45516:&#x637A
CJK UNIFIED IDEOGRAPH:'B1CD:45517:&#x655D
CJK UNIFIED IDEOGRAPH:'B1CE:45518:&#x6556
CJK UNIFIED IDEOGRAPH:'B1CF:45519:&#x6551
CJK UNIFIED IDEOGRAPH:'B1D0:45520:&#x6559
CJK UNIFIED IDEOGRAPH:'B1D1:45521:&#x6557
CJK UNIFIED IDEOGRAPH:'B1D2:45522:&#x555F
CJK UNIFIED IDEOGRAPH:'B1D3:45523:&#x654F
CJK UNIFIED IDEOGRAPH:'B1D4:45524:&#x6558
CJK UNIFIED IDEOGRAPH:'B1D5:45525:&#x6555
CJK UNIFIED IDEOGRAPH:'B1D6:45526:&#x6554
CJK UNIFIED IDEOGRAPH:'B1D7:45527:&#x659C
CJK UNIFIED IDEOGRAPH:'B1D8:45528:&#x659B
CJK UNIFIED IDEOGRAPH:'B1D9:45529:&#x65AC
CJK UNIFIED IDEOGRAPH:'B1DA:45530:&#x65CF
CJK UNIFIED IDEOGRAPH:'B1DB:45531:&#x65CB
CJK UNIFIED IDEOGRAPH:'B1DC:45532:&#x65CC
CJK UNIFIED IDEOGRAPH:'B1DD:45533:&#x65CE
CJK UNIFIED IDEOGRAPH:'B1DE:45534:&#x665D
CJK UNIFIED IDEOGRAPH:'B1DF:45535:&#x665A
CJK UNIFIED IDEOGRAPH:'B1E0:45536:&#x6664
CJK UNIFIED IDEOGRAPH:'B1E1:45537:&#x6668
CJK UNIFIED IDEOGRAPH:'B1E2:45538:&#x6666
CJK UNIFIED IDEOGRAPH:'B1E3:45539:&#x665E
CJK UNIFIED IDEOGRAPH:'B1E4:45540:&#x66F9
CJK UNIFIED IDEOGRAPH:'B1E5:45541:&#x52D7
CJK UNIFIED IDEOGRAPH:'B1E6:45542:&#x671B
CJK UNIFIED IDEOGRAPH:'B1E7:45543:&#x6881
CJK UNIFIED IDEOGRAPH:'B1E8:45544:&#x68AF
CJK UNIFIED IDEOGRAPH:'B1E9:45545:&#x68A2
CJK UNIFIED IDEOGRAPH:'B1EA:45546:&#x6893
CJK UNIFIED IDEOGRAPH:'B1EB:45547:&#x68B5
CJK UNIFIED IDEOGRAPH:'B1EC:45548:&#x687F
CJK UNIFIED IDEOGRAPH:'B1ED:45549:&#x6876
CJK UNIFIED IDEOGRAPH:'B1EE:45550:&#x68B1
CJK UNIFIED IDEOGRAPH:'B1EF:45551:&#x68A7
CJK UNIFIED IDEOGRAPH:'B1F0:45552:&#x6897
CJK UNIFIED IDEOGRAPH:'B1F1:45553:&#x68B0
CJK UNIFIED IDEOGRAPH:'B1F2:45554:&#x6883
CJK UNIFIED IDEOGRAPH:'B1F3:45555:&#x68C4
CJK UNIFIED IDEOGRAPH:'B1F4:45556:&#x68AD
CJK UNIFIED IDEOGRAPH:'B1F5:45557:&#x6886
CJK UNIFIED IDEOGRAPH:'B1F6:45558:&#x6885
CJK UNIFIED IDEOGRAPH:'B1F7:45559:&#x6894
CJK UNIFIED IDEOGRAPH:'B1F8:45560:&#x689D
CJK UNIFIED IDEOGRAPH:'B1F9:45561:&#x68A8
CJK UNIFIED IDEOGRAPH:'B1FA:45562:&#x689F
CJK UNIFIED IDEOGRAPH:'B1FB:45563:&#x68A1
CJK UNIFIED IDEOGRAPH:'B1FC:45564:&#x6882
CJK UNIFIED IDEOGRAPH:'B1FD:45565:&#x6B32
CJK UNIFIED IDEOGRAPH:'B1FE:45566:&#x6BBA
CJK UNIFIED IDEOGRAPH:'B240:45632:&#x6BEB
CJK UNIFIED IDEOGRAPH:'B241:45633:&#x6BEC
CJK UNIFIED IDEOGRAPH:'B242:45634:&#x6C2B
CJK UNIFIED IDEOGRAPH:'B243:45635:&#x6D8E
CJK UNIFIED IDEOGRAPH:'B244:45636:&#x6DBC
CJK UNIFIED IDEOGRAPH:'B245:45637:&#x6DF3
CJK UNIFIED IDEOGRAPH:'B246:45638:&#x6DD9
CJK UNIFIED IDEOGRAPH:'B247:45639:&#x6DB2
CJK UNIFIED IDEOGRAPH:'B248:45640:&#x6DE1
CJK UNIFIED IDEOGRAPH:'B249:45641:&#x6DCC
CJK UNIFIED IDEOGRAPH:'B24A:45642:&#x6DE4
CJK UNIFIED IDEOGRAPH:'B24B:45643:&#x6DFB
CJK UNIFIED IDEOGRAPH:'B24C:45644:&#x6DFA
CJK UNIFIED IDEOGRAPH:'B24D:45645:&#x6E05
CJK UNIFIED IDEOGRAPH:'B24E:45646:&#x6DC7
CJK UNIFIED IDEOGRAPH:'B24F:45647:&#x6DCB
CJK UNIFIED IDEOGRAPH:'B250:45648:&#x6DAF
CJK UNIFIED IDEOGRAPH:'B251:45649:&#x6DD1
CJK UNIFIED IDEOGRAPH:'B252:45650:&#x6DAE
CJK UNIFIED IDEOGRAPH:'B253:45651:&#x6DDE
CJK UNIFIED IDEOGRAPH:'B254:45652:&#x6DF9
CJK UNIFIED IDEOGRAPH:'B255:45653:&#x6DB8
CJK UNIFIED IDEOGRAPH:'B256:45654:&#x6DF7
CJK UNIFIED IDEOGRAPH:'B257:45655:&#x6DF5
CJK UNIFIED IDEOGRAPH:'B258:45656:&#x6DC5
CJK UNIFIED IDEOGRAPH:'B259:45657:&#x6DD2
CJK UNIFIED IDEOGRAPH:'B25A:45658:&#x6E1A
CJK UNIFIED IDEOGRAPH:'B25B:45659:&#x6DB5
CJK UNIFIED IDEOGRAPH:'B25C:45660:&#x6DDA
CJK UNIFIED IDEOGRAPH:'B25D:45661:&#x6DEB
CJK UNIFIED IDEOGRAPH:'B25E:45662:&#x6DD8
CJK UNIFIED IDEOGRAPH:'B25F:45663:&#x6DEA
CJK UNIFIED IDEOGRAPH:'B260:45664:&#x6DF1
CJK UNIFIED IDEOGRAPH:'B261:45665:&#x6DEE
CJK UNIFIED IDEOGRAPH:'B262:45666:&#x6DE8
CJK UNIFIED IDEOGRAPH:'B263:45667:&#x6DC6
CJK UNIFIED IDEOGRAPH:'B264:45668:&#x6DC4
CJK UNIFIED IDEOGRAPH:'B265:45669:&#x6DAA
CJK UNIFIED IDEOGRAPH:'B266:45670:&#x6DEC
CJK UNIFIED IDEOGRAPH:'B267:45671:&#x6DBF
CJK UNIFIED IDEOGRAPH:'B268:45672:&#x6DE6
CJK UNIFIED IDEOGRAPH:'B269:45673:&#x70F9
CJK UNIFIED IDEOGRAPH:'B26A:45674:&#x7109
CJK UNIFIED IDEOGRAPH:'B26B:45675:&#x710A
CJK UNIFIED IDEOGRAPH:'B26C:45676:&#x70FD
CJK UNIFIED IDEOGRAPH:'B26D:45677:&#x70EF
CJK UNIFIED IDEOGRAPH:'B26E:45678:&#x723D
CJK UNIFIED IDEOGRAPH:'B26F:45679:&#x727D
CJK UNIFIED IDEOGRAPH:'B270:45680:&#x7281
CJK UNIFIED IDEOGRAPH:'B271:45681:&#x731C
CJK UNIFIED IDEOGRAPH:'B272:45682:&#x731B
CJK UNIFIED IDEOGRAPH:'B273:45683:&#x7316
CJK UNIFIED IDEOGRAPH:'B274:45684:&#x7313
CJK UNIFIED IDEOGRAPH:'B275:45685:&#x7319
CJK UNIFIED IDEOGRAPH:'B276:45686:&#x7387
CJK UNIFIED IDEOGRAPH:'B277:45687:&#x7405
CJK UNIFIED IDEOGRAPH:'B278:45688:&#x740A
CJK UNIFIED IDEOGRAPH:'B279:45689:&#x7403
CJK UNIFIED IDEOGRAPH:'B27A:45690:&#x7406
CJK UNIFIED IDEOGRAPH:'B27B:45691:&#x73FE
CJK UNIFIED IDEOGRAPH:'B27C:45692:&#x740D
CJK UNIFIED IDEOGRAPH:'B27D:45693:&#x74E0
CJK UNIFIED IDEOGRAPH:'B27E:45694:&#x74F6
CJK UNIFIED IDEOGRAPH:'B2A1:45729:&#x74F7
CJK UNIFIED IDEOGRAPH:'B2A2:45730:&#x751C
CJK UNIFIED IDEOGRAPH:'B2A3:45731:&#x7522
CJK UNIFIED IDEOGRAPH:'B2A4:45732:&#x7565
CJK UNIFIED IDEOGRAPH:'B2A5:45733:&#x7566
CJK UNIFIED IDEOGRAPH:'B2A6:45734:&#x7562
CJK UNIFIED IDEOGRAPH:'B2A7:45735:&#x7570
CJK UNIFIED IDEOGRAPH:'B2A8:45736:&#x758F
CJK UNIFIED IDEOGRAPH:'B2A9:45737:&#x75D4
CJK UNIFIED IDEOGRAPH:'B2AA:45738:&#x75D5
CJK UNIFIED IDEOGRAPH:'B2AB:45739:&#x75B5
CJK UNIFIED IDEOGRAPH:'B2AC:45740:&#x75CA
CJK UNIFIED IDEOGRAPH:'B2AD:45741:&#x75CD
CJK UNIFIED IDEOGRAPH:'B2AE:45742:&#x768E
CJK UNIFIED IDEOGRAPH:'B2AF:45743:&#x76D4
CJK UNIFIED IDEOGRAPH:'B2B0:45744:&#x76D2
CJK UNIFIED IDEOGRAPH:'B2B1:45745:&#x76DB
CJK UNIFIED IDEOGRAPH:'B2B2:45746:&#x7737
CJK UNIFIED IDEOGRAPH:'B2B3:45747:&#x773E
CJK UNIFIED IDEOGRAPH:'B2B4:45748:&#x773C
CJK UNIFIED IDEOGRAPH:'B2B5:45749:&#x7736
CJK UNIFIED IDEOGRAPH:'B2B6:45750:&#x7738
CJK UNIFIED IDEOGRAPH:'B2B7:45751:&#x773A
CJK UNIFIED IDEOGRAPH:'B2B8:45752:&#x786B
CJK UNIFIED IDEOGRAPH:'B2B9:45753:&#x7843
CJK UNIFIED IDEOGRAPH:'B2BA:45754:&#x784E
CJK UNIFIED IDEOGRAPH:'B2BB:45755:&#x7965
CJK UNIFIED IDEOGRAPH:'B2BC:45756:&#x7968
CJK UNIFIED IDEOGRAPH:'B2BD:45757:&#x796D
CJK UNIFIED IDEOGRAPH:'B2BE:45758:&#x79FB
CJK UNIFIED IDEOGRAPH:'B2BF:45759:&#x7A92
CJK UNIFIED IDEOGRAPH:'B2C0:45760:&#x7A95
CJK UNIFIED IDEOGRAPH:'B2C1:45761:&#x7B20
CJK UNIFIED IDEOGRAPH:'B2C2:45762:&#x7B28
CJK UNIFIED IDEOGRAPH:'B2C3:45763:&#x7B1B
CJK UNIFIED IDEOGRAPH:'B2C4:45764:&#x7B2C
CJK UNIFIED IDEOGRAPH:'B2C5:45765:&#x7B26
CJK UNIFIED IDEOGRAPH:'B2C6:45766:&#x7B19
CJK UNIFIED IDEOGRAPH:'B2C7:45767:&#x7B1E
CJK UNIFIED IDEOGRAPH:'B2C8:45768:&#x7B2E
CJK UNIFIED IDEOGRAPH:'B2C9:45769:&#x7C92
CJK UNIFIED IDEOGRAPH:'B2CA:45770:&#x7C97
CJK UNIFIED IDEOGRAPH:'B2CB:45771:&#x7C95
CJK UNIFIED IDEOGRAPH:'B2CC:45772:&#x7D46
CJK UNIFIED IDEOGRAPH:'B2CD:45773:&#x7D43
CJK UNIFIED IDEOGRAPH:'B2CE:45774:&#x7D71
CJK UNIFIED IDEOGRAPH:'B2CF:45775:&#x7D2E
CJK UNIFIED IDEOGRAPH:'B2D0:45776:&#x7D39
CJK UNIFIED IDEOGRAPH:'B2D1:45777:&#x7D3C
CJK UNIFIED IDEOGRAPH:'B2D2:45778:&#x7D40
CJK UNIFIED IDEOGRAPH:'B2D3:45779:&#x7D30
CJK UNIFIED IDEOGRAPH:'B2D4:45780:&#x7D33
CJK UNIFIED IDEOGRAPH:'B2D5:45781:&#x7D44
CJK UNIFIED IDEOGRAPH:'B2D6:45782:&#x7D2F
CJK UNIFIED IDEOGRAPH:'B2D7:45783:&#x7D42
CJK UNIFIED IDEOGRAPH:'B2D8:45784:&#x7D32
CJK UNIFIED IDEOGRAPH:'B2D9:45785:&#x7D31
CJK UNIFIED IDEOGRAPH:'B2DA:45786:&#x7F3D
CJK UNIFIED IDEOGRAPH:'B2DB:45787:&#x7F9E
CJK UNIFIED IDEOGRAPH:'B2DC:45788:&#x7F9A
CJK UNIFIED IDEOGRAPH:'B2DD:45789:&#x7FCC
CJK UNIFIED IDEOGRAPH:'B2DE:45790:&#x7FCE
CJK UNIFIED IDEOGRAPH:'B2DF:45791:&#x7FD2
CJK UNIFIED IDEOGRAPH:'B2E0:45792:&#x801C
CJK UNIFIED IDEOGRAPH:'B2E1:45793:&#x804A
CJK UNIFIED IDEOGRAPH:'B2E2:45794:&#x8046
CJK UNIFIED IDEOGRAPH:'B2E3:45795:&#x812F
CJK UNIFIED IDEOGRAPH:'B2E4:45796:&#x8116
CJK UNIFIED IDEOGRAPH:'B2E5:45797:&#x8123
CJK UNIFIED IDEOGRAPH:'B2E6:45798:&#x812B
CJK UNIFIED IDEOGRAPH:'B2E7:45799:&#x8129
CJK UNIFIED IDEOGRAPH:'B2E8:45800:&#x8130
CJK UNIFIED IDEOGRAPH:'B2E9:45801:&#x8124
CJK UNIFIED IDEOGRAPH:'B2EA:45802:&#x8202
CJK UNIFIED IDEOGRAPH:'B2EB:45803:&#x8235
CJK UNIFIED IDEOGRAPH:'B2EC:45804:&#x8237
CJK UNIFIED IDEOGRAPH:'B2ED:45805:&#x8236
CJK UNIFIED IDEOGRAPH:'B2EE:45806:&#x8239
CJK UNIFIED IDEOGRAPH:'B2EF:45807:&#x838E
CJK UNIFIED IDEOGRAPH:'B2F0:45808:&#x839E
CJK UNIFIED IDEOGRAPH:'B2F1:45809:&#x8398
CJK UNIFIED IDEOGRAPH:'B2F2:45810:&#x8378
CJK UNIFIED IDEOGRAPH:'B2F3:45811:&#x83A2
CJK UNIFIED IDEOGRAPH:'B2F4:45812:&#x8396
CJK UNIFIED IDEOGRAPH:'B2F5:45813:&#x83BD
CJK UNIFIED IDEOGRAPH:'B2F6:45814:&#x83AB
CJK UNIFIED IDEOGRAPH:'B2F7:45815:&#x8392
CJK UNIFIED IDEOGRAPH:'B2F8:45816:&#x838A
CJK UNIFIED IDEOGRAPH:'B2F9:45817:&#x8393
CJK UNIFIED IDEOGRAPH:'B2FA:45818:&#x8389
CJK UNIFIED IDEOGRAPH:'B2FB:45819:&#x83A0
CJK UNIFIED IDEOGRAPH:'B2FC:45820:&#x8377
CJK UNIFIED IDEOGRAPH:'B2FD:45821:&#x837B
CJK UNIFIED IDEOGRAPH:'B2FE:45822:&#x837C
CJK UNIFIED IDEOGRAPH:'B340:45888:&#x8386
CJK UNIFIED IDEOGRAPH:'B341:45889:&#x83A7
CJK UNIFIED IDEOGRAPH:'B342:45890:&#x8655
CJK UNIFIED IDEOGRAPH:'B343:45891:&#x5F6A
CJK UNIFIED IDEOGRAPH:'B344:45892:&#x86C7
CJK UNIFIED IDEOGRAPH:'B345:45893:&#x86C0
CJK UNIFIED IDEOGRAPH:'B346:45894:&#x86B6
CJK UNIFIED IDEOGRAPH:'B347:45895:&#x86C4
CJK UNIFIED IDEOGRAPH:'B348:45896:&#x86B5
CJK UNIFIED IDEOGRAPH:'B349:45897:&#x86C6
CJK UNIFIED IDEOGRAPH:'B34A:45898:&#x86CB
CJK UNIFIED IDEOGRAPH:'B34B:45899:&#x86B1
CJK UNIFIED IDEOGRAPH:'B34C:45900:&#x86AF
CJK UNIFIED IDEOGRAPH:'B34D:45901:&#x86C9
CJK UNIFIED IDEOGRAPH:'B34E:45902:&#x8853
CJK UNIFIED IDEOGRAPH:'B34F:45903:&#x889E
CJK UNIFIED IDEOGRAPH:'B350:45904:&#x8888
CJK UNIFIED IDEOGRAPH:'B351:45905:&#x88AB
CJK UNIFIED IDEOGRAPH:'B352:45906:&#x8892
CJK UNIFIED IDEOGRAPH:'B353:45907:&#x8896
CJK UNIFIED IDEOGRAPH:'B354:45908:&#x888D
CJK UNIFIED IDEOGRAPH:'B355:45909:&#x888B
CJK UNIFIED IDEOGRAPH:'B356:45910:&#x8993
CJK UNIFIED IDEOGRAPH:'B357:45911:&#x898F
CJK UNIFIED IDEOGRAPH:'B358:45912:&#x8A2A
CJK UNIFIED IDEOGRAPH:'B359:45913:&#x8A1D
CJK UNIFIED IDEOGRAPH:'B35A:45914:&#x8A23
CJK UNIFIED IDEOGRAPH:'B35B:45915:&#x8A25
CJK UNIFIED IDEOGRAPH:'B35C:45916:&#x8A31
CJK UNIFIED IDEOGRAPH:'B35D:45917:&#x8A2D
CJK UNIFIED IDEOGRAPH:'B35E:45918:&#x8A1F
CJK UNIFIED IDEOGRAPH:'B35F:45919:&#x8A1B
CJK UNIFIED IDEOGRAPH:'B360:45920:&#x8A22
CJK UNIFIED IDEOGRAPH:'B361:45921:&#x8C49
CJK UNIFIED IDEOGRAPH:'B362:45922:&#x8C5A
CJK UNIFIED IDEOGRAPH:'B363:45923:&#x8CA9
CJK UNIFIED IDEOGRAPH:'B364:45924:&#x8CAC
CJK UNIFIED IDEOGRAPH:'B365:45925:&#x8CAB
CJK UNIFIED IDEOGRAPH:'B366:45926:&#x8CA8
CJK UNIFIED IDEOGRAPH:'B367:45927:&#x8CAA
CJK UNIFIED IDEOGRAPH:'B368:45928:&#x8CA7
CJK UNIFIED IDEOGRAPH:'B369:45929:&#x8D67
CJK UNIFIED IDEOGRAPH:'B36A:45930:&#x8D66
CJK UNIFIED IDEOGRAPH:'B36B:45931:&#x8DBE
CJK UNIFIED IDEOGRAPH:'B36C:45932:&#x8DBA
CJK UNIFIED IDEOGRAPH:'B36D:45933:&#x8EDB
CJK UNIFIED IDEOGRAPH:'B36E:45934:&#x8EDF
CJK UNIFIED IDEOGRAPH:'B36F:45935:&#x9019
CJK UNIFIED IDEOGRAPH:'B370:45936:&#x900D
CJK UNIFIED IDEOGRAPH:'B371:45937:&#x901A
CJK UNIFIED IDEOGRAPH:'B372:45938:&#x9017
CJK UNIFIED IDEOGRAPH:'B373:45939:&#x9023
CJK UNIFIED IDEOGRAPH:'B374:45940:&#x901F
CJK UNIFIED IDEOGRAPH:'B375:45941:&#x901D
CJK UNIFIED IDEOGRAPH:'B376:45942:&#x9010
CJK UNIFIED IDEOGRAPH:'B377:45943:&#x9015
CJK UNIFIED IDEOGRAPH:'B378:45944:&#x901E
CJK UNIFIED IDEOGRAPH:'B379:45945:&#x9020
CJK UNIFIED IDEOGRAPH:'B37A:45946:&#x900F
CJK UNIFIED IDEOGRAPH:'B37B:45947:&#x9022
CJK UNIFIED IDEOGRAPH:'B37C:45948:&#x9016
CJK UNIFIED IDEOGRAPH:'B37D:45949:&#x901B
CJK UNIFIED IDEOGRAPH:'B37E:45950:&#x9014
CJK UNIFIED IDEOGRAPH:'B3A1:45985:&#x90E8
CJK UNIFIED IDEOGRAPH:'B3A2:45986:&#x90ED
CJK UNIFIED IDEOGRAPH:'B3A3:45987:&#x90FD
CJK UNIFIED IDEOGRAPH:'B3A4:45988:&#x9157
CJK UNIFIED IDEOGRAPH:'B3A5:45989:&#x91CE
CJK UNIFIED IDEOGRAPH:'B3A6:45990:&#x91F5
CJK UNIFIED IDEOGRAPH:'B3A7:45991:&#x91E6
CJK UNIFIED IDEOGRAPH:'B3A8:45992:&#x91E3
CJK UNIFIED IDEOGRAPH:'B3A9:45993:&#x91E7
CJK UNIFIED IDEOGRAPH:'B3AA:45994:&#x91ED
CJK UNIFIED IDEOGRAPH:'B3AB:45995:&#x91E9
CJK UNIFIED IDEOGRAPH:'B3AC:45996:&#x9589
CJK UNIFIED IDEOGRAPH:'B3AD:45997:&#x966A
CJK UNIFIED IDEOGRAPH:'B3AE:45998:&#x9675
CJK UNIFIED IDEOGRAPH:'B3AF:45999:&#x9673
CJK UNIFIED IDEOGRAPH:'B3B0:46000:&#x9678
CJK UNIFIED IDEOGRAPH:'B3B1:46001:&#x9670
CJK UNIFIED IDEOGRAPH:'B3B2:46002:&#x9674
CJK UNIFIED IDEOGRAPH:'B3B3:46003:&#x9676
CJK UNIFIED IDEOGRAPH:'B3B4:46004:&#x9677
CJK UNIFIED IDEOGRAPH:'B3B5:46005:&#x966C
CJK UNIFIED IDEOGRAPH:'B3B6:46006:&#x96C0
CJK UNIFIED IDEOGRAPH:'B3B7:46007:&#x96EA
CJK UNIFIED IDEOGRAPH:'B3B8:46008:&#x96E9
CJK UNIFIED IDEOGRAPH:'B3B9:46009:&#x7AE0
CJK UNIFIED IDEOGRAPH:'B3BA:46010:&#x7ADF
CJK UNIFIED IDEOGRAPH:'B3BB:46011:&#x9802
CJK UNIFIED IDEOGRAPH:'B3BC:46012:&#x9803
CJK UNIFIED IDEOGRAPH:'B3BD:46013:&#x9B5A
CJK UNIFIED IDEOGRAPH:'B3BE:46014:&#x9CE5
CJK UNIFIED IDEOGRAPH:'B3BF:46015:&#x9E75
CJK UNIFIED IDEOGRAPH:'B3C0:46016:&#x9E7F
CJK UNIFIED IDEOGRAPH:'B3C1:46017:&#x9EA5
CJK UNIFIED IDEOGRAPH:'B3C2:46018:&#x9EBB
CJK UNIFIED IDEOGRAPH:'B3C3:46019:&#x50A2
CJK UNIFIED IDEOGRAPH:'B3C4:46020:&#x508D
CJK UNIFIED IDEOGRAPH:'B3C5:46021:&#x5085
CJK UNIFIED IDEOGRAPH:'B3C6:46022:&#x5099
CJK UNIFIED IDEOGRAPH:'B3C7:46023:&#x5091
CJK UNIFIED IDEOGRAPH:'B3C8:46024:&#x5080
CJK UNIFIED IDEOGRAPH:'B3C9:46025:&#x5096
CJK UNIFIED IDEOGRAPH:'B3CA:46026:&#x5098
CJK UNIFIED IDEOGRAPH:'B3CB:46027:&#x509A
CJK UNIFIED IDEOGRAPH:'B3CC:46028:&#x6700
CJK UNIFIED IDEOGRAPH:'B3CD:46029:&#x51F1
CJK UNIFIED IDEOGRAPH:'B3CE:46030:&#x5272
CJK UNIFIED IDEOGRAPH:'B3CF:46031:&#x5274
CJK UNIFIED IDEOGRAPH:'B3D0:46032:&#x5275
CJK UNIFIED IDEOGRAPH:'B3D1:46033:&#x5269
CJK UNIFIED IDEOGRAPH:'B3D2:46034:&#x52DE
CJK UNIFIED IDEOGRAPH:'B3D3:46035:&#x52DD
CJK UNIFIED IDEOGRAPH:'B3D4:46036:&#x52DB
CJK UNIFIED IDEOGRAPH:'B3D5:46037:&#x535A
CJK UNIFIED IDEOGRAPH:'B3D6:46038:&#x53A5
CJK UNIFIED IDEOGRAPH:'B3D7:46039:&#x557B
CJK UNIFIED IDEOGRAPH:'B3D8:46040:&#x5580
CJK UNIFIED IDEOGRAPH:'B3D9:46041:&#x55A7
CJK UNIFIED IDEOGRAPH:'B3DA:46042:&#x557C
CJK UNIFIED IDEOGRAPH:'B3DB:46043:&#x558A
CJK UNIFIED IDEOGRAPH:'B3DC:46044:&#x559D
CJK UNIFIED IDEOGRAPH:'B3DD:46045:&#x5598
CJK UNIFIED IDEOGRAPH:'B3DE:46046:&#x5582
CJK UNIFIED IDEOGRAPH:'B3DF:46047:&#x559C
CJK UNIFIED IDEOGRAPH:'B3E0:46048:&#x55AA
CJK UNIFIED IDEOGRAPH:'B3E1:46049:&#x5594
CJK UNIFIED IDEOGRAPH:'B3E2:46050:&#x5587
CJK UNIFIED IDEOGRAPH:'B3E3:46051:&#x558B
CJK UNIFIED IDEOGRAPH:'B3E4:46052:&#x5583
CJK UNIFIED IDEOGRAPH:'B3E5:46053:&#x55B3
CJK UNIFIED IDEOGRAPH:'B3E6:46054:&#x55AE
CJK UNIFIED IDEOGRAPH:'B3E7:46055:&#x559F
CJK UNIFIED IDEOGRAPH:'B3E8:46056:&#x553E
CJK UNIFIED IDEOGRAPH:'B3E9:46057:&#x55B2
CJK UNIFIED IDEOGRAPH:'B3EA:46058:&#x559A
CJK UNIFIED IDEOGRAPH:'B3EB:46059:&#x55BB
CJK UNIFIED IDEOGRAPH:'B3EC:46060:&#x55AC
CJK UNIFIED IDEOGRAPH:'B3ED:46061:&#x55B1
CJK UNIFIED IDEOGRAPH:'B3EE:46062:&#x557E
CJK UNIFIED IDEOGRAPH:'B3EF:46063:&#x5589
CJK UNIFIED IDEOGRAPH:'B3F0:46064:&#x55AB
CJK UNIFIED IDEOGRAPH:'B3F1:46065:&#x5599
CJK UNIFIED IDEOGRAPH:'B3F2:46066:&#x570D
CJK UNIFIED IDEOGRAPH:'B3F3:46067:&#x582F
CJK UNIFIED IDEOGRAPH:'B3F4:46068:&#x582A
CJK UNIFIED IDEOGRAPH:'B3F5:46069:&#x5834
CJK UNIFIED IDEOGRAPH:'B3F6:46070:&#x5824
CJK UNIFIED IDEOGRAPH:'B3F7:46071:&#x5830
CJK UNIFIED IDEOGRAPH:'B3F8:46072:&#x5831
CJK UNIFIED IDEOGRAPH:'B3F9:46073:&#x5821
CJK UNIFIED IDEOGRAPH:'B3FA:46074:&#x581D
CJK UNIFIED IDEOGRAPH:'B3FB:46075:&#x5820
CJK UNIFIED IDEOGRAPH:'B3FC:46076:&#x58F9
CJK UNIFIED IDEOGRAPH:'B3FD:46077:&#x58FA
CJK UNIFIED IDEOGRAPH:'B3FE:46078:&#x5960
CJK UNIFIED IDEOGRAPH:'B440:46144:&#x5A77
CJK UNIFIED IDEOGRAPH:'B441:46145:&#x5A9A
CJK UNIFIED IDEOGRAPH:'B442:46146:&#x5A7F
CJK UNIFIED IDEOGRAPH:'B443:46147:&#x5A92
CJK UNIFIED IDEOGRAPH:'B444:46148:&#x5A9B
CJK UNIFIED IDEOGRAPH:'B445:46149:&#x5AA7
CJK UNIFIED IDEOGRAPH:'B446:46150:&#x5B73
CJK UNIFIED IDEOGRAPH:'B447:46151:&#x5B71
CJK UNIFIED IDEOGRAPH:'B448:46152:&#x5BD2
CJK UNIFIED IDEOGRAPH:'B449:46153:&#x5BCC
CJK UNIFIED IDEOGRAPH:'B44A:46154:&#x5BD3
CJK UNIFIED IDEOGRAPH:'B44B:46155:&#x5BD0
CJK UNIFIED IDEOGRAPH:'B44C:46156:&#x5C0A
CJK UNIFIED IDEOGRAPH:'B44D:46157:&#x5C0B
CJK UNIFIED IDEOGRAPH:'B44E:46158:&#x5C31
CJK UNIFIED IDEOGRAPH:'B44F:46159:&#x5D4C
CJK UNIFIED IDEOGRAPH:'B450:46160:&#x5D50
CJK UNIFIED IDEOGRAPH:'B451:46161:&#x5D34
CJK UNIFIED IDEOGRAPH:'B452:46162:&#x5D47
CJK UNIFIED IDEOGRAPH:'B453:46163:&#x5DFD
CJK UNIFIED IDEOGRAPH:'B454:46164:&#x5E45
CJK UNIFIED IDEOGRAPH:'B455:46165:&#x5E3D
CJK UNIFIED IDEOGRAPH:'B456:46166:&#x5E40
CJK UNIFIED IDEOGRAPH:'B457:46167:&#x5E43
CJK UNIFIED IDEOGRAPH:'B458:46168:&#x5E7E
CJK UNIFIED IDEOGRAPH:'B459:46169:&#x5ECA
CJK UNIFIED IDEOGRAPH:'B45A:46170:&#x5EC1
CJK UNIFIED IDEOGRAPH:'B45B:46171:&#x5EC2
CJK UNIFIED IDEOGRAPH:'B45C:46172:&#x5EC4
CJK UNIFIED IDEOGRAPH:'B45D:46173:&#x5F3C
CJK UNIFIED IDEOGRAPH:'B45E:46174:&#x5F6D
CJK UNIFIED IDEOGRAPH:'B45F:46175:&#x5FA9
CJK UNIFIED IDEOGRAPH:'B460:46176:&#x5FAA
CJK UNIFIED IDEOGRAPH:'B461:46177:&#x5FA8
CJK UNIFIED IDEOGRAPH:'B462:46178:&#x60D1
CJK UNIFIED IDEOGRAPH:'B463:46179:&#x60E1
CJK UNIFIED IDEOGRAPH:'B464:46180:&#x60B2
CJK UNIFIED IDEOGRAPH:'B465:46181:&#x60B6
CJK UNIFIED IDEOGRAPH:'B466:46182:&#x60E0
CJK UNIFIED IDEOGRAPH:'B467:46183:&#x611C
CJK UNIFIED IDEOGRAPH:'B468:46184:&#x6123
CJK UNIFIED IDEOGRAPH:'B469:46185:&#x60FA
CJK UNIFIED IDEOGRAPH:'B46A:46186:&#x6115
CJK UNIFIED IDEOGRAPH:'B46B:46187:&#x60F0
CJK UNIFIED IDEOGRAPH:'B46C:46188:&#x60FB
CJK UNIFIED IDEOGRAPH:'B46D:46189:&#x60F4
CJK UNIFIED IDEOGRAPH:'B46E:46190:&#x6168
CJK UNIFIED IDEOGRAPH:'B46F:46191:&#x60F1
CJK UNIFIED IDEOGRAPH:'B470:46192:&#x610E
CJK UNIFIED IDEOGRAPH:'B471:46193:&#x60F6
CJK UNIFIED IDEOGRAPH:'B472:46194:&#x6109
CJK UNIFIED IDEOGRAPH:'B473:46195:&#x6100
CJK UNIFIED IDEOGRAPH:'B474:46196:&#x6112
CJK UNIFIED IDEOGRAPH:'B475:46197:&#x621F
CJK UNIFIED IDEOGRAPH:'B476:46198:&#x6249
CJK UNIFIED IDEOGRAPH:'B477:46199:&#x63A3
CJK UNIFIED IDEOGRAPH:'B478:46200:&#x638C
CJK UNIFIED IDEOGRAPH:'B479:46201:&#x63CF
CJK UNIFIED IDEOGRAPH:'B47A:46202:&#x63C0
CJK UNIFIED IDEOGRAPH:'B47B:46203:&#x63E9
CJK UNIFIED IDEOGRAPH:'B47C:46204:&#x63C9
CJK UNIFIED IDEOGRAPH:'B47D:46205:&#x63C6
CJK UNIFIED IDEOGRAPH:'B47E:46206:&#x63CD
CJK UNIFIED IDEOGRAPH:'B4A1:46241:&#x63D2
CJK UNIFIED IDEOGRAPH:'B4A2:46242:&#x63E3
CJK UNIFIED IDEOGRAPH:'B4A3:46243:&#x63D0
CJK UNIFIED IDEOGRAPH:'B4A4:46244:&#x63E1
CJK UNIFIED IDEOGRAPH:'B4A5:46245:&#x63D6
CJK UNIFIED IDEOGRAPH:'B4A6:46246:&#x63ED
CJK UNIFIED IDEOGRAPH:'B4A7:46247:&#x63EE
CJK UNIFIED IDEOGRAPH:'B4A8:46248:&#x6376
CJK UNIFIED IDEOGRAPH:'B4A9:46249:&#x63F4
CJK UNIFIED IDEOGRAPH:'B4AA:46250:&#x63EA
CJK UNIFIED IDEOGRAPH:'B4AB:46251:&#x63DB
CJK UNIFIED IDEOGRAPH:'B4AC:46252:&#x6452
CJK UNIFIED IDEOGRAPH:'B4AD:46253:&#x63DA
CJK UNIFIED IDEOGRAPH:'B4AE:46254:&#x63F9
CJK UNIFIED IDEOGRAPH:'B4AF:46255:&#x655E
CJK UNIFIED IDEOGRAPH:'B4B0:46256:&#x6566
CJK UNIFIED IDEOGRAPH:'B4B1:46257:&#x6562
CJK UNIFIED IDEOGRAPH:'B4B2:46258:&#x6563
CJK UNIFIED IDEOGRAPH:'B4B3:46259:&#x6591
CJK UNIFIED IDEOGRAPH:'B4B4:46260:&#x6590
CJK UNIFIED IDEOGRAPH:'B4B5:46261:&#x65AF
CJK UNIFIED IDEOGRAPH:'B4B6:46262:&#x666E
CJK UNIFIED IDEOGRAPH:'B4B7:46263:&#x6670
CJK UNIFIED IDEOGRAPH:'B4B8:46264:&#x6674
CJK UNIFIED IDEOGRAPH:'B4B9:46265:&#x6676
CJK UNIFIED IDEOGRAPH:'B4BA:46266:&#x666F
CJK UNIFIED IDEOGRAPH:'B4BB:46267:&#x6691
CJK UNIFIED IDEOGRAPH:'B4BC:46268:&#x667A
CJK UNIFIED IDEOGRAPH:'B4BD:46269:&#x667E
CJK UNIFIED IDEOGRAPH:'B4BE:46270:&#x6677
CJK UNIFIED IDEOGRAPH:'B4BF:46271:&#x66FE
CJK UNIFIED IDEOGRAPH:'B4C0:46272:&#x66FF
CJK UNIFIED IDEOGRAPH:'B4C1:46273:&#x671F
CJK UNIFIED IDEOGRAPH:'B4C2:46274:&#x671D
CJK UNIFIED IDEOGRAPH:'B4C3:46275:&#x68FA
CJK UNIFIED IDEOGRAPH:'B4C4:46276:&#x68D5
CJK UNIFIED IDEOGRAPH:'B4C5:46277:&#x68E0
CJK UNIFIED IDEOGRAPH:'B4C6:46278:&#x68D8
CJK UNIFIED IDEOGRAPH:'B4C7:46279:&#x68D7
CJK UNIFIED IDEOGRAPH:'B4C8:46280:&#x6905
CJK UNIFIED IDEOGRAPH:'B4C9:46281:&#x68DF
CJK UNIFIED IDEOGRAPH:'B4CA:46282:&#x68F5
CJK UNIFIED IDEOGRAPH:'B4CB:46283:&#x68EE
CJK UNIFIED IDEOGRAPH:'B4CC:46284:&#x68E7
CJK UNIFIED IDEOGRAPH:'B4CD:46285:&#x68F9
CJK UNIFIED IDEOGRAPH:'B4CE:46286:&#x68D2
CJK UNIFIED IDEOGRAPH:'B4CF:46287:&#x68F2
CJK UNIFIED IDEOGRAPH:'B4D0:46288:&#x68E3
CJK UNIFIED IDEOGRAPH:'B4D1:46289:&#x68CB
CJK UNIFIED IDEOGRAPH:'B4D2:46290:&#x68CD
CJK UNIFIED IDEOGRAPH:'B4D3:46291:&#x690D
CJK UNIFIED IDEOGRAPH:'B4D4:46292:&#x6912
CJK UNIFIED IDEOGRAPH:'B4D5:46293:&#x690E
CJK UNIFIED IDEOGRAPH:'B4D6:46294:&#x68C9
CJK UNIFIED IDEOGRAPH:'B4D7:46295:&#x68DA
CJK UNIFIED IDEOGRAPH:'B4D8:46296:&#x696E
CJK UNIFIED IDEOGRAPH:'B4D9:46297:&#x68FB
CJK UNIFIED IDEOGRAPH:'B4DA:46298:&#x6B3E
CJK UNIFIED IDEOGRAPH:'B4DB:46299:&#x6B3A
CJK UNIFIED IDEOGRAPH:'B4DC:46300:&#x6B3D
CJK UNIFIED IDEOGRAPH:'B4DD:46301:&#x6B98
CJK UNIFIED IDEOGRAPH:'B4DE:46302:&#x6B96
CJK UNIFIED IDEOGRAPH:'B4DF:46303:&#x6BBC
CJK UNIFIED IDEOGRAPH:'B4E0:46304:&#x6BEF
CJK UNIFIED IDEOGRAPH:'B4E1:46305:&#x6C2E
CJK UNIFIED IDEOGRAPH:'B4E2:46306:&#x6C2F
CJK UNIFIED IDEOGRAPH:'B4E3:46307:&#x6C2C
CJK UNIFIED IDEOGRAPH:'B4E4:46308:&#x6E2F
CJK UNIFIED IDEOGRAPH:'B4E5:46309:&#x6E38
CJK UNIFIED IDEOGRAPH:'B4E6:46310:&#x6E54
CJK UNIFIED IDEOGRAPH:'B4E7:46311:&#x6E21
CJK UNIFIED IDEOGRAPH:'B4E8:46312:&#x6E32
CJK UNIFIED IDEOGRAPH:'B4E9:46313:&#x6E67
CJK UNIFIED IDEOGRAPH:'B4EA:46314:&#x6E4A
CJK UNIFIED IDEOGRAPH:'B4EB:46315:&#x6E20
CJK UNIFIED IDEOGRAPH:'B4EC:46316:&#x6E25
CJK UNIFIED IDEOGRAPH:'B4ED:46317:&#x6E23
CJK UNIFIED IDEOGRAPH:'B4EE:46318:&#x6E1B
CJK UNIFIED IDEOGRAPH:'B4EF:46319:&#x6E5B
CJK UNIFIED IDEOGRAPH:'B4F0:46320:&#x6E58
CJK UNIFIED IDEOGRAPH:'B4F1:46321:&#x6E24
CJK UNIFIED IDEOGRAPH:'B4F2:46322:&#x6E56
CJK UNIFIED IDEOGRAPH:'B4F3:46323:&#x6E6E
CJK UNIFIED IDEOGRAPH:'B4F4:46324:&#x6E2D
CJK UNIFIED IDEOGRAPH:'B4F5:46325:&#x6E26
CJK UNIFIED IDEOGRAPH:'B4F6:46326:&#x6E6F
CJK UNIFIED IDEOGRAPH:'B4F7:46327:&#x6E34
CJK UNIFIED IDEOGRAPH:'B4F8:46328:&#x6E4D
CJK UNIFIED IDEOGRAPH:'B4F9:46329:&#x6E3A
CJK UNIFIED IDEOGRAPH:'B4FA:46330:&#x6E2C
CJK UNIFIED IDEOGRAPH:'B4FB:46331:&#x6E43
CJK UNIFIED IDEOGRAPH:'B4FC:46332:&#x6E1D
CJK UNIFIED IDEOGRAPH:'B4FD:46333:&#x6E3E
CJK UNIFIED IDEOGRAPH:'B4FE:46334:&#x6ECB
CJK UNIFIED IDEOGRAPH:'B540:46400:&#x6E89
CJK UNIFIED IDEOGRAPH:'B541:46401:&#x6E19
CJK UNIFIED IDEOGRAPH:'B542:46402:&#x6E4E
CJK UNIFIED IDEOGRAPH:'B543:46403:&#x6E63
CJK UNIFIED IDEOGRAPH:'B544:46404:&#x6E44
CJK UNIFIED IDEOGRAPH:'B545:46405:&#x6E72
CJK UNIFIED IDEOGRAPH:'B546:46406:&#x6E69
CJK UNIFIED IDEOGRAPH:'B547:46407:&#x6E5F
CJK UNIFIED IDEOGRAPH:'B548:46408:&#x7119
CJK UNIFIED IDEOGRAPH:'B549:46409:&#x711A
CJK UNIFIED IDEOGRAPH:'B54A:46410:&#x7126
CJK UNIFIED IDEOGRAPH:'B54B:46411:&#x7130
CJK UNIFIED IDEOGRAPH:'B54C:46412:&#x7121
CJK UNIFIED IDEOGRAPH:'B54D:46413:&#x7136
CJK UNIFIED IDEOGRAPH:'B54E:46414:&#x716E
CJK UNIFIED IDEOGRAPH:'B54F:46415:&#x711C
CJK UNIFIED IDEOGRAPH:'B550:46416:&#x724C
CJK UNIFIED IDEOGRAPH:'B551:46417:&#x7284
CJK UNIFIED IDEOGRAPH:'B552:46418:&#x7280
CJK UNIFIED IDEOGRAPH:'B553:46419:&#x7336
CJK UNIFIED IDEOGRAPH:'B554:46420:&#x7325
CJK UNIFIED IDEOGRAPH:'B555:46421:&#x7334
CJK UNIFIED IDEOGRAPH:'B556:46422:&#x7329
CJK UNIFIED IDEOGRAPH:'B557:46423:&#x743A
CJK UNIFIED IDEOGRAPH:'B558:46424:&#x742A
CJK UNIFIED IDEOGRAPH:'B559:46425:&#x7433
CJK UNIFIED IDEOGRAPH:'B55A:46426:&#x7422
CJK UNIFIED IDEOGRAPH:'B55B:46427:&#x7425
CJK UNIFIED IDEOGRAPH:'B55C:46428:&#x7435
CJK UNIFIED IDEOGRAPH:'B55D:46429:&#x7436
CJK UNIFIED IDEOGRAPH:'B55E:46430:&#x7434
CJK UNIFIED IDEOGRAPH:'B55F:46431:&#x742F
CJK UNIFIED IDEOGRAPH:'B560:46432:&#x741B
CJK UNIFIED IDEOGRAPH:'B561:46433:&#x7426
CJK UNIFIED IDEOGRAPH:'B562:46434:&#x7428
CJK UNIFIED IDEOGRAPH:'B563:46435:&#x7525
CJK UNIFIED IDEOGRAPH:'B564:46436:&#x7526
CJK UNIFIED IDEOGRAPH:'B565:46437:&#x756B
CJK UNIFIED IDEOGRAPH:'B566:46438:&#x756A
CJK UNIFIED IDEOGRAPH:'B567:46439:&#x75E2
CJK UNIFIED IDEOGRAPH:'B568:46440:&#x75DB
CJK UNIFIED IDEOGRAPH:'B569:46441:&#x75E3
CJK UNIFIED IDEOGRAPH:'B56A:46442:&#x75D9
CJK UNIFIED IDEOGRAPH:'B56B:46443:&#x75D8
CJK UNIFIED IDEOGRAPH:'B56C:46444:&#x75DE
CJK UNIFIED IDEOGRAPH:'B56D:46445:&#x75E0
CJK UNIFIED IDEOGRAPH:'B56E:46446:&#x767B
CJK UNIFIED IDEOGRAPH:'B56F:46447:&#x767C
CJK UNIFIED IDEOGRAPH:'B570:46448:&#x7696
CJK UNIFIED IDEOGRAPH:'B571:46449:&#x7693
CJK UNIFIED IDEOGRAPH:'B572:46450:&#x76B4
CJK UNIFIED IDEOGRAPH:'B573:46451:&#x76DC
CJK UNIFIED IDEOGRAPH:'B574:46452:&#x774F
CJK UNIFIED IDEOGRAPH:'B575:46453:&#x77ED
CJK UNIFIED IDEOGRAPH:'B576:46454:&#x785D
CJK UNIFIED IDEOGRAPH:'B577:46455:&#x786C
CJK UNIFIED IDEOGRAPH:'B578:46456:&#x786F
CJK UNIFIED IDEOGRAPH:'B579:46457:&#x7A0D
CJK UNIFIED IDEOGRAPH:'B57A:46458:&#x7A08
CJK UNIFIED IDEOGRAPH:'B57B:46459:&#x7A0B
CJK UNIFIED IDEOGRAPH:'B57C:46460:&#x7A05
CJK UNIFIED IDEOGRAPH:'B57D:46461:&#x7A00
CJK UNIFIED IDEOGRAPH:'B57E:46462:&#x7A98
CJK UNIFIED IDEOGRAPH:'B5A1:46497:&#x7A97
CJK UNIFIED IDEOGRAPH:'B5A2:46498:&#x7A96
CJK UNIFIED IDEOGRAPH:'B5A3:46499:&#x7AE5
CJK UNIFIED IDEOGRAPH:'B5A4:46500:&#x7AE3
CJK UNIFIED IDEOGRAPH:'B5A5:46501:&#x7B49
CJK UNIFIED IDEOGRAPH:'B5A6:46502:&#x7B56
CJK UNIFIED IDEOGRAPH:'B5A7:46503:&#x7B46
CJK UNIFIED IDEOGRAPH:'B5A8:46504:&#x7B50
CJK UNIFIED IDEOGRAPH:'B5A9:46505:&#x7B52
CJK UNIFIED IDEOGRAPH:'B5AA:46506:&#x7B54
CJK UNIFIED IDEOGRAPH:'B5AB:46507:&#x7B4D
CJK UNIFIED IDEOGRAPH:'B5AC:46508:&#x7B4B
CJK UNIFIED IDEOGRAPH:'B5AD:46509:&#x7B4F
CJK UNIFIED IDEOGRAPH:'B5AE:46510:&#x7B51
CJK UNIFIED IDEOGRAPH:'B5AF:46511:&#x7C9F
CJK UNIFIED IDEOGRAPH:'B5B0:46512:&#x7CA5
CJK UNIFIED IDEOGRAPH:'B5B1:46513:&#x7D5E
CJK UNIFIED IDEOGRAPH:'B5B2:46514:&#x7D50
CJK UNIFIED IDEOGRAPH:'B5B3:46515:&#x7D68
CJK UNIFIED IDEOGRAPH:'B5B4:46516:&#x7D55
CJK UNIFIED IDEOGRAPH:'B5B5:46517:&#x7D2B
CJK UNIFIED IDEOGRAPH:'B5B6:46518:&#x7D6E
CJK UNIFIED IDEOGRAPH:'B5B7:46519:&#x7D72
CJK UNIFIED IDEOGRAPH:'B5B8:46520:&#x7D61
CJK UNIFIED IDEOGRAPH:'B5B9:46521:&#x7D66
CJK UNIFIED IDEOGRAPH:'B5BA:46522:&#x7D62
CJK UNIFIED IDEOGRAPH:'B5BB:46523:&#x7D70
CJK UNIFIED IDEOGRAPH:'B5BC:46524:&#x7D73
CJK UNIFIED IDEOGRAPH:'B5BD:46525:&#x5584
CJK UNIFIED IDEOGRAPH:'B5BE:46526:&#x7FD4
CJK UNIFIED IDEOGRAPH:'B5BF:46527:&#x7FD5
CJK UNIFIED IDEOGRAPH:'B5C0:46528:&#x800B
CJK UNIFIED IDEOGRAPH:'B5C1:46529:&#x8052
CJK UNIFIED IDEOGRAPH:'B5C2:46530:&#x8085
CJK UNIFIED IDEOGRAPH:'B5C3:46531:&#x8155
CJK UNIFIED IDEOGRAPH:'B5C4:46532:&#x8154
CJK UNIFIED IDEOGRAPH:'B5C5:46533:&#x814B
CJK UNIFIED IDEOGRAPH:'B5C6:46534:&#x8151
CJK UNIFIED IDEOGRAPH:'B5C7:46535:&#x814E
CJK UNIFIED IDEOGRAPH:'B5C8:46536:&#x8139
CJK UNIFIED IDEOGRAPH:'B5C9:46537:&#x8146
CJK UNIFIED IDEOGRAPH:'B5CA:46538:&#x813E
CJK UNIFIED IDEOGRAPH:'B5CB:46539:&#x814C
CJK UNIFIED IDEOGRAPH:'B5CC:46540:&#x8153
CJK UNIFIED IDEOGRAPH:'B5CD:46541:&#x8174
CJK UNIFIED IDEOGRAPH:'B5CE:46542:&#x8212
CJK UNIFIED IDEOGRAPH:'B5CF:46543:&#x821C
CJK UNIFIED IDEOGRAPH:'B5D0:46544:&#x83E9
CJK UNIFIED IDEOGRAPH:'B5D1:46545:&#x8403
CJK UNIFIED IDEOGRAPH:'B5D2:46546:&#x83F8
CJK UNIFIED IDEOGRAPH:'B5D3:46547:&#x840D
CJK UNIFIED IDEOGRAPH:'B5D4:46548:&#x83E0
CJK UNIFIED IDEOGRAPH:'B5D5:46549:&#x83C5
CJK UNIFIED IDEOGRAPH:'B5D6:46550:&#x840B
CJK UNIFIED IDEOGRAPH:'B5D7:46551:&#x83C1
CJK UNIFIED IDEOGRAPH:'B5D8:46552:&#x83EF
CJK UNIFIED IDEOGRAPH:'B5D9:46553:&#x83F1
CJK UNIFIED IDEOGRAPH:'B5DA:46554:&#x83F4
CJK UNIFIED IDEOGRAPH:'B5DB:46555:&#x8457
CJK UNIFIED IDEOGRAPH:'B5DC:46556:&#x840A
CJK UNIFIED IDEOGRAPH:'B5DD:46557:&#x83F0
CJK UNIFIED IDEOGRAPH:'B5DE:46558:&#x840C
CJK UNIFIED IDEOGRAPH:'B5DF:46559:&#x83CC
CJK UNIFIED IDEOGRAPH:'B5E0:46560:&#x83FD
CJK UNIFIED IDEOGRAPH:'B5E1:46561:&#x83F2
CJK UNIFIED IDEOGRAPH:'B5E2:46562:&#x83CA
CJK UNIFIED IDEOGRAPH:'B5E3:46563:&#x8438
CJK UNIFIED IDEOGRAPH:'B5E4:46564:&#x840E
CJK UNIFIED IDEOGRAPH:'B5E5:46565:&#x8404
CJK UNIFIED IDEOGRAPH:'B5E6:46566:&#x83DC
CJK UNIFIED IDEOGRAPH:'B5E7:46567:&#x8407
CJK UNIFIED IDEOGRAPH:'B5E8:46568:&#x83D4
CJK UNIFIED IDEOGRAPH:'B5E9:46569:&#x83DF
CJK UNIFIED IDEOGRAPH:'B5EA:46570:&#x865B
CJK UNIFIED IDEOGRAPH:'B5EB:46571:&#x86DF
CJK UNIFIED IDEOGRAPH:'B5EC:46572:&#x86D9
CJK UNIFIED IDEOGRAPH:'B5ED:46573:&#x86ED
CJK UNIFIED IDEOGRAPH:'B5EE:46574:&#x86D4
CJK UNIFIED IDEOGRAPH:'B5EF:46575:&#x86DB
CJK UNIFIED IDEOGRAPH:'B5F0:46576:&#x86E4
CJK UNIFIED IDEOGRAPH:'B5F1:46577:&#x86D0
CJK UNIFIED IDEOGRAPH:'B5F2:46578:&#x86DE
CJK UNIFIED IDEOGRAPH:'B5F3:46579:&#x8857
CJK UNIFIED IDEOGRAPH:'B5F4:46580:&#x88C1
CJK UNIFIED IDEOGRAPH:'B5F5:46581:&#x88C2
CJK UNIFIED IDEOGRAPH:'B5F6:46582:&#x88B1
CJK UNIFIED IDEOGRAPH:'B5F7:46583:&#x8983
CJK UNIFIED IDEOGRAPH:'B5F8:46584:&#x8996
CJK UNIFIED IDEOGRAPH:'B5F9:46585:&#x8A3B
CJK UNIFIED IDEOGRAPH:'B5FA:46586:&#x8A60
CJK UNIFIED IDEOGRAPH:'B5FB:46587:&#x8A55
CJK UNIFIED IDEOGRAPH:'B5FC:46588:&#x8A5E
CJK UNIFIED IDEOGRAPH:'B5FD:46589:&#x8A3C
CJK UNIFIED IDEOGRAPH:'B5FE:46590:&#x8A41
CJK UNIFIED IDEOGRAPH:'B640:46656:&#x8A54
CJK UNIFIED IDEOGRAPH:'B641:46657:&#x8A5B
CJK UNIFIED IDEOGRAPH:'B642:46658:&#x8A50
CJK UNIFIED IDEOGRAPH:'B643:46659:&#x8A46
CJK UNIFIED IDEOGRAPH:'B644:46660:&#x8A34
CJK UNIFIED IDEOGRAPH:'B645:46661:&#x8A3A
CJK UNIFIED IDEOGRAPH:'B646:46662:&#x8A36
CJK UNIFIED IDEOGRAPH:'B647:46663:&#x8A56
CJK UNIFIED IDEOGRAPH:'B648:46664:&#x8C61
CJK UNIFIED IDEOGRAPH:'B649:46665:&#x8C82
CJK UNIFIED IDEOGRAPH:'B64A:46666:&#x8CAF
CJK UNIFIED IDEOGRAPH:'B64B:46667:&#x8CBC
CJK UNIFIED IDEOGRAPH:'B64C:46668:&#x8CB3
CJK UNIFIED IDEOGRAPH:'B64D:46669:&#x8CBD
CJK UNIFIED IDEOGRAPH:'B64E:46670:&#x8CC1
CJK UNIFIED IDEOGRAPH:'B64F:46671:&#x8CBB
CJK UNIFIED IDEOGRAPH:'B650:46672:&#x8CC0
CJK UNIFIED IDEOGRAPH:'B651:46673:&#x8CB4
CJK UNIFIED IDEOGRAPH:'B652:46674:&#x8CB7
CJK UNIFIED IDEOGRAPH:'B653:46675:&#x8CB6
CJK UNIFIED IDEOGRAPH:'B654:46676:&#x8CBF
CJK UNIFIED IDEOGRAPH:'B655:46677:&#x8CB8
CJK UNIFIED IDEOGRAPH:'B656:46678:&#x8D8A
CJK UNIFIED IDEOGRAPH:'B657:46679:&#x8D85
CJK UNIFIED IDEOGRAPH:'B658:46680:&#x8D81
CJK UNIFIED IDEOGRAPH:'B659:46681:&#x8DCE
CJK UNIFIED IDEOGRAPH:'B65A:46682:&#x8DDD
CJK UNIFIED IDEOGRAPH:'B65B:46683:&#x8DCB
CJK UNIFIED IDEOGRAPH:'B65C:46684:&#x8DDA
CJK UNIFIED IDEOGRAPH:'B65D:46685:&#x8DD1
CJK UNIFIED IDEOGRAPH:'B65E:46686:&#x8DCC
CJK UNIFIED IDEOGRAPH:'B65F:46687:&#x8DDB
CJK UNIFIED IDEOGRAPH:'B660:46688:&#x8DC6
CJK UNIFIED IDEOGRAPH:'B661:46689:&#x8EFB
CJK UNIFIED IDEOGRAPH:'B662:46690:&#x8EF8
CJK UNIFIED IDEOGRAPH:'B663:46691:&#x8EFC
CJK UNIFIED IDEOGRAPH:'B664:46692:&#x8F9C
CJK UNIFIED IDEOGRAPH:'B665:46693:&#x902E
CJK UNIFIED IDEOGRAPH:'B666:46694:&#x9035
CJK UNIFIED IDEOGRAPH:'B667:46695:&#x9031
CJK UNIFIED IDEOGRAPH:'B668:46696:&#x9038
CJK UNIFIED IDEOGRAPH:'B669:46697:&#x9032
CJK UNIFIED IDEOGRAPH:'B66A:46698:&#x9036
CJK UNIFIED IDEOGRAPH:'B66B:46699:&#x9102
CJK UNIFIED IDEOGRAPH:'B66C:46700:&#x90F5
CJK UNIFIED IDEOGRAPH:'B66D:46701:&#x9109
CJK UNIFIED IDEOGRAPH:'B66E:46702:&#x90FE
CJK UNIFIED IDEOGRAPH:'B66F:46703:&#x9163
CJK UNIFIED IDEOGRAPH:'B670:46704:&#x9165
CJK UNIFIED IDEOGRAPH:'B671:46705:&#x91CF
CJK UNIFIED IDEOGRAPH:'B672:46706:&#x9214
CJK UNIFIED IDEOGRAPH:'B673:46707:&#x9215
CJK UNIFIED IDEOGRAPH:'B674:46708:&#x9223
CJK UNIFIED IDEOGRAPH:'B675:46709:&#x9209
CJK UNIFIED IDEOGRAPH:'B676:46710:&#x921E
CJK UNIFIED IDEOGRAPH:'B677:46711:&#x920D
CJK UNIFIED IDEOGRAPH:'B678:46712:&#x9210
CJK UNIFIED IDEOGRAPH:'B679:46713:&#x9207
CJK UNIFIED IDEOGRAPH:'B67A:46714:&#x9211
CJK UNIFIED IDEOGRAPH:'B67B:46715:&#x9594
CJK UNIFIED IDEOGRAPH:'B67C:46716:&#x958F
CJK UNIFIED IDEOGRAPH:'B67D:46717:&#x958B
CJK UNIFIED IDEOGRAPH:'B67E:46718:&#x9591
CJK UNIFIED IDEOGRAPH:'B6A1:46753:&#x9593
CJK UNIFIED IDEOGRAPH:'B6A2:46754:&#x9592
CJK UNIFIED IDEOGRAPH:'B6A3:46755:&#x958E
CJK UNIFIED IDEOGRAPH:'B6A4:46756:&#x968A
CJK UNIFIED IDEOGRAPH:'B6A5:46757:&#x968E
CJK UNIFIED IDEOGRAPH:'B6A6:46758:&#x968B
CJK UNIFIED IDEOGRAPH:'B6A7:46759:&#x967D
CJK UNIFIED IDEOGRAPH:'B6A8:46760:&#x9685
CJK UNIFIED IDEOGRAPH:'B6A9:46761:&#x9686
CJK UNIFIED IDEOGRAPH:'B6AA:46762:&#x968D
CJK UNIFIED IDEOGRAPH:'B6AB:46763:&#x9672
CJK UNIFIED IDEOGRAPH:'B6AC:46764:&#x9684
CJK UNIFIED IDEOGRAPH:'B6AD:46765:&#x96C1
CJK UNIFIED IDEOGRAPH:'B6AE:46766:&#x96C5
CJK UNIFIED IDEOGRAPH:'B6AF:46767:&#x96C4
CJK UNIFIED IDEOGRAPH:'B6B0:46768:&#x96C6
CJK UNIFIED IDEOGRAPH:'B6B1:46769:&#x96C7
CJK UNIFIED IDEOGRAPH:'B6B2:46770:&#x96EF
CJK UNIFIED IDEOGRAPH:'B6B3:46771:&#x96F2
CJK UNIFIED IDEOGRAPH:'B6B4:46772:&#x97CC
CJK UNIFIED IDEOGRAPH:'B6B5:46773:&#x9805
CJK UNIFIED IDEOGRAPH:'B6B6:46774:&#x9806
CJK UNIFIED IDEOGRAPH:'B6B7:46775:&#x9808
CJK UNIFIED IDEOGRAPH:'B6B8:46776:&#x98E7
CJK UNIFIED IDEOGRAPH:'B6B9:46777:&#x98EA
CJK UNIFIED IDEOGRAPH:'B6BA:46778:&#x98EF
CJK UNIFIED IDEOGRAPH:'B6BB:46779:&#x98E9
CJK UNIFIED IDEOGRAPH:'B6BC:46780:&#x98F2
CJK UNIFIED IDEOGRAPH:'B6BD:46781:&#x98ED
CJK UNIFIED IDEOGRAPH:'B6BE:46782:&#x99AE
CJK UNIFIED IDEOGRAPH:'B6BF:46783:&#x99AD
CJK UNIFIED IDEOGRAPH:'B6C0:46784:&#x9EC3
CJK UNIFIED IDEOGRAPH:'B6C1:46785:&#x9ECD
CJK UNIFIED IDEOGRAPH:'B6C2:46786:&#x9ED1
CJK UNIFIED IDEOGRAPH:'B6C3:46787:&#x4E82
CJK UNIFIED IDEOGRAPH:'B6C4:46788:&#x50AD
CJK UNIFIED IDEOGRAPH:'B6C5:46789:&#x50B5
CJK UNIFIED IDEOGRAPH:'B6C6:46790:&#x50B2
CJK UNIFIED IDEOGRAPH:'B6C7:46791:&#x50B3
CJK UNIFIED IDEOGRAPH:'B6C8:46792:&#x50C5
CJK UNIFIED IDEOGRAPH:'B6C9:46793:&#x50BE
CJK UNIFIED IDEOGRAPH:'B6CA:46794:&#x50AC
CJK UNIFIED IDEOGRAPH:'B6CB:46795:&#x50B7
CJK UNIFIED IDEOGRAPH:'B6CC:46796:&#x50BB
CJK UNIFIED IDEOGRAPH:'B6CD:46797:&#x50AF
CJK UNIFIED IDEOGRAPH:'B6CE:46798:&#x50C7
CJK UNIFIED IDEOGRAPH:'B6CF:46799:&#x527F
CJK UNIFIED IDEOGRAPH:'B6D0:46800:&#x5277
CJK UNIFIED IDEOGRAPH:'B6D1:46801:&#x527D
CJK UNIFIED IDEOGRAPH:'B6D2:46802:&#x52DF
CJK UNIFIED IDEOGRAPH:'B6D3:46803:&#x52E6
CJK UNIFIED IDEOGRAPH:'B6D4:46804:&#x52E4
CJK UNIFIED IDEOGRAPH:'B6D5:46805:&#x52E2
CJK UNIFIED IDEOGRAPH:'B6D6:46806:&#x52E3
CJK UNIFIED IDEOGRAPH:'B6D7:46807:&#x532F
CJK UNIFIED IDEOGRAPH:'B6D8:46808:&#x55DF
CJK UNIFIED IDEOGRAPH:'B6D9:46809:&#x55E8
CJK UNIFIED IDEOGRAPH:'B6DA:46810:&#x55D3
CJK UNIFIED IDEOGRAPH:'B6DB:46811:&#x55E6
CJK UNIFIED IDEOGRAPH:'B6DC:46812:&#x55CE
CJK UNIFIED IDEOGRAPH:'B6DD:46813:&#x55DC
CJK UNIFIED IDEOGRAPH:'B6DE:46814:&#x55C7
CJK UNIFIED IDEOGRAPH:'B6DF:46815:&#x55D1
CJK UNIFIED IDEOGRAPH:'B6E0:46816:&#x55E3
CJK UNIFIED IDEOGRAPH:'B6E1:46817:&#x55E4
CJK UNIFIED IDEOGRAPH:'B6E2:46818:&#x55EF
CJK UNIFIED IDEOGRAPH:'B6E3:46819:&#x55DA
CJK UNIFIED IDEOGRAPH:'B6E4:46820:&#x55E1
CJK UNIFIED IDEOGRAPH:'B6E5:46821:&#x55C5
CJK UNIFIED IDEOGRAPH:'B6E6:46822:&#x55C6
CJK UNIFIED IDEOGRAPH:'B6E7:46823:&#x55E5
CJK UNIFIED IDEOGRAPH:'B6E8:46824:&#x55C9
CJK UNIFIED IDEOGRAPH:'B6E9:46825:&#x5712
CJK UNIFIED IDEOGRAPH:'B6EA:46826:&#x5713
CJK UNIFIED IDEOGRAPH:'B6EB:46827:&#x585E
CJK UNIFIED IDEOGRAPH:'B6EC:46828:&#x5851
CJK UNIFIED IDEOGRAPH:'B6ED:46829:&#x5858
CJK UNIFIED IDEOGRAPH:'B6EE:46830:&#x5857
CJK UNIFIED IDEOGRAPH:'B6EF:46831:&#x585A
CJK UNIFIED IDEOGRAPH:'B6F0:46832:&#x5854
CJK UNIFIED IDEOGRAPH:'B6F1:46833:&#x586B
CJK UNIFIED IDEOGRAPH:'B6F2:46834:&#x584C
CJK UNIFIED IDEOGRAPH:'B6F3:46835:&#x586D
CJK UNIFIED IDEOGRAPH:'B6F4:46836:&#x584A
CJK UNIFIED IDEOGRAPH:'B6F5:46837:&#x5862
CJK UNIFIED IDEOGRAPH:'B6F6:46838:&#x5852
CJK UNIFIED IDEOGRAPH:'B6F7:46839:&#x584B
CJK UNIFIED IDEOGRAPH:'B6F8:46840:&#x5967
CJK UNIFIED IDEOGRAPH:'B6F9:46841:&#x5AC1
CJK UNIFIED IDEOGRAPH:'B6FA:46842:&#x5AC9
CJK UNIFIED IDEOGRAPH:'B6FB:46843:&#x5ACC
CJK UNIFIED IDEOGRAPH:'B6FC:46844:&#x5ABE
CJK UNIFIED IDEOGRAPH:'B6FD:46845:&#x5ABD
CJK UNIFIED IDEOGRAPH:'B6FE:46846:&#x5ABC
CJK UNIFIED IDEOGRAPH:'B740:46912:&#x5AB3
CJK UNIFIED IDEOGRAPH:'B741:46913:&#x5AC2
CJK UNIFIED IDEOGRAPH:'B742:46914:&#x5AB2
CJK UNIFIED IDEOGRAPH:'B743:46915:&#x5D69
CJK UNIFIED IDEOGRAPH:'B744:46916:&#x5D6F
CJK UNIFIED IDEOGRAPH:'B745:46917:&#x5E4C
CJK UNIFIED IDEOGRAPH:'B746:46918:&#x5E79
CJK UNIFIED IDEOGRAPH:'B747:46919:&#x5EC9
CJK UNIFIED IDEOGRAPH:'B748:46920:&#x5EC8
CJK UNIFIED IDEOGRAPH:'B749:46921:&#x5F12
CJK UNIFIED IDEOGRAPH:'B74A:46922:&#x5F59
CJK UNIFIED IDEOGRAPH:'B74B:46923:&#x5FAC
CJK UNIFIED IDEOGRAPH:'B74C:46924:&#x5FAE
CJK UNIFIED IDEOGRAPH:'B74D:46925:&#x611A
CJK UNIFIED IDEOGRAPH:'B74E:46926:&#x610F
CJK UNIFIED IDEOGRAPH:'B74F:46927:&#x6148
CJK UNIFIED IDEOGRAPH:'B750:46928:&#x611F
CJK UNIFIED IDEOGRAPH:'B751:46929:&#x60F3
CJK UNIFIED IDEOGRAPH:'B752:46930:&#x611B
CJK UNIFIED IDEOGRAPH:'B753:46931:&#x60F9
CJK UNIFIED IDEOGRAPH:'B754:46932:&#x6101
CJK UNIFIED IDEOGRAPH:'B755:46933:&#x6108
CJK UNIFIED IDEOGRAPH:'B756:46934:&#x614E
CJK UNIFIED IDEOGRAPH:'B757:46935:&#x614C
CJK UNIFIED IDEOGRAPH:'B758:46936:&#x6144
CJK UNIFIED IDEOGRAPH:'B759:46937:&#x614D
CJK UNIFIED IDEOGRAPH:'B75A:46938:&#x613E
CJK UNIFIED IDEOGRAPH:'B75B:46939:&#x6134
CJK UNIFIED IDEOGRAPH:'B75C:46940:&#x6127
CJK UNIFIED IDEOGRAPH:'B75D:46941:&#x610D
CJK UNIFIED IDEOGRAPH:'B75E:46942:&#x6106
CJK UNIFIED IDEOGRAPH:'B75F:46943:&#x6137
CJK UNIFIED IDEOGRAPH:'B760:46944:&#x6221
CJK UNIFIED IDEOGRAPH:'B761:46945:&#x6222
CJK UNIFIED IDEOGRAPH:'B762:46946:&#x6413
CJK UNIFIED IDEOGRAPH:'B763:46947:&#x643E
CJK UNIFIED IDEOGRAPH:'B764:46948:&#x641E
CJK UNIFIED IDEOGRAPH:'B765:46949:&#x642A
CJK UNIFIED IDEOGRAPH:'B766:46950:&#x642D
CJK UNIFIED IDEOGRAPH:'B767:46951:&#x643D
CJK UNIFIED IDEOGRAPH:'B768:46952:&#x642C
CJK UNIFIED IDEOGRAPH:'B769:46953:&#x640F
CJK UNIFIED IDEOGRAPH:'B76A:46954:&#x641C
CJK UNIFIED IDEOGRAPH:'B76B:46955:&#x6414
CJK UNIFIED IDEOGRAPH:'B76C:46956:&#x640D
CJK UNIFIED IDEOGRAPH:'B76D:46957:&#x6436
CJK UNIFIED IDEOGRAPH:'B76E:46958:&#x6416
CJK UNIFIED IDEOGRAPH:'B76F:46959:&#x6417
CJK UNIFIED IDEOGRAPH:'B770:46960:&#x6406
CJK UNIFIED IDEOGRAPH:'B771:46961:&#x656C
CJK UNIFIED IDEOGRAPH:'B772:46962:&#x659F
CJK UNIFIED IDEOGRAPH:'B773:46963:&#x65B0
CJK UNIFIED IDEOGRAPH:'B774:46964:&#x6697
CJK UNIFIED IDEOGRAPH:'B775:46965:&#x6689
CJK UNIFIED IDEOGRAPH:'B776:46966:&#x6687
CJK UNIFIED IDEOGRAPH:'B777:46967:&#x6688
CJK UNIFIED IDEOGRAPH:'B778:46968:&#x6696
CJK UNIFIED IDEOGRAPH:'B779:46969:&#x6684
CJK UNIFIED IDEOGRAPH:'B77A:46970:&#x6698
CJK UNIFIED IDEOGRAPH:'B77B:46971:&#x668D
CJK UNIFIED IDEOGRAPH:'B77C:46972:&#x6703
CJK UNIFIED IDEOGRAPH:'B77D:46973:&#x6994
CJK UNIFIED IDEOGRAPH:'B77E:46974:&#x696D
CJK UNIFIED IDEOGRAPH:'B7A1:47009:&#x695A
CJK UNIFIED IDEOGRAPH:'B7A2:47010:&#x6977
CJK UNIFIED IDEOGRAPH:'B7A3:47011:&#x6960
CJK UNIFIED IDEOGRAPH:'B7A4:47012:&#x6954
CJK UNIFIED IDEOGRAPH:'B7A5:47013:&#x6975
CJK UNIFIED IDEOGRAPH:'B7A6:47014:&#x6930
CJK UNIFIED IDEOGRAPH:'B7A7:47015:&#x6982
CJK UNIFIED IDEOGRAPH:'B7A8:47016:&#x694A
CJK UNIFIED IDEOGRAPH:'B7A9:47017:&#x6968
CJK UNIFIED IDEOGRAPH:'B7AA:47018:&#x696B
CJK UNIFIED IDEOGRAPH:'B7AB:47019:&#x695E
CJK UNIFIED IDEOGRAPH:'B7AC:47020:&#x6953
CJK UNIFIED IDEOGRAPH:'B7AD:47021:&#x6979
CJK UNIFIED IDEOGRAPH:'B7AE:47022:&#x6986
CJK UNIFIED IDEOGRAPH:'B7AF:47023:&#x695D
CJK UNIFIED IDEOGRAPH:'B7B0:47024:&#x6963
CJK UNIFIED IDEOGRAPH:'B7B1:47025:&#x695B
CJK UNIFIED IDEOGRAPH:'B7B2:47026:&#x6B47
CJK UNIFIED IDEOGRAPH:'B7B3:47027:&#x6B72
CJK UNIFIED IDEOGRAPH:'B7B4:47028:&#x6BC0
CJK UNIFIED IDEOGRAPH:'B7B5:47029:&#x6BBF
CJK UNIFIED IDEOGRAPH:'B7B6:47030:&#x6BD3
CJK UNIFIED IDEOGRAPH:'B7B7:47031:&#x6BFD
CJK UNIFIED IDEOGRAPH:'B7B8:47032:&#x6EA2
CJK UNIFIED IDEOGRAPH:'B7B9:47033:&#x6EAF
CJK UNIFIED IDEOGRAPH:'B7BA:47034:&#x6ED3
CJK UNIFIED IDEOGRAPH:'B7BB:47035:&#x6EB6
CJK UNIFIED IDEOGRAPH:'B7BC:47036:&#x6EC2
CJK UNIFIED IDEOGRAPH:'B7BD:47037:&#x6E90
CJK UNIFIED IDEOGRAPH:'B7BE:47038:&#x6E9D
CJK UNIFIED IDEOGRAPH:'B7BF:47039:&#x6EC7
CJK UNIFIED IDEOGRAPH:'B7C0:47040:&#x6EC5
CJK UNIFIED IDEOGRAPH:'B7C1:47041:&#x6EA5
CJK UNIFIED IDEOGRAPH:'B7C2:47042:&#x6E98
CJK UNIFIED IDEOGRAPH:'B7C3:47043:&#x6EBC
CJK UNIFIED IDEOGRAPH:'B7C4:47044:&#x6EBA
CJK UNIFIED IDEOGRAPH:'B7C5:47045:&#x6EAB
CJK UNIFIED IDEOGRAPH:'B7C6:47046:&#x6ED1
CJK UNIFIED IDEOGRAPH:'B7C7:47047:&#x6E96
CJK UNIFIED IDEOGRAPH:'B7C8:47048:&#x6E9C
CJK UNIFIED IDEOGRAPH:'B7C9:47049:&#x6EC4
CJK UNIFIED IDEOGRAPH:'B7CA:47050:&#x6ED4
CJK UNIFIED IDEOGRAPH:'B7CB:47051:&#x6EAA
CJK UNIFIED IDEOGRAPH:'B7CC:47052:&#x6EA7
CJK UNIFIED IDEOGRAPH:'B7CD:47053:&#x6EB4
CJK UNIFIED IDEOGRAPH:'B7CE:47054:&#x714E
CJK UNIFIED IDEOGRAPH:'B7CF:47055:&#x7159
CJK UNIFIED IDEOGRAPH:'B7D0:47056:&#x7169
CJK UNIFIED IDEOGRAPH:'B7D1:47057:&#x7164
CJK UNIFIED IDEOGRAPH:'B7D2:47058:&#x7149
CJK UNIFIED IDEOGRAPH:'B7D3:47059:&#x7167
CJK UNIFIED IDEOGRAPH:'B7D4:47060:&#x715C
CJK UNIFIED IDEOGRAPH:'B7D5:47061:&#x716C
CJK UNIFIED IDEOGRAPH:'B7D6:47062:&#x7166
CJK UNIFIED IDEOGRAPH:'B7D7:47063:&#x714C
CJK UNIFIED IDEOGRAPH:'B7D8:47064:&#x7165
CJK UNIFIED IDEOGRAPH:'B7D9:47065:&#x715E
CJK UNIFIED IDEOGRAPH:'B7DA:47066:&#x7146
CJK UNIFIED IDEOGRAPH:'B7DB:47067:&#x7168
CJK UNIFIED IDEOGRAPH:'B7DC:47068:&#x7156
CJK UNIFIED IDEOGRAPH:'B7DD:47069:&#x723A
CJK UNIFIED IDEOGRAPH:'B7DE:47070:&#x7252
CJK UNIFIED IDEOGRAPH:'B7DF:47071:&#x7337
CJK UNIFIED IDEOGRAPH:'B7E0:47072:&#x7345
CJK UNIFIED IDEOGRAPH:'B7E1:47073:&#x733F
CJK UNIFIED IDEOGRAPH:'B7E2:47074:&#x733E
CJK UNIFIED IDEOGRAPH:'B7E3:47075:&#x746F
CJK UNIFIED IDEOGRAPH:'B7E4:47076:&#x745A
CJK UNIFIED IDEOGRAPH:'B7E5:47077:&#x7455
CJK UNIFIED IDEOGRAPH:'B7E6:47078:&#x745F
CJK UNIFIED IDEOGRAPH:'B7E7:47079:&#x745E
CJK UNIFIED IDEOGRAPH:'B7E8:47080:&#x7441
CJK UNIFIED IDEOGRAPH:'B7E9:47081:&#x743F
CJK UNIFIED IDEOGRAPH:'B7EA:47082:&#x7459
CJK UNIFIED IDEOGRAPH:'B7EB:47083:&#x745B
CJK UNIFIED IDEOGRAPH:'B7EC:47084:&#x745C
CJK UNIFIED IDEOGRAPH:'B7ED:47085:&#x7576
CJK UNIFIED IDEOGRAPH:'B7EE:47086:&#x7578
CJK UNIFIED IDEOGRAPH:'B7EF:47087:&#x7600
CJK UNIFIED IDEOGRAPH:'B7F0:47088:&#x75F0
CJK UNIFIED IDEOGRAPH:'B7F1:47089:&#x7601
CJK UNIFIED IDEOGRAPH:'B7F2:47090:&#x75F2
CJK UNIFIED IDEOGRAPH:'B7F3:47091:&#x75F1
CJK UNIFIED IDEOGRAPH:'B7F4:47092:&#x75FA
CJK UNIFIED IDEOGRAPH:'B7F5:47093:&#x75FF
CJK UNIFIED IDEOGRAPH:'B7F6:47094:&#x75F4
CJK UNIFIED IDEOGRAPH:'B7F7:47095:&#x75F3
CJK UNIFIED IDEOGRAPH:'B7F8:47096:&#x76DE
CJK UNIFIED IDEOGRAPH:'B7F9:47097:&#x76DF
CJK UNIFIED IDEOGRAPH:'B7FA:47098:&#x775B
CJK UNIFIED IDEOGRAPH:'B7FB:47099:&#x776B
CJK UNIFIED IDEOGRAPH:'B7FC:47100:&#x7766
CJK UNIFIED IDEOGRAPH:'B7FD:47101:&#x775E
CJK UNIFIED IDEOGRAPH:'B7FE:47102:&#x7763
CJK UNIFIED IDEOGRAPH:'B840:47168:&#x7779
CJK UNIFIED IDEOGRAPH:'B841:47169:&#x776A
CJK UNIFIED IDEOGRAPH:'B842:47170:&#x776C
CJK UNIFIED IDEOGRAPH:'B843:47171:&#x775C
CJK UNIFIED IDEOGRAPH:'B844:47172:&#x7765
CJK UNIFIED IDEOGRAPH:'B845:47173:&#x7768
CJK UNIFIED IDEOGRAPH:'B846:47174:&#x7762
CJK UNIFIED IDEOGRAPH:'B847:47175:&#x77EE
CJK UNIFIED IDEOGRAPH:'B848:47176:&#x788E
CJK UNIFIED IDEOGRAPH:'B849:47177:&#x78B0
CJK UNIFIED IDEOGRAPH:'B84A:47178:&#x7897
CJK UNIFIED IDEOGRAPH:'B84B:47179:&#x7898
CJK UNIFIED IDEOGRAPH:'B84C:47180:&#x788C
CJK UNIFIED IDEOGRAPH:'B84D:47181:&#x7889
CJK UNIFIED IDEOGRAPH:'B84E:47182:&#x787C
CJK UNIFIED IDEOGRAPH:'B84F:47183:&#x7891
CJK UNIFIED IDEOGRAPH:'B850:47184:&#x7893
CJK UNIFIED IDEOGRAPH:'B851:47185:&#x787F
CJK UNIFIED IDEOGRAPH:'B852:47186:&#x797A
CJK UNIFIED IDEOGRAPH:'B853:47187:&#x797F
CJK UNIFIED IDEOGRAPH:'B854:47188:&#x7981
CJK UNIFIED IDEOGRAPH:'B855:47189:&#x842C
CJK UNIFIED IDEOGRAPH:'B856:47190:&#x79BD
CJK UNIFIED IDEOGRAPH:'B857:47191:&#x7A1C
CJK UNIFIED IDEOGRAPH:'B858:47192:&#x7A1A
CJK UNIFIED IDEOGRAPH:'B859:47193:&#x7A20
CJK UNIFIED IDEOGRAPH:'B85A:47194:&#x7A14
CJK UNIFIED IDEOGRAPH:'B85B:47195:&#x7A1F
CJK UNIFIED IDEOGRAPH:'B85C:47196:&#x7A1E
CJK UNIFIED IDEOGRAPH:'B85D:47197:&#x7A9F
CJK UNIFIED IDEOGRAPH:'B85E:47198:&#x7AA0
CJK UNIFIED IDEOGRAPH:'B85F:47199:&#x7B77
CJK UNIFIED IDEOGRAPH:'B860:47200:&#x7BC0
CJK UNIFIED IDEOGRAPH:'B861:47201:&#x7B60
CJK UNIFIED IDEOGRAPH:'B862:47202:&#x7B6E
CJK UNIFIED IDEOGRAPH:'B863:47203:&#x7B67
CJK UNIFIED IDEOGRAPH:'B864:47204:&#x7CB1
CJK UNIFIED IDEOGRAPH:'B865:47205:&#x7CB3
CJK UNIFIED IDEOGRAPH:'B866:47206:&#x7CB5
CJK UNIFIED IDEOGRAPH:'B867:47207:&#x7D93
CJK UNIFIED IDEOGRAPH:'B868:47208:&#x7D79
CJK UNIFIED IDEOGRAPH:'B869:47209:&#x7D91
CJK UNIFIED IDEOGRAPH:'B86A:47210:&#x7D81
CJK UNIFIED IDEOGRAPH:'B86B:47211:&#x7D8F
CJK UNIFIED IDEOGRAPH:'B86C:47212:&#x7D5B
CJK UNIFIED IDEOGRAPH:'B86D:47213:&#x7F6E
CJK UNIFIED IDEOGRAPH:'B86E:47214:&#x7F69
CJK UNIFIED IDEOGRAPH:'B86F:47215:&#x7F6A
CJK UNIFIED IDEOGRAPH:'B870:47216:&#x7F72
CJK UNIFIED IDEOGRAPH:'B871:47217:&#x7FA9
CJK UNIFIED IDEOGRAPH:'B872:47218:&#x7FA8
CJK UNIFIED IDEOGRAPH:'B873:47219:&#x7FA4
CJK UNIFIED IDEOGRAPH:'B874:47220:&#x8056
CJK UNIFIED IDEOGRAPH:'B875:47221:&#x8058
CJK UNIFIED IDEOGRAPH:'B876:47222:&#x8086
CJK UNIFIED IDEOGRAPH:'B877:47223:&#x8084
CJK UNIFIED IDEOGRAPH:'B878:47224:&#x8171
CJK UNIFIED IDEOGRAPH:'B879:47225:&#x8170
CJK UNIFIED IDEOGRAPH:'B87A:47226:&#x8178
CJK UNIFIED IDEOGRAPH:'B87B:47227:&#x8165
CJK UNIFIED IDEOGRAPH:'B87C:47228:&#x816E
CJK UNIFIED IDEOGRAPH:'B87D:47229:&#x8173
CJK UNIFIED IDEOGRAPH:'B87E:47230:&#x816B
CJK UNIFIED IDEOGRAPH:'B8A1:47265:&#x8179
CJK UNIFIED IDEOGRAPH:'B8A2:47266:&#x817A
CJK UNIFIED IDEOGRAPH:'B8A3:47267:&#x8166
CJK UNIFIED IDEOGRAPH:'B8A4:47268:&#x8205
CJK UNIFIED IDEOGRAPH:'B8A5:47269:&#x8247
CJK UNIFIED IDEOGRAPH:'B8A6:47270:&#x8482
CJK UNIFIED IDEOGRAPH:'B8A7:47271:&#x8477
CJK UNIFIED IDEOGRAPH:'B8A8:47272:&#x843D
CJK UNIFIED IDEOGRAPH:'B8A9:47273:&#x8431
CJK UNIFIED IDEOGRAPH:'B8AA:47274:&#x8475
CJK UNIFIED IDEOGRAPH:'B8AB:47275:&#x8466
CJK UNIFIED IDEOGRAPH:'B8AC:47276:&#x846B
CJK UNIFIED IDEOGRAPH:'B8AD:47277:&#x8449
CJK UNIFIED IDEOGRAPH:'B8AE:47278:&#x846C
CJK UNIFIED IDEOGRAPH:'B8AF:47279:&#x845B
CJK UNIFIED IDEOGRAPH:'B8B0:47280:&#x843C
CJK UNIFIED IDEOGRAPH:'B8B1:47281:&#x8435
CJK UNIFIED IDEOGRAPH:'B8B2:47282:&#x8461
CJK UNIFIED IDEOGRAPH:'B8B3:47283:&#x8463
CJK UNIFIED IDEOGRAPH:'B8B4:47284:&#x8469
CJK UNIFIED IDEOGRAPH:'B8B5:47285:&#x846D
CJK UNIFIED IDEOGRAPH:'B8B6:47286:&#x8446
CJK UNIFIED IDEOGRAPH:'B8B7:47287:&#x865E
CJK UNIFIED IDEOGRAPH:'B8B8:47288:&#x865C
CJK UNIFIED IDEOGRAPH:'B8B9:47289:&#x865F
CJK UNIFIED IDEOGRAPH:'B8BA:47290:&#x86F9
CJK UNIFIED IDEOGRAPH:'B8BB:47291:&#x8713
CJK UNIFIED IDEOGRAPH:'B8BC:47292:&#x8708
CJK UNIFIED IDEOGRAPH:'B8BD:47293:&#x8707
CJK UNIFIED IDEOGRAPH:'B8BE:47294:&#x8700
CJK UNIFIED IDEOGRAPH:'B8BF:47295:&#x86FE
CJK UNIFIED IDEOGRAPH:'B8C0:47296:&#x86FB
CJK UNIFIED IDEOGRAPH:'B8C1:47297:&#x8702
CJK UNIFIED IDEOGRAPH:'B8C2:47298:&#x8703
CJK UNIFIED IDEOGRAPH:'B8C3:47299:&#x8706
CJK UNIFIED IDEOGRAPH:'B8C4:47300:&#x870A
CJK UNIFIED IDEOGRAPH:'B8C5:47301:&#x8859
CJK UNIFIED IDEOGRAPH:'B8C6:47302:&#x88DF
CJK UNIFIED IDEOGRAPH:'B8C7:47303:&#x88D4
CJK UNIFIED IDEOGRAPH:'B8C8:47304:&#x88D9
CJK UNIFIED IDEOGRAPH:'B8C9:47305:&#x88DC
CJK UNIFIED IDEOGRAPH:'B8CA:47306:&#x88D8
CJK UNIFIED IDEOGRAPH:'B8CB:47307:&#x88DD
CJK UNIFIED IDEOGRAPH:'B8CC:47308:&#x88E1
CJK UNIFIED IDEOGRAPH:'B8CD:47309:&#x88CA
CJK UNIFIED IDEOGRAPH:'B8CE:47310:&#x88D5
CJK UNIFIED IDEOGRAPH:'B8CF:47311:&#x88D2
CJK UNIFIED IDEOGRAPH:'B8D0:47312:&#x899C
CJK UNIFIED IDEOGRAPH:'B8D1:47313:&#x89E3
CJK UNIFIED IDEOGRAPH:'B8D2:47314:&#x8A6B
CJK UNIFIED IDEOGRAPH:'B8D3:47315:&#x8A72
CJK UNIFIED IDEOGRAPH:'B8D4:47316:&#x8A73
CJK UNIFIED IDEOGRAPH:'B8D5:47317:&#x8A66
CJK UNIFIED IDEOGRAPH:'B8D6:47318:&#x8A69
CJK UNIFIED IDEOGRAPH:'B8D7:47319:&#x8A70
CJK UNIFIED IDEOGRAPH:'B8D8:47320:&#x8A87
CJK UNIFIED IDEOGRAPH:'B8D9:47321:&#x8A7C
CJK UNIFIED IDEOGRAPH:'B8DA:47322:&#x8A63
CJK UNIFIED IDEOGRAPH:'B8DB:47323:&#x8AA0
CJK UNIFIED IDEOGRAPH:'B8DC:47324:&#x8A71
CJK UNIFIED IDEOGRAPH:'B8DD:47325:&#x8A85
CJK UNIFIED IDEOGRAPH:'B8DE:47326:&#x8A6D
CJK UNIFIED IDEOGRAPH:'B8DF:47327:&#x8A62
CJK UNIFIED IDEOGRAPH:'B8E0:47328:&#x8A6E
CJK UNIFIED IDEOGRAPH:'B8E1:47329:&#x8A6C
CJK UNIFIED IDEOGRAPH:'B8E2:47330:&#x8A79
CJK UNIFIED IDEOGRAPH:'B8E3:47331:&#x8A7B
CJK UNIFIED IDEOGRAPH:'B8E4:47332:&#x8A3E
CJK UNIFIED IDEOGRAPH:'B8E5:47333:&#x8A68
CJK UNIFIED IDEOGRAPH:'B8E6:47334:&#x8C62
CJK UNIFIED IDEOGRAPH:'B8E7:47335:&#x8C8A
CJK UNIFIED IDEOGRAPH:'B8E8:47336:&#x8C89
CJK UNIFIED IDEOGRAPH:'B8E9:47337:&#x8CCA
CJK UNIFIED IDEOGRAPH:'B8EA:47338:&#x8CC7
CJK UNIFIED IDEOGRAPH:'B8EB:47339:&#x8CC8
CJK UNIFIED IDEOGRAPH:'B8EC:47340:&#x8CC4
CJK UNIFIED IDEOGRAPH:'B8ED:47341:&#x8CB2
CJK UNIFIED IDEOGRAPH:'B8EE:47342:&#x8CC3
CJK UNIFIED IDEOGRAPH:'B8EF:47343:&#x8CC2
CJK UNIFIED IDEOGRAPH:'B8F0:47344:&#x8CC5
CJK UNIFIED IDEOGRAPH:'B8F1:47345:&#x8DE1
CJK UNIFIED IDEOGRAPH:'B8F2:47346:&#x8DDF
CJK UNIFIED IDEOGRAPH:'B8F3:47347:&#x8DE8
CJK UNIFIED IDEOGRAPH:'B8F4:47348:&#x8DEF
CJK UNIFIED IDEOGRAPH:'B8F5:47349:&#x8DF3
CJK UNIFIED IDEOGRAPH:'B8F6:47350:&#x8DFA
CJK UNIFIED IDEOGRAPH:'B8F7:47351:&#x8DEA
CJK UNIFIED IDEOGRAPH:'B8F8:47352:&#x8DE4
CJK UNIFIED IDEOGRAPH:'B8F9:47353:&#x8DE6
CJK UNIFIED IDEOGRAPH:'B8FA:47354:&#x8EB2
CJK UNIFIED IDEOGRAPH:'B8FB:47355:&#x8F03
CJK UNIFIED IDEOGRAPH:'B8FC:47356:&#x8F09
CJK UNIFIED IDEOGRAPH:'B8FD:47357:&#x8EFE
CJK UNIFIED IDEOGRAPH:'B8FE:47358:&#x8F0A
CJK UNIFIED IDEOGRAPH:'B940:47424:&#x8F9F
CJK UNIFIED IDEOGRAPH:'B941:47425:&#x8FB2
CJK UNIFIED IDEOGRAPH:'B942:47426:&#x904B
CJK UNIFIED IDEOGRAPH:'B943:47427:&#x904A
CJK UNIFIED IDEOGRAPH:'B944:47428:&#x9053
CJK UNIFIED IDEOGRAPH:'B945:47429:&#x9042
CJK UNIFIED IDEOGRAPH:'B946:47430:&#x9054
CJK UNIFIED IDEOGRAPH:'B947:47431:&#x903C
CJK UNIFIED IDEOGRAPH:'B948:47432:&#x9055
CJK UNIFIED IDEOGRAPH:'B949:47433:&#x9050
CJK UNIFIED IDEOGRAPH:'B94A:47434:&#x9047
CJK UNIFIED IDEOGRAPH:'B94B:47435:&#x904F
CJK UNIFIED IDEOGRAPH:'B94C:47436:&#x904E
CJK UNIFIED IDEOGRAPH:'B94D:47437:&#x904D
CJK UNIFIED IDEOGRAPH:'B94E:47438:&#x9051
CJK UNIFIED IDEOGRAPH:'B94F:47439:&#x903E
CJK UNIFIED IDEOGRAPH:'B950:47440:&#x9041
CJK UNIFIED IDEOGRAPH:'B951:47441:&#x9112
CJK UNIFIED IDEOGRAPH:'B952:47442:&#x9117
CJK UNIFIED IDEOGRAPH:'B953:47443:&#x916C
CJK UNIFIED IDEOGRAPH:'B954:47444:&#x916A
CJK UNIFIED IDEOGRAPH:'B955:47445:&#x9169
CJK UNIFIED IDEOGRAPH:'B956:47446:&#x91C9
CJK UNIFIED IDEOGRAPH:'B957:47447:&#x9237
CJK UNIFIED IDEOGRAPH:'B958:47448:&#x9257
CJK UNIFIED IDEOGRAPH:'B959:47449:&#x9238
CJK UNIFIED IDEOGRAPH:'B95A:47450:&#x923D
CJK UNIFIED IDEOGRAPH:'B95B:47451:&#x9240
CJK UNIFIED IDEOGRAPH:'B95C:47452:&#x923E
CJK UNIFIED IDEOGRAPH:'B95D:47453:&#x925B
CJK UNIFIED IDEOGRAPH:'B95E:47454:&#x924B
CJK UNIFIED IDEOGRAPH:'B95F:47455:&#x9264
CJK UNIFIED IDEOGRAPH:'B960:47456:&#x9251
CJK UNIFIED IDEOGRAPH:'B961:47457:&#x9234
CJK UNIFIED IDEOGRAPH:'B962:47458:&#x9249
CJK UNIFIED IDEOGRAPH:'B963:47459:&#x924D
CJK UNIFIED IDEOGRAPH:'B964:47460:&#x9245
CJK UNIFIED IDEOGRAPH:'B965:47461:&#x9239
CJK UNIFIED IDEOGRAPH:'B966:47462:&#x923F
CJK UNIFIED IDEOGRAPH:'B967:47463:&#x925A
CJK UNIFIED IDEOGRAPH:'B968:47464:&#x9598
CJK UNIFIED IDEOGRAPH:'B969:47465:&#x9698
CJK UNIFIED IDEOGRAPH:'B96A:47466:&#x9694
CJK UNIFIED IDEOGRAPH:'B96B:47467:&#x9695
CJK UNIFIED IDEOGRAPH:'B96C:47468:&#x96CD
CJK UNIFIED IDEOGRAPH:'B96D:47469:&#x96CB
CJK UNIFIED IDEOGRAPH:'B96E:47470:&#x96C9
CJK UNIFIED IDEOGRAPH:'B96F:47471:&#x96CA
CJK UNIFIED IDEOGRAPH:'B970:47472:&#x96F7
CJK UNIFIED IDEOGRAPH:'B971:47473:&#x96FB
CJK UNIFIED IDEOGRAPH:'B972:47474:&#x96F9
CJK UNIFIED IDEOGRAPH:'B973:47475:&#x96F6
CJK UNIFIED IDEOGRAPH:'B974:47476:&#x9756
CJK UNIFIED IDEOGRAPH:'B975:47477:&#x9774
CJK UNIFIED IDEOGRAPH:'B976:47478:&#x9776
CJK UNIFIED IDEOGRAPH:'B977:47479:&#x9810
CJK UNIFIED IDEOGRAPH:'B978:47480:&#x9811
CJK UNIFIED IDEOGRAPH:'B979:47481:&#x9813
CJK UNIFIED IDEOGRAPH:'B97A:47482:&#x980A
CJK UNIFIED IDEOGRAPH:'B97B:47483:&#x9812
CJK UNIFIED IDEOGRAPH:'B97C:47484:&#x980C
CJK UNIFIED IDEOGRAPH:'B97D:47485:&#x98FC
CJK UNIFIED IDEOGRAPH:'B97E:47486:&#x98F4
CJK UNIFIED IDEOGRAPH:'B9A1:47521:&#x98FD
CJK UNIFIED IDEOGRAPH:'B9A2:47522:&#x98FE
CJK UNIFIED IDEOGRAPH:'B9A3:47523:&#x99B3
CJK UNIFIED IDEOGRAPH:'B9A4:47524:&#x99B1
CJK UNIFIED IDEOGRAPH:'B9A5:47525:&#x99B4
CJK UNIFIED IDEOGRAPH:'B9A6:47526:&#x9AE1
CJK UNIFIED IDEOGRAPH:'B9A7:47527:&#x9CE9
CJK UNIFIED IDEOGRAPH:'B9A8:47528:&#x9E82
CJK UNIFIED IDEOGRAPH:'B9A9:47529:&#x9F0E
CJK UNIFIED IDEOGRAPH:'B9AA:47530:&#x9F13
CJK UNIFIED IDEOGRAPH:'B9AB:47531:&#x9F20
CJK UNIFIED IDEOGRAPH:'B9AC:47532:&#x50E7
CJK UNIFIED IDEOGRAPH:'B9AD:47533:&#x50EE
CJK UNIFIED IDEOGRAPH:'B9AE:47534:&#x50E5
CJK UNIFIED IDEOGRAPH:'B9AF:47535:&#x50D6
CJK UNIFIED IDEOGRAPH:'B9B0:47536:&#x50ED
CJK UNIFIED IDEOGRAPH:'B9B1:47537:&#x50DA
CJK UNIFIED IDEOGRAPH:'B9B2:47538:&#x50D5
CJK UNIFIED IDEOGRAPH:'B9B3:47539:&#x50CF
CJK UNIFIED IDEOGRAPH:'B9B4:47540:&#x50D1
CJK UNIFIED IDEOGRAPH:'B9B5:47541:&#x50F1
CJK UNIFIED IDEOGRAPH:'B9B6:47542:&#x50CE
CJK UNIFIED IDEOGRAPH:'B9B7:47543:&#x50E9
CJK UNIFIED IDEOGRAPH:'B9B8:47544:&#x5162
CJK UNIFIED IDEOGRAPH:'B9B9:47545:&#x51F3
CJK UNIFIED IDEOGRAPH:'B9BA:47546:&#x5283
CJK UNIFIED IDEOGRAPH:'B9BB:47547:&#x5282
CJK UNIFIED IDEOGRAPH:'B9BC:47548:&#x5331
CJK UNIFIED IDEOGRAPH:'B9BD:47549:&#x53AD
CJK UNIFIED IDEOGRAPH:'B9BE:47550:&#x55FE
CJK UNIFIED IDEOGRAPH:'B9BF:47551:&#x5600
CJK UNIFIED IDEOGRAPH:'B9C0:47552:&#x561B
CJK UNIFIED IDEOGRAPH:'B9C1:47553:&#x5617
CJK UNIFIED IDEOGRAPH:'B9C2:47554:&#x55FD
CJK UNIFIED IDEOGRAPH:'B9C3:47555:&#x5614
CJK UNIFIED IDEOGRAPH:'B9C4:47556:&#x5606
CJK UNIFIED IDEOGRAPH:'B9C5:47557:&#x5609
CJK UNIFIED IDEOGRAPH:'B9C6:47558:&#x560D
CJK UNIFIED IDEOGRAPH:'B9C7:47559:&#x560E
CJK UNIFIED IDEOGRAPH:'B9C8:47560:&#x55F7
CJK UNIFIED IDEOGRAPH:'B9C9:47561:&#x5616
CJK UNIFIED IDEOGRAPH:'B9CA:47562:&#x561F
CJK UNIFIED IDEOGRAPH:'B9CB:47563:&#x5608
CJK UNIFIED IDEOGRAPH:'B9CC:47564:&#x5610
CJK UNIFIED IDEOGRAPH:'B9CD:47565:&#x55F6
CJK UNIFIED IDEOGRAPH:'B9CE:47566:&#x5718
CJK UNIFIED IDEOGRAPH:'B9CF:47567:&#x5716
CJK UNIFIED IDEOGRAPH:'B9D0:47568:&#x5875
CJK UNIFIED IDEOGRAPH:'B9D1:47569:&#x587E
CJK UNIFIED IDEOGRAPH:'B9D2:47570:&#x5883
CJK UNIFIED IDEOGRAPH:'B9D3:47571:&#x5893
CJK UNIFIED IDEOGRAPH:'B9D4:47572:&#x588A
CJK UNIFIED IDEOGRAPH:'B9D5:47573:&#x5879
CJK UNIFIED IDEOGRAPH:'B9D6:47574:&#x5885
CJK UNIFIED IDEOGRAPH:'B9D7:47575:&#x587D
CJK UNIFIED IDEOGRAPH:'B9D8:47576:&#x58FD
CJK UNIFIED IDEOGRAPH:'B9D9:47577:&#x5925
CJK UNIFIED IDEOGRAPH:'B9DA:47578:&#x5922
CJK UNIFIED IDEOGRAPH:'B9DB:47579:&#x5924
CJK UNIFIED IDEOGRAPH:'B9DC:47580:&#x596A
CJK UNIFIED IDEOGRAPH:'B9DD:47581:&#x5969
CJK UNIFIED IDEOGRAPH:'B9DE:47582:&#x5AE1
CJK UNIFIED IDEOGRAPH:'B9DF:47583:&#x5AE6
CJK UNIFIED IDEOGRAPH:'B9E0:47584:&#x5AE9
CJK UNIFIED IDEOGRAPH:'B9E1:47585:&#x5AD7
CJK UNIFIED IDEOGRAPH:'B9E2:47586:&#x5AD6
CJK UNIFIED IDEOGRAPH:'B9E3:47587:&#x5AD8
CJK UNIFIED IDEOGRAPH:'B9E4:47588:&#x5AE3
CJK UNIFIED IDEOGRAPH:'B9E5:47589:&#x5B75
CJK UNIFIED IDEOGRAPH:'B9E6:47590:&#x5BDE
CJK UNIFIED IDEOGRAPH:'B9E7:47591:&#x5BE7
CJK UNIFIED IDEOGRAPH:'B9E8:47592:&#x5BE1
CJK UNIFIED IDEOGRAPH:'B9E9:47593:&#x5BE5
CJK UNIFIED IDEOGRAPH:'B9EA:47594:&#x5BE6
CJK UNIFIED IDEOGRAPH:'B9EB:47595:&#x5BE8
CJK UNIFIED IDEOGRAPH:'B9EC:47596:&#x5BE2
CJK UNIFIED IDEOGRAPH:'B9ED:47597:&#x5BE4
CJK UNIFIED IDEOGRAPH:'B9EE:47598:&#x5BDF
CJK UNIFIED IDEOGRAPH:'B9EF:47599:&#x5C0D
CJK UNIFIED IDEOGRAPH:'B9F0:47600:&#x5C62
CJK UNIFIED IDEOGRAPH:'B9F1:47601:&#x5D84
CJK UNIFIED IDEOGRAPH:'B9F2:47602:&#x5D87
CJK UNIFIED IDEOGRAPH:'B9F3:47603:&#x5E5B
CJK UNIFIED IDEOGRAPH:'B9F4:47604:&#x5E63
CJK UNIFIED IDEOGRAPH:'B9F5:47605:&#x5E55
CJK UNIFIED IDEOGRAPH:'B9F6:47606:&#x5E57
CJK UNIFIED IDEOGRAPH:'B9F7:47607:&#x5E54
CJK UNIFIED IDEOGRAPH:'B9F8:47608:&#x5ED3
CJK UNIFIED IDEOGRAPH:'B9F9:47609:&#x5ED6
CJK UNIFIED IDEOGRAPH:'B9FA:47610:&#x5F0A
CJK UNIFIED IDEOGRAPH:'B9FB:47611:&#x5F46
CJK UNIFIED IDEOGRAPH:'B9FC:47612:&#x5F70
CJK UNIFIED IDEOGRAPH:'B9FD:47613:&#x5FB9
CJK UNIFIED IDEOGRAPH:'B9FE:47614:&#x6147
CJK UNIFIED IDEOGRAPH:'BA40:47680:&#x613F
CJK UNIFIED IDEOGRAPH:'BA41:47681:&#x614B
CJK UNIFIED IDEOGRAPH:'BA42:47682:&#x6177
CJK UNIFIED IDEOGRAPH:'BA43:47683:&#x6162
CJK UNIFIED IDEOGRAPH:'BA44:47684:&#x6163
CJK UNIFIED IDEOGRAPH:'BA45:47685:&#x615F
CJK UNIFIED IDEOGRAPH:'BA46:47686:&#x615A
CJK UNIFIED IDEOGRAPH:'BA47:47687:&#x6158
CJK UNIFIED IDEOGRAPH:'BA48:47688:&#x6175
CJK UNIFIED IDEOGRAPH:'BA49:47689:&#x622A
CJK UNIFIED IDEOGRAPH:'BA4A:47690:&#x6487
CJK UNIFIED IDEOGRAPH:'BA4B:47691:&#x6458
CJK UNIFIED IDEOGRAPH:'BA4C:47692:&#x6454
CJK UNIFIED IDEOGRAPH:'BA4D:47693:&#x64A4
CJK UNIFIED IDEOGRAPH:'BA4E:47694:&#x6478
CJK UNIFIED IDEOGRAPH:'BA4F:47695:&#x645F
CJK UNIFIED IDEOGRAPH:'BA50:47696:&#x647A
CJK UNIFIED IDEOGRAPH:'BA51:47697:&#x6451
CJK UNIFIED IDEOGRAPH:'BA52:47698:&#x6467
CJK UNIFIED IDEOGRAPH:'BA53:47699:&#x6434
CJK UNIFIED IDEOGRAPH:'BA54:47700:&#x646D
CJK UNIFIED IDEOGRAPH:'BA55:47701:&#x647B
CJK UNIFIED IDEOGRAPH:'BA56:47702:&#x6572
CJK UNIFIED IDEOGRAPH:'BA57:47703:&#x65A1
CJK UNIFIED IDEOGRAPH:'BA58:47704:&#x65D7
CJK UNIFIED IDEOGRAPH:'BA59:47705:&#x65D6
CJK UNIFIED IDEOGRAPH:'BA5A:47706:&#x66A2
CJK UNIFIED IDEOGRAPH:'BA5B:47707:&#x66A8
CJK UNIFIED IDEOGRAPH:'BA5C:47708:&#x669D
CJK UNIFIED IDEOGRAPH:'BA5D:47709:&#x699C
CJK UNIFIED IDEOGRAPH:'BA5E:47710:&#x69A8
CJK UNIFIED IDEOGRAPH:'BA5F:47711:&#x6995
CJK UNIFIED IDEOGRAPH:'BA60:47712:&#x69C1
CJK UNIFIED IDEOGRAPH:'BA61:47713:&#x69AE
CJK UNIFIED IDEOGRAPH:'BA62:47714:&#x69D3
CJK UNIFIED IDEOGRAPH:'BA63:47715:&#x69CB
CJK UNIFIED IDEOGRAPH:'BA64:47716:&#x699B
CJK UNIFIED IDEOGRAPH:'BA65:47717:&#x69B7
CJK UNIFIED IDEOGRAPH:'BA66:47718:&#x69BB
CJK UNIFIED IDEOGRAPH:'BA67:47719:&#x69AB
CJK UNIFIED IDEOGRAPH:'BA68:47720:&#x69B4
CJK UNIFIED IDEOGRAPH:'BA69:47721:&#x69D0
CJK UNIFIED IDEOGRAPH:'BA6A:47722:&#x69CD
CJK UNIFIED IDEOGRAPH:'BA6B:47723:&#x69AD
CJK UNIFIED IDEOGRAPH:'BA6C:47724:&#x69CC
CJK UNIFIED IDEOGRAPH:'BA6D:47725:&#x69A6
CJK UNIFIED IDEOGRAPH:'BA6E:47726:&#x69C3
CJK UNIFIED IDEOGRAPH:'BA6F:47727:&#x69A3
CJK UNIFIED IDEOGRAPH:'BA70:47728:&#x6B49
CJK UNIFIED IDEOGRAPH:'BA71:47729:&#x6B4C
CJK UNIFIED IDEOGRAPH:'BA72:47730:&#x6C33
CJK UNIFIED IDEOGRAPH:'BA73:47731:&#x6F33
CJK UNIFIED IDEOGRAPH:'BA74:47732:&#x6F14
CJK UNIFIED IDEOGRAPH:'BA75:47733:&#x6EFE
CJK UNIFIED IDEOGRAPH:'BA76:47734:&#x6F13
CJK UNIFIED IDEOGRAPH:'BA77:47735:&#x6EF4
CJK UNIFIED IDEOGRAPH:'BA78:47736:&#x6F29
CJK UNIFIED IDEOGRAPH:'BA79:47737:&#x6F3E
CJK UNIFIED IDEOGRAPH:'BA7A:47738:&#x6F20
CJK UNIFIED IDEOGRAPH:'BA7B:47739:&#x6F2C
CJK UNIFIED IDEOGRAPH:'BA7C:47740:&#x6F0F
CJK UNIFIED IDEOGRAPH:'BA7D:47741:&#x6F02
CJK UNIFIED IDEOGRAPH:'BA7E:47742:&#x6F22
CJK UNIFIED IDEOGRAPH:'BAA1:47777:&#x6EFF
CJK UNIFIED IDEOGRAPH:'BAA2:47778:&#x6EEF
CJK UNIFIED IDEOGRAPH:'BAA3:47779:&#x6F06
CJK UNIFIED IDEOGRAPH:'BAA4:47780:&#x6F31
CJK UNIFIED IDEOGRAPH:'BAA5:47781:&#x6F38
CJK UNIFIED IDEOGRAPH:'BAA6:47782:&#x6F32
CJK UNIFIED IDEOGRAPH:'BAA7:47783:&#x6F23
CJK UNIFIED IDEOGRAPH:'BAA8:47784:&#x6F15
CJK UNIFIED IDEOGRAPH:'BAA9:47785:&#x6F2B
CJK UNIFIED IDEOGRAPH:'BAAA:47786:&#x6F2F
CJK UNIFIED IDEOGRAPH:'BAAB:47787:&#x6F88
CJK UNIFIED IDEOGRAPH:'BAAC:47788:&#x6F2A
CJK UNIFIED IDEOGRAPH:'BAAD:47789:&#x6EEC
CJK UNIFIED IDEOGRAPH:'BAAE:47790:&#x6F01
CJK UNIFIED IDEOGRAPH:'BAAF:47791:&#x6EF2
CJK UNIFIED IDEOGRAPH:'BAB0:47792:&#x6ECC
CJK UNIFIED IDEOGRAPH:'BAB1:47793:&#x6EF7
CJK UNIFIED IDEOGRAPH:'BAB2:47794:&#x7194
CJK UNIFIED IDEOGRAPH:'BAB3:47795:&#x7199
CJK UNIFIED IDEOGRAPH:'BAB4:47796:&#x717D
CJK UNIFIED IDEOGRAPH:'BAB5:47797:&#x718A
CJK UNIFIED IDEOGRAPH:'BAB6:47798:&#x7184
CJK UNIFIED IDEOGRAPH:'BAB7:47799:&#x7192
CJK UNIFIED IDEOGRAPH:'BAB8:47800:&#x723E
CJK UNIFIED IDEOGRAPH:'BAB9:47801:&#x7292
CJK UNIFIED IDEOGRAPH:'BABA:47802:&#x7296
CJK UNIFIED IDEOGRAPH:'BABB:47803:&#x7344
CJK UNIFIED IDEOGRAPH:'BABC:47804:&#x7350
CJK UNIFIED IDEOGRAPH:'BABD:47805:&#x7464
CJK UNIFIED IDEOGRAPH:'BABE:47806:&#x7463
CJK UNIFIED IDEOGRAPH:'BABF:47807:&#x746A
CJK UNIFIED IDEOGRAPH:'BAC0:47808:&#x7470
CJK UNIFIED IDEOGRAPH:'BAC1:47809:&#x746D
CJK UNIFIED IDEOGRAPH:'BAC2:47810:&#x7504
CJK UNIFIED IDEOGRAPH:'BAC3:47811:&#x7591
CJK UNIFIED IDEOGRAPH:'BAC4:47812:&#x7627
CJK UNIFIED IDEOGRAPH:'BAC5:47813:&#x760D
CJK UNIFIED IDEOGRAPH:'BAC6:47814:&#x760B
CJK UNIFIED IDEOGRAPH:'BAC7:47815:&#x7609
CJK UNIFIED IDEOGRAPH:'BAC8:47816:&#x7613
CJK UNIFIED IDEOGRAPH:'BAC9:47817:&#x76E1
CJK UNIFIED IDEOGRAPH:'BACA:47818:&#x76E3
CJK UNIFIED IDEOGRAPH:'BACB:47819:&#x7784
CJK UNIFIED IDEOGRAPH:'BACC:47820:&#x777D
CJK UNIFIED IDEOGRAPH:'BACD:47821:&#x777F
CJK UNIFIED IDEOGRAPH:'BACE:47822:&#x7761
CJK UNIFIED IDEOGRAPH:'BACF:47823:&#x78C1
CJK UNIFIED IDEOGRAPH:'BAD0:47824:&#x789F
CJK UNIFIED IDEOGRAPH:'BAD1:47825:&#x78A7
CJK UNIFIED IDEOGRAPH:'BAD2:47826:&#x78B3
CJK UNIFIED IDEOGRAPH:'BAD3:47827:&#x78A9
CJK UNIFIED IDEOGRAPH:'BAD4:47828:&#x78A3
CJK UNIFIED IDEOGRAPH:'BAD5:47829:&#x798E
CJK UNIFIED IDEOGRAPH:'BAD6:47830:&#x798F
CJK UNIFIED IDEOGRAPH:'BAD7:47831:&#x798D
CJK UNIFIED IDEOGRAPH:'BAD8:47832:&#x7A2E
CJK UNIFIED IDEOGRAPH:'BAD9:47833:&#x7A31
CJK UNIFIED IDEOGRAPH:'BADA:47834:&#x7AAA
CJK UNIFIED IDEOGRAPH:'BADB:47835:&#x7AA9
CJK UNIFIED IDEOGRAPH:'BADC:47836:&#x7AED
CJK UNIFIED IDEOGRAPH:'BADD:47837:&#x7AEF
CJK UNIFIED IDEOGRAPH:'BADE:47838:&#x7BA1
CJK UNIFIED IDEOGRAPH:'BADF:47839:&#x7B95
CJK UNIFIED IDEOGRAPH:'BAE0:47840:&#x7B8B
CJK UNIFIED IDEOGRAPH:'BAE1:47841:&#x7B75
CJK UNIFIED IDEOGRAPH:'BAE2:47842:&#x7B97
CJK UNIFIED IDEOGRAPH:'BAE3:47843:&#x7B9D
CJK UNIFIED IDEOGRAPH:'BAE4:47844:&#x7B94
CJK UNIFIED IDEOGRAPH:'BAE5:47845:&#x7B8F
CJK UNIFIED IDEOGRAPH:'BAE6:47846:&#x7BB8
CJK UNIFIED IDEOGRAPH:'BAE7:47847:&#x7B87
CJK UNIFIED IDEOGRAPH:'BAE8:47848:&#x7B84
CJK UNIFIED IDEOGRAPH:'BAE9:47849:&#x7CB9
CJK UNIFIED IDEOGRAPH:'BAEA:47850:&#x7CBD
CJK UNIFIED IDEOGRAPH:'BAEB:47851:&#x7CBE
CJK UNIFIED IDEOGRAPH:'BAEC:47852:&#x7DBB
CJK UNIFIED IDEOGRAPH:'BAED:47853:&#x7DB0
CJK UNIFIED IDEOGRAPH:'BAEE:47854:&#x7D9C
CJK UNIFIED IDEOGRAPH:'BAEF:47855:&#x7DBD
CJK UNIFIED IDEOGRAPH:'BAF0:47856:&#x7DBE
CJK UNIFIED IDEOGRAPH:'BAF1:47857:&#x7DA0
CJK UNIFIED IDEOGRAPH:'BAF2:47858:&#x7DCA
CJK UNIFIED IDEOGRAPH:'BAF3:47859:&#x7DB4
CJK UNIFIED IDEOGRAPH:'BAF4:47860:&#x7DB2
CJK UNIFIED IDEOGRAPH:'BAF5:47861:&#x7DB1
CJK UNIFIED IDEOGRAPH:'BAF6:47862:&#x7DBA
CJK UNIFIED IDEOGRAPH:'BAF7:47863:&#x7DA2
CJK UNIFIED IDEOGRAPH:'BAF8:47864:&#x7DBF
CJK UNIFIED IDEOGRAPH:'BAF9:47865:&#x7DB5
CJK UNIFIED IDEOGRAPH:'BAFA:47866:&#x7DB8
CJK UNIFIED IDEOGRAPH:'BAFB:47867:&#x7DAD
CJK UNIFIED IDEOGRAPH:'BAFC:47868:&#x7DD2
CJK UNIFIED IDEOGRAPH:'BAFD:47869:&#x7DC7
CJK UNIFIED IDEOGRAPH:'BAFE:47870:&#x7DAC
CJK UNIFIED IDEOGRAPH:'BB40:47936:&#x7F70
CJK UNIFIED IDEOGRAPH:'BB41:47937:&#x7FE0
CJK UNIFIED IDEOGRAPH:'BB42:47938:&#x7FE1
CJK UNIFIED IDEOGRAPH:'BB43:47939:&#x7FDF
CJK UNIFIED IDEOGRAPH:'BB44:47940:&#x805E
CJK UNIFIED IDEOGRAPH:'BB45:47941:&#x805A
CJK UNIFIED IDEOGRAPH:'BB46:47942:&#x8087
CJK UNIFIED IDEOGRAPH:'BB47:47943:&#x8150
CJK UNIFIED IDEOGRAPH:'BB48:47944:&#x8180
CJK UNIFIED IDEOGRAPH:'BB49:47945:&#x818F
CJK UNIFIED IDEOGRAPH:'BB4A:47946:&#x8188
CJK UNIFIED IDEOGRAPH:'BB4B:47947:&#x818A
CJK UNIFIED IDEOGRAPH:'BB4C:47948:&#x817F
CJK UNIFIED IDEOGRAPH:'BB4D:47949:&#x8182
CJK UNIFIED IDEOGRAPH:'BB4E:47950:&#x81E7
CJK UNIFIED IDEOGRAPH:'BB4F:47951:&#x81FA
CJK UNIFIED IDEOGRAPH:'BB50:47952:&#x8207
CJK UNIFIED IDEOGRAPH:'BB51:47953:&#x8214
CJK UNIFIED IDEOGRAPH:'BB52:47954:&#x821E
CJK UNIFIED IDEOGRAPH:'BB53:47955:&#x824B
CJK UNIFIED IDEOGRAPH:'BB54:47956:&#x84C9
CJK UNIFIED IDEOGRAPH:'BB55:47957:&#x84BF
CJK UNIFIED IDEOGRAPH:'BB56:47958:&#x84C6
CJK UNIFIED IDEOGRAPH:'BB57:47959:&#x84C4
CJK UNIFIED IDEOGRAPH:'BB58:47960:&#x8499
CJK UNIFIED IDEOGRAPH:'BB59:47961:&#x849E
CJK UNIFIED IDEOGRAPH:'BB5A:47962:&#x84B2
CJK UNIFIED IDEOGRAPH:'BB5B:47963:&#x849C
CJK UNIFIED IDEOGRAPH:'BB5C:47964:&#x84CB
CJK UNIFIED IDEOGRAPH:'BB5D:47965:&#x84B8
CJK UNIFIED IDEOGRAPH:'BB5E:47966:&#x84C0
CJK UNIFIED IDEOGRAPH:'BB5F:47967:&#x84D3
CJK UNIFIED IDEOGRAPH:'BB60:47968:&#x8490
CJK UNIFIED IDEOGRAPH:'BB61:47969:&#x84BC
CJK UNIFIED IDEOGRAPH:'BB62:47970:&#x84D1
CJK UNIFIED IDEOGRAPH:'BB63:47971:&#x84CA
CJK UNIFIED IDEOGRAPH:'BB64:47972:&#x873F
CJK UNIFIED IDEOGRAPH:'BB65:47973:&#x871C
CJK UNIFIED IDEOGRAPH:'BB66:47974:&#x873B
CJK UNIFIED IDEOGRAPH:'BB67:47975:&#x8722
CJK UNIFIED IDEOGRAPH:'BB68:47976:&#x8725
CJK UNIFIED IDEOGRAPH:'BB69:47977:&#x8734
CJK UNIFIED IDEOGRAPH:'BB6A:47978:&#x8718
CJK UNIFIED IDEOGRAPH:'BB6B:47979:&#x8755
CJK UNIFIED IDEOGRAPH:'BB6C:47980:&#x8737
CJK UNIFIED IDEOGRAPH:'BB6D:47981:&#x8729
CJK UNIFIED IDEOGRAPH:'BB6E:47982:&#x88F3
CJK UNIFIED IDEOGRAPH:'BB6F:47983:&#x8902
CJK UNIFIED IDEOGRAPH:'BB70:47984:&#x88F4
CJK UNIFIED IDEOGRAPH:'BB71:47985:&#x88F9
CJK UNIFIED IDEOGRAPH:'BB72:47986:&#x88F8
CJK UNIFIED IDEOGRAPH:'BB73:47987:&#x88FD
CJK UNIFIED IDEOGRAPH:'BB74:47988:&#x88E8
CJK UNIFIED IDEOGRAPH:'BB75:47989:&#x891A
CJK UNIFIED IDEOGRAPH:'BB76:47990:&#x88EF
CJK UNIFIED IDEOGRAPH:'BB77:47991:&#x8AA6
CJK UNIFIED IDEOGRAPH:'BB78:47992:&#x8A8C
CJK UNIFIED IDEOGRAPH:'BB79:47993:&#x8A9E
CJK UNIFIED IDEOGRAPH:'BB7A:47994:&#x8AA3
CJK UNIFIED IDEOGRAPH:'BB7B:47995:&#x8A8D
CJK UNIFIED IDEOGRAPH:'BB7C:47996:&#x8AA1
CJK UNIFIED IDEOGRAPH:'BB7D:47997:&#x8A93
CJK UNIFIED IDEOGRAPH:'BB7E:47998:&#x8AA4
CJK UNIFIED IDEOGRAPH:'BBA1:48033:&#x8AAA
CJK UNIFIED IDEOGRAPH:'BBA2:48034:&#x8AA5
CJK UNIFIED IDEOGRAPH:'BBA3:48035:&#x8AA8
CJK UNIFIED IDEOGRAPH:'BBA4:48036:&#x8A98
CJK UNIFIED IDEOGRAPH:'BBA5:48037:&#x8A91
CJK UNIFIED IDEOGRAPH:'BBA6:48038:&#x8A9A
CJK UNIFIED IDEOGRAPH:'BBA7:48039:&#x8AA7
CJK UNIFIED IDEOGRAPH:'BBA8:48040:&#x8C6A
CJK UNIFIED IDEOGRAPH:'BBA9:48041:&#x8C8D
CJK UNIFIED IDEOGRAPH:'BBAA:48042:&#x8C8C
CJK UNIFIED IDEOGRAPH:'BBAB:48043:&#x8CD3
CJK UNIFIED IDEOGRAPH:'BBAC:48044:&#x8CD1
CJK UNIFIED IDEOGRAPH:'BBAD:48045:&#x8CD2
CJK UNIFIED IDEOGRAPH:'BBAE:48046:&#x8D6B
CJK UNIFIED IDEOGRAPH:'BBAF:48047:&#x8D99
CJK UNIFIED IDEOGRAPH:'BBB0:48048:&#x8D95
CJK UNIFIED IDEOGRAPH:'BBB1:48049:&#x8DFC
CJK UNIFIED IDEOGRAPH:'BBB2:48050:&#x8F14
CJK UNIFIED IDEOGRAPH:'BBB3:48051:&#x8F12
CJK UNIFIED IDEOGRAPH:'BBB4:48052:&#x8F15
CJK UNIFIED IDEOGRAPH:'BBB5:48053:&#x8F13
CJK UNIFIED IDEOGRAPH:'BBB6:48054:&#x8FA3
CJK UNIFIED IDEOGRAPH:'BBB7:48055:&#x9060
CJK UNIFIED IDEOGRAPH:'BBB8:48056:&#x9058
CJK UNIFIED IDEOGRAPH:'BBB9:48057:&#x905C
CJK UNIFIED IDEOGRAPH:'BBBA:48058:&#x9063
CJK UNIFIED IDEOGRAPH:'BBBB:48059:&#x9059
CJK UNIFIED IDEOGRAPH:'BBBC:48060:&#x905E
CJK UNIFIED IDEOGRAPH:'BBBD:48061:&#x9062
CJK UNIFIED IDEOGRAPH:'BBBE:48062:&#x905D
CJK UNIFIED IDEOGRAPH:'BBBF:48063:&#x905B
CJK UNIFIED IDEOGRAPH:'BBC0:48064:&#x9119
CJK UNIFIED IDEOGRAPH:'BBC1:48065:&#x9118
CJK UNIFIED IDEOGRAPH:'BBC2:48066:&#x911E
CJK UNIFIED IDEOGRAPH:'BBC3:48067:&#x9175
CJK UNIFIED IDEOGRAPH:'BBC4:48068:&#x9178
CJK UNIFIED IDEOGRAPH:'BBC5:48069:&#x9177
CJK UNIFIED IDEOGRAPH:'BBC6:48070:&#x9174
CJK UNIFIED IDEOGRAPH:'BBC7:48071:&#x9278
CJK UNIFIED IDEOGRAPH:'BBC8:48072:&#x9280
CJK UNIFIED IDEOGRAPH:'BBC9:48073:&#x9285
CJK UNIFIED IDEOGRAPH:'BBCA:48074:&#x9298
CJK UNIFIED IDEOGRAPH:'BBCB:48075:&#x9296
CJK UNIFIED IDEOGRAPH:'BBCC:48076:&#x927B
CJK UNIFIED IDEOGRAPH:'BBCD:48077:&#x9293
CJK UNIFIED IDEOGRAPH:'BBCE:48078:&#x929C
CJK UNIFIED IDEOGRAPH:'BBCF:48079:&#x92A8
CJK UNIFIED IDEOGRAPH:'BBD0:48080:&#x927C
CJK UNIFIED IDEOGRAPH:'BBD1:48081:&#x9291
CJK UNIFIED IDEOGRAPH:'BBD2:48082:&#x95A1
CJK UNIFIED IDEOGRAPH:'BBD3:48083:&#x95A8
CJK UNIFIED IDEOGRAPH:'BBD4:48084:&#x95A9
CJK UNIFIED IDEOGRAPH:'BBD5:48085:&#x95A3
CJK UNIFIED IDEOGRAPH:'BBD6:48086:&#x95A5
CJK UNIFIED IDEOGRAPH:'BBD7:48087:&#x95A4
CJK UNIFIED IDEOGRAPH:'BBD8:48088:&#x9699
CJK UNIFIED IDEOGRAPH:'BBD9:48089:&#x969C
CJK UNIFIED IDEOGRAPH:'BBDA:48090:&#x969B
CJK UNIFIED IDEOGRAPH:'BBDB:48091:&#x96CC
CJK UNIFIED IDEOGRAPH:'BBDC:48092:&#x96D2
CJK UNIFIED IDEOGRAPH:'BBDD:48093:&#x9700
CJK UNIFIED IDEOGRAPH:'BBDE:48094:&#x977C
CJK UNIFIED IDEOGRAPH:'BBDF:48095:&#x9785
CJK UNIFIED IDEOGRAPH:'BBE0:48096:&#x97F6
CJK UNIFIED IDEOGRAPH:'BBE1:48097:&#x9817
CJK UNIFIED IDEOGRAPH:'BBE2:48098:&#x9818
CJK UNIFIED IDEOGRAPH:'BBE3:48099:&#x98AF
CJK UNIFIED IDEOGRAPH:'BBE4:48100:&#x98B1
CJK UNIFIED IDEOGRAPH:'BBE5:48101:&#x9903
CJK UNIFIED IDEOGRAPH:'BBE6:48102:&#x9905
CJK UNIFIED IDEOGRAPH:'BBE7:48103:&#x990C
CJK UNIFIED IDEOGRAPH:'BBE8:48104:&#x9909
CJK UNIFIED IDEOGRAPH:'BBE9:48105:&#x99C1
CJK UNIFIED IDEOGRAPH:'BBEA:48106:&#x9AAF
CJK UNIFIED IDEOGRAPH:'BBEB:48107:&#x9AB0
CJK UNIFIED IDEOGRAPH:'BBEC:48108:&#x9AE6
CJK UNIFIED IDEOGRAPH:'BBED:48109:&#x9B41
CJK UNIFIED IDEOGRAPH:'BBEE:48110:&#x9B42
CJK UNIFIED IDEOGRAPH:'BBEF:48111:&#x9CF4
CJK UNIFIED IDEOGRAPH:'BBF0:48112:&#x9CF6
CJK UNIFIED IDEOGRAPH:'BBF1:48113:&#x9CF3
CJK UNIFIED IDEOGRAPH:'BBF2:48114:&#x9EBC
CJK UNIFIED IDEOGRAPH:'BBF3:48115:&#x9F3B
CJK UNIFIED IDEOGRAPH:'BBF4:48116:&#x9F4A
CJK UNIFIED IDEOGRAPH:'BBF5:48117:&#x5104
CJK UNIFIED IDEOGRAPH:'BBF6:48118:&#x5100
CJK UNIFIED IDEOGRAPH:'BBF7:48119:&#x50FB
CJK UNIFIED IDEOGRAPH:'BBF8:48120:&#x50F5
CJK UNIFIED IDEOGRAPH:'BBF9:48121:&#x50F9
CJK UNIFIED IDEOGRAPH:'BBFA:48122:&#x5102
CJK UNIFIED IDEOGRAPH:'BBFB:48123:&#x5108
CJK UNIFIED IDEOGRAPH:'BBFC:48124:&#x5109
CJK UNIFIED IDEOGRAPH:'BBFD:48125:&#x5105
CJK UNIFIED IDEOGRAPH:'BBFE:48126:&#x51DC
CJK UNIFIED IDEOGRAPH:'BC40:48192:&#x5287
CJK UNIFIED IDEOGRAPH:'BC41:48193:&#x5288
CJK UNIFIED IDEOGRAPH:'BC42:48194:&#x5289
CJK UNIFIED IDEOGRAPH:'BC43:48195:&#x528D
CJK UNIFIED IDEOGRAPH:'BC44:48196:&#x528A
CJK UNIFIED IDEOGRAPH:'BC45:48197:&#x52F0
CJK UNIFIED IDEOGRAPH:'BC46:48198:&#x53B2
CJK UNIFIED IDEOGRAPH:'BC47:48199:&#x562E
CJK UNIFIED IDEOGRAPH:'BC48:48200:&#x563B
CJK UNIFIED IDEOGRAPH:'BC49:48201:&#x5639
CJK UNIFIED IDEOGRAPH:'BC4A:48202:&#x5632
CJK UNIFIED IDEOGRAPH:'BC4B:48203:&#x563F
CJK UNIFIED IDEOGRAPH:'BC4C:48204:&#x5634
CJK UNIFIED IDEOGRAPH:'BC4D:48205:&#x5629
CJK UNIFIED IDEOGRAPH:'BC4E:48206:&#x5653
CJK UNIFIED IDEOGRAPH:'BC4F:48207:&#x564E
CJK UNIFIED IDEOGRAPH:'BC50:48208:&#x5657
CJK UNIFIED IDEOGRAPH:'BC51:48209:&#x5674
CJK UNIFIED IDEOGRAPH:'BC52:48210:&#x5636
CJK UNIFIED IDEOGRAPH:'BC53:48211:&#x562F
CJK UNIFIED IDEOGRAPH:'BC54:48212:&#x5630
CJK UNIFIED IDEOGRAPH:'BC55:48213:&#x5880
CJK UNIFIED IDEOGRAPH:'BC56:48214:&#x589F
CJK UNIFIED IDEOGRAPH:'BC57:48215:&#x589E
CJK UNIFIED IDEOGRAPH:'BC58:48216:&#x58B3
CJK UNIFIED IDEOGRAPH:'BC59:48217:&#x589C
CJK UNIFIED IDEOGRAPH:'BC5A:48218:&#x58AE
CJK UNIFIED IDEOGRAPH:'BC5B:48219:&#x58A9
CJK UNIFIED IDEOGRAPH:'BC5C:48220:&#x58A6
CJK UNIFIED IDEOGRAPH:'BC5D:48221:&#x596D
CJK UNIFIED IDEOGRAPH:'BC5E:48222:&#x5B09
CJK UNIFIED IDEOGRAPH:'BC5F:48223:&#x5AFB
CJK UNIFIED IDEOGRAPH:'BC60:48224:&#x5B0B
CJK UNIFIED IDEOGRAPH:'BC61:48225:&#x5AF5
CJK UNIFIED IDEOGRAPH:'BC62:48226:&#x5B0C
CJK UNIFIED IDEOGRAPH:'BC63:48227:&#x5B08
CJK UNIFIED IDEOGRAPH:'BC64:48228:&#x5BEE
CJK UNIFIED IDEOGRAPH:'BC65:48229:&#x5BEC
CJK UNIFIED IDEOGRAPH:'BC66:48230:&#x5BE9
CJK UNIFIED IDEOGRAPH:'BC67:48231:&#x5BEB
CJK UNIFIED IDEOGRAPH:'BC68:48232:&#x5C64
CJK UNIFIED IDEOGRAPH:'BC69:48233:&#x5C65
CJK UNIFIED IDEOGRAPH:'BC6A:48234:&#x5D9D
CJK UNIFIED IDEOGRAPH:'BC6B:48235:&#x5D94
CJK UNIFIED IDEOGRAPH:'BC6C:48236:&#x5E62
CJK UNIFIED IDEOGRAPH:'BC6D:48237:&#x5E5F
CJK UNIFIED IDEOGRAPH:'BC6E:48238:&#x5E61
CJK UNIFIED IDEOGRAPH:'BC6F:48239:&#x5EE2
CJK UNIFIED IDEOGRAPH:'BC70:48240:&#x5EDA
CJK UNIFIED IDEOGRAPH:'BC71:48241:&#x5EDF
CJK UNIFIED IDEOGRAPH:'BC72:48242:&#x5EDD
CJK UNIFIED IDEOGRAPH:'BC73:48243:&#x5EE3
CJK UNIFIED IDEOGRAPH:'BC74:48244:&#x5EE0
CJK UNIFIED IDEOGRAPH:'BC75:48245:&#x5F48
CJK UNIFIED IDEOGRAPH:'BC76:48246:&#x5F71
CJK UNIFIED IDEOGRAPH:'BC77:48247:&#x5FB7
CJK UNIFIED IDEOGRAPH:'BC78:48248:&#x5FB5
CJK UNIFIED IDEOGRAPH:'BC79:48249:&#x6176
CJK UNIFIED IDEOGRAPH:'BC7A:48250:&#x6167
CJK UNIFIED IDEOGRAPH:'BC7B:48251:&#x616E
CJK UNIFIED IDEOGRAPH:'BC7C:48252:&#x615D
CJK UNIFIED IDEOGRAPH:'BC7D:48253:&#x6155
CJK UNIFIED IDEOGRAPH:'BC7E:48254:&#x6182
CJK UNIFIED IDEOGRAPH:'BCA1:48289:&#x617C
CJK UNIFIED IDEOGRAPH:'BCA2:48290:&#x6170
CJK UNIFIED IDEOGRAPH:'BCA3:48291:&#x616B
CJK UNIFIED IDEOGRAPH:'BCA4:48292:&#x617E
CJK UNIFIED IDEOGRAPH:'BCA5:48293:&#x61A7
CJK UNIFIED IDEOGRAPH:'BCA6:48294:&#x6190
CJK UNIFIED IDEOGRAPH:'BCA7:48295:&#x61AB
CJK UNIFIED IDEOGRAPH:'BCA8:48296:&#x618E
CJK UNIFIED IDEOGRAPH:'BCA9:48297:&#x61AC
CJK UNIFIED IDEOGRAPH:'BCAA:48298:&#x619A
CJK UNIFIED IDEOGRAPH:'BCAB:48299:&#x61A4
CJK UNIFIED IDEOGRAPH:'BCAC:48300:&#x6194
CJK UNIFIED IDEOGRAPH:'BCAD:48301:&#x61AE
CJK UNIFIED IDEOGRAPH:'BCAE:48302:&#x622E
CJK UNIFIED IDEOGRAPH:'BCAF:48303:&#x6469
CJK UNIFIED IDEOGRAPH:'BCB0:48304:&#x646F
CJK UNIFIED IDEOGRAPH:'BCB1:48305:&#x6479
CJK UNIFIED IDEOGRAPH:'BCB2:48306:&#x649E
CJK UNIFIED IDEOGRAPH:'BCB3:48307:&#x64B2
CJK UNIFIED IDEOGRAPH:'BCB4:48308:&#x6488
CJK UNIFIED IDEOGRAPH:'BCB5:48309:&#x6490
CJK UNIFIED IDEOGRAPH:'BCB6:48310:&#x64B0
CJK UNIFIED IDEOGRAPH:'BCB7:48311:&#x64A5
CJK UNIFIED IDEOGRAPH:'BCB8:48312:&#x6493
CJK UNIFIED IDEOGRAPH:'BCB9:48313:&#x6495
CJK UNIFIED IDEOGRAPH:'BCBA:48314:&#x64A9
CJK UNIFIED IDEOGRAPH:'BCBB:48315:&#x6492
CJK UNIFIED IDEOGRAPH:'BCBC:48316:&#x64AE
CJK UNIFIED IDEOGRAPH:'BCBD:48317:&#x64AD
CJK UNIFIED IDEOGRAPH:'BCBE:48318:&#x64AB
CJK UNIFIED IDEOGRAPH:'BCBF:48319:&#x649A
CJK UNIFIED IDEOGRAPH:'BCC0:48320:&#x64AC
CJK UNIFIED IDEOGRAPH:'BCC1:48321:&#x6499
CJK UNIFIED IDEOGRAPH:'BCC2:48322:&#x64A2
CJK UNIFIED IDEOGRAPH:'BCC3:48323:&#x64B3
CJK UNIFIED IDEOGRAPH:'BCC4:48324:&#x6575
CJK UNIFIED IDEOGRAPH:'BCC5:48325:&#x6577
CJK UNIFIED IDEOGRAPH:'BCC6:48326:&#x6578
CJK UNIFIED IDEOGRAPH:'BCC7:48327:&#x66AE
CJK UNIFIED IDEOGRAPH:'BCC8:48328:&#x66AB
CJK UNIFIED IDEOGRAPH:'BCC9:48329:&#x66B4
CJK UNIFIED IDEOGRAPH:'BCCA:48330:&#x66B1
CJK UNIFIED IDEOGRAPH:'BCCB:48331:&#x6A23
CJK UNIFIED IDEOGRAPH:'BCCC:48332:&#x6A1F
CJK UNIFIED IDEOGRAPH:'BCCD:48333:&#x69E8
CJK UNIFIED IDEOGRAPH:'BCCE:48334:&#x6A01
CJK UNIFIED IDEOGRAPH:'BCCF:48335:&#x6A1E
CJK UNIFIED IDEOGRAPH:'BCD0:48336:&#x6A19
CJK UNIFIED IDEOGRAPH:'BCD1:48337:&#x69FD
CJK UNIFIED IDEOGRAPH:'BCD2:48338:&#x6A21
CJK UNIFIED IDEOGRAPH:'BCD3:48339:&#x6A13
CJK UNIFIED IDEOGRAPH:'BCD4:48340:&#x6A0A
CJK UNIFIED IDEOGRAPH:'BCD5:48341:&#x69F3
CJK UNIFIED IDEOGRAPH:'BCD6:48342:&#x6A02
CJK UNIFIED IDEOGRAPH:'BCD7:48343:&#x6A05
CJK UNIFIED IDEOGRAPH:'BCD8:48344:&#x69ED
CJK UNIFIED IDEOGRAPH:'BCD9:48345:&#x6A11
CJK UNIFIED IDEOGRAPH:'BCDA:48346:&#x6B50
CJK UNIFIED IDEOGRAPH:'BCDB:48347:&#x6B4E
CJK UNIFIED IDEOGRAPH:'BCDC:48348:&#x6BA4
CJK UNIFIED IDEOGRAPH:'BCDD:48349:&#x6BC5
CJK UNIFIED IDEOGRAPH:'BCDE:48350:&#x6BC6
CJK UNIFIED IDEOGRAPH:'BCDF:48351:&#x6F3F
CJK UNIFIED IDEOGRAPH:'BCE0:48352:&#x6F7C
CJK UNIFIED IDEOGRAPH:'BCE1:48353:&#x6F84
CJK UNIFIED IDEOGRAPH:'BCE2:48354:&#x6F51
CJK UNIFIED IDEOGRAPH:'BCE3:48355:&#x6F66
CJK UNIFIED IDEOGRAPH:'BCE4:48356:&#x6F54
CJK UNIFIED IDEOGRAPH:'BCE5:48357:&#x6F86
CJK UNIFIED IDEOGRAPH:'BCE6:48358:&#x6F6D
CJK UNIFIED IDEOGRAPH:'BCE7:48359:&#x6F5B
CJK UNIFIED IDEOGRAPH:'BCE8:48360:&#x6F78
CJK UNIFIED IDEOGRAPH:'BCE9:48361:&#x6F6E
CJK UNIFIED IDEOGRAPH:'BCEA:48362:&#x6F8E
CJK UNIFIED IDEOGRAPH:'BCEB:48363:&#x6F7A
CJK UNIFIED IDEOGRAPH:'BCEC:48364:&#x6F70
CJK UNIFIED IDEOGRAPH:'BCED:48365:&#x6F64
CJK UNIFIED IDEOGRAPH:'BCEE:48366:&#x6F97
CJK UNIFIED IDEOGRAPH:'BCEF:48367:&#x6F58
CJK UNIFIED IDEOGRAPH:'BCF0:48368:&#x6ED5
CJK UNIFIED IDEOGRAPH:'BCF1:48369:&#x6F6F
CJK UNIFIED IDEOGRAPH:'BCF2:48370:&#x6F60
CJK UNIFIED IDEOGRAPH:'BCF3:48371:&#x6F5F
CJK UNIFIED IDEOGRAPH:'BCF4:48372:&#x719F
CJK UNIFIED IDEOGRAPH:'BCF5:48373:&#x71AC
CJK UNIFIED IDEOGRAPH:'BCF6:48374:&#x71B1
CJK UNIFIED IDEOGRAPH:'BCF7:48375:&#x71A8
CJK UNIFIED IDEOGRAPH:'BCF8:48376:&#x7256
CJK UNIFIED IDEOGRAPH:'BCF9:48377:&#x729B
CJK UNIFIED IDEOGRAPH:'BCFA:48378:&#x734E
CJK UNIFIED IDEOGRAPH:'BCFB:48379:&#x7357
CJK UNIFIED IDEOGRAPH:'BCFC:48380:&#x7469
CJK UNIFIED IDEOGRAPH:'BCFD:48381:&#x748B
CJK UNIFIED IDEOGRAPH:'BCFE:48382:&#x7483
CJK UNIFIED IDEOGRAPH:'BD40:48448:&#x747E
CJK UNIFIED IDEOGRAPH:'BD41:48449:&#x7480
CJK UNIFIED IDEOGRAPH:'BD42:48450:&#x757F
CJK UNIFIED IDEOGRAPH:'BD43:48451:&#x7620
CJK UNIFIED IDEOGRAPH:'BD44:48452:&#x7629
CJK UNIFIED IDEOGRAPH:'BD45:48453:&#x761F
CJK UNIFIED IDEOGRAPH:'BD46:48454:&#x7624
CJK UNIFIED IDEOGRAPH:'BD47:48455:&#x7626
CJK UNIFIED IDEOGRAPH:'BD48:48456:&#x7621
CJK UNIFIED IDEOGRAPH:'BD49:48457:&#x7622
CJK UNIFIED IDEOGRAPH:'BD4A:48458:&#x769A
CJK UNIFIED IDEOGRAPH:'BD4B:48459:&#x76BA
CJK UNIFIED IDEOGRAPH:'BD4C:48460:&#x76E4
CJK UNIFIED IDEOGRAPH:'BD4D:48461:&#x778E
CJK UNIFIED IDEOGRAPH:'BD4E:48462:&#x7787
CJK UNIFIED IDEOGRAPH:'BD4F:48463:&#x778C
CJK UNIFIED IDEOGRAPH:'BD50:48464:&#x7791
CJK UNIFIED IDEOGRAPH:'BD51:48465:&#x778B
CJK UNIFIED IDEOGRAPH:'BD52:48466:&#x78CB
CJK UNIFIED IDEOGRAPH:'BD53:48467:&#x78C5
CJK UNIFIED IDEOGRAPH:'BD54:48468:&#x78BA
CJK UNIFIED IDEOGRAPH:'BD55:48469:&#x78CA
CJK UNIFIED IDEOGRAPH:'BD56:48470:&#x78BE
CJK UNIFIED IDEOGRAPH:'BD57:48471:&#x78D5
CJK UNIFIED IDEOGRAPH:'BD58:48472:&#x78BC
CJK UNIFIED IDEOGRAPH:'BD59:48473:&#x78D0
CJK UNIFIED IDEOGRAPH:'BD5A:48474:&#x7A3F
CJK UNIFIED IDEOGRAPH:'BD5B:48475:&#x7A3C
CJK UNIFIED IDEOGRAPH:'BD5C:48476:&#x7A40
CJK UNIFIED IDEOGRAPH:'BD5D:48477:&#x7A3D
CJK UNIFIED IDEOGRAPH:'BD5E:48478:&#x7A37
CJK UNIFIED IDEOGRAPH:'BD5F:48479:&#x7A3B
CJK UNIFIED IDEOGRAPH:'BD60:48480:&#x7AAF
CJK UNIFIED IDEOGRAPH:'BD61:48481:&#x7AAE
CJK UNIFIED IDEOGRAPH:'BD62:48482:&#x7BAD
CJK UNIFIED IDEOGRAPH:'BD63:48483:&#x7BB1
CJK UNIFIED IDEOGRAPH:'BD64:48484:&#x7BC4
CJK UNIFIED IDEOGRAPH:'BD65:48485:&#x7BB4
CJK UNIFIED IDEOGRAPH:'BD66:48486:&#x7BC6
CJK UNIFIED IDEOGRAPH:'BD67:48487:&#x7BC7
CJK UNIFIED IDEOGRAPH:'BD68:48488:&#x7BC1
CJK UNIFIED IDEOGRAPH:'BD69:48489:&#x7BA0
CJK UNIFIED IDEOGRAPH:'BD6A:48490:&#x7BCC
CJK UNIFIED IDEOGRAPH:'BD6B:48491:&#x7CCA
CJK UNIFIED IDEOGRAPH:'BD6C:48492:&#x7DE0
CJK UNIFIED IDEOGRAPH:'BD6D:48493:&#x7DF4
CJK UNIFIED IDEOGRAPH:'BD6E:48494:&#x7DEF
CJK UNIFIED IDEOGRAPH:'BD6F:48495:&#x7DFB
CJK UNIFIED IDEOGRAPH:'BD70:48496:&#x7DD8
CJK UNIFIED IDEOGRAPH:'BD71:48497:&#x7DEC
CJK UNIFIED IDEOGRAPH:'BD72:48498:&#x7DDD
CJK UNIFIED IDEOGRAPH:'BD73:48499:&#x7DE8
CJK UNIFIED IDEOGRAPH:'BD74:48500:&#x7DE3
CJK UNIFIED IDEOGRAPH:'BD75:48501:&#x7DDA
CJK UNIFIED IDEOGRAPH:'BD76:48502:&#x7DDE
CJK UNIFIED IDEOGRAPH:'BD77:48503:&#x7DE9
CJK UNIFIED IDEOGRAPH:'BD78:48504:&#x7D9E
CJK UNIFIED IDEOGRAPH:'BD79:48505:&#x7DD9
CJK UNIFIED IDEOGRAPH:'BD7A:48506:&#x7DF2
CJK UNIFIED IDEOGRAPH:'BD7B:48507:&#x7DF9
CJK UNIFIED IDEOGRAPH:'BD7C:48508:&#x7F75
CJK UNIFIED IDEOGRAPH:'BD7D:48509:&#x7F77
CJK UNIFIED IDEOGRAPH:'BD7E:48510:&#x7FAF
CJK UNIFIED IDEOGRAPH:'BDA1:48545:&#x7FE9
CJK UNIFIED IDEOGRAPH:'BDA2:48546:&#x8026
CJK UNIFIED IDEOGRAPH:'BDA3:48547:&#x819B
CJK UNIFIED IDEOGRAPH:'BDA4:48548:&#x819C
CJK UNIFIED IDEOGRAPH:'BDA5:48549:&#x819D
CJK UNIFIED IDEOGRAPH:'BDA6:48550:&#x81A0
CJK UNIFIED IDEOGRAPH:'BDA7:48551:&#x819A
CJK UNIFIED IDEOGRAPH:'BDA8:48552:&#x8198
CJK UNIFIED IDEOGRAPH:'BDA9:48553:&#x8517
CJK UNIFIED IDEOGRAPH:'BDAA:48554:&#x853D
CJK UNIFIED IDEOGRAPH:'BDAB:48555:&#x851A
CJK UNIFIED IDEOGRAPH:'BDAC:48556:&#x84EE
CJK UNIFIED IDEOGRAPH:'BDAD:48557:&#x852C
CJK UNIFIED IDEOGRAPH:'BDAE:48558:&#x852D
CJK UNIFIED IDEOGRAPH:'BDAF:48559:&#x8513
CJK UNIFIED IDEOGRAPH:'BDB0:48560:&#x8511
CJK UNIFIED IDEOGRAPH:'BDB1:48561:&#x8523
CJK UNIFIED IDEOGRAPH:'BDB2:48562:&#x8521
CJK UNIFIED IDEOGRAPH:'BDB3:48563:&#x8514
CJK UNIFIED IDEOGRAPH:'BDB4:48564:&#x84EC
CJK UNIFIED IDEOGRAPH:'BDB5:48565:&#x8525
CJK UNIFIED IDEOGRAPH:'BDB6:48566:&#x84FF
CJK UNIFIED IDEOGRAPH:'BDB7:48567:&#x8506
CJK UNIFIED IDEOGRAPH:'BDB8:48568:&#x8782
CJK UNIFIED IDEOGRAPH:'BDB9:48569:&#x8774
CJK UNIFIED IDEOGRAPH:'BDBA:48570:&#x8776
CJK UNIFIED IDEOGRAPH:'BDBB:48571:&#x8760
CJK UNIFIED IDEOGRAPH:'BDBC:48572:&#x8766
CJK UNIFIED IDEOGRAPH:'BDBD:48573:&#x8778
CJK UNIFIED IDEOGRAPH:'BDBE:48574:&#x8768
CJK UNIFIED IDEOGRAPH:'BDBF:48575:&#x8759
CJK UNIFIED IDEOGRAPH:'BDC0:48576:&#x8757
CJK UNIFIED IDEOGRAPH:'BDC1:48577:&#x874C
CJK UNIFIED IDEOGRAPH:'BDC2:48578:&#x8753
CJK UNIFIED IDEOGRAPH:'BDC3:48579:&#x885B
CJK UNIFIED IDEOGRAPH:'BDC4:48580:&#x885D
CJK UNIFIED IDEOGRAPH:'BDC5:48581:&#x8910
CJK UNIFIED IDEOGRAPH:'BDC6:48582:&#x8907
CJK UNIFIED IDEOGRAPH:'BDC7:48583:&#x8912
CJK UNIFIED IDEOGRAPH:'BDC8:48584:&#x8913
CJK UNIFIED IDEOGRAPH:'BDC9:48585:&#x8915
CJK UNIFIED IDEOGRAPH:'BDCA:48586:&#x890A
CJK UNIFIED IDEOGRAPH:'BDCB:48587:&#x8ABC
CJK UNIFIED IDEOGRAPH:'BDCC:48588:&#x8AD2
CJK UNIFIED IDEOGRAPH:'BDCD:48589:&#x8AC7
CJK UNIFIED IDEOGRAPH:'BDCE:48590:&#x8AC4
CJK UNIFIED IDEOGRAPH:'BDCF:48591:&#x8A95
CJK UNIFIED IDEOGRAPH:'BDD0:48592:&#x8ACB
CJK UNIFIED IDEOGRAPH:'BDD1:48593:&#x8AF8
CJK UNIFIED IDEOGRAPH:'BDD2:48594:&#x8AB2
CJK UNIFIED IDEOGRAPH:'BDD3:48595:&#x8AC9
CJK UNIFIED IDEOGRAPH:'BDD4:48596:&#x8AC2
CJK UNIFIED IDEOGRAPH:'BDD5:48597:&#x8ABF
CJK UNIFIED IDEOGRAPH:'BDD6:48598:&#x8AB0
CJK UNIFIED IDEOGRAPH:'BDD7:48599:&#x8AD6
CJK UNIFIED IDEOGRAPH:'BDD8:48600:&#x8ACD
CJK UNIFIED IDEOGRAPH:'BDD9:48601:&#x8AB6
CJK UNIFIED IDEOGRAPH:'BDDA:48602:&#x8AB9
CJK UNIFIED IDEOGRAPH:'BDDB:48603:&#x8ADB
CJK UNIFIED IDEOGRAPH:'BDDC:48604:&#x8C4C
CJK UNIFIED IDEOGRAPH:'BDDD:48605:&#x8C4E
CJK UNIFIED IDEOGRAPH:'BDDE:48606:&#x8C6C
CJK UNIFIED IDEOGRAPH:'BDDF:48607:&#x8CE0
CJK UNIFIED IDEOGRAPH:'BDE0:48608:&#x8CDE
CJK UNIFIED IDEOGRAPH:'BDE1:48609:&#x8CE6
CJK UNIFIED IDEOGRAPH:'BDE2:48610:&#x8CE4
CJK UNIFIED IDEOGRAPH:'BDE3:48611:&#x8CEC
CJK UNIFIED IDEOGRAPH:'BDE4:48612:&#x8CED
CJK UNIFIED IDEOGRAPH:'BDE5:48613:&#x8CE2
CJK UNIFIED IDEOGRAPH:'BDE6:48614:&#x8CE3
CJK UNIFIED IDEOGRAPH:'BDE7:48615:&#x8CDC
CJK UNIFIED IDEOGRAPH:'BDE8:48616:&#x8CEA
CJK UNIFIED IDEOGRAPH:'BDE9:48617:&#x8CE1
CJK UNIFIED IDEOGRAPH:'BDEA:48618:&#x8D6D
CJK UNIFIED IDEOGRAPH:'BDEB:48619:&#x8D9F
CJK UNIFIED IDEOGRAPH:'BDEC:48620:&#x8DA3
CJK UNIFIED IDEOGRAPH:'BDED:48621:&#x8E2B
CJK UNIFIED IDEOGRAPH:'BDEE:48622:&#x8E10
CJK UNIFIED IDEOGRAPH:'BDEF:48623:&#x8E1D
CJK UNIFIED IDEOGRAPH:'BDF0:48624:&#x8E22
CJK UNIFIED IDEOGRAPH:'BDF1:48625:&#x8E0F
CJK UNIFIED IDEOGRAPH:'BDF2:48626:&#x8E29
CJK UNIFIED IDEOGRAPH:'BDF3:48627:&#x8E1F
CJK UNIFIED IDEOGRAPH:'BDF4:48628:&#x8E21
CJK UNIFIED IDEOGRAPH:'BDF5:48629:&#x8E1E
CJK UNIFIED IDEOGRAPH:'BDF6:48630:&#x8EBA
CJK UNIFIED IDEOGRAPH:'BDF7:48631:&#x8F1D
CJK UNIFIED IDEOGRAPH:'BDF8:48632:&#x8F1B
CJK UNIFIED IDEOGRAPH:'BDF9:48633:&#x8F1F
CJK UNIFIED IDEOGRAPH:'BDFA:48634:&#x8F29
CJK UNIFIED IDEOGRAPH:'BDFB:48635:&#x8F26
CJK UNIFIED IDEOGRAPH:'BDFC:48636:&#x8F2A
CJK UNIFIED IDEOGRAPH:'BDFD:48637:&#x8F1C
CJK UNIFIED IDEOGRAPH:'BDFE:48638:&#x8F1E
CJK UNIFIED IDEOGRAPH:'BE40:48704:&#x8F25
CJK UNIFIED IDEOGRAPH:'BE41:48705:&#x9069
CJK UNIFIED IDEOGRAPH:'BE42:48706:&#x906E
CJK UNIFIED IDEOGRAPH:'BE43:48707:&#x9068
CJK UNIFIED IDEOGRAPH:'BE44:48708:&#x906D
CJK UNIFIED IDEOGRAPH:'BE45:48709:&#x9077
CJK UNIFIED IDEOGRAPH:'BE46:48710:&#x9130
CJK UNIFIED IDEOGRAPH:'BE47:48711:&#x912D
CJK UNIFIED IDEOGRAPH:'BE48:48712:&#x9127
CJK UNIFIED IDEOGRAPH:'BE49:48713:&#x9131
CJK UNIFIED IDEOGRAPH:'BE4A:48714:&#x9187
CJK UNIFIED IDEOGRAPH:'BE4B:48715:&#x9189
CJK UNIFIED IDEOGRAPH:'BE4C:48716:&#x918B
CJK UNIFIED IDEOGRAPH:'BE4D:48717:&#x9183
CJK UNIFIED IDEOGRAPH:'BE4E:48718:&#x92C5
CJK UNIFIED IDEOGRAPH:'BE4F:48719:&#x92BB
CJK UNIFIED IDEOGRAPH:'BE50:48720:&#x92B7
CJK UNIFIED IDEOGRAPH:'BE51:48721:&#x92EA
CJK UNIFIED IDEOGRAPH:'BE52:48722:&#x92AC
CJK UNIFIED IDEOGRAPH:'BE53:48723:&#x92E4
CJK UNIFIED IDEOGRAPH:'BE54:48724:&#x92C1
CJK UNIFIED IDEOGRAPH:'BE55:48725:&#x92B3
CJK UNIFIED IDEOGRAPH:'BE56:48726:&#x92BC
CJK UNIFIED IDEOGRAPH:'BE57:48727:&#x92D2
CJK UNIFIED IDEOGRAPH:'BE58:48728:&#x92C7
CJK UNIFIED IDEOGRAPH:'BE59:48729:&#x92F0
CJK UNIFIED IDEOGRAPH:'BE5A:48730:&#x92B2
CJK UNIFIED IDEOGRAPH:'BE5B:48731:&#x95AD
CJK UNIFIED IDEOGRAPH:'BE5C:48732:&#x95B1
CJK UNIFIED IDEOGRAPH:'BE5D:48733:&#x9704
CJK UNIFIED IDEOGRAPH:'BE5E:48734:&#x9706
CJK UNIFIED IDEOGRAPH:'BE5F:48735:&#x9707
CJK UNIFIED IDEOGRAPH:'BE60:48736:&#x9709
CJK UNIFIED IDEOGRAPH:'BE61:48737:&#x9760
CJK UNIFIED IDEOGRAPH:'BE62:48738:&#x978D
CJK UNIFIED IDEOGRAPH:'BE63:48739:&#x978B
CJK UNIFIED IDEOGRAPH:'BE64:48740:&#x978F
CJK UNIFIED IDEOGRAPH:'BE65:48741:&#x9821
CJK UNIFIED IDEOGRAPH:'BE66:48742:&#x982B
CJK UNIFIED IDEOGRAPH:'BE67:48743:&#x981C
CJK UNIFIED IDEOGRAPH:'BE68:48744:&#x98B3
CJK UNIFIED IDEOGRAPH:'BE69:48745:&#x990A
CJK UNIFIED IDEOGRAPH:'BE6A:48746:&#x9913
CJK UNIFIED IDEOGRAPH:'BE6B:48747:&#x9912
CJK UNIFIED IDEOGRAPH:'BE6C:48748:&#x9918
CJK UNIFIED IDEOGRAPH:'BE6D:48749:&#x99DD
CJK UNIFIED IDEOGRAPH:'BE6E:48750:&#x99D0
CJK UNIFIED IDEOGRAPH:'BE6F:48751:&#x99DF
CJK UNIFIED IDEOGRAPH:'BE70:48752:&#x99DB
CJK UNIFIED IDEOGRAPH:'BE71:48753:&#x99D1
CJK UNIFIED IDEOGRAPH:'BE72:48754:&#x99D5
CJK UNIFIED IDEOGRAPH:'BE73:48755:&#x99D2
CJK UNIFIED IDEOGRAPH:'BE74:48756:&#x99D9
CJK UNIFIED IDEOGRAPH:'BE75:48757:&#x9AB7
CJK UNIFIED IDEOGRAPH:'BE76:48758:&#x9AEE
CJK UNIFIED IDEOGRAPH:'BE77:48759:&#x9AEF
CJK UNIFIED IDEOGRAPH:'BE78:48760:&#x9B27
CJK UNIFIED IDEOGRAPH:'BE79:48761:&#x9B45
CJK UNIFIED IDEOGRAPH:'BE7A:48762:&#x9B44
CJK UNIFIED IDEOGRAPH:'BE7B:48763:&#x9B77
CJK UNIFIED IDEOGRAPH:'BE7C:48764:&#x9B6F
CJK UNIFIED IDEOGRAPH:'BE7D:48765:&#x9D06
CJK UNIFIED IDEOGRAPH:'BE7E:48766:&#x9D09
CJK UNIFIED IDEOGRAPH:'BEA1:48801:&#x9D03
CJK UNIFIED IDEOGRAPH:'BEA2:48802:&#x9EA9
CJK UNIFIED IDEOGRAPH:'BEA3:48803:&#x9EBE
CJK UNIFIED IDEOGRAPH:'BEA4:48804:&#x9ECE
CJK UNIFIED IDEOGRAPH:'BEA5:48805:&#x58A8
CJK UNIFIED IDEOGRAPH:'BEA6:48806:&#x9F52
CJK UNIFIED IDEOGRAPH:'BEA7:48807:&#x5112
CJK UNIFIED IDEOGRAPH:'BEA8:48808:&#x5118
CJK UNIFIED IDEOGRAPH:'BEA9:48809:&#x5114
CJK UNIFIED IDEOGRAPH:'BEAA:48810:&#x5110
CJK UNIFIED IDEOGRAPH:'BEAB:48811:&#x5115
CJK UNIFIED IDEOGRAPH:'BEAC:48812:&#x5180
CJK UNIFIED IDEOGRAPH:'BEAD:48813:&#x51AA
CJK UNIFIED IDEOGRAPH:'BEAE:48814:&#x51DD
CJK UNIFIED IDEOGRAPH:'BEAF:48815:&#x5291
CJK UNIFIED IDEOGRAPH:'BEB0:48816:&#x5293
CJK UNIFIED IDEOGRAPH:'BEB1:48817:&#x52F3
CJK UNIFIED IDEOGRAPH:'BEB2:48818:&#x5659
CJK UNIFIED IDEOGRAPH:'BEB3:48819:&#x566B
CJK UNIFIED IDEOGRAPH:'BEB4:48820:&#x5679
CJK UNIFIED IDEOGRAPH:'BEB5:48821:&#x5669
CJK UNIFIED IDEOGRAPH:'BEB6:48822:&#x5664
CJK UNIFIED IDEOGRAPH:'BEB7:48823:&#x5678
CJK UNIFIED IDEOGRAPH:'BEB8:48824:&#x566A
CJK UNIFIED IDEOGRAPH:'BEB9:48825:&#x5668
CJK UNIFIED IDEOGRAPH:'BEBA:48826:&#x5665
CJK UNIFIED IDEOGRAPH:'BEBB:48827:&#x5671
CJK UNIFIED IDEOGRAPH:'BEBC:48828:&#x566F
CJK UNIFIED IDEOGRAPH:'BEBD:48829:&#x566C
CJK UNIFIED IDEOGRAPH:'BEBE:48830:&#x5662
CJK UNIFIED IDEOGRAPH:'BEBF:48831:&#x5676
CJK UNIFIED IDEOGRAPH:'BEC0:48832:&#x58C1
CJK UNIFIED IDEOGRAPH:'BEC1:48833:&#x58BE
CJK UNIFIED IDEOGRAPH:'BEC2:48834:&#x58C7
CJK UNIFIED IDEOGRAPH:'BEC3:48835:&#x58C5
CJK UNIFIED IDEOGRAPH:'BEC4:48836:&#x596E
CJK UNIFIED IDEOGRAPH:'BEC5:48837:&#x5B1D
CJK UNIFIED IDEOGRAPH:'BEC6:48838:&#x5B34
CJK UNIFIED IDEOGRAPH:'BEC7:48839:&#x5B78
CJK UNIFIED IDEOGRAPH:'BEC8:48840:&#x5BF0
CJK UNIFIED IDEOGRAPH:'BEC9:48841:&#x5C0E
CJK UNIFIED IDEOGRAPH:'BECA:48842:&#x5F4A
CJK UNIFIED IDEOGRAPH:'BECB:48843:&#x61B2
CJK UNIFIED IDEOGRAPH:'BECC:48844:&#x6191
CJK UNIFIED IDEOGRAPH:'BECD:48845:&#x61A9
CJK UNIFIED IDEOGRAPH:'BECE:48846:&#x618A
CJK UNIFIED IDEOGRAPH:'BECF:48847:&#x61CD
CJK UNIFIED IDEOGRAPH:'BED0:48848:&#x61B6
CJK UNIFIED IDEOGRAPH:'BED1:48849:&#x61BE
CJK UNIFIED IDEOGRAPH:'BED2:48850:&#x61CA
CJK UNIFIED IDEOGRAPH:'BED3:48851:&#x61C8
CJK UNIFIED IDEOGRAPH:'BED4:48852:&#x6230
CJK UNIFIED IDEOGRAPH:'BED5:48853:&#x64C5
CJK UNIFIED IDEOGRAPH:'BED6:48854:&#x64C1
CJK UNIFIED IDEOGRAPH:'BED7:48855:&#x64CB
CJK UNIFIED IDEOGRAPH:'BED8:48856:&#x64BB
CJK UNIFIED IDEOGRAPH:'BED9:48857:&#x64BC
CJK UNIFIED IDEOGRAPH:'BEDA:48858:&#x64DA
CJK UNIFIED IDEOGRAPH:'BEDB:48859:&#x64C4
CJK UNIFIED IDEOGRAPH:'BEDC:48860:&#x64C7
CJK UNIFIED IDEOGRAPH:'BEDD:48861:&#x64C2
CJK UNIFIED IDEOGRAPH:'BEDE:48862:&#x64CD
CJK UNIFIED IDEOGRAPH:'BEDF:48863:&#x64BF
CJK UNIFIED IDEOGRAPH:'BEE0:48864:&#x64D2
CJK UNIFIED IDEOGRAPH:'BEE1:48865:&#x64D4
CJK UNIFIED IDEOGRAPH:'BEE2:48866:&#x64BE
CJK UNIFIED IDEOGRAPH:'BEE3:48867:&#x6574
CJK UNIFIED IDEOGRAPH:'BEE4:48868:&#x66C6
CJK UNIFIED IDEOGRAPH:'BEE5:48869:&#x66C9
CJK UNIFIED IDEOGRAPH:'BEE6:48870:&#x66B9
CJK UNIFIED IDEOGRAPH:'BEE7:48871:&#x66C4
CJK UNIFIED IDEOGRAPH:'BEE8:48872:&#x66C7
CJK UNIFIED IDEOGRAPH:'BEE9:48873:&#x66B8
CJK UNIFIED IDEOGRAPH:'BEEA:48874:&#x6A3D
CJK UNIFIED IDEOGRAPH:'BEEB:48875:&#x6A38
CJK UNIFIED IDEOGRAPH:'BEEC:48876:&#x6A3A
CJK UNIFIED IDEOGRAPH:'BEED:48877:&#x6A59
CJK UNIFIED IDEOGRAPH:'BEEE:48878:&#x6A6B
CJK UNIFIED IDEOGRAPH:'BEEF:48879:&#x6A58
CJK UNIFIED IDEOGRAPH:'BEF0:48880:&#x6A39
CJK UNIFIED IDEOGRAPH:'BEF1:48881:&#x6A44
CJK UNIFIED IDEOGRAPH:'BEF2:48882:&#x6A62
CJK UNIFIED IDEOGRAPH:'BEF3:48883:&#x6A61
CJK UNIFIED IDEOGRAPH:'BEF4:48884:&#x6A4B
CJK UNIFIED IDEOGRAPH:'BEF5:48885:&#x6A47
CJK UNIFIED IDEOGRAPH:'BEF6:48886:&#x6A35
CJK UNIFIED IDEOGRAPH:'BEF7:48887:&#x6A5F
CJK UNIFIED IDEOGRAPH:'BEF8:48888:&#x6A48
CJK UNIFIED IDEOGRAPH:'BEF9:48889:&#x6B59
CJK UNIFIED IDEOGRAPH:'BEFA:48890:&#x6B77
CJK UNIFIED IDEOGRAPH:'BEFB:48891:&#x6C05
CJK UNIFIED IDEOGRAPH:'BEFC:48892:&#x6FC2
CJK UNIFIED IDEOGRAPH:'BEFD:48893:&#x6FB1
CJK UNIFIED IDEOGRAPH:'BEFE:48894:&#x6FA1
CJK UNIFIED IDEOGRAPH:'BF40:48960:&#x6FC3
CJK UNIFIED IDEOGRAPH:'BF41:48961:&#x6FA4
CJK UNIFIED IDEOGRAPH:'BF42:48962:&#x6FC1
CJK UNIFIED IDEOGRAPH:'BF43:48963:&#x6FA7
CJK UNIFIED IDEOGRAPH:'BF44:48964:&#x6FB3
CJK UNIFIED IDEOGRAPH:'BF45:48965:&#x6FC0
CJK UNIFIED IDEOGRAPH:'BF46:48966:&#x6FB9
CJK UNIFIED IDEOGRAPH:'BF47:48967:&#x6FB6
CJK UNIFIED IDEOGRAPH:'BF48:48968:&#x6FA6
CJK UNIFIED IDEOGRAPH:'BF49:48969:&#x6FA0
CJK UNIFIED IDEOGRAPH:'BF4A:48970:&#x6FB4
CJK UNIFIED IDEOGRAPH:'BF4B:48971:&#x71BE
CJK UNIFIED IDEOGRAPH:'BF4C:48972:&#x71C9
CJK UNIFIED IDEOGRAPH:'BF4D:48973:&#x71D0
CJK UNIFIED IDEOGRAPH:'BF4E:48974:&#x71D2
CJK UNIFIED IDEOGRAPH:'BF4F:48975:&#x71C8
CJK UNIFIED IDEOGRAPH:'BF50:48976:&#x71D5
CJK UNIFIED IDEOGRAPH:'BF51:48977:&#x71B9
CJK UNIFIED IDEOGRAPH:'BF52:48978:&#x71CE
CJK UNIFIED IDEOGRAPH:'BF53:48979:&#x71D9
CJK UNIFIED IDEOGRAPH:'BF54:48980:&#x71DC
CJK UNIFIED IDEOGRAPH:'BF55:48981:&#x71C3
CJK UNIFIED IDEOGRAPH:'BF56:48982:&#x71C4
CJK UNIFIED IDEOGRAPH:'BF57:48983:&#x7368
CJK UNIFIED IDEOGRAPH:'BF58:48984:&#x749C
CJK UNIFIED IDEOGRAPH:'BF59:48985:&#x74A3
CJK UNIFIED IDEOGRAPH:'BF5A:48986:&#x7498
CJK UNIFIED IDEOGRAPH:'BF5B:48987:&#x749F
CJK UNIFIED IDEOGRAPH:'BF5C:48988:&#x749E
CJK UNIFIED IDEOGRAPH:'BF5D:48989:&#x74E2
CJK UNIFIED IDEOGRAPH:'BF5E:48990:&#x750C
CJK UNIFIED IDEOGRAPH:'BF5F:48991:&#x750D
CJK UNIFIED IDEOGRAPH:'BF60:48992:&#x7634
CJK UNIFIED IDEOGRAPH:'BF61:48993:&#x7638
CJK UNIFIED IDEOGRAPH:'BF62:48994:&#x763A
CJK UNIFIED IDEOGRAPH:'BF63:48995:&#x76E7
CJK UNIFIED IDEOGRAPH:'BF64:48996:&#x76E5
CJK UNIFIED IDEOGRAPH:'BF65:48997:&#x77A0
CJK UNIFIED IDEOGRAPH:'BF66:48998:&#x779E
CJK UNIFIED IDEOGRAPH:'BF67:48999:&#x779F
CJK UNIFIED IDEOGRAPH:'BF68:49000:&#x77A5
CJK UNIFIED IDEOGRAPH:'BF69:49001:&#x78E8
CJK UNIFIED IDEOGRAPH:'BF6A:49002:&#x78DA
CJK UNIFIED IDEOGRAPH:'BF6B:49003:&#x78EC
CJK UNIFIED IDEOGRAPH:'BF6C:49004:&#x78E7
CJK UNIFIED IDEOGRAPH:'BF6D:49005:&#x79A6
CJK UNIFIED IDEOGRAPH:'BF6E:49006:&#x7A4D
CJK UNIFIED IDEOGRAPH:'BF6F:49007:&#x7A4E
CJK UNIFIED IDEOGRAPH:'BF70:49008:&#x7A46
CJK UNIFIED IDEOGRAPH:'BF71:49009:&#x7A4C
CJK UNIFIED IDEOGRAPH:'BF72:49010:&#x7A4B
CJK UNIFIED IDEOGRAPH:'BF73:49011:&#x7ABA
CJK UNIFIED IDEOGRAPH:'BF74:49012:&#x7BD9
CJK UNIFIED IDEOGRAPH:'BF75:49013:&#x7C11
CJK UNIFIED IDEOGRAPH:'BF76:49014:&#x7BC9
CJK UNIFIED IDEOGRAPH:'BF77:49015:&#x7BE4
CJK UNIFIED IDEOGRAPH:'BF78:49016:&#x7BDB
CJK UNIFIED IDEOGRAPH:'BF79:49017:&#x7BE1
CJK UNIFIED IDEOGRAPH:'BF7A:49018:&#x7BE9
CJK UNIFIED IDEOGRAPH:'BF7B:49019:&#x7BE6
CJK UNIFIED IDEOGRAPH:'BF7C:49020:&#x7CD5
CJK UNIFIED IDEOGRAPH:'BF7D:49021:&#x7CD6
CJK UNIFIED IDEOGRAPH:'BF7E:49022:&#x7E0A
CJK UNIFIED IDEOGRAPH:'BFA1:49057:&#x7E11
CJK UNIFIED IDEOGRAPH:'BFA2:49058:&#x7E08
CJK UNIFIED IDEOGRAPH:'BFA3:49059:&#x7E1B
CJK UNIFIED IDEOGRAPH:'BFA4:49060:&#x7E23
CJK UNIFIED IDEOGRAPH:'BFA5:49061:&#x7E1E
CJK UNIFIED IDEOGRAPH:'BFA6:49062:&#x7E1D
CJK UNIFIED IDEOGRAPH:'BFA7:49063:&#x7E09
CJK UNIFIED IDEOGRAPH:'BFA8:49064:&#x7E10
CJK UNIFIED IDEOGRAPH:'BFA9:49065:&#x7F79
CJK UNIFIED IDEOGRAPH:'BFAA:49066:&#x7FB2
CJK UNIFIED IDEOGRAPH:'BFAB:49067:&#x7FF0
CJK UNIFIED IDEOGRAPH:'BFAC:49068:&#x7FF1
CJK UNIFIED IDEOGRAPH:'BFAD:49069:&#x7FEE
CJK UNIFIED IDEOGRAPH:'BFAE:49070:&#x8028
CJK UNIFIED IDEOGRAPH:'BFAF:49071:&#x81B3
CJK UNIFIED IDEOGRAPH:'BFB0:49072:&#x81A9
CJK UNIFIED IDEOGRAPH:'BFB1:49073:&#x81A8
CJK UNIFIED IDEOGRAPH:'BFB2:49074:&#x81FB
CJK UNIFIED IDEOGRAPH:'BFB3:49075:&#x8208
CJK UNIFIED IDEOGRAPH:'BFB4:49076:&#x8258
CJK UNIFIED IDEOGRAPH:'BFB5:49077:&#x8259
CJK UNIFIED IDEOGRAPH:'BFB6:49078:&#x854A
CJK UNIFIED IDEOGRAPH:'BFB7:49079:&#x8559
CJK UNIFIED IDEOGRAPH:'BFB8:49080:&#x8548
CJK UNIFIED IDEOGRAPH:'BFB9:49081:&#x8568
CJK UNIFIED IDEOGRAPH:'BFBA:49082:&#x8569
CJK UNIFIED IDEOGRAPH:'BFBB:49083:&#x8543
CJK UNIFIED IDEOGRAPH:'BFBC:49084:&#x8549
CJK UNIFIED IDEOGRAPH:'BFBD:49085:&#x856D
CJK UNIFIED IDEOGRAPH:'BFBE:49086:&#x856A
CJK UNIFIED IDEOGRAPH:'BFBF:49087:&#x855E
CJK UNIFIED IDEOGRAPH:'BFC0:49088:&#x8783
CJK UNIFIED IDEOGRAPH:'BFC1:49089:&#x879F
CJK UNIFIED IDEOGRAPH:'BFC2:49090:&#x879E
CJK UNIFIED IDEOGRAPH:'BFC3:49091:&#x87A2
CJK UNIFIED IDEOGRAPH:'BFC4:49092:&#x878D
CJK UNIFIED IDEOGRAPH:'BFC5:49093:&#x8861
CJK UNIFIED IDEOGRAPH:'BFC6:49094:&#x892A
CJK UNIFIED IDEOGRAPH:'BFC7:49095:&#x8932
CJK UNIFIED IDEOGRAPH:'BFC8:49096:&#x8925
CJK UNIFIED IDEOGRAPH:'BFC9:49097:&#x892B
CJK UNIFIED IDEOGRAPH:'BFCA:49098:&#x8921
CJK UNIFIED IDEOGRAPH:'BFCB:49099:&#x89AA
CJK UNIFIED IDEOGRAPH:'BFCC:49100:&#x89A6
CJK UNIFIED IDEOGRAPH:'BFCD:49101:&#x8AE6
CJK UNIFIED IDEOGRAPH:'BFCE:49102:&#x8AFA
CJK UNIFIED IDEOGRAPH:'BFCF:49103:&#x8AEB
CJK UNIFIED IDEOGRAPH:'BFD0:49104:&#x8AF1
CJK UNIFIED IDEOGRAPH:'BFD1:49105:&#x8B00
CJK UNIFIED IDEOGRAPH:'BFD2:49106:&#x8ADC
CJK UNIFIED IDEOGRAPH:'BFD3:49107:&#x8AE7
CJK UNIFIED IDEOGRAPH:'BFD4:49108:&#x8AEE
CJK UNIFIED IDEOGRAPH:'BFD5:49109:&#x8AFE
CJK UNIFIED IDEOGRAPH:'BFD6:49110:&#x8B01
CJK UNIFIED IDEOGRAPH:'BFD7:49111:&#x8B02
CJK UNIFIED IDEOGRAPH:'BFD8:49112:&#x8AF7
CJK UNIFIED IDEOGRAPH:'BFD9:49113:&#x8AED
CJK UNIFIED IDEOGRAPH:'BFDA:49114:&#x8AF3
CJK UNIFIED IDEOGRAPH:'BFDB:49115:&#x8AF6
CJK UNIFIED IDEOGRAPH:'BFDC:49116:&#x8AFC
CJK UNIFIED IDEOGRAPH:'BFDD:49117:&#x8C6B
CJK UNIFIED IDEOGRAPH:'BFDE:49118:&#x8C6D
CJK UNIFIED IDEOGRAPH:'BFDF:49119:&#x8C93
CJK UNIFIED IDEOGRAPH:'BFE0:49120:&#x8CF4
CJK UNIFIED IDEOGRAPH:'BFE1:49121:&#x8E44
CJK UNIFIED IDEOGRAPH:'BFE2:49122:&#x8E31
CJK UNIFIED IDEOGRAPH:'BFE3:49123:&#x8E34
CJK UNIFIED IDEOGRAPH:'BFE4:49124:&#x8E42
CJK UNIFIED IDEOGRAPH:'BFE5:49125:&#x8E39
CJK UNIFIED IDEOGRAPH:'BFE6:49126:&#x8E35
CJK UNIFIED IDEOGRAPH:'BFE7:49127:&#x8F3B
CJK UNIFIED IDEOGRAPH:'BFE8:49128:&#x8F2F
CJK UNIFIED IDEOGRAPH:'BFE9:49129:&#x8F38
CJK UNIFIED IDEOGRAPH:'BFEA:49130:&#x8F33
CJK UNIFIED IDEOGRAPH:'BFEB:49131:&#x8FA8
CJK UNIFIED IDEOGRAPH:'BFEC:49132:&#x8FA6
CJK UNIFIED IDEOGRAPH:'BFED:49133:&#x9075
CJK UNIFIED IDEOGRAPH:'BFEE:49134:&#x9074
CJK UNIFIED IDEOGRAPH:'BFEF:49135:&#x9078
CJK UNIFIED IDEOGRAPH:'BFF0:49136:&#x9072
CJK UNIFIED IDEOGRAPH:'BFF1:49137:&#x907C
CJK UNIFIED IDEOGRAPH:'BFF2:49138:&#x907A
CJK UNIFIED IDEOGRAPH:'BFF3:49139:&#x9134
CJK UNIFIED IDEOGRAPH:'BFF4:49140:&#x9192
CJK UNIFIED IDEOGRAPH:'BFF5:49141:&#x9320
CJK UNIFIED IDEOGRAPH:'BFF6:49142:&#x9336
CJK UNIFIED IDEOGRAPH:'BFF7:49143:&#x92F8
CJK UNIFIED IDEOGRAPH:'BFF8:49144:&#x9333
CJK UNIFIED IDEOGRAPH:'BFF9:49145:&#x932F
CJK UNIFIED IDEOGRAPH:'BFFA:49146:&#x9322
CJK UNIFIED IDEOGRAPH:'BFFB:49147:&#x92FC
CJK UNIFIED IDEOGRAPH:'BFFC:49148:&#x932B
CJK UNIFIED IDEOGRAPH:'BFFD:49149:&#x9304
CJK UNIFIED IDEOGRAPH:'BFFE:49150:&#x931A
CJK UNIFIED IDEOGRAPH:'C040:49216:&#x9310
CJK UNIFIED IDEOGRAPH:'C041:49217:&#x9326
CJK UNIFIED IDEOGRAPH:'C042:49218:&#x9321
CJK UNIFIED IDEOGRAPH:'C043:49219:&#x9315
CJK UNIFIED IDEOGRAPH:'C044:49220:&#x932E
CJK UNIFIED IDEOGRAPH:'C045:49221:&#x9319
CJK UNIFIED IDEOGRAPH:'C046:49222:&#x95BB
CJK UNIFIED IDEOGRAPH:'C047:49223:&#x96A7
CJK UNIFIED IDEOGRAPH:'C048:49224:&#x96A8
CJK UNIFIED IDEOGRAPH:'C049:49225:&#x96AA
CJK UNIFIED IDEOGRAPH:'C04A:49226:&#x96D5
CJK UNIFIED IDEOGRAPH:'C04B:49227:&#x970E
CJK UNIFIED IDEOGRAPH:'C04C:49228:&#x9711
CJK UNIFIED IDEOGRAPH:'C04D:49229:&#x9716
CJK UNIFIED IDEOGRAPH:'C04E:49230:&#x970D
CJK UNIFIED IDEOGRAPH:'C04F:49231:&#x9713
CJK UNIFIED IDEOGRAPH:'C050:49232:&#x970F
CJK UNIFIED IDEOGRAPH:'C051:49233:&#x975B
CJK UNIFIED IDEOGRAPH:'C052:49234:&#x975C
CJK UNIFIED IDEOGRAPH:'C053:49235:&#x9766
CJK UNIFIED IDEOGRAPH:'C054:49236:&#x9798
CJK UNIFIED IDEOGRAPH:'C055:49237:&#x9830
CJK UNIFIED IDEOGRAPH:'C056:49238:&#x9838
CJK UNIFIED IDEOGRAPH:'C057:49239:&#x983B
CJK UNIFIED IDEOGRAPH:'C058:49240:&#x9837
CJK UNIFIED IDEOGRAPH:'C059:49241:&#x982D
CJK UNIFIED IDEOGRAPH:'C05A:49242:&#x9839
CJK UNIFIED IDEOGRAPH:'C05B:49243:&#x9824
CJK UNIFIED IDEOGRAPH:'C05C:49244:&#x9910
CJK UNIFIED IDEOGRAPH:'C05D:49245:&#x9928
CJK UNIFIED IDEOGRAPH:'C05E:49246:&#x991E
CJK UNIFIED IDEOGRAPH:'C05F:49247:&#x991B
CJK UNIFIED IDEOGRAPH:'C060:49248:&#x9921
CJK UNIFIED IDEOGRAPH:'C061:49249:&#x991A
CJK UNIFIED IDEOGRAPH:'C062:49250:&#x99ED
CJK UNIFIED IDEOGRAPH:'C063:49251:&#x99E2
CJK UNIFIED IDEOGRAPH:'C064:49252:&#x99F1
CJK UNIFIED IDEOGRAPH:'C065:49253:&#x9AB8
CJK UNIFIED IDEOGRAPH:'C066:49254:&#x9ABC
CJK UNIFIED IDEOGRAPH:'C067:49255:&#x9AFB
CJK UNIFIED IDEOGRAPH:'C068:49256:&#x9AED
CJK UNIFIED IDEOGRAPH:'C069:49257:&#x9B28
CJK UNIFIED IDEOGRAPH:'C06A:49258:&#x9B91
CJK UNIFIED IDEOGRAPH:'C06B:49259:&#x9D15
CJK UNIFIED IDEOGRAPH:'C06C:49260:&#x9D23
CJK UNIFIED IDEOGRAPH:'C06D:49261:&#x9D26
CJK UNIFIED IDEOGRAPH:'C06E:49262:&#x9D28
CJK UNIFIED IDEOGRAPH:'C06F:49263:&#x9D12
CJK UNIFIED IDEOGRAPH:'C070:49264:&#x9D1B
CJK UNIFIED IDEOGRAPH:'C071:49265:&#x9ED8
CJK UNIFIED IDEOGRAPH:'C072:49266:&#x9ED4
CJK UNIFIED IDEOGRAPH:'C073:49267:&#x9F8D
CJK UNIFIED IDEOGRAPH:'C074:49268:&#x9F9C
CJK UNIFIED IDEOGRAPH:'C075:49269:&#x512A
CJK UNIFIED IDEOGRAPH:'C076:49270:&#x511F
CJK UNIFIED IDEOGRAPH:'C077:49271:&#x5121
CJK UNIFIED IDEOGRAPH:'C078:49272:&#x5132
CJK UNIFIED IDEOGRAPH:'C079:49273:&#x52F5
CJK UNIFIED IDEOGRAPH:'C07A:49274:&#x568E
CJK UNIFIED IDEOGRAPH:'C07B:49275:&#x5680
CJK UNIFIED IDEOGRAPH:'C07C:49276:&#x5690
CJK UNIFIED IDEOGRAPH:'C07D:49277:&#x5685
CJK UNIFIED IDEOGRAPH:'C07E:49278:&#x5687
CJK UNIFIED IDEOGRAPH:'C0A1:49313:&#x568F
CJK UNIFIED IDEOGRAPH:'C0A2:49314:&#x58D5
CJK UNIFIED IDEOGRAPH:'C0A3:49315:&#x58D3
CJK UNIFIED IDEOGRAPH:'C0A4:49316:&#x58D1
CJK UNIFIED IDEOGRAPH:'C0A5:49317:&#x58CE
CJK UNIFIED IDEOGRAPH:'C0A6:49318:&#x5B30
CJK UNIFIED IDEOGRAPH:'C0A7:49319:&#x5B2A
CJK UNIFIED IDEOGRAPH:'C0A8:49320:&#x5B24
CJK UNIFIED IDEOGRAPH:'C0A9:49321:&#x5B7A
CJK UNIFIED IDEOGRAPH:'C0AA:49322:&#x5C37
CJK UNIFIED IDEOGRAPH:'C0AB:49323:&#x5C68
CJK UNIFIED IDEOGRAPH:'C0AC:49324:&#x5DBC
CJK UNIFIED IDEOGRAPH:'C0AD:49325:&#x5DBA
CJK UNIFIED IDEOGRAPH:'C0AE:49326:&#x5DBD
CJK UNIFIED IDEOGRAPH:'C0AF:49327:&#x5DB8
CJK UNIFIED IDEOGRAPH:'C0B0:49328:&#x5E6B
CJK UNIFIED IDEOGRAPH:'C0B1:49329:&#x5F4C
CJK UNIFIED IDEOGRAPH:'C0B2:49330:&#x5FBD
CJK UNIFIED IDEOGRAPH:'C0B3:49331:&#x61C9
CJK UNIFIED IDEOGRAPH:'C0B4:49332:&#x61C2
CJK UNIFIED IDEOGRAPH:'C0B5:49333:&#x61C7
CJK UNIFIED IDEOGRAPH:'C0B6:49334:&#x61E6
CJK UNIFIED IDEOGRAPH:'C0B7:49335:&#x61CB
CJK UNIFIED IDEOGRAPH:'C0B8:49336:&#x6232
CJK UNIFIED IDEOGRAPH:'C0B9:49337:&#x6234
CJK UNIFIED IDEOGRAPH:'C0BA:49338:&#x64CE
CJK UNIFIED IDEOGRAPH:'C0BB:49339:&#x64CA
CJK UNIFIED IDEOGRAPH:'C0BC:49340:&#x64D8
CJK UNIFIED IDEOGRAPH:'C0BD:49341:&#x64E0
CJK UNIFIED IDEOGRAPH:'C0BE:49342:&#x64F0
CJK UNIFIED IDEOGRAPH:'C0BF:49343:&#x64E6
CJK UNIFIED IDEOGRAPH:'C0C0:49344:&#x64EC
CJK UNIFIED IDEOGRAPH:'C0C1:49345:&#x64F1
CJK UNIFIED IDEOGRAPH:'C0C2:49346:&#x64E2
CJK UNIFIED IDEOGRAPH:'C0C3:49347:&#x64ED
CJK UNIFIED IDEOGRAPH:'C0C4:49348:&#x6582
CJK UNIFIED IDEOGRAPH:'C0C5:49349:&#x6583
CJK UNIFIED IDEOGRAPH:'C0C6:49350:&#x66D9
CJK UNIFIED IDEOGRAPH:'C0C7:49351:&#x66D6
CJK UNIFIED IDEOGRAPH:'C0C8:49352:&#x6A80
CJK UNIFIED IDEOGRAPH:'C0C9:49353:&#x6A94
CJK UNIFIED IDEOGRAPH:'C0CA:49354:&#x6A84
CJK UNIFIED IDEOGRAPH:'C0CB:49355:&#x6AA2
CJK UNIFIED IDEOGRAPH:'C0CC:49356:&#x6A9C
CJK UNIFIED IDEOGRAPH:'C0CD:49357:&#x6ADB
CJK UNIFIED IDEOGRAPH:'C0CE:49358:&#x6AA3
CJK UNIFIED IDEOGRAPH:'C0CF:49359:&#x6A7E
CJK UNIFIED IDEOGRAPH:'C0D0:49360:&#x6A97
CJK UNIFIED IDEOGRAPH:'C0D1:49361:&#x6A90
CJK UNIFIED IDEOGRAPH:'C0D2:49362:&#x6AA0
CJK UNIFIED IDEOGRAPH:'C0D3:49363:&#x6B5C
CJK UNIFIED IDEOGRAPH:'C0D4:49364:&#x6BAE
CJK UNIFIED IDEOGRAPH:'C0D5:49365:&#x6BDA
CJK UNIFIED IDEOGRAPH:'C0D6:49366:&#x6C08
CJK UNIFIED IDEOGRAPH:'C0D7:49367:&#x6FD8
CJK UNIFIED IDEOGRAPH:'C0D8:49368:&#x6FF1
CJK UNIFIED IDEOGRAPH:'C0D9:49369:&#x6FDF
CJK UNIFIED IDEOGRAPH:'C0DA:49370:&#x6FE0
CJK UNIFIED IDEOGRAPH:'C0DB:49371:&#x6FDB
CJK UNIFIED IDEOGRAPH:'C0DC:49372:&#x6FE4
CJK UNIFIED IDEOGRAPH:'C0DD:49373:&#x6FEB
CJK UNIFIED IDEOGRAPH:'C0DE:49374:&#x6FEF
CJK UNIFIED IDEOGRAPH:'C0DF:49375:&#x6F80
CJK UNIFIED IDEOGRAPH:'C0E0:49376:&#x6FEC
CJK UNIFIED IDEOGRAPH:'C0E1:49377:&#x6FE1
CJK UNIFIED IDEOGRAPH:'C0E2:49378:&#x6FE9
CJK UNIFIED IDEOGRAPH:'C0E3:49379:&#x6FD5
CJK UNIFIED IDEOGRAPH:'C0E4:49380:&#x6FEE
CJK UNIFIED IDEOGRAPH:'C0E5:49381:&#x6FF0
CJK UNIFIED IDEOGRAPH:'C0E6:49382:&#x71E7
CJK UNIFIED IDEOGRAPH:'C0E7:49383:&#x71DF
CJK UNIFIED IDEOGRAPH:'C0E8:49384:&#x71EE
CJK UNIFIED IDEOGRAPH:'C0E9:49385:&#x71E6
CJK UNIFIED IDEOGRAPH:'C0EA:49386:&#x71E5
CJK UNIFIED IDEOGRAPH:'C0EB:49387:&#x71ED
CJK UNIFIED IDEOGRAPH:'C0EC:49388:&#x71EC
CJK UNIFIED IDEOGRAPH:'C0ED:49389:&#x71F4
CJK UNIFIED IDEOGRAPH:'C0EE:49390:&#x71E0
CJK UNIFIED IDEOGRAPH:'C0EF:49391:&#x7235
CJK UNIFIED IDEOGRAPH:'C0F0:49392:&#x7246
CJK UNIFIED IDEOGRAPH:'C0F1:49393:&#x7370
CJK UNIFIED IDEOGRAPH:'C0F2:49394:&#x7372
CJK UNIFIED IDEOGRAPH:'C0F3:49395:&#x74A9
CJK UNIFIED IDEOGRAPH:'C0F4:49396:&#x74B0
CJK UNIFIED IDEOGRAPH:'C0F5:49397:&#x74A6
CJK UNIFIED IDEOGRAPH:'C0F6:49398:&#x74A8
CJK UNIFIED IDEOGRAPH:'C0F7:49399:&#x7646
CJK UNIFIED IDEOGRAPH:'C0F8:49400:&#x7642
CJK UNIFIED IDEOGRAPH:'C0F9:49401:&#x764C
CJK UNIFIED IDEOGRAPH:'C0FA:49402:&#x76EA
CJK UNIFIED IDEOGRAPH:'C0FB:49403:&#x77B3
CJK UNIFIED IDEOGRAPH:'C0FC:49404:&#x77AA
CJK UNIFIED IDEOGRAPH:'C0FD:49405:&#x77B0
CJK UNIFIED IDEOGRAPH:'C0FE:49406:&#x77AC
CJK UNIFIED IDEOGRAPH:'C140:49472:&#x77A7
CJK UNIFIED IDEOGRAPH:'C141:49473:&#x77AD
CJK UNIFIED IDEOGRAPH:'C142:49474:&#x77EF
CJK UNIFIED IDEOGRAPH:'C143:49475:&#x78F7
CJK UNIFIED IDEOGRAPH:'C144:49476:&#x78FA
CJK UNIFIED IDEOGRAPH:'C145:49477:&#x78F4
CJK UNIFIED IDEOGRAPH:'C146:49478:&#x78EF
CJK UNIFIED IDEOGRAPH:'C147:49479:&#x7901
CJK UNIFIED IDEOGRAPH:'C148:49480:&#x79A7
CJK UNIFIED IDEOGRAPH:'C149:49481:&#x79AA
CJK UNIFIED IDEOGRAPH:'C14A:49482:&#x7A57
CJK UNIFIED IDEOGRAPH:'C14B:49483:&#x7ABF
CJK UNIFIED IDEOGRAPH:'C14C:49484:&#x7C07
CJK UNIFIED IDEOGRAPH:'C14D:49485:&#x7C0D
CJK UNIFIED IDEOGRAPH:'C14E:49486:&#x7BFE
CJK UNIFIED IDEOGRAPH:'C14F:49487:&#x7BF7
CJK UNIFIED IDEOGRAPH:'C150:49488:&#x7C0C
CJK UNIFIED IDEOGRAPH:'C151:49489:&#x7BE0
CJK UNIFIED IDEOGRAPH:'C152:49490:&#x7CE0
CJK UNIFIED IDEOGRAPH:'C153:49491:&#x7CDC
CJK UNIFIED IDEOGRAPH:'C154:49492:&#x7CDE
CJK UNIFIED IDEOGRAPH:'C155:49493:&#x7CE2
CJK UNIFIED IDEOGRAPH:'C156:49494:&#x7CDF
CJK UNIFIED IDEOGRAPH:'C157:49495:&#x7CD9
CJK UNIFIED IDEOGRAPH:'C158:49496:&#x7CDD
CJK UNIFIED IDEOGRAPH:'C159:49497:&#x7E2E
CJK UNIFIED IDEOGRAPH:'C15A:49498:&#x7E3E
CJK UNIFIED IDEOGRAPH:'C15B:49499:&#x7E46
CJK UNIFIED IDEOGRAPH:'C15C:49500:&#x7E37
CJK UNIFIED IDEOGRAPH:'C15D:49501:&#x7E32
CJK UNIFIED IDEOGRAPH:'C15E:49502:&#x7E43
CJK UNIFIED IDEOGRAPH:'C15F:49503:&#x7E2B
CJK UNIFIED IDEOGRAPH:'C160:49504:&#x7E3D
CJK UNIFIED IDEOGRAPH:'C161:49505:&#x7E31
CJK UNIFIED IDEOGRAPH:'C162:49506:&#x7E45
CJK UNIFIED IDEOGRAPH:'C163:49507:&#x7E41
CJK UNIFIED IDEOGRAPH:'C164:49508:&#x7E34
CJK UNIFIED IDEOGRAPH:'C165:49509:&#x7E39
CJK UNIFIED IDEOGRAPH:'C166:49510:&#x7E48
CJK UNIFIED IDEOGRAPH:'C167:49511:&#x7E35
CJK UNIFIED IDEOGRAPH:'C168:49512:&#x7E3F
CJK UNIFIED IDEOGRAPH:'C169:49513:&#x7E2F
CJK UNIFIED IDEOGRAPH:'C16A:49514:&#x7F44
CJK UNIFIED IDEOGRAPH:'C16B:49515:&#x7FF3
CJK UNIFIED IDEOGRAPH:'C16C:49516:&#x7FFC
CJK UNIFIED IDEOGRAPH:'C16D:49517:&#x8071
CJK UNIFIED IDEOGRAPH:'C16E:49518:&#x8072
CJK UNIFIED IDEOGRAPH:'C16F:49519:&#x8070
CJK UNIFIED IDEOGRAPH:'C170:49520:&#x806F
CJK UNIFIED IDEOGRAPH:'C171:49521:&#x8073
CJK UNIFIED IDEOGRAPH:'C172:49522:&#x81C6
CJK UNIFIED IDEOGRAPH:'C173:49523:&#x81C3
CJK UNIFIED IDEOGRAPH:'C174:49524:&#x81BA
CJK UNIFIED IDEOGRAPH:'C175:49525:&#x81C2
CJK UNIFIED IDEOGRAPH:'C176:49526:&#x81C0
CJK UNIFIED IDEOGRAPH:'C177:49527:&#x81BF
CJK UNIFIED IDEOGRAPH:'C178:49528:&#x81BD
CJK UNIFIED IDEOGRAPH:'C179:49529:&#x81C9
CJK UNIFIED IDEOGRAPH:'C17A:49530:&#x81BE
CJK UNIFIED IDEOGRAPH:'C17B:49531:&#x81E8
CJK UNIFIED IDEOGRAPH:'C17C:49532:&#x8209
CJK UNIFIED IDEOGRAPH:'C17D:49533:&#x8271
CJK UNIFIED IDEOGRAPH:'C17E:49534:&#x85AA
CJK UNIFIED IDEOGRAPH:'C1A1:49569:&#x8584
CJK UNIFIED IDEOGRAPH:'C1A2:49570:&#x857E
CJK UNIFIED IDEOGRAPH:'C1A3:49571:&#x859C
CJK UNIFIED IDEOGRAPH:'C1A4:49572:&#x8591
CJK UNIFIED IDEOGRAPH:'C1A5:49573:&#x8594
CJK UNIFIED IDEOGRAPH:'C1A6:49574:&#x85AF
CJK UNIFIED IDEOGRAPH:'C1A7:49575:&#x859B
CJK UNIFIED IDEOGRAPH:'C1A8:49576:&#x8587
CJK UNIFIED IDEOGRAPH:'C1A9:49577:&#x85A8
CJK UNIFIED IDEOGRAPH:'C1AA:49578:&#x858A
CJK UNIFIED IDEOGRAPH:'C1AB:49579:&#x8667
CJK UNIFIED IDEOGRAPH:'C1AC:49580:&#x87C0
CJK UNIFIED IDEOGRAPH:'C1AD:49581:&#x87D1
CJK UNIFIED IDEOGRAPH:'C1AE:49582:&#x87B3
CJK UNIFIED IDEOGRAPH:'C1AF:49583:&#x87D2
CJK UNIFIED IDEOGRAPH:'C1B0:49584:&#x87C6
CJK UNIFIED IDEOGRAPH:'C1B1:49585:&#x87AB
CJK UNIFIED IDEOGRAPH:'C1B2:49586:&#x87BB
CJK UNIFIED IDEOGRAPH:'C1B3:49587:&#x87BA
CJK UNIFIED IDEOGRAPH:'C1B4:49588:&#x87C8
CJK UNIFIED IDEOGRAPH:'C1B5:49589:&#x87CB
CJK UNIFIED IDEOGRAPH:'C1B6:49590:&#x893B
CJK UNIFIED IDEOGRAPH:'C1B7:49591:&#x8936
CJK UNIFIED IDEOGRAPH:'C1B8:49592:&#x8944
CJK UNIFIED IDEOGRAPH:'C1B9:49593:&#x8938
CJK UNIFIED IDEOGRAPH:'C1BA:49594:&#x893D
CJK UNIFIED IDEOGRAPH:'C1BB:49595:&#x89AC
CJK UNIFIED IDEOGRAPH:'C1BC:49596:&#x8B0E
CJK UNIFIED IDEOGRAPH:'C1BD:49597:&#x8B17
CJK UNIFIED IDEOGRAPH:'C1BE:49598:&#x8B19
CJK UNIFIED IDEOGRAPH:'C1BF:49599:&#x8B1B
CJK UNIFIED IDEOGRAPH:'C1C0:49600:&#x8B0A
CJK UNIFIED IDEOGRAPH:'C1C1:49601:&#x8B20
CJK UNIFIED IDEOGRAPH:'C1C2:49602:&#x8B1D
CJK UNIFIED IDEOGRAPH:'C1C3:49603:&#x8B04
CJK UNIFIED IDEOGRAPH:'C1C4:49604:&#x8B10
CJK UNIFIED IDEOGRAPH:'C1C5:49605:&#x8C41
CJK UNIFIED IDEOGRAPH:'C1C6:49606:&#x8C3F
CJK UNIFIED IDEOGRAPH:'C1C7:49607:&#x8C73
CJK UNIFIED IDEOGRAPH:'C1C8:49608:&#x8CFA
CJK UNIFIED IDEOGRAPH:'C1C9:49609:&#x8CFD
CJK UNIFIED IDEOGRAPH:'C1CA:49610:&#x8CFC
CJK UNIFIED IDEOGRAPH:'C1CB:49611:&#x8CF8
CJK UNIFIED IDEOGRAPH:'C1CC:49612:&#x8CFB
CJK UNIFIED IDEOGRAPH:'C1CD:49613:&#x8DA8
CJK UNIFIED IDEOGRAPH:'C1CE:49614:&#x8E49
CJK UNIFIED IDEOGRAPH:'C1CF:49615:&#x8E4B
CJK UNIFIED IDEOGRAPH:'C1D0:49616:&#x8E48
CJK UNIFIED IDEOGRAPH:'C1D1:49617:&#x8E4A
CJK UNIFIED IDEOGRAPH:'C1D2:49618:&#x8F44
CJK UNIFIED IDEOGRAPH:'C1D3:49619:&#x8F3E
CJK UNIFIED IDEOGRAPH:'C1D4:49620:&#x8F42
CJK UNIFIED IDEOGRAPH:'C1D5:49621:&#x8F45
CJK UNIFIED IDEOGRAPH:'C1D6:49622:&#x8F3F
CJK UNIFIED IDEOGRAPH:'C1D7:49623:&#x907F
CJK UNIFIED IDEOGRAPH:'C1D8:49624:&#x907D
CJK UNIFIED IDEOGRAPH:'C1D9:49625:&#x9084
CJK UNIFIED IDEOGRAPH:'C1DA:49626:&#x9081
CJK UNIFIED IDEOGRAPH:'C1DB:49627:&#x9082
CJK UNIFIED IDEOGRAPH:'C1DC:49628:&#x9080
CJK UNIFIED IDEOGRAPH:'C1DD:49629:&#x9139
CJK UNIFIED IDEOGRAPH:'C1DE:49630:&#x91A3
CJK UNIFIED IDEOGRAPH:'C1DF:49631:&#x919E
CJK UNIFIED IDEOGRAPH:'C1E0:49632:&#x919C
CJK UNIFIED IDEOGRAPH:'C1E1:49633:&#x934D
CJK UNIFIED IDEOGRAPH:'C1E2:49634:&#x9382
CJK UNIFIED IDEOGRAPH:'C1E3:49635:&#x9328
CJK UNIFIED IDEOGRAPH:'C1E4:49636:&#x9375
CJK UNIFIED IDEOGRAPH:'C1E5:49637:&#x934A
CJK UNIFIED IDEOGRAPH:'C1E6:49638:&#x9365
CJK UNIFIED IDEOGRAPH:'C1E7:49639:&#x934B
CJK UNIFIED IDEOGRAPH:'C1E8:49640:&#x9318
CJK UNIFIED IDEOGRAPH:'C1E9:49641:&#x937E
CJK UNIFIED IDEOGRAPH:'C1EA:49642:&#x936C
CJK UNIFIED IDEOGRAPH:'C1EB:49643:&#x935B
CJK UNIFIED IDEOGRAPH:'C1EC:49644:&#x9370
CJK UNIFIED IDEOGRAPH:'C1ED:49645:&#x935A
CJK UNIFIED IDEOGRAPH:'C1EE:49646:&#x9354
CJK UNIFIED IDEOGRAPH:'C1EF:49647:&#x95CA
CJK UNIFIED IDEOGRAPH:'C1F0:49648:&#x95CB
CJK UNIFIED IDEOGRAPH:'C1F1:49649:&#x95CC
CJK UNIFIED IDEOGRAPH:'C1F2:49650:&#x95C8
CJK UNIFIED IDEOGRAPH:'C1F3:49651:&#x95C6
CJK UNIFIED IDEOGRAPH:'C1F4:49652:&#x96B1
CJK UNIFIED IDEOGRAPH:'C1F5:49653:&#x96B8
CJK UNIFIED IDEOGRAPH:'C1F6:49654:&#x96D6
CJK UNIFIED IDEOGRAPH:'C1F7:49655:&#x971C
CJK UNIFIED IDEOGRAPH:'C1F8:49656:&#x971E
CJK UNIFIED IDEOGRAPH:'C1F9:49657:&#x97A0
CJK UNIFIED IDEOGRAPH:'C1FA:49658:&#x97D3
CJK UNIFIED IDEOGRAPH:'C1FB:49659:&#x9846
CJK UNIFIED IDEOGRAPH:'C1FC:49660:&#x98B6
CJK UNIFIED IDEOGRAPH:'C1FD:49661:&#x9935
CJK UNIFIED IDEOGRAPH:'C1FE:49662:&#x9A01
CJK UNIFIED IDEOGRAPH:'C240:49728:&#x99FF
CJK UNIFIED IDEOGRAPH:'C241:49729:&#x9BAE
CJK UNIFIED IDEOGRAPH:'C242:49730:&#x9BAB
CJK UNIFIED IDEOGRAPH:'C243:49731:&#x9BAA
CJK UNIFIED IDEOGRAPH:'C244:49732:&#x9BAD
CJK UNIFIED IDEOGRAPH:'C245:49733:&#x9D3B
CJK UNIFIED IDEOGRAPH:'C246:49734:&#x9D3F
CJK UNIFIED IDEOGRAPH:'C247:49735:&#x9E8B
CJK UNIFIED IDEOGRAPH:'C248:49736:&#x9ECF
CJK UNIFIED IDEOGRAPH:'C249:49737:&#x9EDE
CJK UNIFIED IDEOGRAPH:'C24A:49738:&#x9EDC
CJK UNIFIED IDEOGRAPH:'C24B:49739:&#x9EDD
CJK UNIFIED IDEOGRAPH:'C24C:49740:&#x9EDB
CJK UNIFIED IDEOGRAPH:'C24D:49741:&#x9F3E
CJK UNIFIED IDEOGRAPH:'C24E:49742:&#x9F4B
CJK UNIFIED IDEOGRAPH:'C24F:49743:&#x53E2
CJK UNIFIED IDEOGRAPH:'C250:49744:&#x5695
CJK UNIFIED IDEOGRAPH:'C251:49745:&#x56AE
CJK UNIFIED IDEOGRAPH:'C252:49746:&#x58D9
CJK UNIFIED IDEOGRAPH:'C253:49747:&#x58D8
CJK UNIFIED IDEOGRAPH:'C254:49748:&#x5B38
CJK UNIFIED IDEOGRAPH:'C255:49749:&#x5F5D
CJK UNIFIED IDEOGRAPH:'C256:49750:&#x61E3
CJK UNIFIED IDEOGRAPH:'C257:49751:&#x6233
CJK UNIFIED IDEOGRAPH:'C258:49752:&#x64F4
CJK UNIFIED IDEOGRAPH:'C259:49753:&#x64F2
CJK UNIFIED IDEOGRAPH:'C25A:49754:&#x64FE
CJK UNIFIED IDEOGRAPH:'C25B:49755:&#x6506
CJK UNIFIED IDEOGRAPH:'C25C:49756:&#x64FA
CJK UNIFIED IDEOGRAPH:'C25D:49757:&#x64FB
CJK UNIFIED IDEOGRAPH:'C25E:49758:&#x64F7
CJK UNIFIED IDEOGRAPH:'C25F:49759:&#x65B7
CJK UNIFIED IDEOGRAPH:'C260:49760:&#x66DC
CJK UNIFIED IDEOGRAPH:'C261:49761:&#x6726
CJK UNIFIED IDEOGRAPH:'C262:49762:&#x6AB3
CJK UNIFIED IDEOGRAPH:'C263:49763:&#x6AAC
CJK UNIFIED IDEOGRAPH:'C264:49764:&#x6AC3
CJK UNIFIED IDEOGRAPH:'C265:49765:&#x6ABB
CJK UNIFIED IDEOGRAPH:'C266:49766:&#x6AB8
CJK UNIFIED IDEOGRAPH:'C267:49767:&#x6AC2
CJK UNIFIED IDEOGRAPH:'C268:49768:&#x6AAE
CJK UNIFIED IDEOGRAPH:'C269:49769:&#x6AAF
CJK UNIFIED IDEOGRAPH:'C26A:49770:&#x6B5F
CJK UNIFIED IDEOGRAPH:'C26B:49771:&#x6B78
CJK UNIFIED IDEOGRAPH:'C26C:49772:&#x6BAF
CJK UNIFIED IDEOGRAPH:'C26D:49773:&#x7009
CJK UNIFIED IDEOGRAPH:'C26E:49774:&#x700B
CJK UNIFIED IDEOGRAPH:'C26F:49775:&#x6FFE
CJK UNIFIED IDEOGRAPH:'C270:49776:&#x7006
CJK UNIFIED IDEOGRAPH:'C271:49777:&#x6FFA
CJK UNIFIED IDEOGRAPH:'C272:49778:&#x7011
CJK UNIFIED IDEOGRAPH:'C273:49779:&#x700F
CJK UNIFIED IDEOGRAPH:'C274:49780:&#x71FB
CJK UNIFIED IDEOGRAPH:'C275:49781:&#x71FC
CJK UNIFIED IDEOGRAPH:'C276:49782:&#x71FE
CJK UNIFIED IDEOGRAPH:'C277:49783:&#x71F8
CJK UNIFIED IDEOGRAPH:'C278:49784:&#x7377
CJK UNIFIED IDEOGRAPH:'C279:49785:&#x7375
CJK UNIFIED IDEOGRAPH:'C27A:49786:&#x74A7
CJK UNIFIED IDEOGRAPH:'C27B:49787:&#x74BF
CJK UNIFIED IDEOGRAPH:'C27C:49788:&#x7515
CJK UNIFIED IDEOGRAPH:'C27D:49789:&#x7656
CJK UNIFIED IDEOGRAPH:'C27E:49790:&#x7658
CJK UNIFIED IDEOGRAPH:'C2A1:49825:&#x7652
CJK UNIFIED IDEOGRAPH:'C2A2:49826:&#x77BD
CJK UNIFIED IDEOGRAPH:'C2A3:49827:&#x77BF
CJK UNIFIED IDEOGRAPH:'C2A4:49828:&#x77BB
CJK UNIFIED IDEOGRAPH:'C2A5:49829:&#x77BC
CJK UNIFIED IDEOGRAPH:'C2A6:49830:&#x790E
CJK UNIFIED IDEOGRAPH:'C2A7:49831:&#x79AE
CJK UNIFIED IDEOGRAPH:'C2A8:49832:&#x7A61
CJK UNIFIED IDEOGRAPH:'C2A9:49833:&#x7A62
CJK UNIFIED IDEOGRAPH:'C2AA:49834:&#x7A60
CJK UNIFIED IDEOGRAPH:'C2AB:49835:&#x7AC4
CJK UNIFIED IDEOGRAPH:'C2AC:49836:&#x7AC5
CJK UNIFIED IDEOGRAPH:'C2AD:49837:&#x7C2B
CJK UNIFIED IDEOGRAPH:'C2AE:49838:&#x7C27
CJK UNIFIED IDEOGRAPH:'C2AF:49839:&#x7C2A
CJK UNIFIED IDEOGRAPH:'C2B0:49840:&#x7C1E
CJK UNIFIED IDEOGRAPH:'C2B1:49841:&#x7C23
CJK UNIFIED IDEOGRAPH:'C2B2:49842:&#x7C21
CJK UNIFIED IDEOGRAPH:'C2B3:49843:&#x7CE7
CJK UNIFIED IDEOGRAPH:'C2B4:49844:&#x7E54
CJK UNIFIED IDEOGRAPH:'C2B5:49845:&#x7E55
CJK UNIFIED IDEOGRAPH:'C2B6:49846:&#x7E5E
CJK UNIFIED IDEOGRAPH:'C2B7:49847:&#x7E5A
CJK UNIFIED IDEOGRAPH:'C2B8:49848:&#x7E61
CJK UNIFIED IDEOGRAPH:'C2B9:49849:&#x7E52
CJK UNIFIED IDEOGRAPH:'C2BA:49850:&#x7E59
CJK UNIFIED IDEOGRAPH:'C2BB:49851:&#x7F48
CJK UNIFIED IDEOGRAPH:'C2BC:49852:&#x7FF9
CJK UNIFIED IDEOGRAPH:'C2BD:49853:&#x7FFB
CJK UNIFIED IDEOGRAPH:'C2BE:49854:&#x8077
CJK UNIFIED IDEOGRAPH:'C2BF:49855:&#x8076
CJK UNIFIED IDEOGRAPH:'C2C0:49856:&#x81CD
CJK UNIFIED IDEOGRAPH:'C2C1:49857:&#x81CF
CJK UNIFIED IDEOGRAPH:'C2C2:49858:&#x820A
CJK UNIFIED IDEOGRAPH:'C2C3:49859:&#x85CF
CJK UNIFIED IDEOGRAPH:'C2C4:49860:&#x85A9
CJK UNIFIED IDEOGRAPH:'C2C5:49861:&#x85CD
CJK UNIFIED IDEOGRAPH:'C2C6:49862:&#x85D0
CJK UNIFIED IDEOGRAPH:'C2C7:49863:&#x85C9
CJK UNIFIED IDEOGRAPH:'C2C8:49864:&#x85B0
CJK UNIFIED IDEOGRAPH:'C2C9:49865:&#x85BA
CJK UNIFIED IDEOGRAPH:'C2CA:49866:&#x85B9
CJK UNIFIED IDEOGRAPH:'C2CB:49867:&#x85A6
CJK UNIFIED IDEOGRAPH:'C2CC:49868:&#x87EF
CJK UNIFIED IDEOGRAPH:'C2CD:49869:&#x87EC
CJK UNIFIED IDEOGRAPH:'C2CE:49870:&#x87F2
CJK UNIFIED IDEOGRAPH:'C2CF:49871:&#x87E0
CJK UNIFIED IDEOGRAPH:'C2D0:49872:&#x8986
CJK UNIFIED IDEOGRAPH:'C2D1:49873:&#x89B2
CJK UNIFIED IDEOGRAPH:'C2D2:49874:&#x89F4
CJK UNIFIED IDEOGRAPH:'C2D3:49875:&#x8B28
CJK UNIFIED IDEOGRAPH:'C2D4:49876:&#x8B39
CJK UNIFIED IDEOGRAPH:'C2D5:49877:&#x8B2C
CJK UNIFIED IDEOGRAPH:'C2D6:49878:&#x8B2B
CJK UNIFIED IDEOGRAPH:'C2D7:49879:&#x8C50
CJK UNIFIED IDEOGRAPH:'C2D8:49880:&#x8D05
CJK UNIFIED IDEOGRAPH:'C2D9:49881:&#x8E59
CJK UNIFIED IDEOGRAPH:'C2DA:49882:&#x8E63
CJK UNIFIED IDEOGRAPH:'C2DB:49883:&#x8E66
CJK UNIFIED IDEOGRAPH:'C2DC:49884:&#x8E64
CJK UNIFIED IDEOGRAPH:'C2DD:49885:&#x8E5F
CJK UNIFIED IDEOGRAPH:'C2DE:49886:&#x8E55
CJK UNIFIED IDEOGRAPH:'C2DF:49887:&#x8EC0
CJK UNIFIED IDEOGRAPH:'C2E0:49888:&#x8F49
CJK UNIFIED IDEOGRAPH:'C2E1:49889:&#x8F4D
CJK UNIFIED IDEOGRAPH:'C2E2:49890:&#x9087
CJK UNIFIED IDEOGRAPH:'C2E3:49891:&#x9083
CJK UNIFIED IDEOGRAPH:'C2E4:49892:&#x9088
CJK UNIFIED IDEOGRAPH:'C2E5:49893:&#x91AB
CJK UNIFIED IDEOGRAPH:'C2E6:49894:&#x91AC
CJK UNIFIED IDEOGRAPH:'C2E7:49895:&#x91D0
CJK UNIFIED IDEOGRAPH:'C2E8:49896:&#x9394
CJK UNIFIED IDEOGRAPH:'C2E9:49897:&#x938A
CJK UNIFIED IDEOGRAPH:'C2EA:49898:&#x9396
CJK UNIFIED IDEOGRAPH:'C2EB:49899:&#x93A2
CJK UNIFIED IDEOGRAPH:'C2EC:49900:&#x93B3
CJK UNIFIED IDEOGRAPH:'C2ED:49901:&#x93AE
CJK UNIFIED IDEOGRAPH:'C2EE:49902:&#x93AC
CJK UNIFIED IDEOGRAPH:'C2EF:49903:&#x93B0
CJK UNIFIED IDEOGRAPH:'C2F0:49904:&#x9398
CJK UNIFIED IDEOGRAPH:'C2F1:49905:&#x939A
CJK UNIFIED IDEOGRAPH:'C2F2:49906:&#x9397
CJK UNIFIED IDEOGRAPH:'C2F3:49907:&#x95D4
CJK UNIFIED IDEOGRAPH:'C2F4:49908:&#x95D6
CJK UNIFIED IDEOGRAPH:'C2F5:49909:&#x95D0
CJK UNIFIED IDEOGRAPH:'C2F6:49910:&#x95D5
CJK UNIFIED IDEOGRAPH:'C2F7:49911:&#x96E2
CJK UNIFIED IDEOGRAPH:'C2F8:49912:&#x96DC
CJK UNIFIED IDEOGRAPH:'C2F9:49913:&#x96D9
CJK UNIFIED IDEOGRAPH:'C2FA:49914:&#x96DB
CJK UNIFIED IDEOGRAPH:'C2FB:49915:&#x96DE
CJK UNIFIED IDEOGRAPH:'C2FC:49916:&#x9724
CJK UNIFIED IDEOGRAPH:'C2FD:49917:&#x97A3
CJK UNIFIED IDEOGRAPH:'C2FE:49918:&#x97A6
CJK UNIFIED IDEOGRAPH:'C340:49984:&#x97AD
CJK UNIFIED IDEOGRAPH:'C341:49985:&#x97F9
CJK UNIFIED IDEOGRAPH:'C342:49986:&#x984D
CJK UNIFIED IDEOGRAPH:'C343:49987:&#x984F
CJK UNIFIED IDEOGRAPH:'C344:49988:&#x984C
CJK UNIFIED IDEOGRAPH:'C345:49989:&#x984E
CJK UNIFIED IDEOGRAPH:'C346:49990:&#x9853
CJK UNIFIED IDEOGRAPH:'C347:49991:&#x98BA
CJK UNIFIED IDEOGRAPH:'C348:49992:&#x993E
CJK UNIFIED IDEOGRAPH:'C349:49993:&#x993F
CJK UNIFIED IDEOGRAPH:'C34A:49994:&#x993D
CJK UNIFIED IDEOGRAPH:'C34B:49995:&#x992E
CJK UNIFIED IDEOGRAPH:'C34C:49996:&#x99A5
CJK UNIFIED IDEOGRAPH:'C34D:49997:&#x9A0E
CJK UNIFIED IDEOGRAPH:'C34E:49998:&#x9AC1
CJK UNIFIED IDEOGRAPH:'C34F:49999:&#x9B03
CJK UNIFIED IDEOGRAPH:'C350:50000:&#x9B06
CJK UNIFIED IDEOGRAPH:'C351:50001:&#x9B4F
CJK UNIFIED IDEOGRAPH:'C352:50002:&#x9B4E
CJK UNIFIED IDEOGRAPH:'C353:50003:&#x9B4D
CJK UNIFIED IDEOGRAPH:'C354:50004:&#x9BCA
CJK UNIFIED IDEOGRAPH:'C355:50005:&#x9BC9
CJK UNIFIED IDEOGRAPH:'C356:50006:&#x9BFD
CJK UNIFIED IDEOGRAPH:'C357:50007:&#x9BC8
CJK UNIFIED IDEOGRAPH:'C358:50008:&#x9BC0
CJK UNIFIED IDEOGRAPH:'C359:50009:&#x9D51
CJK UNIFIED IDEOGRAPH:'C35A:50010:&#x9D5D
CJK UNIFIED IDEOGRAPH:'C35B:50011:&#x9D60
CJK UNIFIED IDEOGRAPH:'C35C:50012:&#x9EE0
CJK UNIFIED IDEOGRAPH:'C35D:50013:&#x9F15
CJK UNIFIED IDEOGRAPH:'C35E:50014:&#x9F2C
CJK UNIFIED IDEOGRAPH:'C35F:50015:&#x5133
CJK UNIFIED IDEOGRAPH:'C360:50016:&#x56A5
CJK UNIFIED IDEOGRAPH:'C361:50017:&#x58DE
CJK UNIFIED IDEOGRAPH:'C362:50018:&#x58DF
CJK UNIFIED IDEOGRAPH:'C363:50019:&#x58E2
CJK UNIFIED IDEOGRAPH:'C364:50020:&#x5BF5
CJK UNIFIED IDEOGRAPH:'C365:50021:&#x9F90
CJK UNIFIED IDEOGRAPH:'C366:50022:&#x5EEC
CJK UNIFIED IDEOGRAPH:'C367:50023:&#x61F2
CJK UNIFIED IDEOGRAPH:'C368:50024:&#x61F7
CJK UNIFIED IDEOGRAPH:'C369:50025:&#x61F6
CJK UNIFIED IDEOGRAPH:'C36A:50026:&#x61F5
CJK UNIFIED IDEOGRAPH:'C36B:50027:&#x6500
CJK UNIFIED IDEOGRAPH:'C36C:50028:&#x650F
CJK UNIFIED IDEOGRAPH:'C36D:50029:&#x66E0
CJK UNIFIED IDEOGRAPH:'C36E:50030:&#x66DD
CJK UNIFIED IDEOGRAPH:'C36F:50031:&#x6AE5
CJK UNIFIED IDEOGRAPH:'C370:50032:&#x6ADD
CJK UNIFIED IDEOGRAPH:'C371:50033:&#x6ADA
CJK UNIFIED IDEOGRAPH:'C372:50034:&#x6AD3
CJK UNIFIED IDEOGRAPH:'C373:50035:&#x701B
CJK UNIFIED IDEOGRAPH:'C374:50036:&#x701F
CJK UNIFIED IDEOGRAPH:'C375:50037:&#x7028
CJK UNIFIED IDEOGRAPH:'C376:50038:&#x701A
CJK UNIFIED IDEOGRAPH:'C377:50039:&#x701D
CJK UNIFIED IDEOGRAPH:'C378:50040:&#x7015
CJK UNIFIED IDEOGRAPH:'C379:50041:&#x7018
CJK UNIFIED IDEOGRAPH:'C37A:50042:&#x7206
CJK UNIFIED IDEOGRAPH:'C37B:50043:&#x720D
CJK UNIFIED IDEOGRAPH:'C37C:50044:&#x7258
CJK UNIFIED IDEOGRAPH:'C37D:50045:&#x72A2
CJK UNIFIED IDEOGRAPH:'C37E:50046:&#x7378
CJK UNIFIED IDEOGRAPH:'C3A1:50081:&#x737A
CJK UNIFIED IDEOGRAPH:'C3A2:50082:&#x74BD
CJK UNIFIED IDEOGRAPH:'C3A3:50083:&#x74CA
CJK UNIFIED IDEOGRAPH:'C3A4:50084:&#x74E3
CJK UNIFIED IDEOGRAPH:'C3A5:50085:&#x7587
CJK UNIFIED IDEOGRAPH:'C3A6:50086:&#x7586
CJK UNIFIED IDEOGRAPH:'C3A7:50087:&#x765F
CJK UNIFIED IDEOGRAPH:'C3A8:50088:&#x7661
CJK UNIFIED IDEOGRAPH:'C3A9:50089:&#x77C7
CJK UNIFIED IDEOGRAPH:'C3AA:50090:&#x7919
CJK UNIFIED IDEOGRAPH:'C3AB:50091:&#x79B1
CJK UNIFIED IDEOGRAPH:'C3AC:50092:&#x7A6B
CJK UNIFIED IDEOGRAPH:'C3AD:50093:&#x7A69
CJK UNIFIED IDEOGRAPH:'C3AE:50094:&#x7C3E
CJK UNIFIED IDEOGRAPH:'C3AF:50095:&#x7C3F
CJK UNIFIED IDEOGRAPH:'C3B0:50096:&#x7C38
CJK UNIFIED IDEOGRAPH:'C3B1:50097:&#x7C3D
CJK UNIFIED IDEOGRAPH:'C3B2:50098:&#x7C37
CJK UNIFIED IDEOGRAPH:'C3B3:50099:&#x7C40
CJK UNIFIED IDEOGRAPH:'C3B4:50100:&#x7E6B
CJK UNIFIED IDEOGRAPH:'C3B5:50101:&#x7E6D
CJK UNIFIED IDEOGRAPH:'C3B6:50102:&#x7E79
CJK UNIFIED IDEOGRAPH:'C3B7:50103:&#x7E69
CJK UNIFIED IDEOGRAPH:'C3B8:50104:&#x7E6A
CJK UNIFIED IDEOGRAPH:'C3B9:50105:&#x7F85
CJK UNIFIED IDEOGRAPH:'C3BA:50106:&#x7E73
CJK UNIFIED IDEOGRAPH:'C3BB:50107:&#x7FB6
CJK UNIFIED IDEOGRAPH:'C3BC:50108:&#x7FB9
CJK UNIFIED IDEOGRAPH:'C3BD:50109:&#x7FB8
CJK UNIFIED IDEOGRAPH:'C3BE:50110:&#x81D8
CJK UNIFIED IDEOGRAPH:'C3BF:50111:&#x85E9
CJK UNIFIED IDEOGRAPH:'C3C0:50112:&#x85DD
CJK UNIFIED IDEOGRAPH:'C3C1:50113:&#x85EA
CJK UNIFIED IDEOGRAPH:'C3C2:50114:&#x85D5
CJK UNIFIED IDEOGRAPH:'C3C3:50115:&#x85E4
CJK UNIFIED IDEOGRAPH:'C3C4:50116:&#x85E5
CJK UNIFIED IDEOGRAPH:'C3C5:50117:&#x85F7
CJK UNIFIED IDEOGRAPH:'C3C6:50118:&#x87FB
CJK UNIFIED IDEOGRAPH:'C3C7:50119:&#x8805
CJK UNIFIED IDEOGRAPH:'C3C8:50120:&#x880D
CJK UNIFIED IDEOGRAPH:'C3C9:50121:&#x87F9
CJK UNIFIED IDEOGRAPH:'C3CA:50122:&#x87FE
CJK UNIFIED IDEOGRAPH:'C3CB:50123:&#x8960
CJK UNIFIED IDEOGRAPH:'C3CC:50124:&#x895F
CJK UNIFIED IDEOGRAPH:'C3CD:50125:&#x8956
CJK UNIFIED IDEOGRAPH:'C3CE:50126:&#x895E
CJK UNIFIED IDEOGRAPH:'C3CF:50127:&#x8B41
CJK UNIFIED IDEOGRAPH:'C3D0:50128:&#x8B5C
CJK UNIFIED IDEOGRAPH:'C3D1:50129:&#x8B58
CJK UNIFIED IDEOGRAPH:'C3D2:50130:&#x8B49
CJK UNIFIED IDEOGRAPH:'C3D3:50131:&#x8B5A
CJK UNIFIED IDEOGRAPH:'C3D4:50132:&#x8B4E
CJK UNIFIED IDEOGRAPH:'C3D5:50133:&#x8B4F
CJK UNIFIED IDEOGRAPH:'C3D6:50134:&#x8B46
CJK UNIFIED IDEOGRAPH:'C3D7:50135:&#x8B59
CJK UNIFIED IDEOGRAPH:'C3D8:50136:&#x8D08
CJK UNIFIED IDEOGRAPH:'C3D9:50137:&#x8D0A
CJK UNIFIED IDEOGRAPH:'C3DA:50138:&#x8E7C
CJK UNIFIED IDEOGRAPH:'C3DB:50139:&#x8E72
CJK UNIFIED IDEOGRAPH:'C3DC:50140:&#x8E87
CJK UNIFIED IDEOGRAPH:'C3DD:50141:&#x8E76
CJK UNIFIED IDEOGRAPH:'C3DE:50142:&#x8E6C
CJK UNIFIED IDEOGRAPH:'C3DF:50143:&#x8E7A
CJK UNIFIED IDEOGRAPH:'C3E0:50144:&#x8E74
CJK UNIFIED IDEOGRAPH:'C3E1:50145:&#x8F54
CJK UNIFIED IDEOGRAPH:'C3E2:50146:&#x8F4E
CJK UNIFIED IDEOGRAPH:'C3E3:50147:&#x8FAD
CJK UNIFIED IDEOGRAPH:'C3E4:50148:&#x908A
CJK UNIFIED IDEOGRAPH:'C3E5:50149:&#x908B
CJK UNIFIED IDEOGRAPH:'C3E6:50150:&#x91B1
CJK UNIFIED IDEOGRAPH:'C3E7:50151:&#x91AE
CJK UNIFIED IDEOGRAPH:'C3E8:50152:&#x93E1
CJK UNIFIED IDEOGRAPH:'C3E9:50153:&#x93D1
CJK UNIFIED IDEOGRAPH:'C3EA:50154:&#x93DF
CJK UNIFIED IDEOGRAPH:'C3EB:50155:&#x93C3
CJK UNIFIED IDEOGRAPH:'C3EC:50156:&#x93C8
CJK UNIFIED IDEOGRAPH:'C3ED:50157:&#x93DC
CJK UNIFIED IDEOGRAPH:'C3EE:50158:&#x93DD
CJK UNIFIED IDEOGRAPH:'C3EF:50159:&#x93D6
CJK UNIFIED IDEOGRAPH:'C3F0:50160:&#x93E2
CJK UNIFIED IDEOGRAPH:'C3F1:50161:&#x93CD
CJK UNIFIED IDEOGRAPH:'C3F2:50162:&#x93D8
CJK UNIFIED IDEOGRAPH:'C3F3:50163:&#x93E4
CJK UNIFIED IDEOGRAPH:'C3F4:50164:&#x93D7
CJK UNIFIED IDEOGRAPH:'C3F5:50165:&#x93E8
CJK UNIFIED IDEOGRAPH:'C3F6:50166:&#x95DC
CJK UNIFIED IDEOGRAPH:'C3F7:50167:&#x96B4
CJK UNIFIED IDEOGRAPH:'C3F8:50168:&#x96E3
CJK UNIFIED IDEOGRAPH:'C3F9:50169:&#x972A
CJK UNIFIED IDEOGRAPH:'C3FA:50170:&#x9727
CJK UNIFIED IDEOGRAPH:'C3FB:50171:&#x9761
CJK UNIFIED IDEOGRAPH:'C3FC:50172:&#x97DC
CJK UNIFIED IDEOGRAPH:'C3FD:50173:&#x97FB
CJK UNIFIED IDEOGRAPH:'C3FE:50174:&#x985E
CJK UNIFIED IDEOGRAPH:'C440:50240:&#x9858
CJK UNIFIED IDEOGRAPH:'C441:50241:&#x985B
CJK UNIFIED IDEOGRAPH:'C442:50242:&#x98BC
CJK UNIFIED IDEOGRAPH:'C443:50243:&#x9945
CJK UNIFIED IDEOGRAPH:'C444:50244:&#x9949
CJK UNIFIED IDEOGRAPH:'C445:50245:&#x9A16
CJK UNIFIED IDEOGRAPH:'C446:50246:&#x9A19
CJK UNIFIED IDEOGRAPH:'C447:50247:&#x9B0D
CJK UNIFIED IDEOGRAPH:'C448:50248:&#x9BE8
CJK UNIFIED IDEOGRAPH:'C449:50249:&#x9BE7
CJK UNIFIED IDEOGRAPH:'C44A:50250:&#x9BD6
CJK UNIFIED IDEOGRAPH:'C44B:50251:&#x9BDB
CJK UNIFIED IDEOGRAPH:'C44C:50252:&#x9D89
CJK UNIFIED IDEOGRAPH:'C44D:50253:&#x9D61
CJK UNIFIED IDEOGRAPH:'C44E:50254:&#x9D72
CJK UNIFIED IDEOGRAPH:'C44F:50255:&#x9D6A
CJK UNIFIED IDEOGRAPH:'C450:50256:&#x9D6C
CJK UNIFIED IDEOGRAPH:'C451:50257:&#x9E92
CJK UNIFIED IDEOGRAPH:'C452:50258:&#x9E97
CJK UNIFIED IDEOGRAPH:'C453:50259:&#x9E93
CJK UNIFIED IDEOGRAPH:'C454:50260:&#x9EB4
CJK UNIFIED IDEOGRAPH:'C455:50261:&#x52F8
CJK UNIFIED IDEOGRAPH:'C456:50262:&#x56A8
CJK UNIFIED IDEOGRAPH:'C457:50263:&#x56B7
CJK UNIFIED IDEOGRAPH:'C458:50264:&#x56B6
CJK UNIFIED IDEOGRAPH:'C459:50265:&#x56B4
CJK UNIFIED IDEOGRAPH:'C45A:50266:&#x56BC
CJK UNIFIED IDEOGRAPH:'C45B:50267:&#x58E4
CJK UNIFIED IDEOGRAPH:'C45C:50268:&#x5B40
CJK UNIFIED IDEOGRAPH:'C45D:50269:&#x5B43
CJK UNIFIED IDEOGRAPH:'C45E:50270:&#x5B7D
CJK UNIFIED IDEOGRAPH:'C45F:50271:&#x5BF6
CJK UNIFIED IDEOGRAPH:'C460:50272:&#x5DC9
CJK UNIFIED IDEOGRAPH:'C461:50273:&#x61F8
CJK UNIFIED IDEOGRAPH:'C462:50274:&#x61FA
CJK UNIFIED IDEOGRAPH:'C463:50275:&#x6518
CJK UNIFIED IDEOGRAPH:'C464:50276:&#x6514
CJK UNIFIED IDEOGRAPH:'C465:50277:&#x6519
CJK UNIFIED IDEOGRAPH:'C466:50278:&#x66E6
CJK UNIFIED IDEOGRAPH:'C467:50279:&#x6727
CJK UNIFIED IDEOGRAPH:'C468:50280:&#x6AEC
CJK UNIFIED IDEOGRAPH:'C469:50281:&#x703E
CJK UNIFIED IDEOGRAPH:'C46A:50282:&#x7030
CJK UNIFIED IDEOGRAPH:'C46B:50283:&#x7032
CJK UNIFIED IDEOGRAPH:'C46C:50284:&#x7210
CJK UNIFIED IDEOGRAPH:'C46D:50285:&#x737B
CJK UNIFIED IDEOGRAPH:'C46E:50286:&#x74CF
CJK UNIFIED IDEOGRAPH:'C46F:50287:&#x7662
CJK UNIFIED IDEOGRAPH:'C470:50288:&#x7665
CJK UNIFIED IDEOGRAPH:'C471:50289:&#x7926
CJK UNIFIED IDEOGRAPH:'C472:50290:&#x792A
CJK UNIFIED IDEOGRAPH:'C473:50291:&#x792C
CJK UNIFIED IDEOGRAPH:'C474:50292:&#x792B
CJK UNIFIED IDEOGRAPH:'C475:50293:&#x7AC7
CJK UNIFIED IDEOGRAPH:'C476:50294:&#x7AF6
CJK UNIFIED IDEOGRAPH:'C477:50295:&#x7C4C
CJK UNIFIED IDEOGRAPH:'C478:50296:&#x7C43
CJK UNIFIED IDEOGRAPH:'C479:50297:&#x7C4D
CJK UNIFIED IDEOGRAPH:'C47A:50298:&#x7CEF
CJK UNIFIED IDEOGRAPH:'C47B:50299:&#x7CF0
CJK UNIFIED IDEOGRAPH:'C47C:50300:&#x8FAE
CJK UNIFIED IDEOGRAPH:'C47D:50301:&#x7E7D
CJK UNIFIED IDEOGRAPH:'C47E:50302:&#x7E7C
CJK UNIFIED IDEOGRAPH:'C4A1:50337:&#x7E82
CJK UNIFIED IDEOGRAPH:'C4A2:50338:&#x7F4C
CJK UNIFIED IDEOGRAPH:'C4A3:50339:&#x8000
CJK UNIFIED IDEOGRAPH:'C4A4:50340:&#x81DA
CJK UNIFIED IDEOGRAPH:'C4A5:50341:&#x8266
CJK UNIFIED IDEOGRAPH:'C4A6:50342:&#x85FB
CJK UNIFIED IDEOGRAPH:'C4A7:50343:&#x85F9
CJK UNIFIED IDEOGRAPH:'C4A8:50344:&#x8611
CJK UNIFIED IDEOGRAPH:'C4A9:50345:&#x85FA
CJK UNIFIED IDEOGRAPH:'C4AA:50346:&#x8606
CJK UNIFIED IDEOGRAPH:'C4AB:50347:&#x860B
CJK UNIFIED IDEOGRAPH:'C4AC:50348:&#x8607
CJK UNIFIED IDEOGRAPH:'C4AD:50349:&#x860A
CJK UNIFIED IDEOGRAPH:'C4AE:50350:&#x8814
CJK UNIFIED IDEOGRAPH:'C4AF:50351:&#x8815
CJK UNIFIED IDEOGRAPH:'C4B0:50352:&#x8964
CJK UNIFIED IDEOGRAPH:'C4B1:50353:&#x89BA
CJK UNIFIED IDEOGRAPH:'C4B2:50354:&#x89F8
CJK UNIFIED IDEOGRAPH:'C4B3:50355:&#x8B70
CJK UNIFIED IDEOGRAPH:'C4B4:50356:&#x8B6C
CJK UNIFIED IDEOGRAPH:'C4B5:50357:&#x8B66
CJK UNIFIED IDEOGRAPH:'C4B6:50358:&#x8B6F
CJK UNIFIED IDEOGRAPH:'C4B7:50359:&#x8B5F
CJK UNIFIED IDEOGRAPH:'C4B8:50360:&#x8B6B
CJK UNIFIED IDEOGRAPH:'C4B9:50361:&#x8D0F
CJK UNIFIED IDEOGRAPH:'C4BA:50362:&#x8D0D
CJK UNIFIED IDEOGRAPH:'C4BB:50363:&#x8E89
CJK UNIFIED IDEOGRAPH:'C4BC:50364:&#x8E81
CJK UNIFIED IDEOGRAPH:'C4BD:50365:&#x8E85
CJK UNIFIED IDEOGRAPH:'C4BE:50366:&#x8E82
CJK UNIFIED IDEOGRAPH:'C4BF:50367:&#x91B4
CJK UNIFIED IDEOGRAPH:'C4C0:50368:&#x91CB
CJK UNIFIED IDEOGRAPH:'C4C1:50369:&#x9418
CJK UNIFIED IDEOGRAPH:'C4C2:50370:&#x9403
CJK UNIFIED IDEOGRAPH:'C4C3:50371:&#x93FD
CJK UNIFIED IDEOGRAPH:'C4C4:50372:&#x95E1
CJK UNIFIED IDEOGRAPH:'C4C5:50373:&#x9730
CJK UNIFIED IDEOGRAPH:'C4C6:50374:&#x98C4
CJK UNIFIED IDEOGRAPH:'C4C7:50375:&#x9952
CJK UNIFIED IDEOGRAPH:'C4C8:50376:&#x9951
CJK UNIFIED IDEOGRAPH:'C4C9:50377:&#x99A8
CJK UNIFIED IDEOGRAPH:'C4CA:50378:&#x9A2B
CJK UNIFIED IDEOGRAPH:'C4CB:50379:&#x9A30
CJK UNIFIED IDEOGRAPH:'C4CC:50380:&#x9A37
CJK UNIFIED IDEOGRAPH:'C4CD:50381:&#x9A35
CJK UNIFIED IDEOGRAPH:'C4CE:50382:&#x9C13
CJK UNIFIED IDEOGRAPH:'C4CF:50383:&#x9C0D
CJK UNIFIED IDEOGRAPH:'C4D0:50384:&#x9E79
CJK UNIFIED IDEOGRAPH:'C4D1:50385:&#x9EB5
CJK UNIFIED IDEOGRAPH:'C4D2:50386:&#x9EE8
CJK UNIFIED IDEOGRAPH:'C4D3:50387:&#x9F2F
CJK UNIFIED IDEOGRAPH:'C4D4:50388:&#x9F5F
CJK UNIFIED IDEOGRAPH:'C4D5:50389:&#x9F63
CJK UNIFIED IDEOGRAPH:'C4D6:50390:&#x9F61
CJK UNIFIED IDEOGRAPH:'C4D7:50391:&#x5137
CJK UNIFIED IDEOGRAPH:'C4D8:50392:&#x5138
CJK UNIFIED IDEOGRAPH:'C4D9:50393:&#x56C1
CJK UNIFIED IDEOGRAPH:'C4DA:50394:&#x56C0
CJK UNIFIED IDEOGRAPH:'C4DB:50395:&#x56C2
CJK UNIFIED IDEOGRAPH:'C4DC:50396:&#x5914
CJK UNIFIED IDEOGRAPH:'C4DD:50397:&#x5C6C
CJK UNIFIED IDEOGRAPH:'C4DE:50398:&#x5DCD
CJK UNIFIED IDEOGRAPH:'C4DF:50399:&#x61FC
CJK UNIFIED IDEOGRAPH:'C4E0:50400:&#x61FE
CJK UNIFIED IDEOGRAPH:'C4E1:50401:&#x651D
CJK UNIFIED IDEOGRAPH:'C4E2:50402:&#x651C
CJK UNIFIED IDEOGRAPH:'C4E3:50403:&#x6595
CJK UNIFIED IDEOGRAPH:'C4E4:50404:&#x66E9
CJK UNIFIED IDEOGRAPH:'C4E5:50405:&#x6AFB
CJK UNIFIED IDEOGRAPH:'C4E6:50406:&#x6B04
CJK UNIFIED IDEOGRAPH:'C4E7:50407:&#x6AFA
CJK UNIFIED IDEOGRAPH:'C4E8:50408:&#x6BB2
CJK UNIFIED IDEOGRAPH:'C4E9:50409:&#x704C
CJK UNIFIED IDEOGRAPH:'C4EA:50410:&#x721B
CJK UNIFIED IDEOGRAPH:'C4EB:50411:&#x72A7
CJK UNIFIED IDEOGRAPH:'C4EC:50412:&#x74D6
CJK UNIFIED IDEOGRAPH:'C4ED:50413:&#x74D4
CJK UNIFIED IDEOGRAPH:'C4EE:50414:&#x7669
CJK UNIFIED IDEOGRAPH:'C4EF:50415:&#x77D3
CJK UNIFIED IDEOGRAPH:'C4F0:50416:&#x7C50
CJK UNIFIED IDEOGRAPH:'C4F1:50417:&#x7E8F
CJK UNIFIED IDEOGRAPH:'C4F2:50418:&#x7E8C
CJK UNIFIED IDEOGRAPH:'C4F3:50419:&#x7FBC
CJK UNIFIED IDEOGRAPH:'C4F4:50420:&#x8617
CJK UNIFIED IDEOGRAPH:'C4F5:50421:&#x862D
CJK UNIFIED IDEOGRAPH:'C4F6:50422:&#x861A
CJK UNIFIED IDEOGRAPH:'C4F7:50423:&#x8823
CJK UNIFIED IDEOGRAPH:'C4F8:50424:&#x8822
CJK UNIFIED IDEOGRAPH:'C4F9:50425:&#x8821
CJK UNIFIED IDEOGRAPH:'C4FA:50426:&#x881F
CJK UNIFIED IDEOGRAPH:'C4FB:50427:&#x896A
CJK UNIFIED IDEOGRAPH:'C4FC:50428:&#x896C
CJK UNIFIED IDEOGRAPH:'C4FD:50429:&#x89BD
CJK UNIFIED IDEOGRAPH:'C4FE:50430:&#x8B74
CJK UNIFIED IDEOGRAPH:'C540:50496:&#x8B77
CJK UNIFIED IDEOGRAPH:'C541:50497:&#x8B7D
CJK UNIFIED IDEOGRAPH:'C542:50498:&#x8D13
CJK UNIFIED IDEOGRAPH:'C543:50499:&#x8E8A
CJK UNIFIED IDEOGRAPH:'C544:50500:&#x8E8D
CJK UNIFIED IDEOGRAPH:'C545:50501:&#x8E8B
CJK UNIFIED IDEOGRAPH:'C546:50502:&#x8F5F
CJK UNIFIED IDEOGRAPH:'C547:50503:&#x8FAF
CJK UNIFIED IDEOGRAPH:'C548:50504:&#x91BA
CJK UNIFIED IDEOGRAPH:'C549:50505:&#x942E
CJK UNIFIED IDEOGRAPH:'C54A:50506:&#x9433
CJK UNIFIED IDEOGRAPH:'C54B:50507:&#x9435
CJK UNIFIED IDEOGRAPH:'C54C:50508:&#x943A
CJK UNIFIED IDEOGRAPH:'C54D:50509:&#x9438
CJK UNIFIED IDEOGRAPH:'C54E:50510:&#x9432
CJK UNIFIED IDEOGRAPH:'C54F:50511:&#x942B
CJK UNIFIED IDEOGRAPH:'C550:50512:&#x95E2
CJK UNIFIED IDEOGRAPH:'C551:50513:&#x9738
CJK UNIFIED IDEOGRAPH:'C552:50514:&#x9739
CJK UNIFIED IDEOGRAPH:'C553:50515:&#x9732
CJK UNIFIED IDEOGRAPH:'C554:50516:&#x97FF
CJK UNIFIED IDEOGRAPH:'C555:50517:&#x9867
CJK UNIFIED IDEOGRAPH:'C556:50518:&#x9865
CJK UNIFIED IDEOGRAPH:'C557:50519:&#x9957
CJK UNIFIED IDEOGRAPH:'C558:50520:&#x9A45
CJK UNIFIED IDEOGRAPH:'C559:50521:&#x9A43
CJK UNIFIED IDEOGRAPH:'C55A:50522:&#x9A40
CJK UNIFIED IDEOGRAPH:'C55B:50523:&#x9A3E
CJK UNIFIED IDEOGRAPH:'C55C:50524:&#x9ACF
CJK UNIFIED IDEOGRAPH:'C55D:50525:&#x9B54
CJK UNIFIED IDEOGRAPH:'C55E:50526:&#x9B51
CJK UNIFIED IDEOGRAPH:'C55F:50527:&#x9C2D
CJK UNIFIED IDEOGRAPH:'C560:50528:&#x9C25
CJK UNIFIED IDEOGRAPH:'C561:50529:&#x9DAF
CJK UNIFIED IDEOGRAPH:'C562:50530:&#x9DB4
CJK UNIFIED IDEOGRAPH:'C563:50531:&#x9DC2
CJK UNIFIED IDEOGRAPH:'C564:50532:&#x9DB8
CJK UNIFIED IDEOGRAPH:'C565:50533:&#x9E9D
CJK UNIFIED IDEOGRAPH:'C566:50534:&#x9EEF
CJK UNIFIED IDEOGRAPH:'C567:50535:&#x9F19
CJK UNIFIED IDEOGRAPH:'C568:50536:&#x9F5C
CJK UNIFIED IDEOGRAPH:'C569:50537:&#x9F66
CJK UNIFIED IDEOGRAPH:'C56A:50538:&#x9F67
CJK UNIFIED IDEOGRAPH:'C56B:50539:&#x513C
CJK UNIFIED IDEOGRAPH:'C56C:50540:&#x513B
CJK UNIFIED IDEOGRAPH:'C56D:50541:&#x56C8
CJK UNIFIED IDEOGRAPH:'C56E:50542:&#x56CA
CJK UNIFIED IDEOGRAPH:'C56F:50543:&#x56C9
CJK UNIFIED IDEOGRAPH:'C570:50544:&#x5B7F
CJK UNIFIED IDEOGRAPH:'C571:50545:&#x5DD4
CJK UNIFIED IDEOGRAPH:'C572:50546:&#x5DD2
CJK UNIFIED IDEOGRAPH:'C573:50547:&#x5F4E
CJK UNIFIED IDEOGRAPH:'C574:50548:&#x61FF
CJK UNIFIED IDEOGRAPH:'C575:50549:&#x6524
CJK UNIFIED IDEOGRAPH:'C576:50550:&#x6B0A
CJK UNIFIED IDEOGRAPH:'C577:50551:&#x6B61
CJK UNIFIED IDEOGRAPH:'C578:50552:&#x7051
CJK UNIFIED IDEOGRAPH:'C579:50553:&#x7058
CJK UNIFIED IDEOGRAPH:'C57A:50554:&#x7380
CJK UNIFIED IDEOGRAPH:'C57B:50555:&#x74E4
CJK UNIFIED IDEOGRAPH:'C57C:50556:&#x758A
CJK UNIFIED IDEOGRAPH:'C57D:50557:&#x766E
CJK UNIFIED IDEOGRAPH:'C57E:50558:&#x766C
CJK UNIFIED IDEOGRAPH:'C5A1:50593:&#x79B3
CJK UNIFIED IDEOGRAPH:'C5A2:50594:&#x7C60
CJK UNIFIED IDEOGRAPH:'C5A3:50595:&#x7C5F
CJK UNIFIED IDEOGRAPH:'C5A4:50596:&#x807E
CJK UNIFIED IDEOGRAPH:'C5A5:50597:&#x807D
CJK UNIFIED IDEOGRAPH:'C5A6:50598:&#x81DF
CJK UNIFIED IDEOGRAPH:'C5A7:50599:&#x8972
CJK UNIFIED IDEOGRAPH:'C5A8:50600:&#x896F
CJK UNIFIED IDEOGRAPH:'C5A9:50601:&#x89FC
CJK UNIFIED IDEOGRAPH:'C5AA:50602:&#x8B80
CJK UNIFIED IDEOGRAPH:'C5AB:50603:&#x8D16
CJK UNIFIED IDEOGRAPH:'C5AC:50604:&#x8D17
CJK UNIFIED IDEOGRAPH:'C5AD:50605:&#x8E91
CJK UNIFIED IDEOGRAPH:'C5AE:50606:&#x8E93
CJK UNIFIED IDEOGRAPH:'C5AF:50607:&#x8F61
CJK UNIFIED IDEOGRAPH:'C5B0:50608:&#x9148
CJK UNIFIED IDEOGRAPH:'C5B1:50609:&#x9444
CJK UNIFIED IDEOGRAPH:'C5B2:50610:&#x9451
CJK UNIFIED IDEOGRAPH:'C5B3:50611:&#x9452
CJK UNIFIED IDEOGRAPH:'C5B4:50612:&#x973D
CJK UNIFIED IDEOGRAPH:'C5B5:50613:&#x973E
CJK UNIFIED IDEOGRAPH:'C5B6:50614:&#x97C3
CJK UNIFIED IDEOGRAPH:'C5B7:50615:&#x97C1
CJK UNIFIED IDEOGRAPH:'C5B8:50616:&#x986B
CJK UNIFIED IDEOGRAPH:'C5B9:50617:&#x9955
CJK UNIFIED IDEOGRAPH:'C5BA:50618:&#x9A55
CJK UNIFIED IDEOGRAPH:'C5BB:50619:&#x9A4D
CJK UNIFIED IDEOGRAPH:'C5BC:50620:&#x9AD2
CJK UNIFIED IDEOGRAPH:'C5BD:50621:&#x9B1A
CJK UNIFIED IDEOGRAPH:'C5BE:50622:&#x9C49
CJK UNIFIED IDEOGRAPH:'C5BF:50623:&#x9C31
CJK UNIFIED IDEOGRAPH:'C5C0:50624:&#x9C3E
CJK UNIFIED IDEOGRAPH:'C5C1:50625:&#x9C3B
CJK UNIFIED IDEOGRAPH:'C5C2:50626:&#x9DD3
CJK UNIFIED IDEOGRAPH:'C5C3:50627:&#x9DD7
CJK UNIFIED IDEOGRAPH:'C5C4:50628:&#x9F34
CJK UNIFIED IDEOGRAPH:'C5C5:50629:&#x9F6C
CJK UNIFIED IDEOGRAPH:'C5C6:50630:&#x9F6A
CJK UNIFIED IDEOGRAPH:'C5C7:50631:&#x9F94
CJK UNIFIED IDEOGRAPH:'C5C8:50632:&#x56CC
CJK UNIFIED IDEOGRAPH:'C5C9:50633:&#x5DD6
CJK UNIFIED IDEOGRAPH:'C5CA:50634:&#x6200
CJK UNIFIED IDEOGRAPH:'C5CB:50635:&#x6523
CJK UNIFIED IDEOGRAPH:'C5CC:50636:&#x652B
CJK UNIFIED IDEOGRAPH:'C5CD:50637:&#x652A
CJK UNIFIED IDEOGRAPH:'C5CE:50638:&#x66EC
CJK UNIFIED IDEOGRAPH:'C5CF:50639:&#x6B10
CJK UNIFIED IDEOGRAPH:'C5D0:50640:&#x74DA
CJK UNIFIED IDEOGRAPH:'C5D1:50641:&#x7ACA
CJK UNIFIED IDEOGRAPH:'C5D2:50642:&#x7C64
CJK UNIFIED IDEOGRAPH:'C5D3:50643:&#x7C63
CJK UNIFIED IDEOGRAPH:'C5D4:50644:&#x7C65
CJK UNIFIED IDEOGRAPH:'C5D5:50645:&#x7E93
CJK UNIFIED IDEOGRAPH:'C5D6:50646:&#x7E96
CJK UNIFIED IDEOGRAPH:'C5D7:50647:&#x7E94
CJK UNIFIED IDEOGRAPH:'C5D8:50648:&#x81E2
CJK UNIFIED IDEOGRAPH:'C5D9:50649:&#x8638
CJK UNIFIED IDEOGRAPH:'C5DA:50650:&#x863F
CJK UNIFIED IDEOGRAPH:'C5DB:50651:&#x8831
CJK UNIFIED IDEOGRAPH:'C5DC:50652:&#x8B8A
CJK UNIFIED IDEOGRAPH:'C5DD:50653:&#x9090
CJK UNIFIED IDEOGRAPH:'C5DE:50654:&#x908F
CJK UNIFIED IDEOGRAPH:'C5DF:50655:&#x9463
CJK UNIFIED IDEOGRAPH:'C5E0:50656:&#x9460
CJK UNIFIED IDEOGRAPH:'C5E1:50657:&#x9464
CJK UNIFIED IDEOGRAPH:'C5E2:50658:&#x9768
CJK UNIFIED IDEOGRAPH:'C5E3:50659:&#x986F
CJK UNIFIED IDEOGRAPH:'C5E4:50660:&#x995C
CJK UNIFIED IDEOGRAPH:'C5E5:50661:&#x9A5A
CJK UNIFIED IDEOGRAPH:'C5E6:50662:&#x9A5B
CJK UNIFIED IDEOGRAPH:'C5E7:50663:&#x9A57
CJK UNIFIED IDEOGRAPH:'C5E8:50664:&#x9AD3
CJK UNIFIED IDEOGRAPH:'C5E9:50665:&#x9AD4
CJK UNIFIED IDEOGRAPH:'C5EA:50666:&#x9AD1
CJK UNIFIED IDEOGRAPH:'C5EB:50667:&#x9C54
CJK UNIFIED IDEOGRAPH:'C5EC:50668:&#x9C57
CJK UNIFIED IDEOGRAPH:'C5ED:50669:&#x9C56
CJK UNIFIED IDEOGRAPH:'C5EE:50670:&#x9DE5
CJK UNIFIED IDEOGRAPH:'C5EF:50671:&#x9E9F
CJK UNIFIED IDEOGRAPH:'C5F0:50672:&#x9EF4
CJK UNIFIED IDEOGRAPH:'C5F1:50673:&#x56D1
CJK UNIFIED IDEOGRAPH:'C5F2:50674:&#x58E9
CJK UNIFIED IDEOGRAPH:'C5F3:50675:&#x652C
CJK UNIFIED IDEOGRAPH:'C5F4:50676:&#x705E
CJK UNIFIED IDEOGRAPH:'C5F5:50677:&#x7671
CJK UNIFIED IDEOGRAPH:'C5F6:50678:&#x7672
CJK UNIFIED IDEOGRAPH:'C5F7:50679:&#x77D7
CJK UNIFIED IDEOGRAPH:'C5F8:50680:&#x7F50
CJK UNIFIED IDEOGRAPH:'C5F9:50681:&#x7F88
CJK UNIFIED IDEOGRAPH:'C5FA:50682:&#x8836
CJK UNIFIED IDEOGRAPH:'C5FB:50683:&#x8839
CJK UNIFIED IDEOGRAPH:'C5FC:50684:&#x8862
CJK UNIFIED IDEOGRAPH:'C5FD:50685:&#x8B93
CJK UNIFIED IDEOGRAPH:'C5FE:50686:&#x8B92
CJK UNIFIED IDEOGRAPH:'C640:50752:&#x8B96
CJK UNIFIED IDEOGRAPH:'C641:50753:&#x8277
CJK UNIFIED IDEOGRAPH:'C642:50754:&#x8D1B
CJK UNIFIED IDEOGRAPH:'C643:50755:&#x91C0
CJK UNIFIED IDEOGRAPH:'C644:50756:&#x946A
CJK UNIFIED IDEOGRAPH:'C645:50757:&#x9742
CJK UNIFIED IDEOGRAPH:'C646:50758:&#x9748
CJK UNIFIED IDEOGRAPH:'C647:50759:&#x9744
CJK UNIFIED IDEOGRAPH:'C648:50760:&#x97C6
CJK UNIFIED IDEOGRAPH:'C649:50761:&#x9870
CJK UNIFIED IDEOGRAPH:'C64A:50762:&#x9A5F
CJK UNIFIED IDEOGRAPH:'C64B:50763:&#x9B22
CJK UNIFIED IDEOGRAPH:'C64C:50764:&#x9B58
CJK UNIFIED IDEOGRAPH:'C64D:50765:&#x9C5F
CJK UNIFIED IDEOGRAPH:'C64E:50766:&#x9DF9
CJK UNIFIED IDEOGRAPH:'C64F:50767:&#x9DFA
CJK UNIFIED IDEOGRAPH:'C650:50768:&#x9E7C
CJK UNIFIED IDEOGRAPH:'C651:50769:&#x9E7D
CJK UNIFIED IDEOGRAPH:'C652:50770:&#x9F07
CJK UNIFIED IDEOGRAPH:'C653:50771:&#x9F77
CJK UNIFIED IDEOGRAPH:'C654:50772:&#x9F72
CJK UNIFIED IDEOGRAPH:'C655:50773:&#x5EF3
CJK UNIFIED IDEOGRAPH:'C656:50774:&#x6B16
CJK UNIFIED IDEOGRAPH:'C657:50775:&#x7063
CJK UNIFIED IDEOGRAPH:'C658:50776:&#x7C6C
CJK UNIFIED IDEOGRAPH:'C659:50777:&#x7C6E
CJK UNIFIED IDEOGRAPH:'C65A:50778:&#x883B
CJK UNIFIED IDEOGRAPH:'C65B:50779:&#x89C0
CJK UNIFIED IDEOGRAPH:'C65C:50780:&#x8EA1
CJK UNIFIED IDEOGRAPH:'C65D:50781:&#x91C1
CJK UNIFIED IDEOGRAPH:'C65E:50782:&#x9472
CJK UNIFIED IDEOGRAPH:'C65F:50783:&#x9470
CJK UNIFIED IDEOGRAPH:'C660:50784:&#x9871
CJK UNIFIED IDEOGRAPH:'C661:50785:&#x995E
CJK UNIFIED IDEOGRAPH:'C662:50786:&#x9AD6
CJK UNIFIED IDEOGRAPH:'C663:50787:&#x9B23
CJK UNIFIED IDEOGRAPH:'C664:50788:&#x9ECC
CJK UNIFIED IDEOGRAPH:'C665:50789:&#x7064
CJK UNIFIED IDEOGRAPH:'C666:50790:&#x77DA
CJK UNIFIED IDEOGRAPH:'C667:50791:&#x8B9A
CJK UNIFIED IDEOGRAPH:'C668:50792:&#x9477
CJK UNIFIED IDEOGRAPH:'C669:50793:&#x97C9
CJK UNIFIED IDEOGRAPH:'C66A:50794:&#x9A62
CJK UNIFIED IDEOGRAPH:'C66B:50795:&#x9A65
CJK UNIFIED IDEOGRAPH:'C66C:50796:&#x7E9C
CJK UNIFIED IDEOGRAPH:'C66D:50797:&#x8B9C
CJK UNIFIED IDEOGRAPH:'C66E:50798:&#x8EAA
CJK UNIFIED IDEOGRAPH:'C66F:50799:&#x91C5
CJK UNIFIED IDEOGRAPH:'C670:50800:&#x947D
CJK UNIFIED IDEOGRAPH:'C671:50801:&#x947E
CJK UNIFIED IDEOGRAPH:'C672:50802:&#x947C
CJK UNIFIED IDEOGRAPH:'C673:50803:&#x9C77
CJK UNIFIED IDEOGRAPH:'C674:50804:&#x9C78
CJK UNIFIED IDEOGRAPH:'C675:50805:&#x9EF7
CJK UNIFIED IDEOGRAPH:'C676:50806:&#x8C54
CJK UNIFIED IDEOGRAPH:'C677:50807:&#x947F
CJK UNIFIED IDEOGRAPH:'C678:50808:&#x9E1A
CJK UNIFIED IDEOGRAPH:'C679:50809:&#x7228
CJK UNIFIED IDEOGRAPH:'C67A:50810:&#x9A6A
CJK UNIFIED IDEOGRAPH:'C67B:50811:&#x9B31
CJK UNIFIED IDEOGRAPH:'C67C:50812:&#x9E1B
CJK UNIFIED IDEOGRAPH:'C67D:50813:&#x9E1E
CJK UNIFIED IDEOGRAPH:'C67E:50814:&#x7C72
CJK UNIFIED IDEOGRAPH:'C940:51520:&#x4E42
CJK UNIFIED IDEOGRAPH:'C941:51521:&#x4E5C
CJK UNIFIED IDEOGRAPH:'C942:51522:&#x51F5
CJK UNIFIED IDEOGRAPH:'C943:51523:&#x531A
CJK UNIFIED IDEOGRAPH:'C944:51524:&#x5382
CJK UNIFIED IDEOGRAPH:'C945:51525:&#x4E07
CJK UNIFIED IDEOGRAPH:'C946:51526:&#x4E0C
CJK UNIFIED IDEOGRAPH:'C947:51527:&#x4E47
CJK UNIFIED IDEOGRAPH:'C948:51528:&#x4E8D
CJK UNIFIED IDEOGRAPH:'C949:51529:&#x56D7
CJK COMPATIBILITY IDEOGRAPH:'C94A:51530:&#xFA0C
CJK UNIFIED IDEOGRAPH:'C94B:51531:&#x5C6E
CJK UNIFIED IDEOGRAPH:'C94C:51532:&#x5F73
CJK UNIFIED IDEOGRAPH:'C94D:51533:&#x4E0F
CJK UNIFIED IDEOGRAPH:'C94E:51534:&#x5187
CJK UNIFIED IDEOGRAPH:'C94F:51535:&#x4E0E
CJK UNIFIED IDEOGRAPH:'C950:51536:&#x4E2E
CJK UNIFIED IDEOGRAPH:'C951:51537:&#x4E93
CJK UNIFIED IDEOGRAPH:'C952:51538:&#x4EC2
CJK UNIFIED IDEOGRAPH:'C953:51539:&#x4EC9
CJK UNIFIED IDEOGRAPH:'C954:51540:&#x4EC8
CJK UNIFIED IDEOGRAPH:'C955:51541:&#x5198
CJK UNIFIED IDEOGRAPH:'C956:51542:&#x52FC
CJK UNIFIED IDEOGRAPH:'C957:51543:&#x536C
CJK UNIFIED IDEOGRAPH:'C958:51544:&#x53B9
CJK UNIFIED IDEOGRAPH:'C959:51545:&#x5720
CJK UNIFIED IDEOGRAPH:'C95A:51546:&#x5903
CJK UNIFIED IDEOGRAPH:'C95B:51547:&#x592C
CJK UNIFIED IDEOGRAPH:'C95C:51548:&#x5C10
CJK UNIFIED IDEOGRAPH:'C95D:51549:&#x5DFF
CJK UNIFIED IDEOGRAPH:'C95E:51550:&#x65E1
CJK UNIFIED IDEOGRAPH:'C95F:51551:&#x6BB3
CJK UNIFIED IDEOGRAPH:'C960:51552:&#x6BCC
CJK UNIFIED IDEOGRAPH:'C961:51553:&#x6C14
CJK UNIFIED IDEOGRAPH:'C962:51554:&#x723F
CJK UNIFIED IDEOGRAPH:'C963:51555:&#x4E31
CJK UNIFIED IDEOGRAPH:'C964:51556:&#x4E3C
CJK UNIFIED IDEOGRAPH:'C965:51557:&#x4EE8
CJK UNIFIED IDEOGRAPH:'C966:51558:&#x4EDC
CJK UNIFIED IDEOGRAPH:'C967:51559:&#x4EE9
CJK UNIFIED IDEOGRAPH:'C968:51560:&#x4EE1
CJK UNIFIED IDEOGRAPH:'C969:51561:&#x4EDD
CJK UNIFIED IDEOGRAPH:'C96A:51562:&#x4EDA
CJK UNIFIED IDEOGRAPH:'C96B:51563:&#x520C
CJK UNIFIED IDEOGRAPH:'C96C:51564:&#x531C
CJK UNIFIED IDEOGRAPH:'C96D:51565:&#x534C
CJK UNIFIED IDEOGRAPH:'C96E:51566:&#x5722
CJK UNIFIED IDEOGRAPH:'C96F:51567:&#x5723
CJK UNIFIED IDEOGRAPH:'C970:51568:&#x5917
CJK UNIFIED IDEOGRAPH:'C971:51569:&#x592F
CJK UNIFIED IDEOGRAPH:'C972:51570:&#x5B81
CJK UNIFIED IDEOGRAPH:'C973:51571:&#x5B84
CJK UNIFIED IDEOGRAPH:'C974:51572:&#x5C12
CJK UNIFIED IDEOGRAPH:'C975:51573:&#x5C3B
CJK UNIFIED IDEOGRAPH:'C976:51574:&#x5C74
CJK UNIFIED IDEOGRAPH:'C977:51575:&#x5C73
CJK UNIFIED IDEOGRAPH:'C978:51576:&#x5E04
CJK UNIFIED IDEOGRAPH:'C979:51577:&#x5E80
CJK UNIFIED IDEOGRAPH:'C97A:51578:&#x5E82
CJK UNIFIED IDEOGRAPH:'C97B:51579:&#x5FC9
CJK UNIFIED IDEOGRAPH:'C97C:51580:&#x6209
CJK UNIFIED IDEOGRAPH:'C97D:51581:&#x6250
CJK UNIFIED IDEOGRAPH:'C97E:51582:&#x6C15
CJK UNIFIED IDEOGRAPH:'C9A1:51617:&#x6C36
CJK UNIFIED IDEOGRAPH:'C9A2:51618:&#x6C43
CJK UNIFIED IDEOGRAPH:'C9A3:51619:&#x6C3F
CJK UNIFIED IDEOGRAPH:'C9A4:51620:&#x6C3B
CJK UNIFIED IDEOGRAPH:'C9A5:51621:&#x72AE
CJK UNIFIED IDEOGRAPH:'C9A6:51622:&#x72B0
CJK UNIFIED IDEOGRAPH:'C9A7:51623:&#x738A
CJK UNIFIED IDEOGRAPH:'C9A8:51624:&#x79B8
CJK UNIFIED IDEOGRAPH:'C9A9:51625:&#x808A
CJK UNIFIED IDEOGRAPH:'C9AA:51626:&#x961E
CJK UNIFIED IDEOGRAPH:'C9AB:51627:&#x4F0E
CJK UNIFIED IDEOGRAPH:'C9AC:51628:&#x4F18
CJK UNIFIED IDEOGRAPH:'C9AD:51629:&#x4F2C
CJK UNIFIED IDEOGRAPH:'C9AE:51630:&#x4EF5
CJK UNIFIED IDEOGRAPH:'C9AF:51631:&#x4F14
CJK UNIFIED IDEOGRAPH:'C9B0:51632:&#x4EF1
CJK UNIFIED IDEOGRAPH:'C9B1:51633:&#x4F00
CJK UNIFIED IDEOGRAPH:'C9B2:51634:&#x4EF7
CJK UNIFIED IDEOGRAPH:'C9B3:51635:&#x4F08
CJK UNIFIED IDEOGRAPH:'C9B4:51636:&#x4F1D
CJK UNIFIED IDEOGRAPH:'C9B5:51637:&#x4F02
CJK UNIFIED IDEOGRAPH:'C9B6:51638:&#x4F05
CJK UNIFIED IDEOGRAPH:'C9B7:51639:&#x4F22
CJK UNIFIED IDEOGRAPH:'C9B8:51640:&#x4F13
CJK UNIFIED IDEOGRAPH:'C9B9:51641:&#x4F04
CJK UNIFIED IDEOGRAPH:'C9BA:51642:&#x4EF4
CJK UNIFIED IDEOGRAPH:'C9BB:51643:&#x4F12
CJK UNIFIED IDEOGRAPH:'C9BC:51644:&#x51B1
CJK UNIFIED IDEOGRAPH:'C9BD:51645:&#x5213
CJK UNIFIED IDEOGRAPH:'C9BE:51646:&#x5209
CJK UNIFIED IDEOGRAPH:'C9BF:51647:&#x5210
CJK UNIFIED IDEOGRAPH:'C9C0:51648:&#x52A6
CJK UNIFIED IDEOGRAPH:'C9C1:51649:&#x5322
CJK UNIFIED IDEOGRAPH:'C9C2:51650:&#x531F
CJK UNIFIED IDEOGRAPH:'C9C3:51651:&#x534D
CJK UNIFIED IDEOGRAPH:'C9C4:51652:&#x538A
CJK UNIFIED IDEOGRAPH:'C9C5:51653:&#x5407
CJK UNIFIED IDEOGRAPH:'C9C6:51654:&#x56E1
CJK UNIFIED IDEOGRAPH:'C9C7:51655:&#x56DF
CJK UNIFIED IDEOGRAPH:'C9C8:51656:&#x572E
CJK UNIFIED IDEOGRAPH:'C9C9:51657:&#x572A
CJK UNIFIED IDEOGRAPH:'C9CA:51658:&#x5734
CJK UNIFIED IDEOGRAPH:'C9CB:51659:&#x593C
CJK UNIFIED IDEOGRAPH:'C9CC:51660:&#x5980
CJK UNIFIED IDEOGRAPH:'C9CD:51661:&#x597C
CJK UNIFIED IDEOGRAPH:'C9CE:51662:&#x5985
CJK UNIFIED IDEOGRAPH:'C9CF:51663:&#x597B
CJK UNIFIED IDEOGRAPH:'C9D0:51664:&#x597E
CJK UNIFIED IDEOGRAPH:'C9D1:51665:&#x5977
CJK UNIFIED IDEOGRAPH:'C9D2:51666:&#x597F
CJK UNIFIED IDEOGRAPH:'C9D3:51667:&#x5B56
CJK UNIFIED IDEOGRAPH:'C9D4:51668:&#x5C15
CJK UNIFIED IDEOGRAPH:'C9D5:51669:&#x5C25
CJK UNIFIED IDEOGRAPH:'C9D6:51670:&#x5C7C
CJK UNIFIED IDEOGRAPH:'C9D7:51671:&#x5C7A
CJK UNIFIED IDEOGRAPH:'C9D8:51672:&#x5C7B
CJK UNIFIED IDEOGRAPH:'C9D9:51673:&#x5C7E
CJK UNIFIED IDEOGRAPH:'C9DA:51674:&#x5DDF
CJK UNIFIED IDEOGRAPH:'C9DB:51675:&#x5E75
CJK UNIFIED IDEOGRAPH:'C9DC:51676:&#x5E84
CJK UNIFIED IDEOGRAPH:'C9DD:51677:&#x5F02
CJK UNIFIED IDEOGRAPH:'C9DE:51678:&#x5F1A
CJK UNIFIED IDEOGRAPH:'C9DF:51679:&#x5F74
CJK UNIFIED IDEOGRAPH:'C9E0:51680:&#x5FD5
CJK UNIFIED IDEOGRAPH:'C9E1:51681:&#x5FD4
CJK UNIFIED IDEOGRAPH:'C9E2:51682:&#x5FCF
CJK UNIFIED IDEOGRAPH:'C9E3:51683:&#x625C
CJK UNIFIED IDEOGRAPH:'C9E4:51684:&#x625E
CJK UNIFIED IDEOGRAPH:'C9E5:51685:&#x6264
CJK UNIFIED IDEOGRAPH:'C9E6:51686:&#x6261
CJK UNIFIED IDEOGRAPH:'C9E7:51687:&#x6266
CJK UNIFIED IDEOGRAPH:'C9E8:51688:&#x6262
CJK UNIFIED IDEOGRAPH:'C9E9:51689:&#x6259
CJK UNIFIED IDEOGRAPH:'C9EA:51690:&#x6260
CJK UNIFIED IDEOGRAPH:'C9EB:51691:&#x625A
CJK UNIFIED IDEOGRAPH:'C9EC:51692:&#x6265
CJK UNIFIED IDEOGRAPH:'C9ED:51693:&#x65EF
CJK UNIFIED IDEOGRAPH:'C9EE:51694:&#x65EE
CJK UNIFIED IDEOGRAPH:'C9EF:51695:&#x673E
CJK UNIFIED IDEOGRAPH:'C9F0:51696:&#x6739
CJK UNIFIED IDEOGRAPH:'C9F1:51697:&#x6738
CJK UNIFIED IDEOGRAPH:'C9F2:51698:&#x673B
CJK UNIFIED IDEOGRAPH:'C9F3:51699:&#x673A
CJK UNIFIED IDEOGRAPH:'C9F4:51700:&#x673F
CJK UNIFIED IDEOGRAPH:'C9F5:51701:&#x673C
CJK UNIFIED IDEOGRAPH:'C9F6:51702:&#x6733
CJK UNIFIED IDEOGRAPH:'C9F7:51703:&#x6C18
CJK UNIFIED IDEOGRAPH:'C9F8:51704:&#x6C46
CJK UNIFIED IDEOGRAPH:'C9F9:51705:&#x6C52
CJK UNIFIED IDEOGRAPH:'C9FA:51706:&#x6C5C
CJK UNIFIED IDEOGRAPH:'C9FB:51707:&#x6C4F
CJK UNIFIED IDEOGRAPH:'C9FC:51708:&#x6C4A
CJK UNIFIED IDEOGRAPH:'C9FD:51709:&#x6C54
CJK UNIFIED IDEOGRAPH:'C9FE:51710:&#x6C4B
CJK UNIFIED IDEOGRAPH:'CA40:51776:&#x6C4C
CJK UNIFIED IDEOGRAPH:'CA41:51777:&#x7071
CJK UNIFIED IDEOGRAPH:'CA42:51778:&#x725E
CJK UNIFIED IDEOGRAPH:'CA43:51779:&#x72B4
CJK UNIFIED IDEOGRAPH:'CA44:51780:&#x72B5
CJK UNIFIED IDEOGRAPH:'CA45:51781:&#x738E
CJK UNIFIED IDEOGRAPH:'CA46:51782:&#x752A
CJK UNIFIED IDEOGRAPH:'CA47:51783:&#x767F
CJK UNIFIED IDEOGRAPH:'CA48:51784:&#x7A75
CJK UNIFIED IDEOGRAPH:'CA49:51785:&#x7F51
CJK UNIFIED IDEOGRAPH:'CA4A:51786:&#x8278
CJK UNIFIED IDEOGRAPH:'CA4B:51787:&#x827C
CJK UNIFIED IDEOGRAPH:'CA4C:51788:&#x8280
CJK UNIFIED IDEOGRAPH:'CA4D:51789:&#x827D
CJK UNIFIED IDEOGRAPH:'CA4E:51790:&#x827F
CJK UNIFIED IDEOGRAPH:'CA4F:51791:&#x864D
CJK UNIFIED IDEOGRAPH:'CA50:51792:&#x897E
CJK UNIFIED IDEOGRAPH:'CA51:51793:&#x9099
CJK UNIFIED IDEOGRAPH:'CA52:51794:&#x9097
CJK UNIFIED IDEOGRAPH:'CA53:51795:&#x9098
CJK UNIFIED IDEOGRAPH:'CA54:51796:&#x909B
CJK UNIFIED IDEOGRAPH:'CA55:51797:&#x9094
CJK UNIFIED IDEOGRAPH:'CA56:51798:&#x9622
CJK UNIFIED IDEOGRAPH:'CA57:51799:&#x9624
CJK UNIFIED IDEOGRAPH:'CA58:51800:&#x9620
CJK UNIFIED IDEOGRAPH:'CA59:51801:&#x9623
CJK UNIFIED IDEOGRAPH:'CA5A:51802:&#x4F56
CJK UNIFIED IDEOGRAPH:'CA5B:51803:&#x4F3B
CJK UNIFIED IDEOGRAPH:'CA5C:51804:&#x4F62
CJK UNIFIED IDEOGRAPH:'CA5D:51805:&#x4F49
CJK UNIFIED IDEOGRAPH:'CA5E:51806:&#x4F53
CJK UNIFIED IDEOGRAPH:'CA5F:51807:&#x4F64
CJK UNIFIED IDEOGRAPH:'CA60:51808:&#x4F3E
CJK UNIFIED IDEOGRAPH:'CA61:51809:&#x4F67
CJK UNIFIED IDEOGRAPH:'CA62:51810:&#x4F52
CJK UNIFIED IDEOGRAPH:'CA63:51811:&#x4F5F
CJK UNIFIED IDEOGRAPH:'CA64:51812:&#x4F41
CJK UNIFIED IDEOGRAPH:'CA65:51813:&#x4F58
CJK UNIFIED IDEOGRAPH:'CA66:51814:&#x4F2D
CJK UNIFIED IDEOGRAPH:'CA67:51815:&#x4F33
CJK UNIFIED IDEOGRAPH:'CA68:51816:&#x4F3F
CJK UNIFIED IDEOGRAPH:'CA69:51817:&#x4F61
CJK UNIFIED IDEOGRAPH:'CA6A:51818:&#x518F
CJK UNIFIED IDEOGRAPH:'CA6B:51819:&#x51B9
CJK UNIFIED IDEOGRAPH:'CA6C:51820:&#x521C
CJK UNIFIED IDEOGRAPH:'CA6D:51821:&#x521E
CJK UNIFIED IDEOGRAPH:'CA6E:51822:&#x5221
CJK UNIFIED IDEOGRAPH:'CA6F:51823:&#x52AD
CJK UNIFIED IDEOGRAPH:'CA70:51824:&#x52AE
CJK UNIFIED IDEOGRAPH:'CA71:51825:&#x5309
CJK UNIFIED IDEOGRAPH:'CA72:51826:&#x5363
CJK UNIFIED IDEOGRAPH:'CA73:51827:&#x5372
CJK UNIFIED IDEOGRAPH:'CA74:51828:&#x538E
CJK UNIFIED IDEOGRAPH:'CA75:51829:&#x538F
CJK UNIFIED IDEOGRAPH:'CA76:51830:&#x5430
CJK UNIFIED IDEOGRAPH:'CA77:51831:&#x5437
CJK UNIFIED IDEOGRAPH:'CA78:51832:&#x542A
CJK UNIFIED IDEOGRAPH:'CA79:51833:&#x5454
CJK UNIFIED IDEOGRAPH:'CA7A:51834:&#x5445
CJK UNIFIED IDEOGRAPH:'CA7B:51835:&#x5419
CJK UNIFIED IDEOGRAPH:'CA7C:51836:&#x541C
CJK UNIFIED IDEOGRAPH:'CA7D:51837:&#x5425
CJK UNIFIED IDEOGRAPH:'CA7E:51838:&#x5418
CJK UNIFIED IDEOGRAPH:'CAA1:51873:&#x543D
CJK UNIFIED IDEOGRAPH:'CAA2:51874:&#x544F
CJK UNIFIED IDEOGRAPH:'CAA3:51875:&#x5441
CJK UNIFIED IDEOGRAPH:'CAA4:51876:&#x5428
CJK UNIFIED IDEOGRAPH:'CAA5:51877:&#x5424
CJK UNIFIED IDEOGRAPH:'CAA6:51878:&#x5447
CJK UNIFIED IDEOGRAPH:'CAA7:51879:&#x56EE
CJK UNIFIED IDEOGRAPH:'CAA8:51880:&#x56E7
CJK UNIFIED IDEOGRAPH:'CAA9:51881:&#x56E5
CJK UNIFIED IDEOGRAPH:'CAAA:51882:&#x5741
CJK UNIFIED IDEOGRAPH:'CAAB:51883:&#x5745
CJK UNIFIED IDEOGRAPH:'CAAC:51884:&#x574C
CJK UNIFIED IDEOGRAPH:'CAAD:51885:&#x5749
CJK UNIFIED IDEOGRAPH:'CAAE:51886:&#x574B
CJK UNIFIED IDEOGRAPH:'CAAF:51887:&#x5752
CJK UNIFIED IDEOGRAPH:'CAB0:51888:&#x5906
CJK UNIFIED IDEOGRAPH:'CAB1:51889:&#x5940
CJK UNIFIED IDEOGRAPH:'CAB2:51890:&#x59A6
CJK UNIFIED IDEOGRAPH:'CAB3:51891:&#x5998
CJK UNIFIED IDEOGRAPH:'CAB4:51892:&#x59A0
CJK UNIFIED IDEOGRAPH:'CAB5:51893:&#x5997
CJK UNIFIED IDEOGRAPH:'CAB6:51894:&#x598E
CJK UNIFIED IDEOGRAPH:'CAB7:51895:&#x59A2
CJK UNIFIED IDEOGRAPH:'CAB8:51896:&#x5990
CJK UNIFIED IDEOGRAPH:'CAB9:51897:&#x598F
CJK UNIFIED IDEOGRAPH:'CABA:51898:&#x59A7
CJK UNIFIED IDEOGRAPH:'CABB:51899:&#x59A1
CJK UNIFIED IDEOGRAPH:'CABC:51900:&#x5B8E
CJK UNIFIED IDEOGRAPH:'CABD:51901:&#x5B92
CJK UNIFIED IDEOGRAPH:'CABE:51902:&#x5C28
CJK UNIFIED IDEOGRAPH:'CABF:51903:&#x5C2A
CJK UNIFIED IDEOGRAPH:'CAC0:51904:&#x5C8D
CJK UNIFIED IDEOGRAPH:'CAC1:51905:&#x5C8F
CJK UNIFIED IDEOGRAPH:'CAC2:51906:&#x5C88
CJK UNIFIED IDEOGRAPH:'CAC3:51907:&#x5C8B
CJK UNIFIED IDEOGRAPH:'CAC4:51908:&#x5C89
CJK UNIFIED IDEOGRAPH:'CAC5:51909:&#x5C92
CJK UNIFIED IDEOGRAPH:'CAC6:51910:&#x5C8A
CJK UNIFIED IDEOGRAPH:'CAC7:51911:&#x5C86
CJK UNIFIED IDEOGRAPH:'CAC8:51912:&#x5C93
CJK UNIFIED IDEOGRAPH:'CAC9:51913:&#x5C95
CJK UNIFIED IDEOGRAPH:'CACA:51914:&#x5DE0
CJK UNIFIED IDEOGRAPH:'CACB:51915:&#x5E0A
CJK UNIFIED IDEOGRAPH:'CACC:51916:&#x5E0E
CJK UNIFIED IDEOGRAPH:'CACD:51917:&#x5E8B
CJK UNIFIED IDEOGRAPH:'CACE:51918:&#x5E89
CJK UNIFIED IDEOGRAPH:'CACF:51919:&#x5E8C
CJK UNIFIED IDEOGRAPH:'CAD0:51920:&#x5E88
CJK UNIFIED IDEOGRAPH:'CAD1:51921:&#x5E8D
CJK UNIFIED IDEOGRAPH:'CAD2:51922:&#x5F05
CJK UNIFIED IDEOGRAPH:'CAD3:51923:&#x5F1D
CJK UNIFIED IDEOGRAPH:'CAD4:51924:&#x5F78
CJK UNIFIED IDEOGRAPH:'CAD5:51925:&#x5F76
CJK UNIFIED IDEOGRAPH:'CAD6:51926:&#x5FD2
CJK UNIFIED IDEOGRAPH:'CAD7:51927:&#x5FD1
CJK UNIFIED IDEOGRAPH:'CAD8:51928:&#x5FD0
CJK UNIFIED IDEOGRAPH:'CAD9:51929:&#x5FED
CJK UNIFIED IDEOGRAPH:'CADA:51930:&#x5FE8
CJK UNIFIED IDEOGRAPH:'CADB:51931:&#x5FEE
CJK UNIFIED IDEOGRAPH:'CADC:51932:&#x5FF3
CJK UNIFIED IDEOGRAPH:'CADD:51933:&#x5FE1
CJK UNIFIED IDEOGRAPH:'CADE:51934:&#x5FE4
CJK UNIFIED IDEOGRAPH:'CADF:51935:&#x5FE3
CJK UNIFIED IDEOGRAPH:'CAE0:51936:&#x5FFA
CJK UNIFIED IDEOGRAPH:'CAE1:51937:&#x5FEF
CJK UNIFIED IDEOGRAPH:'CAE2:51938:&#x5FF7
CJK UNIFIED IDEOGRAPH:'CAE3:51939:&#x5FFB
CJK UNIFIED IDEOGRAPH:'CAE4:51940:&#x6000
CJK UNIFIED IDEOGRAPH:'CAE5:51941:&#x5FF4
CJK UNIFIED IDEOGRAPH:'CAE6:51942:&#x623A
CJK UNIFIED IDEOGRAPH:'CAE7:51943:&#x6283
CJK UNIFIED IDEOGRAPH:'CAE8:51944:&#x628C
CJK UNIFIED IDEOGRAPH:'CAE9:51945:&#x628E
CJK UNIFIED IDEOGRAPH:'CAEA:51946:&#x628F
CJK UNIFIED IDEOGRAPH:'CAEB:51947:&#x6294
CJK UNIFIED IDEOGRAPH:'CAEC:51948:&#x6287
CJK UNIFIED IDEOGRAPH:'CAED:51949:&#x6271
CJK UNIFIED IDEOGRAPH:'CAEE:51950:&#x627B
CJK UNIFIED IDEOGRAPH:'CAEF:51951:&#x627A
CJK UNIFIED IDEOGRAPH:'CAF0:51952:&#x6270
CJK UNIFIED IDEOGRAPH:'CAF1:51953:&#x6281
CJK UNIFIED IDEOGRAPH:'CAF2:51954:&#x6288
CJK UNIFIED IDEOGRAPH:'CAF3:51955:&#x6277
CJK UNIFIED IDEOGRAPH:'CAF4:51956:&#x627D
CJK UNIFIED IDEOGRAPH:'CAF5:51957:&#x6272
CJK UNIFIED IDEOGRAPH:'CAF6:51958:&#x6274
CJK UNIFIED IDEOGRAPH:'CAF7:51959:&#x6537
CJK UNIFIED IDEOGRAPH:'CAF8:51960:&#x65F0
CJK UNIFIED IDEOGRAPH:'CAF9:51961:&#x65F4
CJK UNIFIED IDEOGRAPH:'CAFA:51962:&#x65F3
CJK UNIFIED IDEOGRAPH:'CAFB:51963:&#x65F2
CJK UNIFIED IDEOGRAPH:'CAFC:51964:&#x65F5
CJK UNIFIED IDEOGRAPH:'CAFD:51965:&#x6745
CJK UNIFIED IDEOGRAPH:'CAFE:51966:&#x6747
CJK UNIFIED IDEOGRAPH:'CB40:52032:&#x6759
CJK UNIFIED IDEOGRAPH:'CB41:52033:&#x6755
CJK UNIFIED IDEOGRAPH:'CB42:52034:&#x674C
CJK UNIFIED IDEOGRAPH:'CB43:52035:&#x6748
CJK UNIFIED IDEOGRAPH:'CB44:52036:&#x675D
CJK UNIFIED IDEOGRAPH:'CB45:52037:&#x674D
CJK UNIFIED IDEOGRAPH:'CB46:52038:&#x675A
CJK UNIFIED IDEOGRAPH:'CB47:52039:&#x674B
CJK UNIFIED IDEOGRAPH:'CB48:52040:&#x6BD0
CJK UNIFIED IDEOGRAPH:'CB49:52041:&#x6C19
CJK UNIFIED IDEOGRAPH:'CB4A:52042:&#x6C1A
CJK UNIFIED IDEOGRAPH:'CB4B:52043:&#x6C78
CJK UNIFIED IDEOGRAPH:'CB4C:52044:&#x6C67
CJK UNIFIED IDEOGRAPH:'CB4D:52045:&#x6C6B
CJK UNIFIED IDEOGRAPH:'CB4E:52046:&#x6C84
CJK UNIFIED IDEOGRAPH:'CB4F:52047:&#x6C8B
CJK UNIFIED IDEOGRAPH:'CB50:52048:&#x6C8F
CJK UNIFIED IDEOGRAPH:'CB51:52049:&#x6C71
CJK UNIFIED IDEOGRAPH:'CB52:52050:&#x6C6F
CJK UNIFIED IDEOGRAPH:'CB53:52051:&#x6C69
CJK UNIFIED IDEOGRAPH:'CB54:52052:&#x6C9A
CJK UNIFIED IDEOGRAPH:'CB55:52053:&#x6C6D
CJK UNIFIED IDEOGRAPH:'CB56:52054:&#x6C87
CJK UNIFIED IDEOGRAPH:'CB57:52055:&#x6C95
CJK UNIFIED IDEOGRAPH:'CB58:52056:&#x6C9C
CJK UNIFIED IDEOGRAPH:'CB59:52057:&#x6C66
CJK UNIFIED IDEOGRAPH:'CB5A:52058:&#x6C73
CJK UNIFIED IDEOGRAPH:'CB5B:52059:&#x6C65
CJK UNIFIED IDEOGRAPH:'CB5C:52060:&#x6C7B
CJK UNIFIED IDEOGRAPH:'CB5D:52061:&#x6C8E
CJK UNIFIED IDEOGRAPH:'CB5E:52062:&#x7074
CJK UNIFIED IDEOGRAPH:'CB5F:52063:&#x707A
CJK UNIFIED IDEOGRAPH:'CB60:52064:&#x7263
CJK UNIFIED IDEOGRAPH:'CB61:52065:&#x72BF
CJK UNIFIED IDEOGRAPH:'CB62:52066:&#x72BD
CJK UNIFIED IDEOGRAPH:'CB63:52067:&#x72C3
CJK UNIFIED IDEOGRAPH:'CB64:52068:&#x72C6
CJK UNIFIED IDEOGRAPH:'CB65:52069:&#x72C1
CJK UNIFIED IDEOGRAPH:'CB66:52070:&#x72BA
CJK UNIFIED IDEOGRAPH:'CB67:52071:&#x72C5
CJK UNIFIED IDEOGRAPH:'CB68:52072:&#x7395
CJK UNIFIED IDEOGRAPH:'CB69:52073:&#x7397
CJK UNIFIED IDEOGRAPH:'CB6A:52074:&#x7393
CJK UNIFIED IDEOGRAPH:'CB6B:52075:&#x7394
CJK UNIFIED IDEOGRAPH:'CB6C:52076:&#x7392
CJK UNIFIED IDEOGRAPH:'CB6D:52077:&#x753A
CJK UNIFIED IDEOGRAPH:'CB6E:52078:&#x7539
CJK UNIFIED IDEOGRAPH:'CB6F:52079:&#x7594
CJK UNIFIED IDEOGRAPH:'CB70:52080:&#x7595
CJK UNIFIED IDEOGRAPH:'CB71:52081:&#x7681
CJK UNIFIED IDEOGRAPH:'CB72:52082:&#x793D
CJK UNIFIED IDEOGRAPH:'CB73:52083:&#x8034
CJK UNIFIED IDEOGRAPH:'CB74:52084:&#x8095
CJK UNIFIED IDEOGRAPH:'CB75:52085:&#x8099
CJK UNIFIED IDEOGRAPH:'CB76:52086:&#x8090
CJK UNIFIED IDEOGRAPH:'CB77:52087:&#x8092
CJK UNIFIED IDEOGRAPH:'CB78:52088:&#x809C
CJK UNIFIED IDEOGRAPH:'CB79:52089:&#x8290
CJK UNIFIED IDEOGRAPH:'CB7A:52090:&#x828F
CJK UNIFIED IDEOGRAPH:'CB7B:52091:&#x8285
CJK UNIFIED IDEOGRAPH:'CB7C:52092:&#x828E
CJK UNIFIED IDEOGRAPH:'CB7D:52093:&#x8291
CJK UNIFIED IDEOGRAPH:'CB7E:52094:&#x8293
CJK UNIFIED IDEOGRAPH:'CBA1:52129:&#x828A
CJK UNIFIED IDEOGRAPH:'CBA2:52130:&#x8283
CJK UNIFIED IDEOGRAPH:'CBA3:52131:&#x8284
CJK UNIFIED IDEOGRAPH:'CBA4:52132:&#x8C78
CJK UNIFIED IDEOGRAPH:'CBA5:52133:&#x8FC9
CJK UNIFIED IDEOGRAPH:'CBA6:52134:&#x8FBF
CJK UNIFIED IDEOGRAPH:'CBA7:52135:&#x909F
CJK UNIFIED IDEOGRAPH:'CBA8:52136:&#x90A1
CJK UNIFIED IDEOGRAPH:'CBA9:52137:&#x90A5
CJK UNIFIED IDEOGRAPH:'CBAA:52138:&#x909E
CJK UNIFIED IDEOGRAPH:'CBAB:52139:&#x90A7
CJK UNIFIED IDEOGRAPH:'CBAC:52140:&#x90A0
CJK UNIFIED IDEOGRAPH:'CBAD:52141:&#x9630
CJK UNIFIED IDEOGRAPH:'CBAE:52142:&#x9628
CJK UNIFIED IDEOGRAPH:'CBAF:52143:&#x962F
CJK UNIFIED IDEOGRAPH:'CBB0:52144:&#x962D
CJK UNIFIED IDEOGRAPH:'CBB1:52145:&#x4E33
CJK UNIFIED IDEOGRAPH:'CBB2:52146:&#x4F98
CJK UNIFIED IDEOGRAPH:'CBB3:52147:&#x4F7C
CJK UNIFIED IDEOGRAPH:'CBB4:52148:&#x4F85
CJK UNIFIED IDEOGRAPH:'CBB5:52149:&#x4F7D
CJK UNIFIED IDEOGRAPH:'CBB6:52150:&#x4F80
CJK UNIFIED IDEOGRAPH:'CBB7:52151:&#x4F87
CJK UNIFIED IDEOGRAPH:'CBB8:52152:&#x4F76
CJK UNIFIED IDEOGRAPH:'CBB9:52153:&#x4F74
CJK UNIFIED IDEOGRAPH:'CBBA:52154:&#x4F89
CJK UNIFIED IDEOGRAPH:'CBBB:52155:&#x4F84
CJK UNIFIED IDEOGRAPH:'CBBC:52156:&#x4F77
CJK UNIFIED IDEOGRAPH:'CBBD:52157:&#x4F4C
CJK UNIFIED IDEOGRAPH:'CBBE:52158:&#x4F97
CJK UNIFIED IDEOGRAPH:'CBBF:52159:&#x4F6A
CJK UNIFIED IDEOGRAPH:'CBC0:52160:&#x4F9A
CJK UNIFIED IDEOGRAPH:'CBC1:52161:&#x4F79
CJK UNIFIED IDEOGRAPH:'CBC2:52162:&#x4F81
CJK UNIFIED IDEOGRAPH:'CBC3:52163:&#x4F78
CJK UNIFIED IDEOGRAPH:'CBC4:52164:&#x4F90
CJK UNIFIED IDEOGRAPH:'CBC5:52165:&#x4F9C
CJK UNIFIED IDEOGRAPH:'CBC6:52166:&#x4F94
CJK UNIFIED IDEOGRAPH:'CBC7:52167:&#x4F9E
CJK UNIFIED IDEOGRAPH:'CBC8:52168:&#x4F92
CJK UNIFIED IDEOGRAPH:'CBC9:52169:&#x4F82
CJK UNIFIED IDEOGRAPH:'CBCA:52170:&#x4F95
CJK UNIFIED IDEOGRAPH:'CBCB:52171:&#x4F6B
CJK UNIFIED IDEOGRAPH:'CBCC:52172:&#x4F6E
CJK UNIFIED IDEOGRAPH:'CBCD:52173:&#x519E
CJK UNIFIED IDEOGRAPH:'CBCE:52174:&#x51BC
CJK UNIFIED IDEOGRAPH:'CBCF:52175:&#x51BE
CJK UNIFIED IDEOGRAPH:'CBD0:52176:&#x5235
CJK UNIFIED IDEOGRAPH:'CBD1:52177:&#x5232
CJK UNIFIED IDEOGRAPH:'CBD2:52178:&#x5233
CJK UNIFIED IDEOGRAPH:'CBD3:52179:&#x5246
CJK UNIFIED IDEOGRAPH:'CBD4:52180:&#x5231
CJK UNIFIED IDEOGRAPH:'CBD5:52181:&#x52BC
CJK UNIFIED IDEOGRAPH:'CBD6:52182:&#x530A
CJK UNIFIED IDEOGRAPH:'CBD7:52183:&#x530B
CJK UNIFIED IDEOGRAPH:'CBD8:52184:&#x533C
CJK UNIFIED IDEOGRAPH:'CBD9:52185:&#x5392
CJK UNIFIED IDEOGRAPH:'CBDA:52186:&#x5394
CJK UNIFIED IDEOGRAPH:'CBDB:52187:&#x5487
CJK UNIFIED IDEOGRAPH:'CBDC:52188:&#x547F
CJK UNIFIED IDEOGRAPH:'CBDD:52189:&#x5481
CJK UNIFIED IDEOGRAPH:'CBDE:52190:&#x5491
CJK UNIFIED IDEOGRAPH:'CBDF:52191:&#x5482
CJK UNIFIED IDEOGRAPH:'CBE0:52192:&#x5488
CJK UNIFIED IDEOGRAPH:'CBE1:52193:&#x546B
CJK UNIFIED IDEOGRAPH:'CBE2:52194:&#x547A
CJK UNIFIED IDEOGRAPH:'CBE3:52195:&#x547E
CJK UNIFIED IDEOGRAPH:'CBE4:52196:&#x5465
CJK UNIFIED IDEOGRAPH:'CBE5:52197:&#x546C
CJK UNIFIED IDEOGRAPH:'CBE6:52198:&#x5474
CJK UNIFIED IDEOGRAPH:'CBE7:52199:&#x5466
CJK UNIFIED IDEOGRAPH:'CBE8:52200:&#x548D
CJK UNIFIED IDEOGRAPH:'CBE9:52201:&#x546F
CJK UNIFIED IDEOGRAPH:'CBEA:52202:&#x5461
CJK UNIFIED IDEOGRAPH:'CBEB:52203:&#x5460
CJK UNIFIED IDEOGRAPH:'CBEC:52204:&#x5498
CJK UNIFIED IDEOGRAPH:'CBED:52205:&#x5463
CJK UNIFIED IDEOGRAPH:'CBEE:52206:&#x5467
CJK UNIFIED IDEOGRAPH:'CBEF:52207:&#x5464
CJK UNIFIED IDEOGRAPH:'CBF0:52208:&#x56F7
CJK UNIFIED IDEOGRAPH:'CBF1:52209:&#x56F9
CJK UNIFIED IDEOGRAPH:'CBF2:52210:&#x576F
CJK UNIFIED IDEOGRAPH:'CBF3:52211:&#x5772
CJK UNIFIED IDEOGRAPH:'CBF4:52212:&#x576D
CJK UNIFIED IDEOGRAPH:'CBF5:52213:&#x576B
CJK UNIFIED IDEOGRAPH:'CBF6:52214:&#x5771
CJK UNIFIED IDEOGRAPH:'CBF7:52215:&#x5770
CJK UNIFIED IDEOGRAPH:'CBF8:52216:&#x5776
CJK UNIFIED IDEOGRAPH:'CBF9:52217:&#x5780
CJK UNIFIED IDEOGRAPH:'CBFA:52218:&#x5775
CJK UNIFIED IDEOGRAPH:'CBFB:52219:&#x577B
CJK UNIFIED IDEOGRAPH:'CBFC:52220:&#x5773
CJK UNIFIED IDEOGRAPH:'CBFD:52221:&#x5774
CJK UNIFIED IDEOGRAPH:'CBFE:52222:&#x5762
CJK UNIFIED IDEOGRAPH:'CC40:52288:&#x5768
CJK UNIFIED IDEOGRAPH:'CC41:52289:&#x577D
CJK UNIFIED IDEOGRAPH:'CC42:52290:&#x590C
CJK UNIFIED IDEOGRAPH:'CC43:52291:&#x5945
CJK UNIFIED IDEOGRAPH:'CC44:52292:&#x59B5
CJK UNIFIED IDEOGRAPH:'CC45:52293:&#x59BA
CJK UNIFIED IDEOGRAPH:'CC46:52294:&#x59CF
CJK UNIFIED IDEOGRAPH:'CC47:52295:&#x59CE
CJK UNIFIED IDEOGRAPH:'CC48:52296:&#x59B2
CJK UNIFIED IDEOGRAPH:'CC49:52297:&#x59CC
CJK UNIFIED IDEOGRAPH:'CC4A:52298:&#x59C1
CJK UNIFIED IDEOGRAPH:'CC4B:52299:&#x59B6
CJK UNIFIED IDEOGRAPH:'CC4C:52300:&#x59BC
CJK UNIFIED IDEOGRAPH:'CC4D:52301:&#x59C3
CJK UNIFIED IDEOGRAPH:'CC4E:52302:&#x59D6
CJK UNIFIED IDEOGRAPH:'CC4F:52303:&#x59B1
CJK UNIFIED IDEOGRAPH:'CC50:52304:&#x59BD
CJK UNIFIED IDEOGRAPH:'CC51:52305:&#x59C0
CJK UNIFIED IDEOGRAPH:'CC52:52306:&#x59C8
CJK UNIFIED IDEOGRAPH:'CC53:52307:&#x59B4
CJK UNIFIED IDEOGRAPH:'CC54:52308:&#x59C7
CJK UNIFIED IDEOGRAPH:'CC55:52309:&#x5B62
CJK UNIFIED IDEOGRAPH:'CC56:52310:&#x5B65
CJK UNIFIED IDEOGRAPH:'CC57:52311:&#x5B93
CJK UNIFIED IDEOGRAPH:'CC58:52312:&#x5B95
CJK UNIFIED IDEOGRAPH:'CC59:52313:&#x5C44
CJK UNIFIED IDEOGRAPH:'CC5A:52314:&#x5C47
CJK UNIFIED IDEOGRAPH:'CC5B:52315:&#x5CAE
CJK UNIFIED IDEOGRAPH:'CC5C:52316:&#x5CA4
CJK UNIFIED IDEOGRAPH:'CC5D:52317:&#x5CA0
CJK UNIFIED IDEOGRAPH:'CC5E:52318:&#x5CB5
CJK UNIFIED IDEOGRAPH:'CC5F:52319:&#x5CAF
CJK UNIFIED IDEOGRAPH:'CC60:52320:&#x5CA8
CJK UNIFIED IDEOGRAPH:'CC61:52321:&#x5CAC
CJK UNIFIED IDEOGRAPH:'CC62:52322:&#x5C9F
CJK UNIFIED IDEOGRAPH:'CC63:52323:&#x5CA3
CJK UNIFIED IDEOGRAPH:'CC64:52324:&#x5CAD
CJK UNIFIED IDEOGRAPH:'CC65:52325:&#x5CA2
CJK UNIFIED IDEOGRAPH:'CC66:52326:&#x5CAA
CJK UNIFIED IDEOGRAPH:'CC67:52327:&#x5CA7
CJK UNIFIED IDEOGRAPH:'CC68:52328:&#x5C9D
CJK UNIFIED IDEOGRAPH:'CC69:52329:&#x5CA5
CJK UNIFIED IDEOGRAPH:'CC6A:52330:&#x5CB6
CJK UNIFIED IDEOGRAPH:'CC6B:52331:&#x5CB0
CJK UNIFIED IDEOGRAPH:'CC6C:52332:&#x5CA6
CJK UNIFIED IDEOGRAPH:'CC6D:52333:&#x5E17
CJK UNIFIED IDEOGRAPH:'CC6E:52334:&#x5E14
CJK UNIFIED IDEOGRAPH:'CC6F:52335:&#x5E19
CJK UNIFIED IDEOGRAPH:'CC70:52336:&#x5F28
CJK UNIFIED IDEOGRAPH:'CC71:52337:&#x5F22
CJK UNIFIED IDEOGRAPH:'CC72:52338:&#x5F23
CJK UNIFIED IDEOGRAPH:'CC73:52339:&#x5F24
CJK UNIFIED IDEOGRAPH:'CC74:52340:&#x5F54
CJK UNIFIED IDEOGRAPH:'CC75:52341:&#x5F82
CJK UNIFIED IDEOGRAPH:'CC76:52342:&#x5F7E
CJK UNIFIED IDEOGRAPH:'CC77:52343:&#x5F7D
CJK UNIFIED IDEOGRAPH:'CC78:52344:&#x5FDE
CJK UNIFIED IDEOGRAPH:'CC79:52345:&#x5FE5
CJK UNIFIED IDEOGRAPH:'CC7A:52346:&#x602D
CJK UNIFIED IDEOGRAPH:'CC7B:52347:&#x6026
CJK UNIFIED IDEOGRAPH:'CC7C:52348:&#x6019
CJK UNIFIED IDEOGRAPH:'CC7D:52349:&#x6032
CJK UNIFIED IDEOGRAPH:'CC7E:52350:&#x600B
CJK UNIFIED IDEOGRAPH:'CCA1:52385:&#x6034
CJK UNIFIED IDEOGRAPH:'CCA2:52386:&#x600A
CJK UNIFIED IDEOGRAPH:'CCA3:52387:&#x6017
CJK UNIFIED IDEOGRAPH:'CCA4:52388:&#x6033
CJK UNIFIED IDEOGRAPH:'CCA5:52389:&#x601A
CJK UNIFIED IDEOGRAPH:'CCA6:52390:&#x601E
CJK UNIFIED IDEOGRAPH:'CCA7:52391:&#x602C
CJK UNIFIED IDEOGRAPH:'CCA8:52392:&#x6022
CJK UNIFIED IDEOGRAPH:'CCA9:52393:&#x600D
CJK UNIFIED IDEOGRAPH:'CCAA:52394:&#x6010
CJK UNIFIED IDEOGRAPH:'CCAB:52395:&#x602E
CJK UNIFIED IDEOGRAPH:'CCAC:52396:&#x6013
CJK UNIFIED IDEOGRAPH:'CCAD:52397:&#x6011
CJK UNIFIED IDEOGRAPH:'CCAE:52398:&#x600C
CJK UNIFIED IDEOGRAPH:'CCAF:52399:&#x6009
CJK UNIFIED IDEOGRAPH:'CCB0:52400:&#x601C
CJK UNIFIED IDEOGRAPH:'CCB1:52401:&#x6214
CJK UNIFIED IDEOGRAPH:'CCB2:52402:&#x623D
CJK UNIFIED IDEOGRAPH:'CCB3:52403:&#x62AD
CJK UNIFIED IDEOGRAPH:'CCB4:52404:&#x62B4
CJK UNIFIED IDEOGRAPH:'CCB5:52405:&#x62D1
CJK UNIFIED IDEOGRAPH:'CCB6:52406:&#x62BE
CJK UNIFIED IDEOGRAPH:'CCB7:52407:&#x62AA
CJK UNIFIED IDEOGRAPH:'CCB8:52408:&#x62B6
CJK UNIFIED IDEOGRAPH:'CCB9:52409:&#x62CA
CJK UNIFIED IDEOGRAPH:'CCBA:52410:&#x62AE
CJK UNIFIED IDEOGRAPH:'CCBB:52411:&#x62B3
CJK UNIFIED IDEOGRAPH:'CCBC:52412:&#x62AF
CJK UNIFIED IDEOGRAPH:'CCBD:52413:&#x62BB
CJK UNIFIED IDEOGRAPH:'CCBE:52414:&#x62A9
CJK UNIFIED IDEOGRAPH:'CCBF:52415:&#x62B0
CJK UNIFIED IDEOGRAPH:'CCC0:52416:&#x62B8
CJK UNIFIED IDEOGRAPH:'CCC1:52417:&#x653D
CJK UNIFIED IDEOGRAPH:'CCC2:52418:&#x65A8
CJK UNIFIED IDEOGRAPH:'CCC3:52419:&#x65BB
CJK UNIFIED IDEOGRAPH:'CCC4:52420:&#x6609
CJK UNIFIED IDEOGRAPH:'CCC5:52421:&#x65FC
CJK UNIFIED IDEOGRAPH:'CCC6:52422:&#x6604
CJK UNIFIED IDEOGRAPH:'CCC7:52423:&#x6612
CJK UNIFIED IDEOGRAPH:'CCC8:52424:&#x6608
CJK UNIFIED IDEOGRAPH:'CCC9:52425:&#x65FB
CJK UNIFIED IDEOGRAPH:'CCCA:52426:&#x6603
CJK UNIFIED IDEOGRAPH:'CCCB:52427:&#x660B
CJK UNIFIED IDEOGRAPH:'CCCC:52428:&#x660D
CJK UNIFIED IDEOGRAPH:'CCCD:52429:&#x6605
CJK UNIFIED IDEOGRAPH:'CCCE:52430:&#x65FD
CJK UNIFIED IDEOGRAPH:'CCCF:52431:&#x6611
CJK UNIFIED IDEOGRAPH:'CCD0:52432:&#x6610
CJK UNIFIED IDEOGRAPH:'CCD1:52433:&#x66F6
CJK UNIFIED IDEOGRAPH:'CCD2:52434:&#x670A
CJK UNIFIED IDEOGRAPH:'CCD3:52435:&#x6785
CJK UNIFIED IDEOGRAPH:'CCD4:52436:&#x676C
CJK UNIFIED IDEOGRAPH:'CCD5:52437:&#x678E
CJK UNIFIED IDEOGRAPH:'CCD6:52438:&#x6792
CJK UNIFIED IDEOGRAPH:'CCD7:52439:&#x6776
CJK UNIFIED IDEOGRAPH:'CCD8:52440:&#x677B
CJK UNIFIED IDEOGRAPH:'CCD9:52441:&#x6798
CJK UNIFIED IDEOGRAPH:'CCDA:52442:&#x6786
CJK UNIFIED IDEOGRAPH:'CCDB:52443:&#x6784
CJK UNIFIED IDEOGRAPH:'CCDC:52444:&#x6774
CJK UNIFIED IDEOGRAPH:'CCDD:52445:&#x678D
CJK UNIFIED IDEOGRAPH:'CCDE:52446:&#x678C
CJK UNIFIED IDEOGRAPH:'CCDF:52447:&#x677A
CJK UNIFIED IDEOGRAPH:'CCE0:52448:&#x679F
CJK UNIFIED IDEOGRAPH:'CCE1:52449:&#x6791
CJK UNIFIED IDEOGRAPH:'CCE2:52450:&#x6799
CJK UNIFIED IDEOGRAPH:'CCE3:52451:&#x6783
CJK UNIFIED IDEOGRAPH:'CCE4:52452:&#x677D
CJK UNIFIED IDEOGRAPH:'CCE5:52453:&#x6781
CJK UNIFIED IDEOGRAPH:'CCE6:52454:&#x6778
CJK UNIFIED IDEOGRAPH:'CCE7:52455:&#x6779
CJK UNIFIED IDEOGRAPH:'CCE8:52456:&#x6794
CJK UNIFIED IDEOGRAPH:'CCE9:52457:&#x6B25
CJK UNIFIED IDEOGRAPH:'CCEA:52458:&#x6B80
CJK UNIFIED IDEOGRAPH:'CCEB:52459:&#x6B7E
CJK UNIFIED IDEOGRAPH:'CCEC:52460:&#x6BDE
CJK UNIFIED IDEOGRAPH:'CCED:52461:&#x6C1D
CJK UNIFIED IDEOGRAPH:'CCEE:52462:&#x6C93
CJK UNIFIED IDEOGRAPH:'CCEF:52463:&#x6CEC
CJK UNIFIED IDEOGRAPH:'CCF0:52464:&#x6CEB
CJK UNIFIED IDEOGRAPH:'CCF1:52465:&#x6CEE
CJK UNIFIED IDEOGRAPH:'CCF2:52466:&#x6CD9
CJK UNIFIED IDEOGRAPH:'CCF3:52467:&#x6CB6
CJK UNIFIED IDEOGRAPH:'CCF4:52468:&#x6CD4
CJK UNIFIED IDEOGRAPH:'CCF5:52469:&#x6CAD
CJK UNIFIED IDEOGRAPH:'CCF6:52470:&#x6CE7
CJK UNIFIED IDEOGRAPH:'CCF7:52471:&#x6CB7
CJK UNIFIED IDEOGRAPH:'CCF8:52472:&#x6CD0
CJK UNIFIED IDEOGRAPH:'CCF9:52473:&#x6CC2
CJK UNIFIED IDEOGRAPH:'CCFA:52474:&#x6CBA
CJK UNIFIED IDEOGRAPH:'CCFB:52475:&#x6CC3
CJK UNIFIED IDEOGRAPH:'CCFC:52476:&#x6CC6
CJK UNIFIED IDEOGRAPH:'CCFD:52477:&#x6CED
CJK UNIFIED IDEOGRAPH:'CCFE:52478:&#x6CF2
CJK UNIFIED IDEOGRAPH:'CD40:52544:&#x6CD2
CJK UNIFIED IDEOGRAPH:'CD41:52545:&#x6CDD
CJK UNIFIED IDEOGRAPH:'CD42:52546:&#x6CB4
CJK UNIFIED IDEOGRAPH:'CD43:52547:&#x6C8A
CJK UNIFIED IDEOGRAPH:'CD44:52548:&#x6C9D
CJK UNIFIED IDEOGRAPH:'CD45:52549:&#x6C80
CJK UNIFIED IDEOGRAPH:'CD46:52550:&#x6CDE
CJK UNIFIED IDEOGRAPH:'CD47:52551:&#x6CC0
CJK UNIFIED IDEOGRAPH:'CD48:52552:&#x6D30
CJK UNIFIED IDEOGRAPH:'CD49:52553:&#x6CCD
CJK UNIFIED IDEOGRAPH:'CD4A:52554:&#x6CC7
CJK UNIFIED IDEOGRAPH:'CD4B:52555:&#x6CB0
CJK UNIFIED IDEOGRAPH:'CD4C:52556:&#x6CF9
CJK UNIFIED IDEOGRAPH:'CD4D:52557:&#x6CCF
CJK UNIFIED IDEOGRAPH:'CD4E:52558:&#x6CE9
CJK UNIFIED IDEOGRAPH:'CD4F:52559:&#x6CD1
CJK UNIFIED IDEOGRAPH:'CD50:52560:&#x7094
CJK UNIFIED IDEOGRAPH:'CD51:52561:&#x7098
CJK UNIFIED IDEOGRAPH:'CD52:52562:&#x7085
CJK UNIFIED IDEOGRAPH:'CD53:52563:&#x7093
CJK UNIFIED IDEOGRAPH:'CD54:52564:&#x7086
CJK UNIFIED IDEOGRAPH:'CD55:52565:&#x7084
CJK UNIFIED IDEOGRAPH:'CD56:52566:&#x7091
CJK UNIFIED IDEOGRAPH:'CD57:52567:&#x7096
CJK UNIFIED IDEOGRAPH:'CD58:52568:&#x7082
CJK UNIFIED IDEOGRAPH:'CD59:52569:&#x709A
CJK UNIFIED IDEOGRAPH:'CD5A:52570:&#x7083
CJK UNIFIED IDEOGRAPH:'CD5B:52571:&#x726A
CJK UNIFIED IDEOGRAPH:'CD5C:52572:&#x72D6
CJK UNIFIED IDEOGRAPH:'CD5D:52573:&#x72CB
CJK UNIFIED IDEOGRAPH:'CD5E:52574:&#x72D8
CJK UNIFIED IDEOGRAPH:'CD5F:52575:&#x72C9
CJK UNIFIED IDEOGRAPH:'CD60:52576:&#x72DC
CJK UNIFIED IDEOGRAPH:'CD61:52577:&#x72D2
CJK UNIFIED IDEOGRAPH:'CD62:52578:&#x72D4
CJK UNIFIED IDEOGRAPH:'CD63:52579:&#x72DA
CJK UNIFIED IDEOGRAPH:'CD64:52580:&#x72CC
CJK UNIFIED IDEOGRAPH:'CD65:52581:&#x72D1
CJK UNIFIED IDEOGRAPH:'CD66:52582:&#x73A4
CJK UNIFIED IDEOGRAPH:'CD67:52583:&#x73A1
CJK UNIFIED IDEOGRAPH:'CD68:52584:&#x73AD
CJK UNIFIED IDEOGRAPH:'CD69:52585:&#x73A6
CJK UNIFIED IDEOGRAPH:'CD6A:52586:&#x73A2
CJK UNIFIED IDEOGRAPH:'CD6B:52587:&#x73A0
CJK UNIFIED IDEOGRAPH:'CD6C:52588:&#x73AC
CJK UNIFIED IDEOGRAPH:'CD6D:52589:&#x739D
CJK UNIFIED IDEOGRAPH:'CD6E:52590:&#x74DD
CJK UNIFIED IDEOGRAPH:'CD6F:52591:&#x74E8
CJK UNIFIED IDEOGRAPH:'CD70:52592:&#x753F
CJK UNIFIED IDEOGRAPH:'CD71:52593:&#x7540
CJK UNIFIED IDEOGRAPH:'CD72:52594:&#x753E
CJK UNIFIED IDEOGRAPH:'CD73:52595:&#x758C
CJK UNIFIED IDEOGRAPH:'CD74:52596:&#x7598
CJK UNIFIED IDEOGRAPH:'CD75:52597:&#x76AF
CJK UNIFIED IDEOGRAPH:'CD76:52598:&#x76F3
CJK UNIFIED IDEOGRAPH:'CD77:52599:&#x76F1
CJK UNIFIED IDEOGRAPH:'CD78:52600:&#x76F0
CJK UNIFIED IDEOGRAPH:'CD79:52601:&#x76F5
CJK UNIFIED IDEOGRAPH:'CD7A:52602:&#x77F8
CJK UNIFIED IDEOGRAPH:'CD7B:52603:&#x77FC
CJK UNIFIED IDEOGRAPH:'CD7C:52604:&#x77F9
CJK UNIFIED IDEOGRAPH:'CD7D:52605:&#x77FB
CJK UNIFIED IDEOGRAPH:'CD7E:52606:&#x77FA
CJK UNIFIED IDEOGRAPH:'CDA1:52641:&#x77F7
CJK UNIFIED IDEOGRAPH:'CDA2:52642:&#x7942
CJK UNIFIED IDEOGRAPH:'CDA3:52643:&#x793F
CJK UNIFIED IDEOGRAPH:'CDA4:52644:&#x79C5
CJK UNIFIED IDEOGRAPH:'CDA5:52645:&#x7A78
CJK UNIFIED IDEOGRAPH:'CDA6:52646:&#x7A7B
CJK UNIFIED IDEOGRAPH:'CDA7:52647:&#x7AFB
CJK UNIFIED IDEOGRAPH:'CDA8:52648:&#x7C75
CJK UNIFIED IDEOGRAPH:'CDA9:52649:&#x7CFD
CJK UNIFIED IDEOGRAPH:'CDAA:52650:&#x8035
CJK UNIFIED IDEOGRAPH:'CDAB:52651:&#x808F
CJK UNIFIED IDEOGRAPH:'CDAC:52652:&#x80AE
CJK UNIFIED IDEOGRAPH:'CDAD:52653:&#x80A3
CJK UNIFIED IDEOGRAPH:'CDAE:52654:&#x80B8
CJK UNIFIED IDEOGRAPH:'CDAF:52655:&#x80B5
CJK UNIFIED IDEOGRAPH:'CDB0:52656:&#x80AD
CJK UNIFIED IDEOGRAPH:'CDB1:52657:&#x8220
CJK UNIFIED IDEOGRAPH:'CDB2:52658:&#x82A0
CJK UNIFIED IDEOGRAPH:'CDB3:52659:&#x82C0
CJK UNIFIED IDEOGRAPH:'CDB4:52660:&#x82AB
CJK UNIFIED IDEOGRAPH:'CDB5:52661:&#x829A
CJK UNIFIED IDEOGRAPH:'CDB6:52662:&#x8298
CJK UNIFIED IDEOGRAPH:'CDB7:52663:&#x829B
CJK UNIFIED IDEOGRAPH:'CDB8:52664:&#x82B5
CJK UNIFIED IDEOGRAPH:'CDB9:52665:&#x82A7
CJK UNIFIED IDEOGRAPH:'CDBA:52666:&#x82AE
CJK UNIFIED IDEOGRAPH:'CDBB:52667:&#x82BC
CJK UNIFIED IDEOGRAPH:'CDBC:52668:&#x829E
CJK UNIFIED IDEOGRAPH:'CDBD:52669:&#x82BA
CJK UNIFIED IDEOGRAPH:'CDBE:52670:&#x82B4
CJK UNIFIED IDEOGRAPH:'CDBF:52671:&#x82A8
CJK UNIFIED IDEOGRAPH:'CDC0:52672:&#x82A1
CJK UNIFIED IDEOGRAPH:'CDC1:52673:&#x82A9
CJK UNIFIED IDEOGRAPH:'CDC2:52674:&#x82C2
CJK UNIFIED IDEOGRAPH:'CDC3:52675:&#x82A4
CJK UNIFIED IDEOGRAPH:'CDC4:52676:&#x82C3
CJK UNIFIED IDEOGRAPH:'CDC5:52677:&#x82B6
CJK UNIFIED IDEOGRAPH:'CDC6:52678:&#x82A2
CJK UNIFIED IDEOGRAPH:'CDC7:52679:&#x8670
CJK UNIFIED IDEOGRAPH:'CDC8:52680:&#x866F
CJK UNIFIED IDEOGRAPH:'CDC9:52681:&#x866D
CJK UNIFIED IDEOGRAPH:'CDCA:52682:&#x866E
CJK UNIFIED IDEOGRAPH:'CDCB:52683:&#x8C56
CJK UNIFIED IDEOGRAPH:'CDCC:52684:&#x8FD2
CJK UNIFIED IDEOGRAPH:'CDCD:52685:&#x8FCB
CJK UNIFIED IDEOGRAPH:'CDCE:52686:&#x8FD3
CJK UNIFIED IDEOGRAPH:'CDCF:52687:&#x8FCD
CJK UNIFIED IDEOGRAPH:'CDD0:52688:&#x8FD6
CJK UNIFIED IDEOGRAPH:'CDD1:52689:&#x8FD5
CJK UNIFIED IDEOGRAPH:'CDD2:52690:&#x8FD7
CJK UNIFIED IDEOGRAPH:'CDD3:52691:&#x90B2
CJK UNIFIED IDEOGRAPH:'CDD4:52692:&#x90B4
CJK UNIFIED IDEOGRAPH:'CDD5:52693:&#x90AF
CJK UNIFIED IDEOGRAPH:'CDD6:52694:&#x90B3
CJK UNIFIED IDEOGRAPH:'CDD7:52695:&#x90B0
CJK UNIFIED IDEOGRAPH:'CDD8:52696:&#x9639
CJK UNIFIED IDEOGRAPH:'CDD9:52697:&#x963D
CJK UNIFIED IDEOGRAPH:'CDDA:52698:&#x963C
CJK UNIFIED IDEOGRAPH:'CDDB:52699:&#x963A
CJK UNIFIED IDEOGRAPH:'CDDC:52700:&#x9643
CJK UNIFIED IDEOGRAPH:'CDDD:52701:&#x4FCD
CJK UNIFIED IDEOGRAPH:'CDDE:52702:&#x4FC5
CJK UNIFIED IDEOGRAPH:'CDDF:52703:&#x4FD3
CJK UNIFIED IDEOGRAPH:'CDE0:52704:&#x4FB2
CJK UNIFIED IDEOGRAPH:'CDE1:52705:&#x4FC9
CJK UNIFIED IDEOGRAPH:'CDE2:52706:&#x4FCB
CJK UNIFIED IDEOGRAPH:'CDE3:52707:&#x4FC1
CJK UNIFIED IDEOGRAPH:'CDE4:52708:&#x4FD4
CJK UNIFIED IDEOGRAPH:'CDE5:52709:&#x4FDC
CJK UNIFIED IDEOGRAPH:'CDE6:52710:&#x4FD9
CJK UNIFIED IDEOGRAPH:'CDE7:52711:&#x4FBB
CJK UNIFIED IDEOGRAPH:'CDE8:52712:&#x4FB3
CJK UNIFIED IDEOGRAPH:'CDE9:52713:&#x4FDB
CJK UNIFIED IDEOGRAPH:'CDEA:52714:&#x4FC7
CJK UNIFIED IDEOGRAPH:'CDEB:52715:&#x4FD6
CJK UNIFIED IDEOGRAPH:'CDEC:52716:&#x4FBA
CJK UNIFIED IDEOGRAPH:'CDED:52717:&#x4FC0
CJK UNIFIED IDEOGRAPH:'CDEE:52718:&#x4FB9
CJK UNIFIED IDEOGRAPH:'CDEF:52719:&#x4FEC
CJK UNIFIED IDEOGRAPH:'CDF0:52720:&#x5244
CJK UNIFIED IDEOGRAPH:'CDF1:52721:&#x5249
CJK UNIFIED IDEOGRAPH:'CDF2:52722:&#x52C0
CJK UNIFIED IDEOGRAPH:'CDF3:52723:&#x52C2
CJK UNIFIED IDEOGRAPH:'CDF4:52724:&#x533D
CJK UNIFIED IDEOGRAPH:'CDF5:52725:&#x537C
CJK UNIFIED IDEOGRAPH:'CDF6:52726:&#x5397
CJK UNIFIED IDEOGRAPH:'CDF7:52727:&#x5396
CJK UNIFIED IDEOGRAPH:'CDF8:52728:&#x5399
CJK UNIFIED IDEOGRAPH:'CDF9:52729:&#x5398
CJK UNIFIED IDEOGRAPH:'CDFA:52730:&#x54BA
CJK UNIFIED IDEOGRAPH:'CDFB:52731:&#x54A1
CJK UNIFIED IDEOGRAPH:'CDFC:52732:&#x54AD
CJK UNIFIED IDEOGRAPH:'CDFD:52733:&#x54A5
CJK UNIFIED IDEOGRAPH:'CDFE:52734:&#x54CF
CJK UNIFIED IDEOGRAPH:'CE40:52800:&#x54C3
CJK UNIFIED IDEOGRAPH:'CE41:52801:&#x830D
CJK UNIFIED IDEOGRAPH:'CE42:52802:&#x54B7
CJK UNIFIED IDEOGRAPH:'CE43:52803:&#x54AE
CJK UNIFIED IDEOGRAPH:'CE44:52804:&#x54D6
CJK UNIFIED IDEOGRAPH:'CE45:52805:&#x54B6
CJK UNIFIED IDEOGRAPH:'CE46:52806:&#x54C5
CJK UNIFIED IDEOGRAPH:'CE47:52807:&#x54C6
CJK UNIFIED IDEOGRAPH:'CE48:52808:&#x54A0
CJK UNIFIED IDEOGRAPH:'CE49:52809:&#x5470
CJK UNIFIED IDEOGRAPH:'CE4A:52810:&#x54BC
CJK UNIFIED IDEOGRAPH:'CE4B:52811:&#x54A2
CJK UNIFIED IDEOGRAPH:'CE4C:52812:&#x54BE
CJK UNIFIED IDEOGRAPH:'CE4D:52813:&#x5472
CJK UNIFIED IDEOGRAPH:'CE4E:52814:&#x54DE
CJK UNIFIED IDEOGRAPH:'CE4F:52815:&#x54B0
CJK UNIFIED IDEOGRAPH:'CE50:52816:&#x57B5
CJK UNIFIED IDEOGRAPH:'CE51:52817:&#x579E
CJK UNIFIED IDEOGRAPH:'CE52:52818:&#x579F
CJK UNIFIED IDEOGRAPH:'CE53:52819:&#x57A4
CJK UNIFIED IDEOGRAPH:'CE54:52820:&#x578C
CJK UNIFIED IDEOGRAPH:'CE55:52821:&#x5797
CJK UNIFIED IDEOGRAPH:'CE56:52822:&#x579D
CJK UNIFIED IDEOGRAPH:'CE57:52823:&#x579B
CJK UNIFIED IDEOGRAPH:'CE58:52824:&#x5794
CJK UNIFIED IDEOGRAPH:'CE59:52825:&#x5798
CJK UNIFIED IDEOGRAPH:'CE5A:52826:&#x578F
CJK UNIFIED IDEOGRAPH:'CE5B:52827:&#x5799
CJK UNIFIED IDEOGRAPH:'CE5C:52828:&#x57A5
CJK UNIFIED IDEOGRAPH:'CE5D:52829:&#x579A
CJK UNIFIED IDEOGRAPH:'CE5E:52830:&#x5795
CJK UNIFIED IDEOGRAPH:'CE5F:52831:&#x58F4
CJK UNIFIED IDEOGRAPH:'CE60:52832:&#x590D
CJK UNIFIED IDEOGRAPH:'CE61:52833:&#x5953
CJK UNIFIED IDEOGRAPH:'CE62:52834:&#x59E1
CJK UNIFIED IDEOGRAPH:'CE63:52835:&#x59DE
CJK UNIFIED IDEOGRAPH:'CE64:52836:&#x59EE
CJK UNIFIED IDEOGRAPH:'CE65:52837:&#x5A00
CJK UNIFIED IDEOGRAPH:'CE66:52838:&#x59F1
CJK UNIFIED IDEOGRAPH:'CE67:52839:&#x59DD
CJK UNIFIED IDEOGRAPH:'CE68:52840:&#x59FA
CJK UNIFIED IDEOGRAPH:'CE69:52841:&#x59FD
CJK UNIFIED IDEOGRAPH:'CE6A:52842:&#x59FC
CJK UNIFIED IDEOGRAPH:'CE6B:52843:&#x59F6
CJK UNIFIED IDEOGRAPH:'CE6C:52844:&#x59E4
CJK UNIFIED IDEOGRAPH:'CE6D:52845:&#x59F2
CJK UNIFIED IDEOGRAPH:'CE6E:52846:&#x59F7
CJK UNIFIED IDEOGRAPH:'CE6F:52847:&#x59DB
CJK UNIFIED IDEOGRAPH:'CE70:52848:&#x59E9
CJK UNIFIED IDEOGRAPH:'CE71:52849:&#x59F3
CJK UNIFIED IDEOGRAPH:'CE72:52850:&#x59F5
CJK UNIFIED IDEOGRAPH:'CE73:52851:&#x59E0
CJK UNIFIED IDEOGRAPH:'CE74:52852:&#x59FE
CJK UNIFIED IDEOGRAPH:'CE75:52853:&#x59F4
CJK UNIFIED IDEOGRAPH:'CE76:52854:&#x59ED
CJK UNIFIED IDEOGRAPH:'CE77:52855:&#x5BA8
CJK UNIFIED IDEOGRAPH:'CE78:52856:&#x5C4C
CJK UNIFIED IDEOGRAPH:'CE79:52857:&#x5CD0
CJK UNIFIED IDEOGRAPH:'CE7A:52858:&#x5CD8
CJK UNIFIED IDEOGRAPH:'CE7B:52859:&#x5CCC
CJK UNIFIED IDEOGRAPH:'CE7C:52860:&#x5CD7
CJK UNIFIED IDEOGRAPH:'CE7D:52861:&#x5CCB
CJK UNIFIED IDEOGRAPH:'CE7E:52862:&#x5CDB
CJK UNIFIED IDEOGRAPH:'CEA1:52897:&#x5CDE
CJK UNIFIED IDEOGRAPH:'CEA2:52898:&#x5CDA
CJK UNIFIED IDEOGRAPH:'CEA3:52899:&#x5CC9
CJK UNIFIED IDEOGRAPH:'CEA4:52900:&#x5CC7
CJK UNIFIED IDEOGRAPH:'CEA5:52901:&#x5CCA
CJK UNIFIED IDEOGRAPH:'CEA6:52902:&#x5CD6
CJK UNIFIED IDEOGRAPH:'CEA7:52903:&#x5CD3
CJK UNIFIED IDEOGRAPH:'CEA8:52904:&#x5CD4
CJK UNIFIED IDEOGRAPH:'CEA9:52905:&#x5CCF
CJK UNIFIED IDEOGRAPH:'CEAA:52906:&#x5CC8
CJK UNIFIED IDEOGRAPH:'CEAB:52907:&#x5CC6
CJK UNIFIED IDEOGRAPH:'CEAC:52908:&#x5CCE
CJK UNIFIED IDEOGRAPH:'CEAD:52909:&#x5CDF
CJK UNIFIED IDEOGRAPH:'CEAE:52910:&#x5CF8
CJK UNIFIED IDEOGRAPH:'CEAF:52911:&#x5DF9
CJK UNIFIED IDEOGRAPH:'CEB0:52912:&#x5E21
CJK UNIFIED IDEOGRAPH:'CEB1:52913:&#x5E22
CJK UNIFIED IDEOGRAPH:'CEB2:52914:&#x5E23
CJK UNIFIED IDEOGRAPH:'CEB3:52915:&#x5E20
CJK UNIFIED IDEOGRAPH:'CEB4:52916:&#x5E24
CJK UNIFIED IDEOGRAPH:'CEB5:52917:&#x5EB0
CJK UNIFIED IDEOGRAPH:'CEB6:52918:&#x5EA4
CJK UNIFIED IDEOGRAPH:'CEB7:52919:&#x5EA2
CJK UNIFIED IDEOGRAPH:'CEB8:52920:&#x5E9B
CJK UNIFIED IDEOGRAPH:'CEB9:52921:&#x5EA3
CJK UNIFIED IDEOGRAPH:'CEBA:52922:&#x5EA5
CJK UNIFIED IDEOGRAPH:'CEBB:52923:&#x5F07
CJK UNIFIED IDEOGRAPH:'CEBC:52924:&#x5F2E
CJK UNIFIED IDEOGRAPH:'CEBD:52925:&#x5F56
CJK UNIFIED IDEOGRAPH:'CEBE:52926:&#x5F86
CJK UNIFIED IDEOGRAPH:'CEBF:52927:&#x6037
CJK UNIFIED IDEOGRAPH:'CEC0:52928:&#x6039
CJK UNIFIED IDEOGRAPH:'CEC1:52929:&#x6054
CJK UNIFIED IDEOGRAPH:'CEC2:52930:&#x6072
CJK UNIFIED IDEOGRAPH:'CEC3:52931:&#x605E
CJK UNIFIED IDEOGRAPH:'CEC4:52932:&#x6045
CJK UNIFIED IDEOGRAPH:'CEC5:52933:&#x6053
CJK UNIFIED IDEOGRAPH:'CEC6:52934:&#x6047
CJK UNIFIED IDEOGRAPH:'CEC7:52935:&#x6049
CJK UNIFIED IDEOGRAPH:'CEC8:52936:&#x605B
CJK UNIFIED IDEOGRAPH:'CEC9:52937:&#x604C
CJK UNIFIED IDEOGRAPH:'CECA:52938:&#x6040
CJK UNIFIED IDEOGRAPH:'CECB:52939:&#x6042
CJK UNIFIED IDEOGRAPH:'CECC:52940:&#x605F
CJK UNIFIED IDEOGRAPH:'CECD:52941:&#x6024
CJK UNIFIED IDEOGRAPH:'CECE:52942:&#x6044
CJK UNIFIED IDEOGRAPH:'CECF:52943:&#x6058
CJK UNIFIED IDEOGRAPH:'CED0:52944:&#x6066
CJK UNIFIED IDEOGRAPH:'CED1:52945:&#x606E
CJK UNIFIED IDEOGRAPH:'CED2:52946:&#x6242
CJK UNIFIED IDEOGRAPH:'CED3:52947:&#x6243
CJK UNIFIED IDEOGRAPH:'CED4:52948:&#x62CF
CJK UNIFIED IDEOGRAPH:'CED5:52949:&#x630D
CJK UNIFIED IDEOGRAPH:'CED6:52950:&#x630B
CJK UNIFIED IDEOGRAPH:'CED7:52951:&#x62F5
CJK UNIFIED IDEOGRAPH:'CED8:52952:&#x630E
CJK UNIFIED IDEOGRAPH:'CED9:52953:&#x6303
CJK UNIFIED IDEOGRAPH:'CEDA:52954:&#x62EB
CJK UNIFIED IDEOGRAPH:'CEDB:52955:&#x62F9
CJK UNIFIED IDEOGRAPH:'CEDC:52956:&#x630F
CJK UNIFIED IDEOGRAPH:'CEDD:52957:&#x630C
CJK UNIFIED IDEOGRAPH:'CEDE:52958:&#x62F8
CJK UNIFIED IDEOGRAPH:'CEDF:52959:&#x62F6
CJK UNIFIED IDEOGRAPH:'CEE0:52960:&#x6300
CJK UNIFIED IDEOGRAPH:'CEE1:52961:&#x6313
CJK UNIFIED IDEOGRAPH:'CEE2:52962:&#x6314
CJK UNIFIED IDEOGRAPH:'CEE3:52963:&#x62FA
CJK UNIFIED IDEOGRAPH:'CEE4:52964:&#x6315
CJK UNIFIED IDEOGRAPH:'CEE5:52965:&#x62FB
CJK UNIFIED IDEOGRAPH:'CEE6:52966:&#x62F0
CJK UNIFIED IDEOGRAPH:'CEE7:52967:&#x6541
CJK UNIFIED IDEOGRAPH:'CEE8:52968:&#x6543
CJK UNIFIED IDEOGRAPH:'CEE9:52969:&#x65AA
CJK UNIFIED IDEOGRAPH:'CEEA:52970:&#x65BF
CJK UNIFIED IDEOGRAPH:'CEEB:52971:&#x6636
CJK UNIFIED IDEOGRAPH:'CEEC:52972:&#x6621
CJK UNIFIED IDEOGRAPH:'CEED:52973:&#x6632
CJK UNIFIED IDEOGRAPH:'CEEE:52974:&#x6635
CJK UNIFIED IDEOGRAPH:'CEEF:52975:&#x661C
CJK UNIFIED IDEOGRAPH:'CEF0:52976:&#x6626
CJK UNIFIED IDEOGRAPH:'CEF1:52977:&#x6622
CJK UNIFIED IDEOGRAPH:'CEF2:52978:&#x6633
CJK UNIFIED IDEOGRAPH:'CEF3:52979:&#x662B
CJK UNIFIED IDEOGRAPH:'CEF4:52980:&#x663A
CJK UNIFIED IDEOGRAPH:'CEF5:52981:&#x661D
CJK UNIFIED IDEOGRAPH:'CEF6:52982:&#x6634
CJK UNIFIED IDEOGRAPH:'CEF7:52983:&#x6639
CJK UNIFIED IDEOGRAPH:'CEF8:52984:&#x662E
CJK UNIFIED IDEOGRAPH:'CEF9:52985:&#x670F
CJK UNIFIED IDEOGRAPH:'CEFA:52986:&#x6710
CJK UNIFIED IDEOGRAPH:'CEFB:52987:&#x67C1
CJK UNIFIED IDEOGRAPH:'CEFC:52988:&#x67F2
CJK UNIFIED IDEOGRAPH:'CEFD:52989:&#x67C8
CJK UNIFIED IDEOGRAPH:'CEFE:52990:&#x67BA
CJK UNIFIED IDEOGRAPH:'CF40:53056:&#x67DC
CJK UNIFIED IDEOGRAPH:'CF41:53057:&#x67BB
CJK UNIFIED IDEOGRAPH:'CF42:53058:&#x67F8
CJK UNIFIED IDEOGRAPH:'CF43:53059:&#x67D8
CJK UNIFIED IDEOGRAPH:'CF44:53060:&#x67C0
CJK UNIFIED IDEOGRAPH:'CF45:53061:&#x67B7
CJK UNIFIED IDEOGRAPH:'CF46:53062:&#x67C5
CJK UNIFIED IDEOGRAPH:'CF47:53063:&#x67EB
CJK UNIFIED IDEOGRAPH:'CF48:53064:&#x67E4
CJK UNIFIED IDEOGRAPH:'CF49:53065:&#x67DF
CJK UNIFIED IDEOGRAPH:'CF4A:53066:&#x67B5
CJK UNIFIED IDEOGRAPH:'CF4B:53067:&#x67CD
CJK UNIFIED IDEOGRAPH:'CF4C:53068:&#x67B3
CJK UNIFIED IDEOGRAPH:'CF4D:53069:&#x67F7
CJK UNIFIED IDEOGRAPH:'CF4E:53070:&#x67F6
CJK UNIFIED IDEOGRAPH:'CF4F:53071:&#x67EE
CJK UNIFIED IDEOGRAPH:'CF50:53072:&#x67E3
CJK UNIFIED IDEOGRAPH:'CF51:53073:&#x67C2
CJK UNIFIED IDEOGRAPH:'CF52:53074:&#x67B9
CJK UNIFIED IDEOGRAPH:'CF53:53075:&#x67CE
CJK UNIFIED IDEOGRAPH:'CF54:53076:&#x67E7
CJK UNIFIED IDEOGRAPH:'CF55:53077:&#x67F0
CJK UNIFIED IDEOGRAPH:'CF56:53078:&#x67B2
CJK UNIFIED IDEOGRAPH:'CF57:53079:&#x67FC
CJK UNIFIED IDEOGRAPH:'CF58:53080:&#x67C6
CJK UNIFIED IDEOGRAPH:'CF59:53081:&#x67ED
CJK UNIFIED IDEOGRAPH:'CF5A:53082:&#x67CC
CJK UNIFIED IDEOGRAPH:'CF5B:53083:&#x67AE
CJK UNIFIED IDEOGRAPH:'CF5C:53084:&#x67E6
CJK UNIFIED IDEOGRAPH:'CF5D:53085:&#x67DB
CJK UNIFIED IDEOGRAPH:'CF5E:53086:&#x67FA
CJK UNIFIED IDEOGRAPH:'CF5F:53087:&#x67C9
CJK UNIFIED IDEOGRAPH:'CF60:53088:&#x67CA
CJK UNIFIED IDEOGRAPH:'CF61:53089:&#x67C3
CJK UNIFIED IDEOGRAPH:'CF62:53090:&#x67EA
CJK UNIFIED IDEOGRAPH:'CF63:53091:&#x67CB
CJK UNIFIED IDEOGRAPH:'CF64:53092:&#x6B28
CJK UNIFIED IDEOGRAPH:'CF65:53093:&#x6B82
CJK UNIFIED IDEOGRAPH:'CF66:53094:&#x6B84
CJK UNIFIED IDEOGRAPH:'CF67:53095:&#x6BB6
CJK UNIFIED IDEOGRAPH:'CF68:53096:&#x6BD6
CJK UNIFIED IDEOGRAPH:'CF69:53097:&#x6BD8
CJK UNIFIED IDEOGRAPH:'CF6A:53098:&#x6BE0
CJK UNIFIED IDEOGRAPH:'CF6B:53099:&#x6C20
CJK UNIFIED IDEOGRAPH:'CF6C:53100:&#x6C21
CJK UNIFIED IDEOGRAPH:'CF6D:53101:&#x6D28
CJK UNIFIED IDEOGRAPH:'CF6E:53102:&#x6D34
CJK UNIFIED IDEOGRAPH:'CF6F:53103:&#x6D2D
CJK UNIFIED IDEOGRAPH:'CF70:53104:&#x6D1F
CJK UNIFIED IDEOGRAPH:'CF71:53105:&#x6D3C
CJK UNIFIED IDEOGRAPH:'CF72:53106:&#x6D3F
CJK UNIFIED IDEOGRAPH:'CF73:53107:&#x6D12
CJK UNIFIED IDEOGRAPH:'CF74:53108:&#x6D0A
CJK UNIFIED IDEOGRAPH:'CF75:53109:&#x6CDA
CJK UNIFIED IDEOGRAPH:'CF76:53110:&#x6D33
CJK UNIFIED IDEOGRAPH:'CF77:53111:&#x6D04
CJK UNIFIED IDEOGRAPH:'CF78:53112:&#x6D19
CJK UNIFIED IDEOGRAPH:'CF79:53113:&#x6D3A
CJK UNIFIED IDEOGRAPH:'CF7A:53114:&#x6D1A
CJK UNIFIED IDEOGRAPH:'CF7B:53115:&#x6D11
CJK UNIFIED IDEOGRAPH:'CF7C:53116:&#x6D00
CJK UNIFIED IDEOGRAPH:'CF7D:53117:&#x6D1D
CJK UNIFIED IDEOGRAPH:'CF7E:53118:&#x6D42
CJK UNIFIED IDEOGRAPH:'CFA1:53153:&#x6D01
CJK UNIFIED IDEOGRAPH:'CFA2:53154:&#x6D18
CJK UNIFIED IDEOGRAPH:'CFA3:53155:&#x6D37
CJK UNIFIED IDEOGRAPH:'CFA4:53156:&#x6D03
CJK UNIFIED IDEOGRAPH:'CFA5:53157:&#x6D0F
CJK UNIFIED IDEOGRAPH:'CFA6:53158:&#x6D40
CJK UNIFIED IDEOGRAPH:'CFA7:53159:&#x6D07
CJK UNIFIED IDEOGRAPH:'CFA8:53160:&#x6D20
CJK UNIFIED IDEOGRAPH:'CFA9:53161:&#x6D2C
CJK UNIFIED IDEOGRAPH:'CFAA:53162:&#x6D08
CJK UNIFIED IDEOGRAPH:'CFAB:53163:&#x6D22
CJK UNIFIED IDEOGRAPH:'CFAC:53164:&#x6D09
CJK UNIFIED IDEOGRAPH:'CFAD:53165:&#x6D10
CJK UNIFIED IDEOGRAPH:'CFAE:53166:&#x70B7
CJK UNIFIED IDEOGRAPH:'CFAF:53167:&#x709F
CJK UNIFIED IDEOGRAPH:'CFB0:53168:&#x70BE
CJK UNIFIED IDEOGRAPH:'CFB1:53169:&#x70B1
CJK UNIFIED IDEOGRAPH:'CFB2:53170:&#x70B0
CJK UNIFIED IDEOGRAPH:'CFB3:53171:&#x70A1
CJK UNIFIED IDEOGRAPH:'CFB4:53172:&#x70B4
CJK UNIFIED IDEOGRAPH:'CFB5:53173:&#x70B5
CJK UNIFIED IDEOGRAPH:'CFB6:53174:&#x70A9
CJK UNIFIED IDEOGRAPH:'CFB7:53175:&#x7241
CJK UNIFIED IDEOGRAPH:'CFB8:53176:&#x7249
CJK UNIFIED IDEOGRAPH:'CFB9:53177:&#x724A
CJK UNIFIED IDEOGRAPH:'CFBA:53178:&#x726C
CJK UNIFIED IDEOGRAPH:'CFBB:53179:&#x7270
CJK UNIFIED IDEOGRAPH:'CFBC:53180:&#x7273
CJK UNIFIED IDEOGRAPH:'CFBD:53181:&#x726E
CJK UNIFIED IDEOGRAPH:'CFBE:53182:&#x72CA
CJK UNIFIED IDEOGRAPH:'CFBF:53183:&#x72E4
CJK UNIFIED IDEOGRAPH:'CFC0:53184:&#x72E8
CJK UNIFIED IDEOGRAPH:'CFC1:53185:&#x72EB
CJK UNIFIED IDEOGRAPH:'CFC2:53186:&#x72DF
CJK UNIFIED IDEOGRAPH:'CFC3:53187:&#x72EA
CJK UNIFIED IDEOGRAPH:'CFC4:53188:&#x72E6
CJK UNIFIED IDEOGRAPH:'CFC5:53189:&#x72E3
CJK UNIFIED IDEOGRAPH:'CFC6:53190:&#x7385
CJK UNIFIED IDEOGRAPH:'CFC7:53191:&#x73CC
CJK UNIFIED IDEOGRAPH:'CFC8:53192:&#x73C2
CJK UNIFIED IDEOGRAPH:'CFC9:53193:&#x73C8
CJK UNIFIED IDEOGRAPH:'CFCA:53194:&#x73C5
CJK UNIFIED IDEOGRAPH:'CFCB:53195:&#x73B9
CJK UNIFIED IDEOGRAPH:'CFCC:53196:&#x73B6
CJK UNIFIED IDEOGRAPH:'CFCD:53197:&#x73B5
CJK UNIFIED IDEOGRAPH:'CFCE:53198:&#x73B4
CJK UNIFIED IDEOGRAPH:'CFCF:53199:&#x73EB
CJK UNIFIED IDEOGRAPH:'CFD0:53200:&#x73BF
CJK UNIFIED IDEOGRAPH:'CFD1:53201:&#x73C7
CJK UNIFIED IDEOGRAPH:'CFD2:53202:&#x73BE
CJK UNIFIED IDEOGRAPH:'CFD3:53203:&#x73C3
CJK UNIFIED IDEOGRAPH:'CFD4:53204:&#x73C6
CJK UNIFIED IDEOGRAPH:'CFD5:53205:&#x73B8
CJK UNIFIED IDEOGRAPH:'CFD6:53206:&#x73CB
CJK UNIFIED IDEOGRAPH:'CFD7:53207:&#x74EC
CJK UNIFIED IDEOGRAPH:'CFD8:53208:&#x74EE
CJK UNIFIED IDEOGRAPH:'CFD9:53209:&#x752E
CJK UNIFIED IDEOGRAPH:'CFDA:53210:&#x7547
CJK UNIFIED IDEOGRAPH:'CFDB:53211:&#x7548
CJK UNIFIED IDEOGRAPH:'CFDC:53212:&#x75A7
CJK UNIFIED IDEOGRAPH:'CFDD:53213:&#x75AA
CJK UNIFIED IDEOGRAPH:'CFDE:53214:&#x7679
CJK UNIFIED IDEOGRAPH:'CFDF:53215:&#x76C4
CJK UNIFIED IDEOGRAPH:'CFE0:53216:&#x7708
CJK UNIFIED IDEOGRAPH:'CFE1:53217:&#x7703
CJK UNIFIED IDEOGRAPH:'CFE2:53218:&#x7704
CJK UNIFIED IDEOGRAPH:'CFE3:53219:&#x7705
CJK UNIFIED IDEOGRAPH:'CFE4:53220:&#x770A
CJK UNIFIED IDEOGRAPH:'CFE5:53221:&#x76F7
CJK UNIFIED IDEOGRAPH:'CFE6:53222:&#x76FB
CJK UNIFIED IDEOGRAPH:'CFE7:53223:&#x76FA
CJK UNIFIED IDEOGRAPH:'CFE8:53224:&#x77E7
CJK UNIFIED IDEOGRAPH:'CFE9:53225:&#x77E8
CJK UNIFIED IDEOGRAPH:'CFEA:53226:&#x7806
CJK UNIFIED IDEOGRAPH:'CFEB:53227:&#x7811
CJK UNIFIED IDEOGRAPH:'CFEC:53228:&#x7812
CJK UNIFIED IDEOGRAPH:'CFED:53229:&#x7805
CJK UNIFIED IDEOGRAPH:'CFEE:53230:&#x7810
CJK UNIFIED IDEOGRAPH:'CFEF:53231:&#x780F
CJK UNIFIED IDEOGRAPH:'CFF0:53232:&#x780E
CJK UNIFIED IDEOGRAPH:'CFF1:53233:&#x7809
CJK UNIFIED IDEOGRAPH:'CFF2:53234:&#x7803
CJK UNIFIED IDEOGRAPH:'CFF3:53235:&#x7813
CJK UNIFIED IDEOGRAPH:'CFF4:53236:&#x794A
CJK UNIFIED IDEOGRAPH:'CFF5:53237:&#x794C
CJK UNIFIED IDEOGRAPH:'CFF6:53238:&#x794B
CJK UNIFIED IDEOGRAPH:'CFF7:53239:&#x7945
CJK UNIFIED IDEOGRAPH:'CFF8:53240:&#x7944
CJK UNIFIED IDEOGRAPH:'CFF9:53241:&#x79D5
CJK UNIFIED IDEOGRAPH:'CFFA:53242:&#x79CD
CJK UNIFIED IDEOGRAPH:'CFFB:53243:&#x79CF
CJK UNIFIED IDEOGRAPH:'CFFC:53244:&#x79D6
CJK UNIFIED IDEOGRAPH:'CFFD:53245:&#x79CE
CJK UNIFIED IDEOGRAPH:'CFFE:53246:&#x7A80
CJK UNIFIED IDEOGRAPH:'D040:53312:&#x7A7E
CJK UNIFIED IDEOGRAPH:'D041:53313:&#x7AD1
CJK UNIFIED IDEOGRAPH:'D042:53314:&#x7B00
CJK UNIFIED IDEOGRAPH:'D043:53315:&#x7B01
CJK UNIFIED IDEOGRAPH:'D044:53316:&#x7C7A
CJK UNIFIED IDEOGRAPH:'D045:53317:&#x7C78
CJK UNIFIED IDEOGRAPH:'D046:53318:&#x7C79
CJK UNIFIED IDEOGRAPH:'D047:53319:&#x7C7F
CJK UNIFIED IDEOGRAPH:'D048:53320:&#x7C80
CJK UNIFIED IDEOGRAPH:'D049:53321:&#x7C81
CJK UNIFIED IDEOGRAPH:'D04A:53322:&#x7D03
CJK UNIFIED IDEOGRAPH:'D04B:53323:&#x7D08
CJK UNIFIED IDEOGRAPH:'D04C:53324:&#x7D01
CJK UNIFIED IDEOGRAPH:'D04D:53325:&#x7F58
CJK UNIFIED IDEOGRAPH:'D04E:53326:&#x7F91
CJK UNIFIED IDEOGRAPH:'D04F:53327:&#x7F8D
CJK UNIFIED IDEOGRAPH:'D050:53328:&#x7FBE
CJK UNIFIED IDEOGRAPH:'D051:53329:&#x8007
CJK UNIFIED IDEOGRAPH:'D052:53330:&#x800E
CJK UNIFIED IDEOGRAPH:'D053:53331:&#x800F
CJK UNIFIED IDEOGRAPH:'D054:53332:&#x8014
CJK UNIFIED IDEOGRAPH:'D055:53333:&#x8037
CJK UNIFIED IDEOGRAPH:'D056:53334:&#x80D8
CJK UNIFIED IDEOGRAPH:'D057:53335:&#x80C7
CJK UNIFIED IDEOGRAPH:'D058:53336:&#x80E0
CJK UNIFIED IDEOGRAPH:'D059:53337:&#x80D1
CJK UNIFIED IDEOGRAPH:'D05A:53338:&#x80C8
CJK UNIFIED IDEOGRAPH:'D05B:53339:&#x80C2
CJK UNIFIED IDEOGRAPH:'D05C:53340:&#x80D0
CJK UNIFIED IDEOGRAPH:'D05D:53341:&#x80C5
CJK UNIFIED IDEOGRAPH:'D05E:53342:&#x80E3
CJK UNIFIED IDEOGRAPH:'D05F:53343:&#x80D9
CJK UNIFIED IDEOGRAPH:'D060:53344:&#x80DC
CJK UNIFIED IDEOGRAPH:'D061:53345:&#x80CA
CJK UNIFIED IDEOGRAPH:'D062:53346:&#x80D5
CJK UNIFIED IDEOGRAPH:'D063:53347:&#x80C9
CJK UNIFIED IDEOGRAPH:'D064:53348:&#x80CF
CJK UNIFIED IDEOGRAPH:'D065:53349:&#x80D7
CJK UNIFIED IDEOGRAPH:'D066:53350:&#x80E6
CJK UNIFIED IDEOGRAPH:'D067:53351:&#x80CD
CJK UNIFIED IDEOGRAPH:'D068:53352:&#x81FF
CJK UNIFIED IDEOGRAPH:'D069:53353:&#x8221
CJK UNIFIED IDEOGRAPH:'D06A:53354:&#x8294
CJK UNIFIED IDEOGRAPH:'D06B:53355:&#x82D9
CJK UNIFIED IDEOGRAPH:'D06C:53356:&#x82FE
CJK UNIFIED IDEOGRAPH:'D06D:53357:&#x82F9
CJK UNIFIED IDEOGRAPH:'D06E:53358:&#x8307
CJK UNIFIED IDEOGRAPH:'D06F:53359:&#x82E8
CJK UNIFIED IDEOGRAPH:'D070:53360:&#x8300
CJK UNIFIED IDEOGRAPH:'D071:53361:&#x82D5
CJK UNIFIED IDEOGRAPH:'D072:53362:&#x833A
CJK UNIFIED IDEOGRAPH:'D073:53363:&#x82EB
CJK UNIFIED IDEOGRAPH:'D074:53364:&#x82D6
CJK UNIFIED IDEOGRAPH:'D075:53365:&#x82F4
CJK UNIFIED IDEOGRAPH:'D076:53366:&#x82EC
CJK UNIFIED IDEOGRAPH:'D077:53367:&#x82E1
CJK UNIFIED IDEOGRAPH:'D078:53368:&#x82F2
CJK UNIFIED IDEOGRAPH:'D079:53369:&#x82F5
CJK UNIFIED IDEOGRAPH:'D07A:53370:&#x830C
CJK UNIFIED IDEOGRAPH:'D07B:53371:&#x82FB
CJK UNIFIED IDEOGRAPH:'D07C:53372:&#x82F6
CJK UNIFIED IDEOGRAPH:'D07D:53373:&#x82F0
CJK UNIFIED IDEOGRAPH:'D07E:53374:&#x82EA
CJK UNIFIED IDEOGRAPH:'D0A1:53409:&#x82E4
CJK UNIFIED IDEOGRAPH:'D0A2:53410:&#x82E0
CJK UNIFIED IDEOGRAPH:'D0A3:53411:&#x82FA
CJK UNIFIED IDEOGRAPH:'D0A4:53412:&#x82F3
CJK UNIFIED IDEOGRAPH:'D0A5:53413:&#x82ED
CJK UNIFIED IDEOGRAPH:'D0A6:53414:&#x8677
CJK UNIFIED IDEOGRAPH:'D0A7:53415:&#x8674
CJK UNIFIED IDEOGRAPH:'D0A8:53416:&#x867C
CJK UNIFIED IDEOGRAPH:'D0A9:53417:&#x8673
CJK UNIFIED IDEOGRAPH:'D0AA:53418:&#x8841
CJK UNIFIED IDEOGRAPH:'D0AB:53419:&#x884E
CJK UNIFIED IDEOGRAPH:'D0AC:53420:&#x8867
CJK UNIFIED IDEOGRAPH:'D0AD:53421:&#x886A
CJK UNIFIED IDEOGRAPH:'D0AE:53422:&#x8869
CJK UNIFIED IDEOGRAPH:'D0AF:53423:&#x89D3
CJK UNIFIED IDEOGRAPH:'D0B0:53424:&#x8A04
CJK UNIFIED IDEOGRAPH:'D0B1:53425:&#x8A07
CJK UNIFIED IDEOGRAPH:'D0B2:53426:&#x8D72
CJK UNIFIED IDEOGRAPH:'D0B3:53427:&#x8FE3
CJK UNIFIED IDEOGRAPH:'D0B4:53428:&#x8FE1
CJK UNIFIED IDEOGRAPH:'D0B5:53429:&#x8FEE
CJK UNIFIED IDEOGRAPH:'D0B6:53430:&#x8FE0
CJK UNIFIED IDEOGRAPH:'D0B7:53431:&#x90F1
CJK UNIFIED IDEOGRAPH:'D0B8:53432:&#x90BD
CJK UNIFIED IDEOGRAPH:'D0B9:53433:&#x90BF
CJK UNIFIED IDEOGRAPH:'D0BA:53434:&#x90D5
CJK UNIFIED IDEOGRAPH:'D0BB:53435:&#x90C5
CJK UNIFIED IDEOGRAPH:'D0BC:53436:&#x90BE
CJK UNIFIED IDEOGRAPH:'D0BD:53437:&#x90C7
CJK UNIFIED IDEOGRAPH:'D0BE:53438:&#x90CB
CJK UNIFIED IDEOGRAPH:'D0BF:53439:&#x90C8
CJK UNIFIED IDEOGRAPH:'D0C0:53440:&#x91D4
CJK UNIFIED IDEOGRAPH:'D0C1:53441:&#x91D3
CJK UNIFIED IDEOGRAPH:'D0C2:53442:&#x9654
CJK UNIFIED IDEOGRAPH:'D0C3:53443:&#x964F
CJK UNIFIED IDEOGRAPH:'D0C4:53444:&#x9651
CJK UNIFIED IDEOGRAPH:'D0C5:53445:&#x9653
CJK UNIFIED IDEOGRAPH:'D0C6:53446:&#x964A
CJK UNIFIED IDEOGRAPH:'D0C7:53447:&#x964E
CJK UNIFIED IDEOGRAPH:'D0C8:53448:&#x501E
CJK UNIFIED IDEOGRAPH:'D0C9:53449:&#x5005
CJK UNIFIED IDEOGRAPH:'D0CA:53450:&#x5007
CJK UNIFIED IDEOGRAPH:'D0CB:53451:&#x5013
CJK UNIFIED IDEOGRAPH:'D0CC:53452:&#x5022
CJK UNIFIED IDEOGRAPH:'D0CD:53453:&#x5030
CJK UNIFIED IDEOGRAPH:'D0CE:53454:&#x501B
CJK UNIFIED IDEOGRAPH:'D0CF:53455:&#x4FF5
CJK UNIFIED IDEOGRAPH:'D0D0:53456:&#x4FF4
CJK UNIFIED IDEOGRAPH:'D0D1:53457:&#x5033
CJK UNIFIED IDEOGRAPH:'D0D2:53458:&#x5037
CJK UNIFIED IDEOGRAPH:'D0D3:53459:&#x502C
CJK UNIFIED IDEOGRAPH:'D0D4:53460:&#x4FF6
CJK UNIFIED IDEOGRAPH:'D0D5:53461:&#x4FF7
CJK UNIFIED IDEOGRAPH:'D0D6:53462:&#x5017
CJK UNIFIED IDEOGRAPH:'D0D7:53463:&#x501C
CJK UNIFIED IDEOGRAPH:'D0D8:53464:&#x5020
CJK UNIFIED IDEOGRAPH:'D0D9:53465:&#x5027
CJK UNIFIED IDEOGRAPH:'D0DA:53466:&#x5035
CJK UNIFIED IDEOGRAPH:'D0DB:53467:&#x502F
CJK UNIFIED IDEOGRAPH:'D0DC:53468:&#x5031
CJK UNIFIED IDEOGRAPH:'D0DD:53469:&#x500E
CJK UNIFIED IDEOGRAPH:'D0DE:53470:&#x515A
CJK UNIFIED IDEOGRAPH:'D0DF:53471:&#x5194
CJK UNIFIED IDEOGRAPH:'D0E0:53472:&#x5193
CJK UNIFIED IDEOGRAPH:'D0E1:53473:&#x51CA
CJK UNIFIED IDEOGRAPH:'D0E2:53474:&#x51C4
CJK UNIFIED IDEOGRAPH:'D0E3:53475:&#x51C5
CJK UNIFIED IDEOGRAPH:'D0E4:53476:&#x51C8
CJK UNIFIED IDEOGRAPH:'D0E5:53477:&#x51CE
CJK UNIFIED IDEOGRAPH:'D0E6:53478:&#x5261
CJK UNIFIED IDEOGRAPH:'D0E7:53479:&#x525A
CJK UNIFIED IDEOGRAPH:'D0E8:53480:&#x5252
CJK UNIFIED IDEOGRAPH:'D0E9:53481:&#x525E
CJK UNIFIED IDEOGRAPH:'D0EA:53482:&#x525F
CJK UNIFIED IDEOGRAPH:'D0EB:53483:&#x5255
CJK UNIFIED IDEOGRAPH:'D0EC:53484:&#x5262
CJK UNIFIED IDEOGRAPH:'D0ED:53485:&#x52CD
CJK UNIFIED IDEOGRAPH:'D0EE:53486:&#x530E
CJK UNIFIED IDEOGRAPH:'D0EF:53487:&#x539E
CJK UNIFIED IDEOGRAPH:'D0F0:53488:&#x5526
CJK UNIFIED IDEOGRAPH:'D0F1:53489:&#x54E2
CJK UNIFIED IDEOGRAPH:'D0F2:53490:&#x5517
CJK UNIFIED IDEOGRAPH:'D0F3:53491:&#x5512
CJK UNIFIED IDEOGRAPH:'D0F4:53492:&#x54E7
CJK UNIFIED IDEOGRAPH:'D0F5:53493:&#x54F3
CJK UNIFIED IDEOGRAPH:'D0F6:53494:&#x54E4
CJK UNIFIED IDEOGRAPH:'D0F7:53495:&#x551A
CJK UNIFIED IDEOGRAPH:'D0F8:53496:&#x54FF
CJK UNIFIED IDEOGRAPH:'D0F9:53497:&#x5504
CJK UNIFIED IDEOGRAPH:'D0FA:53498:&#x5508
CJK UNIFIED IDEOGRAPH:'D0FB:53499:&#x54EB
CJK UNIFIED IDEOGRAPH:'D0FC:53500:&#x5511
CJK UNIFIED IDEOGRAPH:'D0FD:53501:&#x5505
CJK UNIFIED IDEOGRAPH:'D0FE:53502:&#x54F1
CJK UNIFIED IDEOGRAPH:'D140:53568:&#x550A
CJK UNIFIED IDEOGRAPH:'D141:53569:&#x54FB
CJK UNIFIED IDEOGRAPH:'D142:53570:&#x54F7
CJK UNIFIED IDEOGRAPH:'D143:53571:&#x54F8
CJK UNIFIED IDEOGRAPH:'D144:53572:&#x54E0
CJK UNIFIED IDEOGRAPH:'D145:53573:&#x550E
CJK UNIFIED IDEOGRAPH:'D146:53574:&#x5503
CJK UNIFIED IDEOGRAPH:'D147:53575:&#x550B
CJK UNIFIED IDEOGRAPH:'D148:53576:&#x5701
CJK UNIFIED IDEOGRAPH:'D149:53577:&#x5702
CJK UNIFIED IDEOGRAPH:'D14A:53578:&#x57CC
CJK UNIFIED IDEOGRAPH:'D14B:53579:&#x5832
CJK UNIFIED IDEOGRAPH:'D14C:53580:&#x57D5
CJK UNIFIED IDEOGRAPH:'D14D:53581:&#x57D2
CJK UNIFIED IDEOGRAPH:'D14E:53582:&#x57BA
CJK UNIFIED IDEOGRAPH:'D14F:53583:&#x57C6
CJK UNIFIED IDEOGRAPH:'D150:53584:&#x57BD
CJK UNIFIED IDEOGRAPH:'D151:53585:&#x57BC
CJK UNIFIED IDEOGRAPH:'D152:53586:&#x57B8
CJK UNIFIED IDEOGRAPH:'D153:53587:&#x57B6
CJK UNIFIED IDEOGRAPH:'D154:53588:&#x57BF
CJK UNIFIED IDEOGRAPH:'D155:53589:&#x57C7
CJK UNIFIED IDEOGRAPH:'D156:53590:&#x57D0
CJK UNIFIED IDEOGRAPH:'D157:53591:&#x57B9
CJK UNIFIED IDEOGRAPH:'D158:53592:&#x57C1
CJK UNIFIED IDEOGRAPH:'D159:53593:&#x590E
CJK UNIFIED IDEOGRAPH:'D15A:53594:&#x594A
CJK UNIFIED IDEOGRAPH:'D15B:53595:&#x5A19
CJK UNIFIED IDEOGRAPH:'D15C:53596:&#x5A16
CJK UNIFIED IDEOGRAPH:'D15D:53597:&#x5A2D
CJK UNIFIED IDEOGRAPH:'D15E:53598:&#x5A2E
CJK UNIFIED IDEOGRAPH:'D15F:53599:&#x5A15
CJK UNIFIED IDEOGRAPH:'D160:53600:&#x5A0F
CJK UNIFIED IDEOGRAPH:'D161:53601:&#x5A17
CJK UNIFIED IDEOGRAPH:'D162:53602:&#x5A0A
CJK UNIFIED IDEOGRAPH:'D163:53603:&#x5A1E
CJK UNIFIED IDEOGRAPH:'D164:53604:&#x5A33
CJK UNIFIED IDEOGRAPH:'D165:53605:&#x5B6C
CJK UNIFIED IDEOGRAPH:'D166:53606:&#x5BA7
CJK UNIFIED IDEOGRAPH:'D167:53607:&#x5BAD
CJK UNIFIED IDEOGRAPH:'D168:53608:&#x5BAC
CJK UNIFIED IDEOGRAPH:'D169:53609:&#x5C03
CJK UNIFIED IDEOGRAPH:'D16A:53610:&#x5C56
CJK UNIFIED IDEOGRAPH:'D16B:53611:&#x5C54
CJK UNIFIED IDEOGRAPH:'D16C:53612:&#x5CEC
CJK UNIFIED IDEOGRAPH:'D16D:53613:&#x5CFF
CJK UNIFIED IDEOGRAPH:'D16E:53614:&#x5CEE
CJK UNIFIED IDEOGRAPH:'D16F:53615:&#x5CF1
CJK UNIFIED IDEOGRAPH:'D170:53616:&#x5CF7
CJK UNIFIED IDEOGRAPH:'D171:53617:&#x5D00
CJK UNIFIED IDEOGRAPH:'D172:53618:&#x5CF9
CJK UNIFIED IDEOGRAPH:'D173:53619:&#x5E29
CJK UNIFIED IDEOGRAPH:'D174:53620:&#x5E28
CJK UNIFIED IDEOGRAPH:'D175:53621:&#x5EA8
CJK UNIFIED IDEOGRAPH:'D176:53622:&#x5EAE
CJK UNIFIED IDEOGRAPH:'D177:53623:&#x5EAA
CJK UNIFIED IDEOGRAPH:'D178:53624:&#x5EAC
CJK UNIFIED IDEOGRAPH:'D179:53625:&#x5F33
CJK UNIFIED IDEOGRAPH:'D17A:53626:&#x5F30
CJK UNIFIED IDEOGRAPH:'D17B:53627:&#x5F67
CJK UNIFIED IDEOGRAPH:'D17C:53628:&#x605D
CJK UNIFIED IDEOGRAPH:'D17D:53629:&#x605A
CJK UNIFIED IDEOGRAPH:'D17E:53630:&#x6067
CJK UNIFIED IDEOGRAPH:'D1A1:53665:&#x6041
CJK UNIFIED IDEOGRAPH:'D1A2:53666:&#x60A2
CJK UNIFIED IDEOGRAPH:'D1A3:53667:&#x6088
CJK UNIFIED IDEOGRAPH:'D1A4:53668:&#x6080
CJK UNIFIED IDEOGRAPH:'D1A5:53669:&#x6092
CJK UNIFIED IDEOGRAPH:'D1A6:53670:&#x6081
CJK UNIFIED IDEOGRAPH:'D1A7:53671:&#x609D
CJK UNIFIED IDEOGRAPH:'D1A8:53672:&#x6083
CJK UNIFIED IDEOGRAPH:'D1A9:53673:&#x6095
CJK UNIFIED IDEOGRAPH:'D1AA:53674:&#x609B
CJK UNIFIED IDEOGRAPH:'D1AB:53675:&#x6097
CJK UNIFIED IDEOGRAPH:'D1AC:53676:&#x6087
CJK UNIFIED IDEOGRAPH:'D1AD:53677:&#x609C
CJK UNIFIED IDEOGRAPH:'D1AE:53678:&#x608E
CJK UNIFIED IDEOGRAPH:'D1AF:53679:&#x6219
CJK UNIFIED IDEOGRAPH:'D1B0:53680:&#x6246
CJK UNIFIED IDEOGRAPH:'D1B1:53681:&#x62F2
CJK UNIFIED IDEOGRAPH:'D1B2:53682:&#x6310
CJK UNIFIED IDEOGRAPH:'D1B3:53683:&#x6356
CJK UNIFIED IDEOGRAPH:'D1B4:53684:&#x632C
CJK UNIFIED IDEOGRAPH:'D1B5:53685:&#x6344
CJK UNIFIED IDEOGRAPH:'D1B6:53686:&#x6345
CJK UNIFIED IDEOGRAPH:'D1B7:53687:&#x6336
CJK UNIFIED IDEOGRAPH:'D1B8:53688:&#x6343
CJK UNIFIED IDEOGRAPH:'D1B9:53689:&#x63E4
CJK UNIFIED IDEOGRAPH:'D1BA:53690:&#x6339
CJK UNIFIED IDEOGRAPH:'D1BB:53691:&#x634B
CJK UNIFIED IDEOGRAPH:'D1BC:53692:&#x634A
CJK UNIFIED IDEOGRAPH:'D1BD:53693:&#x633C
CJK UNIFIED IDEOGRAPH:'D1BE:53694:&#x6329
CJK UNIFIED IDEOGRAPH:'D1BF:53695:&#x6341
CJK UNIFIED IDEOGRAPH:'D1C0:53696:&#x6334
CJK UNIFIED IDEOGRAPH:'D1C1:53697:&#x6358
CJK UNIFIED IDEOGRAPH:'D1C2:53698:&#x6354
CJK UNIFIED IDEOGRAPH:'D1C3:53699:&#x6359
CJK UNIFIED IDEOGRAPH:'D1C4:53700:&#x632D
CJK UNIFIED IDEOGRAPH:'D1C5:53701:&#x6347
CJK UNIFIED IDEOGRAPH:'D1C6:53702:&#x6333
CJK UNIFIED IDEOGRAPH:'D1C7:53703:&#x635A
CJK UNIFIED IDEOGRAPH:'D1C8:53704:&#x6351
CJK UNIFIED IDEOGRAPH:'D1C9:53705:&#x6338
CJK UNIFIED IDEOGRAPH:'D1CA:53706:&#x6357
CJK UNIFIED IDEOGRAPH:'D1CB:53707:&#x6340
CJK UNIFIED IDEOGRAPH:'D1CC:53708:&#x6348
CJK UNIFIED IDEOGRAPH:'D1CD:53709:&#x654A
CJK UNIFIED IDEOGRAPH:'D1CE:53710:&#x6546
CJK UNIFIED IDEOGRAPH:'D1CF:53711:&#x65C6
CJK UNIFIED IDEOGRAPH:'D1D0:53712:&#x65C3
CJK UNIFIED IDEOGRAPH:'D1D1:53713:&#x65C4
CJK UNIFIED IDEOGRAPH:'D1D2:53714:&#x65C2
CJK UNIFIED IDEOGRAPH:'D1D3:53715:&#x664A
CJK UNIFIED IDEOGRAPH:'D1D4:53716:&#x665F
CJK UNIFIED IDEOGRAPH:'D1D5:53717:&#x6647
CJK UNIFIED IDEOGRAPH:'D1D6:53718:&#x6651
CJK UNIFIED IDEOGRAPH:'D1D7:53719:&#x6712
CJK UNIFIED IDEOGRAPH:'D1D8:53720:&#x6713
CJK UNIFIED IDEOGRAPH:'D1D9:53721:&#x681F
CJK UNIFIED IDEOGRAPH:'D1DA:53722:&#x681A
CJK UNIFIED IDEOGRAPH:'D1DB:53723:&#x6849
CJK UNIFIED IDEOGRAPH:'D1DC:53724:&#x6832
CJK UNIFIED IDEOGRAPH:'D1DD:53725:&#x6833
CJK UNIFIED IDEOGRAPH:'D1DE:53726:&#x683B
CJK UNIFIED IDEOGRAPH:'D1DF:53727:&#x684B
CJK UNIFIED IDEOGRAPH:'D1E0:53728:&#x684F
CJK UNIFIED IDEOGRAPH:'D1E1:53729:&#x6816
CJK UNIFIED IDEOGRAPH:'D1E2:53730:&#x6831
CJK UNIFIED IDEOGRAPH:'D1E3:53731:&#x681C
CJK UNIFIED IDEOGRAPH:'D1E4:53732:&#x6835
CJK UNIFIED IDEOGRAPH:'D1E5:53733:&#x682B
CJK UNIFIED IDEOGRAPH:'D1E6:53734:&#x682D
CJK UNIFIED IDEOGRAPH:'D1E7:53735:&#x682F
CJK UNIFIED IDEOGRAPH:'D1E8:53736:&#x684E
CJK UNIFIED IDEOGRAPH:'D1E9:53737:&#x6844
CJK UNIFIED IDEOGRAPH:'D1EA:53738:&#x6834
CJK UNIFIED IDEOGRAPH:'D1EB:53739:&#x681D
CJK UNIFIED IDEOGRAPH:'D1EC:53740:&#x6812
CJK UNIFIED IDEOGRAPH:'D1ED:53741:&#x6814
CJK UNIFIED IDEOGRAPH:'D1EE:53742:&#x6826
CJK UNIFIED IDEOGRAPH:'D1EF:53743:&#x6828
CJK UNIFIED IDEOGRAPH:'D1F0:53744:&#x682E
CJK UNIFIED IDEOGRAPH:'D1F1:53745:&#x684D
CJK UNIFIED IDEOGRAPH:'D1F2:53746:&#x683A
CJK UNIFIED IDEOGRAPH:'D1F3:53747:&#x6825
CJK UNIFIED IDEOGRAPH:'D1F4:53748:&#x6820
CJK UNIFIED IDEOGRAPH:'D1F5:53749:&#x6B2C
CJK UNIFIED IDEOGRAPH:'D1F6:53750:&#x6B2F
CJK UNIFIED IDEOGRAPH:'D1F7:53751:&#x6B2D
CJK UNIFIED IDEOGRAPH:'D1F8:53752:&#x6B31
CJK UNIFIED IDEOGRAPH:'D1F9:53753:&#x6B34
CJK UNIFIED IDEOGRAPH:'D1FA:53754:&#x6B6D
CJK UNIFIED IDEOGRAPH:'D1FB:53755:&#x8082
CJK UNIFIED IDEOGRAPH:'D1FC:53756:&#x6B88
CJK UNIFIED IDEOGRAPH:'D1FD:53757:&#x6BE6
CJK UNIFIED IDEOGRAPH:'D1FE:53758:&#x6BE4
CJK UNIFIED IDEOGRAPH:'D240:53824:&#x6BE8
CJK UNIFIED IDEOGRAPH:'D241:53825:&#x6BE3
CJK UNIFIED IDEOGRAPH:'D242:53826:&#x6BE2
CJK UNIFIED IDEOGRAPH:'D243:53827:&#x6BE7
CJK UNIFIED IDEOGRAPH:'D244:53828:&#x6C25
CJK UNIFIED IDEOGRAPH:'D245:53829:&#x6D7A
CJK UNIFIED IDEOGRAPH:'D246:53830:&#x6D63
CJK UNIFIED IDEOGRAPH:'D247:53831:&#x6D64
CJK UNIFIED IDEOGRAPH:'D248:53832:&#x6D76
CJK UNIFIED IDEOGRAPH:'D249:53833:&#x6D0D
CJK UNIFIED IDEOGRAPH:'D24A:53834:&#x6D61
CJK UNIFIED IDEOGRAPH:'D24B:53835:&#x6D92
CJK UNIFIED IDEOGRAPH:'D24C:53836:&#x6D58
CJK UNIFIED IDEOGRAPH:'D24D:53837:&#x6D62
CJK UNIFIED IDEOGRAPH:'D24E:53838:&#x6D6D
CJK UNIFIED IDEOGRAPH:'D24F:53839:&#x6D6F
CJK UNIFIED IDEOGRAPH:'D250:53840:&#x6D91
CJK UNIFIED IDEOGRAPH:'D251:53841:&#x6D8D
CJK UNIFIED IDEOGRAPH:'D252:53842:&#x6DEF
CJK UNIFIED IDEOGRAPH:'D253:53843:&#x6D7F
CJK UNIFIED IDEOGRAPH:'D254:53844:&#x6D86
CJK UNIFIED IDEOGRAPH:'D255:53845:&#x6D5E
CJK UNIFIED IDEOGRAPH:'D256:53846:&#x6D67
CJK UNIFIED IDEOGRAPH:'D257:53847:&#x6D60
CJK UNIFIED IDEOGRAPH:'D258:53848:&#x6D97
CJK UNIFIED IDEOGRAPH:'D259:53849:&#x6D70
CJK UNIFIED IDEOGRAPH:'D25A:53850:&#x6D7C
CJK UNIFIED IDEOGRAPH:'D25B:53851:&#x6D5F
CJK UNIFIED IDEOGRAPH:'D25C:53852:&#x6D82
CJK UNIFIED IDEOGRAPH:'D25D:53853:&#x6D98
CJK UNIFIED IDEOGRAPH:'D25E:53854:&#x6D2F
CJK UNIFIED IDEOGRAPH:'D25F:53855:&#x6D68
CJK UNIFIED IDEOGRAPH:'D260:53856:&#x6D8B
CJK UNIFIED IDEOGRAPH:'D261:53857:&#x6D7E
CJK UNIFIED IDEOGRAPH:'D262:53858:&#x6D80
CJK UNIFIED IDEOGRAPH:'D263:53859:&#x6D84
CJK UNIFIED IDEOGRAPH:'D264:53860:&#x6D16
CJK UNIFIED IDEOGRAPH:'D265:53861:&#x6D83
CJK UNIFIED IDEOGRAPH:'D266:53862:&#x6D7B
CJK UNIFIED IDEOGRAPH:'D267:53863:&#x6D7D
CJK UNIFIED IDEOGRAPH:'D268:53864:&#x6D75
CJK UNIFIED IDEOGRAPH:'D269:53865:&#x6D90
CJK UNIFIED IDEOGRAPH:'D26A:53866:&#x70DC
CJK UNIFIED IDEOGRAPH:'D26B:53867:&#x70D3
CJK UNIFIED IDEOGRAPH:'D26C:53868:&#x70D1
CJK UNIFIED IDEOGRAPH:'D26D:53869:&#x70DD
CJK UNIFIED IDEOGRAPH:'D26E:53870:&#x70CB
CJK UNIFIED IDEOGRAPH:'D26F:53871:&#x7F39
CJK UNIFIED IDEOGRAPH:'D270:53872:&#x70E2
CJK UNIFIED IDEOGRAPH:'D271:53873:&#x70D7
CJK UNIFIED IDEOGRAPH:'D272:53874:&#x70D2
CJK UNIFIED IDEOGRAPH:'D273:53875:&#x70DE
CJK UNIFIED IDEOGRAPH:'D274:53876:&#x70E0
CJK UNIFIED IDEOGRAPH:'D275:53877:&#x70D4
CJK UNIFIED IDEOGRAPH:'D276:53878:&#x70CD
CJK UNIFIED IDEOGRAPH:'D277:53879:&#x70C5
CJK UNIFIED IDEOGRAPH:'D278:53880:&#x70C6
CJK UNIFIED IDEOGRAPH:'D279:53881:&#x70C7
CJK UNIFIED IDEOGRAPH:'D27A:53882:&#x70DA
CJK UNIFIED IDEOGRAPH:'D27B:53883:&#x70CE
CJK UNIFIED IDEOGRAPH:'D27C:53884:&#x70E1
CJK UNIFIED IDEOGRAPH:'D27D:53885:&#x7242
CJK UNIFIED IDEOGRAPH:'D27E:53886:&#x7278
CJK UNIFIED IDEOGRAPH:'D2A1:53921:&#x7277
CJK UNIFIED IDEOGRAPH:'D2A2:53922:&#x7276
CJK UNIFIED IDEOGRAPH:'D2A3:53923:&#x7300
CJK UNIFIED IDEOGRAPH:'D2A4:53924:&#x72FA
CJK UNIFIED IDEOGRAPH:'D2A5:53925:&#x72F4
CJK UNIFIED IDEOGRAPH:'D2A6:53926:&#x72FE
CJK UNIFIED IDEOGRAPH:'D2A7:53927:&#x72F6
CJK UNIFIED IDEOGRAPH:'D2A8:53928:&#x72F3
CJK UNIFIED IDEOGRAPH:'D2A9:53929:&#x72FB
CJK UNIFIED IDEOGRAPH:'D2AA:53930:&#x7301
CJK UNIFIED IDEOGRAPH:'D2AB:53931:&#x73D3
CJK UNIFIED IDEOGRAPH:'D2AC:53932:&#x73D9
CJK UNIFIED IDEOGRAPH:'D2AD:53933:&#x73E5
CJK UNIFIED IDEOGRAPH:'D2AE:53934:&#x73D6
CJK UNIFIED IDEOGRAPH:'D2AF:53935:&#x73BC
CJK UNIFIED IDEOGRAPH:'D2B0:53936:&#x73E7
CJK UNIFIED IDEOGRAPH:'D2B1:53937:&#x73E3
CJK UNIFIED IDEOGRAPH:'D2B2:53938:&#x73E9
CJK UNIFIED IDEOGRAPH:'D2B3:53939:&#x73DC
CJK UNIFIED IDEOGRAPH:'D2B4:53940:&#x73D2
CJK UNIFIED IDEOGRAPH:'D2B5:53941:&#x73DB
CJK UNIFIED IDEOGRAPH:'D2B6:53942:&#x73D4
CJK UNIFIED IDEOGRAPH:'D2B7:53943:&#x73DD
CJK UNIFIED IDEOGRAPH:'D2B8:53944:&#x73DA
CJK UNIFIED IDEOGRAPH:'D2B9:53945:&#x73D7
CJK UNIFIED IDEOGRAPH:'D2BA:53946:&#x73D8
CJK UNIFIED IDEOGRAPH:'D2BB:53947:&#x73E8
CJK UNIFIED IDEOGRAPH:'D2BC:53948:&#x74DE
CJK UNIFIED IDEOGRAPH:'D2BD:53949:&#x74DF
CJK UNIFIED IDEOGRAPH:'D2BE:53950:&#x74F4
CJK UNIFIED IDEOGRAPH:'D2BF:53951:&#x74F5
CJK UNIFIED IDEOGRAPH:'D2C0:53952:&#x7521
CJK UNIFIED IDEOGRAPH:'D2C1:53953:&#x755B
CJK UNIFIED IDEOGRAPH:'D2C2:53954:&#x755F
CJK UNIFIED IDEOGRAPH:'D2C3:53955:&#x75B0
CJK UNIFIED IDEOGRAPH:'D2C4:53956:&#x75C1
CJK UNIFIED IDEOGRAPH:'D2C5:53957:&#x75BB
CJK UNIFIED IDEOGRAPH:'D2C6:53958:&#x75C4
CJK UNIFIED IDEOGRAPH:'D2C7:53959:&#x75C0
CJK UNIFIED IDEOGRAPH:'D2C8:53960:&#x75BF
CJK UNIFIED IDEOGRAPH:'D2C9:53961:&#x75B6
CJK UNIFIED IDEOGRAPH:'D2CA:53962:&#x75BA
CJK UNIFIED IDEOGRAPH:'D2CB:53963:&#x768A
CJK UNIFIED IDEOGRAPH:'D2CC:53964:&#x76C9
CJK UNIFIED IDEOGRAPH:'D2CD:53965:&#x771D
CJK UNIFIED IDEOGRAPH:'D2CE:53966:&#x771B
CJK UNIFIED IDEOGRAPH:'D2CF:53967:&#x7710
CJK UNIFIED IDEOGRAPH:'D2D0:53968:&#x7713
CJK UNIFIED IDEOGRAPH:'D2D1:53969:&#x7712
CJK UNIFIED IDEOGRAPH:'D2D2:53970:&#x7723
CJK UNIFIED IDEOGRAPH:'D2D3:53971:&#x7711
CJK UNIFIED IDEOGRAPH:'D2D4:53972:&#x7715
CJK UNIFIED IDEOGRAPH:'D2D5:53973:&#x7719
CJK UNIFIED IDEOGRAPH:'D2D6:53974:&#x771A
CJK UNIFIED IDEOGRAPH:'D2D7:53975:&#x7722
CJK UNIFIED IDEOGRAPH:'D2D8:53976:&#x7727
CJK UNIFIED IDEOGRAPH:'D2D9:53977:&#x7823
CJK UNIFIED IDEOGRAPH:'D2DA:53978:&#x782C
CJK UNIFIED IDEOGRAPH:'D2DB:53979:&#x7822
CJK UNIFIED IDEOGRAPH:'D2DC:53980:&#x7835
CJK UNIFIED IDEOGRAPH:'D2DD:53981:&#x782F
CJK UNIFIED IDEOGRAPH:'D2DE:53982:&#x7828
CJK UNIFIED IDEOGRAPH:'D2DF:53983:&#x782E
CJK UNIFIED IDEOGRAPH:'D2E0:53984:&#x782B
CJK UNIFIED IDEOGRAPH:'D2E1:53985:&#x7821
CJK UNIFIED IDEOGRAPH:'D2E2:53986:&#x7829
CJK UNIFIED IDEOGRAPH:'D2E3:53987:&#x7833
CJK UNIFIED IDEOGRAPH:'D2E4:53988:&#x782A
CJK UNIFIED IDEOGRAPH:'D2E5:53989:&#x7831
CJK UNIFIED IDEOGRAPH:'D2E6:53990:&#x7954
CJK UNIFIED IDEOGRAPH:'D2E7:53991:&#x795B
CJK UNIFIED IDEOGRAPH:'D2E8:53992:&#x794F
CJK UNIFIED IDEOGRAPH:'D2E9:53993:&#x795C
CJK UNIFIED IDEOGRAPH:'D2EA:53994:&#x7953
CJK UNIFIED IDEOGRAPH:'D2EB:53995:&#x7952
CJK UNIFIED IDEOGRAPH:'D2EC:53996:&#x7951
CJK UNIFIED IDEOGRAPH:'D2ED:53997:&#x79EB
CJK UNIFIED IDEOGRAPH:'D2EE:53998:&#x79EC
CJK UNIFIED IDEOGRAPH:'D2EF:53999:&#x79E0
CJK UNIFIED IDEOGRAPH:'D2F0:54000:&#x79EE
CJK UNIFIED IDEOGRAPH:'D2F1:54001:&#x79ED
CJK UNIFIED IDEOGRAPH:'D2F2:54002:&#x79EA
CJK UNIFIED IDEOGRAPH:'D2F3:54003:&#x79DC
CJK UNIFIED IDEOGRAPH:'D2F4:54004:&#x79DE
CJK UNIFIED IDEOGRAPH:'D2F5:54005:&#x79DD
CJK UNIFIED IDEOGRAPH:'D2F6:54006:&#x7A86
CJK UNIFIED IDEOGRAPH:'D2F7:54007:&#x7A89
CJK UNIFIED IDEOGRAPH:'D2F8:54008:&#x7A85
CJK UNIFIED IDEOGRAPH:'D2F9:54009:&#x7A8B
CJK UNIFIED IDEOGRAPH:'D2FA:54010:&#x7A8C
CJK UNIFIED IDEOGRAPH:'D2FB:54011:&#x7A8A
CJK UNIFIED IDEOGRAPH:'D2FC:54012:&#x7A87
CJK UNIFIED IDEOGRAPH:'D2FD:54013:&#x7AD8
CJK UNIFIED IDEOGRAPH:'D2FE:54014:&#x7B10
CJK UNIFIED IDEOGRAPH:'D340:54080:&#x7B04
CJK UNIFIED IDEOGRAPH:'D341:54081:&#x7B13
CJK UNIFIED IDEOGRAPH:'D342:54082:&#x7B05
CJK UNIFIED IDEOGRAPH:'D343:54083:&#x7B0F
CJK UNIFIED IDEOGRAPH:'D344:54084:&#x7B08
CJK UNIFIED IDEOGRAPH:'D345:54085:&#x7B0A
CJK UNIFIED IDEOGRAPH:'D346:54086:&#x7B0E
CJK UNIFIED IDEOGRAPH:'D347:54087:&#x7B09
CJK UNIFIED IDEOGRAPH:'D348:54088:&#x7B12
CJK UNIFIED IDEOGRAPH:'D349:54089:&#x7C84
CJK UNIFIED IDEOGRAPH:'D34A:54090:&#x7C91
CJK UNIFIED IDEOGRAPH:'D34B:54091:&#x7C8A
CJK UNIFIED IDEOGRAPH:'D34C:54092:&#x7C8C
CJK UNIFIED IDEOGRAPH:'D34D:54093:&#x7C88
CJK UNIFIED IDEOGRAPH:'D34E:54094:&#x7C8D
CJK UNIFIED IDEOGRAPH:'D34F:54095:&#x7C85
CJK UNIFIED IDEOGRAPH:'D350:54096:&#x7D1E
CJK UNIFIED IDEOGRAPH:'D351:54097:&#x7D1D
CJK UNIFIED IDEOGRAPH:'D352:54098:&#x7D11
CJK UNIFIED IDEOGRAPH:'D353:54099:&#x7D0E
CJK UNIFIED IDEOGRAPH:'D354:54100:&#x7D18
CJK UNIFIED IDEOGRAPH:'D355:54101:&#x7D16
CJK UNIFIED IDEOGRAPH:'D356:54102:&#x7D13
CJK UNIFIED IDEOGRAPH:'D357:54103:&#x7D1F
CJK UNIFIED IDEOGRAPH:'D358:54104:&#x7D12
CJK UNIFIED IDEOGRAPH:'D359:54105:&#x7D0F
CJK UNIFIED IDEOGRAPH:'D35A:54106:&#x7D0C
CJK UNIFIED IDEOGRAPH:'D35B:54107:&#x7F5C
CJK UNIFIED IDEOGRAPH:'D35C:54108:&#x7F61
CJK UNIFIED IDEOGRAPH:'D35D:54109:&#x7F5E
CJK UNIFIED IDEOGRAPH:'D35E:54110:&#x7F60
CJK UNIFIED IDEOGRAPH:'D35F:54111:&#x7F5D
CJK UNIFIED IDEOGRAPH:'D360:54112:&#x7F5B
CJK UNIFIED IDEOGRAPH:'D361:54113:&#x7F96
CJK UNIFIED IDEOGRAPH:'D362:54114:&#x7F92
CJK UNIFIED IDEOGRAPH:'D363:54115:&#x7FC3
CJK UNIFIED IDEOGRAPH:'D364:54116:&#x7FC2
CJK UNIFIED IDEOGRAPH:'D365:54117:&#x7FC0
CJK UNIFIED IDEOGRAPH:'D366:54118:&#x8016
CJK UNIFIED IDEOGRAPH:'D367:54119:&#x803E
CJK UNIFIED IDEOGRAPH:'D368:54120:&#x8039
CJK UNIFIED IDEOGRAPH:'D369:54121:&#x80FA
CJK UNIFIED IDEOGRAPH:'D36A:54122:&#x80F2
CJK UNIFIED IDEOGRAPH:'D36B:54123:&#x80F9
CJK UNIFIED IDEOGRAPH:'D36C:54124:&#x80F5
CJK UNIFIED IDEOGRAPH:'D36D:54125:&#x8101
CJK UNIFIED IDEOGRAPH:'D36E:54126:&#x80FB
CJK UNIFIED IDEOGRAPH:'D36F:54127:&#x8100
CJK UNIFIED IDEOGRAPH:'D370:54128:&#x8201
CJK UNIFIED IDEOGRAPH:'D371:54129:&#x822F
CJK UNIFIED IDEOGRAPH:'D372:54130:&#x8225
CJK UNIFIED IDEOGRAPH:'D373:54131:&#x8333
CJK UNIFIED IDEOGRAPH:'D374:54132:&#x832D
CJK UNIFIED IDEOGRAPH:'D375:54133:&#x8344
CJK UNIFIED IDEOGRAPH:'D376:54134:&#x8319
CJK UNIFIED IDEOGRAPH:'D377:54135:&#x8351
CJK UNIFIED IDEOGRAPH:'D378:54136:&#x8325
CJK UNIFIED IDEOGRAPH:'D379:54137:&#x8356
CJK UNIFIED IDEOGRAPH:'D37A:54138:&#x833F
CJK UNIFIED IDEOGRAPH:'D37B:54139:&#x8341
CJK UNIFIED IDEOGRAPH:'D37C:54140:&#x8326
CJK UNIFIED IDEOGRAPH:'D37D:54141:&#x831C
CJK UNIFIED IDEOGRAPH:'D37E:54142:&#x8322
CJK UNIFIED IDEOGRAPH:'D3A1:54177:&#x8342
CJK UNIFIED IDEOGRAPH:'D3A2:54178:&#x834E
CJK UNIFIED IDEOGRAPH:'D3A3:54179:&#x831B
CJK UNIFIED IDEOGRAPH:'D3A4:54180:&#x832A
CJK UNIFIED IDEOGRAPH:'D3A5:54181:&#x8308
CJK UNIFIED IDEOGRAPH:'D3A6:54182:&#x833C
CJK UNIFIED IDEOGRAPH:'D3A7:54183:&#x834D
CJK UNIFIED IDEOGRAPH:'D3A8:54184:&#x8316
CJK UNIFIED IDEOGRAPH:'D3A9:54185:&#x8324
CJK UNIFIED IDEOGRAPH:'D3AA:54186:&#x8320
CJK UNIFIED IDEOGRAPH:'D3AB:54187:&#x8337
CJK UNIFIED IDEOGRAPH:'D3AC:54188:&#x832F
CJK UNIFIED IDEOGRAPH:'D3AD:54189:&#x8329
CJK UNIFIED IDEOGRAPH:'D3AE:54190:&#x8347
CJK UNIFIED IDEOGRAPH:'D3AF:54191:&#x8345
CJK UNIFIED IDEOGRAPH:'D3B0:54192:&#x834C
CJK UNIFIED IDEOGRAPH:'D3B1:54193:&#x8353
CJK UNIFIED IDEOGRAPH:'D3B2:54194:&#x831E
CJK UNIFIED IDEOGRAPH:'D3B3:54195:&#x832C
CJK UNIFIED IDEOGRAPH:'D3B4:54196:&#x834B
CJK UNIFIED IDEOGRAPH:'D3B5:54197:&#x8327
CJK UNIFIED IDEOGRAPH:'D3B6:54198:&#x8348
CJK UNIFIED IDEOGRAPH:'D3B7:54199:&#x8653
CJK UNIFIED IDEOGRAPH:'D3B8:54200:&#x8652
CJK UNIFIED IDEOGRAPH:'D3B9:54201:&#x86A2
CJK UNIFIED IDEOGRAPH:'D3BA:54202:&#x86A8
CJK UNIFIED IDEOGRAPH:'D3BB:54203:&#x8696
CJK UNIFIED IDEOGRAPH:'D3BC:54204:&#x868D
CJK UNIFIED IDEOGRAPH:'D3BD:54205:&#x8691
CJK UNIFIED IDEOGRAPH:'D3BE:54206:&#x869E
CJK UNIFIED IDEOGRAPH:'D3BF:54207:&#x8687
CJK UNIFIED IDEOGRAPH:'D3C0:54208:&#x8697
CJK UNIFIED IDEOGRAPH:'D3C1:54209:&#x8686
CJK UNIFIED IDEOGRAPH:'D3C2:54210:&#x868B
CJK UNIFIED IDEOGRAPH:'D3C3:54211:&#x869A
CJK UNIFIED IDEOGRAPH:'D3C4:54212:&#x8685
CJK UNIFIED IDEOGRAPH:'D3C5:54213:&#x86A5
CJK UNIFIED IDEOGRAPH:'D3C6:54214:&#x8699
CJK UNIFIED IDEOGRAPH:'D3C7:54215:&#x86A1
CJK UNIFIED IDEOGRAPH:'D3C8:54216:&#x86A7
CJK UNIFIED IDEOGRAPH:'D3C9:54217:&#x8695
CJK UNIFIED IDEOGRAPH:'D3CA:54218:&#x8698
CJK UNIFIED IDEOGRAPH:'D3CB:54219:&#x868E
CJK UNIFIED IDEOGRAPH:'D3CC:54220:&#x869D
CJK UNIFIED IDEOGRAPH:'D3CD:54221:&#x8690
CJK UNIFIED IDEOGRAPH:'D3CE:54222:&#x8694
CJK UNIFIED IDEOGRAPH:'D3CF:54223:&#x8843
CJK UNIFIED IDEOGRAPH:'D3D0:54224:&#x8844
CJK UNIFIED IDEOGRAPH:'D3D1:54225:&#x886D
CJK UNIFIED IDEOGRAPH:'D3D2:54226:&#x8875
CJK UNIFIED IDEOGRAPH:'D3D3:54227:&#x8876
CJK UNIFIED IDEOGRAPH:'D3D4:54228:&#x8872
CJK UNIFIED IDEOGRAPH:'D3D5:54229:&#x8880
CJK UNIFIED IDEOGRAPH:'D3D6:54230:&#x8871
CJK UNIFIED IDEOGRAPH:'D3D7:54231:&#x887F
CJK UNIFIED IDEOGRAPH:'D3D8:54232:&#x886F
CJK UNIFIED IDEOGRAPH:'D3D9:54233:&#x8883
CJK UNIFIED IDEOGRAPH:'D3DA:54234:&#x887E
CJK UNIFIED IDEOGRAPH:'D3DB:54235:&#x8874
CJK UNIFIED IDEOGRAPH:'D3DC:54236:&#x887C
CJK UNIFIED IDEOGRAPH:'D3DD:54237:&#x8A12
CJK UNIFIED IDEOGRAPH:'D3DE:54238:&#x8C47
CJK UNIFIED IDEOGRAPH:'D3DF:54239:&#x8C57
CJK UNIFIED IDEOGRAPH:'D3E0:54240:&#x8C7B
CJK UNIFIED IDEOGRAPH:'D3E1:54241:&#x8CA4
CJK UNIFIED IDEOGRAPH:'D3E2:54242:&#x8CA3
CJK UNIFIED IDEOGRAPH:'D3E3:54243:&#x8D76
CJK UNIFIED IDEOGRAPH:'D3E4:54244:&#x8D78
CJK UNIFIED IDEOGRAPH:'D3E5:54245:&#x8DB5
CJK UNIFIED IDEOGRAPH:'D3E6:54246:&#x8DB7
CJK UNIFIED IDEOGRAPH:'D3E7:54247:&#x8DB6
CJK UNIFIED IDEOGRAPH:'D3E8:54248:&#x8ED1
CJK UNIFIED IDEOGRAPH:'D3E9:54249:&#x8ED3
CJK UNIFIED IDEOGRAPH:'D3EA:54250:&#x8FFE
CJK UNIFIED IDEOGRAPH:'D3EB:54251:&#x8FF5
CJK UNIFIED IDEOGRAPH:'D3EC:54252:&#x9002
CJK UNIFIED IDEOGRAPH:'D3ED:54253:&#x8FFF
CJK UNIFIED IDEOGRAPH:'D3EE:54254:&#x8FFB
CJK UNIFIED IDEOGRAPH:'D3EF:54255:&#x9004
CJK UNIFIED IDEOGRAPH:'D3F0:54256:&#x8FFC
CJK UNIFIED IDEOGRAPH:'D3F1:54257:&#x8FF6
CJK UNIFIED IDEOGRAPH:'D3F2:54258:&#x90D6
CJK UNIFIED IDEOGRAPH:'D3F3:54259:&#x90E0
CJK UNIFIED IDEOGRAPH:'D3F4:54260:&#x90D9
CJK UNIFIED IDEOGRAPH:'D3F5:54261:&#x90DA
CJK UNIFIED IDEOGRAPH:'D3F6:54262:&#x90E3
CJK UNIFIED IDEOGRAPH:'D3F7:54263:&#x90DF
CJK UNIFIED IDEOGRAPH:'D3F8:54264:&#x90E5
CJK UNIFIED IDEOGRAPH:'D3F9:54265:&#x90D8
CJK UNIFIED IDEOGRAPH:'D3FA:54266:&#x90DB
CJK UNIFIED IDEOGRAPH:'D3FB:54267:&#x90D7
CJK UNIFIED IDEOGRAPH:'D3FC:54268:&#x90DC
CJK UNIFIED IDEOGRAPH:'D3FD:54269:&#x90E4
CJK UNIFIED IDEOGRAPH:'D3FE:54270:&#x9150
CJK UNIFIED IDEOGRAPH:'D440:54336:&#x914E
CJK UNIFIED IDEOGRAPH:'D441:54337:&#x914F
CJK UNIFIED IDEOGRAPH:'D442:54338:&#x91D5
CJK UNIFIED IDEOGRAPH:'D443:54339:&#x91E2
CJK UNIFIED IDEOGRAPH:'D444:54340:&#x91DA
CJK UNIFIED IDEOGRAPH:'D445:54341:&#x965C
CJK UNIFIED IDEOGRAPH:'D446:54342:&#x965F
CJK UNIFIED IDEOGRAPH:'D447:54343:&#x96BC
CJK UNIFIED IDEOGRAPH:'D448:54344:&#x98E3
CJK UNIFIED IDEOGRAPH:'D449:54345:&#x9ADF
CJK UNIFIED IDEOGRAPH:'D44A:54346:&#x9B2F
CJK UNIFIED IDEOGRAPH:'D44B:54347:&#x4E7F
CJK UNIFIED IDEOGRAPH:'D44C:54348:&#x5070
CJK UNIFIED IDEOGRAPH:'D44D:54349:&#x506A
CJK UNIFIED IDEOGRAPH:'D44E:54350:&#x5061
CJK UNIFIED IDEOGRAPH:'D44F:54351:&#x505E
CJK UNIFIED IDEOGRAPH:'D450:54352:&#x5060
CJK UNIFIED IDEOGRAPH:'D451:54353:&#x5053
CJK UNIFIED IDEOGRAPH:'D452:54354:&#x504B
CJK UNIFIED IDEOGRAPH:'D453:54355:&#x505D
CJK UNIFIED IDEOGRAPH:'D454:54356:&#x5072
CJK UNIFIED IDEOGRAPH:'D455:54357:&#x5048
CJK UNIFIED IDEOGRAPH:'D456:54358:&#x504D
CJK UNIFIED IDEOGRAPH:'D457:54359:&#x5041
CJK UNIFIED IDEOGRAPH:'D458:54360:&#x505B
CJK UNIFIED IDEOGRAPH:'D459:54361:&#x504A
CJK UNIFIED IDEOGRAPH:'D45A:54362:&#x5062
CJK UNIFIED IDEOGRAPH:'D45B:54363:&#x5015
CJK UNIFIED IDEOGRAPH:'D45C:54364:&#x5045
CJK UNIFIED IDEOGRAPH:'D45D:54365:&#x505F
CJK UNIFIED IDEOGRAPH:'D45E:54366:&#x5069
CJK UNIFIED IDEOGRAPH:'D45F:54367:&#x506B
CJK UNIFIED IDEOGRAPH:'D460:54368:&#x5063
CJK UNIFIED IDEOGRAPH:'D461:54369:&#x5064
CJK UNIFIED IDEOGRAPH:'D462:54370:&#x5046
CJK UNIFIED IDEOGRAPH:'D463:54371:&#x5040
CJK UNIFIED IDEOGRAPH:'D464:54372:&#x506E
CJK UNIFIED IDEOGRAPH:'D465:54373:&#x5073
CJK UNIFIED IDEOGRAPH:'D466:54374:&#x5057
CJK UNIFIED IDEOGRAPH:'D467:54375:&#x5051
CJK UNIFIED IDEOGRAPH:'D468:54376:&#x51D0
CJK UNIFIED IDEOGRAPH:'D469:54377:&#x526B
CJK UNIFIED IDEOGRAPH:'D46A:54378:&#x526D
CJK UNIFIED IDEOGRAPH:'D46B:54379:&#x526C
CJK UNIFIED IDEOGRAPH:'D46C:54380:&#x526E
CJK UNIFIED IDEOGRAPH:'D46D:54381:&#x52D6
CJK UNIFIED IDEOGRAPH:'D46E:54382:&#x52D3
CJK UNIFIED IDEOGRAPH:'D46F:54383:&#x532D
CJK UNIFIED IDEOGRAPH:'D470:54384:&#x539C
CJK UNIFIED IDEOGRAPH:'D471:54385:&#x5575
CJK UNIFIED IDEOGRAPH:'D472:54386:&#x5576
CJK UNIFIED IDEOGRAPH:'D473:54387:&#x553C
CJK UNIFIED IDEOGRAPH:'D474:54388:&#x554D
CJK UNIFIED IDEOGRAPH:'D475:54389:&#x5550
CJK UNIFIED IDEOGRAPH:'D476:54390:&#x5534
CJK UNIFIED IDEOGRAPH:'D477:54391:&#x552A
CJK UNIFIED IDEOGRAPH:'D478:54392:&#x5551
CJK UNIFIED IDEOGRAPH:'D479:54393:&#x5562
CJK UNIFIED IDEOGRAPH:'D47A:54394:&#x5536
CJK UNIFIED IDEOGRAPH:'D47B:54395:&#x5535
CJK UNIFIED IDEOGRAPH:'D47C:54396:&#x5530
CJK UNIFIED IDEOGRAPH:'D47D:54397:&#x5552
CJK UNIFIED IDEOGRAPH:'D47E:54398:&#x5545
CJK UNIFIED IDEOGRAPH:'D4A1:54433:&#x550C
CJK UNIFIED IDEOGRAPH:'D4A2:54434:&#x5532
CJK UNIFIED IDEOGRAPH:'D4A3:54435:&#x5565
CJK UNIFIED IDEOGRAPH:'D4A4:54436:&#x554E
CJK UNIFIED IDEOGRAPH:'D4A5:54437:&#x5539
CJK UNIFIED IDEOGRAPH:'D4A6:54438:&#x5548
CJK UNIFIED IDEOGRAPH:'D4A7:54439:&#x552D
CJK UNIFIED IDEOGRAPH:'D4A8:54440:&#x553B
CJK UNIFIED IDEOGRAPH:'D4A9:54441:&#x5540
CJK UNIFIED IDEOGRAPH:'D4AA:54442:&#x554B
CJK UNIFIED IDEOGRAPH:'D4AB:54443:&#x570A
CJK UNIFIED IDEOGRAPH:'D4AC:54444:&#x5707
CJK UNIFIED IDEOGRAPH:'D4AD:54445:&#x57FB
CJK UNIFIED IDEOGRAPH:'D4AE:54446:&#x5814
CJK UNIFIED IDEOGRAPH:'D4AF:54447:&#x57E2
CJK UNIFIED IDEOGRAPH:'D4B0:54448:&#x57F6
CJK UNIFIED IDEOGRAPH:'D4B1:54449:&#x57DC
CJK UNIFIED IDEOGRAPH:'D4B2:54450:&#x57F4
CJK UNIFIED IDEOGRAPH:'D4B3:54451:&#x5800
CJK UNIFIED IDEOGRAPH:'D4B4:54452:&#x57ED
CJK UNIFIED IDEOGRAPH:'D4B5:54453:&#x57FD
CJK UNIFIED IDEOGRAPH:'D4B6:54454:&#x5808
CJK UNIFIED IDEOGRAPH:'D4B7:54455:&#x57F8
CJK UNIFIED IDEOGRAPH:'D4B8:54456:&#x580B
CJK UNIFIED IDEOGRAPH:'D4B9:54457:&#x57F3
CJK UNIFIED IDEOGRAPH:'D4BA:54458:&#x57CF
CJK UNIFIED IDEOGRAPH:'D4BB:54459:&#x5807
CJK UNIFIED IDEOGRAPH:'D4BC:54460:&#x57EE
CJK UNIFIED IDEOGRAPH:'D4BD:54461:&#x57E3
CJK UNIFIED IDEOGRAPH:'D4BE:54462:&#x57F2
CJK UNIFIED IDEOGRAPH:'D4BF:54463:&#x57E5
CJK UNIFIED IDEOGRAPH:'D4C0:54464:&#x57EC
CJK UNIFIED IDEOGRAPH:'D4C1:54465:&#x57E1
CJK UNIFIED IDEOGRAPH:'D4C2:54466:&#x580E
CJK UNIFIED IDEOGRAPH:'D4C3:54467:&#x57FC
CJK UNIFIED IDEOGRAPH:'D4C4:54468:&#x5810
CJK UNIFIED IDEOGRAPH:'D4C5:54469:&#x57E7
CJK UNIFIED IDEOGRAPH:'D4C6:54470:&#x5801
CJK UNIFIED IDEOGRAPH:'D4C7:54471:&#x580C
CJK UNIFIED IDEOGRAPH:'D4C8:54472:&#x57F1
CJK UNIFIED IDEOGRAPH:'D4C9:54473:&#x57E9
CJK UNIFIED IDEOGRAPH:'D4CA:54474:&#x57F0
CJK UNIFIED IDEOGRAPH:'D4CB:54475:&#x580D
CJK UNIFIED IDEOGRAPH:'D4CC:54476:&#x5804
CJK UNIFIED IDEOGRAPH:'D4CD:54477:&#x595C
CJK UNIFIED IDEOGRAPH:'D4CE:54478:&#x5A60
CJK UNIFIED IDEOGRAPH:'D4CF:54479:&#x5A58
CJK UNIFIED IDEOGRAPH:'D4D0:54480:&#x5A55
CJK UNIFIED IDEOGRAPH:'D4D1:54481:&#x5A67
CJK UNIFIED IDEOGRAPH:'D4D2:54482:&#x5A5E
CJK UNIFIED IDEOGRAPH:'D4D3:54483:&#x5A38
CJK UNIFIED IDEOGRAPH:'D4D4:54484:&#x5A35
CJK UNIFIED IDEOGRAPH:'D4D5:54485:&#x5A6D
CJK UNIFIED IDEOGRAPH:'D4D6:54486:&#x5A50
CJK UNIFIED IDEOGRAPH:'D4D7:54487:&#x5A5F
CJK UNIFIED IDEOGRAPH:'D4D8:54488:&#x5A65
CJK UNIFIED IDEOGRAPH:'D4D9:54489:&#x5A6C
CJK UNIFIED IDEOGRAPH:'D4DA:54490:&#x5A53
CJK UNIFIED IDEOGRAPH:'D4DB:54491:&#x5A64
CJK UNIFIED IDEOGRAPH:'D4DC:54492:&#x5A57
CJK UNIFIED IDEOGRAPH:'D4DD:54493:&#x5A43
CJK UNIFIED IDEOGRAPH:'D4DE:54494:&#x5A5D
CJK UNIFIED IDEOGRAPH:'D4DF:54495:&#x5A52
CJK UNIFIED IDEOGRAPH:'D4E0:54496:&#x5A44
CJK UNIFIED IDEOGRAPH:'D4E1:54497:&#x5A5B
CJK UNIFIED IDEOGRAPH:'D4E2:54498:&#x5A48
CJK UNIFIED IDEOGRAPH:'D4E3:54499:&#x5A8E
CJK UNIFIED IDEOGRAPH:'D4E4:54500:&#x5A3E
CJK UNIFIED IDEOGRAPH:'D4E5:54501:&#x5A4D
CJK UNIFIED IDEOGRAPH:'D4E6:54502:&#x5A39
CJK UNIFIED IDEOGRAPH:'D4E7:54503:&#x5A4C
CJK UNIFIED IDEOGRAPH:'D4E8:54504:&#x5A70
CJK UNIFIED IDEOGRAPH:'D4E9:54505:&#x5A69
CJK UNIFIED IDEOGRAPH:'D4EA:54506:&#x5A47
CJK UNIFIED IDEOGRAPH:'D4EB:54507:&#x5A51
CJK UNIFIED IDEOGRAPH:'D4EC:54508:&#x5A56
CJK UNIFIED IDEOGRAPH:'D4ED:54509:&#x5A42
CJK UNIFIED IDEOGRAPH:'D4EE:54510:&#x5A5C
CJK UNIFIED IDEOGRAPH:'D4EF:54511:&#x5B72
CJK UNIFIED IDEOGRAPH:'D4F0:54512:&#x5B6E
CJK UNIFIED IDEOGRAPH:'D4F1:54513:&#x5BC1
CJK UNIFIED IDEOGRAPH:'D4F2:54514:&#x5BC0
CJK UNIFIED IDEOGRAPH:'D4F3:54515:&#x5C59
CJK UNIFIED IDEOGRAPH:'D4F4:54516:&#x5D1E
CJK UNIFIED IDEOGRAPH:'D4F5:54517:&#x5D0B
CJK UNIFIED IDEOGRAPH:'D4F6:54518:&#x5D1D
CJK UNIFIED IDEOGRAPH:'D4F7:54519:&#x5D1A
CJK UNIFIED IDEOGRAPH:'D4F8:54520:&#x5D20
CJK UNIFIED IDEOGRAPH:'D4F9:54521:&#x5D0C
CJK UNIFIED IDEOGRAPH:'D4FA:54522:&#x5D28
CJK UNIFIED IDEOGRAPH:'D4FB:54523:&#x5D0D
CJK UNIFIED IDEOGRAPH:'D4FC:54524:&#x5D26
CJK UNIFIED IDEOGRAPH:'D4FD:54525:&#x5D25
CJK UNIFIED IDEOGRAPH:'D4FE:54526:&#x5D0F
CJK UNIFIED IDEOGRAPH:'D540:54592:&#x5D30
CJK UNIFIED IDEOGRAPH:'D541:54593:&#x5D12
CJK UNIFIED IDEOGRAPH:'D542:54594:&#x5D23
CJK UNIFIED IDEOGRAPH:'D543:54595:&#x5D1F
CJK UNIFIED IDEOGRAPH:'D544:54596:&#x5D2E
CJK UNIFIED IDEOGRAPH:'D545:54597:&#x5E3E
CJK UNIFIED IDEOGRAPH:'D546:54598:&#x5E34
CJK UNIFIED IDEOGRAPH:'D547:54599:&#x5EB1
CJK UNIFIED IDEOGRAPH:'D548:54600:&#x5EB4
CJK UNIFIED IDEOGRAPH:'D549:54601:&#x5EB9
CJK UNIFIED IDEOGRAPH:'D54A:54602:&#x5EB2
CJK UNIFIED IDEOGRAPH:'D54B:54603:&#x5EB3
CJK UNIFIED IDEOGRAPH:'D54C:54604:&#x5F36
CJK UNIFIED IDEOGRAPH:'D54D:54605:&#x5F38
CJK UNIFIED IDEOGRAPH:'D54E:54606:&#x5F9B
CJK UNIFIED IDEOGRAPH:'D54F:54607:&#x5F96
CJK UNIFIED IDEOGRAPH:'D550:54608:&#x5F9F
CJK UNIFIED IDEOGRAPH:'D551:54609:&#x608A
CJK UNIFIED IDEOGRAPH:'D552:54610:&#x6090
CJK UNIFIED IDEOGRAPH:'D553:54611:&#x6086
CJK UNIFIED IDEOGRAPH:'D554:54612:&#x60BE
CJK UNIFIED IDEOGRAPH:'D555:54613:&#x60B0
CJK UNIFIED IDEOGRAPH:'D556:54614:&#x60BA
CJK UNIFIED IDEOGRAPH:'D557:54615:&#x60D3
CJK UNIFIED IDEOGRAPH:'D558:54616:&#x60D4
CJK UNIFIED IDEOGRAPH:'D559:54617:&#x60CF
CJK UNIFIED IDEOGRAPH:'D55A:54618:&#x60E4
CJK UNIFIED IDEOGRAPH:'D55B:54619:&#x60D9
CJK UNIFIED IDEOGRAPH:'D55C:54620:&#x60DD
CJK UNIFIED IDEOGRAPH:'D55D:54621:&#x60C8
CJK UNIFIED IDEOGRAPH:'D55E:54622:&#x60B1
CJK UNIFIED IDEOGRAPH:'D55F:54623:&#x60DB
CJK UNIFIED IDEOGRAPH:'D560:54624:&#x60B7
CJK UNIFIED IDEOGRAPH:'D561:54625:&#x60CA
CJK UNIFIED IDEOGRAPH:'D562:54626:&#x60BF
CJK UNIFIED IDEOGRAPH:'D563:54627:&#x60C3
CJK UNIFIED IDEOGRAPH:'D564:54628:&#x60CD
CJK UNIFIED IDEOGRAPH:'D565:54629:&#x60C0
CJK UNIFIED IDEOGRAPH:'D566:54630:&#x6332
CJK UNIFIED IDEOGRAPH:'D567:54631:&#x6365
CJK UNIFIED IDEOGRAPH:'D568:54632:&#x638A
CJK UNIFIED IDEOGRAPH:'D569:54633:&#x6382
CJK UNIFIED IDEOGRAPH:'D56A:54634:&#x637D
CJK UNIFIED IDEOGRAPH:'D56B:54635:&#x63BD
CJK UNIFIED IDEOGRAPH:'D56C:54636:&#x639E
CJK UNIFIED IDEOGRAPH:'D56D:54637:&#x63AD
CJK UNIFIED IDEOGRAPH:'D56E:54638:&#x639D
CJK UNIFIED IDEOGRAPH:'D56F:54639:&#x6397
CJK UNIFIED IDEOGRAPH:'D570:54640:&#x63AB
CJK UNIFIED IDEOGRAPH:'D571:54641:&#x638E
CJK UNIFIED IDEOGRAPH:'D572:54642:&#x636F
CJK UNIFIED IDEOGRAPH:'D573:54643:&#x6387
CJK UNIFIED IDEOGRAPH:'D574:54644:&#x6390
CJK UNIFIED IDEOGRAPH:'D575:54645:&#x636E
CJK UNIFIED IDEOGRAPH:'D576:54646:&#x63AF
CJK UNIFIED IDEOGRAPH:'D577:54647:&#x6375
CJK UNIFIED IDEOGRAPH:'D578:54648:&#x639C
CJK UNIFIED IDEOGRAPH:'D579:54649:&#x636D
CJK UNIFIED IDEOGRAPH:'D57A:54650:&#x63AE
CJK UNIFIED IDEOGRAPH:'D57B:54651:&#x637C
CJK UNIFIED IDEOGRAPH:'D57C:54652:&#x63A4
CJK UNIFIED IDEOGRAPH:'D57D:54653:&#x633B
CJK UNIFIED IDEOGRAPH:'D57E:54654:&#x639F
CJK UNIFIED IDEOGRAPH:'D5A1:54689:&#x6378
CJK UNIFIED IDEOGRAPH:'D5A2:54690:&#x6385
CJK UNIFIED IDEOGRAPH:'D5A3:54691:&#x6381
CJK UNIFIED IDEOGRAPH:'D5A4:54692:&#x6391
CJK UNIFIED IDEOGRAPH:'D5A5:54693:&#x638D
CJK UNIFIED IDEOGRAPH:'D5A6:54694:&#x6370
CJK UNIFIED IDEOGRAPH:'D5A7:54695:&#x6553
CJK UNIFIED IDEOGRAPH:'D5A8:54696:&#x65CD
CJK UNIFIED IDEOGRAPH:'D5A9:54697:&#x6665
CJK UNIFIED IDEOGRAPH:'D5AA:54698:&#x6661
CJK UNIFIED IDEOGRAPH:'D5AB:54699:&#x665B
CJK UNIFIED IDEOGRAPH:'D5AC:54700:&#x6659
CJK UNIFIED IDEOGRAPH:'D5AD:54701:&#x665C
CJK UNIFIED IDEOGRAPH:'D5AE:54702:&#x6662
CJK UNIFIED IDEOGRAPH:'D5AF:54703:&#x6718
CJK UNIFIED IDEOGRAPH:'D5B0:54704:&#x6879
CJK UNIFIED IDEOGRAPH:'D5B1:54705:&#x6887
CJK UNIFIED IDEOGRAPH:'D5B2:54706:&#x6890
CJK UNIFIED IDEOGRAPH:'D5B3:54707:&#x689C
CJK UNIFIED IDEOGRAPH:'D5B4:54708:&#x686D
CJK UNIFIED IDEOGRAPH:'D5B5:54709:&#x686E
CJK UNIFIED IDEOGRAPH:'D5B6:54710:&#x68AE
CJK UNIFIED IDEOGRAPH:'D5B7:54711:&#x68AB
CJK UNIFIED IDEOGRAPH:'D5B8:54712:&#x6956
CJK UNIFIED IDEOGRAPH:'D5B9:54713:&#x686F
CJK UNIFIED IDEOGRAPH:'D5BA:54714:&#x68A3
CJK UNIFIED IDEOGRAPH:'D5BB:54715:&#x68AC
CJK UNIFIED IDEOGRAPH:'D5BC:54716:&#x68A9
CJK UNIFIED IDEOGRAPH:'D5BD:54717:&#x6875
CJK UNIFIED IDEOGRAPH:'D5BE:54718:&#x6874
CJK UNIFIED IDEOGRAPH:'D5BF:54719:&#x68B2
CJK UNIFIED IDEOGRAPH:'D5C0:54720:&#x688F
CJK UNIFIED IDEOGRAPH:'D5C1:54721:&#x6877
CJK UNIFIED IDEOGRAPH:'D5C2:54722:&#x6892
CJK UNIFIED IDEOGRAPH:'D5C3:54723:&#x687C
CJK UNIFIED IDEOGRAPH:'D5C4:54724:&#x686B
CJK UNIFIED IDEOGRAPH:'D5C5:54725:&#x6872
CJK UNIFIED IDEOGRAPH:'D5C6:54726:&#x68AA
CJK UNIFIED IDEOGRAPH:'D5C7:54727:&#x6880
CJK UNIFIED IDEOGRAPH:'D5C8:54728:&#x6871
CJK UNIFIED IDEOGRAPH:'D5C9:54729:&#x687E
CJK UNIFIED IDEOGRAPH:'D5CA:54730:&#x689B
CJK UNIFIED IDEOGRAPH:'D5CB:54731:&#x6896
CJK UNIFIED IDEOGRAPH:'D5CC:54732:&#x688B
CJK UNIFIED IDEOGRAPH:'D5CD:54733:&#x68A0
CJK UNIFIED IDEOGRAPH:'D5CE:54734:&#x6889
CJK UNIFIED IDEOGRAPH:'D5CF:54735:&#x68A4
CJK UNIFIED IDEOGRAPH:'D5D0:54736:&#x6878
CJK UNIFIED IDEOGRAPH:'D5D1:54737:&#x687B
CJK UNIFIED IDEOGRAPH:'D5D2:54738:&#x6891
CJK UNIFIED IDEOGRAPH:'D5D3:54739:&#x688C
CJK UNIFIED IDEOGRAPH:'D5D4:54740:&#x688A
CJK UNIFIED IDEOGRAPH:'D5D5:54741:&#x687D
CJK UNIFIED IDEOGRAPH:'D5D6:54742:&#x6B36
CJK UNIFIED IDEOGRAPH:'D5D7:54743:&#x6B33
CJK UNIFIED IDEOGRAPH:'D5D8:54744:&#x6B37
CJK UNIFIED IDEOGRAPH:'D5D9:54745:&#x6B38
CJK UNIFIED IDEOGRAPH:'D5DA:54746:&#x6B91
CJK UNIFIED IDEOGRAPH:'D5DB:54747:&#x6B8F
CJK UNIFIED IDEOGRAPH:'D5DC:54748:&#x6B8D
CJK UNIFIED IDEOGRAPH:'D5DD:54749:&#x6B8E
CJK UNIFIED IDEOGRAPH:'D5DE:54750:&#x6B8C
CJK UNIFIED IDEOGRAPH:'D5DF:54751:&#x6C2A
CJK UNIFIED IDEOGRAPH:'D5E0:54752:&#x6DC0
CJK UNIFIED IDEOGRAPH:'D5E1:54753:&#x6DAB
CJK UNIFIED IDEOGRAPH:'D5E2:54754:&#x6DB4
CJK UNIFIED IDEOGRAPH:'D5E3:54755:&#x6DB3
CJK UNIFIED IDEOGRAPH:'D5E4:54756:&#x6E74
CJK UNIFIED IDEOGRAPH:'D5E5:54757:&#x6DAC
CJK UNIFIED IDEOGRAPH:'D5E6:54758:&#x6DE9
CJK UNIFIED IDEOGRAPH:'D5E7:54759:&#x6DE2
CJK UNIFIED IDEOGRAPH:'D5E8:54760:&#x6DB7
CJK UNIFIED IDEOGRAPH:'D5E9:54761:&#x6DF6
CJK UNIFIED IDEOGRAPH:'D5EA:54762:&#x6DD4
CJK UNIFIED IDEOGRAPH:'D5EB:54763:&#x6E00
CJK UNIFIED IDEOGRAPH:'D5EC:54764:&#x6DC8
CJK UNIFIED IDEOGRAPH:'D5ED:54765:&#x6DE0
CJK UNIFIED IDEOGRAPH:'D5EE:54766:&#x6DDF
CJK UNIFIED IDEOGRAPH:'D5EF:54767:&#x6DD6
CJK UNIFIED IDEOGRAPH:'D5F0:54768:&#x6DBE
CJK UNIFIED IDEOGRAPH:'D5F1:54769:&#x6DE5
CJK UNIFIED IDEOGRAPH:'D5F2:54770:&#x6DDC
CJK UNIFIED IDEOGRAPH:'D5F3:54771:&#x6DDD
CJK UNIFIED IDEOGRAPH:'D5F4:54772:&#x6DDB
CJK UNIFIED IDEOGRAPH:'D5F5:54773:&#x6DF4
CJK UNIFIED IDEOGRAPH:'D5F6:54774:&#x6DCA
CJK UNIFIED IDEOGRAPH:'D5F7:54775:&#x6DBD
CJK UNIFIED IDEOGRAPH:'D5F8:54776:&#x6DED
CJK UNIFIED IDEOGRAPH:'D5F9:54777:&#x6DF0
CJK UNIFIED IDEOGRAPH:'D5FA:54778:&#x6DBA
CJK UNIFIED IDEOGRAPH:'D5FB:54779:&#x6DD5
CJK UNIFIED IDEOGRAPH:'D5FC:54780:&#x6DC2
CJK UNIFIED IDEOGRAPH:'D5FD:54781:&#x6DCF
CJK UNIFIED IDEOGRAPH:'D5FE:54782:&#x6DC9
CJK UNIFIED IDEOGRAPH:'D640:54848:&#x6DD0
CJK UNIFIED IDEOGRAPH:'D641:54849:&#x6DF2
CJK UNIFIED IDEOGRAPH:'D642:54850:&#x6DD3
CJK UNIFIED IDEOGRAPH:'D643:54851:&#x6DFD
CJK UNIFIED IDEOGRAPH:'D644:54852:&#x6DD7
CJK UNIFIED IDEOGRAPH:'D645:54853:&#x6DCD
CJK UNIFIED IDEOGRAPH:'D646:54854:&#x6DE3
CJK UNIFIED IDEOGRAPH:'D647:54855:&#x6DBB
CJK UNIFIED IDEOGRAPH:'D648:54856:&#x70FA
CJK UNIFIED IDEOGRAPH:'D649:54857:&#x710D
CJK UNIFIED IDEOGRAPH:'D64A:54858:&#x70F7
CJK UNIFIED IDEOGRAPH:'D64B:54859:&#x7117
CJK UNIFIED IDEOGRAPH:'D64C:54860:&#x70F4
CJK UNIFIED IDEOGRAPH:'D64D:54861:&#x710C
CJK UNIFIED IDEOGRAPH:'D64E:54862:&#x70F0
CJK UNIFIED IDEOGRAPH:'D64F:54863:&#x7104
CJK UNIFIED IDEOGRAPH:'D650:54864:&#x70F3
CJK UNIFIED IDEOGRAPH:'D651:54865:&#x7110
CJK UNIFIED IDEOGRAPH:'D652:54866:&#x70FC
CJK UNIFIED IDEOGRAPH:'D653:54867:&#x70FF
CJK UNIFIED IDEOGRAPH:'D654:54868:&#x7106
CJK UNIFIED IDEOGRAPH:'D655:54869:&#x7113
CJK UNIFIED IDEOGRAPH:'D656:54870:&#x7100
CJK UNIFIED IDEOGRAPH:'D657:54871:&#x70F8
CJK UNIFIED IDEOGRAPH:'D658:54872:&#x70F6
CJK UNIFIED IDEOGRAPH:'D659:54873:&#x710B
CJK UNIFIED IDEOGRAPH:'D65A:54874:&#x7102
CJK UNIFIED IDEOGRAPH:'D65B:54875:&#x710E
CJK UNIFIED IDEOGRAPH:'D65C:54876:&#x727E
CJK UNIFIED IDEOGRAPH:'D65D:54877:&#x727B
CJK UNIFIED IDEOGRAPH:'D65E:54878:&#x727C
CJK UNIFIED IDEOGRAPH:'D65F:54879:&#x727F
CJK UNIFIED IDEOGRAPH:'D660:54880:&#x731D
CJK UNIFIED IDEOGRAPH:'D661:54881:&#x7317
CJK UNIFIED IDEOGRAPH:'D662:54882:&#x7307
CJK UNIFIED IDEOGRAPH:'D663:54883:&#x7311
CJK UNIFIED IDEOGRAPH:'D664:54884:&#x7318
CJK UNIFIED IDEOGRAPH:'D665:54885:&#x730A
CJK UNIFIED IDEOGRAPH:'D666:54886:&#x7308
CJK UNIFIED IDEOGRAPH:'D667:54887:&#x72FF
CJK UNIFIED IDEOGRAPH:'D668:54888:&#x730F
CJK UNIFIED IDEOGRAPH:'D669:54889:&#x731E
CJK UNIFIED IDEOGRAPH:'D66A:54890:&#x7388
CJK UNIFIED IDEOGRAPH:'D66B:54891:&#x73F6
CJK UNIFIED IDEOGRAPH:'D66C:54892:&#x73F8
CJK UNIFIED IDEOGRAPH:'D66D:54893:&#x73F5
CJK UNIFIED IDEOGRAPH:'D66E:54894:&#x7404
CJK UNIFIED IDEOGRAPH:'D66F:54895:&#x7401
CJK UNIFIED IDEOGRAPH:'D670:54896:&#x73FD
CJK UNIFIED IDEOGRAPH:'D671:54897:&#x7407
CJK UNIFIED IDEOGRAPH:'D672:54898:&#x7400
CJK UNIFIED IDEOGRAPH:'D673:54899:&#x73FA
CJK UNIFIED IDEOGRAPH:'D674:54900:&#x73FC
CJK UNIFIED IDEOGRAPH:'D675:54901:&#x73FF
CJK UNIFIED IDEOGRAPH:'D676:54902:&#x740C
CJK UNIFIED IDEOGRAPH:'D677:54903:&#x740B
CJK UNIFIED IDEOGRAPH:'D678:54904:&#x73F4
CJK UNIFIED IDEOGRAPH:'D679:54905:&#x7408
CJK UNIFIED IDEOGRAPH:'D67A:54906:&#x7564
CJK UNIFIED IDEOGRAPH:'D67B:54907:&#x7563
CJK UNIFIED IDEOGRAPH:'D67C:54908:&#x75CE
CJK UNIFIED IDEOGRAPH:'D67D:54909:&#x75D2
CJK UNIFIED IDEOGRAPH:'D67E:54910:&#x75CF
CJK UNIFIED IDEOGRAPH:'D6A1:54945:&#x75CB
CJK UNIFIED IDEOGRAPH:'D6A2:54946:&#x75CC
CJK UNIFIED IDEOGRAPH:'D6A3:54947:&#x75D1
CJK UNIFIED IDEOGRAPH:'D6A4:54948:&#x75D0
CJK UNIFIED IDEOGRAPH:'D6A5:54949:&#x768F
CJK UNIFIED IDEOGRAPH:'D6A6:54950:&#x7689
CJK UNIFIED IDEOGRAPH:'D6A7:54951:&#x76D3
CJK UNIFIED IDEOGRAPH:'D6A8:54952:&#x7739
CJK UNIFIED IDEOGRAPH:'D6A9:54953:&#x772F
CJK UNIFIED IDEOGRAPH:'D6AA:54954:&#x772D
CJK UNIFIED IDEOGRAPH:'D6AB:54955:&#x7731
CJK UNIFIED IDEOGRAPH:'D6AC:54956:&#x7732
CJK UNIFIED IDEOGRAPH:'D6AD:54957:&#x7734
CJK UNIFIED IDEOGRAPH:'D6AE:54958:&#x7733
CJK UNIFIED IDEOGRAPH:'D6AF:54959:&#x773D
CJK UNIFIED IDEOGRAPH:'D6B0:54960:&#x7725
CJK UNIFIED IDEOGRAPH:'D6B1:54961:&#x773B
CJK UNIFIED IDEOGRAPH:'D6B2:54962:&#x7735
CJK UNIFIED IDEOGRAPH:'D6B3:54963:&#x7848
CJK UNIFIED IDEOGRAPH:'D6B4:54964:&#x7852
CJK UNIFIED IDEOGRAPH:'D6B5:54965:&#x7849
CJK UNIFIED IDEOGRAPH:'D6B6:54966:&#x784D
CJK UNIFIED IDEOGRAPH:'D6B7:54967:&#x784A
CJK UNIFIED IDEOGRAPH:'D6B8:54968:&#x784C
CJK UNIFIED IDEOGRAPH:'D6B9:54969:&#x7826
CJK UNIFIED IDEOGRAPH:'D6BA:54970:&#x7845
CJK UNIFIED IDEOGRAPH:'D6BB:54971:&#x7850
CJK UNIFIED IDEOGRAPH:'D6BC:54972:&#x7964
CJK UNIFIED IDEOGRAPH:'D6BD:54973:&#x7967
CJK UNIFIED IDEOGRAPH:'D6BE:54974:&#x7969
CJK UNIFIED IDEOGRAPH:'D6BF:54975:&#x796A
CJK UNIFIED IDEOGRAPH:'D6C0:54976:&#x7963
CJK UNIFIED IDEOGRAPH:'D6C1:54977:&#x796B
CJK UNIFIED IDEOGRAPH:'D6C2:54978:&#x7961
CJK UNIFIED IDEOGRAPH:'D6C3:54979:&#x79BB
CJK UNIFIED IDEOGRAPH:'D6C4:54980:&#x79FA
CJK UNIFIED IDEOGRAPH:'D6C5:54981:&#x79F8
CJK UNIFIED IDEOGRAPH:'D6C6:54982:&#x79F6
CJK UNIFIED IDEOGRAPH:'D6C7:54983:&#x79F7
CJK UNIFIED IDEOGRAPH:'D6C8:54984:&#x7A8F
CJK UNIFIED IDEOGRAPH:'D6C9:54985:&#x7A94
CJK UNIFIED IDEOGRAPH:'D6CA:54986:&#x7A90
CJK UNIFIED IDEOGRAPH:'D6CB:54987:&#x7B35
CJK UNIFIED IDEOGRAPH:'D6CC:54988:&#x7B47
CJK UNIFIED IDEOGRAPH:'D6CD:54989:&#x7B34
CJK UNIFIED IDEOGRAPH:'D6CE:54990:&#x7B25
CJK UNIFIED IDEOGRAPH:'D6CF:54991:&#x7B30
CJK UNIFIED IDEOGRAPH:'D6D0:54992:&#x7B22
CJK UNIFIED IDEOGRAPH:'D6D1:54993:&#x7B24
CJK UNIFIED IDEOGRAPH:'D6D2:54994:&#x7B33
CJK UNIFIED IDEOGRAPH:'D6D3:54995:&#x7B18
CJK UNIFIED IDEOGRAPH:'D6D4:54996:&#x7B2A
CJK UNIFIED IDEOGRAPH:'D6D5:54997:&#x7B1D
CJK UNIFIED IDEOGRAPH:'D6D6:54998:&#x7B31
CJK UNIFIED IDEOGRAPH:'D6D7:54999:&#x7B2B
CJK UNIFIED IDEOGRAPH:'D6D8:55000:&#x7B2D
CJK UNIFIED IDEOGRAPH:'D6D9:55001:&#x7B2F
CJK UNIFIED IDEOGRAPH:'D6DA:55002:&#x7B32
CJK UNIFIED IDEOGRAPH:'D6DB:55003:&#x7B38
CJK UNIFIED IDEOGRAPH:'D6DC:55004:&#x7B1A
CJK UNIFIED IDEOGRAPH:'D6DD:55005:&#x7B23
CJK UNIFIED IDEOGRAPH:'D6DE:55006:&#x7C94
CJK UNIFIED IDEOGRAPH:'D6DF:55007:&#x7C98
CJK UNIFIED IDEOGRAPH:'D6E0:55008:&#x7C96
CJK UNIFIED IDEOGRAPH:'D6E1:55009:&#x7CA3
CJK UNIFIED IDEOGRAPH:'D6E2:55010:&#x7D35
CJK UNIFIED IDEOGRAPH:'D6E3:55011:&#x7D3D
CJK UNIFIED IDEOGRAPH:'D6E4:55012:&#x7D38
CJK UNIFIED IDEOGRAPH:'D6E5:55013:&#x7D36
CJK UNIFIED IDEOGRAPH:'D6E6:55014:&#x7D3A
CJK UNIFIED IDEOGRAPH:'D6E7:55015:&#x7D45
CJK UNIFIED IDEOGRAPH:'D6E8:55016:&#x7D2C
CJK UNIFIED IDEOGRAPH:'D6E9:55017:&#x7D29
CJK UNIFIED IDEOGRAPH:'D6EA:55018:&#x7D41
CJK UNIFIED IDEOGRAPH:'D6EB:55019:&#x7D47
CJK UNIFIED IDEOGRAPH:'D6EC:55020:&#x7D3E
CJK UNIFIED IDEOGRAPH:'D6ED:55021:&#x7D3F
CJK UNIFIED IDEOGRAPH:'D6EE:55022:&#x7D4A
CJK UNIFIED IDEOGRAPH:'D6EF:55023:&#x7D3B
CJK UNIFIED IDEOGRAPH:'D6F0:55024:&#x7D28
CJK UNIFIED IDEOGRAPH:'D6F1:55025:&#x7F63
CJK UNIFIED IDEOGRAPH:'D6F2:55026:&#x7F95
CJK UNIFIED IDEOGRAPH:'D6F3:55027:&#x7F9C
CJK UNIFIED IDEOGRAPH:'D6F4:55028:&#x7F9D
CJK UNIFIED IDEOGRAPH:'D6F5:55029:&#x7F9B
CJK UNIFIED IDEOGRAPH:'D6F6:55030:&#x7FCA
CJK UNIFIED IDEOGRAPH:'D6F7:55031:&#x7FCB
CJK UNIFIED IDEOGRAPH:'D6F8:55032:&#x7FCD
CJK UNIFIED IDEOGRAPH:'D6F9:55033:&#x7FD0
CJK UNIFIED IDEOGRAPH:'D6FA:55034:&#x7FD1
CJK UNIFIED IDEOGRAPH:'D6FB:55035:&#x7FC7
CJK UNIFIED IDEOGRAPH:'D6FC:55036:&#x7FCF
CJK UNIFIED IDEOGRAPH:'D6FD:55037:&#x7FC9
CJK UNIFIED IDEOGRAPH:'D6FE:55038:&#x801F
CJK UNIFIED IDEOGRAPH:'D740:55104:&#x801E
CJK UNIFIED IDEOGRAPH:'D741:55105:&#x801B
CJK UNIFIED IDEOGRAPH:'D742:55106:&#x8047
CJK UNIFIED IDEOGRAPH:'D743:55107:&#x8043
CJK UNIFIED IDEOGRAPH:'D744:55108:&#x8048
CJK UNIFIED IDEOGRAPH:'D745:55109:&#x8118
CJK UNIFIED IDEOGRAPH:'D746:55110:&#x8125
CJK UNIFIED IDEOGRAPH:'D747:55111:&#x8119
CJK UNIFIED IDEOGRAPH:'D748:55112:&#x811B
CJK UNIFIED IDEOGRAPH:'D749:55113:&#x812D
CJK UNIFIED IDEOGRAPH:'D74A:55114:&#x811F
CJK UNIFIED IDEOGRAPH:'D74B:55115:&#x812C
CJK UNIFIED IDEOGRAPH:'D74C:55116:&#x811E
CJK UNIFIED IDEOGRAPH:'D74D:55117:&#x8121
CJK UNIFIED IDEOGRAPH:'D74E:55118:&#x8115
CJK UNIFIED IDEOGRAPH:'D74F:55119:&#x8127
CJK UNIFIED IDEOGRAPH:'D750:55120:&#x811D
CJK UNIFIED IDEOGRAPH:'D751:55121:&#x8122
CJK UNIFIED IDEOGRAPH:'D752:55122:&#x8211
CJK UNIFIED IDEOGRAPH:'D753:55123:&#x8238
CJK UNIFIED IDEOGRAPH:'D754:55124:&#x8233
CJK UNIFIED IDEOGRAPH:'D755:55125:&#x823A
CJK UNIFIED IDEOGRAPH:'D756:55126:&#x8234
CJK UNIFIED IDEOGRAPH:'D757:55127:&#x8232
CJK UNIFIED IDEOGRAPH:'D758:55128:&#x8274
CJK UNIFIED IDEOGRAPH:'D759:55129:&#x8390
CJK UNIFIED IDEOGRAPH:'D75A:55130:&#x83A3
CJK UNIFIED IDEOGRAPH:'D75B:55131:&#x83A8
CJK UNIFIED IDEOGRAPH:'D75C:55132:&#x838D
CJK UNIFIED IDEOGRAPH:'D75D:55133:&#x837A
CJK UNIFIED IDEOGRAPH:'D75E:55134:&#x8373
CJK UNIFIED IDEOGRAPH:'D75F:55135:&#x83A4
CJK UNIFIED IDEOGRAPH:'D760:55136:&#x8374
CJK UNIFIED IDEOGRAPH:'D761:55137:&#x838F
CJK UNIFIED IDEOGRAPH:'D762:55138:&#x8381
CJK UNIFIED IDEOGRAPH:'D763:55139:&#x8395
CJK UNIFIED IDEOGRAPH:'D764:55140:&#x8399
CJK UNIFIED IDEOGRAPH:'D765:55141:&#x8375
CJK UNIFIED IDEOGRAPH:'D766:55142:&#x8394
CJK UNIFIED IDEOGRAPH:'D767:55143:&#x83A9
CJK UNIFIED IDEOGRAPH:'D768:55144:&#x837D
CJK UNIFIED IDEOGRAPH:'D769:55145:&#x8383
CJK UNIFIED IDEOGRAPH:'D76A:55146:&#x838C
CJK UNIFIED IDEOGRAPH:'D76B:55147:&#x839D
CJK UNIFIED IDEOGRAPH:'D76C:55148:&#x839B
CJK UNIFIED IDEOGRAPH:'D76D:55149:&#x83AA
CJK UNIFIED IDEOGRAPH:'D76E:55150:&#x838B
CJK UNIFIED IDEOGRAPH:'D76F:55151:&#x837E
CJK UNIFIED IDEOGRAPH:'D770:55152:&#x83A5
CJK UNIFIED IDEOGRAPH:'D771:55153:&#x83AF
CJK UNIFIED IDEOGRAPH:'D772:55154:&#x8388
CJK UNIFIED IDEOGRAPH:'D773:55155:&#x8397
CJK UNIFIED IDEOGRAPH:'D774:55156:&#x83B0
CJK UNIFIED IDEOGRAPH:'D775:55157:&#x837F
CJK UNIFIED IDEOGRAPH:'D776:55158:&#x83A6
CJK UNIFIED IDEOGRAPH:'D777:55159:&#x8387
CJK UNIFIED IDEOGRAPH:'D778:55160:&#x83AE
CJK UNIFIED IDEOGRAPH:'D779:55161:&#x8376
CJK UNIFIED IDEOGRAPH:'D77A:55162:&#x839A
CJK UNIFIED IDEOGRAPH:'D77B:55163:&#x8659
CJK UNIFIED IDEOGRAPH:'D77C:55164:&#x8656
CJK UNIFIED IDEOGRAPH:'D77D:55165:&#x86BF
CJK UNIFIED IDEOGRAPH:'D77E:55166:&#x86B7
CJK UNIFIED IDEOGRAPH:'D7A1:55201:&#x86C2
CJK UNIFIED IDEOGRAPH:'D7A2:55202:&#x86C1
CJK UNIFIED IDEOGRAPH:'D7A3:55203:&#x86C5
CJK UNIFIED IDEOGRAPH:'D7A4:55204:&#x86BA
CJK UNIFIED IDEOGRAPH:'D7A5:55205:&#x86B0
CJK UNIFIED IDEOGRAPH:'D7A6:55206:&#x86C8
CJK UNIFIED IDEOGRAPH:'D7A7:55207:&#x86B9
CJK UNIFIED IDEOGRAPH:'D7A8:55208:&#x86B3
CJK UNIFIED IDEOGRAPH:'D7A9:55209:&#x86B8
CJK UNIFIED IDEOGRAPH:'D7AA:55210:&#x86CC
CJK UNIFIED IDEOGRAPH:'D7AB:55211:&#x86B4
CJK UNIFIED IDEOGRAPH:'D7AC:55212:&#x86BB
CJK UNIFIED IDEOGRAPH:'D7AD:55213:&#x86BC
CJK UNIFIED IDEOGRAPH:'D7AE:55214:&#x86C3
CJK UNIFIED IDEOGRAPH:'D7AF:55215:&#x86BD
CJK UNIFIED IDEOGRAPH:'D7B0:55216:&#x86BE
CJK UNIFIED IDEOGRAPH:'D7B1:55217:&#x8852
CJK UNIFIED IDEOGRAPH:'D7B2:55218:&#x8889
CJK UNIFIED IDEOGRAPH:'D7B3:55219:&#x8895
CJK UNIFIED IDEOGRAPH:'D7B4:55220:&#x88A8
CJK UNIFIED IDEOGRAPH:'D7B5:55221:&#x88A2
CJK UNIFIED IDEOGRAPH:'D7B6:55222:&#x88AA
CJK UNIFIED IDEOGRAPH:'D7B7:55223:&#x889A
CJK UNIFIED IDEOGRAPH:'D7B8:55224:&#x8891
CJK UNIFIED IDEOGRAPH:'D7B9:55225:&#x88A1
CJK UNIFIED IDEOGRAPH:'D7BA:55226:&#x889F
CJK UNIFIED IDEOGRAPH:'D7BB:55227:&#x8898
CJK UNIFIED IDEOGRAPH:'D7BC:55228:&#x88A7
CJK UNIFIED IDEOGRAPH:'D7BD:55229:&#x8899
CJK UNIFIED IDEOGRAPH:'D7BE:55230:&#x889B
CJK UNIFIED IDEOGRAPH:'D7BF:55231:&#x8897
CJK UNIFIED IDEOGRAPH:'D7C0:55232:&#x88A4
CJK UNIFIED IDEOGRAPH:'D7C1:55233:&#x88AC
CJK UNIFIED IDEOGRAPH:'D7C2:55234:&#x888C
CJK UNIFIED IDEOGRAPH:'D7C3:55235:&#x8893
CJK UNIFIED IDEOGRAPH:'D7C4:55236:&#x888E
CJK UNIFIED IDEOGRAPH:'D7C5:55237:&#x8982
CJK UNIFIED IDEOGRAPH:'D7C6:55238:&#x89D6
CJK UNIFIED IDEOGRAPH:'D7C7:55239:&#x89D9
CJK UNIFIED IDEOGRAPH:'D7C8:55240:&#x89D5
CJK UNIFIED IDEOGRAPH:'D7C9:55241:&#x8A30
CJK UNIFIED IDEOGRAPH:'D7CA:55242:&#x8A27
CJK UNIFIED IDEOGRAPH:'D7CB:55243:&#x8A2C
CJK UNIFIED IDEOGRAPH:'D7CC:55244:&#x8A1E
CJK UNIFIED IDEOGRAPH:'D7CD:55245:&#x8C39
CJK UNIFIED IDEOGRAPH:'D7CE:55246:&#x8C3B
CJK UNIFIED IDEOGRAPH:'D7CF:55247:&#x8C5C
CJK UNIFIED IDEOGRAPH:'D7D0:55248:&#x8C5D
CJK UNIFIED IDEOGRAPH:'D7D1:55249:&#x8C7D
CJK UNIFIED IDEOGRAPH:'D7D2:55250:&#x8CA5
CJK UNIFIED IDEOGRAPH:'D7D3:55251:&#x8D7D
CJK UNIFIED IDEOGRAPH:'D7D4:55252:&#x8D7B
CJK UNIFIED IDEOGRAPH:'D7D5:55253:&#x8D79
CJK UNIFIED IDEOGRAPH:'D7D6:55254:&#x8DBC
CJK UNIFIED IDEOGRAPH:'D7D7:55255:&#x8DC2
CJK UNIFIED IDEOGRAPH:'D7D8:55256:&#x8DB9
CJK UNIFIED IDEOGRAPH:'D7D9:55257:&#x8DBF
CJK UNIFIED IDEOGRAPH:'D7DA:55258:&#x8DC1
CJK UNIFIED IDEOGRAPH:'D7DB:55259:&#x8ED8
CJK UNIFIED IDEOGRAPH:'D7DC:55260:&#x8EDE
CJK UNIFIED IDEOGRAPH:'D7DD:55261:&#x8EDD
CJK UNIFIED IDEOGRAPH:'D7DE:55262:&#x8EDC
CJK UNIFIED IDEOGRAPH:'D7DF:55263:&#x8ED7
CJK UNIFIED IDEOGRAPH:'D7E0:55264:&#x8EE0
CJK UNIFIED IDEOGRAPH:'D7E1:55265:&#x8EE1
CJK UNIFIED IDEOGRAPH:'D7E2:55266:&#x9024
CJK UNIFIED IDEOGRAPH:'D7E3:55267:&#x900B
CJK UNIFIED IDEOGRAPH:'D7E4:55268:&#x9011
CJK UNIFIED IDEOGRAPH:'D7E5:55269:&#x901C
CJK UNIFIED IDEOGRAPH:'D7E6:55270:&#x900C
CJK UNIFIED IDEOGRAPH:'D7E7:55271:&#x9021
CJK UNIFIED IDEOGRAPH:'D7E8:55272:&#x90EF
CJK UNIFIED IDEOGRAPH:'D7E9:55273:&#x90EA
CJK UNIFIED IDEOGRAPH:'D7EA:55274:&#x90F0
CJK UNIFIED IDEOGRAPH:'D7EB:55275:&#x90F4
CJK UNIFIED IDEOGRAPH:'D7EC:55276:&#x90F2
CJK UNIFIED IDEOGRAPH:'D7ED:55277:&#x90F3
CJK UNIFIED IDEOGRAPH:'D7EE:55278:&#x90D4
CJK UNIFIED IDEOGRAPH:'D7EF:55279:&#x90EB
CJK UNIFIED IDEOGRAPH:'D7F0:55280:&#x90EC
CJK UNIFIED IDEOGRAPH:'D7F1:55281:&#x90E9
CJK UNIFIED IDEOGRAPH:'D7F2:55282:&#x9156
CJK UNIFIED IDEOGRAPH:'D7F3:55283:&#x9158
CJK UNIFIED IDEOGRAPH:'D7F4:55284:&#x915A
CJK UNIFIED IDEOGRAPH:'D7F5:55285:&#x9153
CJK UNIFIED IDEOGRAPH:'D7F6:55286:&#x9155
CJK UNIFIED IDEOGRAPH:'D7F7:55287:&#x91EC
CJK UNIFIED IDEOGRAPH:'D7F8:55288:&#x91F4
CJK UNIFIED IDEOGRAPH:'D7F9:55289:&#x91F1
CJK UNIFIED IDEOGRAPH:'D7FA:55290:&#x91F3
CJK UNIFIED IDEOGRAPH:'D7FB:55291:&#x91F8
CJK UNIFIED IDEOGRAPH:'D7FC:55292:&#x91E4
CJK UNIFIED IDEOGRAPH:'D7FD:55293:&#x91F9
CJK UNIFIED IDEOGRAPH:'D7FE:55294:&#x91EA
CJK UNIFIED IDEOGRAPH:'D840:55360:&#x91EB
CJK UNIFIED IDEOGRAPH:'D841:55361:&#x91F7
CJK UNIFIED IDEOGRAPH:'D842:55362:&#x91E8
CJK UNIFIED IDEOGRAPH:'D843:55363:&#x91EE
CJK UNIFIED IDEOGRAPH:'D844:55364:&#x957A
CJK UNIFIED IDEOGRAPH:'D845:55365:&#x9586
CJK UNIFIED IDEOGRAPH:'D846:55366:&#x9588
CJK UNIFIED IDEOGRAPH:'D847:55367:&#x967C
CJK UNIFIED IDEOGRAPH:'D848:55368:&#x966D
CJK UNIFIED IDEOGRAPH:'D849:55369:&#x966B
CJK UNIFIED IDEOGRAPH:'D84A:55370:&#x9671
CJK UNIFIED IDEOGRAPH:'D84B:55371:&#x966F
CJK UNIFIED IDEOGRAPH:'D84C:55372:&#x96BF
CJK UNIFIED IDEOGRAPH:'D84D:55373:&#x976A
CJK UNIFIED IDEOGRAPH:'D84E:55374:&#x9804
CJK UNIFIED IDEOGRAPH:'D84F:55375:&#x98E5
CJK UNIFIED IDEOGRAPH:'D850:55376:&#x9997
CJK UNIFIED IDEOGRAPH:'D851:55377:&#x509B
CJK UNIFIED IDEOGRAPH:'D852:55378:&#x5095
CJK UNIFIED IDEOGRAPH:'D853:55379:&#x5094
CJK UNIFIED IDEOGRAPH:'D854:55380:&#x509E
CJK UNIFIED IDEOGRAPH:'D855:55381:&#x508B
CJK UNIFIED IDEOGRAPH:'D856:55382:&#x50A3
CJK UNIFIED IDEOGRAPH:'D857:55383:&#x5083
CJK UNIFIED IDEOGRAPH:'D858:55384:&#x508C
CJK UNIFIED IDEOGRAPH:'D859:55385:&#x508E
CJK UNIFIED IDEOGRAPH:'D85A:55386:&#x509D
CJK UNIFIED IDEOGRAPH:'D85B:55387:&#x5068
CJK UNIFIED IDEOGRAPH:'D85C:55388:&#x509C
CJK UNIFIED IDEOGRAPH:'D85D:55389:&#x5092
CJK UNIFIED IDEOGRAPH:'D85E:55390:&#x5082
CJK UNIFIED IDEOGRAPH:'D85F:55391:&#x5087
CJK UNIFIED IDEOGRAPH:'D860:55392:&#x515F
CJK UNIFIED IDEOGRAPH:'D861:55393:&#x51D4
CJK UNIFIED IDEOGRAPH:'D862:55394:&#x5312
CJK UNIFIED IDEOGRAPH:'D863:55395:&#x5311
CJK UNIFIED IDEOGRAPH:'D864:55396:&#x53A4
CJK UNIFIED IDEOGRAPH:'D865:55397:&#x53A7
CJK UNIFIED IDEOGRAPH:'D866:55398:&#x5591
CJK UNIFIED IDEOGRAPH:'D867:55399:&#x55A8
CJK UNIFIED IDEOGRAPH:'D868:55400:&#x55A5
CJK UNIFIED IDEOGRAPH:'D869:55401:&#x55AD
CJK UNIFIED IDEOGRAPH:'D86A:55402:&#x5577
CJK UNIFIED IDEOGRAPH:'D86B:55403:&#x5645
CJK UNIFIED IDEOGRAPH:'D86C:55404:&#x55A2
CJK UNIFIED IDEOGRAPH:'D86D:55405:&#x5593
CJK UNIFIED IDEOGRAPH:'D86E:55406:&#x5588
CJK UNIFIED IDEOGRAPH:'D86F:55407:&#x558F
CJK UNIFIED IDEOGRAPH:'D870:55408:&#x55B5
CJK UNIFIED IDEOGRAPH:'D871:55409:&#x5581
CJK UNIFIED IDEOGRAPH:'D872:55410:&#x55A3
CJK UNIFIED IDEOGRAPH:'D873:55411:&#x5592
CJK UNIFIED IDEOGRAPH:'D874:55412:&#x55A4
CJK UNIFIED IDEOGRAPH:'D875:55413:&#x557D
CJK UNIFIED IDEOGRAPH:'D876:55414:&#x558C
CJK UNIFIED IDEOGRAPH:'D877:55415:&#x55A6
CJK UNIFIED IDEOGRAPH:'D878:55416:&#x557F
CJK UNIFIED IDEOGRAPH:'D879:55417:&#x5595
CJK UNIFIED IDEOGRAPH:'D87A:55418:&#x55A1
CJK UNIFIED IDEOGRAPH:'D87B:55419:&#x558E
CJK UNIFIED IDEOGRAPH:'D87C:55420:&#x570C
CJK UNIFIED IDEOGRAPH:'D87D:55421:&#x5829
CJK UNIFIED IDEOGRAPH:'D87E:55422:&#x5837
CJK UNIFIED IDEOGRAPH:'D8A1:55457:&#x5819
CJK UNIFIED IDEOGRAPH:'D8A2:55458:&#x581E
CJK UNIFIED IDEOGRAPH:'D8A3:55459:&#x5827
CJK UNIFIED IDEOGRAPH:'D8A4:55460:&#x5823
CJK UNIFIED IDEOGRAPH:'D8A5:55461:&#x5828
CJK UNIFIED IDEOGRAPH:'D8A6:55462:&#x57F5
CJK UNIFIED IDEOGRAPH:'D8A7:55463:&#x5848
CJK UNIFIED IDEOGRAPH:'D8A8:55464:&#x5825
CJK UNIFIED IDEOGRAPH:'D8A9:55465:&#x581C
CJK UNIFIED IDEOGRAPH:'D8AA:55466:&#x581B
CJK UNIFIED IDEOGRAPH:'D8AB:55467:&#x5833
CJK UNIFIED IDEOGRAPH:'D8AC:55468:&#x583F
CJK UNIFIED IDEOGRAPH:'D8AD:55469:&#x5836
CJK UNIFIED IDEOGRAPH:'D8AE:55470:&#x582E
CJK UNIFIED IDEOGRAPH:'D8AF:55471:&#x5839
CJK UNIFIED IDEOGRAPH:'D8B0:55472:&#x5838
CJK UNIFIED IDEOGRAPH:'D8B1:55473:&#x582D
CJK UNIFIED IDEOGRAPH:'D8B2:55474:&#x582C
CJK UNIFIED IDEOGRAPH:'D8B3:55475:&#x583B
CJK UNIFIED IDEOGRAPH:'D8B4:55476:&#x5961
CJK UNIFIED IDEOGRAPH:'D8B5:55477:&#x5AAF
CJK UNIFIED IDEOGRAPH:'D8B6:55478:&#x5A94
CJK UNIFIED IDEOGRAPH:'D8B7:55479:&#x5A9F
CJK UNIFIED IDEOGRAPH:'D8B8:55480:&#x5A7A
CJK UNIFIED IDEOGRAPH:'D8B9:55481:&#x5AA2
CJK UNIFIED IDEOGRAPH:'D8BA:55482:&#x5A9E
CJK UNIFIED IDEOGRAPH:'D8BB:55483:&#x5A78
CJK UNIFIED IDEOGRAPH:'D8BC:55484:&#x5AA6
CJK UNIFIED IDEOGRAPH:'D8BD:55485:&#x5A7C
CJK UNIFIED IDEOGRAPH:'D8BE:55486:&#x5AA5
CJK UNIFIED IDEOGRAPH:'D8BF:55487:&#x5AAC
CJK UNIFIED IDEOGRAPH:'D8C0:55488:&#x5A95
CJK UNIFIED IDEOGRAPH:'D8C1:55489:&#x5AAE
CJK UNIFIED IDEOGRAPH:'D8C2:55490:&#x5A37
CJK UNIFIED IDEOGRAPH:'D8C3:55491:&#x5A84
CJK UNIFIED IDEOGRAPH:'D8C4:55492:&#x5A8A
CJK UNIFIED IDEOGRAPH:'D8C5:55493:&#x5A97
CJK UNIFIED IDEOGRAPH:'D8C6:55494:&#x5A83
CJK UNIFIED IDEOGRAPH:'D8C7:55495:&#x5A8B
CJK UNIFIED IDEOGRAPH:'D8C8:55496:&#x5AA9
CJK UNIFIED IDEOGRAPH:'D8C9:55497:&#x5A7B
CJK UNIFIED IDEOGRAPH:'D8CA:55498:&#x5A7D
CJK UNIFIED IDEOGRAPH:'D8CB:55499:&#x5A8C
CJK UNIFIED IDEOGRAPH:'D8CC:55500:&#x5A9C
CJK UNIFIED IDEOGRAPH:'D8CD:55501:&#x5A8F
CJK UNIFIED IDEOGRAPH:'D8CE:55502:&#x5A93
CJK UNIFIED IDEOGRAPH:'D8CF:55503:&#x5A9D
CJK UNIFIED IDEOGRAPH:'D8D0:55504:&#x5BEA
CJK UNIFIED IDEOGRAPH:'D8D1:55505:&#x5BCD
CJK UNIFIED IDEOGRAPH:'D8D2:55506:&#x5BCB
CJK UNIFIED IDEOGRAPH:'D8D3:55507:&#x5BD4
CJK UNIFIED IDEOGRAPH:'D8D4:55508:&#x5BD1
CJK UNIFIED IDEOGRAPH:'D8D5:55509:&#x5BCA
CJK UNIFIED IDEOGRAPH:'D8D6:55510:&#x5BCE
CJK UNIFIED IDEOGRAPH:'D8D7:55511:&#x5C0C
CJK UNIFIED IDEOGRAPH:'D8D8:55512:&#x5C30
CJK UNIFIED IDEOGRAPH:'D8D9:55513:&#x5D37
CJK UNIFIED IDEOGRAPH:'D8DA:55514:&#x5D43
CJK UNIFIED IDEOGRAPH:'D8DB:55515:&#x5D6B
CJK UNIFIED IDEOGRAPH:'D8DC:55516:&#x5D41
CJK UNIFIED IDEOGRAPH:'D8DD:55517:&#x5D4B
CJK UNIFIED IDEOGRAPH:'D8DE:55518:&#x5D3F
CJK UNIFIED IDEOGRAPH:'D8DF:55519:&#x5D35
CJK UNIFIED IDEOGRAPH:'D8E0:55520:&#x5D51
CJK UNIFIED IDEOGRAPH:'D8E1:55521:&#x5D4E
CJK UNIFIED IDEOGRAPH:'D8E2:55522:&#x5D55
CJK UNIFIED IDEOGRAPH:'D8E3:55523:&#x5D33
CJK UNIFIED IDEOGRAPH:'D8E4:55524:&#x5D3A
CJK UNIFIED IDEOGRAPH:'D8E5:55525:&#x5D52
CJK UNIFIED IDEOGRAPH:'D8E6:55526:&#x5D3D
CJK UNIFIED IDEOGRAPH:'D8E7:55527:&#x5D31
CJK UNIFIED IDEOGRAPH:'D8E8:55528:&#x5D59
CJK UNIFIED IDEOGRAPH:'D8E9:55529:&#x5D42
CJK UNIFIED IDEOGRAPH:'D8EA:55530:&#x5D39
CJK UNIFIED IDEOGRAPH:'D8EB:55531:&#x5D49
CJK UNIFIED IDEOGRAPH:'D8EC:55532:&#x5D38
CJK UNIFIED IDEOGRAPH:'D8ED:55533:&#x5D3C
CJK UNIFIED IDEOGRAPH:'D8EE:55534:&#x5D32
CJK UNIFIED IDEOGRAPH:'D8EF:55535:&#x5D36
CJK UNIFIED IDEOGRAPH:'D8F0:55536:&#x5D40
CJK UNIFIED IDEOGRAPH:'D8F1:55537:&#x5D45
CJK UNIFIED IDEOGRAPH:'D8F2:55538:&#x5E44
CJK UNIFIED IDEOGRAPH:'D8F3:55539:&#x5E41
CJK UNIFIED IDEOGRAPH:'D8F4:55540:&#x5F58
CJK UNIFIED IDEOGRAPH:'D8F5:55541:&#x5FA6
CJK UNIFIED IDEOGRAPH:'D8F6:55542:&#x5FA5
CJK UNIFIED IDEOGRAPH:'D8F7:55543:&#x5FAB
CJK UNIFIED IDEOGRAPH:'D8F8:55544:&#x60C9
CJK UNIFIED IDEOGRAPH:'D8F9:55545:&#x60B9
CJK UNIFIED IDEOGRAPH:'D8FA:55546:&#x60CC
CJK UNIFIED IDEOGRAPH:'D8FB:55547:&#x60E2
CJK UNIFIED IDEOGRAPH:'D8FC:55548:&#x60CE
CJK UNIFIED IDEOGRAPH:'D8FD:55549:&#x60C4
CJK UNIFIED IDEOGRAPH:'D8FE:55550:&#x6114
CJK UNIFIED IDEOGRAPH:'D940:55616:&#x60F2
CJK UNIFIED IDEOGRAPH:'D941:55617:&#x610A
CJK UNIFIED IDEOGRAPH:'D942:55618:&#x6116
CJK UNIFIED IDEOGRAPH:'D943:55619:&#x6105
CJK UNIFIED IDEOGRAPH:'D944:55620:&#x60F5
CJK UNIFIED IDEOGRAPH:'D945:55621:&#x6113
CJK UNIFIED IDEOGRAPH:'D946:55622:&#x60F8
CJK UNIFIED IDEOGRAPH:'D947:55623:&#x60FC
CJK UNIFIED IDEOGRAPH:'D948:55624:&#x60FE
CJK UNIFIED IDEOGRAPH:'D949:55625:&#x60C1
CJK UNIFIED IDEOGRAPH:'D94A:55626:&#x6103
CJK UNIFIED IDEOGRAPH:'D94B:55627:&#x6118
CJK UNIFIED IDEOGRAPH:'D94C:55628:&#x611D
CJK UNIFIED IDEOGRAPH:'D94D:55629:&#x6110
CJK UNIFIED IDEOGRAPH:'D94E:55630:&#x60FF
CJK UNIFIED IDEOGRAPH:'D94F:55631:&#x6104
CJK UNIFIED IDEOGRAPH:'D950:55632:&#x610B
CJK UNIFIED IDEOGRAPH:'D951:55633:&#x624A
CJK UNIFIED IDEOGRAPH:'D952:55634:&#x6394
CJK UNIFIED IDEOGRAPH:'D953:55635:&#x63B1
CJK UNIFIED IDEOGRAPH:'D954:55636:&#x63B0
CJK UNIFIED IDEOGRAPH:'D955:55637:&#x63CE
CJK UNIFIED IDEOGRAPH:'D956:55638:&#x63E5
CJK UNIFIED IDEOGRAPH:'D957:55639:&#x63E8
CJK UNIFIED IDEOGRAPH:'D958:55640:&#x63EF
CJK UNIFIED IDEOGRAPH:'D959:55641:&#x63C3
CJK UNIFIED IDEOGRAPH:'D95A:55642:&#x649D
CJK UNIFIED IDEOGRAPH:'D95B:55643:&#x63F3
CJK UNIFIED IDEOGRAPH:'D95C:55644:&#x63CA
CJK UNIFIED IDEOGRAPH:'D95D:55645:&#x63E0
CJK UNIFIED IDEOGRAPH:'D95E:55646:&#x63F6
CJK UNIFIED IDEOGRAPH:'D95F:55647:&#x63D5
CJK UNIFIED IDEOGRAPH:'D960:55648:&#x63F2
CJK UNIFIED IDEOGRAPH:'D961:55649:&#x63F5
CJK UNIFIED IDEOGRAPH:'D962:55650:&#x6461
CJK UNIFIED IDEOGRAPH:'D963:55651:&#x63DF
CJK UNIFIED IDEOGRAPH:'D964:55652:&#x63BE
CJK UNIFIED IDEOGRAPH:'D965:55653:&#x63DD
CJK UNIFIED IDEOGRAPH:'D966:55654:&#x63DC
CJK UNIFIED IDEOGRAPH:'D967:55655:&#x63C4
CJK UNIFIED IDEOGRAPH:'D968:55656:&#x63D8
CJK UNIFIED IDEOGRAPH:'D969:55657:&#x63D3
CJK UNIFIED IDEOGRAPH:'D96A:55658:&#x63C2
CJK UNIFIED IDEOGRAPH:'D96B:55659:&#x63C7
CJK UNIFIED IDEOGRAPH:'D96C:55660:&#x63CC
CJK UNIFIED IDEOGRAPH:'D96D:55661:&#x63CB
CJK UNIFIED IDEOGRAPH:'D96E:55662:&#x63C8
CJK UNIFIED IDEOGRAPH:'D96F:55663:&#x63F0
CJK UNIFIED IDEOGRAPH:'D970:55664:&#x63D7
CJK UNIFIED IDEOGRAPH:'D971:55665:&#x63D9
CJK UNIFIED IDEOGRAPH:'D972:55666:&#x6532
CJK UNIFIED IDEOGRAPH:'D973:55667:&#x6567
CJK UNIFIED IDEOGRAPH:'D974:55668:&#x656A
CJK UNIFIED IDEOGRAPH:'D975:55669:&#x6564
CJK UNIFIED IDEOGRAPH:'D976:55670:&#x655C
CJK UNIFIED IDEOGRAPH:'D977:55671:&#x6568
CJK UNIFIED IDEOGRAPH:'D978:55672:&#x6565
CJK UNIFIED IDEOGRAPH:'D979:55673:&#x658C
CJK UNIFIED IDEOGRAPH:'D97A:55674:&#x659D
CJK UNIFIED IDEOGRAPH:'D97B:55675:&#x659E
CJK UNIFIED IDEOGRAPH:'D97C:55676:&#x65AE
CJK UNIFIED IDEOGRAPH:'D97D:55677:&#x65D0
CJK UNIFIED IDEOGRAPH:'D97E:55678:&#x65D2
CJK UNIFIED IDEOGRAPH:'D9A1:55713:&#x667C
CJK UNIFIED IDEOGRAPH:'D9A2:55714:&#x666C
CJK UNIFIED IDEOGRAPH:'D9A3:55715:&#x667B
CJK UNIFIED IDEOGRAPH:'D9A4:55716:&#x6680
CJK UNIFIED IDEOGRAPH:'D9A5:55717:&#x6671
CJK UNIFIED IDEOGRAPH:'D9A6:55718:&#x6679
CJK UNIFIED IDEOGRAPH:'D9A7:55719:&#x666A
CJK UNIFIED IDEOGRAPH:'D9A8:55720:&#x6672
CJK UNIFIED IDEOGRAPH:'D9A9:55721:&#x6701
CJK UNIFIED IDEOGRAPH:'D9AA:55722:&#x690C
CJK UNIFIED IDEOGRAPH:'D9AB:55723:&#x68D3
CJK UNIFIED IDEOGRAPH:'D9AC:55724:&#x6904
CJK UNIFIED IDEOGRAPH:'D9AD:55725:&#x68DC
CJK UNIFIED IDEOGRAPH:'D9AE:55726:&#x692A
CJK UNIFIED IDEOGRAPH:'D9AF:55727:&#x68EC
CJK UNIFIED IDEOGRAPH:'D9B0:55728:&#x68EA
CJK UNIFIED IDEOGRAPH:'D9B1:55729:&#x68F1
CJK UNIFIED IDEOGRAPH:'D9B2:55730:&#x690F
CJK UNIFIED IDEOGRAPH:'D9B3:55731:&#x68D6
CJK UNIFIED IDEOGRAPH:'D9B4:55732:&#x68F7
CJK UNIFIED IDEOGRAPH:'D9B5:55733:&#x68EB
CJK UNIFIED IDEOGRAPH:'D9B6:55734:&#x68E4
CJK UNIFIED IDEOGRAPH:'D9B7:55735:&#x68F6
CJK UNIFIED IDEOGRAPH:'D9B8:55736:&#x6913
CJK UNIFIED IDEOGRAPH:'D9B9:55737:&#x6910
CJK UNIFIED IDEOGRAPH:'D9BA:55738:&#x68F3
CJK UNIFIED IDEOGRAPH:'D9BB:55739:&#x68E1
CJK UNIFIED IDEOGRAPH:'D9BC:55740:&#x6907
CJK UNIFIED IDEOGRAPH:'D9BD:55741:&#x68CC
CJK UNIFIED IDEOGRAPH:'D9BE:55742:&#x6908
CJK UNIFIED IDEOGRAPH:'D9BF:55743:&#x6970
CJK UNIFIED IDEOGRAPH:'D9C0:55744:&#x68B4
CJK UNIFIED IDEOGRAPH:'D9C1:55745:&#x6911
CJK UNIFIED IDEOGRAPH:'D9C2:55746:&#x68EF
CJK UNIFIED IDEOGRAPH:'D9C3:55747:&#x68C6
CJK UNIFIED IDEOGRAPH:'D9C4:55748:&#x6914
CJK UNIFIED IDEOGRAPH:'D9C5:55749:&#x68F8
CJK UNIFIED IDEOGRAPH:'D9C6:55750:&#x68D0
CJK UNIFIED IDEOGRAPH:'D9C7:55751:&#x68FD
CJK UNIFIED IDEOGRAPH:'D9C8:55752:&#x68FC
CJK UNIFIED IDEOGRAPH:'D9C9:55753:&#x68E8
CJK UNIFIED IDEOGRAPH:'D9CA:55754:&#x690B
CJK UNIFIED IDEOGRAPH:'D9CB:55755:&#x690A
CJK UNIFIED IDEOGRAPH:'D9CC:55756:&#x6917
CJK UNIFIED IDEOGRAPH:'D9CD:55757:&#x68CE
CJK UNIFIED IDEOGRAPH:'D9CE:55758:&#x68C8
CJK UNIFIED IDEOGRAPH:'D9CF:55759:&#x68DD
CJK UNIFIED IDEOGRAPH:'D9D0:55760:&#x68DE
CJK UNIFIED IDEOGRAPH:'D9D1:55761:&#x68E6
CJK UNIFIED IDEOGRAPH:'D9D2:55762:&#x68F4
CJK UNIFIED IDEOGRAPH:'D9D3:55763:&#x68D1
CJK UNIFIED IDEOGRAPH:'D9D4:55764:&#x6906
CJK UNIFIED IDEOGRAPH:'D9D5:55765:&#x68D4
CJK UNIFIED IDEOGRAPH:'D9D6:55766:&#x68E9
CJK UNIFIED IDEOGRAPH:'D9D7:55767:&#x6915
CJK UNIFIED IDEOGRAPH:'D9D8:55768:&#x6925
CJK UNIFIED IDEOGRAPH:'D9D9:55769:&#x68C7
CJK UNIFIED IDEOGRAPH:'D9DA:55770:&#x6B39
CJK UNIFIED IDEOGRAPH:'D9DB:55771:&#x6B3B
CJK UNIFIED IDEOGRAPH:'D9DC:55772:&#x6B3F
CJK UNIFIED IDEOGRAPH:'D9DD:55773:&#x6B3C
CJK UNIFIED IDEOGRAPH:'D9DE:55774:&#x6B94
CJK UNIFIED IDEOGRAPH:'D9DF:55775:&#x6B97
CJK UNIFIED IDEOGRAPH:'D9E0:55776:&#x6B99
CJK UNIFIED IDEOGRAPH:'D9E1:55777:&#x6B95
CJK UNIFIED IDEOGRAPH:'D9E2:55778:&#x6BBD
CJK UNIFIED IDEOGRAPH:'D9E3:55779:&#x6BF0
CJK UNIFIED IDEOGRAPH:'D9E4:55780:&#x6BF2
CJK UNIFIED IDEOGRAPH:'D9E5:55781:&#x6BF3
CJK UNIFIED IDEOGRAPH:'D9E6:55782:&#x6C30
CJK UNIFIED IDEOGRAPH:'D9E7:55783:&#x6DFC
CJK UNIFIED IDEOGRAPH:'D9E8:55784:&#x6E46
CJK UNIFIED IDEOGRAPH:'D9E9:55785:&#x6E47
CJK UNIFIED IDEOGRAPH:'D9EA:55786:&#x6E1F
CJK UNIFIED IDEOGRAPH:'D9EB:55787:&#x6E49
CJK UNIFIED IDEOGRAPH:'D9EC:55788:&#x6E88
CJK UNIFIED IDEOGRAPH:'D9ED:55789:&#x6E3C
CJK UNIFIED IDEOGRAPH:'D9EE:55790:&#x6E3D
CJK UNIFIED IDEOGRAPH:'D9EF:55791:&#x6E45
CJK UNIFIED IDEOGRAPH:'D9F0:55792:&#x6E62
CJK UNIFIED IDEOGRAPH:'D9F1:55793:&#x6E2B
CJK UNIFIED IDEOGRAPH:'D9F2:55794:&#x6E3F
CJK UNIFIED IDEOGRAPH:'D9F3:55795:&#x6E41
CJK UNIFIED IDEOGRAPH:'D9F4:55796:&#x6E5D
CJK UNIFIED IDEOGRAPH:'D9F5:55797:&#x6E73
CJK UNIFIED IDEOGRAPH:'D9F6:55798:&#x6E1C
CJK UNIFIED IDEOGRAPH:'D9F7:55799:&#x6E33
CJK UNIFIED IDEOGRAPH:'D9F8:55800:&#x6E4B
CJK UNIFIED IDEOGRAPH:'D9F9:55801:&#x6E40
CJK UNIFIED IDEOGRAPH:'D9FA:55802:&#x6E51
CJK UNIFIED IDEOGRAPH:'D9FB:55803:&#x6E3B
CJK UNIFIED IDEOGRAPH:'D9FC:55804:&#x6E03
CJK UNIFIED IDEOGRAPH:'D9FD:55805:&#x6E2E
CJK UNIFIED IDEOGRAPH:'D9FE:55806:&#x6E5E
CJK UNIFIED IDEOGRAPH:'DA40:55872:&#x6E68
CJK UNIFIED IDEOGRAPH:'DA41:55873:&#x6E5C
CJK UNIFIED IDEOGRAPH:'DA42:55874:&#x6E61
CJK UNIFIED IDEOGRAPH:'DA43:55875:&#x6E31
CJK UNIFIED IDEOGRAPH:'DA44:55876:&#x6E28
CJK UNIFIED IDEOGRAPH:'DA45:55877:&#x6E60
CJK UNIFIED IDEOGRAPH:'DA46:55878:&#x6E71
CJK UNIFIED IDEOGRAPH:'DA47:55879:&#x6E6B
CJK UNIFIED IDEOGRAPH:'DA48:55880:&#x6E39
CJK UNIFIED IDEOGRAPH:'DA49:55881:&#x6E22
CJK UNIFIED IDEOGRAPH:'DA4A:55882:&#x6E30
CJK UNIFIED IDEOGRAPH:'DA4B:55883:&#x6E53
CJK UNIFIED IDEOGRAPH:'DA4C:55884:&#x6E65
CJK UNIFIED IDEOGRAPH:'DA4D:55885:&#x6E27
CJK UNIFIED IDEOGRAPH:'DA4E:55886:&#x6E78
CJK UNIFIED IDEOGRAPH:'DA4F:55887:&#x6E64
CJK UNIFIED IDEOGRAPH:'DA50:55888:&#x6E77
CJK UNIFIED IDEOGRAPH:'DA51:55889:&#x6E55
CJK UNIFIED IDEOGRAPH:'DA52:55890:&#x6E79
CJK UNIFIED IDEOGRAPH:'DA53:55891:&#x6E52
CJK UNIFIED IDEOGRAPH:'DA54:55892:&#x6E66
CJK UNIFIED IDEOGRAPH:'DA55:55893:&#x6E35
CJK UNIFIED IDEOGRAPH:'DA56:55894:&#x6E36
CJK UNIFIED IDEOGRAPH:'DA57:55895:&#x6E5A
CJK UNIFIED IDEOGRAPH:'DA58:55896:&#x7120
CJK UNIFIED IDEOGRAPH:'DA59:55897:&#x711E
CJK UNIFIED IDEOGRAPH:'DA5A:55898:&#x712F
CJK UNIFIED IDEOGRAPH:'DA5B:55899:&#x70FB
CJK UNIFIED IDEOGRAPH:'DA5C:55900:&#x712E
CJK UNIFIED IDEOGRAPH:'DA5D:55901:&#x7131
CJK UNIFIED IDEOGRAPH:'DA5E:55902:&#x7123
CJK UNIFIED IDEOGRAPH:'DA5F:55903:&#x7125
CJK UNIFIED IDEOGRAPH:'DA60:55904:&#x7122
CJK UNIFIED IDEOGRAPH:'DA61:55905:&#x7132
CJK UNIFIED IDEOGRAPH:'DA62:55906:&#x711F
CJK UNIFIED IDEOGRAPH:'DA63:55907:&#x7128
CJK UNIFIED IDEOGRAPH:'DA64:55908:&#x713A
CJK UNIFIED IDEOGRAPH:'DA65:55909:&#x711B
CJK UNIFIED IDEOGRAPH:'DA66:55910:&#x724B
CJK UNIFIED IDEOGRAPH:'DA67:55911:&#x725A
CJK UNIFIED IDEOGRAPH:'DA68:55912:&#x7288
CJK UNIFIED IDEOGRAPH:'DA69:55913:&#x7289
CJK UNIFIED IDEOGRAPH:'DA6A:55914:&#x7286
CJK UNIFIED IDEOGRAPH:'DA6B:55915:&#x7285
CJK UNIFIED IDEOGRAPH:'DA6C:55916:&#x728B
CJK UNIFIED IDEOGRAPH:'DA6D:55917:&#x7312
CJK UNIFIED IDEOGRAPH:'DA6E:55918:&#x730B
CJK UNIFIED IDEOGRAPH:'DA6F:55919:&#x7330
CJK UNIFIED IDEOGRAPH:'DA70:55920:&#x7322
CJK UNIFIED IDEOGRAPH:'DA71:55921:&#x7331
CJK UNIFIED IDEOGRAPH:'DA72:55922:&#x7333
CJK UNIFIED IDEOGRAPH:'DA73:55923:&#x7327
CJK UNIFIED IDEOGRAPH:'DA74:55924:&#x7332
CJK UNIFIED IDEOGRAPH:'DA75:55925:&#x732D
CJK UNIFIED IDEOGRAPH:'DA76:55926:&#x7326
CJK UNIFIED IDEOGRAPH:'DA77:55927:&#x7323
CJK UNIFIED IDEOGRAPH:'DA78:55928:&#x7335
CJK UNIFIED IDEOGRAPH:'DA79:55929:&#x730C
CJK UNIFIED IDEOGRAPH:'DA7A:55930:&#x742E
CJK UNIFIED IDEOGRAPH:'DA7B:55931:&#x742C
CJK UNIFIED IDEOGRAPH:'DA7C:55932:&#x7430
CJK UNIFIED IDEOGRAPH:'DA7D:55933:&#x742B
CJK UNIFIED IDEOGRAPH:'DA7E:55934:&#x7416
CJK UNIFIED IDEOGRAPH:'DAA1:55969:&#x741A
CJK UNIFIED IDEOGRAPH:'DAA2:55970:&#x7421
CJK UNIFIED IDEOGRAPH:'DAA3:55971:&#x742D
CJK UNIFIED IDEOGRAPH:'DAA4:55972:&#x7431
CJK UNIFIED IDEOGRAPH:'DAA5:55973:&#x7424
CJK UNIFIED IDEOGRAPH:'DAA6:55974:&#x7423
CJK UNIFIED IDEOGRAPH:'DAA7:55975:&#x741D
CJK UNIFIED IDEOGRAPH:'DAA8:55976:&#x7429
CJK UNIFIED IDEOGRAPH:'DAA9:55977:&#x7420
CJK UNIFIED IDEOGRAPH:'DAAA:55978:&#x7432
CJK UNIFIED IDEOGRAPH:'DAAB:55979:&#x74FB
CJK UNIFIED IDEOGRAPH:'DAAC:55980:&#x752F
CJK UNIFIED IDEOGRAPH:'DAAD:55981:&#x756F
CJK UNIFIED IDEOGRAPH:'DAAE:55982:&#x756C
CJK UNIFIED IDEOGRAPH:'DAAF:55983:&#x75E7
CJK UNIFIED IDEOGRAPH:'DAB0:55984:&#x75DA
CJK UNIFIED IDEOGRAPH:'DAB1:55985:&#x75E1
CJK UNIFIED IDEOGRAPH:'DAB2:55986:&#x75E6
CJK UNIFIED IDEOGRAPH:'DAB3:55987:&#x75DD
CJK UNIFIED IDEOGRAPH:'DAB4:55988:&#x75DF
CJK UNIFIED IDEOGRAPH:'DAB5:55989:&#x75E4
CJK UNIFIED IDEOGRAPH:'DAB6:55990:&#x75D7
CJK UNIFIED IDEOGRAPH:'DAB7:55991:&#x7695
CJK UNIFIED IDEOGRAPH:'DAB8:55992:&#x7692
CJK UNIFIED IDEOGRAPH:'DAB9:55993:&#x76DA
CJK UNIFIED IDEOGRAPH:'DABA:55994:&#x7746
CJK UNIFIED IDEOGRAPH:'DABB:55995:&#x7747
CJK UNIFIED IDEOGRAPH:'DABC:55996:&#x7744
CJK UNIFIED IDEOGRAPH:'DABD:55997:&#x774D
CJK UNIFIED IDEOGRAPH:'DABE:55998:&#x7745
CJK UNIFIED IDEOGRAPH:'DABF:55999:&#x774A
CJK UNIFIED IDEOGRAPH:'DAC0:56000:&#x774E
CJK UNIFIED IDEOGRAPH:'DAC1:56001:&#x774B
CJK UNIFIED IDEOGRAPH:'DAC2:56002:&#x774C
CJK UNIFIED IDEOGRAPH:'DAC3:56003:&#x77DE
CJK UNIFIED IDEOGRAPH:'DAC4:56004:&#x77EC
CJK UNIFIED IDEOGRAPH:'DAC5:56005:&#x7860
CJK UNIFIED IDEOGRAPH:'DAC6:56006:&#x7864
CJK UNIFIED IDEOGRAPH:'DAC7:56007:&#x7865
CJK UNIFIED IDEOGRAPH:'DAC8:56008:&#x785C
CJK UNIFIED IDEOGRAPH:'DAC9:56009:&#x786D
CJK UNIFIED IDEOGRAPH:'DACA:56010:&#x7871
CJK UNIFIED IDEOGRAPH:'DACB:56011:&#x786A
CJK UNIFIED IDEOGRAPH:'DACC:56012:&#x786E
CJK UNIFIED IDEOGRAPH:'DACD:56013:&#x7870
CJK UNIFIED IDEOGRAPH:'DACE:56014:&#x7869
CJK UNIFIED IDEOGRAPH:'DACF:56015:&#x7868
CJK UNIFIED IDEOGRAPH:'DAD0:56016:&#x785E
CJK UNIFIED IDEOGRAPH:'DAD1:56017:&#x7862
CJK UNIFIED IDEOGRAPH:'DAD2:56018:&#x7974
CJK UNIFIED IDEOGRAPH:'DAD3:56019:&#x7973
CJK UNIFIED IDEOGRAPH:'DAD4:56020:&#x7972
CJK UNIFIED IDEOGRAPH:'DAD5:56021:&#x7970
CJK UNIFIED IDEOGRAPH:'DAD6:56022:&#x7A02
CJK UNIFIED IDEOGRAPH:'DAD7:56023:&#x7A0A
CJK UNIFIED IDEOGRAPH:'DAD8:56024:&#x7A03
CJK UNIFIED IDEOGRAPH:'DAD9:56025:&#x7A0C
CJK UNIFIED IDEOGRAPH:'DADA:56026:&#x7A04
CJK UNIFIED IDEOGRAPH:'DADB:56027:&#x7A99
CJK UNIFIED IDEOGRAPH:'DADC:56028:&#x7AE6
CJK UNIFIED IDEOGRAPH:'DADD:56029:&#x7AE4
CJK UNIFIED IDEOGRAPH:'DADE:56030:&#x7B4A
CJK UNIFIED IDEOGRAPH:'DADF:56031:&#x7B3B
CJK UNIFIED IDEOGRAPH:'DAE0:56032:&#x7B44
CJK UNIFIED IDEOGRAPH:'DAE1:56033:&#x7B48
CJK UNIFIED IDEOGRAPH:'DAE2:56034:&#x7B4C
CJK UNIFIED IDEOGRAPH:'DAE3:56035:&#x7B4E
CJK UNIFIED IDEOGRAPH:'DAE4:56036:&#x7B40
CJK UNIFIED IDEOGRAPH:'DAE5:56037:&#x7B58
CJK UNIFIED IDEOGRAPH:'DAE6:56038:&#x7B45
CJK UNIFIED IDEOGRAPH:'DAE7:56039:&#x7CA2
CJK UNIFIED IDEOGRAPH:'DAE8:56040:&#x7C9E
CJK UNIFIED IDEOGRAPH:'DAE9:56041:&#x7CA8
CJK UNIFIED IDEOGRAPH:'DAEA:56042:&#x7CA1
CJK UNIFIED IDEOGRAPH:'DAEB:56043:&#x7D58
CJK UNIFIED IDEOGRAPH:'DAEC:56044:&#x7D6F
CJK UNIFIED IDEOGRAPH:'DAED:56045:&#x7D63
CJK UNIFIED IDEOGRAPH:'DAEE:56046:&#x7D53
CJK UNIFIED IDEOGRAPH:'DAEF:56047:&#x7D56
CJK UNIFIED IDEOGRAPH:'DAF0:56048:&#x7D67
CJK UNIFIED IDEOGRAPH:'DAF1:56049:&#x7D6A
CJK UNIFIED IDEOGRAPH:'DAF2:56050:&#x7D4F
CJK UNIFIED IDEOGRAPH:'DAF3:56051:&#x7D6D
CJK UNIFIED IDEOGRAPH:'DAF4:56052:&#x7D5C
CJK UNIFIED IDEOGRAPH:'DAF5:56053:&#x7D6B
CJK UNIFIED IDEOGRAPH:'DAF6:56054:&#x7D52
CJK UNIFIED IDEOGRAPH:'DAF7:56055:&#x7D54
CJK UNIFIED IDEOGRAPH:'DAF8:56056:&#x7D69
CJK UNIFIED IDEOGRAPH:'DAF9:56057:&#x7D51
CJK UNIFIED IDEOGRAPH:'DAFA:56058:&#x7D5F
CJK UNIFIED IDEOGRAPH:'DAFB:56059:&#x7D4E
CJK UNIFIED IDEOGRAPH:'DAFC:56060:&#x7F3E
CJK UNIFIED IDEOGRAPH:'DAFD:56061:&#x7F3F
CJK UNIFIED IDEOGRAPH:'DAFE:56062:&#x7F65
CJK UNIFIED IDEOGRAPH:'DB40:56128:&#x7F66
CJK UNIFIED IDEOGRAPH:'DB41:56129:&#x7FA2
CJK UNIFIED IDEOGRAPH:'DB42:56130:&#x7FA0
CJK UNIFIED IDEOGRAPH:'DB43:56131:&#x7FA1
CJK UNIFIED IDEOGRAPH:'DB44:56132:&#x7FD7
CJK UNIFIED IDEOGRAPH:'DB45:56133:&#x8051
CJK UNIFIED IDEOGRAPH:'DB46:56134:&#x804F
CJK UNIFIED IDEOGRAPH:'DB47:56135:&#x8050
CJK UNIFIED IDEOGRAPH:'DB48:56136:&#x80FE
CJK UNIFIED IDEOGRAPH:'DB49:56137:&#x80D4
CJK UNIFIED IDEOGRAPH:'DB4A:56138:&#x8143
CJK UNIFIED IDEOGRAPH:'DB4B:56139:&#x814A
CJK UNIFIED IDEOGRAPH:'DB4C:56140:&#x8152
CJK UNIFIED IDEOGRAPH:'DB4D:56141:&#x814F
CJK UNIFIED IDEOGRAPH:'DB4E:56142:&#x8147
CJK UNIFIED IDEOGRAPH:'DB4F:56143:&#x813D
CJK UNIFIED IDEOGRAPH:'DB50:56144:&#x814D
CJK UNIFIED IDEOGRAPH:'DB51:56145:&#x813A
CJK UNIFIED IDEOGRAPH:'DB52:56146:&#x81E6
CJK UNIFIED IDEOGRAPH:'DB53:56147:&#x81EE
CJK UNIFIED IDEOGRAPH:'DB54:56148:&#x81F7
CJK UNIFIED IDEOGRAPH:'DB55:56149:&#x81F8
CJK UNIFIED IDEOGRAPH:'DB56:56150:&#x81F9
CJK UNIFIED IDEOGRAPH:'DB57:56151:&#x8204
CJK UNIFIED IDEOGRAPH:'DB58:56152:&#x823C
CJK UNIFIED IDEOGRAPH:'DB59:56153:&#x823D
CJK UNIFIED IDEOGRAPH:'DB5A:56154:&#x823F
CJK UNIFIED IDEOGRAPH:'DB5B:56155:&#x8275
CJK UNIFIED IDEOGRAPH:'DB5C:56156:&#x833B
CJK UNIFIED IDEOGRAPH:'DB5D:56157:&#x83CF
CJK UNIFIED IDEOGRAPH:'DB5E:56158:&#x83F9
CJK UNIFIED IDEOGRAPH:'DB5F:56159:&#x8423
CJK UNIFIED IDEOGRAPH:'DB60:56160:&#x83C0
CJK UNIFIED IDEOGRAPH:'DB61:56161:&#x83E8
CJK UNIFIED IDEOGRAPH:'DB62:56162:&#x8412
CJK UNIFIED IDEOGRAPH:'DB63:56163:&#x83E7
CJK UNIFIED IDEOGRAPH:'DB64:56164:&#x83E4
CJK UNIFIED IDEOGRAPH:'DB65:56165:&#x83FC
CJK UNIFIED IDEOGRAPH:'DB66:56166:&#x83F6
CJK UNIFIED IDEOGRAPH:'DB67:56167:&#x8410
CJK UNIFIED IDEOGRAPH:'DB68:56168:&#x83C6
CJK UNIFIED IDEOGRAPH:'DB69:56169:&#x83C8
CJK UNIFIED IDEOGRAPH:'DB6A:56170:&#x83EB
CJK UNIFIED IDEOGRAPH:'DB6B:56171:&#x83E3
CJK UNIFIED IDEOGRAPH:'DB6C:56172:&#x83BF
CJK UNIFIED IDEOGRAPH:'DB6D:56173:&#x8401
CJK UNIFIED IDEOGRAPH:'DB6E:56174:&#x83DD
CJK UNIFIED IDEOGRAPH:'DB6F:56175:&#x83E5
CJK UNIFIED IDEOGRAPH:'DB70:56176:&#x83D8
CJK UNIFIED IDEOGRAPH:'DB71:56177:&#x83FF
CJK UNIFIED IDEOGRAPH:'DB72:56178:&#x83E1
CJK UNIFIED IDEOGRAPH:'DB73:56179:&#x83CB
CJK UNIFIED IDEOGRAPH:'DB74:56180:&#x83CE
CJK UNIFIED IDEOGRAPH:'DB75:56181:&#x83D6
CJK UNIFIED IDEOGRAPH:'DB76:56182:&#x83F5
CJK UNIFIED IDEOGRAPH:'DB77:56183:&#x83C9
CJK UNIFIED IDEOGRAPH:'DB78:56184:&#x8409
CJK UNIFIED IDEOGRAPH:'DB79:56185:&#x840F
CJK UNIFIED IDEOGRAPH:'DB7A:56186:&#x83DE
CJK UNIFIED IDEOGRAPH:'DB7B:56187:&#x8411
CJK UNIFIED IDEOGRAPH:'DB7C:56188:&#x8406
CJK UNIFIED IDEOGRAPH:'DB7D:56189:&#x83C2
CJK UNIFIED IDEOGRAPH:'DB7E:56190:&#x83F3
CJK UNIFIED IDEOGRAPH:'DBA1:56225:&#x83D5
CJK UNIFIED IDEOGRAPH:'DBA2:56226:&#x83FA
CJK UNIFIED IDEOGRAPH:'DBA3:56227:&#x83C7
CJK UNIFIED IDEOGRAPH:'DBA4:56228:&#x83D1
CJK UNIFIED IDEOGRAPH:'DBA5:56229:&#x83EA
CJK UNIFIED IDEOGRAPH:'DBA6:56230:&#x8413
CJK UNIFIED IDEOGRAPH:'DBA7:56231:&#x83C3
CJK UNIFIED IDEOGRAPH:'DBA8:56232:&#x83EC
CJK UNIFIED IDEOGRAPH:'DBA9:56233:&#x83EE
CJK UNIFIED IDEOGRAPH:'DBAA:56234:&#x83C4
CJK UNIFIED IDEOGRAPH:'DBAB:56235:&#x83FB
CJK UNIFIED IDEOGRAPH:'DBAC:56236:&#x83D7
CJK UNIFIED IDEOGRAPH:'DBAD:56237:&#x83E2
CJK UNIFIED IDEOGRAPH:'DBAE:56238:&#x841B
CJK UNIFIED IDEOGRAPH:'DBAF:56239:&#x83DB
CJK UNIFIED IDEOGRAPH:'DBB0:56240:&#x83FE
CJK UNIFIED IDEOGRAPH:'DBB1:56241:&#x86D8
CJK UNIFIED IDEOGRAPH:'DBB2:56242:&#x86E2
CJK UNIFIED IDEOGRAPH:'DBB3:56243:&#x86E6
CJK UNIFIED IDEOGRAPH:'DBB4:56244:&#x86D3
CJK UNIFIED IDEOGRAPH:'DBB5:56245:&#x86E3
CJK UNIFIED IDEOGRAPH:'DBB6:56246:&#x86DA
CJK UNIFIED IDEOGRAPH:'DBB7:56247:&#x86EA
CJK UNIFIED IDEOGRAPH:'DBB8:56248:&#x86DD
CJK UNIFIED IDEOGRAPH:'DBB9:56249:&#x86EB
CJK UNIFIED IDEOGRAPH:'DBBA:56250:&#x86DC
CJK UNIFIED IDEOGRAPH:'DBBB:56251:&#x86EC
CJK UNIFIED IDEOGRAPH:'DBBC:56252:&#x86E9
CJK UNIFIED IDEOGRAPH:'DBBD:56253:&#x86D7
CJK UNIFIED IDEOGRAPH:'DBBE:56254:&#x86E8
CJK UNIFIED IDEOGRAPH:'DBBF:56255:&#x86D1
CJK UNIFIED IDEOGRAPH:'DBC0:56256:&#x8848
CJK UNIFIED IDEOGRAPH:'DBC1:56257:&#x8856
CJK UNIFIED IDEOGRAPH:'DBC2:56258:&#x8855
CJK UNIFIED IDEOGRAPH:'DBC3:56259:&#x88BA
CJK UNIFIED IDEOGRAPH:'DBC4:56260:&#x88D7
CJK UNIFIED IDEOGRAPH:'DBC5:56261:&#x88B9
CJK UNIFIED IDEOGRAPH:'DBC6:56262:&#x88B8
CJK UNIFIED IDEOGRAPH:'DBC7:56263:&#x88C0
CJK UNIFIED IDEOGRAPH:'DBC8:56264:&#x88BE
CJK UNIFIED IDEOGRAPH:'DBC9:56265:&#x88B6
CJK UNIFIED IDEOGRAPH:'DBCA:56266:&#x88BC
CJK UNIFIED IDEOGRAPH:'DBCB:56267:&#x88B7
CJK UNIFIED IDEOGRAPH:'DBCC:56268:&#x88BD
CJK UNIFIED IDEOGRAPH:'DBCD:56269:&#x88B2
CJK UNIFIED IDEOGRAPH:'DBCE:56270:&#x8901
CJK UNIFIED IDEOGRAPH:'DBCF:56271:&#x88C9
CJK UNIFIED IDEOGRAPH:'DBD0:56272:&#x8995
CJK UNIFIED IDEOGRAPH:'DBD1:56273:&#x8998
CJK UNIFIED IDEOGRAPH:'DBD2:56274:&#x8997
CJK UNIFIED IDEOGRAPH:'DBD3:56275:&#x89DD
CJK UNIFIED IDEOGRAPH:'DBD4:56276:&#x89DA
CJK UNIFIED IDEOGRAPH:'DBD5:56277:&#x89DB
CJK UNIFIED IDEOGRAPH:'DBD6:56278:&#x8A4E
CJK UNIFIED IDEOGRAPH:'DBD7:56279:&#x8A4D
CJK UNIFIED IDEOGRAPH:'DBD8:56280:&#x8A39
CJK UNIFIED IDEOGRAPH:'DBD9:56281:&#x8A59
CJK UNIFIED IDEOGRAPH:'DBDA:56282:&#x8A40
CJK UNIFIED IDEOGRAPH:'DBDB:56283:&#x8A57
CJK UNIFIED IDEOGRAPH:'DBDC:56284:&#x8A58
CJK UNIFIED IDEOGRAPH:'DBDD:56285:&#x8A44
CJK UNIFIED IDEOGRAPH:'DBDE:56286:&#x8A45
CJK UNIFIED IDEOGRAPH:'DBDF:56287:&#x8A52
CJK UNIFIED IDEOGRAPH:'DBE0:56288:&#x8A48
CJK UNIFIED IDEOGRAPH:'DBE1:56289:&#x8A51
CJK UNIFIED IDEOGRAPH:'DBE2:56290:&#x8A4A
CJK UNIFIED IDEOGRAPH:'DBE3:56291:&#x8A4C
CJK UNIFIED IDEOGRAPH:'DBE4:56292:&#x8A4F
CJK UNIFIED IDEOGRAPH:'DBE5:56293:&#x8C5F
CJK UNIFIED IDEOGRAPH:'DBE6:56294:&#x8C81
CJK UNIFIED IDEOGRAPH:'DBE7:56295:&#x8C80
CJK UNIFIED IDEOGRAPH:'DBE8:56296:&#x8CBA
CJK UNIFIED IDEOGRAPH:'DBE9:56297:&#x8CBE
CJK UNIFIED IDEOGRAPH:'DBEA:56298:&#x8CB0
CJK UNIFIED IDEOGRAPH:'DBEB:56299:&#x8CB9
CJK UNIFIED IDEOGRAPH:'DBEC:56300:&#x8CB5
CJK UNIFIED IDEOGRAPH:'DBED:56301:&#x8D84
CJK UNIFIED IDEOGRAPH:'DBEE:56302:&#x8D80
CJK UNIFIED IDEOGRAPH:'DBEF:56303:&#x8D89
CJK UNIFIED IDEOGRAPH:'DBF0:56304:&#x8DD8
CJK UNIFIED IDEOGRAPH:'DBF1:56305:&#x8DD3
CJK UNIFIED IDEOGRAPH:'DBF2:56306:&#x8DCD
CJK UNIFIED IDEOGRAPH:'DBF3:56307:&#x8DC7
CJK UNIFIED IDEOGRAPH:'DBF4:56308:&#x8DD6
CJK UNIFIED IDEOGRAPH:'DBF5:56309:&#x8DDC
CJK UNIFIED IDEOGRAPH:'DBF6:56310:&#x8DCF
CJK UNIFIED IDEOGRAPH:'DBF7:56311:&#x8DD5
CJK UNIFIED IDEOGRAPH:'DBF8:56312:&#x8DD9
CJK UNIFIED IDEOGRAPH:'DBF9:56313:&#x8DC8
CJK UNIFIED IDEOGRAPH:'DBFA:56314:&#x8DD7
CJK UNIFIED IDEOGRAPH:'DBFB:56315:&#x8DC5
CJK UNIFIED IDEOGRAPH:'DBFC:56316:&#x8EEF
CJK UNIFIED IDEOGRAPH:'DBFD:56317:&#x8EF7
CJK UNIFIED IDEOGRAPH:'DBFE:56318:&#x8EFA
CJK UNIFIED IDEOGRAPH:'DC40:56384:&#x8EF9
CJK UNIFIED IDEOGRAPH:'DC41:56385:&#x8EE6
CJK UNIFIED IDEOGRAPH:'DC42:56386:&#x8EEE
CJK UNIFIED IDEOGRAPH:'DC43:56387:&#x8EE5
CJK UNIFIED IDEOGRAPH:'DC44:56388:&#x8EF5
CJK UNIFIED IDEOGRAPH:'DC45:56389:&#x8EE7
CJK UNIFIED IDEOGRAPH:'DC46:56390:&#x8EE8
CJK UNIFIED IDEOGRAPH:'DC47:56391:&#x8EF6
CJK UNIFIED IDEOGRAPH:'DC48:56392:&#x8EEB
CJK UNIFIED IDEOGRAPH:'DC49:56393:&#x8EF1
CJK UNIFIED IDEOGRAPH:'DC4A:56394:&#x8EEC
CJK UNIFIED IDEOGRAPH:'DC4B:56395:&#x8EF4
CJK UNIFIED IDEOGRAPH:'DC4C:56396:&#x8EE9
CJK UNIFIED IDEOGRAPH:'DC4D:56397:&#x902D
CJK UNIFIED IDEOGRAPH:'DC4E:56398:&#x9034
CJK UNIFIED IDEOGRAPH:'DC4F:56399:&#x902F
CJK UNIFIED IDEOGRAPH:'DC50:56400:&#x9106
CJK UNIFIED IDEOGRAPH:'DC51:56401:&#x912C
CJK UNIFIED IDEOGRAPH:'DC52:56402:&#x9104
CJK UNIFIED IDEOGRAPH:'DC53:56403:&#x90FF
CJK UNIFIED IDEOGRAPH:'DC54:56404:&#x90FC
CJK UNIFIED IDEOGRAPH:'DC55:56405:&#x9108
CJK UNIFIED IDEOGRAPH:'DC56:56406:&#x90F9
CJK UNIFIED IDEOGRAPH:'DC57:56407:&#x90FB
CJK UNIFIED IDEOGRAPH:'DC58:56408:&#x9101
CJK UNIFIED IDEOGRAPH:'DC59:56409:&#x9100
CJK UNIFIED IDEOGRAPH:'DC5A:56410:&#x9107
CJK UNIFIED IDEOGRAPH:'DC5B:56411:&#x9105
CJK UNIFIED IDEOGRAPH:'DC5C:56412:&#x9103
CJK UNIFIED IDEOGRAPH:'DC5D:56413:&#x9161
CJK UNIFIED IDEOGRAPH:'DC5E:56414:&#x9164
CJK UNIFIED IDEOGRAPH:'DC5F:56415:&#x915F
CJK UNIFIED IDEOGRAPH:'DC60:56416:&#x9162
CJK UNIFIED IDEOGRAPH:'DC61:56417:&#x9160
CJK UNIFIED IDEOGRAPH:'DC62:56418:&#x9201
CJK UNIFIED IDEOGRAPH:'DC63:56419:&#x920A
CJK UNIFIED IDEOGRAPH:'DC64:56420:&#x9225
CJK UNIFIED IDEOGRAPH:'DC65:56421:&#x9203
CJK UNIFIED IDEOGRAPH:'DC66:56422:&#x921A
CJK UNIFIED IDEOGRAPH:'DC67:56423:&#x9226
CJK UNIFIED IDEOGRAPH:'DC68:56424:&#x920F
CJK UNIFIED IDEOGRAPH:'DC69:56425:&#x920C
CJK UNIFIED IDEOGRAPH:'DC6A:56426:&#x9200
CJK UNIFIED IDEOGRAPH:'DC6B:56427:&#x9212
CJK UNIFIED IDEOGRAPH:'DC6C:56428:&#x91FF
CJK UNIFIED IDEOGRAPH:'DC6D:56429:&#x91FD
CJK UNIFIED IDEOGRAPH:'DC6E:56430:&#x9206
CJK UNIFIED IDEOGRAPH:'DC6F:56431:&#x9204
CJK UNIFIED IDEOGRAPH:'DC70:56432:&#x9227
CJK UNIFIED IDEOGRAPH:'DC71:56433:&#x9202
CJK UNIFIED IDEOGRAPH:'DC72:56434:&#x921C
CJK UNIFIED IDEOGRAPH:'DC73:56435:&#x9224
CJK UNIFIED IDEOGRAPH:'DC74:56436:&#x9219
CJK UNIFIED IDEOGRAPH:'DC75:56437:&#x9217
CJK UNIFIED IDEOGRAPH:'DC76:56438:&#x9205
CJK UNIFIED IDEOGRAPH:'DC77:56439:&#x9216
CJK UNIFIED IDEOGRAPH:'DC78:56440:&#x957B
CJK UNIFIED IDEOGRAPH:'DC79:56441:&#x958D
CJK UNIFIED IDEOGRAPH:'DC7A:56442:&#x958C
CJK UNIFIED IDEOGRAPH:'DC7B:56443:&#x9590
CJK UNIFIED IDEOGRAPH:'DC7C:56444:&#x9687
CJK UNIFIED IDEOGRAPH:'DC7D:56445:&#x967E
CJK UNIFIED IDEOGRAPH:'DC7E:56446:&#x9688
CJK UNIFIED IDEOGRAPH:'DCA1:56481:&#x9689
CJK UNIFIED IDEOGRAPH:'DCA2:56482:&#x9683
CJK UNIFIED IDEOGRAPH:'DCA3:56483:&#x9680
CJK UNIFIED IDEOGRAPH:'DCA4:56484:&#x96C2
CJK UNIFIED IDEOGRAPH:'DCA5:56485:&#x96C8
CJK UNIFIED IDEOGRAPH:'DCA6:56486:&#x96C3
CJK UNIFIED IDEOGRAPH:'DCA7:56487:&#x96F1
CJK UNIFIED IDEOGRAPH:'DCA8:56488:&#x96F0
CJK UNIFIED IDEOGRAPH:'DCA9:56489:&#x976C
CJK UNIFIED IDEOGRAPH:'DCAA:56490:&#x9770
CJK UNIFIED IDEOGRAPH:'DCAB:56491:&#x976E
CJK UNIFIED IDEOGRAPH:'DCAC:56492:&#x9807
CJK UNIFIED IDEOGRAPH:'DCAD:56493:&#x98A9
CJK UNIFIED IDEOGRAPH:'DCAE:56494:&#x98EB
CJK UNIFIED IDEOGRAPH:'DCAF:56495:&#x9CE6
CJK UNIFIED IDEOGRAPH:'DCB0:56496:&#x9EF9
CJK UNIFIED IDEOGRAPH:'DCB1:56497:&#x4E83
CJK UNIFIED IDEOGRAPH:'DCB2:56498:&#x4E84
CJK UNIFIED IDEOGRAPH:'DCB3:56499:&#x4EB6
CJK UNIFIED IDEOGRAPH:'DCB4:56500:&#x50BD
CJK UNIFIED IDEOGRAPH:'DCB5:56501:&#x50BF
CJK UNIFIED IDEOGRAPH:'DCB6:56502:&#x50C6
CJK UNIFIED IDEOGRAPH:'DCB7:56503:&#x50AE
CJK UNIFIED IDEOGRAPH:'DCB8:56504:&#x50C4
CJK UNIFIED IDEOGRAPH:'DCB9:56505:&#x50CA
CJK UNIFIED IDEOGRAPH:'DCBA:56506:&#x50B4
CJK UNIFIED IDEOGRAPH:'DCBB:56507:&#x50C8
CJK UNIFIED IDEOGRAPH:'DCBC:56508:&#x50C2
CJK UNIFIED IDEOGRAPH:'DCBD:56509:&#x50B0
CJK UNIFIED IDEOGRAPH:'DCBE:56510:&#x50C1
CJK UNIFIED IDEOGRAPH:'DCBF:56511:&#x50BA
CJK UNIFIED IDEOGRAPH:'DCC0:56512:&#x50B1
CJK UNIFIED IDEOGRAPH:'DCC1:56513:&#x50CB
CJK UNIFIED IDEOGRAPH:'DCC2:56514:&#x50C9
CJK UNIFIED IDEOGRAPH:'DCC3:56515:&#x50B6
CJK UNIFIED IDEOGRAPH:'DCC4:56516:&#x50B8
CJK UNIFIED IDEOGRAPH:'DCC5:56517:&#x51D7
CJK UNIFIED IDEOGRAPH:'DCC6:56518:&#x527A
CJK UNIFIED IDEOGRAPH:'DCC7:56519:&#x5278
CJK UNIFIED IDEOGRAPH:'DCC8:56520:&#x527B
CJK UNIFIED IDEOGRAPH:'DCC9:56521:&#x527C
CJK UNIFIED IDEOGRAPH:'DCCA:56522:&#x55C3
CJK UNIFIED IDEOGRAPH:'DCCB:56523:&#x55DB
CJK UNIFIED IDEOGRAPH:'DCCC:56524:&#x55CC
CJK UNIFIED IDEOGRAPH:'DCCD:56525:&#x55D0
CJK UNIFIED IDEOGRAPH:'DCCE:56526:&#x55CB
CJK UNIFIED IDEOGRAPH:'DCCF:56527:&#x55CA
CJK UNIFIED IDEOGRAPH:'DCD0:56528:&#x55DD
CJK UNIFIED IDEOGRAPH:'DCD1:56529:&#x55C0
CJK UNIFIED IDEOGRAPH:'DCD2:56530:&#x55D4
CJK UNIFIED IDEOGRAPH:'DCD3:56531:&#x55C4
CJK UNIFIED IDEOGRAPH:'DCD4:56532:&#x55E9
CJK UNIFIED IDEOGRAPH:'DCD5:56533:&#x55BF
CJK UNIFIED IDEOGRAPH:'DCD6:56534:&#x55D2
CJK UNIFIED IDEOGRAPH:'DCD7:56535:&#x558D
CJK UNIFIED IDEOGRAPH:'DCD8:56536:&#x55CF
CJK UNIFIED IDEOGRAPH:'DCD9:56537:&#x55D5
CJK UNIFIED IDEOGRAPH:'DCDA:56538:&#x55E2
CJK UNIFIED IDEOGRAPH:'DCDB:56539:&#x55D6
CJK UNIFIED IDEOGRAPH:'DCDC:56540:&#x55C8
CJK UNIFIED IDEOGRAPH:'DCDD:56541:&#x55F2
CJK UNIFIED IDEOGRAPH:'DCDE:56542:&#x55CD
CJK UNIFIED IDEOGRAPH:'DCDF:56543:&#x55D9
CJK UNIFIED IDEOGRAPH:'DCE0:56544:&#x55C2
CJK UNIFIED IDEOGRAPH:'DCE1:56545:&#x5714
CJK UNIFIED IDEOGRAPH:'DCE2:56546:&#x5853
CJK UNIFIED IDEOGRAPH:'DCE3:56547:&#x5868
CJK UNIFIED IDEOGRAPH:'DCE4:56548:&#x5864
CJK UNIFIED IDEOGRAPH:'DCE5:56549:&#x584F
CJK UNIFIED IDEOGRAPH:'DCE6:56550:&#x584D
CJK UNIFIED IDEOGRAPH:'DCE7:56551:&#x5849
CJK UNIFIED IDEOGRAPH:'DCE8:56552:&#x586F
CJK UNIFIED IDEOGRAPH:'DCE9:56553:&#x5855
CJK UNIFIED IDEOGRAPH:'DCEA:56554:&#x584E
CJK UNIFIED IDEOGRAPH:'DCEB:56555:&#x585D
CJK UNIFIED IDEOGRAPH:'DCEC:56556:&#x5859
CJK UNIFIED IDEOGRAPH:'DCED:56557:&#x5865
CJK UNIFIED IDEOGRAPH:'DCEE:56558:&#x585B
CJK UNIFIED IDEOGRAPH:'DCEF:56559:&#x583D
CJK UNIFIED IDEOGRAPH:'DCF0:56560:&#x5863
CJK UNIFIED IDEOGRAPH:'DCF1:56561:&#x5871
CJK UNIFIED IDEOGRAPH:'DCF2:56562:&#x58FC
CJK UNIFIED IDEOGRAPH:'DCF3:56563:&#x5AC7
CJK UNIFIED IDEOGRAPH:'DCF4:56564:&#x5AC4
CJK UNIFIED IDEOGRAPH:'DCF5:56565:&#x5ACB
CJK UNIFIED IDEOGRAPH:'DCF6:56566:&#x5ABA
CJK UNIFIED IDEOGRAPH:'DCF7:56567:&#x5AB8
CJK UNIFIED IDEOGRAPH:'DCF8:56568:&#x5AB1
CJK UNIFIED IDEOGRAPH:'DCF9:56569:&#x5AB5
CJK UNIFIED IDEOGRAPH:'DCFA:56570:&#x5AB0
CJK UNIFIED IDEOGRAPH:'DCFB:56571:&#x5ABF
CJK UNIFIED IDEOGRAPH:'DCFC:56572:&#x5AC8
CJK UNIFIED IDEOGRAPH:'DCFD:56573:&#x5ABB
CJK UNIFIED IDEOGRAPH:'DCFE:56574:&#x5AC6
CJK UNIFIED IDEOGRAPH:'DD40:56640:&#x5AB7
CJK UNIFIED IDEOGRAPH:'DD41:56641:&#x5AC0
CJK UNIFIED IDEOGRAPH:'DD42:56642:&#x5ACA
CJK UNIFIED IDEOGRAPH:'DD43:56643:&#x5AB4
CJK UNIFIED IDEOGRAPH:'DD44:56644:&#x5AB6
CJK UNIFIED IDEOGRAPH:'DD45:56645:&#x5ACD
CJK UNIFIED IDEOGRAPH:'DD46:56646:&#x5AB9
CJK UNIFIED IDEOGRAPH:'DD47:56647:&#x5A90
CJK UNIFIED IDEOGRAPH:'DD48:56648:&#x5BD6
CJK UNIFIED IDEOGRAPH:'DD49:56649:&#x5BD8
CJK UNIFIED IDEOGRAPH:'DD4A:56650:&#x5BD9
CJK UNIFIED IDEOGRAPH:'DD4B:56651:&#x5C1F
CJK UNIFIED IDEOGRAPH:'DD4C:56652:&#x5C33
CJK UNIFIED IDEOGRAPH:'DD4D:56653:&#x5D71
CJK UNIFIED IDEOGRAPH:'DD4E:56654:&#x5D63
CJK UNIFIED IDEOGRAPH:'DD4F:56655:&#x5D4A
CJK UNIFIED IDEOGRAPH:'DD50:56656:&#x5D65
CJK UNIFIED IDEOGRAPH:'DD51:56657:&#x5D72
CJK UNIFIED IDEOGRAPH:'DD52:56658:&#x5D6C
CJK UNIFIED IDEOGRAPH:'DD53:56659:&#x5D5E
CJK UNIFIED IDEOGRAPH:'DD54:56660:&#x5D68
CJK UNIFIED IDEOGRAPH:'DD55:56661:&#x5D67
CJK UNIFIED IDEOGRAPH:'DD56:56662:&#x5D62
CJK UNIFIED IDEOGRAPH:'DD57:56663:&#x5DF0
CJK UNIFIED IDEOGRAPH:'DD58:56664:&#x5E4F
CJK UNIFIED IDEOGRAPH:'DD59:56665:&#x5E4E
CJK UNIFIED IDEOGRAPH:'DD5A:56666:&#x5E4A
CJK UNIFIED IDEOGRAPH:'DD5B:56667:&#x5E4D
CJK UNIFIED IDEOGRAPH:'DD5C:56668:&#x5E4B
CJK UNIFIED IDEOGRAPH:'DD5D:56669:&#x5EC5
CJK UNIFIED IDEOGRAPH:'DD5E:56670:&#x5ECC
CJK UNIFIED IDEOGRAPH:'DD5F:56671:&#x5EC6
CJK UNIFIED IDEOGRAPH:'DD60:56672:&#x5ECB
CJK UNIFIED IDEOGRAPH:'DD61:56673:&#x5EC7
CJK UNIFIED IDEOGRAPH:'DD62:56674:&#x5F40
CJK UNIFIED IDEOGRAPH:'DD63:56675:&#x5FAF
CJK UNIFIED IDEOGRAPH:'DD64:56676:&#x5FAD
CJK UNIFIED IDEOGRAPH:'DD65:56677:&#x60F7
CJK UNIFIED IDEOGRAPH:'DD66:56678:&#x6149
CJK UNIFIED IDEOGRAPH:'DD67:56679:&#x614A
CJK UNIFIED IDEOGRAPH:'DD68:56680:&#x612B
CJK UNIFIED IDEOGRAPH:'DD69:56681:&#x6145
CJK UNIFIED IDEOGRAPH:'DD6A:56682:&#x6136
CJK UNIFIED IDEOGRAPH:'DD6B:56683:&#x6132
CJK UNIFIED IDEOGRAPH:'DD6C:56684:&#x612E
CJK UNIFIED IDEOGRAPH:'DD6D:56685:&#x6146
CJK UNIFIED IDEOGRAPH:'DD6E:56686:&#x612F
CJK UNIFIED IDEOGRAPH:'DD6F:56687:&#x614F
CJK UNIFIED IDEOGRAPH:'DD70:56688:&#x6129
CJK UNIFIED IDEOGRAPH:'DD71:56689:&#x6140
CJK UNIFIED IDEOGRAPH:'DD72:56690:&#x6220
CJK UNIFIED IDEOGRAPH:'DD73:56691:&#x9168
CJK UNIFIED IDEOGRAPH:'DD74:56692:&#x6223
CJK UNIFIED IDEOGRAPH:'DD75:56693:&#x6225
CJK UNIFIED IDEOGRAPH:'DD76:56694:&#x6224
CJK UNIFIED IDEOGRAPH:'DD77:56695:&#x63C5
CJK UNIFIED IDEOGRAPH:'DD78:56696:&#x63F1
CJK UNIFIED IDEOGRAPH:'DD79:56697:&#x63EB
CJK UNIFIED IDEOGRAPH:'DD7A:56698:&#x6410
CJK UNIFIED IDEOGRAPH:'DD7B:56699:&#x6412
CJK UNIFIED IDEOGRAPH:'DD7C:56700:&#x6409
CJK UNIFIED IDEOGRAPH:'DD7D:56701:&#x6420
CJK UNIFIED IDEOGRAPH:'DD7E:56702:&#x6424
CJK UNIFIED IDEOGRAPH:'DDA1:56737:&#x6433
CJK UNIFIED IDEOGRAPH:'DDA2:56738:&#x6443
CJK UNIFIED IDEOGRAPH:'DDA3:56739:&#x641F
CJK UNIFIED IDEOGRAPH:'DDA4:56740:&#x6415
CJK UNIFIED IDEOGRAPH:'DDA5:56741:&#x6418
CJK UNIFIED IDEOGRAPH:'DDA6:56742:&#x6439
CJK UNIFIED IDEOGRAPH:'DDA7:56743:&#x6437
CJK UNIFIED IDEOGRAPH:'DDA8:56744:&#x6422
CJK UNIFIED IDEOGRAPH:'DDA9:56745:&#x6423
CJK UNIFIED IDEOGRAPH:'DDAA:56746:&#x640C
CJK UNIFIED IDEOGRAPH:'DDAB:56747:&#x6426
CJK UNIFIED IDEOGRAPH:'DDAC:56748:&#x6430
CJK UNIFIED IDEOGRAPH:'DDAD:56749:&#x6428
CJK UNIFIED IDEOGRAPH:'DDAE:56750:&#x6441
CJK UNIFIED IDEOGRAPH:'DDAF:56751:&#x6435
CJK UNIFIED IDEOGRAPH:'DDB0:56752:&#x642F
CJK UNIFIED IDEOGRAPH:'DDB1:56753:&#x640A
CJK UNIFIED IDEOGRAPH:'DDB2:56754:&#x641A
CJK UNIFIED IDEOGRAPH:'DDB3:56755:&#x6440
CJK UNIFIED IDEOGRAPH:'DDB4:56756:&#x6425
CJK UNIFIED IDEOGRAPH:'DDB5:56757:&#x6427
CJK UNIFIED IDEOGRAPH:'DDB6:56758:&#x640B
CJK UNIFIED IDEOGRAPH:'DDB7:56759:&#x63E7
CJK UNIFIED IDEOGRAPH:'DDB8:56760:&#x641B
CJK UNIFIED IDEOGRAPH:'DDB9:56761:&#x642E
CJK UNIFIED IDEOGRAPH:'DDBA:56762:&#x6421
CJK UNIFIED IDEOGRAPH:'DDBB:56763:&#x640E
CJK UNIFIED IDEOGRAPH:'DDBC:56764:&#x656F
CJK UNIFIED IDEOGRAPH:'DDBD:56765:&#x6592
CJK UNIFIED IDEOGRAPH:'DDBE:56766:&#x65D3
CJK UNIFIED IDEOGRAPH:'DDBF:56767:&#x6686
CJK UNIFIED IDEOGRAPH:'DDC0:56768:&#x668C
CJK UNIFIED IDEOGRAPH:'DDC1:56769:&#x6695
CJK UNIFIED IDEOGRAPH:'DDC2:56770:&#x6690
CJK UNIFIED IDEOGRAPH:'DDC3:56771:&#x668B
CJK UNIFIED IDEOGRAPH:'DDC4:56772:&#x668A
CJK UNIFIED IDEOGRAPH:'DDC5:56773:&#x6699
CJK UNIFIED IDEOGRAPH:'DDC6:56774:&#x6694
CJK UNIFIED IDEOGRAPH:'DDC7:56775:&#x6678
CJK UNIFIED IDEOGRAPH:'DDC8:56776:&#x6720
CJK UNIFIED IDEOGRAPH:'DDC9:56777:&#x6966
CJK UNIFIED IDEOGRAPH:'DDCA:56778:&#x695F
CJK UNIFIED IDEOGRAPH:'DDCB:56779:&#x6938
CJK UNIFIED IDEOGRAPH:'DDCC:56780:&#x694E
CJK UNIFIED IDEOGRAPH:'DDCD:56781:&#x6962
CJK UNIFIED IDEOGRAPH:'DDCE:56782:&#x6971
CJK UNIFIED IDEOGRAPH:'DDCF:56783:&#x693F
CJK UNIFIED IDEOGRAPH:'DDD0:56784:&#x6945
CJK UNIFIED IDEOGRAPH:'DDD1:56785:&#x696A
CJK UNIFIED IDEOGRAPH:'DDD2:56786:&#x6939
CJK UNIFIED IDEOGRAPH:'DDD3:56787:&#x6942
CJK UNIFIED IDEOGRAPH:'DDD4:56788:&#x6957
CJK UNIFIED IDEOGRAPH:'DDD5:56789:&#x6959
CJK UNIFIED IDEOGRAPH:'DDD6:56790:&#x697A
CJK UNIFIED IDEOGRAPH:'DDD7:56791:&#x6948
CJK UNIFIED IDEOGRAPH:'DDD8:56792:&#x6949
CJK UNIFIED IDEOGRAPH:'DDD9:56793:&#x6935
CJK UNIFIED IDEOGRAPH:'DDDA:56794:&#x696C
CJK UNIFIED IDEOGRAPH:'DDDB:56795:&#x6933
CJK UNIFIED IDEOGRAPH:'DDDC:56796:&#x693D
CJK UNIFIED IDEOGRAPH:'DDDD:56797:&#x6965
CJK UNIFIED IDEOGRAPH:'DDDE:56798:&#x68F0
CJK UNIFIED IDEOGRAPH:'DDDF:56799:&#x6978
CJK UNIFIED IDEOGRAPH:'DDE0:56800:&#x6934
CJK UNIFIED IDEOGRAPH:'DDE1:56801:&#x6969
CJK UNIFIED IDEOGRAPH:'DDE2:56802:&#x6940
CJK UNIFIED IDEOGRAPH:'DDE3:56803:&#x696F
CJK UNIFIED IDEOGRAPH:'DDE4:56804:&#x6944
CJK UNIFIED IDEOGRAPH:'DDE5:56805:&#x6976
CJK UNIFIED IDEOGRAPH:'DDE6:56806:&#x6958
CJK UNIFIED IDEOGRAPH:'DDE7:56807:&#x6941
CJK UNIFIED IDEOGRAPH:'DDE8:56808:&#x6974
CJK UNIFIED IDEOGRAPH:'DDE9:56809:&#x694C
CJK UNIFIED IDEOGRAPH:'DDEA:56810:&#x693B
CJK UNIFIED IDEOGRAPH:'DDEB:56811:&#x694B
CJK UNIFIED IDEOGRAPH:'DDEC:56812:&#x6937
CJK UNIFIED IDEOGRAPH:'DDED:56813:&#x695C
CJK UNIFIED IDEOGRAPH:'DDEE:56814:&#x694F
CJK UNIFIED IDEOGRAPH:'DDEF:56815:&#x6951
CJK UNIFIED IDEOGRAPH:'DDF0:56816:&#x6932
CJK UNIFIED IDEOGRAPH:'DDF1:56817:&#x6952
CJK UNIFIED IDEOGRAPH:'DDF2:56818:&#x692F
CJK UNIFIED IDEOGRAPH:'DDF3:56819:&#x697B
CJK UNIFIED IDEOGRAPH:'DDF4:56820:&#x693C
CJK UNIFIED IDEOGRAPH:'DDF5:56821:&#x6B46
CJK UNIFIED IDEOGRAPH:'DDF6:56822:&#x6B45
CJK UNIFIED IDEOGRAPH:'DDF7:56823:&#x6B43
CJK UNIFIED IDEOGRAPH:'DDF8:56824:&#x6B42
CJK UNIFIED IDEOGRAPH:'DDF9:56825:&#x6B48
CJK UNIFIED IDEOGRAPH:'DDFA:56826:&#x6B41
CJK UNIFIED IDEOGRAPH:'DDFB:56827:&#x6B9B
CJK COMPATIBILITY IDEOGRAPH:'DDFC:56828:&#xFA0D
CJK UNIFIED IDEOGRAPH:'DDFD:56829:&#x6BFB
CJK UNIFIED IDEOGRAPH:'DDFE:56830:&#x6BFC
CJK UNIFIED IDEOGRAPH:'DE40:56896:&#x6BF9
CJK UNIFIED IDEOGRAPH:'DE41:56897:&#x6BF7
CJK UNIFIED IDEOGRAPH:'DE42:56898:&#x6BF8
CJK UNIFIED IDEOGRAPH:'DE43:56899:&#x6E9B
CJK UNIFIED IDEOGRAPH:'DE44:56900:&#x6ED6
CJK UNIFIED IDEOGRAPH:'DE45:56901:&#x6EC8
CJK UNIFIED IDEOGRAPH:'DE46:56902:&#x6E8F
CJK UNIFIED IDEOGRAPH:'DE47:56903:&#x6EC0
CJK UNIFIED IDEOGRAPH:'DE48:56904:&#x6E9F
CJK UNIFIED IDEOGRAPH:'DE49:56905:&#x6E93
CJK UNIFIED IDEOGRAPH:'DE4A:56906:&#x6E94
CJK UNIFIED IDEOGRAPH:'DE4B:56907:&#x6EA0
CJK UNIFIED IDEOGRAPH:'DE4C:56908:&#x6EB1
CJK UNIFIED IDEOGRAPH:'DE4D:56909:&#x6EB9
CJK UNIFIED IDEOGRAPH:'DE4E:56910:&#x6EC6
CJK UNIFIED IDEOGRAPH:'DE4F:56911:&#x6ED2
CJK UNIFIED IDEOGRAPH:'DE50:56912:&#x6EBD
CJK UNIFIED IDEOGRAPH:'DE51:56913:&#x6EC1
CJK UNIFIED IDEOGRAPH:'DE52:56914:&#x6E9E
CJK UNIFIED IDEOGRAPH:'DE53:56915:&#x6EC9
CJK UNIFIED IDEOGRAPH:'DE54:56916:&#x6EB7
CJK UNIFIED IDEOGRAPH:'DE55:56917:&#x6EB0
CJK UNIFIED IDEOGRAPH:'DE56:56918:&#x6ECD
CJK UNIFIED IDEOGRAPH:'DE57:56919:&#x6EA6
CJK UNIFIED IDEOGRAPH:'DE58:56920:&#x6ECF
CJK UNIFIED IDEOGRAPH:'DE59:56921:&#x6EB2
CJK UNIFIED IDEOGRAPH:'DE5A:56922:&#x6EBE
CJK UNIFIED IDEOGRAPH:'DE5B:56923:&#x6EC3
CJK UNIFIED IDEOGRAPH:'DE5C:56924:&#x6EDC
CJK UNIFIED IDEOGRAPH:'DE5D:56925:&#x6ED8
CJK UNIFIED IDEOGRAPH:'DE5E:56926:&#x6E99
CJK UNIFIED IDEOGRAPH:'DE5F:56927:&#x6E92
CJK UNIFIED IDEOGRAPH:'DE60:56928:&#x6E8E
CJK UNIFIED IDEOGRAPH:'DE61:56929:&#x6E8D
CJK UNIFIED IDEOGRAPH:'DE62:56930:&#x6EA4
CJK UNIFIED IDEOGRAPH:'DE63:56931:&#x6EA1
CJK UNIFIED IDEOGRAPH:'DE64:56932:&#x6EBF
CJK UNIFIED IDEOGRAPH:'DE65:56933:&#x6EB3
CJK UNIFIED IDEOGRAPH:'DE66:56934:&#x6ED0
CJK UNIFIED IDEOGRAPH:'DE67:56935:&#x6ECA
CJK UNIFIED IDEOGRAPH:'DE68:56936:&#x6E97
CJK UNIFIED IDEOGRAPH:'DE69:56937:&#x6EAE
CJK UNIFIED IDEOGRAPH:'DE6A:56938:&#x6EA3
CJK UNIFIED IDEOGRAPH:'DE6B:56939:&#x7147
CJK UNIFIED IDEOGRAPH:'DE6C:56940:&#x7154
CJK UNIFIED IDEOGRAPH:'DE6D:56941:&#x7152
CJK UNIFIED IDEOGRAPH:'DE6E:56942:&#x7163
CJK UNIFIED IDEOGRAPH:'DE6F:56943:&#x7160
CJK UNIFIED IDEOGRAPH:'DE70:56944:&#x7141
CJK UNIFIED IDEOGRAPH:'DE71:56945:&#x715D
CJK UNIFIED IDEOGRAPH:'DE72:56946:&#x7162
CJK UNIFIED IDEOGRAPH:'DE73:56947:&#x7172
CJK UNIFIED IDEOGRAPH:'DE74:56948:&#x7178
CJK UNIFIED IDEOGRAPH:'DE75:56949:&#x716A
CJK UNIFIED IDEOGRAPH:'DE76:56950:&#x7161
CJK UNIFIED IDEOGRAPH:'DE77:56951:&#x7142
CJK UNIFIED IDEOGRAPH:'DE78:56952:&#x7158
CJK UNIFIED IDEOGRAPH:'DE79:56953:&#x7143
CJK UNIFIED IDEOGRAPH:'DE7A:56954:&#x714B
CJK UNIFIED IDEOGRAPH:'DE7B:56955:&#x7170
CJK UNIFIED IDEOGRAPH:'DE7C:56956:&#x715F
CJK UNIFIED IDEOGRAPH:'DE7D:56957:&#x7150
CJK UNIFIED IDEOGRAPH:'DE7E:56958:&#x7153
CJK UNIFIED IDEOGRAPH:'DEA1:56993:&#x7144
CJK UNIFIED IDEOGRAPH:'DEA2:56994:&#x714D
CJK UNIFIED IDEOGRAPH:'DEA3:56995:&#x715A
CJK UNIFIED IDEOGRAPH:'DEA4:56996:&#x724F
CJK UNIFIED IDEOGRAPH:'DEA5:56997:&#x728D
CJK UNIFIED IDEOGRAPH:'DEA6:56998:&#x728C
CJK UNIFIED IDEOGRAPH:'DEA7:56999:&#x7291
CJK UNIFIED IDEOGRAPH:'DEA8:57000:&#x7290
CJK UNIFIED IDEOGRAPH:'DEA9:57001:&#x728E
CJK UNIFIED IDEOGRAPH:'DEAA:57002:&#x733C
CJK UNIFIED IDEOGRAPH:'DEAB:57003:&#x7342
CJK UNIFIED IDEOGRAPH:'DEAC:57004:&#x733B
CJK UNIFIED IDEOGRAPH:'DEAD:57005:&#x733A
CJK UNIFIED IDEOGRAPH:'DEAE:57006:&#x7340
CJK UNIFIED IDEOGRAPH:'DEAF:57007:&#x734A
CJK UNIFIED IDEOGRAPH:'DEB0:57008:&#x7349
CJK UNIFIED IDEOGRAPH:'DEB1:57009:&#x7444
CJK UNIFIED IDEOGRAPH:'DEB2:57010:&#x744A
CJK UNIFIED IDEOGRAPH:'DEB3:57011:&#x744B
CJK UNIFIED IDEOGRAPH:'DEB4:57012:&#x7452
CJK UNIFIED IDEOGRAPH:'DEB5:57013:&#x7451
CJK UNIFIED IDEOGRAPH:'DEB6:57014:&#x7457
CJK UNIFIED IDEOGRAPH:'DEB7:57015:&#x7440
CJK UNIFIED IDEOGRAPH:'DEB8:57016:&#x744F
CJK UNIFIED IDEOGRAPH:'DEB9:57017:&#x7450
CJK UNIFIED IDEOGRAPH:'DEBA:57018:&#x744E
CJK UNIFIED IDEOGRAPH:'DEBB:57019:&#x7442
CJK UNIFIED IDEOGRAPH:'DEBC:57020:&#x7446
CJK UNIFIED IDEOGRAPH:'DEBD:57021:&#x744D
CJK UNIFIED IDEOGRAPH:'DEBE:57022:&#x7454
CJK UNIFIED IDEOGRAPH:'DEBF:57023:&#x74E1
CJK UNIFIED IDEOGRAPH:'DEC0:57024:&#x74FF
CJK UNIFIED IDEOGRAPH:'DEC1:57025:&#x74FE
CJK UNIFIED IDEOGRAPH:'DEC2:57026:&#x74FD
CJK UNIFIED IDEOGRAPH:'DEC3:57027:&#x751D
CJK UNIFIED IDEOGRAPH:'DEC4:57028:&#x7579
CJK UNIFIED IDEOGRAPH:'DEC5:57029:&#x7577
CJK UNIFIED IDEOGRAPH:'DEC6:57030:&#x6983
CJK UNIFIED IDEOGRAPH:'DEC7:57031:&#x75EF
CJK UNIFIED IDEOGRAPH:'DEC8:57032:&#x760F
CJK UNIFIED IDEOGRAPH:'DEC9:57033:&#x7603
CJK UNIFIED IDEOGRAPH:'DECA:57034:&#x75F7
CJK UNIFIED IDEOGRAPH:'DECB:57035:&#x75FE
CJK UNIFIED IDEOGRAPH:'DECC:57036:&#x75FC
CJK UNIFIED IDEOGRAPH:'DECD:57037:&#x75F9
CJK UNIFIED IDEOGRAPH:'DECE:57038:&#x75F8
CJK UNIFIED IDEOGRAPH:'DECF:57039:&#x7610
CJK UNIFIED IDEOGRAPH:'DED0:57040:&#x75FB
CJK UNIFIED IDEOGRAPH:'DED1:57041:&#x75F6
CJK UNIFIED IDEOGRAPH:'DED2:57042:&#x75ED
CJK UNIFIED IDEOGRAPH:'DED3:57043:&#x75F5
CJK UNIFIED IDEOGRAPH:'DED4:57044:&#x75FD
CJK UNIFIED IDEOGRAPH:'DED5:57045:&#x7699
CJK UNIFIED IDEOGRAPH:'DED6:57046:&#x76B5
CJK UNIFIED IDEOGRAPH:'DED7:57047:&#x76DD
CJK UNIFIED IDEOGRAPH:'DED8:57048:&#x7755
CJK UNIFIED IDEOGRAPH:'DED9:57049:&#x775F
CJK UNIFIED IDEOGRAPH:'DEDA:57050:&#x7760
CJK UNIFIED IDEOGRAPH:'DEDB:57051:&#x7752
CJK UNIFIED IDEOGRAPH:'DEDC:57052:&#x7756
CJK UNIFIED IDEOGRAPH:'DEDD:57053:&#x775A
CJK UNIFIED IDEOGRAPH:'DEDE:57054:&#x7769
CJK UNIFIED IDEOGRAPH:'DEDF:57055:&#x7767
CJK UNIFIED IDEOGRAPH:'DEE0:57056:&#x7754
CJK UNIFIED IDEOGRAPH:'DEE1:57057:&#x7759
CJK UNIFIED IDEOGRAPH:'DEE2:57058:&#x776D
CJK UNIFIED IDEOGRAPH:'DEE3:57059:&#x77E0
CJK UNIFIED IDEOGRAPH:'DEE4:57060:&#x7887
CJK UNIFIED IDEOGRAPH:'DEE5:57061:&#x789A
CJK UNIFIED IDEOGRAPH:'DEE6:57062:&#x7894
CJK UNIFIED IDEOGRAPH:'DEE7:57063:&#x788F
CJK UNIFIED IDEOGRAPH:'DEE8:57064:&#x7884
CJK UNIFIED IDEOGRAPH:'DEE9:57065:&#x7895
CJK UNIFIED IDEOGRAPH:'DEEA:57066:&#x7885
CJK UNIFIED IDEOGRAPH:'DEEB:57067:&#x7886
CJK UNIFIED IDEOGRAPH:'DEEC:57068:&#x78A1
CJK UNIFIED IDEOGRAPH:'DEED:57069:&#x7883
CJK UNIFIED IDEOGRAPH:'DEEE:57070:&#x7879
CJK UNIFIED IDEOGRAPH:'DEEF:57071:&#x7899
CJK UNIFIED IDEOGRAPH:'DEF0:57072:&#x7880
CJK UNIFIED IDEOGRAPH:'DEF1:57073:&#x7896
CJK UNIFIED IDEOGRAPH:'DEF2:57074:&#x787B
CJK UNIFIED IDEOGRAPH:'DEF3:57075:&#x797C
CJK UNIFIED IDEOGRAPH:'DEF4:57076:&#x7982
CJK UNIFIED IDEOGRAPH:'DEF5:57077:&#x797D
CJK UNIFIED IDEOGRAPH:'DEF6:57078:&#x7979
CJK UNIFIED IDEOGRAPH:'DEF7:57079:&#x7A11
CJK UNIFIED IDEOGRAPH:'DEF8:57080:&#x7A18
CJK UNIFIED IDEOGRAPH:'DEF9:57081:&#x7A19
CJK UNIFIED IDEOGRAPH:'DEFA:57082:&#x7A12
CJK UNIFIED IDEOGRAPH:'DEFB:57083:&#x7A17
CJK UNIFIED IDEOGRAPH:'DEFC:57084:&#x7A15
CJK UNIFIED IDEOGRAPH:'DEFD:57085:&#x7A22
CJK UNIFIED IDEOGRAPH:'DEFE:57086:&#x7A13
CJK UNIFIED IDEOGRAPH:'DF40:57152:&#x7A1B
CJK UNIFIED IDEOGRAPH:'DF41:57153:&#x7A10
CJK UNIFIED IDEOGRAPH:'DF42:57154:&#x7AA3
CJK UNIFIED IDEOGRAPH:'DF43:57155:&#x7AA2
CJK UNIFIED IDEOGRAPH:'DF44:57156:&#x7A9E
CJK UNIFIED IDEOGRAPH:'DF45:57157:&#x7AEB
CJK UNIFIED IDEOGRAPH:'DF46:57158:&#x7B66
CJK UNIFIED IDEOGRAPH:'DF47:57159:&#x7B64
CJK UNIFIED IDEOGRAPH:'DF48:57160:&#x7B6D
CJK UNIFIED IDEOGRAPH:'DF49:57161:&#x7B74
CJK UNIFIED IDEOGRAPH:'DF4A:57162:&#x7B69
CJK UNIFIED IDEOGRAPH:'DF4B:57163:&#x7B72
CJK UNIFIED IDEOGRAPH:'DF4C:57164:&#x7B65
CJK UNIFIED IDEOGRAPH:'DF4D:57165:&#x7B73
CJK UNIFIED IDEOGRAPH:'DF4E:57166:&#x7B71
CJK UNIFIED IDEOGRAPH:'DF4F:57167:&#x7B70
CJK UNIFIED IDEOGRAPH:'DF50:57168:&#x7B61
CJK UNIFIED IDEOGRAPH:'DF51:57169:&#x7B78
CJK UNIFIED IDEOGRAPH:'DF52:57170:&#x7B76
CJK UNIFIED IDEOGRAPH:'DF53:57171:&#x7B63
CJK UNIFIED IDEOGRAPH:'DF54:57172:&#x7CB2
CJK UNIFIED IDEOGRAPH:'DF55:57173:&#x7CB4
CJK UNIFIED IDEOGRAPH:'DF56:57174:&#x7CAF
CJK UNIFIED IDEOGRAPH:'DF57:57175:&#x7D88
CJK UNIFIED IDEOGRAPH:'DF58:57176:&#x7D86
CJK UNIFIED IDEOGRAPH:'DF59:57177:&#x7D80
CJK UNIFIED IDEOGRAPH:'DF5A:57178:&#x7D8D
CJK UNIFIED IDEOGRAPH:'DF5B:57179:&#x7D7F
CJK UNIFIED IDEOGRAPH:'DF5C:57180:&#x7D85
CJK UNIFIED IDEOGRAPH:'DF5D:57181:&#x7D7A
CJK UNIFIED IDEOGRAPH:'DF5E:57182:&#x7D8E
CJK UNIFIED IDEOGRAPH:'DF5F:57183:&#x7D7B
CJK UNIFIED IDEOGRAPH:'DF60:57184:&#x7D83
CJK UNIFIED IDEOGRAPH:'DF61:57185:&#x7D7C
CJK UNIFIED IDEOGRAPH:'DF62:57186:&#x7D8C
CJK UNIFIED IDEOGRAPH:'DF63:57187:&#x7D94
CJK UNIFIED IDEOGRAPH:'DF64:57188:&#x7D84
CJK UNIFIED IDEOGRAPH:'DF65:57189:&#x7D7D
CJK UNIFIED IDEOGRAPH:'DF66:57190:&#x7D92
CJK UNIFIED IDEOGRAPH:'DF67:57191:&#x7F6D
CJK UNIFIED IDEOGRAPH:'DF68:57192:&#x7F6B
CJK UNIFIED IDEOGRAPH:'DF69:57193:&#x7F67
CJK UNIFIED IDEOGRAPH:'DF6A:57194:&#x7F68
CJK UNIFIED IDEOGRAPH:'DF6B:57195:&#x7F6C
CJK UNIFIED IDEOGRAPH:'DF6C:57196:&#x7FA6
CJK UNIFIED IDEOGRAPH:'DF6D:57197:&#x7FA5
CJK UNIFIED IDEOGRAPH:'DF6E:57198:&#x7FA7
CJK UNIFIED IDEOGRAPH:'DF6F:57199:&#x7FDB
CJK UNIFIED IDEOGRAPH:'DF70:57200:&#x7FDC
CJK UNIFIED IDEOGRAPH:'DF71:57201:&#x8021
CJK UNIFIED IDEOGRAPH:'DF72:57202:&#x8164
CJK UNIFIED IDEOGRAPH:'DF73:57203:&#x8160
CJK UNIFIED IDEOGRAPH:'DF74:57204:&#x8177
CJK UNIFIED IDEOGRAPH:'DF75:57205:&#x815C
CJK UNIFIED IDEOGRAPH:'DF76:57206:&#x8169
CJK UNIFIED IDEOGRAPH:'DF77:57207:&#x815B
CJK UNIFIED IDEOGRAPH:'DF78:57208:&#x8162
CJK UNIFIED IDEOGRAPH:'DF79:57209:&#x8172
CJK UNIFIED IDEOGRAPH:'DF7A:57210:&#x6721
CJK UNIFIED IDEOGRAPH:'DF7B:57211:&#x815E
CJK UNIFIED IDEOGRAPH:'DF7C:57212:&#x8176
CJK UNIFIED IDEOGRAPH:'DF7D:57213:&#x8167
CJK UNIFIED IDEOGRAPH:'DF7E:57214:&#x816F
CJK UNIFIED IDEOGRAPH:'DFA1:57249:&#x8144
CJK UNIFIED IDEOGRAPH:'DFA2:57250:&#x8161
CJK UNIFIED IDEOGRAPH:'DFA3:57251:&#x821D
CJK UNIFIED IDEOGRAPH:'DFA4:57252:&#x8249
CJK UNIFIED IDEOGRAPH:'DFA5:57253:&#x8244
CJK UNIFIED IDEOGRAPH:'DFA6:57254:&#x8240
CJK UNIFIED IDEOGRAPH:'DFA7:57255:&#x8242
CJK UNIFIED IDEOGRAPH:'DFA8:57256:&#x8245
CJK UNIFIED IDEOGRAPH:'DFA9:57257:&#x84F1
CJK UNIFIED IDEOGRAPH:'DFAA:57258:&#x843F
CJK UNIFIED IDEOGRAPH:'DFAB:57259:&#x8456
CJK UNIFIED IDEOGRAPH:'DFAC:57260:&#x8476
CJK UNIFIED IDEOGRAPH:'DFAD:57261:&#x8479
CJK UNIFIED IDEOGRAPH:'DFAE:57262:&#x848F
CJK UNIFIED IDEOGRAPH:'DFAF:57263:&#x848D
CJK UNIFIED IDEOGRAPH:'DFB0:57264:&#x8465
CJK UNIFIED IDEOGRAPH:'DFB1:57265:&#x8451
CJK UNIFIED IDEOGRAPH:'DFB2:57266:&#x8440
CJK UNIFIED IDEOGRAPH:'DFB3:57267:&#x8486
CJK UNIFIED IDEOGRAPH:'DFB4:57268:&#x8467
CJK UNIFIED IDEOGRAPH:'DFB5:57269:&#x8430
CJK UNIFIED IDEOGRAPH:'DFB6:57270:&#x844D
CJK UNIFIED IDEOGRAPH:'DFB7:57271:&#x847D
CJK UNIFIED IDEOGRAPH:'DFB8:57272:&#x845A
CJK UNIFIED IDEOGRAPH:'DFB9:57273:&#x8459
CJK UNIFIED IDEOGRAPH:'DFBA:57274:&#x8474
CJK UNIFIED IDEOGRAPH:'DFBB:57275:&#x8473
CJK UNIFIED IDEOGRAPH:'DFBC:57276:&#x845D
CJK UNIFIED IDEOGRAPH:'DFBD:57277:&#x8507
CJK UNIFIED IDEOGRAPH:'DFBE:57278:&#x845E
CJK UNIFIED IDEOGRAPH:'DFBF:57279:&#x8437
CJK UNIFIED IDEOGRAPH:'DFC0:57280:&#x843A
CJK UNIFIED IDEOGRAPH:'DFC1:57281:&#x8434
CJK UNIFIED IDEOGRAPH:'DFC2:57282:&#x847A
CJK UNIFIED IDEOGRAPH:'DFC3:57283:&#x8443
CJK UNIFIED IDEOGRAPH:'DFC4:57284:&#x8478
CJK UNIFIED IDEOGRAPH:'DFC5:57285:&#x8432
CJK UNIFIED IDEOGRAPH:'DFC6:57286:&#x8445
CJK UNIFIED IDEOGRAPH:'DFC7:57287:&#x8429
CJK UNIFIED IDEOGRAPH:'DFC8:57288:&#x83D9
CJK UNIFIED IDEOGRAPH:'DFC9:57289:&#x844B
CJK UNIFIED IDEOGRAPH:'DFCA:57290:&#x842F
CJK UNIFIED IDEOGRAPH:'DFCB:57291:&#x8442
CJK UNIFIED IDEOGRAPH:'DFCC:57292:&#x842D
CJK UNIFIED IDEOGRAPH:'DFCD:57293:&#x845F
CJK UNIFIED IDEOGRAPH:'DFCE:57294:&#x8470
CJK UNIFIED IDEOGRAPH:'DFCF:57295:&#x8439
CJK UNIFIED IDEOGRAPH:'DFD0:57296:&#x844E
CJK UNIFIED IDEOGRAPH:'DFD1:57297:&#x844C
CJK UNIFIED IDEOGRAPH:'DFD2:57298:&#x8452
CJK UNIFIED IDEOGRAPH:'DFD3:57299:&#x846F
CJK UNIFIED IDEOGRAPH:'DFD4:57300:&#x84C5
CJK UNIFIED IDEOGRAPH:'DFD5:57301:&#x848E
CJK UNIFIED IDEOGRAPH:'DFD6:57302:&#x843B
CJK UNIFIED IDEOGRAPH:'DFD7:57303:&#x8447
CJK UNIFIED IDEOGRAPH:'DFD8:57304:&#x8436
CJK UNIFIED IDEOGRAPH:'DFD9:57305:&#x8433
CJK UNIFIED IDEOGRAPH:'DFDA:57306:&#x8468
CJK UNIFIED IDEOGRAPH:'DFDB:57307:&#x847E
CJK UNIFIED IDEOGRAPH:'DFDC:57308:&#x8444
CJK UNIFIED IDEOGRAPH:'DFDD:57309:&#x842B
CJK UNIFIED IDEOGRAPH:'DFDE:57310:&#x8460
CJK UNIFIED IDEOGRAPH:'DFDF:57311:&#x8454
CJK UNIFIED IDEOGRAPH:'DFE0:57312:&#x846E
CJK UNIFIED IDEOGRAPH:'DFE1:57313:&#x8450
CJK UNIFIED IDEOGRAPH:'DFE2:57314:&#x870B
CJK UNIFIED IDEOGRAPH:'DFE3:57315:&#x8704
CJK UNIFIED IDEOGRAPH:'DFE4:57316:&#x86F7
CJK UNIFIED IDEOGRAPH:'DFE5:57317:&#x870C
CJK UNIFIED IDEOGRAPH:'DFE6:57318:&#x86FA
CJK UNIFIED IDEOGRAPH:'DFE7:57319:&#x86D6
CJK UNIFIED IDEOGRAPH:'DFE8:57320:&#x86F5
CJK UNIFIED IDEOGRAPH:'DFE9:57321:&#x874D
CJK UNIFIED IDEOGRAPH:'DFEA:57322:&#x86F8
CJK UNIFIED IDEOGRAPH:'DFEB:57323:&#x870E
CJK UNIFIED IDEOGRAPH:'DFEC:57324:&#x8709
CJK UNIFIED IDEOGRAPH:'DFED:57325:&#x8701
CJK UNIFIED IDEOGRAPH:'DFEE:57326:&#x86F6
CJK UNIFIED IDEOGRAPH:'DFEF:57327:&#x870D
CJK UNIFIED IDEOGRAPH:'DFF0:57328:&#x8705
CJK UNIFIED IDEOGRAPH:'DFF1:57329:&#x88D6
CJK UNIFIED IDEOGRAPH:'DFF2:57330:&#x88CB
CJK UNIFIED IDEOGRAPH:'DFF3:57331:&#x88CD
CJK UNIFIED IDEOGRAPH:'DFF4:57332:&#x88CE
CJK UNIFIED IDEOGRAPH:'DFF5:57333:&#x88DE
CJK UNIFIED IDEOGRAPH:'DFF6:57334:&#x88DB
CJK UNIFIED IDEOGRAPH:'DFF7:57335:&#x88DA
CJK UNIFIED IDEOGRAPH:'DFF8:57336:&#x88CC
CJK UNIFIED IDEOGRAPH:'DFF9:57337:&#x88D0
CJK UNIFIED IDEOGRAPH:'DFFA:57338:&#x8985
CJK UNIFIED IDEOGRAPH:'DFFB:57339:&#x899B
CJK UNIFIED IDEOGRAPH:'DFFC:57340:&#x89DF
CJK UNIFIED IDEOGRAPH:'DFFD:57341:&#x89E5
CJK UNIFIED IDEOGRAPH:'DFFE:57342:&#x89E4
CJK UNIFIED IDEOGRAPH:'E040:57408:&#x89E1
CJK UNIFIED IDEOGRAPH:'E041:57409:&#x89E0
CJK UNIFIED IDEOGRAPH:'E042:57410:&#x89E2
CJK UNIFIED IDEOGRAPH:'E043:57411:&#x89DC
CJK UNIFIED IDEOGRAPH:'E044:57412:&#x89E6
CJK UNIFIED IDEOGRAPH:'E045:57413:&#x8A76
CJK UNIFIED IDEOGRAPH:'E046:57414:&#x8A86
CJK UNIFIED IDEOGRAPH:'E047:57415:&#x8A7F
CJK UNIFIED IDEOGRAPH:'E048:57416:&#x8A61
CJK UNIFIED IDEOGRAPH:'E049:57417:&#x8A3F
CJK UNIFIED IDEOGRAPH:'E04A:57418:&#x8A77
CJK UNIFIED IDEOGRAPH:'E04B:57419:&#x8A82
CJK UNIFIED IDEOGRAPH:'E04C:57420:&#x8A84
CJK UNIFIED IDEOGRAPH:'E04D:57421:&#x8A75
CJK UNIFIED IDEOGRAPH:'E04E:57422:&#x8A83
CJK UNIFIED IDEOGRAPH:'E04F:57423:&#x8A81
CJK UNIFIED IDEOGRAPH:'E050:57424:&#x8A74
CJK UNIFIED IDEOGRAPH:'E051:57425:&#x8A7A
CJK UNIFIED IDEOGRAPH:'E052:57426:&#x8C3C
CJK UNIFIED IDEOGRAPH:'E053:57427:&#x8C4B
CJK UNIFIED IDEOGRAPH:'E054:57428:&#x8C4A
CJK UNIFIED IDEOGRAPH:'E055:57429:&#x8C65
CJK UNIFIED IDEOGRAPH:'E056:57430:&#x8C64
CJK UNIFIED IDEOGRAPH:'E057:57431:&#x8C66
CJK UNIFIED IDEOGRAPH:'E058:57432:&#x8C86
CJK UNIFIED IDEOGRAPH:'E059:57433:&#x8C84
CJK UNIFIED IDEOGRAPH:'E05A:57434:&#x8C85
CJK UNIFIED IDEOGRAPH:'E05B:57435:&#x8CCC
CJK UNIFIED IDEOGRAPH:'E05C:57436:&#x8D68
CJK UNIFIED IDEOGRAPH:'E05D:57437:&#x8D69
CJK UNIFIED IDEOGRAPH:'E05E:57438:&#x8D91
CJK UNIFIED IDEOGRAPH:'E05F:57439:&#x8D8C
CJK UNIFIED IDEOGRAPH:'E060:57440:&#x8D8E
CJK UNIFIED IDEOGRAPH:'E061:57441:&#x8D8F
CJK UNIFIED IDEOGRAPH:'E062:57442:&#x8D8D
CJK UNIFIED IDEOGRAPH:'E063:57443:&#x8D93
CJK UNIFIED IDEOGRAPH:'E064:57444:&#x8D94
CJK UNIFIED IDEOGRAPH:'E065:57445:&#x8D90
CJK UNIFIED IDEOGRAPH:'E066:57446:&#x8D92
CJK UNIFIED IDEOGRAPH:'E067:57447:&#x8DF0
CJK UNIFIED IDEOGRAPH:'E068:57448:&#x8DE0
CJK UNIFIED IDEOGRAPH:'E069:57449:&#x8DEC
CJK UNIFIED IDEOGRAPH:'E06A:57450:&#x8DF1
CJK UNIFIED IDEOGRAPH:'E06B:57451:&#x8DEE
CJK UNIFIED IDEOGRAPH:'E06C:57452:&#x8DD0
CJK UNIFIED IDEOGRAPH:'E06D:57453:&#x8DE9
CJK UNIFIED IDEOGRAPH:'E06E:57454:&#x8DE3
CJK UNIFIED IDEOGRAPH:'E06F:57455:&#x8DE2
CJK UNIFIED IDEOGRAPH:'E070:57456:&#x8DE7
CJK UNIFIED IDEOGRAPH:'E071:57457:&#x8DF2
CJK UNIFIED IDEOGRAPH:'E072:57458:&#x8DEB
CJK UNIFIED IDEOGRAPH:'E073:57459:&#x8DF4
CJK UNIFIED IDEOGRAPH:'E074:57460:&#x8F06
CJK UNIFIED IDEOGRAPH:'E075:57461:&#x8EFF
CJK UNIFIED IDEOGRAPH:'E076:57462:&#x8F01
CJK UNIFIED IDEOGRAPH:'E077:57463:&#x8F00
CJK UNIFIED IDEOGRAPH:'E078:57464:&#x8F05
CJK UNIFIED IDEOGRAPH:'E079:57465:&#x8F07
CJK UNIFIED IDEOGRAPH:'E07A:57466:&#x8F08
CJK UNIFIED IDEOGRAPH:'E07B:57467:&#x8F02
CJK UNIFIED IDEOGRAPH:'E07C:57468:&#x8F0B
CJK UNIFIED IDEOGRAPH:'E07D:57469:&#x9052
CJK UNIFIED IDEOGRAPH:'E07E:57470:&#x903F
CJK UNIFIED IDEOGRAPH:'E0A1:57505:&#x9044
CJK UNIFIED IDEOGRAPH:'E0A2:57506:&#x9049
CJK UNIFIED IDEOGRAPH:'E0A3:57507:&#x903D
CJK UNIFIED IDEOGRAPH:'E0A4:57508:&#x9110
CJK UNIFIED IDEOGRAPH:'E0A5:57509:&#x910D
CJK UNIFIED IDEOGRAPH:'E0A6:57510:&#x910F
CJK UNIFIED IDEOGRAPH:'E0A7:57511:&#x9111
CJK UNIFIED IDEOGRAPH:'E0A8:57512:&#x9116
CJK UNIFIED IDEOGRAPH:'E0A9:57513:&#x9114
CJK UNIFIED IDEOGRAPH:'E0AA:57514:&#x910B
CJK UNIFIED IDEOGRAPH:'E0AB:57515:&#x910E
CJK UNIFIED IDEOGRAPH:'E0AC:57516:&#x916E
CJK UNIFIED IDEOGRAPH:'E0AD:57517:&#x916F
CJK UNIFIED IDEOGRAPH:'E0AE:57518:&#x9248
CJK UNIFIED IDEOGRAPH:'E0AF:57519:&#x9252
CJK UNIFIED IDEOGRAPH:'E0B0:57520:&#x9230
CJK UNIFIED IDEOGRAPH:'E0B1:57521:&#x923A
CJK UNIFIED IDEOGRAPH:'E0B2:57522:&#x9266
CJK UNIFIED IDEOGRAPH:'E0B3:57523:&#x9233
CJK UNIFIED IDEOGRAPH:'E0B4:57524:&#x9265
CJK UNIFIED IDEOGRAPH:'E0B5:57525:&#x925E
CJK UNIFIED IDEOGRAPH:'E0B6:57526:&#x9283
CJK UNIFIED IDEOGRAPH:'E0B7:57527:&#x922E
CJK UNIFIED IDEOGRAPH:'E0B8:57528:&#x924A
CJK UNIFIED IDEOGRAPH:'E0B9:57529:&#x9246
CJK UNIFIED IDEOGRAPH:'E0BA:57530:&#x926D
CJK UNIFIED IDEOGRAPH:'E0BB:57531:&#x926C
CJK UNIFIED IDEOGRAPH:'E0BC:57532:&#x924F
CJK UNIFIED IDEOGRAPH:'E0BD:57533:&#x9260
CJK UNIFIED IDEOGRAPH:'E0BE:57534:&#x9267
CJK UNIFIED IDEOGRAPH:'E0BF:57535:&#x926F
CJK UNIFIED IDEOGRAPH:'E0C0:57536:&#x9236
CJK UNIFIED IDEOGRAPH:'E0C1:57537:&#x9261
CJK UNIFIED IDEOGRAPH:'E0C2:57538:&#x9270
CJK UNIFIED IDEOGRAPH:'E0C3:57539:&#x9231
CJK UNIFIED IDEOGRAPH:'E0C4:57540:&#x9254
CJK UNIFIED IDEOGRAPH:'E0C5:57541:&#x9263
CJK UNIFIED IDEOGRAPH:'E0C6:57542:&#x9250
CJK UNIFIED IDEOGRAPH:'E0C7:57543:&#x9272
CJK UNIFIED IDEOGRAPH:'E0C8:57544:&#x924E
CJK UNIFIED IDEOGRAPH:'E0C9:57545:&#x9253
CJK UNIFIED IDEOGRAPH:'E0CA:57546:&#x924C
CJK UNIFIED IDEOGRAPH:'E0CB:57547:&#x9256
CJK UNIFIED IDEOGRAPH:'E0CC:57548:&#x9232
CJK UNIFIED IDEOGRAPH:'E0CD:57549:&#x959F
CJK UNIFIED IDEOGRAPH:'E0CE:57550:&#x959C
CJK UNIFIED IDEOGRAPH:'E0CF:57551:&#x959E
CJK UNIFIED IDEOGRAPH:'E0D0:57552:&#x959B
CJK UNIFIED IDEOGRAPH:'E0D1:57553:&#x9692
CJK UNIFIED IDEOGRAPH:'E0D2:57554:&#x9693
CJK UNIFIED IDEOGRAPH:'E0D3:57555:&#x9691
CJK UNIFIED IDEOGRAPH:'E0D4:57556:&#x9697
CJK UNIFIED IDEOGRAPH:'E0D5:57557:&#x96CE
CJK UNIFIED IDEOGRAPH:'E0D6:57558:&#x96FA
CJK UNIFIED IDEOGRAPH:'E0D7:57559:&#x96FD
CJK UNIFIED IDEOGRAPH:'E0D8:57560:&#x96F8
CJK UNIFIED IDEOGRAPH:'E0D9:57561:&#x96F5
CJK UNIFIED IDEOGRAPH:'E0DA:57562:&#x9773
CJK UNIFIED IDEOGRAPH:'E0DB:57563:&#x9777
CJK UNIFIED IDEOGRAPH:'E0DC:57564:&#x9778
CJK UNIFIED IDEOGRAPH:'E0DD:57565:&#x9772
CJK UNIFIED IDEOGRAPH:'E0DE:57566:&#x980F
CJK UNIFIED IDEOGRAPH:'E0DF:57567:&#x980D
CJK UNIFIED IDEOGRAPH:'E0E0:57568:&#x980E
CJK UNIFIED IDEOGRAPH:'E0E1:57569:&#x98AC
CJK UNIFIED IDEOGRAPH:'E0E2:57570:&#x98F6
CJK UNIFIED IDEOGRAPH:'E0E3:57571:&#x98F9
CJK UNIFIED IDEOGRAPH:'E0E4:57572:&#x99AF
CJK UNIFIED IDEOGRAPH:'E0E5:57573:&#x99B2
CJK UNIFIED IDEOGRAPH:'E0E6:57574:&#x99B0
CJK UNIFIED IDEOGRAPH:'E0E7:57575:&#x99B5
CJK UNIFIED IDEOGRAPH:'E0E8:57576:&#x9AAD
CJK UNIFIED IDEOGRAPH:'E0E9:57577:&#x9AAB
CJK UNIFIED IDEOGRAPH:'E0EA:57578:&#x9B5B
CJK UNIFIED IDEOGRAPH:'E0EB:57579:&#x9CEA
CJK UNIFIED IDEOGRAPH:'E0EC:57580:&#x9CED
CJK UNIFIED IDEOGRAPH:'E0ED:57581:&#x9CE7
CJK UNIFIED IDEOGRAPH:'E0EE:57582:&#x9E80
CJK UNIFIED IDEOGRAPH:'E0EF:57583:&#x9EFD
CJK UNIFIED IDEOGRAPH:'E0F0:57584:&#x50E6
CJK UNIFIED IDEOGRAPH:'E0F1:57585:&#x50D4
CJK UNIFIED IDEOGRAPH:'E0F2:57586:&#x50D7
CJK UNIFIED IDEOGRAPH:'E0F3:57587:&#x50E8
CJK UNIFIED IDEOGRAPH:'E0F4:57588:&#x50F3
CJK UNIFIED IDEOGRAPH:'E0F5:57589:&#x50DB
CJK UNIFIED IDEOGRAPH:'E0F6:57590:&#x50EA
CJK UNIFIED IDEOGRAPH:'E0F7:57591:&#x50DD
CJK UNIFIED IDEOGRAPH:'E0F8:57592:&#x50E4
CJK UNIFIED IDEOGRAPH:'E0F9:57593:&#x50D3
CJK UNIFIED IDEOGRAPH:'E0FA:57594:&#x50EC
CJK UNIFIED IDEOGRAPH:'E0FB:57595:&#x50F0
CJK UNIFIED IDEOGRAPH:'E0FC:57596:&#x50EF
CJK UNIFIED IDEOGRAPH:'E0FD:57597:&#x50E3
CJK UNIFIED IDEOGRAPH:'E0FE:57598:&#x50E0
CJK UNIFIED IDEOGRAPH:'E140:57664:&#x51D8
CJK UNIFIED IDEOGRAPH:'E141:57665:&#x5280
CJK UNIFIED IDEOGRAPH:'E142:57666:&#x5281
CJK UNIFIED IDEOGRAPH:'E143:57667:&#x52E9
CJK UNIFIED IDEOGRAPH:'E144:57668:&#x52EB
CJK UNIFIED IDEOGRAPH:'E145:57669:&#x5330
CJK UNIFIED IDEOGRAPH:'E146:57670:&#x53AC
CJK UNIFIED IDEOGRAPH:'E147:57671:&#x5627
CJK UNIFIED IDEOGRAPH:'E148:57672:&#x5615
CJK UNIFIED IDEOGRAPH:'E149:57673:&#x560C
CJK UNIFIED IDEOGRAPH:'E14A:57674:&#x5612
CJK UNIFIED IDEOGRAPH:'E14B:57675:&#x55FC
CJK UNIFIED IDEOGRAPH:'E14C:57676:&#x560F
CJK UNIFIED IDEOGRAPH:'E14D:57677:&#x561C
CJK UNIFIED IDEOGRAPH:'E14E:57678:&#x5601
CJK UNIFIED IDEOGRAPH:'E14F:57679:&#x5613
CJK UNIFIED IDEOGRAPH:'E150:57680:&#x5602
CJK UNIFIED IDEOGRAPH:'E151:57681:&#x55FA
CJK UNIFIED IDEOGRAPH:'E152:57682:&#x561D
CJK UNIFIED IDEOGRAPH:'E153:57683:&#x5604
CJK UNIFIED IDEOGRAPH:'E154:57684:&#x55FF
CJK UNIFIED IDEOGRAPH:'E155:57685:&#x55F9
CJK UNIFIED IDEOGRAPH:'E156:57686:&#x5889
CJK UNIFIED IDEOGRAPH:'E157:57687:&#x587C
CJK UNIFIED IDEOGRAPH:'E158:57688:&#x5890
CJK UNIFIED IDEOGRAPH:'E159:57689:&#x5898
CJK UNIFIED IDEOGRAPH:'E15A:57690:&#x5886
CJK UNIFIED IDEOGRAPH:'E15B:57691:&#x5881
CJK UNIFIED IDEOGRAPH:'E15C:57692:&#x587F
CJK UNIFIED IDEOGRAPH:'E15D:57693:&#x5874
CJK UNIFIED IDEOGRAPH:'E15E:57694:&#x588B
CJK UNIFIED IDEOGRAPH:'E15F:57695:&#x587A
CJK UNIFIED IDEOGRAPH:'E160:57696:&#x5887
CJK UNIFIED IDEOGRAPH:'E161:57697:&#x5891
CJK UNIFIED IDEOGRAPH:'E162:57698:&#x588E
CJK UNIFIED IDEOGRAPH:'E163:57699:&#x5876
CJK UNIFIED IDEOGRAPH:'E164:57700:&#x5882
CJK UNIFIED IDEOGRAPH:'E165:57701:&#x5888
CJK UNIFIED IDEOGRAPH:'E166:57702:&#x587B
CJK UNIFIED IDEOGRAPH:'E167:57703:&#x5894
CJK UNIFIED IDEOGRAPH:'E168:57704:&#x588F
CJK UNIFIED IDEOGRAPH:'E169:57705:&#x58FE
CJK UNIFIED IDEOGRAPH:'E16A:57706:&#x596B
CJK UNIFIED IDEOGRAPH:'E16B:57707:&#x5ADC
CJK UNIFIED IDEOGRAPH:'E16C:57708:&#x5AEE
CJK UNIFIED IDEOGRAPH:'E16D:57709:&#x5AE5
CJK UNIFIED IDEOGRAPH:'E16E:57710:&#x5AD5
CJK UNIFIED IDEOGRAPH:'E16F:57711:&#x5AEA
CJK UNIFIED IDEOGRAPH:'E170:57712:&#x5ADA
CJK UNIFIED IDEOGRAPH:'E171:57713:&#x5AED
CJK UNIFIED IDEOGRAPH:'E172:57714:&#x5AEB
CJK UNIFIED IDEOGRAPH:'E173:57715:&#x5AF3
CJK UNIFIED IDEOGRAPH:'E174:57716:&#x5AE2
CJK UNIFIED IDEOGRAPH:'E175:57717:&#x5AE0
CJK UNIFIED IDEOGRAPH:'E176:57718:&#x5ADB
CJK UNIFIED IDEOGRAPH:'E177:57719:&#x5AEC
CJK UNIFIED IDEOGRAPH:'E178:57720:&#x5ADE
CJK UNIFIED IDEOGRAPH:'E179:57721:&#x5ADD
CJK UNIFIED IDEOGRAPH:'E17A:57722:&#x5AD9
CJK UNIFIED IDEOGRAPH:'E17B:57723:&#x5AE8
CJK UNIFIED IDEOGRAPH:'E17C:57724:&#x5ADF
CJK UNIFIED IDEOGRAPH:'E17D:57725:&#x5B77
CJK UNIFIED IDEOGRAPH:'E17E:57726:&#x5BE0
CJK UNIFIED IDEOGRAPH:'E1A1:57761:&#x5BE3
CJK UNIFIED IDEOGRAPH:'E1A2:57762:&#x5C63
CJK UNIFIED IDEOGRAPH:'E1A3:57763:&#x5D82
CJK UNIFIED IDEOGRAPH:'E1A4:57764:&#x5D80
CJK UNIFIED IDEOGRAPH:'E1A5:57765:&#x5D7D
CJK UNIFIED IDEOGRAPH:'E1A6:57766:&#x5D86
CJK UNIFIED IDEOGRAPH:'E1A7:57767:&#x5D7A
CJK UNIFIED IDEOGRAPH:'E1A8:57768:&#x5D81
CJK UNIFIED IDEOGRAPH:'E1A9:57769:&#x5D77
CJK UNIFIED IDEOGRAPH:'E1AA:57770:&#x5D8A
CJK UNIFIED IDEOGRAPH:'E1AB:57771:&#x5D89
CJK UNIFIED IDEOGRAPH:'E1AC:57772:&#x5D88
CJK UNIFIED IDEOGRAPH:'E1AD:57773:&#x5D7E
CJK UNIFIED IDEOGRAPH:'E1AE:57774:&#x5D7C
CJK UNIFIED IDEOGRAPH:'E1AF:57775:&#x5D8D
CJK UNIFIED IDEOGRAPH:'E1B0:57776:&#x5D79
CJK UNIFIED IDEOGRAPH:'E1B1:57777:&#x5D7F
CJK UNIFIED IDEOGRAPH:'E1B2:57778:&#x5E58
CJK UNIFIED IDEOGRAPH:'E1B3:57779:&#x5E59
CJK UNIFIED IDEOGRAPH:'E1B4:57780:&#x5E53
CJK UNIFIED IDEOGRAPH:'E1B5:57781:&#x5ED8
CJK UNIFIED IDEOGRAPH:'E1B6:57782:&#x5ED1
CJK UNIFIED IDEOGRAPH:'E1B7:57783:&#x5ED7
CJK UNIFIED IDEOGRAPH:'E1B8:57784:&#x5ECE
CJK UNIFIED IDEOGRAPH:'E1B9:57785:&#x5EDC
CJK UNIFIED IDEOGRAPH:'E1BA:57786:&#x5ED5
CJK UNIFIED IDEOGRAPH:'E1BB:57787:&#x5ED9
CJK UNIFIED IDEOGRAPH:'E1BC:57788:&#x5ED2
CJK UNIFIED IDEOGRAPH:'E1BD:57789:&#x5ED4
CJK UNIFIED IDEOGRAPH:'E1BE:57790:&#x5F44
CJK UNIFIED IDEOGRAPH:'E1BF:57791:&#x5F43
CJK UNIFIED IDEOGRAPH:'E1C0:57792:&#x5F6F
CJK UNIFIED IDEOGRAPH:'E1C1:57793:&#x5FB6
CJK UNIFIED IDEOGRAPH:'E1C2:57794:&#x612C
CJK UNIFIED IDEOGRAPH:'E1C3:57795:&#x6128
CJK UNIFIED IDEOGRAPH:'E1C4:57796:&#x6141
CJK UNIFIED IDEOGRAPH:'E1C5:57797:&#x615E
CJK UNIFIED IDEOGRAPH:'E1C6:57798:&#x6171
CJK UNIFIED IDEOGRAPH:'E1C7:57799:&#x6173
CJK UNIFIED IDEOGRAPH:'E1C8:57800:&#x6152
CJK UNIFIED IDEOGRAPH:'E1C9:57801:&#x6153
CJK UNIFIED IDEOGRAPH:'E1CA:57802:&#x6172
CJK UNIFIED IDEOGRAPH:'E1CB:57803:&#x616C
CJK UNIFIED IDEOGRAPH:'E1CC:57804:&#x6180
CJK UNIFIED IDEOGRAPH:'E1CD:57805:&#x6174
CJK UNIFIED IDEOGRAPH:'E1CE:57806:&#x6154
CJK UNIFIED IDEOGRAPH:'E1CF:57807:&#x617A
CJK UNIFIED IDEOGRAPH:'E1D0:57808:&#x615B
CJK UNIFIED IDEOGRAPH:'E1D1:57809:&#x6165
CJK UNIFIED IDEOGRAPH:'E1D2:57810:&#x613B
CJK UNIFIED IDEOGRAPH:'E1D3:57811:&#x616A
CJK UNIFIED IDEOGRAPH:'E1D4:57812:&#x6161
CJK UNIFIED IDEOGRAPH:'E1D5:57813:&#x6156
CJK UNIFIED IDEOGRAPH:'E1D6:57814:&#x6229
CJK UNIFIED IDEOGRAPH:'E1D7:57815:&#x6227
CJK UNIFIED IDEOGRAPH:'E1D8:57816:&#x622B
CJK UNIFIED IDEOGRAPH:'E1D9:57817:&#x642B
CJK UNIFIED IDEOGRAPH:'E1DA:57818:&#x644D
CJK UNIFIED IDEOGRAPH:'E1DB:57819:&#x645B
CJK UNIFIED IDEOGRAPH:'E1DC:57820:&#x645D
CJK UNIFIED IDEOGRAPH:'E1DD:57821:&#x6474
CJK UNIFIED IDEOGRAPH:'E1DE:57822:&#x6476
CJK UNIFIED IDEOGRAPH:'E1DF:57823:&#x6472
CJK UNIFIED IDEOGRAPH:'E1E0:57824:&#x6473
CJK UNIFIED IDEOGRAPH:'E1E1:57825:&#x647D
CJK UNIFIED IDEOGRAPH:'E1E2:57826:&#x6475
CJK UNIFIED IDEOGRAPH:'E1E3:57827:&#x6466
CJK UNIFIED IDEOGRAPH:'E1E4:57828:&#x64A6
CJK UNIFIED IDEOGRAPH:'E1E5:57829:&#x644E
CJK UNIFIED IDEOGRAPH:'E1E6:57830:&#x6482
CJK UNIFIED IDEOGRAPH:'E1E7:57831:&#x645E
CJK UNIFIED IDEOGRAPH:'E1E8:57832:&#x645C
CJK UNIFIED IDEOGRAPH:'E1E9:57833:&#x644B
CJK UNIFIED IDEOGRAPH:'E1EA:57834:&#x6453
CJK UNIFIED IDEOGRAPH:'E1EB:57835:&#x6460
CJK UNIFIED IDEOGRAPH:'E1EC:57836:&#x6450
CJK UNIFIED IDEOGRAPH:'E1ED:57837:&#x647F
CJK UNIFIED IDEOGRAPH:'E1EE:57838:&#x643F
CJK UNIFIED IDEOGRAPH:'E1EF:57839:&#x646C
CJK UNIFIED IDEOGRAPH:'E1F0:57840:&#x646B
CJK UNIFIED IDEOGRAPH:'E1F1:57841:&#x6459
CJK UNIFIED IDEOGRAPH:'E1F2:57842:&#x6465
CJK UNIFIED IDEOGRAPH:'E1F3:57843:&#x6477
CJK UNIFIED IDEOGRAPH:'E1F4:57844:&#x6573
CJK UNIFIED IDEOGRAPH:'E1F5:57845:&#x65A0
CJK UNIFIED IDEOGRAPH:'E1F6:57846:&#x66A1
CJK UNIFIED IDEOGRAPH:'E1F7:57847:&#x66A0
CJK UNIFIED IDEOGRAPH:'E1F8:57848:&#x669F
CJK UNIFIED IDEOGRAPH:'E1F9:57849:&#x6705
CJK UNIFIED IDEOGRAPH:'E1FA:57850:&#x6704
CJK UNIFIED IDEOGRAPH:'E1FB:57851:&#x6722
CJK UNIFIED IDEOGRAPH:'E1FC:57852:&#x69B1
CJK UNIFIED IDEOGRAPH:'E1FD:57853:&#x69B6
CJK UNIFIED IDEOGRAPH:'E1FE:57854:&#x69C9
CJK UNIFIED IDEOGRAPH:'E240:57920:&#x69A0
CJK UNIFIED IDEOGRAPH:'E241:57921:&#x69CE
CJK UNIFIED IDEOGRAPH:'E242:57922:&#x6996
CJK UNIFIED IDEOGRAPH:'E243:57923:&#x69B0
CJK UNIFIED IDEOGRAPH:'E244:57924:&#x69AC
CJK UNIFIED IDEOGRAPH:'E245:57925:&#x69BC
CJK UNIFIED IDEOGRAPH:'E246:57926:&#x6991
CJK UNIFIED IDEOGRAPH:'E247:57927:&#x6999
CJK UNIFIED IDEOGRAPH:'E248:57928:&#x698E
CJK UNIFIED IDEOGRAPH:'E249:57929:&#x69A7
CJK UNIFIED IDEOGRAPH:'E24A:57930:&#x698D
CJK UNIFIED IDEOGRAPH:'E24B:57931:&#x69A9
CJK UNIFIED IDEOGRAPH:'E24C:57932:&#x69BE
CJK UNIFIED IDEOGRAPH:'E24D:57933:&#x69AF
CJK UNIFIED IDEOGRAPH:'E24E:57934:&#x69BF
CJK UNIFIED IDEOGRAPH:'E24F:57935:&#x69C4
CJK UNIFIED IDEOGRAPH:'E250:57936:&#x69BD
CJK UNIFIED IDEOGRAPH:'E251:57937:&#x69A4
CJK UNIFIED IDEOGRAPH:'E252:57938:&#x69D4
CJK UNIFIED IDEOGRAPH:'E253:57939:&#x69B9
CJK UNIFIED IDEOGRAPH:'E254:57940:&#x69CA
CJK UNIFIED IDEOGRAPH:'E255:57941:&#x699A
CJK UNIFIED IDEOGRAPH:'E256:57942:&#x69CF
CJK UNIFIED IDEOGRAPH:'E257:57943:&#x69B3
CJK UNIFIED IDEOGRAPH:'E258:57944:&#x6993
CJK UNIFIED IDEOGRAPH:'E259:57945:&#x69AA
CJK UNIFIED IDEOGRAPH:'E25A:57946:&#x69A1
CJK UNIFIED IDEOGRAPH:'E25B:57947:&#x699E
CJK UNIFIED IDEOGRAPH:'E25C:57948:&#x69D9
CJK UNIFIED IDEOGRAPH:'E25D:57949:&#x6997
CJK UNIFIED IDEOGRAPH:'E25E:57950:&#x6990
CJK UNIFIED IDEOGRAPH:'E25F:57951:&#x69C2
CJK UNIFIED IDEOGRAPH:'E260:57952:&#x69B5
CJK UNIFIED IDEOGRAPH:'E261:57953:&#x69A5
CJK UNIFIED IDEOGRAPH:'E262:57954:&#x69C6
CJK UNIFIED IDEOGRAPH:'E263:57955:&#x6B4A
CJK UNIFIED IDEOGRAPH:'E264:57956:&#x6B4D
CJK UNIFIED IDEOGRAPH:'E265:57957:&#x6B4B
CJK UNIFIED IDEOGRAPH:'E266:57958:&#x6B9E
CJK UNIFIED IDEOGRAPH:'E267:57959:&#x6B9F
CJK UNIFIED IDEOGRAPH:'E268:57960:&#x6BA0
CJK UNIFIED IDEOGRAPH:'E269:57961:&#x6BC3
CJK UNIFIED IDEOGRAPH:'E26A:57962:&#x6BC4
CJK UNIFIED IDEOGRAPH:'E26B:57963:&#x6BFE
CJK UNIFIED IDEOGRAPH:'E26C:57964:&#x6ECE
CJK UNIFIED IDEOGRAPH:'E26D:57965:&#x6EF5
CJK UNIFIED IDEOGRAPH:'E26E:57966:&#x6EF1
CJK UNIFIED IDEOGRAPH:'E26F:57967:&#x6F03
CJK UNIFIED IDEOGRAPH:'E270:57968:&#x6F25
CJK UNIFIED IDEOGRAPH:'E271:57969:&#x6EF8
CJK UNIFIED IDEOGRAPH:'E272:57970:&#x6F37
CJK UNIFIED IDEOGRAPH:'E273:57971:&#x6EFB
CJK UNIFIED IDEOGRAPH:'E274:57972:&#x6F2E
CJK UNIFIED IDEOGRAPH:'E275:57973:&#x6F09
CJK UNIFIED IDEOGRAPH:'E276:57974:&#x6F4E
CJK UNIFIED IDEOGRAPH:'E277:57975:&#x6F19
CJK UNIFIED IDEOGRAPH:'E278:57976:&#x6F1A
CJK UNIFIED IDEOGRAPH:'E279:57977:&#x6F27
CJK UNIFIED IDEOGRAPH:'E27A:57978:&#x6F18
CJK UNIFIED IDEOGRAPH:'E27B:57979:&#x6F3B
CJK UNIFIED IDEOGRAPH:'E27C:57980:&#x6F12
CJK UNIFIED IDEOGRAPH:'E27D:57981:&#x6EED
CJK UNIFIED IDEOGRAPH:'E27E:57982:&#x6F0A
CJK UNIFIED IDEOGRAPH:'E2A1:58017:&#x6F36
CJK UNIFIED IDEOGRAPH:'E2A2:58018:&#x6F73
CJK UNIFIED IDEOGRAPH:'E2A3:58019:&#x6EF9
CJK UNIFIED IDEOGRAPH:'E2A4:58020:&#x6EEE
CJK UNIFIED IDEOGRAPH:'E2A5:58021:&#x6F2D
CJK UNIFIED IDEOGRAPH:'E2A6:58022:&#x6F40
CJK UNIFIED IDEOGRAPH:'E2A7:58023:&#x6F30
CJK UNIFIED IDEOGRAPH:'E2A8:58024:&#x6F3C
CJK UNIFIED IDEOGRAPH:'E2A9:58025:&#x6F35
CJK UNIFIED IDEOGRAPH:'E2AA:58026:&#x6EEB
CJK UNIFIED IDEOGRAPH:'E2AB:58027:&#x6F07
CJK UNIFIED IDEOGRAPH:'E2AC:58028:&#x6F0E
CJK UNIFIED IDEOGRAPH:'E2AD:58029:&#x6F43
CJK UNIFIED IDEOGRAPH:'E2AE:58030:&#x6F05
CJK UNIFIED IDEOGRAPH:'E2AF:58031:&#x6EFD
CJK UNIFIED IDEOGRAPH:'E2B0:58032:&#x6EF6
CJK UNIFIED IDEOGRAPH:'E2B1:58033:&#x6F39
CJK UNIFIED IDEOGRAPH:'E2B2:58034:&#x6F1C
CJK UNIFIED IDEOGRAPH:'E2B3:58035:&#x6EFC
CJK UNIFIED IDEOGRAPH:'E2B4:58036:&#x6F3A
CJK UNIFIED IDEOGRAPH:'E2B5:58037:&#x6F1F
CJK UNIFIED IDEOGRAPH:'E2B6:58038:&#x6F0D
CJK UNIFIED IDEOGRAPH:'E2B7:58039:&#x6F1E
CJK UNIFIED IDEOGRAPH:'E2B8:58040:&#x6F08
CJK UNIFIED IDEOGRAPH:'E2B9:58041:&#x6F21
CJK UNIFIED IDEOGRAPH:'E2BA:58042:&#x7187
CJK UNIFIED IDEOGRAPH:'E2BB:58043:&#x7190
CJK UNIFIED IDEOGRAPH:'E2BC:58044:&#x7189
CJK UNIFIED IDEOGRAPH:'E2BD:58045:&#x7180
CJK UNIFIED IDEOGRAPH:'E2BE:58046:&#x7185
CJK UNIFIED IDEOGRAPH:'E2BF:58047:&#x7182
CJK UNIFIED IDEOGRAPH:'E2C0:58048:&#x718F
CJK UNIFIED IDEOGRAPH:'E2C1:58049:&#x717B
CJK UNIFIED IDEOGRAPH:'E2C2:58050:&#x7186
CJK UNIFIED IDEOGRAPH:'E2C3:58051:&#x7181
CJK UNIFIED IDEOGRAPH:'E2C4:58052:&#x7197
CJK UNIFIED IDEOGRAPH:'E2C5:58053:&#x7244
CJK UNIFIED IDEOGRAPH:'E2C6:58054:&#x7253
CJK UNIFIED IDEOGRAPH:'E2C7:58055:&#x7297
CJK UNIFIED IDEOGRAPH:'E2C8:58056:&#x7295
CJK UNIFIED IDEOGRAPH:'E2C9:58057:&#x7293
CJK UNIFIED IDEOGRAPH:'E2CA:58058:&#x7343
CJK UNIFIED IDEOGRAPH:'E2CB:58059:&#x734D
CJK UNIFIED IDEOGRAPH:'E2CC:58060:&#x7351
CJK UNIFIED IDEOGRAPH:'E2CD:58061:&#x734C
CJK UNIFIED IDEOGRAPH:'E2CE:58062:&#x7462
CJK UNIFIED IDEOGRAPH:'E2CF:58063:&#x7473
CJK UNIFIED IDEOGRAPH:'E2D0:58064:&#x7471
CJK UNIFIED IDEOGRAPH:'E2D1:58065:&#x7475
CJK UNIFIED IDEOGRAPH:'E2D2:58066:&#x7472
CJK UNIFIED IDEOGRAPH:'E2D3:58067:&#x7467
CJK UNIFIED IDEOGRAPH:'E2D4:58068:&#x746E
CJK UNIFIED IDEOGRAPH:'E2D5:58069:&#x7500
CJK UNIFIED IDEOGRAPH:'E2D6:58070:&#x7502
CJK UNIFIED IDEOGRAPH:'E2D7:58071:&#x7503
CJK UNIFIED IDEOGRAPH:'E2D8:58072:&#x757D
CJK UNIFIED IDEOGRAPH:'E2D9:58073:&#x7590
CJK UNIFIED IDEOGRAPH:'E2DA:58074:&#x7616
CJK UNIFIED IDEOGRAPH:'E2DB:58075:&#x7608
CJK UNIFIED IDEOGRAPH:'E2DC:58076:&#x760C
CJK UNIFIED IDEOGRAPH:'E2DD:58077:&#x7615
CJK UNIFIED IDEOGRAPH:'E2DE:58078:&#x7611
CJK UNIFIED IDEOGRAPH:'E2DF:58079:&#x760A
CJK UNIFIED IDEOGRAPH:'E2E0:58080:&#x7614
CJK UNIFIED IDEOGRAPH:'E2E1:58081:&#x76B8
CJK UNIFIED IDEOGRAPH:'E2E2:58082:&#x7781
CJK UNIFIED IDEOGRAPH:'E2E3:58083:&#x777C
CJK UNIFIED IDEOGRAPH:'E2E4:58084:&#x7785
CJK UNIFIED IDEOGRAPH:'E2E5:58085:&#x7782
CJK UNIFIED IDEOGRAPH:'E2E6:58086:&#x776E
CJK UNIFIED IDEOGRAPH:'E2E7:58087:&#x7780
CJK UNIFIED IDEOGRAPH:'E2E8:58088:&#x776F
CJK UNIFIED IDEOGRAPH:'E2E9:58089:&#x777E
CJK UNIFIED IDEOGRAPH:'E2EA:58090:&#x7783
CJK UNIFIED IDEOGRAPH:'E2EB:58091:&#x78B2
CJK UNIFIED IDEOGRAPH:'E2EC:58092:&#x78AA
CJK UNIFIED IDEOGRAPH:'E2ED:58093:&#x78B4
CJK UNIFIED IDEOGRAPH:'E2EE:58094:&#x78AD
CJK UNIFIED IDEOGRAPH:'E2EF:58095:&#x78A8
CJK UNIFIED IDEOGRAPH:'E2F0:58096:&#x787E
CJK UNIFIED IDEOGRAPH:'E2F1:58097:&#x78AB
CJK UNIFIED IDEOGRAPH:'E2F2:58098:&#x789E
CJK UNIFIED IDEOGRAPH:'E2F3:58099:&#x78A5
CJK UNIFIED IDEOGRAPH:'E2F4:58100:&#x78A0
CJK UNIFIED IDEOGRAPH:'E2F5:58101:&#x78AC
CJK UNIFIED IDEOGRAPH:'E2F6:58102:&#x78A2
CJK UNIFIED IDEOGRAPH:'E2F7:58103:&#x78A4
CJK UNIFIED IDEOGRAPH:'E2F8:58104:&#x7998
CJK UNIFIED IDEOGRAPH:'E2F9:58105:&#x798A
CJK UNIFIED IDEOGRAPH:'E2FA:58106:&#x798B
CJK UNIFIED IDEOGRAPH:'E2FB:58107:&#x7996
CJK UNIFIED IDEOGRAPH:'E2FC:58108:&#x7995
CJK UNIFIED IDEOGRAPH:'E2FD:58109:&#x7994
CJK UNIFIED IDEOGRAPH:'E2FE:58110:&#x7993
CJK UNIFIED IDEOGRAPH:'E340:58176:&#x7997
CJK UNIFIED IDEOGRAPH:'E341:58177:&#x7988
CJK UNIFIED IDEOGRAPH:'E342:58178:&#x7992
CJK UNIFIED IDEOGRAPH:'E343:58179:&#x7990
CJK UNIFIED IDEOGRAPH:'E344:58180:&#x7A2B
CJK UNIFIED IDEOGRAPH:'E345:58181:&#x7A4A
CJK UNIFIED IDEOGRAPH:'E346:58182:&#x7A30
CJK UNIFIED IDEOGRAPH:'E347:58183:&#x7A2F
CJK UNIFIED IDEOGRAPH:'E348:58184:&#x7A28
CJK UNIFIED IDEOGRAPH:'E349:58185:&#x7A26
CJK UNIFIED IDEOGRAPH:'E34A:58186:&#x7AA8
CJK UNIFIED IDEOGRAPH:'E34B:58187:&#x7AAB
CJK UNIFIED IDEOGRAPH:'E34C:58188:&#x7AAC
CJK UNIFIED IDEOGRAPH:'E34D:58189:&#x7AEE
CJK UNIFIED IDEOGRAPH:'E34E:58190:&#x7B88
CJK UNIFIED IDEOGRAPH:'E34F:58191:&#x7B9C
CJK UNIFIED IDEOGRAPH:'E350:58192:&#x7B8A
CJK UNIFIED IDEOGRAPH:'E351:58193:&#x7B91
CJK UNIFIED IDEOGRAPH:'E352:58194:&#x7B90
CJK UNIFIED IDEOGRAPH:'E353:58195:&#x7B96
CJK UNIFIED IDEOGRAPH:'E354:58196:&#x7B8D
CJK UNIFIED IDEOGRAPH:'E355:58197:&#x7B8C
CJK UNIFIED IDEOGRAPH:'E356:58198:&#x7B9B
CJK UNIFIED IDEOGRAPH:'E357:58199:&#x7B8E
CJK UNIFIED IDEOGRAPH:'E358:58200:&#x7B85
CJK UNIFIED IDEOGRAPH:'E359:58201:&#x7B98
CJK UNIFIED IDEOGRAPH:'E35A:58202:&#x5284
CJK UNIFIED IDEOGRAPH:'E35B:58203:&#x7B99
CJK UNIFIED IDEOGRAPH:'E35C:58204:&#x7BA4
CJK UNIFIED IDEOGRAPH:'E35D:58205:&#x7B82
CJK UNIFIED IDEOGRAPH:'E35E:58206:&#x7CBB
CJK UNIFIED IDEOGRAPH:'E35F:58207:&#x7CBF
CJK UNIFIED IDEOGRAPH:'E360:58208:&#x7CBC
CJK UNIFIED IDEOGRAPH:'E361:58209:&#x7CBA
CJK UNIFIED IDEOGRAPH:'E362:58210:&#x7DA7
CJK UNIFIED IDEOGRAPH:'E363:58211:&#x7DB7
CJK UNIFIED IDEOGRAPH:'E364:58212:&#x7DC2
CJK UNIFIED IDEOGRAPH:'E365:58213:&#x7DA3
CJK UNIFIED IDEOGRAPH:'E366:58214:&#x7DAA
CJK UNIFIED IDEOGRAPH:'E367:58215:&#x7DC1
CJK UNIFIED IDEOGRAPH:'E368:58216:&#x7DC0
CJK UNIFIED IDEOGRAPH:'E369:58217:&#x7DC5
CJK UNIFIED IDEOGRAPH:'E36A:58218:&#x7D9D
CJK UNIFIED IDEOGRAPH:'E36B:58219:&#x7DCE
CJK UNIFIED IDEOGRAPH:'E36C:58220:&#x7DC4
CJK UNIFIED IDEOGRAPH:'E36D:58221:&#x7DC6
CJK UNIFIED IDEOGRAPH:'E36E:58222:&#x7DCB
CJK UNIFIED IDEOGRAPH:'E36F:58223:&#x7DCC
CJK UNIFIED IDEOGRAPH:'E370:58224:&#x7DAF
CJK UNIFIED IDEOGRAPH:'E371:58225:&#x7DB9
CJK UNIFIED IDEOGRAPH:'E372:58226:&#x7D96
CJK UNIFIED IDEOGRAPH:'E373:58227:&#x7DBC
CJK UNIFIED IDEOGRAPH:'E374:58228:&#x7D9F
CJK UNIFIED IDEOGRAPH:'E375:58229:&#x7DA6
CJK UNIFIED IDEOGRAPH:'E376:58230:&#x7DAE
CJK UNIFIED IDEOGRAPH:'E377:58231:&#x7DA9
CJK UNIFIED IDEOGRAPH:'E378:58232:&#x7DA1
CJK UNIFIED IDEOGRAPH:'E379:58233:&#x7DC9
CJK UNIFIED IDEOGRAPH:'E37A:58234:&#x7F73
CJK UNIFIED IDEOGRAPH:'E37B:58235:&#x7FE2
CJK UNIFIED IDEOGRAPH:'E37C:58236:&#x7FE3
CJK UNIFIED IDEOGRAPH:'E37D:58237:&#x7FE5
CJK UNIFIED IDEOGRAPH:'E37E:58238:&#x7FDE
CJK UNIFIED IDEOGRAPH:'E3A1:58273:&#x8024
CJK UNIFIED IDEOGRAPH:'E3A2:58274:&#x805D
CJK UNIFIED IDEOGRAPH:'E3A3:58275:&#x805C
CJK UNIFIED IDEOGRAPH:'E3A4:58276:&#x8189
CJK UNIFIED IDEOGRAPH:'E3A5:58277:&#x8186
CJK UNIFIED IDEOGRAPH:'E3A6:58278:&#x8183
CJK UNIFIED IDEOGRAPH:'E3A7:58279:&#x8187
CJK UNIFIED IDEOGRAPH:'E3A8:58280:&#x818D
CJK UNIFIED IDEOGRAPH:'E3A9:58281:&#x818C
CJK UNIFIED IDEOGRAPH:'E3AA:58282:&#x818B
CJK UNIFIED IDEOGRAPH:'E3AB:58283:&#x8215
CJK UNIFIED IDEOGRAPH:'E3AC:58284:&#x8497
CJK UNIFIED IDEOGRAPH:'E3AD:58285:&#x84A4
CJK UNIFIED IDEOGRAPH:'E3AE:58286:&#x84A1
CJK UNIFIED IDEOGRAPH:'E3AF:58287:&#x849F
CJK UNIFIED IDEOGRAPH:'E3B0:58288:&#x84BA
CJK UNIFIED IDEOGRAPH:'E3B1:58289:&#x84CE
CJK UNIFIED IDEOGRAPH:'E3B2:58290:&#x84C2
CJK UNIFIED IDEOGRAPH:'E3B3:58291:&#x84AC
CJK UNIFIED IDEOGRAPH:'E3B4:58292:&#x84AE
CJK UNIFIED IDEOGRAPH:'E3B5:58293:&#x84AB
CJK UNIFIED IDEOGRAPH:'E3B6:58294:&#x84B9
CJK UNIFIED IDEOGRAPH:'E3B7:58295:&#x84B4
CJK UNIFIED IDEOGRAPH:'E3B8:58296:&#x84C1
CJK UNIFIED IDEOGRAPH:'E3B9:58297:&#x84CD
CJK UNIFIED IDEOGRAPH:'E3BA:58298:&#x84AA
CJK UNIFIED IDEOGRAPH:'E3BB:58299:&#x849A
CJK UNIFIED IDEOGRAPH:'E3BC:58300:&#x84B1
CJK UNIFIED IDEOGRAPH:'E3BD:58301:&#x84D0
CJK UNIFIED IDEOGRAPH:'E3BE:58302:&#x849D
CJK UNIFIED IDEOGRAPH:'E3BF:58303:&#x84A7
CJK UNIFIED IDEOGRAPH:'E3C0:58304:&#x84BB
CJK UNIFIED IDEOGRAPH:'E3C1:58305:&#x84A2
CJK UNIFIED IDEOGRAPH:'E3C2:58306:&#x8494
CJK UNIFIED IDEOGRAPH:'E3C3:58307:&#x84C7
CJK UNIFIED IDEOGRAPH:'E3C4:58308:&#x84CC
CJK UNIFIED IDEOGRAPH:'E3C5:58309:&#x849B
CJK UNIFIED IDEOGRAPH:'E3C6:58310:&#x84A9
CJK UNIFIED IDEOGRAPH:'E3C7:58311:&#x84AF
CJK UNIFIED IDEOGRAPH:'E3C8:58312:&#x84A8
CJK UNIFIED IDEOGRAPH:'E3C9:58313:&#x84D6
CJK UNIFIED IDEOGRAPH:'E3CA:58314:&#x8498
CJK UNIFIED IDEOGRAPH:'E3CB:58315:&#x84B6
CJK UNIFIED IDEOGRAPH:'E3CC:58316:&#x84CF
CJK UNIFIED IDEOGRAPH:'E3CD:58317:&#x84A0
CJK UNIFIED IDEOGRAPH:'E3CE:58318:&#x84D7
CJK UNIFIED IDEOGRAPH:'E3CF:58319:&#x84D4
CJK UNIFIED IDEOGRAPH:'E3D0:58320:&#x84D2
CJK UNIFIED IDEOGRAPH:'E3D1:58321:&#x84DB
CJK UNIFIED IDEOGRAPH:'E3D2:58322:&#x84B0
CJK UNIFIED IDEOGRAPH:'E3D3:58323:&#x8491
CJK UNIFIED IDEOGRAPH:'E3D4:58324:&#x8661
CJK UNIFIED IDEOGRAPH:'E3D5:58325:&#x8733
CJK UNIFIED IDEOGRAPH:'E3D6:58326:&#x8723
CJK UNIFIED IDEOGRAPH:'E3D7:58327:&#x8728
CJK UNIFIED IDEOGRAPH:'E3D8:58328:&#x876B
CJK UNIFIED IDEOGRAPH:'E3D9:58329:&#x8740
CJK UNIFIED IDEOGRAPH:'E3DA:58330:&#x872E
CJK UNIFIED IDEOGRAPH:'E3DB:58331:&#x871E
CJK UNIFIED IDEOGRAPH:'E3DC:58332:&#x8721
CJK UNIFIED IDEOGRAPH:'E3DD:58333:&#x8719
CJK UNIFIED IDEOGRAPH:'E3DE:58334:&#x871B
CJK UNIFIED IDEOGRAPH:'E3DF:58335:&#x8743
CJK UNIFIED IDEOGRAPH:'E3E0:58336:&#x872C
CJK UNIFIED IDEOGRAPH:'E3E1:58337:&#x8741
CJK UNIFIED IDEOGRAPH:'E3E2:58338:&#x873E
CJK UNIFIED IDEOGRAPH:'E3E3:58339:&#x8746
CJK UNIFIED IDEOGRAPH:'E3E4:58340:&#x8720
CJK UNIFIED IDEOGRAPH:'E3E5:58341:&#x8732
CJK UNIFIED IDEOGRAPH:'E3E6:58342:&#x872A
CJK UNIFIED IDEOGRAPH:'E3E7:58343:&#x872D
CJK UNIFIED IDEOGRAPH:'E3E8:58344:&#x873C
CJK UNIFIED IDEOGRAPH:'E3E9:58345:&#x8712
CJK UNIFIED IDEOGRAPH:'E3EA:58346:&#x873A
CJK UNIFIED IDEOGRAPH:'E3EB:58347:&#x8731
CJK UNIFIED IDEOGRAPH:'E3EC:58348:&#x8735
CJK UNIFIED IDEOGRAPH:'E3ED:58349:&#x8742
CJK UNIFIED IDEOGRAPH:'E3EE:58350:&#x8726
CJK UNIFIED IDEOGRAPH:'E3EF:58351:&#x8727
CJK UNIFIED IDEOGRAPH:'E3F0:58352:&#x8738
CJK UNIFIED IDEOGRAPH:'E3F1:58353:&#x8724
CJK UNIFIED IDEOGRAPH:'E3F2:58354:&#x871A
CJK UNIFIED IDEOGRAPH:'E3F3:58355:&#x8730
CJK UNIFIED IDEOGRAPH:'E3F4:58356:&#x8711
CJK UNIFIED IDEOGRAPH:'E3F5:58357:&#x88F7
CJK UNIFIED IDEOGRAPH:'E3F6:58358:&#x88E7
CJK UNIFIED IDEOGRAPH:'E3F7:58359:&#x88F1
CJK UNIFIED IDEOGRAPH:'E3F8:58360:&#x88F2
CJK UNIFIED IDEOGRAPH:'E3F9:58361:&#x88FA
CJK UNIFIED IDEOGRAPH:'E3FA:58362:&#x88FE
CJK UNIFIED IDEOGRAPH:'E3FB:58363:&#x88EE
CJK UNIFIED IDEOGRAPH:'E3FC:58364:&#x88FC
CJK UNIFIED IDEOGRAPH:'E3FD:58365:&#x88F6
CJK UNIFIED IDEOGRAPH:'E3FE:58366:&#x88FB
CJK UNIFIED IDEOGRAPH:'E440:58432:&#x88F0
CJK UNIFIED IDEOGRAPH:'E441:58433:&#x88EC
CJK UNIFIED IDEOGRAPH:'E442:58434:&#x88EB
CJK UNIFIED IDEOGRAPH:'E443:58435:&#x899D
CJK UNIFIED IDEOGRAPH:'E444:58436:&#x89A1
CJK UNIFIED IDEOGRAPH:'E445:58437:&#x899F
CJK UNIFIED IDEOGRAPH:'E446:58438:&#x899E
CJK UNIFIED IDEOGRAPH:'E447:58439:&#x89E9
CJK UNIFIED IDEOGRAPH:'E448:58440:&#x89EB
CJK UNIFIED IDEOGRAPH:'E449:58441:&#x89E8
CJK UNIFIED IDEOGRAPH:'E44A:58442:&#x8AAB
CJK UNIFIED IDEOGRAPH:'E44B:58443:&#x8A99
CJK UNIFIED IDEOGRAPH:'E44C:58444:&#x8A8B
CJK UNIFIED IDEOGRAPH:'E44D:58445:&#x8A92
CJK UNIFIED IDEOGRAPH:'E44E:58446:&#x8A8F
CJK UNIFIED IDEOGRAPH:'E44F:58447:&#x8A96
CJK UNIFIED IDEOGRAPH:'E450:58448:&#x8C3D
CJK UNIFIED IDEOGRAPH:'E451:58449:&#x8C68
CJK UNIFIED IDEOGRAPH:'E452:58450:&#x8C69
CJK UNIFIED IDEOGRAPH:'E453:58451:&#x8CD5
CJK UNIFIED IDEOGRAPH:'E454:58452:&#x8CCF
CJK UNIFIED IDEOGRAPH:'E455:58453:&#x8CD7
CJK UNIFIED IDEOGRAPH:'E456:58454:&#x8D96
CJK UNIFIED IDEOGRAPH:'E457:58455:&#x8E09
CJK UNIFIED IDEOGRAPH:'E458:58456:&#x8E02
CJK UNIFIED IDEOGRAPH:'E459:58457:&#x8DFF
CJK UNIFIED IDEOGRAPH:'E45A:58458:&#x8E0D
CJK UNIFIED IDEOGRAPH:'E45B:58459:&#x8DFD
CJK UNIFIED IDEOGRAPH:'E45C:58460:&#x8E0A
CJK UNIFIED IDEOGRAPH:'E45D:58461:&#x8E03
CJK UNIFIED IDEOGRAPH:'E45E:58462:&#x8E07
CJK UNIFIED IDEOGRAPH:'E45F:58463:&#x8E06
CJK UNIFIED IDEOGRAPH:'E460:58464:&#x8E05
CJK UNIFIED IDEOGRAPH:'E461:58465:&#x8DFE
CJK UNIFIED IDEOGRAPH:'E462:58466:&#x8E00
CJK UNIFIED IDEOGRAPH:'E463:58467:&#x8E04
CJK UNIFIED IDEOGRAPH:'E464:58468:&#x8F10
CJK UNIFIED IDEOGRAPH:'E465:58469:&#x8F11
CJK UNIFIED IDEOGRAPH:'E466:58470:&#x8F0E
CJK UNIFIED IDEOGRAPH:'E467:58471:&#x8F0D
CJK UNIFIED IDEOGRAPH:'E468:58472:&#x9123
CJK UNIFIED IDEOGRAPH:'E469:58473:&#x911C
CJK UNIFIED IDEOGRAPH:'E46A:58474:&#x9120
CJK UNIFIED IDEOGRAPH:'E46B:58475:&#x9122
CJK UNIFIED IDEOGRAPH:'E46C:58476:&#x911F
CJK UNIFIED IDEOGRAPH:'E46D:58477:&#x911D
CJK UNIFIED IDEOGRAPH:'E46E:58478:&#x911A
CJK UNIFIED IDEOGRAPH:'E46F:58479:&#x9124
CJK UNIFIED IDEOGRAPH:'E470:58480:&#x9121
CJK UNIFIED IDEOGRAPH:'E471:58481:&#x911B
CJK UNIFIED IDEOGRAPH:'E472:58482:&#x917A
CJK UNIFIED IDEOGRAPH:'E473:58483:&#x9172
CJK UNIFIED IDEOGRAPH:'E474:58484:&#x9179
CJK UNIFIED IDEOGRAPH:'E475:58485:&#x9173
CJK UNIFIED IDEOGRAPH:'E476:58486:&#x92A5
CJK UNIFIED IDEOGRAPH:'E477:58487:&#x92A4
CJK UNIFIED IDEOGRAPH:'E478:58488:&#x9276
CJK UNIFIED IDEOGRAPH:'E479:58489:&#x929B
CJK UNIFIED IDEOGRAPH:'E47A:58490:&#x927A
CJK UNIFIED IDEOGRAPH:'E47B:58491:&#x92A0
CJK UNIFIED IDEOGRAPH:'E47C:58492:&#x9294
CJK UNIFIED IDEOGRAPH:'E47D:58493:&#x92AA
CJK UNIFIED IDEOGRAPH:'E47E:58494:&#x928D
CJK UNIFIED IDEOGRAPH:'E4A1:58529:&#x92A6
CJK UNIFIED IDEOGRAPH:'E4A2:58530:&#x929A
CJK UNIFIED IDEOGRAPH:'E4A3:58531:&#x92AB
CJK UNIFIED IDEOGRAPH:'E4A4:58532:&#x9279
CJK UNIFIED IDEOGRAPH:'E4A5:58533:&#x9297
CJK UNIFIED IDEOGRAPH:'E4A6:58534:&#x927F
CJK UNIFIED IDEOGRAPH:'E4A7:58535:&#x92A3
CJK UNIFIED IDEOGRAPH:'E4A8:58536:&#x92EE
CJK UNIFIED IDEOGRAPH:'E4A9:58537:&#x928E
CJK UNIFIED IDEOGRAPH:'E4AA:58538:&#x9282
CJK UNIFIED IDEOGRAPH:'E4AB:58539:&#x9295
CJK UNIFIED IDEOGRAPH:'E4AC:58540:&#x92A2
CJK UNIFIED IDEOGRAPH:'E4AD:58541:&#x927D
CJK UNIFIED IDEOGRAPH:'E4AE:58542:&#x9288
CJK UNIFIED IDEOGRAPH:'E4AF:58543:&#x92A1
CJK UNIFIED IDEOGRAPH:'E4B0:58544:&#x928A
CJK UNIFIED IDEOGRAPH:'E4B1:58545:&#x9286
CJK UNIFIED IDEOGRAPH:'E4B2:58546:&#x928C
CJK UNIFIED IDEOGRAPH:'E4B3:58547:&#x9299
CJK UNIFIED IDEOGRAPH:'E4B4:58548:&#x92A7
CJK UNIFIED IDEOGRAPH:'E4B5:58549:&#x927E
CJK UNIFIED IDEOGRAPH:'E4B6:58550:&#x9287
CJK UNIFIED IDEOGRAPH:'E4B7:58551:&#x92A9
CJK UNIFIED IDEOGRAPH:'E4B8:58552:&#x929D
CJK UNIFIED IDEOGRAPH:'E4B9:58553:&#x928B
CJK UNIFIED IDEOGRAPH:'E4BA:58554:&#x922D
CJK UNIFIED IDEOGRAPH:'E4BB:58555:&#x969E
CJK UNIFIED IDEOGRAPH:'E4BC:58556:&#x96A1
CJK UNIFIED IDEOGRAPH:'E4BD:58557:&#x96FF
CJK UNIFIED IDEOGRAPH:'E4BE:58558:&#x9758
CJK UNIFIED IDEOGRAPH:'E4BF:58559:&#x977D
CJK UNIFIED IDEOGRAPH:'E4C0:58560:&#x977A
CJK UNIFIED IDEOGRAPH:'E4C1:58561:&#x977E
CJK UNIFIED IDEOGRAPH:'E4C2:58562:&#x9783
CJK UNIFIED IDEOGRAPH:'E4C3:58563:&#x9780
CJK UNIFIED IDEOGRAPH:'E4C4:58564:&#x9782
CJK UNIFIED IDEOGRAPH:'E4C5:58565:&#x977B
CJK UNIFIED IDEOGRAPH:'E4C6:58566:&#x9784
CJK UNIFIED IDEOGRAPH:'E4C7:58567:&#x9781
CJK UNIFIED IDEOGRAPH:'E4C8:58568:&#x977F
CJK UNIFIED IDEOGRAPH:'E4C9:58569:&#x97CE
CJK UNIFIED IDEOGRAPH:'E4CA:58570:&#x97CD
CJK UNIFIED IDEOGRAPH:'E4CB:58571:&#x9816
CJK UNIFIED IDEOGRAPH:'E4CC:58572:&#x98AD
CJK UNIFIED IDEOGRAPH:'E4CD:58573:&#x98AE
CJK UNIFIED IDEOGRAPH:'E4CE:58574:&#x9902
CJK UNIFIED IDEOGRAPH:'E4CF:58575:&#x9900
CJK UNIFIED IDEOGRAPH:'E4D0:58576:&#x9907
CJK UNIFIED IDEOGRAPH:'E4D1:58577:&#x999D
CJK UNIFIED IDEOGRAPH:'E4D2:58578:&#x999C
CJK UNIFIED IDEOGRAPH:'E4D3:58579:&#x99C3
CJK UNIFIED IDEOGRAPH:'E4D4:58580:&#x99B9
CJK UNIFIED IDEOGRAPH:'E4D5:58581:&#x99BB
CJK UNIFIED IDEOGRAPH:'E4D6:58582:&#x99BA
CJK UNIFIED IDEOGRAPH:'E4D7:58583:&#x99C2
CJK UNIFIED IDEOGRAPH:'E4D8:58584:&#x99BD
CJK UNIFIED IDEOGRAPH:'E4D9:58585:&#x99C7
CJK UNIFIED IDEOGRAPH:'E4DA:58586:&#x9AB1
CJK UNIFIED IDEOGRAPH:'E4DB:58587:&#x9AE3
CJK UNIFIED IDEOGRAPH:'E4DC:58588:&#x9AE7
CJK UNIFIED IDEOGRAPH:'E4DD:58589:&#x9B3E
CJK UNIFIED IDEOGRAPH:'E4DE:58590:&#x9B3F
CJK UNIFIED IDEOGRAPH:'E4DF:58591:&#x9B60
CJK UNIFIED IDEOGRAPH:'E4E0:58592:&#x9B61
CJK UNIFIED IDEOGRAPH:'E4E1:58593:&#x9B5F
CJK UNIFIED IDEOGRAPH:'E4E2:58594:&#x9CF1
CJK UNIFIED IDEOGRAPH:'E4E3:58595:&#x9CF2
CJK UNIFIED IDEOGRAPH:'E4E4:58596:&#x9CF5
CJK UNIFIED IDEOGRAPH:'E4E5:58597:&#x9EA7
CJK UNIFIED IDEOGRAPH:'E4E6:58598:&#x50FF
CJK UNIFIED IDEOGRAPH:'E4E7:58599:&#x5103
CJK UNIFIED IDEOGRAPH:'E4E8:58600:&#x5130
CJK UNIFIED IDEOGRAPH:'E4E9:58601:&#x50F8
CJK UNIFIED IDEOGRAPH:'E4EA:58602:&#x5106
CJK UNIFIED IDEOGRAPH:'E4EB:58603:&#x5107
CJK UNIFIED IDEOGRAPH:'E4EC:58604:&#x50F6
CJK UNIFIED IDEOGRAPH:'E4ED:58605:&#x50FE
CJK UNIFIED IDEOGRAPH:'E4EE:58606:&#x510B
CJK UNIFIED IDEOGRAPH:'E4EF:58607:&#x510C
CJK UNIFIED IDEOGRAPH:'E4F0:58608:&#x50FD
CJK UNIFIED IDEOGRAPH:'E4F1:58609:&#x510A
CJK UNIFIED IDEOGRAPH:'E4F2:58610:&#x528B
CJK UNIFIED IDEOGRAPH:'E4F3:58611:&#x528C
CJK UNIFIED IDEOGRAPH:'E4F4:58612:&#x52F1
CJK UNIFIED IDEOGRAPH:'E4F5:58613:&#x52EF
CJK UNIFIED IDEOGRAPH:'E4F6:58614:&#x5648
CJK UNIFIED IDEOGRAPH:'E4F7:58615:&#x5642
CJK UNIFIED IDEOGRAPH:'E4F8:58616:&#x564C
CJK UNIFIED IDEOGRAPH:'E4F9:58617:&#x5635
CJK UNIFIED IDEOGRAPH:'E4FA:58618:&#x5641
CJK UNIFIED IDEOGRAPH:'E4FB:58619:&#x564A
CJK UNIFIED IDEOGRAPH:'E4FC:58620:&#x5649
CJK UNIFIED IDEOGRAPH:'E4FD:58621:&#x5646
CJK UNIFIED IDEOGRAPH:'E4FE:58622:&#x5658
CJK UNIFIED IDEOGRAPH:'E540:58688:&#x565A
CJK UNIFIED IDEOGRAPH:'E541:58689:&#x5640
CJK UNIFIED IDEOGRAPH:'E542:58690:&#x5633
CJK UNIFIED IDEOGRAPH:'E543:58691:&#x563D
CJK UNIFIED IDEOGRAPH:'E544:58692:&#x562C
CJK UNIFIED IDEOGRAPH:'E545:58693:&#x563E
CJK UNIFIED IDEOGRAPH:'E546:58694:&#x5638
CJK UNIFIED IDEOGRAPH:'E547:58695:&#x562A
CJK UNIFIED IDEOGRAPH:'E548:58696:&#x563A
CJK UNIFIED IDEOGRAPH:'E549:58697:&#x571A
CJK UNIFIED IDEOGRAPH:'E54A:58698:&#x58AB
CJK UNIFIED IDEOGRAPH:'E54B:58699:&#x589D
CJK UNIFIED IDEOGRAPH:'E54C:58700:&#x58B1
CJK UNIFIED IDEOGRAPH:'E54D:58701:&#x58A0
CJK UNIFIED IDEOGRAPH:'E54E:58702:&#x58A3
CJK UNIFIED IDEOGRAPH:'E54F:58703:&#x58AF
CJK UNIFIED IDEOGRAPH:'E550:58704:&#x58AC
CJK UNIFIED IDEOGRAPH:'E551:58705:&#x58A5
CJK UNIFIED IDEOGRAPH:'E552:58706:&#x58A1
CJK UNIFIED IDEOGRAPH:'E553:58707:&#x58FF
CJK UNIFIED IDEOGRAPH:'E554:58708:&#x5AFF
CJK UNIFIED IDEOGRAPH:'E555:58709:&#x5AF4
CJK UNIFIED IDEOGRAPH:'E556:58710:&#x5AFD
CJK UNIFIED IDEOGRAPH:'E557:58711:&#x5AF7
CJK UNIFIED IDEOGRAPH:'E558:58712:&#x5AF6
CJK UNIFIED IDEOGRAPH:'E559:58713:&#x5B03
CJK UNIFIED IDEOGRAPH:'E55A:58714:&#x5AF8
CJK UNIFIED IDEOGRAPH:'E55B:58715:&#x5B02
CJK UNIFIED IDEOGRAPH:'E55C:58716:&#x5AF9
CJK UNIFIED IDEOGRAPH:'E55D:58717:&#x5B01
CJK UNIFIED IDEOGRAPH:'E55E:58718:&#x5B07
CJK UNIFIED IDEOGRAPH:'E55F:58719:&#x5B05
CJK UNIFIED IDEOGRAPH:'E560:58720:&#x5B0F
CJK UNIFIED IDEOGRAPH:'E561:58721:&#x5C67
CJK UNIFIED IDEOGRAPH:'E562:58722:&#x5D99
CJK UNIFIED IDEOGRAPH:'E563:58723:&#x5D97
CJK UNIFIED IDEOGRAPH:'E564:58724:&#x5D9F
CJK UNIFIED IDEOGRAPH:'E565:58725:&#x5D92
CJK UNIFIED IDEOGRAPH:'E566:58726:&#x5DA2
CJK UNIFIED IDEOGRAPH:'E567:58727:&#x5D93
CJK UNIFIED IDEOGRAPH:'E568:58728:&#x5D95
CJK UNIFIED IDEOGRAPH:'E569:58729:&#x5DA0
CJK UNIFIED IDEOGRAPH:'E56A:58730:&#x5D9C
CJK UNIFIED IDEOGRAPH:'E56B:58731:&#x5DA1
CJK UNIFIED IDEOGRAPH:'E56C:58732:&#x5D9A
CJK UNIFIED IDEOGRAPH:'E56D:58733:&#x5D9E
CJK UNIFIED IDEOGRAPH:'E56E:58734:&#x5E69
CJK UNIFIED IDEOGRAPH:'E56F:58735:&#x5E5D
CJK UNIFIED IDEOGRAPH:'E570:58736:&#x5E60
CJK UNIFIED IDEOGRAPH:'E571:58737:&#x5E5C
CJK UNIFIED IDEOGRAPH:'E572:58738:&#x7DF3
CJK UNIFIED IDEOGRAPH:'E573:58739:&#x5EDB
CJK UNIFIED IDEOGRAPH:'E574:58740:&#x5EDE
CJK UNIFIED IDEOGRAPH:'E575:58741:&#x5EE1
CJK UNIFIED IDEOGRAPH:'E576:58742:&#x5F49
CJK UNIFIED IDEOGRAPH:'E577:58743:&#x5FB2
CJK UNIFIED IDEOGRAPH:'E578:58744:&#x618B
CJK UNIFIED IDEOGRAPH:'E579:58745:&#x6183
CJK UNIFIED IDEOGRAPH:'E57A:58746:&#x6179
CJK UNIFIED IDEOGRAPH:'E57B:58747:&#x61B1
CJK UNIFIED IDEOGRAPH:'E57C:58748:&#x61B0
CJK UNIFIED IDEOGRAPH:'E57D:58749:&#x61A2
CJK UNIFIED IDEOGRAPH:'E57E:58750:&#x6189
CJK UNIFIED IDEOGRAPH:'E5A1:58785:&#x619B
CJK UNIFIED IDEOGRAPH:'E5A2:58786:&#x6193
CJK UNIFIED IDEOGRAPH:'E5A3:58787:&#x61AF
CJK UNIFIED IDEOGRAPH:'E5A4:58788:&#x61AD
CJK UNIFIED IDEOGRAPH:'E5A5:58789:&#x619F
CJK UNIFIED IDEOGRAPH:'E5A6:58790:&#x6192
CJK UNIFIED IDEOGRAPH:'E5A7:58791:&#x61AA
CJK UNIFIED IDEOGRAPH:'E5A8:58792:&#x61A1
CJK UNIFIED IDEOGRAPH:'E5A9:58793:&#x618D
CJK UNIFIED IDEOGRAPH:'E5AA:58794:&#x6166
CJK UNIFIED IDEOGRAPH:'E5AB:58795:&#x61B3
CJK UNIFIED IDEOGRAPH:'E5AC:58796:&#x622D
CJK UNIFIED IDEOGRAPH:'E5AD:58797:&#x646E
CJK UNIFIED IDEOGRAPH:'E5AE:58798:&#x6470
CJK UNIFIED IDEOGRAPH:'E5AF:58799:&#x6496
CJK UNIFIED IDEOGRAPH:'E5B0:58800:&#x64A0
CJK UNIFIED IDEOGRAPH:'E5B1:58801:&#x6485
CJK UNIFIED IDEOGRAPH:'E5B2:58802:&#x6497
CJK UNIFIED IDEOGRAPH:'E5B3:58803:&#x649C
CJK UNIFIED IDEOGRAPH:'E5B4:58804:&#x648F
CJK UNIFIED IDEOGRAPH:'E5B5:58805:&#x648B
CJK UNIFIED IDEOGRAPH:'E5B6:58806:&#x648A
CJK UNIFIED IDEOGRAPH:'E5B7:58807:&#x648C
CJK UNIFIED IDEOGRAPH:'E5B8:58808:&#x64A3
CJK UNIFIED IDEOGRAPH:'E5B9:58809:&#x649F
CJK UNIFIED IDEOGRAPH:'E5BA:58810:&#x6468
CJK UNIFIED IDEOGRAPH:'E5BB:58811:&#x64B1
CJK UNIFIED IDEOGRAPH:'E5BC:58812:&#x6498
CJK UNIFIED IDEOGRAPH:'E5BD:58813:&#x6576
CJK UNIFIED IDEOGRAPH:'E5BE:58814:&#x657A
CJK UNIFIED IDEOGRAPH:'E5BF:58815:&#x6579
CJK UNIFIED IDEOGRAPH:'E5C0:58816:&#x657B
CJK UNIFIED IDEOGRAPH:'E5C1:58817:&#x65B2
CJK UNIFIED IDEOGRAPH:'E5C2:58818:&#x65B3
CJK UNIFIED IDEOGRAPH:'E5C3:58819:&#x66B5
CJK UNIFIED IDEOGRAPH:'E5C4:58820:&#x66B0
CJK UNIFIED IDEOGRAPH:'E5C5:58821:&#x66A9
CJK UNIFIED IDEOGRAPH:'E5C6:58822:&#x66B2
CJK UNIFIED IDEOGRAPH:'E5C7:58823:&#x66B7
CJK UNIFIED IDEOGRAPH:'E5C8:58824:&#x66AA
CJK UNIFIED IDEOGRAPH:'E5C9:58825:&#x66AF
CJK UNIFIED IDEOGRAPH:'E5CA:58826:&#x6A00
CJK UNIFIED IDEOGRAPH:'E5CB:58827:&#x6A06
CJK UNIFIED IDEOGRAPH:'E5CC:58828:&#x6A17
CJK UNIFIED IDEOGRAPH:'E5CD:58829:&#x69E5
CJK UNIFIED IDEOGRAPH:'E5CE:58830:&#x69F8
CJK UNIFIED IDEOGRAPH:'E5CF:58831:&#x6A15
CJK UNIFIED IDEOGRAPH:'E5D0:58832:&#x69F1
CJK UNIFIED IDEOGRAPH:'E5D1:58833:&#x69E4
CJK UNIFIED IDEOGRAPH:'E5D2:58834:&#x6A20
CJK UNIFIED IDEOGRAPH:'E5D3:58835:&#x69FF
CJK UNIFIED IDEOGRAPH:'E5D4:58836:&#x69EC
CJK UNIFIED IDEOGRAPH:'E5D5:58837:&#x69E2
CJK UNIFIED IDEOGRAPH:'E5D6:58838:&#x6A1B
CJK UNIFIED IDEOGRAPH:'E5D7:58839:&#x6A1D
CJK UNIFIED IDEOGRAPH:'E5D8:58840:&#x69FE
CJK UNIFIED IDEOGRAPH:'E5D9:58841:&#x6A27
CJK UNIFIED IDEOGRAPH:'E5DA:58842:&#x69F2
CJK UNIFIED IDEOGRAPH:'E5DB:58843:&#x69EE
CJK UNIFIED IDEOGRAPH:'E5DC:58844:&#x6A14
CJK UNIFIED IDEOGRAPH:'E5DD:58845:&#x69F7
CJK UNIFIED IDEOGRAPH:'E5DE:58846:&#x69E7
CJK UNIFIED IDEOGRAPH:'E5DF:58847:&#x6A40
CJK UNIFIED IDEOGRAPH:'E5E0:58848:&#x6A08
CJK UNIFIED IDEOGRAPH:'E5E1:58849:&#x69E6
CJK UNIFIED IDEOGRAPH:'E5E2:58850:&#x69FB
CJK UNIFIED IDEOGRAPH:'E5E3:58851:&#x6A0D
CJK UNIFIED IDEOGRAPH:'E5E4:58852:&#x69FC
CJK UNIFIED IDEOGRAPH:'E5E5:58853:&#x69EB
CJK UNIFIED IDEOGRAPH:'E5E6:58854:&#x6A09
CJK UNIFIED IDEOGRAPH:'E5E7:58855:&#x6A04
CJK UNIFIED IDEOGRAPH:'E5E8:58856:&#x6A18
CJK UNIFIED IDEOGRAPH:'E5E9:58857:&#x6A25
CJK UNIFIED IDEOGRAPH:'E5EA:58858:&#x6A0F
CJK UNIFIED IDEOGRAPH:'E5EB:58859:&#x69F6
CJK UNIFIED IDEOGRAPH:'E5EC:58860:&#x6A26
CJK UNIFIED IDEOGRAPH:'E5ED:58861:&#x6A07
CJK UNIFIED IDEOGRAPH:'E5EE:58862:&#x69F4
CJK UNIFIED IDEOGRAPH:'E5EF:58863:&#x6A16
CJK UNIFIED IDEOGRAPH:'E5F0:58864:&#x6B51
CJK UNIFIED IDEOGRAPH:'E5F1:58865:&#x6BA5
CJK UNIFIED IDEOGRAPH:'E5F2:58866:&#x6BA3
CJK UNIFIED IDEOGRAPH:'E5F3:58867:&#x6BA2
CJK UNIFIED IDEOGRAPH:'E5F4:58868:&#x6BA6
CJK UNIFIED IDEOGRAPH:'E5F5:58869:&#x6C01
CJK UNIFIED IDEOGRAPH:'E5F6:58870:&#x6C00
CJK UNIFIED IDEOGRAPH:'E5F7:58871:&#x6BFF
CJK UNIFIED IDEOGRAPH:'E5F8:58872:&#x6C02
CJK UNIFIED IDEOGRAPH:'E5F9:58873:&#x6F41
CJK UNIFIED IDEOGRAPH:'E5FA:58874:&#x6F26
CJK UNIFIED IDEOGRAPH:'E5FB:58875:&#x6F7E
CJK UNIFIED IDEOGRAPH:'E5FC:58876:&#x6F87
CJK UNIFIED IDEOGRAPH:'E5FD:58877:&#x6FC6
CJK UNIFIED IDEOGRAPH:'E5FE:58878:&#x6F92
CJK UNIFIED IDEOGRAPH:'E640:58944:&#x6F8D
CJK UNIFIED IDEOGRAPH:'E641:58945:&#x6F89
CJK UNIFIED IDEOGRAPH:'E642:58946:&#x6F8C
CJK UNIFIED IDEOGRAPH:'E643:58947:&#x6F62
CJK UNIFIED IDEOGRAPH:'E644:58948:&#x6F4F
CJK UNIFIED IDEOGRAPH:'E645:58949:&#x6F85
CJK UNIFIED IDEOGRAPH:'E646:58950:&#x6F5A
CJK UNIFIED IDEOGRAPH:'E647:58951:&#x6F96
CJK UNIFIED IDEOGRAPH:'E648:58952:&#x6F76
CJK UNIFIED IDEOGRAPH:'E649:58953:&#x6F6C
CJK UNIFIED IDEOGRAPH:'E64A:58954:&#x6F82
CJK UNIFIED IDEOGRAPH:'E64B:58955:&#x6F55
CJK UNIFIED IDEOGRAPH:'E64C:58956:&#x6F72
CJK UNIFIED IDEOGRAPH:'E64D:58957:&#x6F52
CJK UNIFIED IDEOGRAPH:'E64E:58958:&#x6F50
CJK UNIFIED IDEOGRAPH:'E64F:58959:&#x6F57
CJK UNIFIED IDEOGRAPH:'E650:58960:&#x6F94
CJK UNIFIED IDEOGRAPH:'E651:58961:&#x6F93
CJK UNIFIED IDEOGRAPH:'E652:58962:&#x6F5D
CJK UNIFIED IDEOGRAPH:'E653:58963:&#x6F00
CJK UNIFIED IDEOGRAPH:'E654:58964:&#x6F61
CJK UNIFIED IDEOGRAPH:'E655:58965:&#x6F6B
CJK UNIFIED IDEOGRAPH:'E656:58966:&#x6F7D
CJK UNIFIED IDEOGRAPH:'E657:58967:&#x6F67
CJK UNIFIED IDEOGRAPH:'E658:58968:&#x6F90
CJK UNIFIED IDEOGRAPH:'E659:58969:&#x6F53
CJK UNIFIED IDEOGRAPH:'E65A:58970:&#x6F8B
CJK UNIFIED IDEOGRAPH:'E65B:58971:&#x6F69
CJK UNIFIED IDEOGRAPH:'E65C:58972:&#x6F7F
CJK UNIFIED IDEOGRAPH:'E65D:58973:&#x6F95
CJK UNIFIED IDEOGRAPH:'E65E:58974:&#x6F63
CJK UNIFIED IDEOGRAPH:'E65F:58975:&#x6F77
CJK UNIFIED IDEOGRAPH:'E660:58976:&#x6F6A
CJK UNIFIED IDEOGRAPH:'E661:58977:&#x6F7B
CJK UNIFIED IDEOGRAPH:'E662:58978:&#x71B2
CJK UNIFIED IDEOGRAPH:'E663:58979:&#x71AF
CJK UNIFIED IDEOGRAPH:'E664:58980:&#x719B
CJK UNIFIED IDEOGRAPH:'E665:58981:&#x71B0
CJK UNIFIED IDEOGRAPH:'E666:58982:&#x71A0
CJK UNIFIED IDEOGRAPH:'E667:58983:&#x719A
CJK UNIFIED IDEOGRAPH:'E668:58984:&#x71A9
CJK UNIFIED IDEOGRAPH:'E669:58985:&#x71B5
CJK UNIFIED IDEOGRAPH:'E66A:58986:&#x719D
CJK UNIFIED IDEOGRAPH:'E66B:58987:&#x71A5
CJK UNIFIED IDEOGRAPH:'E66C:58988:&#x719E
CJK UNIFIED IDEOGRAPH:'E66D:58989:&#x71A4
CJK UNIFIED IDEOGRAPH:'E66E:58990:&#x71A1
CJK UNIFIED IDEOGRAPH:'E66F:58991:&#x71AA
CJK UNIFIED IDEOGRAPH:'E670:58992:&#x719C
CJK UNIFIED IDEOGRAPH:'E671:58993:&#x71A7
CJK UNIFIED IDEOGRAPH:'E672:58994:&#x71B3
CJK UNIFIED IDEOGRAPH:'E673:58995:&#x7298
CJK UNIFIED IDEOGRAPH:'E674:58996:&#x729A
CJK UNIFIED IDEOGRAPH:'E675:58997:&#x7358
CJK UNIFIED IDEOGRAPH:'E676:58998:&#x7352
CJK UNIFIED IDEOGRAPH:'E677:58999:&#x735E
CJK UNIFIED IDEOGRAPH:'E678:59000:&#x735F
CJK UNIFIED IDEOGRAPH:'E679:59001:&#x7360
CJK UNIFIED IDEOGRAPH:'E67A:59002:&#x735D
CJK UNIFIED IDEOGRAPH:'E67B:59003:&#x735B
CJK UNIFIED IDEOGRAPH:'E67C:59004:&#x7361
CJK UNIFIED IDEOGRAPH:'E67D:59005:&#x735A
CJK UNIFIED IDEOGRAPH:'E67E:59006:&#x7359
CJK UNIFIED IDEOGRAPH:'E6A1:59041:&#x7362
CJK UNIFIED IDEOGRAPH:'E6A2:59042:&#x7487
CJK UNIFIED IDEOGRAPH:'E6A3:59043:&#x7489
CJK UNIFIED IDEOGRAPH:'E6A4:59044:&#x748A
CJK UNIFIED IDEOGRAPH:'E6A5:59045:&#x7486
CJK UNIFIED IDEOGRAPH:'E6A6:59046:&#x7481
CJK UNIFIED IDEOGRAPH:'E6A7:59047:&#x747D
CJK UNIFIED IDEOGRAPH:'E6A8:59048:&#x7485
CJK UNIFIED IDEOGRAPH:'E6A9:59049:&#x7488
CJK UNIFIED IDEOGRAPH:'E6AA:59050:&#x747C
CJK UNIFIED IDEOGRAPH:'E6AB:59051:&#x7479
CJK UNIFIED IDEOGRAPH:'E6AC:59052:&#x7508
CJK UNIFIED IDEOGRAPH:'E6AD:59053:&#x7507
CJK UNIFIED IDEOGRAPH:'E6AE:59054:&#x757E
CJK UNIFIED IDEOGRAPH:'E6AF:59055:&#x7625
CJK UNIFIED IDEOGRAPH:'E6B0:59056:&#x761E
CJK UNIFIED IDEOGRAPH:'E6B1:59057:&#x7619
CJK UNIFIED IDEOGRAPH:'E6B2:59058:&#x761D
CJK UNIFIED IDEOGRAPH:'E6B3:59059:&#x761C
CJK UNIFIED IDEOGRAPH:'E6B4:59060:&#x7623
CJK UNIFIED IDEOGRAPH:'E6B5:59061:&#x761A
CJK UNIFIED IDEOGRAPH:'E6B6:59062:&#x7628
CJK UNIFIED IDEOGRAPH:'E6B7:59063:&#x761B
CJK UNIFIED IDEOGRAPH:'E6B8:59064:&#x769C
CJK UNIFIED IDEOGRAPH:'E6B9:59065:&#x769D
CJK UNIFIED IDEOGRAPH:'E6BA:59066:&#x769E
CJK UNIFIED IDEOGRAPH:'E6BB:59067:&#x769B
CJK UNIFIED IDEOGRAPH:'E6BC:59068:&#x778D
CJK UNIFIED IDEOGRAPH:'E6BD:59069:&#x778F
CJK UNIFIED IDEOGRAPH:'E6BE:59070:&#x7789
CJK UNIFIED IDEOGRAPH:'E6BF:59071:&#x7788
CJK UNIFIED IDEOGRAPH:'E6C0:59072:&#x78CD
CJK UNIFIED IDEOGRAPH:'E6C1:59073:&#x78BB
CJK UNIFIED IDEOGRAPH:'E6C2:59074:&#x78CF
CJK UNIFIED IDEOGRAPH:'E6C3:59075:&#x78CC
CJK UNIFIED IDEOGRAPH:'E6C4:59076:&#x78D1
CJK UNIFIED IDEOGRAPH:'E6C5:59077:&#x78CE
CJK UNIFIED IDEOGRAPH:'E6C6:59078:&#x78D4
CJK UNIFIED IDEOGRAPH:'E6C7:59079:&#x78C8
CJK UNIFIED IDEOGRAPH:'E6C8:59080:&#x78C3
CJK UNIFIED IDEOGRAPH:'E6C9:59081:&#x78C4
CJK UNIFIED IDEOGRAPH:'E6CA:59082:&#x78C9
CJK UNIFIED IDEOGRAPH:'E6CB:59083:&#x799A
CJK UNIFIED IDEOGRAPH:'E6CC:59084:&#x79A1
CJK UNIFIED IDEOGRAPH:'E6CD:59085:&#x79A0
CJK UNIFIED IDEOGRAPH:'E6CE:59086:&#x799C
CJK UNIFIED IDEOGRAPH:'E6CF:59087:&#x79A2
CJK UNIFIED IDEOGRAPH:'E6D0:59088:&#x799B
CJK UNIFIED IDEOGRAPH:'E6D1:59089:&#x6B76
CJK UNIFIED IDEOGRAPH:'E6D2:59090:&#x7A39
CJK UNIFIED IDEOGRAPH:'E6D3:59091:&#x7AB2
CJK UNIFIED IDEOGRAPH:'E6D4:59092:&#x7AB4
CJK UNIFIED IDEOGRAPH:'E6D5:59093:&#x7AB3
CJK UNIFIED IDEOGRAPH:'E6D6:59094:&#x7BB7
CJK UNIFIED IDEOGRAPH:'E6D7:59095:&#x7BCB
CJK UNIFIED IDEOGRAPH:'E6D8:59096:&#x7BBE
CJK UNIFIED IDEOGRAPH:'E6D9:59097:&#x7BAC
CJK UNIFIED IDEOGRAPH:'E6DA:59098:&#x7BCE
CJK UNIFIED IDEOGRAPH:'E6DB:59099:&#x7BAF
CJK UNIFIED IDEOGRAPH:'E6DC:59100:&#x7BB9
CJK UNIFIED IDEOGRAPH:'E6DD:59101:&#x7BCA
CJK UNIFIED IDEOGRAPH:'E6DE:59102:&#x7BB5
CJK UNIFIED IDEOGRAPH:'E6DF:59103:&#x7CC5
CJK UNIFIED IDEOGRAPH:'E6E0:59104:&#x7CC8
CJK UNIFIED IDEOGRAPH:'E6E1:59105:&#x7CCC
CJK UNIFIED IDEOGRAPH:'E6E2:59106:&#x7CCB
CJK UNIFIED IDEOGRAPH:'E6E3:59107:&#x7DF7
CJK UNIFIED IDEOGRAPH:'E6E4:59108:&#x7DDB
CJK UNIFIED IDEOGRAPH:'E6E5:59109:&#x7DEA
CJK UNIFIED IDEOGRAPH:'E6E6:59110:&#x7DE7
CJK UNIFIED IDEOGRAPH:'E6E7:59111:&#x7DD7
CJK UNIFIED IDEOGRAPH:'E6E8:59112:&#x7DE1
CJK UNIFIED IDEOGRAPH:'E6E9:59113:&#x7E03
CJK UNIFIED IDEOGRAPH:'E6EA:59114:&#x7DFA
CJK UNIFIED IDEOGRAPH:'E6EB:59115:&#x7DE6
CJK UNIFIED IDEOGRAPH:'E6EC:59116:&#x7DF6
CJK UNIFIED IDEOGRAPH:'E6ED:59117:&#x7DF1
CJK UNIFIED IDEOGRAPH:'E6EE:59118:&#x7DF0
CJK UNIFIED IDEOGRAPH:'E6EF:59119:&#x7DEE
CJK UNIFIED IDEOGRAPH:'E6F0:59120:&#x7DDF
CJK UNIFIED IDEOGRAPH:'E6F1:59121:&#x7F76
CJK UNIFIED IDEOGRAPH:'E6F2:59122:&#x7FAC
CJK UNIFIED IDEOGRAPH:'E6F3:59123:&#x7FB0
CJK UNIFIED IDEOGRAPH:'E6F4:59124:&#x7FAD
CJK UNIFIED IDEOGRAPH:'E6F5:59125:&#x7FED
CJK UNIFIED IDEOGRAPH:'E6F6:59126:&#x7FEB
CJK UNIFIED IDEOGRAPH:'E6F7:59127:&#x7FEA
CJK UNIFIED IDEOGRAPH:'E6F8:59128:&#x7FEC
CJK UNIFIED IDEOGRAPH:'E6F9:59129:&#x7FE6
CJK UNIFIED IDEOGRAPH:'E6FA:59130:&#x7FE8
CJK UNIFIED IDEOGRAPH:'E6FB:59131:&#x8064
CJK UNIFIED IDEOGRAPH:'E6FC:59132:&#x8067
CJK UNIFIED IDEOGRAPH:'E6FD:59133:&#x81A3
CJK UNIFIED IDEOGRAPH:'E6FE:59134:&#x819F
CJK UNIFIED IDEOGRAPH:'E740:59200:&#x819E
CJK UNIFIED IDEOGRAPH:'E741:59201:&#x8195
CJK UNIFIED IDEOGRAPH:'E742:59202:&#x81A2
CJK UNIFIED IDEOGRAPH:'E743:59203:&#x8199
CJK UNIFIED IDEOGRAPH:'E744:59204:&#x8197
CJK UNIFIED IDEOGRAPH:'E745:59205:&#x8216
CJK UNIFIED IDEOGRAPH:'E746:59206:&#x824F
CJK UNIFIED IDEOGRAPH:'E747:59207:&#x8253
CJK UNIFIED IDEOGRAPH:'E748:59208:&#x8252
CJK UNIFIED IDEOGRAPH:'E749:59209:&#x8250
CJK UNIFIED IDEOGRAPH:'E74A:59210:&#x824E
CJK UNIFIED IDEOGRAPH:'E74B:59211:&#x8251
CJK UNIFIED IDEOGRAPH:'E74C:59212:&#x8524
CJK UNIFIED IDEOGRAPH:'E74D:59213:&#x853B
CJK UNIFIED IDEOGRAPH:'E74E:59214:&#x850F
CJK UNIFIED IDEOGRAPH:'E74F:59215:&#x8500
CJK UNIFIED IDEOGRAPH:'E750:59216:&#x8529
CJK UNIFIED IDEOGRAPH:'E751:59217:&#x850E
CJK UNIFIED IDEOGRAPH:'E752:59218:&#x8509
CJK UNIFIED IDEOGRAPH:'E753:59219:&#x850D
CJK UNIFIED IDEOGRAPH:'E754:59220:&#x851F
CJK UNIFIED IDEOGRAPH:'E755:59221:&#x850A
CJK UNIFIED IDEOGRAPH:'E756:59222:&#x8527
CJK UNIFIED IDEOGRAPH:'E757:59223:&#x851C
CJK UNIFIED IDEOGRAPH:'E758:59224:&#x84FB
CJK UNIFIED IDEOGRAPH:'E759:59225:&#x852B
CJK UNIFIED IDEOGRAPH:'E75A:59226:&#x84FA
CJK UNIFIED IDEOGRAPH:'E75B:59227:&#x8508
CJK UNIFIED IDEOGRAPH:'E75C:59228:&#x850C
CJK UNIFIED IDEOGRAPH:'E75D:59229:&#x84F4
CJK UNIFIED IDEOGRAPH:'E75E:59230:&#x852A
CJK UNIFIED IDEOGRAPH:'E75F:59231:&#x84F2
CJK UNIFIED IDEOGRAPH:'E760:59232:&#x8515
CJK UNIFIED IDEOGRAPH:'E761:59233:&#x84F7
CJK UNIFIED IDEOGRAPH:'E762:59234:&#x84EB
CJK UNIFIED IDEOGRAPH:'E763:59235:&#x84F3
CJK UNIFIED IDEOGRAPH:'E764:59236:&#x84FC
CJK UNIFIED IDEOGRAPH:'E765:59237:&#x8512
CJK UNIFIED IDEOGRAPH:'E766:59238:&#x84EA
CJK UNIFIED IDEOGRAPH:'E767:59239:&#x84E9
CJK UNIFIED IDEOGRAPH:'E768:59240:&#x8516
CJK UNIFIED IDEOGRAPH:'E769:59241:&#x84FE
CJK UNIFIED IDEOGRAPH:'E76A:59242:&#x8528
CJK UNIFIED IDEOGRAPH:'E76B:59243:&#x851D
CJK UNIFIED IDEOGRAPH:'E76C:59244:&#x852E
CJK UNIFIED IDEOGRAPH:'E76D:59245:&#x8502
CJK UNIFIED IDEOGRAPH:'E76E:59246:&#x84FD
CJK UNIFIED IDEOGRAPH:'E76F:59247:&#x851E
CJK UNIFIED IDEOGRAPH:'E770:59248:&#x84F6
CJK UNIFIED IDEOGRAPH:'E771:59249:&#x8531
CJK UNIFIED IDEOGRAPH:'E772:59250:&#x8526
CJK UNIFIED IDEOGRAPH:'E773:59251:&#x84E7
CJK UNIFIED IDEOGRAPH:'E774:59252:&#x84E8
CJK UNIFIED IDEOGRAPH:'E775:59253:&#x84F0
CJK UNIFIED IDEOGRAPH:'E776:59254:&#x84EF
CJK UNIFIED IDEOGRAPH:'E777:59255:&#x84F9
CJK UNIFIED IDEOGRAPH:'E778:59256:&#x8518
CJK UNIFIED IDEOGRAPH:'E779:59257:&#x8520
CJK UNIFIED IDEOGRAPH:'E77A:59258:&#x8530
CJK UNIFIED IDEOGRAPH:'E77B:59259:&#x850B
CJK UNIFIED IDEOGRAPH:'E77C:59260:&#x8519
CJK UNIFIED IDEOGRAPH:'E77D:59261:&#x852F
CJK UNIFIED IDEOGRAPH:'E77E:59262:&#x8662
CJK UNIFIED IDEOGRAPH:'E7A1:59297:&#x8756
CJK UNIFIED IDEOGRAPH:'E7A2:59298:&#x8763
CJK UNIFIED IDEOGRAPH:'E7A3:59299:&#x8764
CJK UNIFIED IDEOGRAPH:'E7A4:59300:&#x8777
CJK UNIFIED IDEOGRAPH:'E7A5:59301:&#x87E1
CJK UNIFIED IDEOGRAPH:'E7A6:59302:&#x8773
CJK UNIFIED IDEOGRAPH:'E7A7:59303:&#x8758
CJK UNIFIED IDEOGRAPH:'E7A8:59304:&#x8754
CJK UNIFIED IDEOGRAPH:'E7A9:59305:&#x875B
CJK UNIFIED IDEOGRAPH:'E7AA:59306:&#x8752
CJK UNIFIED IDEOGRAPH:'E7AB:59307:&#x8761
CJK UNIFIED IDEOGRAPH:'E7AC:59308:&#x875A
CJK UNIFIED IDEOGRAPH:'E7AD:59309:&#x8751
CJK UNIFIED IDEOGRAPH:'E7AE:59310:&#x875E
CJK UNIFIED IDEOGRAPH:'E7AF:59311:&#x876D
CJK UNIFIED IDEOGRAPH:'E7B0:59312:&#x876A
CJK UNIFIED IDEOGRAPH:'E7B1:59313:&#x8750
CJK UNIFIED IDEOGRAPH:'E7B2:59314:&#x874E
CJK UNIFIED IDEOGRAPH:'E7B3:59315:&#x875F
CJK UNIFIED IDEOGRAPH:'E7B4:59316:&#x875D
CJK UNIFIED IDEOGRAPH:'E7B5:59317:&#x876F
CJK UNIFIED IDEOGRAPH:'E7B6:59318:&#x876C
CJK UNIFIED IDEOGRAPH:'E7B7:59319:&#x877A
CJK UNIFIED IDEOGRAPH:'E7B8:59320:&#x876E
CJK UNIFIED IDEOGRAPH:'E7B9:59321:&#x875C
CJK UNIFIED IDEOGRAPH:'E7BA:59322:&#x8765
CJK UNIFIED IDEOGRAPH:'E7BB:59323:&#x874F
CJK UNIFIED IDEOGRAPH:'E7BC:59324:&#x877B
CJK UNIFIED IDEOGRAPH:'E7BD:59325:&#x8775
CJK UNIFIED IDEOGRAPH:'E7BE:59326:&#x8762
CJK UNIFIED IDEOGRAPH:'E7BF:59327:&#x8767
CJK UNIFIED IDEOGRAPH:'E7C0:59328:&#x8769
CJK UNIFIED IDEOGRAPH:'E7C1:59329:&#x885A
CJK UNIFIED IDEOGRAPH:'E7C2:59330:&#x8905
CJK UNIFIED IDEOGRAPH:'E7C3:59331:&#x890C
CJK UNIFIED IDEOGRAPH:'E7C4:59332:&#x8914
CJK UNIFIED IDEOGRAPH:'E7C5:59333:&#x890B
CJK UNIFIED IDEOGRAPH:'E7C6:59334:&#x8917
CJK UNIFIED IDEOGRAPH:'E7C7:59335:&#x8918
CJK UNIFIED IDEOGRAPH:'E7C8:59336:&#x8919
CJK UNIFIED IDEOGRAPH:'E7C9:59337:&#x8906
CJK UNIFIED IDEOGRAPH:'E7CA:59338:&#x8916
CJK UNIFIED IDEOGRAPH:'E7CB:59339:&#x8911
CJK UNIFIED IDEOGRAPH:'E7CC:59340:&#x890E
CJK UNIFIED IDEOGRAPH:'E7CD:59341:&#x8909
CJK UNIFIED IDEOGRAPH:'E7CE:59342:&#x89A2
CJK UNIFIED IDEOGRAPH:'E7CF:59343:&#x89A4
CJK UNIFIED IDEOGRAPH:'E7D0:59344:&#x89A3
CJK UNIFIED IDEOGRAPH:'E7D1:59345:&#x89ED
CJK UNIFIED IDEOGRAPH:'E7D2:59346:&#x89F0
CJK UNIFIED IDEOGRAPH:'E7D3:59347:&#x89EC
CJK UNIFIED IDEOGRAPH:'E7D4:59348:&#x8ACF
CJK UNIFIED IDEOGRAPH:'E7D5:59349:&#x8AC6
CJK UNIFIED IDEOGRAPH:'E7D6:59350:&#x8AB8
CJK UNIFIED IDEOGRAPH:'E7D7:59351:&#x8AD3
CJK UNIFIED IDEOGRAPH:'E7D8:59352:&#x8AD1
CJK UNIFIED IDEOGRAPH:'E7D9:59353:&#x8AD4
CJK UNIFIED IDEOGRAPH:'E7DA:59354:&#x8AD5
CJK UNIFIED IDEOGRAPH:'E7DB:59355:&#x8ABB
CJK UNIFIED IDEOGRAPH:'E7DC:59356:&#x8AD7
CJK UNIFIED IDEOGRAPH:'E7DD:59357:&#x8ABE
CJK UNIFIED IDEOGRAPH:'E7DE:59358:&#x8AC0
CJK UNIFIED IDEOGRAPH:'E7DF:59359:&#x8AC5
CJK UNIFIED IDEOGRAPH:'E7E0:59360:&#x8AD8
CJK UNIFIED IDEOGRAPH:'E7E1:59361:&#x8AC3
CJK UNIFIED IDEOGRAPH:'E7E2:59362:&#x8ABA
CJK UNIFIED IDEOGRAPH:'E7E3:59363:&#x8ABD
CJK UNIFIED IDEOGRAPH:'E7E4:59364:&#x8AD9
CJK UNIFIED IDEOGRAPH:'E7E5:59365:&#x8C3E
CJK UNIFIED IDEOGRAPH:'E7E6:59366:&#x8C4D
CJK UNIFIED IDEOGRAPH:'E7E7:59367:&#x8C8F
CJK UNIFIED IDEOGRAPH:'E7E8:59368:&#x8CE5
CJK UNIFIED IDEOGRAPH:'E7E9:59369:&#x8CDF
CJK UNIFIED IDEOGRAPH:'E7EA:59370:&#x8CD9
CJK UNIFIED IDEOGRAPH:'E7EB:59371:&#x8CE8
CJK UNIFIED IDEOGRAPH:'E7EC:59372:&#x8CDA
CJK UNIFIED IDEOGRAPH:'E7ED:59373:&#x8CDD
CJK UNIFIED IDEOGRAPH:'E7EE:59374:&#x8CE7
CJK UNIFIED IDEOGRAPH:'E7EF:59375:&#x8DA0
CJK UNIFIED IDEOGRAPH:'E7F0:59376:&#x8D9C
CJK UNIFIED IDEOGRAPH:'E7F1:59377:&#x8DA1
CJK UNIFIED IDEOGRAPH:'E7F2:59378:&#x8D9B
CJK UNIFIED IDEOGRAPH:'E7F3:59379:&#x8E20
CJK UNIFIED IDEOGRAPH:'E7F4:59380:&#x8E23
CJK UNIFIED IDEOGRAPH:'E7F5:59381:&#x8E25
CJK UNIFIED IDEOGRAPH:'E7F6:59382:&#x8E24
CJK UNIFIED IDEOGRAPH:'E7F7:59383:&#x8E2E
CJK UNIFIED IDEOGRAPH:'E7F8:59384:&#x8E15
CJK UNIFIED IDEOGRAPH:'E7F9:59385:&#x8E1B
CJK UNIFIED IDEOGRAPH:'E7FA:59386:&#x8E16
CJK UNIFIED IDEOGRAPH:'E7FB:59387:&#x8E11
CJK UNIFIED IDEOGRAPH:'E7FC:59388:&#x8E19
CJK UNIFIED IDEOGRAPH:'E7FD:59389:&#x8E26
CJK UNIFIED IDEOGRAPH:'E7FE:59390:&#x8E27
CJK UNIFIED IDEOGRAPH:'E840:59456:&#x8E14
CJK UNIFIED IDEOGRAPH:'E841:59457:&#x8E12
CJK UNIFIED IDEOGRAPH:'E842:59458:&#x8E18
CJK UNIFIED IDEOGRAPH:'E843:59459:&#x8E13
CJK UNIFIED IDEOGRAPH:'E844:59460:&#x8E1C
CJK UNIFIED IDEOGRAPH:'E845:59461:&#x8E17
CJK UNIFIED IDEOGRAPH:'E846:59462:&#x8E1A
CJK UNIFIED IDEOGRAPH:'E847:59463:&#x8F2C
CJK UNIFIED IDEOGRAPH:'E848:59464:&#x8F24
CJK UNIFIED IDEOGRAPH:'E849:59465:&#x8F18
CJK UNIFIED IDEOGRAPH:'E84A:59466:&#x8F1A
CJK UNIFIED IDEOGRAPH:'E84B:59467:&#x8F20
CJK UNIFIED IDEOGRAPH:'E84C:59468:&#x8F23
CJK UNIFIED IDEOGRAPH:'E84D:59469:&#x8F16
CJK UNIFIED IDEOGRAPH:'E84E:59470:&#x8F17
CJK UNIFIED IDEOGRAPH:'E84F:59471:&#x9073
CJK UNIFIED IDEOGRAPH:'E850:59472:&#x9070
CJK UNIFIED IDEOGRAPH:'E851:59473:&#x906F
CJK UNIFIED IDEOGRAPH:'E852:59474:&#x9067
CJK UNIFIED IDEOGRAPH:'E853:59475:&#x906B
CJK UNIFIED IDEOGRAPH:'E854:59476:&#x912F
CJK UNIFIED IDEOGRAPH:'E855:59477:&#x912B
CJK UNIFIED IDEOGRAPH:'E856:59478:&#x9129
CJK UNIFIED IDEOGRAPH:'E857:59479:&#x912A
CJK UNIFIED IDEOGRAPH:'E858:59480:&#x9132
CJK UNIFIED IDEOGRAPH:'E859:59481:&#x9126
CJK UNIFIED IDEOGRAPH:'E85A:59482:&#x912E
CJK UNIFIED IDEOGRAPH:'E85B:59483:&#x9185
CJK UNIFIED IDEOGRAPH:'E85C:59484:&#x9186
CJK UNIFIED IDEOGRAPH:'E85D:59485:&#x918A
CJK UNIFIED IDEOGRAPH:'E85E:59486:&#x9181
CJK UNIFIED IDEOGRAPH:'E85F:59487:&#x9182
CJK UNIFIED IDEOGRAPH:'E860:59488:&#x9184
CJK UNIFIED IDEOGRAPH:'E861:59489:&#x9180
CJK UNIFIED IDEOGRAPH:'E862:59490:&#x92D0
CJK UNIFIED IDEOGRAPH:'E863:59491:&#x92C3
CJK UNIFIED IDEOGRAPH:'E864:59492:&#x92C4
CJK UNIFIED IDEOGRAPH:'E865:59493:&#x92C0
CJK UNIFIED IDEOGRAPH:'E866:59494:&#x92D9
CJK UNIFIED IDEOGRAPH:'E867:59495:&#x92B6
CJK UNIFIED IDEOGRAPH:'E868:59496:&#x92CF
CJK UNIFIED IDEOGRAPH:'E869:59497:&#x92F1
CJK UNIFIED IDEOGRAPH:'E86A:59498:&#x92DF
CJK UNIFIED IDEOGRAPH:'E86B:59499:&#x92D8
CJK UNIFIED IDEOGRAPH:'E86C:59500:&#x92E9
CJK UNIFIED IDEOGRAPH:'E86D:59501:&#x92D7
CJK UNIFIED IDEOGRAPH:'E86E:59502:&#x92DD
CJK UNIFIED IDEOGRAPH:'E86F:59503:&#x92CC
CJK UNIFIED IDEOGRAPH:'E870:59504:&#x92EF
CJK UNIFIED IDEOGRAPH:'E871:59505:&#x92C2
CJK UNIFIED IDEOGRAPH:'E872:59506:&#x92E8
CJK UNIFIED IDEOGRAPH:'E873:59507:&#x92CA
CJK UNIFIED IDEOGRAPH:'E874:59508:&#x92C8
CJK UNIFIED IDEOGRAPH:'E875:59509:&#x92CE
CJK UNIFIED IDEOGRAPH:'E876:59510:&#x92E6
CJK UNIFIED IDEOGRAPH:'E877:59511:&#x92CD
CJK UNIFIED IDEOGRAPH:'E878:59512:&#x92D5
CJK UNIFIED IDEOGRAPH:'E879:59513:&#x92C9
CJK UNIFIED IDEOGRAPH:'E87A:59514:&#x92E0
CJK UNIFIED IDEOGRAPH:'E87B:59515:&#x92DE
CJK UNIFIED IDEOGRAPH:'E87C:59516:&#x92E7
CJK UNIFIED IDEOGRAPH:'E87D:59517:&#x92D1
CJK UNIFIED IDEOGRAPH:'E87E:59518:&#x92D3
CJK UNIFIED IDEOGRAPH:'E8A1:59553:&#x92B5
CJK UNIFIED IDEOGRAPH:'E8A2:59554:&#x92E1
CJK UNIFIED IDEOGRAPH:'E8A3:59555:&#x92C6
CJK UNIFIED IDEOGRAPH:'E8A4:59556:&#x92B4
CJK UNIFIED IDEOGRAPH:'E8A5:59557:&#x957C
CJK UNIFIED IDEOGRAPH:'E8A6:59558:&#x95AC
CJK UNIFIED IDEOGRAPH:'E8A7:59559:&#x95AB
CJK UNIFIED IDEOGRAPH:'E8A8:59560:&#x95AE
CJK UNIFIED IDEOGRAPH:'E8A9:59561:&#x95B0
CJK UNIFIED IDEOGRAPH:'E8AA:59562:&#x96A4
CJK UNIFIED IDEOGRAPH:'E8AB:59563:&#x96A2
CJK UNIFIED IDEOGRAPH:'E8AC:59564:&#x96D3
CJK UNIFIED IDEOGRAPH:'E8AD:59565:&#x9705
CJK UNIFIED IDEOGRAPH:'E8AE:59566:&#x9708
CJK UNIFIED IDEOGRAPH:'E8AF:59567:&#x9702
CJK UNIFIED IDEOGRAPH:'E8B0:59568:&#x975A
CJK UNIFIED IDEOGRAPH:'E8B1:59569:&#x978A
CJK UNIFIED IDEOGRAPH:'E8B2:59570:&#x978E
CJK UNIFIED IDEOGRAPH:'E8B3:59571:&#x9788
CJK UNIFIED IDEOGRAPH:'E8B4:59572:&#x97D0
CJK UNIFIED IDEOGRAPH:'E8B5:59573:&#x97CF
CJK UNIFIED IDEOGRAPH:'E8B6:59574:&#x981E
CJK UNIFIED IDEOGRAPH:'E8B7:59575:&#x981D
CJK UNIFIED IDEOGRAPH:'E8B8:59576:&#x9826
CJK UNIFIED IDEOGRAPH:'E8B9:59577:&#x9829
CJK UNIFIED IDEOGRAPH:'E8BA:59578:&#x9828
CJK UNIFIED IDEOGRAPH:'E8BB:59579:&#x9820
CJK UNIFIED IDEOGRAPH:'E8BC:59580:&#x981B
CJK UNIFIED IDEOGRAPH:'E8BD:59581:&#x9827
CJK UNIFIED IDEOGRAPH:'E8BE:59582:&#x98B2
CJK UNIFIED IDEOGRAPH:'E8BF:59583:&#x9908
CJK UNIFIED IDEOGRAPH:'E8C0:59584:&#x98FA
CJK UNIFIED IDEOGRAPH:'E8C1:59585:&#x9911
CJK UNIFIED IDEOGRAPH:'E8C2:59586:&#x9914
CJK UNIFIED IDEOGRAPH:'E8C3:59587:&#x9916
CJK UNIFIED IDEOGRAPH:'E8C4:59588:&#x9917
CJK UNIFIED IDEOGRAPH:'E8C5:59589:&#x9915
CJK UNIFIED IDEOGRAPH:'E8C6:59590:&#x99DC
CJK UNIFIED IDEOGRAPH:'E8C7:59591:&#x99CD
CJK UNIFIED IDEOGRAPH:'E8C8:59592:&#x99CF
CJK UNIFIED IDEOGRAPH:'E8C9:59593:&#x99D3
CJK UNIFIED IDEOGRAPH:'E8CA:59594:&#x99D4
CJK UNIFIED IDEOGRAPH:'E8CB:59595:&#x99CE
CJK UNIFIED IDEOGRAPH:'E8CC:59596:&#x99C9
CJK UNIFIED IDEOGRAPH:'E8CD:59597:&#x99D6
CJK UNIFIED IDEOGRAPH:'E8CE:59598:&#x99D8
CJK UNIFIED IDEOGRAPH:'E8CF:59599:&#x99CB
CJK UNIFIED IDEOGRAPH:'E8D0:59600:&#x99D7
CJK UNIFIED IDEOGRAPH:'E8D1:59601:&#x99CC
CJK UNIFIED IDEOGRAPH:'E8D2:59602:&#x9AB3
CJK UNIFIED IDEOGRAPH:'E8D3:59603:&#x9AEC
CJK UNIFIED IDEOGRAPH:'E8D4:59604:&#x9AEB
CJK UNIFIED IDEOGRAPH:'E8D5:59605:&#x9AF3
CJK UNIFIED IDEOGRAPH:'E8D6:59606:&#x9AF2
CJK UNIFIED IDEOGRAPH:'E8D7:59607:&#x9AF1
CJK UNIFIED IDEOGRAPH:'E8D8:59608:&#x9B46
CJK UNIFIED IDEOGRAPH:'E8D9:59609:&#x9B43
CJK UNIFIED IDEOGRAPH:'E8DA:59610:&#x9B67
CJK UNIFIED IDEOGRAPH:'E8DB:59611:&#x9B74
CJK UNIFIED IDEOGRAPH:'E8DC:59612:&#x9B71
CJK UNIFIED IDEOGRAPH:'E8DD:59613:&#x9B66
CJK UNIFIED IDEOGRAPH:'E8DE:59614:&#x9B76
CJK UNIFIED IDEOGRAPH:'E8DF:59615:&#x9B75
CJK UNIFIED IDEOGRAPH:'E8E0:59616:&#x9B70
CJK UNIFIED IDEOGRAPH:'E8E1:59617:&#x9B68
CJK UNIFIED IDEOGRAPH:'E8E2:59618:&#x9B64
CJK UNIFIED IDEOGRAPH:'E8E3:59619:&#x9B6C
CJK UNIFIED IDEOGRAPH:'E8E4:59620:&#x9CFC
CJK UNIFIED IDEOGRAPH:'E8E5:59621:&#x9CFA
CJK UNIFIED IDEOGRAPH:'E8E6:59622:&#x9CFD
CJK UNIFIED IDEOGRAPH:'E8E7:59623:&#x9CFF
CJK UNIFIED IDEOGRAPH:'E8E8:59624:&#x9CF7
CJK UNIFIED IDEOGRAPH:'E8E9:59625:&#x9D07
CJK UNIFIED IDEOGRAPH:'E8EA:59626:&#x9D00
CJK UNIFIED IDEOGRAPH:'E8EB:59627:&#x9CF9
CJK UNIFIED IDEOGRAPH:'E8EC:59628:&#x9CFB
CJK UNIFIED IDEOGRAPH:'E8ED:59629:&#x9D08
CJK UNIFIED IDEOGRAPH:'E8EE:59630:&#x9D05
CJK UNIFIED IDEOGRAPH:'E8EF:59631:&#x9D04
CJK UNIFIED IDEOGRAPH:'E8F0:59632:&#x9E83
CJK UNIFIED IDEOGRAPH:'E8F1:59633:&#x9ED3
CJK UNIFIED IDEOGRAPH:'E8F2:59634:&#x9F0F
CJK UNIFIED IDEOGRAPH:'E8F3:59635:&#x9F10
CJK UNIFIED IDEOGRAPH:'E8F4:59636:&#x511C
CJK UNIFIED IDEOGRAPH:'E8F5:59637:&#x5113
CJK UNIFIED IDEOGRAPH:'E8F6:59638:&#x5117
CJK UNIFIED IDEOGRAPH:'E8F7:59639:&#x511A
CJK UNIFIED IDEOGRAPH:'E8F8:59640:&#x5111
CJK UNIFIED IDEOGRAPH:'E8F9:59641:&#x51DE
CJK UNIFIED IDEOGRAPH:'E8FA:59642:&#x5334
CJK UNIFIED IDEOGRAPH:'E8FB:59643:&#x53E1
CJK UNIFIED IDEOGRAPH:'E8FC:59644:&#x5670
CJK UNIFIED IDEOGRAPH:'E8FD:59645:&#x5660
CJK UNIFIED IDEOGRAPH:'E8FE:59646:&#x566E
CJK UNIFIED IDEOGRAPH:'E940:59712:&#x5673
CJK UNIFIED IDEOGRAPH:'E941:59713:&#x5666
CJK UNIFIED IDEOGRAPH:'E942:59714:&#x5663
CJK UNIFIED IDEOGRAPH:'E943:59715:&#x566D
CJK UNIFIED IDEOGRAPH:'E944:59716:&#x5672
CJK UNIFIED IDEOGRAPH:'E945:59717:&#x565E
CJK UNIFIED IDEOGRAPH:'E946:59718:&#x5677
CJK UNIFIED IDEOGRAPH:'E947:59719:&#x571C
CJK UNIFIED IDEOGRAPH:'E948:59720:&#x571B
CJK UNIFIED IDEOGRAPH:'E949:59721:&#x58C8
CJK UNIFIED IDEOGRAPH:'E94A:59722:&#x58BD
CJK UNIFIED IDEOGRAPH:'E94B:59723:&#x58C9
CJK UNIFIED IDEOGRAPH:'E94C:59724:&#x58BF
CJK UNIFIED IDEOGRAPH:'E94D:59725:&#x58BA
CJK UNIFIED IDEOGRAPH:'E94E:59726:&#x58C2
CJK UNIFIED IDEOGRAPH:'E94F:59727:&#x58BC
CJK UNIFIED IDEOGRAPH:'E950:59728:&#x58C6
CJK UNIFIED IDEOGRAPH:'E951:59729:&#x5B17
CJK UNIFIED IDEOGRAPH:'E952:59730:&#x5B19
CJK UNIFIED IDEOGRAPH:'E953:59731:&#x5B1B
CJK UNIFIED IDEOGRAPH:'E954:59732:&#x5B21
CJK UNIFIED IDEOGRAPH:'E955:59733:&#x5B14
CJK UNIFIED IDEOGRAPH:'E956:59734:&#x5B13
CJK UNIFIED IDEOGRAPH:'E957:59735:&#x5B10
CJK UNIFIED IDEOGRAPH:'E958:59736:&#x5B16
CJK UNIFIED IDEOGRAPH:'E959:59737:&#x5B28
CJK UNIFIED IDEOGRAPH:'E95A:59738:&#x5B1A
CJK UNIFIED IDEOGRAPH:'E95B:59739:&#x5B20
CJK UNIFIED IDEOGRAPH:'E95C:59740:&#x5B1E
CJK UNIFIED IDEOGRAPH:'E95D:59741:&#x5BEF
CJK UNIFIED IDEOGRAPH:'E95E:59742:&#x5DAC
CJK UNIFIED IDEOGRAPH:'E95F:59743:&#x5DB1
CJK UNIFIED IDEOGRAPH:'E960:59744:&#x5DA9
CJK UNIFIED IDEOGRAPH:'E961:59745:&#x5DA7
CJK UNIFIED IDEOGRAPH:'E962:59746:&#x5DB5
CJK UNIFIED IDEOGRAPH:'E963:59747:&#x5DB0
CJK UNIFIED IDEOGRAPH:'E964:59748:&#x5DAE
CJK UNIFIED IDEOGRAPH:'E965:59749:&#x5DAA
CJK UNIFIED IDEOGRAPH:'E966:59750:&#x5DA8
CJK UNIFIED IDEOGRAPH:'E967:59751:&#x5DB2
CJK UNIFIED IDEOGRAPH:'E968:59752:&#x5DAD
CJK UNIFIED IDEOGRAPH:'E969:59753:&#x5DAF
CJK UNIFIED IDEOGRAPH:'E96A:59754:&#x5DB4
CJK UNIFIED IDEOGRAPH:'E96B:59755:&#x5E67
CJK UNIFIED IDEOGRAPH:'E96C:59756:&#x5E68
CJK UNIFIED IDEOGRAPH:'E96D:59757:&#x5E66
CJK UNIFIED IDEOGRAPH:'E96E:59758:&#x5E6F
CJK UNIFIED IDEOGRAPH:'E96F:59759:&#x5EE9
CJK UNIFIED IDEOGRAPH:'E970:59760:&#x5EE7
CJK UNIFIED IDEOGRAPH:'E971:59761:&#x5EE6
CJK UNIFIED IDEOGRAPH:'E972:59762:&#x5EE8
CJK UNIFIED IDEOGRAPH:'E973:59763:&#x5EE5
CJK UNIFIED IDEOGRAPH:'E974:59764:&#x5F4B
CJK UNIFIED IDEOGRAPH:'E975:59765:&#x5FBC
CJK UNIFIED IDEOGRAPH:'E976:59766:&#x619D
CJK UNIFIED IDEOGRAPH:'E977:59767:&#x61A8
CJK UNIFIED IDEOGRAPH:'E978:59768:&#x6196
CJK UNIFIED IDEOGRAPH:'E979:59769:&#x61C5
CJK UNIFIED IDEOGRAPH:'E97A:59770:&#x61B4
CJK UNIFIED IDEOGRAPH:'E97B:59771:&#x61C6
CJK UNIFIED IDEOGRAPH:'E97C:59772:&#x61C1
CJK UNIFIED IDEOGRAPH:'E97D:59773:&#x61CC
CJK UNIFIED IDEOGRAPH:'E97E:59774:&#x61BA
CJK UNIFIED IDEOGRAPH:'E9A1:59809:&#x61BF
CJK UNIFIED IDEOGRAPH:'E9A2:59810:&#x61B8
CJK UNIFIED IDEOGRAPH:'E9A3:59811:&#x618C
CJK UNIFIED IDEOGRAPH:'E9A4:59812:&#x64D7
CJK UNIFIED IDEOGRAPH:'E9A5:59813:&#x64D6
CJK UNIFIED IDEOGRAPH:'E9A6:59814:&#x64D0
CJK UNIFIED IDEOGRAPH:'E9A7:59815:&#x64CF
CJK UNIFIED IDEOGRAPH:'E9A8:59816:&#x64C9
CJK UNIFIED IDEOGRAPH:'E9A9:59817:&#x64BD
CJK UNIFIED IDEOGRAPH:'E9AA:59818:&#x6489
CJK UNIFIED IDEOGRAPH:'E9AB:59819:&#x64C3
CJK UNIFIED IDEOGRAPH:'E9AC:59820:&#x64DB
CJK UNIFIED IDEOGRAPH:'E9AD:59821:&#x64F3
CJK UNIFIED IDEOGRAPH:'E9AE:59822:&#x64D9
CJK UNIFIED IDEOGRAPH:'E9AF:59823:&#x6533
CJK UNIFIED IDEOGRAPH:'E9B0:59824:&#x657F
CJK UNIFIED IDEOGRAPH:'E9B1:59825:&#x657C
CJK UNIFIED IDEOGRAPH:'E9B2:59826:&#x65A2
CJK UNIFIED IDEOGRAPH:'E9B3:59827:&#x66C8
CJK UNIFIED IDEOGRAPH:'E9B4:59828:&#x66BE
CJK UNIFIED IDEOGRAPH:'E9B5:59829:&#x66C0
CJK UNIFIED IDEOGRAPH:'E9B6:59830:&#x66CA
CJK UNIFIED IDEOGRAPH:'E9B7:59831:&#x66CB
CJK UNIFIED IDEOGRAPH:'E9B8:59832:&#x66CF
CJK UNIFIED IDEOGRAPH:'E9B9:59833:&#x66BD
CJK UNIFIED IDEOGRAPH:'E9BA:59834:&#x66BB
CJK UNIFIED IDEOGRAPH:'E9BB:59835:&#x66BA
CJK UNIFIED IDEOGRAPH:'E9BC:59836:&#x66CC
CJK UNIFIED IDEOGRAPH:'E9BD:59837:&#x6723
CJK UNIFIED IDEOGRAPH:'E9BE:59838:&#x6A34
CJK UNIFIED IDEOGRAPH:'E9BF:59839:&#x6A66
CJK UNIFIED IDEOGRAPH:'E9C0:59840:&#x6A49
CJK UNIFIED IDEOGRAPH:'E9C1:59841:&#x6A67
CJK UNIFIED IDEOGRAPH:'E9C2:59842:&#x6A32
CJK UNIFIED IDEOGRAPH:'E9C3:59843:&#x6A68
CJK UNIFIED IDEOGRAPH:'E9C4:59844:&#x6A3E
CJK UNIFIED IDEOGRAPH:'E9C5:59845:&#x6A5D
CJK UNIFIED IDEOGRAPH:'E9C6:59846:&#x6A6D
CJK UNIFIED IDEOGRAPH:'E9C7:59847:&#x6A76
CJK UNIFIED IDEOGRAPH:'E9C8:59848:&#x6A5B
CJK UNIFIED IDEOGRAPH:'E9C9:59849:&#x6A51
CJK UNIFIED IDEOGRAPH:'E9CA:59850:&#x6A28
CJK UNIFIED IDEOGRAPH:'E9CB:59851:&#x6A5A
CJK UNIFIED IDEOGRAPH:'E9CC:59852:&#x6A3B
CJK UNIFIED IDEOGRAPH:'E9CD:59853:&#x6A3F
CJK UNIFIED IDEOGRAPH:'E9CE:59854:&#x6A41
CJK UNIFIED IDEOGRAPH:'E9CF:59855:&#x6A6A
CJK UNIFIED IDEOGRAPH:'E9D0:59856:&#x6A64
CJK UNIFIED IDEOGRAPH:'E9D1:59857:&#x6A50
CJK UNIFIED IDEOGRAPH:'E9D2:59858:&#x6A4F
CJK UNIFIED IDEOGRAPH:'E9D3:59859:&#x6A54
CJK UNIFIED IDEOGRAPH:'E9D4:59860:&#x6A6F
CJK UNIFIED IDEOGRAPH:'E9D5:59861:&#x6A69
CJK UNIFIED IDEOGRAPH:'E9D6:59862:&#x6A60
CJK UNIFIED IDEOGRAPH:'E9D7:59863:&#x6A3C
CJK UNIFIED IDEOGRAPH:'E9D8:59864:&#x6A5E
CJK UNIFIED IDEOGRAPH:'E9D9:59865:&#x6A56
CJK UNIFIED IDEOGRAPH:'E9DA:59866:&#x6A55
CJK UNIFIED IDEOGRAPH:'E9DB:59867:&#x6A4D
CJK UNIFIED IDEOGRAPH:'E9DC:59868:&#x6A4E
CJK UNIFIED IDEOGRAPH:'E9DD:59869:&#x6A46
CJK UNIFIED IDEOGRAPH:'E9DE:59870:&#x6B55
CJK UNIFIED IDEOGRAPH:'E9DF:59871:&#x6B54
CJK UNIFIED IDEOGRAPH:'E9E0:59872:&#x6B56
CJK UNIFIED IDEOGRAPH:'E9E1:59873:&#x6BA7
CJK UNIFIED IDEOGRAPH:'E9E2:59874:&#x6BAA
CJK UNIFIED IDEOGRAPH:'E9E3:59875:&#x6BAB
CJK UNIFIED IDEOGRAPH:'E9E4:59876:&#x6BC8
CJK UNIFIED IDEOGRAPH:'E9E5:59877:&#x6BC7
CJK UNIFIED IDEOGRAPH:'E9E6:59878:&#x6C04
CJK UNIFIED IDEOGRAPH:'E9E7:59879:&#x6C03
CJK UNIFIED IDEOGRAPH:'E9E8:59880:&#x6C06
CJK UNIFIED IDEOGRAPH:'E9E9:59881:&#x6FAD
CJK UNIFIED IDEOGRAPH:'E9EA:59882:&#x6FCB
CJK UNIFIED IDEOGRAPH:'E9EB:59883:&#x6FA3
CJK UNIFIED IDEOGRAPH:'E9EC:59884:&#x6FC7
CJK UNIFIED IDEOGRAPH:'E9ED:59885:&#x6FBC
CJK UNIFIED IDEOGRAPH:'E9EE:59886:&#x6FCE
CJK UNIFIED IDEOGRAPH:'E9EF:59887:&#x6FC8
CJK UNIFIED IDEOGRAPH:'E9F0:59888:&#x6F5E
CJK UNIFIED IDEOGRAPH:'E9F1:59889:&#x6FC4
CJK UNIFIED IDEOGRAPH:'E9F2:59890:&#x6FBD
CJK UNIFIED IDEOGRAPH:'E9F3:59891:&#x6F9E
CJK UNIFIED IDEOGRAPH:'E9F4:59892:&#x6FCA
CJK UNIFIED IDEOGRAPH:'E9F5:59893:&#x6FA8
CJK UNIFIED IDEOGRAPH:'E9F6:59894:&#x7004
CJK UNIFIED IDEOGRAPH:'E9F7:59895:&#x6FA5
CJK UNIFIED IDEOGRAPH:'E9F8:59896:&#x6FAE
CJK UNIFIED IDEOGRAPH:'E9F9:59897:&#x6FBA
CJK UNIFIED IDEOGRAPH:'E9FA:59898:&#x6FAC
CJK UNIFIED IDEOGRAPH:'E9FB:59899:&#x6FAA
CJK UNIFIED IDEOGRAPH:'E9FC:59900:&#x6FCF
CJK UNIFIED IDEOGRAPH:'E9FD:59901:&#x6FBF
CJK UNIFIED IDEOGRAPH:'E9FE:59902:&#x6FB8
CJK UNIFIED IDEOGRAPH:'EA40:59968:&#x6FA2
CJK UNIFIED IDEOGRAPH:'EA41:59969:&#x6FC9
CJK UNIFIED IDEOGRAPH:'EA42:59970:&#x6FAB
CJK UNIFIED IDEOGRAPH:'EA43:59971:&#x6FCD
CJK UNIFIED IDEOGRAPH:'EA44:59972:&#x6FAF
CJK UNIFIED IDEOGRAPH:'EA45:59973:&#x6FB2
CJK UNIFIED IDEOGRAPH:'EA46:59974:&#x6FB0
CJK UNIFIED IDEOGRAPH:'EA47:59975:&#x71C5
CJK UNIFIED IDEOGRAPH:'EA48:59976:&#x71C2
CJK UNIFIED IDEOGRAPH:'EA49:59977:&#x71BF
CJK UNIFIED IDEOGRAPH:'EA4A:59978:&#x71B8
CJK UNIFIED IDEOGRAPH:'EA4B:59979:&#x71D6
CJK UNIFIED IDEOGRAPH:'EA4C:59980:&#x71C0
CJK UNIFIED IDEOGRAPH:'EA4D:59981:&#x71C1
CJK UNIFIED IDEOGRAPH:'EA4E:59982:&#x71CB
CJK UNIFIED IDEOGRAPH:'EA4F:59983:&#x71D4
CJK UNIFIED IDEOGRAPH:'EA50:59984:&#x71CA
CJK UNIFIED IDEOGRAPH:'EA51:59985:&#x71C7
CJK UNIFIED IDEOGRAPH:'EA52:59986:&#x71CF
CJK UNIFIED IDEOGRAPH:'EA53:59987:&#x71BD
CJK UNIFIED IDEOGRAPH:'EA54:59988:&#x71D8
CJK UNIFIED IDEOGRAPH:'EA55:59989:&#x71BC
CJK UNIFIED IDEOGRAPH:'EA56:59990:&#x71C6
CJK UNIFIED IDEOGRAPH:'EA57:59991:&#x71DA
CJK UNIFIED IDEOGRAPH:'EA58:59992:&#x71DB
CJK UNIFIED IDEOGRAPH:'EA59:59993:&#x729D
CJK UNIFIED IDEOGRAPH:'EA5A:59994:&#x729E
CJK UNIFIED IDEOGRAPH:'EA5B:59995:&#x7369
CJK UNIFIED IDEOGRAPH:'EA5C:59996:&#x7366
CJK UNIFIED IDEOGRAPH:'EA5D:59997:&#x7367
CJK UNIFIED IDEOGRAPH:'EA5E:59998:&#x736C
CJK UNIFIED IDEOGRAPH:'EA5F:59999:&#x7365
CJK UNIFIED IDEOGRAPH:'EA60:60000:&#x736B
CJK UNIFIED IDEOGRAPH:'EA61:60001:&#x736A
CJK UNIFIED IDEOGRAPH:'EA62:60002:&#x747F
CJK UNIFIED IDEOGRAPH:'EA63:60003:&#x749A
CJK UNIFIED IDEOGRAPH:'EA64:60004:&#x74A0
CJK UNIFIED IDEOGRAPH:'EA65:60005:&#x7494
CJK UNIFIED IDEOGRAPH:'EA66:60006:&#x7492
CJK UNIFIED IDEOGRAPH:'EA67:60007:&#x7495
CJK UNIFIED IDEOGRAPH:'EA68:60008:&#x74A1
CJK UNIFIED IDEOGRAPH:'EA69:60009:&#x750B
CJK UNIFIED IDEOGRAPH:'EA6A:60010:&#x7580
CJK UNIFIED IDEOGRAPH:'EA6B:60011:&#x762F
CJK UNIFIED IDEOGRAPH:'EA6C:60012:&#x762D
CJK UNIFIED IDEOGRAPH:'EA6D:60013:&#x7631
CJK UNIFIED IDEOGRAPH:'EA6E:60014:&#x763D
CJK UNIFIED IDEOGRAPH:'EA6F:60015:&#x7633
CJK UNIFIED IDEOGRAPH:'EA70:60016:&#x763C
CJK UNIFIED IDEOGRAPH:'EA71:60017:&#x7635
CJK UNIFIED IDEOGRAPH:'EA72:60018:&#x7632
CJK UNIFIED IDEOGRAPH:'EA73:60019:&#x7630
CJK UNIFIED IDEOGRAPH:'EA74:60020:&#x76BB
CJK UNIFIED IDEOGRAPH:'EA75:60021:&#x76E6
CJK UNIFIED IDEOGRAPH:'EA76:60022:&#x779A
CJK UNIFIED IDEOGRAPH:'EA77:60023:&#x779D
CJK UNIFIED IDEOGRAPH:'EA78:60024:&#x77A1
CJK UNIFIED IDEOGRAPH:'EA79:60025:&#x779C
CJK UNIFIED IDEOGRAPH:'EA7A:60026:&#x779B
CJK UNIFIED IDEOGRAPH:'EA7B:60027:&#x77A2
CJK UNIFIED IDEOGRAPH:'EA7C:60028:&#x77A3
CJK UNIFIED IDEOGRAPH:'EA7D:60029:&#x7795
CJK UNIFIED IDEOGRAPH:'EA7E:60030:&#x7799
CJK UNIFIED IDEOGRAPH:'EAA1:60065:&#x7797
CJK UNIFIED IDEOGRAPH:'EAA2:60066:&#x78DD
CJK UNIFIED IDEOGRAPH:'EAA3:60067:&#x78E9
CJK UNIFIED IDEOGRAPH:'EAA4:60068:&#x78E5
CJK UNIFIED IDEOGRAPH:'EAA5:60069:&#x78EA
CJK UNIFIED IDEOGRAPH:'EAA6:60070:&#x78DE
CJK UNIFIED IDEOGRAPH:'EAA7:60071:&#x78E3
CJK UNIFIED IDEOGRAPH:'EAA8:60072:&#x78DB
CJK UNIFIED IDEOGRAPH:'EAA9:60073:&#x78E1
CJK UNIFIED IDEOGRAPH:'EAAA:60074:&#x78E2
CJK UNIFIED IDEOGRAPH:'EAAB:60075:&#x78ED
CJK UNIFIED IDEOGRAPH:'EAAC:60076:&#x78DF
CJK UNIFIED IDEOGRAPH:'EAAD:60077:&#x78E0
CJK UNIFIED IDEOGRAPH:'EAAE:60078:&#x79A4
CJK UNIFIED IDEOGRAPH:'EAAF:60079:&#x7A44
CJK UNIFIED IDEOGRAPH:'EAB0:60080:&#x7A48
CJK UNIFIED IDEOGRAPH:'EAB1:60081:&#x7A47
CJK UNIFIED IDEOGRAPH:'EAB2:60082:&#x7AB6
CJK UNIFIED IDEOGRAPH:'EAB3:60083:&#x7AB8
CJK UNIFIED IDEOGRAPH:'EAB4:60084:&#x7AB5
CJK UNIFIED IDEOGRAPH:'EAB5:60085:&#x7AB1
CJK UNIFIED IDEOGRAPH:'EAB6:60086:&#x7AB7
CJK UNIFIED IDEOGRAPH:'EAB7:60087:&#x7BDE
CJK UNIFIED IDEOGRAPH:'EAB8:60088:&#x7BE3
CJK UNIFIED IDEOGRAPH:'EAB9:60089:&#x7BE7
CJK UNIFIED IDEOGRAPH:'EABA:60090:&#x7BDD
CJK UNIFIED IDEOGRAPH:'EABB:60091:&#x7BD5
CJK UNIFIED IDEOGRAPH:'EABC:60092:&#x7BE5
CJK UNIFIED IDEOGRAPH:'EABD:60093:&#x7BDA
CJK UNIFIED IDEOGRAPH:'EABE:60094:&#x7BE8
CJK UNIFIED IDEOGRAPH:'EABF:60095:&#x7BF9
CJK UNIFIED IDEOGRAPH:'EAC0:60096:&#x7BD4
CJK UNIFIED IDEOGRAPH:'EAC1:60097:&#x7BEA
CJK UNIFIED IDEOGRAPH:'EAC2:60098:&#x7BE2
CJK UNIFIED IDEOGRAPH:'EAC3:60099:&#x7BDC
CJK UNIFIED IDEOGRAPH:'EAC4:60100:&#x7BEB
CJK UNIFIED IDEOGRAPH:'EAC5:60101:&#x7BD8
CJK UNIFIED IDEOGRAPH:'EAC6:60102:&#x7BDF
CJK UNIFIED IDEOGRAPH:'EAC7:60103:&#x7CD2
CJK UNIFIED IDEOGRAPH:'EAC8:60104:&#x7CD4
CJK UNIFIED IDEOGRAPH:'EAC9:60105:&#x7CD7
CJK UNIFIED IDEOGRAPH:'EACA:60106:&#x7CD0
CJK UNIFIED IDEOGRAPH:'EACB:60107:&#x7CD1
CJK UNIFIED IDEOGRAPH:'EACC:60108:&#x7E12
CJK UNIFIED IDEOGRAPH:'EACD:60109:&#x7E21
CJK UNIFIED IDEOGRAPH:'EACE:60110:&#x7E17
CJK UNIFIED IDEOGRAPH:'EACF:60111:&#x7E0C
CJK UNIFIED IDEOGRAPH:'EAD0:60112:&#x7E1F
CJK UNIFIED IDEOGRAPH:'EAD1:60113:&#x7E20
CJK UNIFIED IDEOGRAPH:'EAD2:60114:&#x7E13
CJK UNIFIED IDEOGRAPH:'EAD3:60115:&#x7E0E
CJK UNIFIED IDEOGRAPH:'EAD4:60116:&#x7E1C
CJK UNIFIED IDEOGRAPH:'EAD5:60117:&#x7E15
CJK UNIFIED IDEOGRAPH:'EAD6:60118:&#x7E1A
CJK UNIFIED IDEOGRAPH:'EAD7:60119:&#x7E22
CJK UNIFIED IDEOGRAPH:'EAD8:60120:&#x7E0B
CJK UNIFIED IDEOGRAPH:'EAD9:60121:&#x7E0F
CJK UNIFIED IDEOGRAPH:'EADA:60122:&#x7E16
CJK UNIFIED IDEOGRAPH:'EADB:60123:&#x7E0D
CJK UNIFIED IDEOGRAPH:'EADC:60124:&#x7E14
CJK UNIFIED IDEOGRAPH:'EADD:60125:&#x7E25
CJK UNIFIED IDEOGRAPH:'EADE:60126:&#x7E24
CJK UNIFIED IDEOGRAPH:'EADF:60127:&#x7F43
CJK UNIFIED IDEOGRAPH:'EAE0:60128:&#x7F7B
CJK UNIFIED IDEOGRAPH:'EAE1:60129:&#x7F7C
CJK UNIFIED IDEOGRAPH:'EAE2:60130:&#x7F7A
CJK UNIFIED IDEOGRAPH:'EAE3:60131:&#x7FB1
CJK UNIFIED IDEOGRAPH:'EAE4:60132:&#x7FEF
CJK UNIFIED IDEOGRAPH:'EAE5:60133:&#x802A
CJK UNIFIED IDEOGRAPH:'EAE6:60134:&#x8029
CJK UNIFIED IDEOGRAPH:'EAE7:60135:&#x806C
CJK UNIFIED IDEOGRAPH:'EAE8:60136:&#x81B1
CJK UNIFIED IDEOGRAPH:'EAE9:60137:&#x81A6
CJK UNIFIED IDEOGRAPH:'EAEA:60138:&#x81AE
CJK UNIFIED IDEOGRAPH:'EAEB:60139:&#x81B9
CJK UNIFIED IDEOGRAPH:'EAEC:60140:&#x81B5
CJK UNIFIED IDEOGRAPH:'EAED:60141:&#x81AB
CJK UNIFIED IDEOGRAPH:'EAEE:60142:&#x81B0
CJK UNIFIED IDEOGRAPH:'EAEF:60143:&#x81AC
CJK UNIFIED IDEOGRAPH:'EAF0:60144:&#x81B4
CJK UNIFIED IDEOGRAPH:'EAF1:60145:&#x81B2
CJK UNIFIED IDEOGRAPH:'EAF2:60146:&#x81B7
CJK UNIFIED IDEOGRAPH:'EAF3:60147:&#x81A7
CJK UNIFIED IDEOGRAPH:'EAF4:60148:&#x81F2
CJK UNIFIED IDEOGRAPH:'EAF5:60149:&#x8255
CJK UNIFIED IDEOGRAPH:'EAF6:60150:&#x8256
CJK UNIFIED IDEOGRAPH:'EAF7:60151:&#x8257
CJK UNIFIED IDEOGRAPH:'EAF8:60152:&#x8556
CJK UNIFIED IDEOGRAPH:'EAF9:60153:&#x8545
CJK UNIFIED IDEOGRAPH:'EAFA:60154:&#x856B
CJK UNIFIED IDEOGRAPH:'EAFB:60155:&#x854D
CJK UNIFIED IDEOGRAPH:'EAFC:60156:&#x8553
CJK UNIFIED IDEOGRAPH:'EAFD:60157:&#x8561
CJK UNIFIED IDEOGRAPH:'EAFE:60158:&#x8558
CJK UNIFIED IDEOGRAPH:'EB40:60224:&#x8540
CJK UNIFIED IDEOGRAPH:'EB41:60225:&#x8546
CJK UNIFIED IDEOGRAPH:'EB42:60226:&#x8564
CJK UNIFIED IDEOGRAPH:'EB43:60227:&#x8541
CJK UNIFIED IDEOGRAPH:'EB44:60228:&#x8562
CJK UNIFIED IDEOGRAPH:'EB45:60229:&#x8544
CJK UNIFIED IDEOGRAPH:'EB46:60230:&#x8551
CJK UNIFIED IDEOGRAPH:'EB47:60231:&#x8547
CJK UNIFIED IDEOGRAPH:'EB48:60232:&#x8563
CJK UNIFIED IDEOGRAPH:'EB49:60233:&#x853E
CJK UNIFIED IDEOGRAPH:'EB4A:60234:&#x855B
CJK UNIFIED IDEOGRAPH:'EB4B:60235:&#x8571
CJK UNIFIED IDEOGRAPH:'EB4C:60236:&#x854E
CJK UNIFIED IDEOGRAPH:'EB4D:60237:&#x856E
CJK UNIFIED IDEOGRAPH:'EB4E:60238:&#x8575
CJK UNIFIED IDEOGRAPH:'EB4F:60239:&#x8555
CJK UNIFIED IDEOGRAPH:'EB50:60240:&#x8567
CJK UNIFIED IDEOGRAPH:'EB51:60241:&#x8560
CJK UNIFIED IDEOGRAPH:'EB52:60242:&#x858C
CJK UNIFIED IDEOGRAPH:'EB53:60243:&#x8566
CJK UNIFIED IDEOGRAPH:'EB54:60244:&#x855D
CJK UNIFIED IDEOGRAPH:'EB55:60245:&#x8554
CJK UNIFIED IDEOGRAPH:'EB56:60246:&#x8565
CJK UNIFIED IDEOGRAPH:'EB57:60247:&#x856C
CJK UNIFIED IDEOGRAPH:'EB58:60248:&#x8663
CJK UNIFIED IDEOGRAPH:'EB59:60249:&#x8665
CJK UNIFIED IDEOGRAPH:'EB5A:60250:&#x8664
CJK UNIFIED IDEOGRAPH:'EB5B:60251:&#x879B
CJK UNIFIED IDEOGRAPH:'EB5C:60252:&#x878F
CJK UNIFIED IDEOGRAPH:'EB5D:60253:&#x8797
CJK UNIFIED IDEOGRAPH:'EB5E:60254:&#x8793
CJK UNIFIED IDEOGRAPH:'EB5F:60255:&#x8792
CJK UNIFIED IDEOGRAPH:'EB60:60256:&#x8788
CJK UNIFIED IDEOGRAPH:'EB61:60257:&#x8781
CJK UNIFIED IDEOGRAPH:'EB62:60258:&#x8796
CJK UNIFIED IDEOGRAPH:'EB63:60259:&#x8798
CJK UNIFIED IDEOGRAPH:'EB64:60260:&#x8779
CJK UNIFIED IDEOGRAPH:'EB65:60261:&#x8787
CJK UNIFIED IDEOGRAPH:'EB66:60262:&#x87A3
CJK UNIFIED IDEOGRAPH:'EB67:60263:&#x8785
CJK UNIFIED IDEOGRAPH:'EB68:60264:&#x8790
CJK UNIFIED IDEOGRAPH:'EB69:60265:&#x8791
CJK UNIFIED IDEOGRAPH:'EB6A:60266:&#x879D
CJK UNIFIED IDEOGRAPH:'EB6B:60267:&#x8784
CJK UNIFIED IDEOGRAPH:'EB6C:60268:&#x8794
CJK UNIFIED IDEOGRAPH:'EB6D:60269:&#x879C
CJK UNIFIED IDEOGRAPH:'EB6E:60270:&#x879A
CJK UNIFIED IDEOGRAPH:'EB6F:60271:&#x8789
CJK UNIFIED IDEOGRAPH:'EB70:60272:&#x891E
CJK UNIFIED IDEOGRAPH:'EB71:60273:&#x8926
CJK UNIFIED IDEOGRAPH:'EB72:60274:&#x8930
CJK UNIFIED IDEOGRAPH:'EB73:60275:&#x892D
CJK UNIFIED IDEOGRAPH:'EB74:60276:&#x892E
CJK UNIFIED IDEOGRAPH:'EB75:60277:&#x8927
CJK UNIFIED IDEOGRAPH:'EB76:60278:&#x8931
CJK UNIFIED IDEOGRAPH:'EB77:60279:&#x8922
CJK UNIFIED IDEOGRAPH:'EB78:60280:&#x8929
CJK UNIFIED IDEOGRAPH:'EB79:60281:&#x8923
CJK UNIFIED IDEOGRAPH:'EB7A:60282:&#x892F
CJK UNIFIED IDEOGRAPH:'EB7B:60283:&#x892C
CJK UNIFIED IDEOGRAPH:'EB7C:60284:&#x891F
CJK UNIFIED IDEOGRAPH:'EB7D:60285:&#x89F1
CJK UNIFIED IDEOGRAPH:'EB7E:60286:&#x8AE0
CJK UNIFIED IDEOGRAPH:'EBA1:60321:&#x8AE2
CJK UNIFIED IDEOGRAPH:'EBA2:60322:&#x8AF2
CJK UNIFIED IDEOGRAPH:'EBA3:60323:&#x8AF4
CJK UNIFIED IDEOGRAPH:'EBA4:60324:&#x8AF5
CJK UNIFIED IDEOGRAPH:'EBA5:60325:&#x8ADD
CJK UNIFIED IDEOGRAPH:'EBA6:60326:&#x8B14
CJK UNIFIED IDEOGRAPH:'EBA7:60327:&#x8AE4
CJK UNIFIED IDEOGRAPH:'EBA8:60328:&#x8ADF
CJK UNIFIED IDEOGRAPH:'EBA9:60329:&#x8AF0
CJK UNIFIED IDEOGRAPH:'EBAA:60330:&#x8AC8
CJK UNIFIED IDEOGRAPH:'EBAB:60331:&#x8ADE
CJK UNIFIED IDEOGRAPH:'EBAC:60332:&#x8AE1
CJK UNIFIED IDEOGRAPH:'EBAD:60333:&#x8AE8
CJK UNIFIED IDEOGRAPH:'EBAE:60334:&#x8AFF
CJK UNIFIED IDEOGRAPH:'EBAF:60335:&#x8AEF
CJK UNIFIED IDEOGRAPH:'EBB0:60336:&#x8AFB
CJK UNIFIED IDEOGRAPH:'EBB1:60337:&#x8C91
CJK UNIFIED IDEOGRAPH:'EBB2:60338:&#x8C92
CJK UNIFIED IDEOGRAPH:'EBB3:60339:&#x8C90
CJK UNIFIED IDEOGRAPH:'EBB4:60340:&#x8CF5
CJK UNIFIED IDEOGRAPH:'EBB5:60341:&#x8CEE
CJK UNIFIED IDEOGRAPH:'EBB6:60342:&#x8CF1
CJK UNIFIED IDEOGRAPH:'EBB7:60343:&#x8CF0
CJK UNIFIED IDEOGRAPH:'EBB8:60344:&#x8CF3
CJK UNIFIED IDEOGRAPH:'EBB9:60345:&#x8D6C
CJK UNIFIED IDEOGRAPH:'EBBA:60346:&#x8D6E
CJK UNIFIED IDEOGRAPH:'EBBB:60347:&#x8DA5
CJK UNIFIED IDEOGRAPH:'EBBC:60348:&#x8DA7
CJK UNIFIED IDEOGRAPH:'EBBD:60349:&#x8E33
CJK UNIFIED IDEOGRAPH:'EBBE:60350:&#x8E3E
CJK UNIFIED IDEOGRAPH:'EBBF:60351:&#x8E38
CJK UNIFIED IDEOGRAPH:'EBC0:60352:&#x8E40
CJK UNIFIED IDEOGRAPH:'EBC1:60353:&#x8E45
CJK UNIFIED IDEOGRAPH:'EBC2:60354:&#x8E36
CJK UNIFIED IDEOGRAPH:'EBC3:60355:&#x8E3C
CJK UNIFIED IDEOGRAPH:'EBC4:60356:&#x8E3D
CJK UNIFIED IDEOGRAPH:'EBC5:60357:&#x8E41
CJK UNIFIED IDEOGRAPH:'EBC6:60358:&#x8E30
CJK UNIFIED IDEOGRAPH:'EBC7:60359:&#x8E3F
CJK UNIFIED IDEOGRAPH:'EBC8:60360:&#x8EBD
CJK UNIFIED IDEOGRAPH:'EBC9:60361:&#x8F36
CJK UNIFIED IDEOGRAPH:'EBCA:60362:&#x8F2E
CJK UNIFIED IDEOGRAPH:'EBCB:60363:&#x8F35
CJK UNIFIED IDEOGRAPH:'EBCC:60364:&#x8F32
CJK UNIFIED IDEOGRAPH:'EBCD:60365:&#x8F39
CJK UNIFIED IDEOGRAPH:'EBCE:60366:&#x8F37
CJK UNIFIED IDEOGRAPH:'EBCF:60367:&#x8F34
CJK UNIFIED IDEOGRAPH:'EBD0:60368:&#x9076
CJK UNIFIED IDEOGRAPH:'EBD1:60369:&#x9079
CJK UNIFIED IDEOGRAPH:'EBD2:60370:&#x907B
CJK UNIFIED IDEOGRAPH:'EBD3:60371:&#x9086
CJK UNIFIED IDEOGRAPH:'EBD4:60372:&#x90FA
CJK UNIFIED IDEOGRAPH:'EBD5:60373:&#x9133
CJK UNIFIED IDEOGRAPH:'EBD6:60374:&#x9135
CJK UNIFIED IDEOGRAPH:'EBD7:60375:&#x9136
CJK UNIFIED IDEOGRAPH:'EBD8:60376:&#x9193
CJK UNIFIED IDEOGRAPH:'EBD9:60377:&#x9190
CJK UNIFIED IDEOGRAPH:'EBDA:60378:&#x9191
CJK UNIFIED IDEOGRAPH:'EBDB:60379:&#x918D
CJK UNIFIED IDEOGRAPH:'EBDC:60380:&#x918F
CJK UNIFIED IDEOGRAPH:'EBDD:60381:&#x9327
CJK UNIFIED IDEOGRAPH:'EBDE:60382:&#x931E
CJK UNIFIED IDEOGRAPH:'EBDF:60383:&#x9308
CJK UNIFIED IDEOGRAPH:'EBE0:60384:&#x931F
CJK UNIFIED IDEOGRAPH:'EBE1:60385:&#x9306
CJK UNIFIED IDEOGRAPH:'EBE2:60386:&#x930F
CJK UNIFIED IDEOGRAPH:'EBE3:60387:&#x937A
CJK UNIFIED IDEOGRAPH:'EBE4:60388:&#x9338
CJK UNIFIED IDEOGRAPH:'EBE5:60389:&#x933C
CJK UNIFIED IDEOGRAPH:'EBE6:60390:&#x931B
CJK UNIFIED IDEOGRAPH:'EBE7:60391:&#x9323
CJK UNIFIED IDEOGRAPH:'EBE8:60392:&#x9312
CJK UNIFIED IDEOGRAPH:'EBE9:60393:&#x9301
CJK UNIFIED IDEOGRAPH:'EBEA:60394:&#x9346
CJK UNIFIED IDEOGRAPH:'EBEB:60395:&#x932D
CJK UNIFIED IDEOGRAPH:'EBEC:60396:&#x930E
CJK UNIFIED IDEOGRAPH:'EBED:60397:&#x930D
CJK UNIFIED IDEOGRAPH:'EBEE:60398:&#x92CB
CJK UNIFIED IDEOGRAPH:'EBEF:60399:&#x931D
CJK UNIFIED IDEOGRAPH:'EBF0:60400:&#x92FA
CJK UNIFIED IDEOGRAPH:'EBF1:60401:&#x9325
CJK UNIFIED IDEOGRAPH:'EBF2:60402:&#x9313
CJK UNIFIED IDEOGRAPH:'EBF3:60403:&#x92F9
CJK UNIFIED IDEOGRAPH:'EBF4:60404:&#x92F7
CJK UNIFIED IDEOGRAPH:'EBF5:60405:&#x9334
CJK UNIFIED IDEOGRAPH:'EBF6:60406:&#x9302
CJK UNIFIED IDEOGRAPH:'EBF7:60407:&#x9324
CJK UNIFIED IDEOGRAPH:'EBF8:60408:&#x92FF
CJK UNIFIED IDEOGRAPH:'EBF9:60409:&#x9329
CJK UNIFIED IDEOGRAPH:'EBFA:60410:&#x9339
CJK UNIFIED IDEOGRAPH:'EBFB:60411:&#x9335
CJK UNIFIED IDEOGRAPH:'EBFC:60412:&#x932A
CJK UNIFIED IDEOGRAPH:'EBFD:60413:&#x9314
CJK UNIFIED IDEOGRAPH:'EBFE:60414:&#x930C
CJK UNIFIED IDEOGRAPH:'EC40:60480:&#x930B
CJK UNIFIED IDEOGRAPH:'EC41:60481:&#x92FE
CJK UNIFIED IDEOGRAPH:'EC42:60482:&#x9309
CJK UNIFIED IDEOGRAPH:'EC43:60483:&#x9300
CJK UNIFIED IDEOGRAPH:'EC44:60484:&#x92FB
CJK UNIFIED IDEOGRAPH:'EC45:60485:&#x9316
CJK UNIFIED IDEOGRAPH:'EC46:60486:&#x95BC
CJK UNIFIED IDEOGRAPH:'EC47:60487:&#x95CD
CJK UNIFIED IDEOGRAPH:'EC48:60488:&#x95BE
CJK UNIFIED IDEOGRAPH:'EC49:60489:&#x95B9
CJK UNIFIED IDEOGRAPH:'EC4A:60490:&#x95BA
CJK UNIFIED IDEOGRAPH:'EC4B:60491:&#x95B6
CJK UNIFIED IDEOGRAPH:'EC4C:60492:&#x95BF
CJK UNIFIED IDEOGRAPH:'EC4D:60493:&#x95B5
CJK UNIFIED IDEOGRAPH:'EC4E:60494:&#x95BD
CJK UNIFIED IDEOGRAPH:'EC4F:60495:&#x96A9
CJK UNIFIED IDEOGRAPH:'EC50:60496:&#x96D4
CJK UNIFIED IDEOGRAPH:'EC51:60497:&#x970B
CJK UNIFIED IDEOGRAPH:'EC52:60498:&#x9712
CJK UNIFIED IDEOGRAPH:'EC53:60499:&#x9710
CJK UNIFIED IDEOGRAPH:'EC54:60500:&#x9799
CJK UNIFIED IDEOGRAPH:'EC55:60501:&#x9797
CJK UNIFIED IDEOGRAPH:'EC56:60502:&#x9794
CJK UNIFIED IDEOGRAPH:'EC57:60503:&#x97F0
CJK UNIFIED IDEOGRAPH:'EC58:60504:&#x97F8
CJK UNIFIED IDEOGRAPH:'EC59:60505:&#x9835
CJK UNIFIED IDEOGRAPH:'EC5A:60506:&#x982F
CJK UNIFIED IDEOGRAPH:'EC5B:60507:&#x9832
CJK UNIFIED IDEOGRAPH:'EC5C:60508:&#x9924
CJK UNIFIED IDEOGRAPH:'EC5D:60509:&#x991F
CJK UNIFIED IDEOGRAPH:'EC5E:60510:&#x9927
CJK UNIFIED IDEOGRAPH:'EC5F:60511:&#x9929
CJK UNIFIED IDEOGRAPH:'EC60:60512:&#x999E
CJK UNIFIED IDEOGRAPH:'EC61:60513:&#x99EE
CJK UNIFIED IDEOGRAPH:'EC62:60514:&#x99EC
CJK UNIFIED IDEOGRAPH:'EC63:60515:&#x99E5
CJK UNIFIED IDEOGRAPH:'EC64:60516:&#x99E4
CJK UNIFIED IDEOGRAPH:'EC65:60517:&#x99F0
CJK UNIFIED IDEOGRAPH:'EC66:60518:&#x99E3
CJK UNIFIED IDEOGRAPH:'EC67:60519:&#x99EA
CJK UNIFIED IDEOGRAPH:'EC68:60520:&#x99E9
CJK UNIFIED IDEOGRAPH:'EC69:60521:&#x99E7
CJK UNIFIED IDEOGRAPH:'EC6A:60522:&#x9AB9
CJK UNIFIED IDEOGRAPH:'EC6B:60523:&#x9ABF
CJK UNIFIED IDEOGRAPH:'EC6C:60524:&#x9AB4
CJK UNIFIED IDEOGRAPH:'EC6D:60525:&#x9ABB
CJK UNIFIED IDEOGRAPH:'EC6E:60526:&#x9AF6
CJK UNIFIED IDEOGRAPH:'EC6F:60527:&#x9AFA
CJK UNIFIED IDEOGRAPH:'EC70:60528:&#x9AF9
CJK UNIFIED IDEOGRAPH:'EC71:60529:&#x9AF7
CJK UNIFIED IDEOGRAPH:'EC72:60530:&#x9B33
CJK UNIFIED IDEOGRAPH:'EC73:60531:&#x9B80
CJK UNIFIED IDEOGRAPH:'EC74:60532:&#x9B85
CJK UNIFIED IDEOGRAPH:'EC75:60533:&#x9B87
CJK UNIFIED IDEOGRAPH:'EC76:60534:&#x9B7C
CJK UNIFIED IDEOGRAPH:'EC77:60535:&#x9B7E
CJK UNIFIED IDEOGRAPH:'EC78:60536:&#x9B7B
CJK UNIFIED IDEOGRAPH:'EC79:60537:&#x9B82
CJK UNIFIED IDEOGRAPH:'EC7A:60538:&#x9B93
CJK UNIFIED IDEOGRAPH:'EC7B:60539:&#x9B92
CJK UNIFIED IDEOGRAPH:'EC7C:60540:&#x9B90
CJK UNIFIED IDEOGRAPH:'EC7D:60541:&#x9B7A
CJK UNIFIED IDEOGRAPH:'EC7E:60542:&#x9B95
CJK UNIFIED IDEOGRAPH:'ECA1:60577:&#x9B7D
CJK UNIFIED IDEOGRAPH:'ECA2:60578:&#x9B88
CJK UNIFIED IDEOGRAPH:'ECA3:60579:&#x9D25
CJK UNIFIED IDEOGRAPH:'ECA4:60580:&#x9D17
CJK UNIFIED IDEOGRAPH:'ECA5:60581:&#x9D20
CJK UNIFIED IDEOGRAPH:'ECA6:60582:&#x9D1E
CJK UNIFIED IDEOGRAPH:'ECA7:60583:&#x9D14
CJK UNIFIED IDEOGRAPH:'ECA8:60584:&#x9D29
CJK UNIFIED IDEOGRAPH:'ECA9:60585:&#x9D1D
CJK UNIFIED IDEOGRAPH:'ECAA:60586:&#x9D18
CJK UNIFIED IDEOGRAPH:'ECAB:60587:&#x9D22
CJK UNIFIED IDEOGRAPH:'ECAC:60588:&#x9D10
CJK UNIFIED IDEOGRAPH:'ECAD:60589:&#x9D19
CJK UNIFIED IDEOGRAPH:'ECAE:60590:&#x9D1F
CJK UNIFIED IDEOGRAPH:'ECAF:60591:&#x9E88
CJK UNIFIED IDEOGRAPH:'ECB0:60592:&#x9E86
CJK UNIFIED IDEOGRAPH:'ECB1:60593:&#x9E87
CJK UNIFIED IDEOGRAPH:'ECB2:60594:&#x9EAE
CJK UNIFIED IDEOGRAPH:'ECB3:60595:&#x9EAD
CJK UNIFIED IDEOGRAPH:'ECB4:60596:&#x9ED5
CJK UNIFIED IDEOGRAPH:'ECB5:60597:&#x9ED6
CJK UNIFIED IDEOGRAPH:'ECB6:60598:&#x9EFA
CJK UNIFIED IDEOGRAPH:'ECB7:60599:&#x9F12
CJK UNIFIED IDEOGRAPH:'ECB8:60600:&#x9F3D
CJK UNIFIED IDEOGRAPH:'ECB9:60601:&#x5126
CJK UNIFIED IDEOGRAPH:'ECBA:60602:&#x5125
CJK UNIFIED IDEOGRAPH:'ECBB:60603:&#x5122
CJK UNIFIED IDEOGRAPH:'ECBC:60604:&#x5124
CJK UNIFIED IDEOGRAPH:'ECBD:60605:&#x5120
CJK UNIFIED IDEOGRAPH:'ECBE:60606:&#x5129
CJK UNIFIED IDEOGRAPH:'ECBF:60607:&#x52F4
CJK UNIFIED IDEOGRAPH:'ECC0:60608:&#x5693
CJK UNIFIED IDEOGRAPH:'ECC1:60609:&#x568C
CJK UNIFIED IDEOGRAPH:'ECC2:60610:&#x568D
CJK UNIFIED IDEOGRAPH:'ECC3:60611:&#x5686
CJK UNIFIED IDEOGRAPH:'ECC4:60612:&#x5684
CJK UNIFIED IDEOGRAPH:'ECC5:60613:&#x5683
CJK UNIFIED IDEOGRAPH:'ECC6:60614:&#x567E
CJK UNIFIED IDEOGRAPH:'ECC7:60615:&#x5682
CJK UNIFIED IDEOGRAPH:'ECC8:60616:&#x567F
CJK UNIFIED IDEOGRAPH:'ECC9:60617:&#x5681
CJK UNIFIED IDEOGRAPH:'ECCA:60618:&#x58D6
CJK UNIFIED IDEOGRAPH:'ECCB:60619:&#x58D4
CJK UNIFIED IDEOGRAPH:'ECCC:60620:&#x58CF
CJK UNIFIED IDEOGRAPH:'ECCD:60621:&#x58D2
CJK UNIFIED IDEOGRAPH:'ECCE:60622:&#x5B2D
CJK UNIFIED IDEOGRAPH:'ECCF:60623:&#x5B25
CJK UNIFIED IDEOGRAPH:'ECD0:60624:&#x5B32
CJK UNIFIED IDEOGRAPH:'ECD1:60625:&#x5B23
CJK UNIFIED IDEOGRAPH:'ECD2:60626:&#x5B2C
CJK UNIFIED IDEOGRAPH:'ECD3:60627:&#x5B27
CJK UNIFIED IDEOGRAPH:'ECD4:60628:&#x5B26
CJK UNIFIED IDEOGRAPH:'ECD5:60629:&#x5B2F
CJK UNIFIED IDEOGRAPH:'ECD6:60630:&#x5B2E
CJK UNIFIED IDEOGRAPH:'ECD7:60631:&#x5B7B
CJK UNIFIED IDEOGRAPH:'ECD8:60632:&#x5BF1
CJK UNIFIED IDEOGRAPH:'ECD9:60633:&#x5BF2
CJK UNIFIED IDEOGRAPH:'ECDA:60634:&#x5DB7
CJK UNIFIED IDEOGRAPH:'ECDB:60635:&#x5E6C
CJK UNIFIED IDEOGRAPH:'ECDC:60636:&#x5E6A
CJK UNIFIED IDEOGRAPH:'ECDD:60637:&#x5FBE
CJK UNIFIED IDEOGRAPH:'ECDE:60638:&#x5FBB
CJK UNIFIED IDEOGRAPH:'ECDF:60639:&#x61C3
CJK UNIFIED IDEOGRAPH:'ECE0:60640:&#x61B5
CJK UNIFIED IDEOGRAPH:'ECE1:60641:&#x61BC
CJK UNIFIED IDEOGRAPH:'ECE2:60642:&#x61E7
CJK UNIFIED IDEOGRAPH:'ECE3:60643:&#x61E0
CJK UNIFIED IDEOGRAPH:'ECE4:60644:&#x61E5
CJK UNIFIED IDEOGRAPH:'ECE5:60645:&#x61E4
CJK UNIFIED IDEOGRAPH:'ECE6:60646:&#x61E8
CJK UNIFIED IDEOGRAPH:'ECE7:60647:&#x61DE
CJK UNIFIED IDEOGRAPH:'ECE8:60648:&#x64EF
CJK UNIFIED IDEOGRAPH:'ECE9:60649:&#x64E9
CJK UNIFIED IDEOGRAPH:'ECEA:60650:&#x64E3
CJK UNIFIED IDEOGRAPH:'ECEB:60651:&#x64EB
CJK UNIFIED IDEOGRAPH:'ECEC:60652:&#x64E4
CJK UNIFIED IDEOGRAPH:'ECED:60653:&#x64E8
CJK UNIFIED IDEOGRAPH:'ECEE:60654:&#x6581
CJK UNIFIED IDEOGRAPH:'ECEF:60655:&#x6580
CJK UNIFIED IDEOGRAPH:'ECF0:60656:&#x65B6
CJK UNIFIED IDEOGRAPH:'ECF1:60657:&#x65DA
CJK UNIFIED IDEOGRAPH:'ECF2:60658:&#x66D2
CJK UNIFIED IDEOGRAPH:'ECF3:60659:&#x6A8D
CJK UNIFIED IDEOGRAPH:'ECF4:60660:&#x6A96
CJK UNIFIED IDEOGRAPH:'ECF5:60661:&#x6A81
CJK UNIFIED IDEOGRAPH:'ECF6:60662:&#x6AA5
CJK UNIFIED IDEOGRAPH:'ECF7:60663:&#x6A89
CJK UNIFIED IDEOGRAPH:'ECF8:60664:&#x6A9F
CJK UNIFIED IDEOGRAPH:'ECF9:60665:&#x6A9B
CJK UNIFIED IDEOGRAPH:'ECFA:60666:&#x6AA1
CJK UNIFIED IDEOGRAPH:'ECFB:60667:&#x6A9E
CJK UNIFIED IDEOGRAPH:'ECFC:60668:&#x6A87
CJK UNIFIED IDEOGRAPH:'ECFD:60669:&#x6A93
CJK UNIFIED IDEOGRAPH:'ECFE:60670:&#x6A8E
CJK UNIFIED IDEOGRAPH:'ED40:60736:&#x6A95
CJK UNIFIED IDEOGRAPH:'ED41:60737:&#x6A83
CJK UNIFIED IDEOGRAPH:'ED42:60738:&#x6AA8
CJK UNIFIED IDEOGRAPH:'ED43:60739:&#x6AA4
CJK UNIFIED IDEOGRAPH:'ED44:60740:&#x6A91
CJK UNIFIED IDEOGRAPH:'ED45:60741:&#x6A7F
CJK UNIFIED IDEOGRAPH:'ED46:60742:&#x6AA6
CJK UNIFIED IDEOGRAPH:'ED47:60743:&#x6A9A
CJK UNIFIED IDEOGRAPH:'ED48:60744:&#x6A85
CJK UNIFIED IDEOGRAPH:'ED49:60745:&#x6A8C
CJK UNIFIED IDEOGRAPH:'ED4A:60746:&#x6A92
CJK UNIFIED IDEOGRAPH:'ED4B:60747:&#x6B5B
CJK UNIFIED IDEOGRAPH:'ED4C:60748:&#x6BAD
CJK UNIFIED IDEOGRAPH:'ED4D:60749:&#x6C09
CJK UNIFIED IDEOGRAPH:'ED4E:60750:&#x6FCC
CJK UNIFIED IDEOGRAPH:'ED4F:60751:&#x6FA9
CJK UNIFIED IDEOGRAPH:'ED50:60752:&#x6FF4
CJK UNIFIED IDEOGRAPH:'ED51:60753:&#x6FD4
CJK UNIFIED IDEOGRAPH:'ED52:60754:&#x6FE3
CJK UNIFIED IDEOGRAPH:'ED53:60755:&#x6FDC
CJK UNIFIED IDEOGRAPH:'ED54:60756:&#x6FED
CJK UNIFIED IDEOGRAPH:'ED55:60757:&#x6FE7
CJK UNIFIED IDEOGRAPH:'ED56:60758:&#x6FE6
CJK UNIFIED IDEOGRAPH:'ED57:60759:&#x6FDE
CJK UNIFIED IDEOGRAPH:'ED58:60760:&#x6FF2
CJK UNIFIED IDEOGRAPH:'ED59:60761:&#x6FDD
CJK UNIFIED IDEOGRAPH:'ED5A:60762:&#x6FE2
CJK UNIFIED IDEOGRAPH:'ED5B:60763:&#x6FE8
CJK UNIFIED IDEOGRAPH:'ED5C:60764:&#x71E1
CJK UNIFIED IDEOGRAPH:'ED5D:60765:&#x71F1
CJK UNIFIED IDEOGRAPH:'ED5E:60766:&#x71E8
CJK UNIFIED IDEOGRAPH:'ED5F:60767:&#x71F2
CJK UNIFIED IDEOGRAPH:'ED60:60768:&#x71E4
CJK UNIFIED IDEOGRAPH:'ED61:60769:&#x71F0
CJK UNIFIED IDEOGRAPH:'ED62:60770:&#x71E2
CJK UNIFIED IDEOGRAPH:'ED63:60771:&#x7373
CJK UNIFIED IDEOGRAPH:'ED64:60772:&#x736E
CJK UNIFIED IDEOGRAPH:'ED65:60773:&#x736F
CJK UNIFIED IDEOGRAPH:'ED66:60774:&#x7497
CJK UNIFIED IDEOGRAPH:'ED67:60775:&#x74B2
CJK UNIFIED IDEOGRAPH:'ED68:60776:&#x74AB
CJK UNIFIED IDEOGRAPH:'ED69:60777:&#x7490
CJK UNIFIED IDEOGRAPH:'ED6A:60778:&#x74AA
CJK UNIFIED IDEOGRAPH:'ED6B:60779:&#x74AD
CJK UNIFIED IDEOGRAPH:'ED6C:60780:&#x74B1
CJK UNIFIED IDEOGRAPH:'ED6D:60781:&#x74A5
CJK UNIFIED IDEOGRAPH:'ED6E:60782:&#x74AF
CJK UNIFIED IDEOGRAPH:'ED6F:60783:&#x7510
CJK UNIFIED IDEOGRAPH:'ED70:60784:&#x7511
CJK UNIFIED IDEOGRAPH:'ED71:60785:&#x7512
CJK UNIFIED IDEOGRAPH:'ED72:60786:&#x750F
CJK UNIFIED IDEOGRAPH:'ED73:60787:&#x7584
CJK UNIFIED IDEOGRAPH:'ED74:60788:&#x7643
CJK UNIFIED IDEOGRAPH:'ED75:60789:&#x7648
CJK UNIFIED IDEOGRAPH:'ED76:60790:&#x7649
CJK UNIFIED IDEOGRAPH:'ED77:60791:&#x7647
CJK UNIFIED IDEOGRAPH:'ED78:60792:&#x76A4
CJK UNIFIED IDEOGRAPH:'ED79:60793:&#x76E9
CJK UNIFIED IDEOGRAPH:'ED7A:60794:&#x77B5
CJK UNIFIED IDEOGRAPH:'ED7B:60795:&#x77AB
CJK UNIFIED IDEOGRAPH:'ED7C:60796:&#x77B2
CJK UNIFIED IDEOGRAPH:'ED7D:60797:&#x77B7
CJK UNIFIED IDEOGRAPH:'ED7E:60798:&#x77B6
CJK UNIFIED IDEOGRAPH:'EDA1:60833:&#x77B4
CJK UNIFIED IDEOGRAPH:'EDA2:60834:&#x77B1
CJK UNIFIED IDEOGRAPH:'EDA3:60835:&#x77A8
CJK UNIFIED IDEOGRAPH:'EDA4:60836:&#x77F0
CJK UNIFIED IDEOGRAPH:'EDA5:60837:&#x78F3
CJK UNIFIED IDEOGRAPH:'EDA6:60838:&#x78FD
CJK UNIFIED IDEOGRAPH:'EDA7:60839:&#x7902
CJK UNIFIED IDEOGRAPH:'EDA8:60840:&#x78FB
CJK UNIFIED IDEOGRAPH:'EDA9:60841:&#x78FC
CJK UNIFIED IDEOGRAPH:'EDAA:60842:&#x78F2
CJK UNIFIED IDEOGRAPH:'EDAB:60843:&#x7905
CJK UNIFIED IDEOGRAPH:'EDAC:60844:&#x78F9
CJK UNIFIED IDEOGRAPH:'EDAD:60845:&#x78FE
CJK UNIFIED IDEOGRAPH:'EDAE:60846:&#x7904
CJK UNIFIED IDEOGRAPH:'EDAF:60847:&#x79AB
CJK UNIFIED IDEOGRAPH:'EDB0:60848:&#x79A8
CJK UNIFIED IDEOGRAPH:'EDB1:60849:&#x7A5C
CJK UNIFIED IDEOGRAPH:'EDB2:60850:&#x7A5B
CJK UNIFIED IDEOGRAPH:'EDB3:60851:&#x7A56
CJK UNIFIED IDEOGRAPH:'EDB4:60852:&#x7A58
CJK UNIFIED IDEOGRAPH:'EDB5:60853:&#x7A54
CJK UNIFIED IDEOGRAPH:'EDB6:60854:&#x7A5A
CJK UNIFIED IDEOGRAPH:'EDB7:60855:&#x7ABE
CJK UNIFIED IDEOGRAPH:'EDB8:60856:&#x7AC0
CJK UNIFIED IDEOGRAPH:'EDB9:60857:&#x7AC1
CJK UNIFIED IDEOGRAPH:'EDBA:60858:&#x7C05
CJK UNIFIED IDEOGRAPH:'EDBB:60859:&#x7C0F
CJK UNIFIED IDEOGRAPH:'EDBC:60860:&#x7BF2
CJK UNIFIED IDEOGRAPH:'EDBD:60861:&#x7C00
CJK UNIFIED IDEOGRAPH:'EDBE:60862:&#x7BFF
CJK UNIFIED IDEOGRAPH:'EDBF:60863:&#x7BFB
CJK UNIFIED IDEOGRAPH:'EDC0:60864:&#x7C0E
CJK UNIFIED IDEOGRAPH:'EDC1:60865:&#x7BF4
CJK UNIFIED IDEOGRAPH:'EDC2:60866:&#x7C0B
CJK UNIFIED IDEOGRAPH:'EDC3:60867:&#x7BF3
CJK UNIFIED IDEOGRAPH:'EDC4:60868:&#x7C02
CJK UNIFIED IDEOGRAPH:'EDC5:60869:&#x7C09
CJK UNIFIED IDEOGRAPH:'EDC6:60870:&#x7C03
CJK UNIFIED IDEOGRAPH:'EDC7:60871:&#x7C01
CJK UNIFIED IDEOGRAPH:'EDC8:60872:&#x7BF8
CJK UNIFIED IDEOGRAPH:'EDC9:60873:&#x7BFD
CJK UNIFIED IDEOGRAPH:'EDCA:60874:&#x7C06
CJK UNIFIED IDEOGRAPH:'EDCB:60875:&#x7BF0
CJK UNIFIED IDEOGRAPH:'EDCC:60876:&#x7BF1
CJK UNIFIED IDEOGRAPH:'EDCD:60877:&#x7C10
CJK UNIFIED IDEOGRAPH:'EDCE:60878:&#x7C0A
CJK UNIFIED IDEOGRAPH:'EDCF:60879:&#x7CE8
CJK UNIFIED IDEOGRAPH:'EDD0:60880:&#x7E2D
CJK UNIFIED IDEOGRAPH:'EDD1:60881:&#x7E3C
CJK UNIFIED IDEOGRAPH:'EDD2:60882:&#x7E42
CJK UNIFIED IDEOGRAPH:'EDD3:60883:&#x7E33
CJK UNIFIED IDEOGRAPH:'EDD4:60884:&#x9848
CJK UNIFIED IDEOGRAPH:'EDD5:60885:&#x7E38
CJK UNIFIED IDEOGRAPH:'EDD6:60886:&#x7E2A
CJK UNIFIED IDEOGRAPH:'EDD7:60887:&#x7E49
CJK UNIFIED IDEOGRAPH:'EDD8:60888:&#x7E40
CJK UNIFIED IDEOGRAPH:'EDD9:60889:&#x7E47
CJK UNIFIED IDEOGRAPH:'EDDA:60890:&#x7E29
CJK UNIFIED IDEOGRAPH:'EDDB:60891:&#x7E4C
CJK UNIFIED IDEOGRAPH:'EDDC:60892:&#x7E30
CJK UNIFIED IDEOGRAPH:'EDDD:60893:&#x7E3B
CJK UNIFIED IDEOGRAPH:'EDDE:60894:&#x7E36
CJK UNIFIED IDEOGRAPH:'EDDF:60895:&#x7E44
CJK UNIFIED IDEOGRAPH:'EDE0:60896:&#x7E3A
CJK UNIFIED IDEOGRAPH:'EDE1:60897:&#x7F45
CJK UNIFIED IDEOGRAPH:'EDE2:60898:&#x7F7F
CJK UNIFIED IDEOGRAPH:'EDE3:60899:&#x7F7E
CJK UNIFIED IDEOGRAPH:'EDE4:60900:&#x7F7D
CJK UNIFIED IDEOGRAPH:'EDE5:60901:&#x7FF4
CJK UNIFIED IDEOGRAPH:'EDE6:60902:&#x7FF2
CJK UNIFIED IDEOGRAPH:'EDE7:60903:&#x802C
CJK UNIFIED IDEOGRAPH:'EDE8:60904:&#x81BB
CJK UNIFIED IDEOGRAPH:'EDE9:60905:&#x81C4
CJK UNIFIED IDEOGRAPH:'EDEA:60906:&#x81CC
CJK UNIFIED IDEOGRAPH:'EDEB:60907:&#x81CA
CJK UNIFIED IDEOGRAPH:'EDEC:60908:&#x81C5
CJK UNIFIED IDEOGRAPH:'EDED:60909:&#x81C7
CJK UNIFIED IDEOGRAPH:'EDEE:60910:&#x81BC
CJK UNIFIED IDEOGRAPH:'EDEF:60911:&#x81E9
CJK UNIFIED IDEOGRAPH:'EDF0:60912:&#x825B
CJK UNIFIED IDEOGRAPH:'EDF1:60913:&#x825A
CJK UNIFIED IDEOGRAPH:'EDF2:60914:&#x825C
CJK UNIFIED IDEOGRAPH:'EDF3:60915:&#x8583
CJK UNIFIED IDEOGRAPH:'EDF4:60916:&#x8580
CJK UNIFIED IDEOGRAPH:'EDF5:60917:&#x858F
CJK UNIFIED IDEOGRAPH:'EDF6:60918:&#x85A7
CJK UNIFIED IDEOGRAPH:'EDF7:60919:&#x8595
CJK UNIFIED IDEOGRAPH:'EDF8:60920:&#x85A0
CJK UNIFIED IDEOGRAPH:'EDF9:60921:&#x858B
CJK UNIFIED IDEOGRAPH:'EDFA:60922:&#x85A3
CJK UNIFIED IDEOGRAPH:'EDFB:60923:&#x857B
CJK UNIFIED IDEOGRAPH:'EDFC:60924:&#x85A4
CJK UNIFIED IDEOGRAPH:'EDFD:60925:&#x859A
CJK UNIFIED IDEOGRAPH:'EDFE:60926:&#x859E
CJK UNIFIED IDEOGRAPH:'EE40:60992:&#x8577
CJK UNIFIED IDEOGRAPH:'EE41:60993:&#x857C
CJK UNIFIED IDEOGRAPH:'EE42:60994:&#x8589
CJK UNIFIED IDEOGRAPH:'EE43:60995:&#x85A1
CJK UNIFIED IDEOGRAPH:'EE44:60996:&#x857A
CJK UNIFIED IDEOGRAPH:'EE45:60997:&#x8578
CJK UNIFIED IDEOGRAPH:'EE46:60998:&#x8557
CJK UNIFIED IDEOGRAPH:'EE47:60999:&#x858E
CJK UNIFIED IDEOGRAPH:'EE48:61000:&#x8596
CJK UNIFIED IDEOGRAPH:'EE49:61001:&#x8586
CJK UNIFIED IDEOGRAPH:'EE4A:61002:&#x858D
CJK UNIFIED IDEOGRAPH:'EE4B:61003:&#x8599
CJK UNIFIED IDEOGRAPH:'EE4C:61004:&#x859D
CJK UNIFIED IDEOGRAPH:'EE4D:61005:&#x8581
CJK UNIFIED IDEOGRAPH:'EE4E:61006:&#x85A2
CJK UNIFIED IDEOGRAPH:'EE4F:61007:&#x8582
CJK UNIFIED IDEOGRAPH:'EE50:61008:&#x8588
CJK UNIFIED IDEOGRAPH:'EE51:61009:&#x8585
CJK UNIFIED IDEOGRAPH:'EE52:61010:&#x8579
CJK UNIFIED IDEOGRAPH:'EE53:61011:&#x8576
CJK UNIFIED IDEOGRAPH:'EE54:61012:&#x8598
CJK UNIFIED IDEOGRAPH:'EE55:61013:&#x8590
CJK UNIFIED IDEOGRAPH:'EE56:61014:&#x859F
CJK UNIFIED IDEOGRAPH:'EE57:61015:&#x8668
CJK UNIFIED IDEOGRAPH:'EE58:61016:&#x87BE
CJK UNIFIED IDEOGRAPH:'EE59:61017:&#x87AA
CJK UNIFIED IDEOGRAPH:'EE5A:61018:&#x87AD
CJK UNIFIED IDEOGRAPH:'EE5B:61019:&#x87C5
CJK UNIFIED IDEOGRAPH:'EE5C:61020:&#x87B0
CJK UNIFIED IDEOGRAPH:'EE5D:61021:&#x87AC
CJK UNIFIED IDEOGRAPH:'EE5E:61022:&#x87B9
CJK UNIFIED IDEOGRAPH:'EE5F:61023:&#x87B5
CJK UNIFIED IDEOGRAPH:'EE60:61024:&#x87BC
CJK UNIFIED IDEOGRAPH:'EE61:61025:&#x87AE
CJK UNIFIED IDEOGRAPH:'EE62:61026:&#x87C9
CJK UNIFIED IDEOGRAPH:'EE63:61027:&#x87C3
CJK UNIFIED IDEOGRAPH:'EE64:61028:&#x87C2
CJK UNIFIED IDEOGRAPH:'EE65:61029:&#x87CC
CJK UNIFIED IDEOGRAPH:'EE66:61030:&#x87B7
CJK UNIFIED IDEOGRAPH:'EE67:61031:&#x87AF
CJK UNIFIED IDEOGRAPH:'EE68:61032:&#x87C4
CJK UNIFIED IDEOGRAPH:'EE69:61033:&#x87CA
CJK UNIFIED IDEOGRAPH:'EE6A:61034:&#x87B4
CJK UNIFIED IDEOGRAPH:'EE6B:61035:&#x87B6
CJK UNIFIED IDEOGRAPH:'EE6C:61036:&#x87BF
CJK UNIFIED IDEOGRAPH:'EE6D:61037:&#x87B8
CJK UNIFIED IDEOGRAPH:'EE6E:61038:&#x87BD
CJK UNIFIED IDEOGRAPH:'EE6F:61039:&#x87DE
CJK UNIFIED IDEOGRAPH:'EE70:61040:&#x87B2
CJK UNIFIED IDEOGRAPH:'EE71:61041:&#x8935
CJK UNIFIED IDEOGRAPH:'EE72:61042:&#x8933
CJK UNIFIED IDEOGRAPH:'EE73:61043:&#x893C
CJK UNIFIED IDEOGRAPH:'EE74:61044:&#x893E
CJK UNIFIED IDEOGRAPH:'EE75:61045:&#x8941
CJK UNIFIED IDEOGRAPH:'EE76:61046:&#x8952
CJK UNIFIED IDEOGRAPH:'EE77:61047:&#x8937
CJK UNIFIED IDEOGRAPH:'EE78:61048:&#x8942
CJK UNIFIED IDEOGRAPH:'EE79:61049:&#x89AD
CJK UNIFIED IDEOGRAPH:'EE7A:61050:&#x89AF
CJK UNIFIED IDEOGRAPH:'EE7B:61051:&#x89AE
CJK UNIFIED IDEOGRAPH:'EE7C:61052:&#x89F2
CJK UNIFIED IDEOGRAPH:'EE7D:61053:&#x89F3
CJK UNIFIED IDEOGRAPH:'EE7E:61054:&#x8B1E
CJK UNIFIED IDEOGRAPH:'EEA1:61089:&#x8B18
CJK UNIFIED IDEOGRAPH:'EEA2:61090:&#x8B16
CJK UNIFIED IDEOGRAPH:'EEA3:61091:&#x8B11
CJK UNIFIED IDEOGRAPH:'EEA4:61092:&#x8B05
CJK UNIFIED IDEOGRAPH:'EEA5:61093:&#x8B0B
CJK UNIFIED IDEOGRAPH:'EEA6:61094:&#x8B22
CJK UNIFIED IDEOGRAPH:'EEA7:61095:&#x8B0F
CJK UNIFIED IDEOGRAPH:'EEA8:61096:&#x8B12
CJK UNIFIED IDEOGRAPH:'EEA9:61097:&#x8B15
CJK UNIFIED IDEOGRAPH:'EEAA:61098:&#x8B07
CJK UNIFIED IDEOGRAPH:'EEAB:61099:&#x8B0D
CJK UNIFIED IDEOGRAPH:'EEAC:61100:&#x8B08
CJK UNIFIED IDEOGRAPH:'EEAD:61101:&#x8B06
CJK UNIFIED IDEOGRAPH:'EEAE:61102:&#x8B1C
CJK UNIFIED IDEOGRAPH:'EEAF:61103:&#x8B13
CJK UNIFIED IDEOGRAPH:'EEB0:61104:&#x8B1A
CJK UNIFIED IDEOGRAPH:'EEB1:61105:&#x8C4F
CJK UNIFIED IDEOGRAPH:'EEB2:61106:&#x8C70
CJK UNIFIED IDEOGRAPH:'EEB3:61107:&#x8C72
CJK UNIFIED IDEOGRAPH:'EEB4:61108:&#x8C71
CJK UNIFIED IDEOGRAPH:'EEB5:61109:&#x8C6F
CJK UNIFIED IDEOGRAPH:'EEB6:61110:&#x8C95
CJK UNIFIED IDEOGRAPH:'EEB7:61111:&#x8C94
CJK UNIFIED IDEOGRAPH:'EEB8:61112:&#x8CF9
CJK UNIFIED IDEOGRAPH:'EEB9:61113:&#x8D6F
CJK UNIFIED IDEOGRAPH:'EEBA:61114:&#x8E4E
CJK UNIFIED IDEOGRAPH:'EEBB:61115:&#x8E4D
CJK UNIFIED IDEOGRAPH:'EEBC:61116:&#x8E53
CJK UNIFIED IDEOGRAPH:'EEBD:61117:&#x8E50
CJK UNIFIED IDEOGRAPH:'EEBE:61118:&#x8E4C
CJK UNIFIED IDEOGRAPH:'EEBF:61119:&#x8E47
CJK UNIFIED IDEOGRAPH:'EEC0:61120:&#x8F43
CJK UNIFIED IDEOGRAPH:'EEC1:61121:&#x8F40
CJK UNIFIED IDEOGRAPH:'EEC2:61122:&#x9085
CJK UNIFIED IDEOGRAPH:'EEC3:61123:&#x907E
CJK UNIFIED IDEOGRAPH:'EEC4:61124:&#x9138
CJK UNIFIED IDEOGRAPH:'EEC5:61125:&#x919A
CJK UNIFIED IDEOGRAPH:'EEC6:61126:&#x91A2
CJK UNIFIED IDEOGRAPH:'EEC7:61127:&#x919B
CJK UNIFIED IDEOGRAPH:'EEC8:61128:&#x9199
CJK UNIFIED IDEOGRAPH:'EEC9:61129:&#x919F
CJK UNIFIED IDEOGRAPH:'EECA:61130:&#x91A1
CJK UNIFIED IDEOGRAPH:'EECB:61131:&#x919D
CJK UNIFIED IDEOGRAPH:'EECC:61132:&#x91A0
CJK UNIFIED IDEOGRAPH:'EECD:61133:&#x93A1
CJK UNIFIED IDEOGRAPH:'EECE:61134:&#x9383
CJK UNIFIED IDEOGRAPH:'EECF:61135:&#x93AF
CJK UNIFIED IDEOGRAPH:'EED0:61136:&#x9364
CJK UNIFIED IDEOGRAPH:'EED1:61137:&#x9356
CJK UNIFIED IDEOGRAPH:'EED2:61138:&#x9347
CJK UNIFIED IDEOGRAPH:'EED3:61139:&#x937C
CJK UNIFIED IDEOGRAPH:'EED4:61140:&#x9358
CJK UNIFIED IDEOGRAPH:'EED5:61141:&#x935C
CJK UNIFIED IDEOGRAPH:'EED6:61142:&#x9376
CJK UNIFIED IDEOGRAPH:'EED7:61143:&#x9349
CJK UNIFIED IDEOGRAPH:'EED8:61144:&#x9350
CJK UNIFIED IDEOGRAPH:'EED9:61145:&#x9351
CJK UNIFIED IDEOGRAPH:'EEDA:61146:&#x9360
CJK UNIFIED IDEOGRAPH:'EEDB:61147:&#x936D
CJK UNIFIED IDEOGRAPH:'EEDC:61148:&#x938F
CJK UNIFIED IDEOGRAPH:'EEDD:61149:&#x934C
CJK UNIFIED IDEOGRAPH:'EEDE:61150:&#x936A
CJK UNIFIED IDEOGRAPH:'EEDF:61151:&#x9379
CJK UNIFIED IDEOGRAPH:'EEE0:61152:&#x9357
CJK UNIFIED IDEOGRAPH:'EEE1:61153:&#x9355
CJK UNIFIED IDEOGRAPH:'EEE2:61154:&#x9352
CJK UNIFIED IDEOGRAPH:'EEE3:61155:&#x934F
CJK UNIFIED IDEOGRAPH:'EEE4:61156:&#x9371
CJK UNIFIED IDEOGRAPH:'EEE5:61157:&#x9377
CJK UNIFIED IDEOGRAPH:'EEE6:61158:&#x937B
CJK UNIFIED IDEOGRAPH:'EEE7:61159:&#x9361
CJK UNIFIED IDEOGRAPH:'EEE8:61160:&#x935E
CJK UNIFIED IDEOGRAPH:'EEE9:61161:&#x9363
CJK UNIFIED IDEOGRAPH:'EEEA:61162:&#x9367
CJK UNIFIED IDEOGRAPH:'EEEB:61163:&#x9380
CJK UNIFIED IDEOGRAPH:'EEEC:61164:&#x934E
CJK UNIFIED IDEOGRAPH:'EEED:61165:&#x9359
CJK UNIFIED IDEOGRAPH:'EEEE:61166:&#x95C7
CJK UNIFIED IDEOGRAPH:'EEEF:61167:&#x95C0
CJK UNIFIED IDEOGRAPH:'EEF0:61168:&#x95C9
CJK UNIFIED IDEOGRAPH:'EEF1:61169:&#x95C3
CJK UNIFIED IDEOGRAPH:'EEF2:61170:&#x95C5
CJK UNIFIED IDEOGRAPH:'EEF3:61171:&#x95B7
CJK UNIFIED IDEOGRAPH:'EEF4:61172:&#x96AE
CJK UNIFIED IDEOGRAPH:'EEF5:61173:&#x96B0
CJK UNIFIED IDEOGRAPH:'EEF6:61174:&#x96AC
CJK UNIFIED IDEOGRAPH:'EEF7:61175:&#x9720
CJK UNIFIED IDEOGRAPH:'EEF8:61176:&#x971F
CJK UNIFIED IDEOGRAPH:'EEF9:61177:&#x9718
CJK UNIFIED IDEOGRAPH:'EEFA:61178:&#x971D
CJK UNIFIED IDEOGRAPH:'EEFB:61179:&#x9719
CJK UNIFIED IDEOGRAPH:'EEFC:61180:&#x979A
CJK UNIFIED IDEOGRAPH:'EEFD:61181:&#x97A1
CJK UNIFIED IDEOGRAPH:'EEFE:61182:&#x979C
CJK UNIFIED IDEOGRAPH:'EF40:61248:&#x979E
CJK UNIFIED IDEOGRAPH:'EF41:61249:&#x979D
CJK UNIFIED IDEOGRAPH:'EF42:61250:&#x97D5
CJK UNIFIED IDEOGRAPH:'EF43:61251:&#x97D4
CJK UNIFIED IDEOGRAPH:'EF44:61252:&#x97F1
CJK UNIFIED IDEOGRAPH:'EF45:61253:&#x9841
CJK UNIFIED IDEOGRAPH:'EF46:61254:&#x9844
CJK UNIFIED IDEOGRAPH:'EF47:61255:&#x984A
CJK UNIFIED IDEOGRAPH:'EF48:61256:&#x9849
CJK UNIFIED IDEOGRAPH:'EF49:61257:&#x9845
CJK UNIFIED IDEOGRAPH:'EF4A:61258:&#x9843
CJK UNIFIED IDEOGRAPH:'EF4B:61259:&#x9925
CJK UNIFIED IDEOGRAPH:'EF4C:61260:&#x992B
CJK UNIFIED IDEOGRAPH:'EF4D:61261:&#x992C
CJK UNIFIED IDEOGRAPH:'EF4E:61262:&#x992A
CJK UNIFIED IDEOGRAPH:'EF4F:61263:&#x9933
CJK UNIFIED IDEOGRAPH:'EF50:61264:&#x9932
CJK UNIFIED IDEOGRAPH:'EF51:61265:&#x992F
CJK UNIFIED IDEOGRAPH:'EF52:61266:&#x992D
CJK UNIFIED IDEOGRAPH:'EF53:61267:&#x9931
CJK UNIFIED IDEOGRAPH:'EF54:61268:&#x9930
CJK UNIFIED IDEOGRAPH:'EF55:61269:&#x9998
CJK UNIFIED IDEOGRAPH:'EF56:61270:&#x99A3
CJK UNIFIED IDEOGRAPH:'EF57:61271:&#x99A1
CJK UNIFIED IDEOGRAPH:'EF58:61272:&#x9A02
CJK UNIFIED IDEOGRAPH:'EF59:61273:&#x99FA
CJK UNIFIED IDEOGRAPH:'EF5A:61274:&#x99F4
CJK UNIFIED IDEOGRAPH:'EF5B:61275:&#x99F7
CJK UNIFIED IDEOGRAPH:'EF5C:61276:&#x99F9
CJK UNIFIED IDEOGRAPH:'EF5D:61277:&#x99F8
CJK UNIFIED IDEOGRAPH:'EF5E:61278:&#x99F6
CJK UNIFIED IDEOGRAPH:'EF5F:61279:&#x99FB
CJK UNIFIED IDEOGRAPH:'EF60:61280:&#x99FD
CJK UNIFIED IDEOGRAPH:'EF61:61281:&#x99FE
CJK UNIFIED IDEOGRAPH:'EF62:61282:&#x99FC
CJK UNIFIED IDEOGRAPH:'EF63:61283:&#x9A03
CJK UNIFIED IDEOGRAPH:'EF64:61284:&#x9ABE
CJK UNIFIED IDEOGRAPH:'EF65:61285:&#x9AFE
CJK UNIFIED IDEOGRAPH:'EF66:61286:&#x9AFD
CJK UNIFIED IDEOGRAPH:'EF67:61287:&#x9B01
CJK UNIFIED IDEOGRAPH:'EF68:61288:&#x9AFC
CJK UNIFIED IDEOGRAPH:'EF69:61289:&#x9B48
CJK UNIFIED IDEOGRAPH:'EF6A:61290:&#x9B9A
CJK UNIFIED IDEOGRAPH:'EF6B:61291:&#x9BA8
CJK UNIFIED IDEOGRAPH:'EF6C:61292:&#x9B9E
CJK UNIFIED IDEOGRAPH:'EF6D:61293:&#x9B9B
CJK UNIFIED IDEOGRAPH:'EF6E:61294:&#x9BA6
CJK UNIFIED IDEOGRAPH:'EF6F:61295:&#x9BA1
CJK UNIFIED IDEOGRAPH:'EF70:61296:&#x9BA5
CJK UNIFIED IDEOGRAPH:'EF71:61297:&#x9BA4
CJK UNIFIED IDEOGRAPH:'EF72:61298:&#x9B86
CJK UNIFIED IDEOGRAPH:'EF73:61299:&#x9BA2
CJK UNIFIED IDEOGRAPH:'EF74:61300:&#x9BA0
CJK UNIFIED IDEOGRAPH:'EF75:61301:&#x9BAF
CJK UNIFIED IDEOGRAPH:'EF76:61302:&#x9D33
CJK UNIFIED IDEOGRAPH:'EF77:61303:&#x9D41
CJK UNIFIED IDEOGRAPH:'EF78:61304:&#x9D67
CJK UNIFIED IDEOGRAPH:'EF79:61305:&#x9D36
CJK UNIFIED IDEOGRAPH:'EF7A:61306:&#x9D2E
CJK UNIFIED IDEOGRAPH:'EF7B:61307:&#x9D2F
CJK UNIFIED IDEOGRAPH:'EF7C:61308:&#x9D31
CJK UNIFIED IDEOGRAPH:'EF7D:61309:&#x9D38
CJK UNIFIED IDEOGRAPH:'EF7E:61310:&#x9D30
CJK UNIFIED IDEOGRAPH:'EFA1:61345:&#x9D45
CJK UNIFIED IDEOGRAPH:'EFA2:61346:&#x9D42
CJK UNIFIED IDEOGRAPH:'EFA3:61347:&#x9D43
CJK UNIFIED IDEOGRAPH:'EFA4:61348:&#x9D3E
CJK UNIFIED IDEOGRAPH:'EFA5:61349:&#x9D37
CJK UNIFIED IDEOGRAPH:'EFA6:61350:&#x9D40
CJK UNIFIED IDEOGRAPH:'EFA7:61351:&#x9D3D
CJK UNIFIED IDEOGRAPH:'EFA8:61352:&#x7FF5
CJK UNIFIED IDEOGRAPH:'EFA9:61353:&#x9D2D
CJK UNIFIED IDEOGRAPH:'EFAA:61354:&#x9E8A
CJK UNIFIED IDEOGRAPH:'EFAB:61355:&#x9E89
CJK UNIFIED IDEOGRAPH:'EFAC:61356:&#x9E8D
CJK UNIFIED IDEOGRAPH:'EFAD:61357:&#x9EB0
CJK UNIFIED IDEOGRAPH:'EFAE:61358:&#x9EC8
CJK UNIFIED IDEOGRAPH:'EFAF:61359:&#x9EDA
CJK UNIFIED IDEOGRAPH:'EFB0:61360:&#x9EFB
CJK UNIFIED IDEOGRAPH:'EFB1:61361:&#x9EFF
CJK UNIFIED IDEOGRAPH:'EFB2:61362:&#x9F24
CJK UNIFIED IDEOGRAPH:'EFB3:61363:&#x9F23
CJK UNIFIED IDEOGRAPH:'EFB4:61364:&#x9F22
CJK UNIFIED IDEOGRAPH:'EFB5:61365:&#x9F54
CJK UNIFIED IDEOGRAPH:'EFB6:61366:&#x9FA0
CJK UNIFIED IDEOGRAPH:'EFB7:61367:&#x5131
CJK UNIFIED IDEOGRAPH:'EFB8:61368:&#x512D
CJK UNIFIED IDEOGRAPH:'EFB9:61369:&#x512E
CJK UNIFIED IDEOGRAPH:'EFBA:61370:&#x5698
CJK UNIFIED IDEOGRAPH:'EFBB:61371:&#x569C
CJK UNIFIED IDEOGRAPH:'EFBC:61372:&#x5697
CJK UNIFIED IDEOGRAPH:'EFBD:61373:&#x569A
CJK UNIFIED IDEOGRAPH:'EFBE:61374:&#x569D
CJK UNIFIED IDEOGRAPH:'EFBF:61375:&#x5699
CJK UNIFIED IDEOGRAPH:'EFC0:61376:&#x5970
CJK UNIFIED IDEOGRAPH:'EFC1:61377:&#x5B3C
CJK UNIFIED IDEOGRAPH:'EFC2:61378:&#x5C69
CJK UNIFIED IDEOGRAPH:'EFC3:61379:&#x5C6A
CJK UNIFIED IDEOGRAPH:'EFC4:61380:&#x5DC0
CJK UNIFIED IDEOGRAPH:'EFC5:61381:&#x5E6D
CJK UNIFIED IDEOGRAPH:'EFC6:61382:&#x5E6E
CJK UNIFIED IDEOGRAPH:'EFC7:61383:&#x61D8
CJK UNIFIED IDEOGRAPH:'EFC8:61384:&#x61DF
CJK UNIFIED IDEOGRAPH:'EFC9:61385:&#x61ED
CJK UNIFIED IDEOGRAPH:'EFCA:61386:&#x61EE
CJK UNIFIED IDEOGRAPH:'EFCB:61387:&#x61F1
CJK UNIFIED IDEOGRAPH:'EFCC:61388:&#x61EA
CJK UNIFIED IDEOGRAPH:'EFCD:61389:&#x61F0
CJK UNIFIED IDEOGRAPH:'EFCE:61390:&#x61EB
CJK UNIFIED IDEOGRAPH:'EFCF:61391:&#x61D6
CJK UNIFIED IDEOGRAPH:'EFD0:61392:&#x61E9
CJK UNIFIED IDEOGRAPH:'EFD1:61393:&#x64FF
CJK UNIFIED IDEOGRAPH:'EFD2:61394:&#x6504
CJK UNIFIED IDEOGRAPH:'EFD3:61395:&#x64FD
CJK UNIFIED IDEOGRAPH:'EFD4:61396:&#x64F8
CJK UNIFIED IDEOGRAPH:'EFD5:61397:&#x6501
CJK UNIFIED IDEOGRAPH:'EFD6:61398:&#x6503
CJK UNIFIED IDEOGRAPH:'EFD7:61399:&#x64FC
CJK UNIFIED IDEOGRAPH:'EFD8:61400:&#x6594
CJK UNIFIED IDEOGRAPH:'EFD9:61401:&#x65DB
CJK UNIFIED IDEOGRAPH:'EFDA:61402:&#x66DA
CJK UNIFIED IDEOGRAPH:'EFDB:61403:&#x66DB
CJK UNIFIED IDEOGRAPH:'EFDC:61404:&#x66D8
CJK UNIFIED IDEOGRAPH:'EFDD:61405:&#x6AC5
CJK UNIFIED IDEOGRAPH:'EFDE:61406:&#x6AB9
CJK UNIFIED IDEOGRAPH:'EFDF:61407:&#x6ABD
CJK UNIFIED IDEOGRAPH:'EFE0:61408:&#x6AE1
CJK UNIFIED IDEOGRAPH:'EFE1:61409:&#x6AC6
CJK UNIFIED IDEOGRAPH:'EFE2:61410:&#x6ABA
CJK UNIFIED IDEOGRAPH:'EFE3:61411:&#x6AB6
CJK UNIFIED IDEOGRAPH:'EFE4:61412:&#x6AB7
CJK UNIFIED IDEOGRAPH:'EFE5:61413:&#x6AC7
CJK UNIFIED IDEOGRAPH:'EFE6:61414:&#x6AB4
CJK UNIFIED IDEOGRAPH:'EFE7:61415:&#x6AAD
CJK UNIFIED IDEOGRAPH:'EFE8:61416:&#x6B5E
CJK UNIFIED IDEOGRAPH:'EFE9:61417:&#x6BC9
CJK UNIFIED IDEOGRAPH:'EFEA:61418:&#x6C0B
CJK UNIFIED IDEOGRAPH:'EFEB:61419:&#x7007
CJK UNIFIED IDEOGRAPH:'EFEC:61420:&#x700C
CJK UNIFIED IDEOGRAPH:'EFED:61421:&#x700D
CJK UNIFIED IDEOGRAPH:'EFEE:61422:&#x7001
CJK UNIFIED IDEOGRAPH:'EFEF:61423:&#x7005
CJK UNIFIED IDEOGRAPH:'EFF0:61424:&#x7014
CJK UNIFIED IDEOGRAPH:'EFF1:61425:&#x700E
CJK UNIFIED IDEOGRAPH:'EFF2:61426:&#x6FFF
CJK UNIFIED IDEOGRAPH:'EFF3:61427:&#x7000
CJK UNIFIED IDEOGRAPH:'EFF4:61428:&#x6FFB
CJK UNIFIED IDEOGRAPH:'EFF5:61429:&#x7026
CJK UNIFIED IDEOGRAPH:'EFF6:61430:&#x6FFC
CJK UNIFIED IDEOGRAPH:'EFF7:61431:&#x6FF7
CJK UNIFIED IDEOGRAPH:'EFF8:61432:&#x700A
CJK UNIFIED IDEOGRAPH:'EFF9:61433:&#x7201
CJK UNIFIED IDEOGRAPH:'EFFA:61434:&#x71FF
CJK UNIFIED IDEOGRAPH:'EFFB:61435:&#x71F9
CJK UNIFIED IDEOGRAPH:'EFFC:61436:&#x7203
CJK UNIFIED IDEOGRAPH:'EFFD:61437:&#x71FD
CJK UNIFIED IDEOGRAPH:'EFFE:61438:&#x7376
CJK UNIFIED IDEOGRAPH:'F040:61504:&#x74B8
CJK UNIFIED IDEOGRAPH:'F041:61505:&#x74C0
CJK UNIFIED IDEOGRAPH:'F042:61506:&#x74B5
CJK UNIFIED IDEOGRAPH:'F043:61507:&#x74C1
CJK UNIFIED IDEOGRAPH:'F044:61508:&#x74BE
CJK UNIFIED IDEOGRAPH:'F045:61509:&#x74B6
CJK UNIFIED IDEOGRAPH:'F046:61510:&#x74BB
CJK UNIFIED IDEOGRAPH:'F047:61511:&#x74C2
CJK UNIFIED IDEOGRAPH:'F048:61512:&#x7514
CJK UNIFIED IDEOGRAPH:'F049:61513:&#x7513
CJK UNIFIED IDEOGRAPH:'F04A:61514:&#x765C
CJK UNIFIED IDEOGRAPH:'F04B:61515:&#x7664
CJK UNIFIED IDEOGRAPH:'F04C:61516:&#x7659
CJK UNIFIED IDEOGRAPH:'F04D:61517:&#x7650
CJK UNIFIED IDEOGRAPH:'F04E:61518:&#x7653
CJK UNIFIED IDEOGRAPH:'F04F:61519:&#x7657
CJK UNIFIED IDEOGRAPH:'F050:61520:&#x765A
CJK UNIFIED IDEOGRAPH:'F051:61521:&#x76A6
CJK UNIFIED IDEOGRAPH:'F052:61522:&#x76BD
CJK UNIFIED IDEOGRAPH:'F053:61523:&#x76EC
CJK UNIFIED IDEOGRAPH:'F054:61524:&#x77C2
CJK UNIFIED IDEOGRAPH:'F055:61525:&#x77BA
CJK UNIFIED IDEOGRAPH:'F056:61526:&#x78FF
CJK UNIFIED IDEOGRAPH:'F057:61527:&#x790C
CJK UNIFIED IDEOGRAPH:'F058:61528:&#x7913
CJK UNIFIED IDEOGRAPH:'F059:61529:&#x7914
CJK UNIFIED IDEOGRAPH:'F05A:61530:&#x7909
CJK UNIFIED IDEOGRAPH:'F05B:61531:&#x7910
CJK UNIFIED IDEOGRAPH:'F05C:61532:&#x7912
CJK UNIFIED IDEOGRAPH:'F05D:61533:&#x7911
CJK UNIFIED IDEOGRAPH:'F05E:61534:&#x79AD
CJK UNIFIED IDEOGRAPH:'F05F:61535:&#x79AC
CJK UNIFIED IDEOGRAPH:'F060:61536:&#x7A5F
CJK UNIFIED IDEOGRAPH:'F061:61537:&#x7C1C
CJK UNIFIED IDEOGRAPH:'F062:61538:&#x7C29
CJK UNIFIED IDEOGRAPH:'F063:61539:&#x7C19
CJK UNIFIED IDEOGRAPH:'F064:61540:&#x7C20
CJK UNIFIED IDEOGRAPH:'F065:61541:&#x7C1F
CJK UNIFIED IDEOGRAPH:'F066:61542:&#x7C2D
CJK UNIFIED IDEOGRAPH:'F067:61543:&#x7C1D
CJK UNIFIED IDEOGRAPH:'F068:61544:&#x7C26
CJK UNIFIED IDEOGRAPH:'F069:61545:&#x7C28
CJK UNIFIED IDEOGRAPH:'F06A:61546:&#x7C22
CJK UNIFIED IDEOGRAPH:'F06B:61547:&#x7C25
CJK UNIFIED IDEOGRAPH:'F06C:61548:&#x7C30
CJK UNIFIED IDEOGRAPH:'F06D:61549:&#x7E5C
CJK UNIFIED IDEOGRAPH:'F06E:61550:&#x7E50
CJK UNIFIED IDEOGRAPH:'F06F:61551:&#x7E56
CJK UNIFIED IDEOGRAPH:'F070:61552:&#x7E63
CJK UNIFIED IDEOGRAPH:'F071:61553:&#x7E58
CJK UNIFIED IDEOGRAPH:'F072:61554:&#x7E62
CJK UNIFIED IDEOGRAPH:'F073:61555:&#x7E5F
CJK UNIFIED IDEOGRAPH:'F074:61556:&#x7E51
CJK UNIFIED IDEOGRAPH:'F075:61557:&#x7E60
CJK UNIFIED IDEOGRAPH:'F076:61558:&#x7E57
CJK UNIFIED IDEOGRAPH:'F077:61559:&#x7E53
CJK UNIFIED IDEOGRAPH:'F078:61560:&#x7FB5
CJK UNIFIED IDEOGRAPH:'F079:61561:&#x7FB3
CJK UNIFIED IDEOGRAPH:'F07A:61562:&#x7FF7
CJK UNIFIED IDEOGRAPH:'F07B:61563:&#x7FF8
CJK UNIFIED IDEOGRAPH:'F07C:61564:&#x8075
CJK UNIFIED IDEOGRAPH:'F07D:61565:&#x81D1
CJK UNIFIED IDEOGRAPH:'F07E:61566:&#x81D2
CJK UNIFIED IDEOGRAPH:'F0A1:61601:&#x81D0
CJK UNIFIED IDEOGRAPH:'F0A2:61602:&#x825F
CJK UNIFIED IDEOGRAPH:'F0A3:61603:&#x825E
CJK UNIFIED IDEOGRAPH:'F0A4:61604:&#x85B4
CJK UNIFIED IDEOGRAPH:'F0A5:61605:&#x85C6
CJK UNIFIED IDEOGRAPH:'F0A6:61606:&#x85C0
CJK UNIFIED IDEOGRAPH:'F0A7:61607:&#x85C3
CJK UNIFIED IDEOGRAPH:'F0A8:61608:&#x85C2
CJK UNIFIED IDEOGRAPH:'F0A9:61609:&#x85B3
CJK UNIFIED IDEOGRAPH:'F0AA:61610:&#x85B5
CJK UNIFIED IDEOGRAPH:'F0AB:61611:&#x85BD
CJK UNIFIED IDEOGRAPH:'F0AC:61612:&#x85C7
CJK UNIFIED IDEOGRAPH:'F0AD:61613:&#x85C4
CJK UNIFIED IDEOGRAPH:'F0AE:61614:&#x85BF
CJK UNIFIED IDEOGRAPH:'F0AF:61615:&#x85CB
CJK UNIFIED IDEOGRAPH:'F0B0:61616:&#x85CE
CJK UNIFIED IDEOGRAPH:'F0B1:61617:&#x85C8
CJK UNIFIED IDEOGRAPH:'F0B2:61618:&#x85C5
CJK UNIFIED IDEOGRAPH:'F0B3:61619:&#x85B1
CJK UNIFIED IDEOGRAPH:'F0B4:61620:&#x85B6
CJK UNIFIED IDEOGRAPH:'F0B5:61621:&#x85D2
CJK UNIFIED IDEOGRAPH:'F0B6:61622:&#x8624
CJK UNIFIED IDEOGRAPH:'F0B7:61623:&#x85B8
CJK UNIFIED IDEOGRAPH:'F0B8:61624:&#x85B7
CJK UNIFIED IDEOGRAPH:'F0B9:61625:&#x85BE
CJK UNIFIED IDEOGRAPH:'F0BA:61626:&#x8669
CJK UNIFIED IDEOGRAPH:'F0BB:61627:&#x87E7
CJK UNIFIED IDEOGRAPH:'F0BC:61628:&#x87E6
CJK UNIFIED IDEOGRAPH:'F0BD:61629:&#x87E2
CJK UNIFIED IDEOGRAPH:'F0BE:61630:&#x87DB
CJK UNIFIED IDEOGRAPH:'F0BF:61631:&#x87EB
CJK UNIFIED IDEOGRAPH:'F0C0:61632:&#x87EA
CJK UNIFIED IDEOGRAPH:'F0C1:61633:&#x87E5
CJK UNIFIED IDEOGRAPH:'F0C2:61634:&#x87DF
CJK UNIFIED IDEOGRAPH:'F0C3:61635:&#x87F3
CJK UNIFIED IDEOGRAPH:'F0C4:61636:&#x87E4
CJK UNIFIED IDEOGRAPH:'F0C5:61637:&#x87D4
CJK UNIFIED IDEOGRAPH:'F0C6:61638:&#x87DC
CJK UNIFIED IDEOGRAPH:'F0C7:61639:&#x87D3
CJK UNIFIED IDEOGRAPH:'F0C8:61640:&#x87ED
CJK UNIFIED IDEOGRAPH:'F0C9:61641:&#x87D8
CJK UNIFIED IDEOGRAPH:'F0CA:61642:&#x87E3
CJK UNIFIED IDEOGRAPH:'F0CB:61643:&#x87A4
CJK UNIFIED IDEOGRAPH:'F0CC:61644:&#x87D7
CJK UNIFIED IDEOGRAPH:'F0CD:61645:&#x87D9
CJK UNIFIED IDEOGRAPH:'F0CE:61646:&#x8801
CJK UNIFIED IDEOGRAPH:'F0CF:61647:&#x87F4
CJK UNIFIED IDEOGRAPH:'F0D0:61648:&#x87E8
CJK UNIFIED IDEOGRAPH:'F0D1:61649:&#x87DD
CJK UNIFIED IDEOGRAPH:'F0D2:61650:&#x8953
CJK UNIFIED IDEOGRAPH:'F0D3:61651:&#x894B
CJK UNIFIED IDEOGRAPH:'F0D4:61652:&#x894F
CJK UNIFIED IDEOGRAPH:'F0D5:61653:&#x894C
CJK UNIFIED IDEOGRAPH:'F0D6:61654:&#x8946
CJK UNIFIED IDEOGRAPH:'F0D7:61655:&#x8950
CJK UNIFIED IDEOGRAPH:'F0D8:61656:&#x8951
CJK UNIFIED IDEOGRAPH:'F0D9:61657:&#x8949
CJK UNIFIED IDEOGRAPH:'F0DA:61658:&#x8B2A
CJK UNIFIED IDEOGRAPH:'F0DB:61659:&#x8B27
CJK UNIFIED IDEOGRAPH:'F0DC:61660:&#x8B23
CJK UNIFIED IDEOGRAPH:'F0DD:61661:&#x8B33
CJK UNIFIED IDEOGRAPH:'F0DE:61662:&#x8B30
CJK UNIFIED IDEOGRAPH:'F0DF:61663:&#x8B35
CJK UNIFIED IDEOGRAPH:'F0E0:61664:&#x8B47
CJK UNIFIED IDEOGRAPH:'F0E1:61665:&#x8B2F
CJK UNIFIED IDEOGRAPH:'F0E2:61666:&#x8B3C
CJK UNIFIED IDEOGRAPH:'F0E3:61667:&#x8B3E
CJK UNIFIED IDEOGRAPH:'F0E4:61668:&#x8B31
CJK UNIFIED IDEOGRAPH:'F0E5:61669:&#x8B25
CJK UNIFIED IDEOGRAPH:'F0E6:61670:&#x8B37
CJK UNIFIED IDEOGRAPH:'F0E7:61671:&#x8B26
CJK UNIFIED IDEOGRAPH:'F0E8:61672:&#x8B36
CJK UNIFIED IDEOGRAPH:'F0E9:61673:&#x8B2E
CJK UNIFIED IDEOGRAPH:'F0EA:61674:&#x8B24
CJK UNIFIED IDEOGRAPH:'F0EB:61675:&#x8B3B
CJK UNIFIED IDEOGRAPH:'F0EC:61676:&#x8B3D
CJK UNIFIED IDEOGRAPH:'F0ED:61677:&#x8B3A
CJK UNIFIED IDEOGRAPH:'F0EE:61678:&#x8C42
CJK UNIFIED IDEOGRAPH:'F0EF:61679:&#x8C75
CJK UNIFIED IDEOGRAPH:'F0F0:61680:&#x8C99
CJK UNIFIED IDEOGRAPH:'F0F1:61681:&#x8C98
CJK UNIFIED IDEOGRAPH:'F0F2:61682:&#x8C97
CJK UNIFIED IDEOGRAPH:'F0F3:61683:&#x8CFE
CJK UNIFIED IDEOGRAPH:'F0F4:61684:&#x8D04
CJK UNIFIED IDEOGRAPH:'F0F5:61685:&#x8D02
CJK UNIFIED IDEOGRAPH:'F0F6:61686:&#x8D00
CJK UNIFIED IDEOGRAPH:'F0F7:61687:&#x8E5C
CJK UNIFIED IDEOGRAPH:'F0F8:61688:&#x8E62
CJK UNIFIED IDEOGRAPH:'F0F9:61689:&#x8E60
CJK UNIFIED IDEOGRAPH:'F0FA:61690:&#x8E57
CJK UNIFIED IDEOGRAPH:'F0FB:61691:&#x8E56
CJK UNIFIED IDEOGRAPH:'F0FC:61692:&#x8E5E
CJK UNIFIED IDEOGRAPH:'F0FD:61693:&#x8E65
CJK UNIFIED IDEOGRAPH:'F0FE:61694:&#x8E67
CJK UNIFIED IDEOGRAPH:'F140:61760:&#x8E5B
CJK UNIFIED IDEOGRAPH:'F141:61761:&#x8E5A
CJK UNIFIED IDEOGRAPH:'F142:61762:&#x8E61
CJK UNIFIED IDEOGRAPH:'F143:61763:&#x8E5D
CJK UNIFIED IDEOGRAPH:'F144:61764:&#x8E69
CJK UNIFIED IDEOGRAPH:'F145:61765:&#x8E54
CJK UNIFIED IDEOGRAPH:'F146:61766:&#x8F46
CJK UNIFIED IDEOGRAPH:'F147:61767:&#x8F47
CJK UNIFIED IDEOGRAPH:'F148:61768:&#x8F48
CJK UNIFIED IDEOGRAPH:'F149:61769:&#x8F4B
CJK UNIFIED IDEOGRAPH:'F14A:61770:&#x9128
CJK UNIFIED IDEOGRAPH:'F14B:61771:&#x913A
CJK UNIFIED IDEOGRAPH:'F14C:61772:&#x913B
CJK UNIFIED IDEOGRAPH:'F14D:61773:&#x913E
CJK UNIFIED IDEOGRAPH:'F14E:61774:&#x91A8
CJK UNIFIED IDEOGRAPH:'F14F:61775:&#x91A5
CJK UNIFIED IDEOGRAPH:'F150:61776:&#x91A7
CJK UNIFIED IDEOGRAPH:'F151:61777:&#x91AF
CJK UNIFIED IDEOGRAPH:'F152:61778:&#x91AA
CJK UNIFIED IDEOGRAPH:'F153:61779:&#x93B5
CJK UNIFIED IDEOGRAPH:'F154:61780:&#x938C
CJK UNIFIED IDEOGRAPH:'F155:61781:&#x9392
CJK UNIFIED IDEOGRAPH:'F156:61782:&#x93B7
CJK UNIFIED IDEOGRAPH:'F157:61783:&#x939B
CJK UNIFIED IDEOGRAPH:'F158:61784:&#x939D
CJK UNIFIED IDEOGRAPH:'F159:61785:&#x9389
CJK UNIFIED IDEOGRAPH:'F15A:61786:&#x93A7
CJK UNIFIED IDEOGRAPH:'F15B:61787:&#x938E
CJK UNIFIED IDEOGRAPH:'F15C:61788:&#x93AA
CJK UNIFIED IDEOGRAPH:'F15D:61789:&#x939E
CJK UNIFIED IDEOGRAPH:'F15E:61790:&#x93A6
CJK UNIFIED IDEOGRAPH:'F15F:61791:&#x9395
CJK UNIFIED IDEOGRAPH:'F160:61792:&#x9388
CJK UNIFIED IDEOGRAPH:'F161:61793:&#x9399
CJK UNIFIED IDEOGRAPH:'F162:61794:&#x939F
CJK UNIFIED IDEOGRAPH:'F163:61795:&#x938D
CJK UNIFIED IDEOGRAPH:'F164:61796:&#x93B1
CJK UNIFIED IDEOGRAPH:'F165:61797:&#x9391
CJK UNIFIED IDEOGRAPH:'F166:61798:&#x93B2
CJK UNIFIED IDEOGRAPH:'F167:61799:&#x93A4
CJK UNIFIED IDEOGRAPH:'F168:61800:&#x93A8
CJK UNIFIED IDEOGRAPH:'F169:61801:&#x93B4
CJK UNIFIED IDEOGRAPH:'F16A:61802:&#x93A3
CJK UNIFIED IDEOGRAPH:'F16B:61803:&#x93A5
CJK UNIFIED IDEOGRAPH:'F16C:61804:&#x95D2
CJK UNIFIED IDEOGRAPH:'F16D:61805:&#x95D3
CJK UNIFIED IDEOGRAPH:'F16E:61806:&#x95D1
CJK UNIFIED IDEOGRAPH:'F16F:61807:&#x96B3
CJK UNIFIED IDEOGRAPH:'F170:61808:&#x96D7
CJK UNIFIED IDEOGRAPH:'F171:61809:&#x96DA
CJK UNIFIED IDEOGRAPH:'F172:61810:&#x5DC2
CJK UNIFIED IDEOGRAPH:'F173:61811:&#x96DF
CJK UNIFIED IDEOGRAPH:'F174:61812:&#x96D8
CJK UNIFIED IDEOGRAPH:'F175:61813:&#x96DD
CJK UNIFIED IDEOGRAPH:'F176:61814:&#x9723
CJK UNIFIED IDEOGRAPH:'F177:61815:&#x9722
CJK UNIFIED IDEOGRAPH:'F178:61816:&#x9725
CJK UNIFIED IDEOGRAPH:'F179:61817:&#x97AC
CJK UNIFIED IDEOGRAPH:'F17A:61818:&#x97AE
CJK UNIFIED IDEOGRAPH:'F17B:61819:&#x97A8
CJK UNIFIED IDEOGRAPH:'F17C:61820:&#x97AB
CJK UNIFIED IDEOGRAPH:'F17D:61821:&#x97A4
CJK UNIFIED IDEOGRAPH:'F17E:61822:&#x97AA
CJK UNIFIED IDEOGRAPH:'F1A1:61857:&#x97A2
CJK UNIFIED IDEOGRAPH:'F1A2:61858:&#x97A5
CJK UNIFIED IDEOGRAPH:'F1A3:61859:&#x97D7
CJK UNIFIED IDEOGRAPH:'F1A4:61860:&#x97D9
CJK UNIFIED IDEOGRAPH:'F1A5:61861:&#x97D6
CJK UNIFIED IDEOGRAPH:'F1A6:61862:&#x97D8
CJK UNIFIED IDEOGRAPH:'F1A7:61863:&#x97FA
CJK UNIFIED IDEOGRAPH:'F1A8:61864:&#x9850
CJK UNIFIED IDEOGRAPH:'F1A9:61865:&#x9851
CJK UNIFIED IDEOGRAPH:'F1AA:61866:&#x9852
CJK UNIFIED IDEOGRAPH:'F1AB:61867:&#x98B8
CJK UNIFIED IDEOGRAPH:'F1AC:61868:&#x9941
CJK UNIFIED IDEOGRAPH:'F1AD:61869:&#x993C
CJK UNIFIED IDEOGRAPH:'F1AE:61870:&#x993A
CJK UNIFIED IDEOGRAPH:'F1AF:61871:&#x9A0F
CJK UNIFIED IDEOGRAPH:'F1B0:61872:&#x9A0B
CJK UNIFIED IDEOGRAPH:'F1B1:61873:&#x9A09
CJK UNIFIED IDEOGRAPH:'F1B2:61874:&#x9A0D
CJK UNIFIED IDEOGRAPH:'F1B3:61875:&#x9A04
CJK UNIFIED IDEOGRAPH:'F1B4:61876:&#x9A11
CJK UNIFIED IDEOGRAPH:'F1B5:61877:&#x9A0A
CJK UNIFIED IDEOGRAPH:'F1B6:61878:&#x9A05
CJK UNIFIED IDEOGRAPH:'F1B7:61879:&#x9A07
CJK UNIFIED IDEOGRAPH:'F1B8:61880:&#x9A06
CJK UNIFIED IDEOGRAPH:'F1B9:61881:&#x9AC0
CJK UNIFIED IDEOGRAPH:'F1BA:61882:&#x9ADC
CJK UNIFIED IDEOGRAPH:'F1BB:61883:&#x9B08
CJK UNIFIED IDEOGRAPH:'F1BC:61884:&#x9B04
CJK UNIFIED IDEOGRAPH:'F1BD:61885:&#x9B05
CJK UNIFIED IDEOGRAPH:'F1BE:61886:&#x9B29
CJK UNIFIED IDEOGRAPH:'F1BF:61887:&#x9B35
CJK UNIFIED IDEOGRAPH:'F1C0:61888:&#x9B4A
CJK UNIFIED IDEOGRAPH:'F1C1:61889:&#x9B4C
CJK UNIFIED IDEOGRAPH:'F1C2:61890:&#x9B4B
CJK UNIFIED IDEOGRAPH:'F1C3:61891:&#x9BC7
CJK UNIFIED IDEOGRAPH:'F1C4:61892:&#x9BC6
CJK UNIFIED IDEOGRAPH:'F1C5:61893:&#x9BC3
CJK UNIFIED IDEOGRAPH:'F1C6:61894:&#x9BBF
CJK UNIFIED IDEOGRAPH:'F1C7:61895:&#x9BC1
CJK UNIFIED IDEOGRAPH:'F1C8:61896:&#x9BB5
CJK UNIFIED IDEOGRAPH:'F1C9:61897:&#x9BB8
CJK UNIFIED IDEOGRAPH:'F1CA:61898:&#x9BD3
CJK UNIFIED IDEOGRAPH:'F1CB:61899:&#x9BB6
CJK UNIFIED IDEOGRAPH:'F1CC:61900:&#x9BC4
CJK UNIFIED IDEOGRAPH:'F1CD:61901:&#x9BB9
CJK UNIFIED IDEOGRAPH:'F1CE:61902:&#x9BBD
CJK UNIFIED IDEOGRAPH:'F1CF:61903:&#x9D5C
CJK UNIFIED IDEOGRAPH:'F1D0:61904:&#x9D53
CJK UNIFIED IDEOGRAPH:'F1D1:61905:&#x9D4F
CJK UNIFIED IDEOGRAPH:'F1D2:61906:&#x9D4A
CJK UNIFIED IDEOGRAPH:'F1D3:61907:&#x9D5B
CJK UNIFIED IDEOGRAPH:'F1D4:61908:&#x9D4B
CJK UNIFIED IDEOGRAPH:'F1D5:61909:&#x9D59
CJK UNIFIED IDEOGRAPH:'F1D6:61910:&#x9D56
CJK UNIFIED IDEOGRAPH:'F1D7:61911:&#x9D4C
CJK UNIFIED IDEOGRAPH:'F1D8:61912:&#x9D57
CJK UNIFIED IDEOGRAPH:'F1D9:61913:&#x9D52
CJK UNIFIED IDEOGRAPH:'F1DA:61914:&#x9D54
CJK UNIFIED IDEOGRAPH:'F1DB:61915:&#x9D5F
CJK UNIFIED IDEOGRAPH:'F1DC:61916:&#x9D58
CJK UNIFIED IDEOGRAPH:'F1DD:61917:&#x9D5A
CJK UNIFIED IDEOGRAPH:'F1DE:61918:&#x9E8E
CJK UNIFIED IDEOGRAPH:'F1DF:61919:&#x9E8C
CJK UNIFIED IDEOGRAPH:'F1E0:61920:&#x9EDF
CJK UNIFIED IDEOGRAPH:'F1E1:61921:&#x9F01
CJK UNIFIED IDEOGRAPH:'F1E2:61922:&#x9F00
CJK UNIFIED IDEOGRAPH:'F1E3:61923:&#x9F16
CJK UNIFIED IDEOGRAPH:'F1E4:61924:&#x9F25
CJK UNIFIED IDEOGRAPH:'F1E5:61925:&#x9F2B
CJK UNIFIED IDEOGRAPH:'F1E6:61926:&#x9F2A
CJK UNIFIED IDEOGRAPH:'F1E7:61927:&#x9F29
CJK UNIFIED IDEOGRAPH:'F1E8:61928:&#x9F28
CJK UNIFIED IDEOGRAPH:'F1E9:61929:&#x9F4C
CJK UNIFIED IDEOGRAPH:'F1EA:61930:&#x9F55
CJK UNIFIED IDEOGRAPH:'F1EB:61931:&#x5134
CJK UNIFIED IDEOGRAPH:'F1EC:61932:&#x5135
CJK UNIFIED IDEOGRAPH:'F1ED:61933:&#x5296
CJK UNIFIED IDEOGRAPH:'F1EE:61934:&#x52F7
CJK UNIFIED IDEOGRAPH:'F1EF:61935:&#x53B4
CJK UNIFIED IDEOGRAPH:'F1F0:61936:&#x56AB
CJK UNIFIED IDEOGRAPH:'F1F1:61937:&#x56AD
CJK UNIFIED IDEOGRAPH:'F1F2:61938:&#x56A6
CJK UNIFIED IDEOGRAPH:'F1F3:61939:&#x56A7
CJK UNIFIED IDEOGRAPH:'F1F4:61940:&#x56AA
CJK UNIFIED IDEOGRAPH:'F1F5:61941:&#x56AC
CJK UNIFIED IDEOGRAPH:'F1F6:61942:&#x58DA
CJK UNIFIED IDEOGRAPH:'F1F7:61943:&#x58DD
CJK UNIFIED IDEOGRAPH:'F1F8:61944:&#x58DB
CJK UNIFIED IDEOGRAPH:'F1F9:61945:&#x5912
CJK UNIFIED IDEOGRAPH:'F1FA:61946:&#x5B3D
CJK UNIFIED IDEOGRAPH:'F1FB:61947:&#x5B3E
CJK UNIFIED IDEOGRAPH:'F1FC:61948:&#x5B3F
CJK UNIFIED IDEOGRAPH:'F1FD:61949:&#x5DC3
CJK UNIFIED IDEOGRAPH:'F1FE:61950:&#x5E70
CJK UNIFIED IDEOGRAPH:'F240:62016:&#x5FBF
CJK UNIFIED IDEOGRAPH:'F241:62017:&#x61FB
CJK UNIFIED IDEOGRAPH:'F242:62018:&#x6507
CJK UNIFIED IDEOGRAPH:'F243:62019:&#x6510
CJK UNIFIED IDEOGRAPH:'F244:62020:&#x650D
CJK UNIFIED IDEOGRAPH:'F245:62021:&#x6509
CJK UNIFIED IDEOGRAPH:'F246:62022:&#x650C
CJK UNIFIED IDEOGRAPH:'F247:62023:&#x650E
CJK UNIFIED IDEOGRAPH:'F248:62024:&#x6584
CJK UNIFIED IDEOGRAPH:'F249:62025:&#x65DE
CJK UNIFIED IDEOGRAPH:'F24A:62026:&#x65DD
CJK UNIFIED IDEOGRAPH:'F24B:62027:&#x66DE
CJK UNIFIED IDEOGRAPH:'F24C:62028:&#x6AE7
CJK UNIFIED IDEOGRAPH:'F24D:62029:&#x6AE0
CJK UNIFIED IDEOGRAPH:'F24E:62030:&#x6ACC
CJK UNIFIED IDEOGRAPH:'F24F:62031:&#x6AD1
CJK UNIFIED IDEOGRAPH:'F250:62032:&#x6AD9
CJK UNIFIED IDEOGRAPH:'F251:62033:&#x6ACB
CJK UNIFIED IDEOGRAPH:'F252:62034:&#x6ADF
CJK UNIFIED IDEOGRAPH:'F253:62035:&#x6ADC
CJK UNIFIED IDEOGRAPH:'F254:62036:&#x6AD0
CJK UNIFIED IDEOGRAPH:'F255:62037:&#x6AEB
CJK UNIFIED IDEOGRAPH:'F256:62038:&#x6ACF
CJK UNIFIED IDEOGRAPH:'F257:62039:&#x6ACD
CJK UNIFIED IDEOGRAPH:'F258:62040:&#x6ADE
CJK UNIFIED IDEOGRAPH:'F259:62041:&#x6B60
CJK UNIFIED IDEOGRAPH:'F25A:62042:&#x6BB0
CJK UNIFIED IDEOGRAPH:'F25B:62043:&#x6C0C
CJK UNIFIED IDEOGRAPH:'F25C:62044:&#x7019
CJK UNIFIED IDEOGRAPH:'F25D:62045:&#x7027
CJK UNIFIED IDEOGRAPH:'F25E:62046:&#x7020
CJK UNIFIED IDEOGRAPH:'F25F:62047:&#x7016
CJK UNIFIED IDEOGRAPH:'F260:62048:&#x702B
CJK UNIFIED IDEOGRAPH:'F261:62049:&#x7021
CJK UNIFIED IDEOGRAPH:'F262:62050:&#x7022
CJK UNIFIED IDEOGRAPH:'F263:62051:&#x7023
CJK UNIFIED IDEOGRAPH:'F264:62052:&#x7029
CJK UNIFIED IDEOGRAPH:'F265:62053:&#x7017
CJK UNIFIED IDEOGRAPH:'F266:62054:&#x7024
CJK UNIFIED IDEOGRAPH:'F267:62055:&#x701C
CJK UNIFIED IDEOGRAPH:'F268:62056:&#x702A
CJK UNIFIED IDEOGRAPH:'F269:62057:&#x720C
CJK UNIFIED IDEOGRAPH:'F26A:62058:&#x720A
CJK UNIFIED IDEOGRAPH:'F26B:62059:&#x7207
CJK UNIFIED IDEOGRAPH:'F26C:62060:&#x7202
CJK UNIFIED IDEOGRAPH:'F26D:62061:&#x7205
CJK UNIFIED IDEOGRAPH:'F26E:62062:&#x72A5
CJK UNIFIED IDEOGRAPH:'F26F:62063:&#x72A6
CJK UNIFIED IDEOGRAPH:'F270:62064:&#x72A4
CJK UNIFIED IDEOGRAPH:'F271:62065:&#x72A3
CJK UNIFIED IDEOGRAPH:'F272:62066:&#x72A1
CJK UNIFIED IDEOGRAPH:'F273:62067:&#x74CB
CJK UNIFIED IDEOGRAPH:'F274:62068:&#x74C5
CJK UNIFIED IDEOGRAPH:'F275:62069:&#x74B7
CJK UNIFIED IDEOGRAPH:'F276:62070:&#x74C3
CJK UNIFIED IDEOGRAPH:'F277:62071:&#x7516
CJK UNIFIED IDEOGRAPH:'F278:62072:&#x7660
CJK UNIFIED IDEOGRAPH:'F279:62073:&#x77C9
CJK UNIFIED IDEOGRAPH:'F27A:62074:&#x77CA
CJK UNIFIED IDEOGRAPH:'F27B:62075:&#x77C4
CJK UNIFIED IDEOGRAPH:'F27C:62076:&#x77F1
CJK UNIFIED IDEOGRAPH:'F27D:62077:&#x791D
CJK UNIFIED IDEOGRAPH:'F27E:62078:&#x791B
CJK UNIFIED IDEOGRAPH:'F2A1:62113:&#x7921
CJK UNIFIED IDEOGRAPH:'F2A2:62114:&#x791C
CJK UNIFIED IDEOGRAPH:'F2A3:62115:&#x7917
CJK UNIFIED IDEOGRAPH:'F2A4:62116:&#x791E
CJK UNIFIED IDEOGRAPH:'F2A5:62117:&#x79B0
CJK UNIFIED IDEOGRAPH:'F2A6:62118:&#x7A67
CJK UNIFIED IDEOGRAPH:'F2A7:62119:&#x7A68
CJK UNIFIED IDEOGRAPH:'F2A8:62120:&#x7C33
CJK UNIFIED IDEOGRAPH:'F2A9:62121:&#x7C3C
CJK UNIFIED IDEOGRAPH:'F2AA:62122:&#x7C39
CJK UNIFIED IDEOGRAPH:'F2AB:62123:&#x7C2C
CJK UNIFIED IDEOGRAPH:'F2AC:62124:&#x7C3B
CJK UNIFIED IDEOGRAPH:'F2AD:62125:&#x7CEC
CJK UNIFIED IDEOGRAPH:'F2AE:62126:&#x7CEA
CJK UNIFIED IDEOGRAPH:'F2AF:62127:&#x7E76
CJK UNIFIED IDEOGRAPH:'F2B0:62128:&#x7E75
CJK UNIFIED IDEOGRAPH:'F2B1:62129:&#x7E78
CJK UNIFIED IDEOGRAPH:'F2B2:62130:&#x7E70
CJK UNIFIED IDEOGRAPH:'F2B3:62131:&#x7E77
CJK UNIFIED IDEOGRAPH:'F2B4:62132:&#x7E6F
CJK UNIFIED IDEOGRAPH:'F2B5:62133:&#x7E7A
CJK UNIFIED IDEOGRAPH:'F2B6:62134:&#x7E72
CJK UNIFIED IDEOGRAPH:'F2B7:62135:&#x7E74
CJK UNIFIED IDEOGRAPH:'F2B8:62136:&#x7E68
CJK UNIFIED IDEOGRAPH:'F2B9:62137:&#x7F4B
CJK UNIFIED IDEOGRAPH:'F2BA:62138:&#x7F4A
CJK UNIFIED IDEOGRAPH:'F2BB:62139:&#x7F83
CJK UNIFIED IDEOGRAPH:'F2BC:62140:&#x7F86
CJK UNIFIED IDEOGRAPH:'F2BD:62141:&#x7FB7
CJK UNIFIED IDEOGRAPH:'F2BE:62142:&#x7FFD
CJK UNIFIED IDEOGRAPH:'F2BF:62143:&#x7FFE
CJK UNIFIED IDEOGRAPH:'F2C0:62144:&#x8078
CJK UNIFIED IDEOGRAPH:'F2C1:62145:&#x81D7
CJK UNIFIED IDEOGRAPH:'F2C2:62146:&#x81D5
CJK UNIFIED IDEOGRAPH:'F2C3:62147:&#x8264
CJK UNIFIED IDEOGRAPH:'F2C4:62148:&#x8261
CJK UNIFIED IDEOGRAPH:'F2C5:62149:&#x8263
CJK UNIFIED IDEOGRAPH:'F2C6:62150:&#x85EB
CJK UNIFIED IDEOGRAPH:'F2C7:62151:&#x85F1
CJK UNIFIED IDEOGRAPH:'F2C8:62152:&#x85ED
CJK UNIFIED IDEOGRAPH:'F2C9:62153:&#x85D9
CJK UNIFIED IDEOGRAPH:'F2CA:62154:&#x85E1
CJK UNIFIED IDEOGRAPH:'F2CB:62155:&#x85E8
CJK UNIFIED IDEOGRAPH:'F2CC:62156:&#x85DA
CJK UNIFIED IDEOGRAPH:'F2CD:62157:&#x85D7
CJK UNIFIED IDEOGRAPH:'F2CE:62158:&#x85EC
CJK UNIFIED IDEOGRAPH:'F2CF:62159:&#x85F2
CJK UNIFIED IDEOGRAPH:'F2D0:62160:&#x85F8
CJK UNIFIED IDEOGRAPH:'F2D1:62161:&#x85D8
CJK UNIFIED IDEOGRAPH:'F2D2:62162:&#x85DF
CJK UNIFIED IDEOGRAPH:'F2D3:62163:&#x85E3
CJK UNIFIED IDEOGRAPH:'F2D4:62164:&#x85DC
CJK UNIFIED IDEOGRAPH:'F2D5:62165:&#x85D1
CJK UNIFIED IDEOGRAPH:'F2D6:62166:&#x85F0
CJK UNIFIED IDEOGRAPH:'F2D7:62167:&#x85E6
CJK UNIFIED IDEOGRAPH:'F2D8:62168:&#x85EF
CJK UNIFIED IDEOGRAPH:'F2D9:62169:&#x85DE
CJK UNIFIED IDEOGRAPH:'F2DA:62170:&#x85E2
CJK UNIFIED IDEOGRAPH:'F2DB:62171:&#x8800
CJK UNIFIED IDEOGRAPH:'F2DC:62172:&#x87FA
CJK UNIFIED IDEOGRAPH:'F2DD:62173:&#x8803
CJK UNIFIED IDEOGRAPH:'F2DE:62174:&#x87F6
CJK UNIFIED IDEOGRAPH:'F2DF:62175:&#x87F7
CJK UNIFIED IDEOGRAPH:'F2E0:62176:&#x8809
CJK UNIFIED IDEOGRAPH:'F2E1:62177:&#x880C
CJK UNIFIED IDEOGRAPH:'F2E2:62178:&#x880B
CJK UNIFIED IDEOGRAPH:'F2E3:62179:&#x8806
CJK UNIFIED IDEOGRAPH:'F2E4:62180:&#x87FC
CJK UNIFIED IDEOGRAPH:'F2E5:62181:&#x8808
CJK UNIFIED IDEOGRAPH:'F2E6:62182:&#x87FF
CJK UNIFIED IDEOGRAPH:'F2E7:62183:&#x880A
CJK UNIFIED IDEOGRAPH:'F2E8:62184:&#x8802
CJK UNIFIED IDEOGRAPH:'F2E9:62185:&#x8962
CJK UNIFIED IDEOGRAPH:'F2EA:62186:&#x895A
CJK UNIFIED IDEOGRAPH:'F2EB:62187:&#x895B
CJK UNIFIED IDEOGRAPH:'F2EC:62188:&#x8957
CJK UNIFIED IDEOGRAPH:'F2ED:62189:&#x8961
CJK UNIFIED IDEOGRAPH:'F2EE:62190:&#x895C
CJK UNIFIED IDEOGRAPH:'F2EF:62191:&#x8958
CJK UNIFIED IDEOGRAPH:'F2F0:62192:&#x895D
CJK UNIFIED IDEOGRAPH:'F2F1:62193:&#x8959
CJK UNIFIED IDEOGRAPH:'F2F2:62194:&#x8988
CJK UNIFIED IDEOGRAPH:'F2F3:62195:&#x89B7
CJK UNIFIED IDEOGRAPH:'F2F4:62196:&#x89B6
CJK UNIFIED IDEOGRAPH:'F2F5:62197:&#x89F6
CJK UNIFIED IDEOGRAPH:'F2F6:62198:&#x8B50
CJK UNIFIED IDEOGRAPH:'F2F7:62199:&#x8B48
CJK UNIFIED IDEOGRAPH:'F2F8:62200:&#x8B4A
CJK UNIFIED IDEOGRAPH:'F2F9:62201:&#x8B40
CJK UNIFIED IDEOGRAPH:'F2FA:62202:&#x8B53
CJK UNIFIED IDEOGRAPH:'F2FB:62203:&#x8B56
CJK UNIFIED IDEOGRAPH:'F2FC:62204:&#x8B54
CJK UNIFIED IDEOGRAPH:'F2FD:62205:&#x8B4B
CJK UNIFIED IDEOGRAPH:'F2FE:62206:&#x8B55
CJK UNIFIED IDEOGRAPH:'F340:62272:&#x8B51
CJK UNIFIED IDEOGRAPH:'F341:62273:&#x8B42
CJK UNIFIED IDEOGRAPH:'F342:62274:&#x8B52
CJK UNIFIED IDEOGRAPH:'F343:62275:&#x8B57
CJK UNIFIED IDEOGRAPH:'F344:62276:&#x8C43
CJK UNIFIED IDEOGRAPH:'F345:62277:&#x8C77
CJK UNIFIED IDEOGRAPH:'F346:62278:&#x8C76
CJK UNIFIED IDEOGRAPH:'F347:62279:&#x8C9A
CJK UNIFIED IDEOGRAPH:'F348:62280:&#x8D06
CJK UNIFIED IDEOGRAPH:'F349:62281:&#x8D07
CJK UNIFIED IDEOGRAPH:'F34A:62282:&#x8D09
CJK UNIFIED IDEOGRAPH:'F34B:62283:&#x8DAC
CJK UNIFIED IDEOGRAPH:'F34C:62284:&#x8DAA
CJK UNIFIED IDEOGRAPH:'F34D:62285:&#x8DAD
CJK UNIFIED IDEOGRAPH:'F34E:62286:&#x8DAB
CJK UNIFIED IDEOGRAPH:'F34F:62287:&#x8E6D
CJK UNIFIED IDEOGRAPH:'F350:62288:&#x8E78
CJK UNIFIED IDEOGRAPH:'F351:62289:&#x8E73
CJK UNIFIED IDEOGRAPH:'F352:62290:&#x8E6A
CJK UNIFIED IDEOGRAPH:'F353:62291:&#x8E6F
CJK UNIFIED IDEOGRAPH:'F354:62292:&#x8E7B
CJK UNIFIED IDEOGRAPH:'F355:62293:&#x8EC2
CJK UNIFIED IDEOGRAPH:'F356:62294:&#x8F52
CJK UNIFIED IDEOGRAPH:'F357:62295:&#x8F51
CJK UNIFIED IDEOGRAPH:'F358:62296:&#x8F4F
CJK UNIFIED IDEOGRAPH:'F359:62297:&#x8F50
CJK UNIFIED IDEOGRAPH:'F35A:62298:&#x8F53
CJK UNIFIED IDEOGRAPH:'F35B:62299:&#x8FB4
CJK UNIFIED IDEOGRAPH:'F35C:62300:&#x9140
CJK UNIFIED IDEOGRAPH:'F35D:62301:&#x913F
CJK UNIFIED IDEOGRAPH:'F35E:62302:&#x91B0
CJK UNIFIED IDEOGRAPH:'F35F:62303:&#x91AD
CJK UNIFIED IDEOGRAPH:'F360:62304:&#x93DE
CJK UNIFIED IDEOGRAPH:'F361:62305:&#x93C7
CJK UNIFIED IDEOGRAPH:'F362:62306:&#x93CF
CJK UNIFIED IDEOGRAPH:'F363:62307:&#x93C2
CJK UNIFIED IDEOGRAPH:'F364:62308:&#x93DA
CJK UNIFIED IDEOGRAPH:'F365:62309:&#x93D0
CJK UNIFIED IDEOGRAPH:'F366:62310:&#x93F9
CJK UNIFIED IDEOGRAPH:'F367:62311:&#x93EC
CJK UNIFIED IDEOGRAPH:'F368:62312:&#x93CC
CJK UNIFIED IDEOGRAPH:'F369:62313:&#x93D9
CJK UNIFIED IDEOGRAPH:'F36A:62314:&#x93A9
CJK UNIFIED IDEOGRAPH:'F36B:62315:&#x93E6
CJK UNIFIED IDEOGRAPH:'F36C:62316:&#x93CA
CJK UNIFIED IDEOGRAPH:'F36D:62317:&#x93D4
CJK UNIFIED IDEOGRAPH:'F36E:62318:&#x93EE
CJK UNIFIED IDEOGRAPH:'F36F:62319:&#x93E3
CJK UNIFIED IDEOGRAPH:'F370:62320:&#x93D5
CJK UNIFIED IDEOGRAPH:'F371:62321:&#x93C4
CJK UNIFIED IDEOGRAPH:'F372:62322:&#x93CE
CJK UNIFIED IDEOGRAPH:'F373:62323:&#x93C0
CJK UNIFIED IDEOGRAPH:'F374:62324:&#x93D2
CJK UNIFIED IDEOGRAPH:'F375:62325:&#x93E7
CJK UNIFIED IDEOGRAPH:'F376:62326:&#x957D
CJK UNIFIED IDEOGRAPH:'F377:62327:&#x95DA
CJK UNIFIED IDEOGRAPH:'F378:62328:&#x95DB
CJK UNIFIED IDEOGRAPH:'F379:62329:&#x96E1
CJK UNIFIED IDEOGRAPH:'F37A:62330:&#x9729
CJK UNIFIED IDEOGRAPH:'F37B:62331:&#x972B
CJK UNIFIED IDEOGRAPH:'F37C:62332:&#x972C
CJK UNIFIED IDEOGRAPH:'F37D:62333:&#x9728
CJK UNIFIED IDEOGRAPH:'F37E:62334:&#x9726
CJK UNIFIED IDEOGRAPH:'F3A1:62369:&#x97B3
CJK UNIFIED IDEOGRAPH:'F3A2:62370:&#x97B7
CJK UNIFIED IDEOGRAPH:'F3A3:62371:&#x97B6
CJK UNIFIED IDEOGRAPH:'F3A4:62372:&#x97DD
CJK UNIFIED IDEOGRAPH:'F3A5:62373:&#x97DE
CJK UNIFIED IDEOGRAPH:'F3A6:62374:&#x97DF
CJK UNIFIED IDEOGRAPH:'F3A7:62375:&#x985C
CJK UNIFIED IDEOGRAPH:'F3A8:62376:&#x9859
CJK UNIFIED IDEOGRAPH:'F3A9:62377:&#x985D
CJK UNIFIED IDEOGRAPH:'F3AA:62378:&#x9857
CJK UNIFIED IDEOGRAPH:'F3AB:62379:&#x98BF
CJK UNIFIED IDEOGRAPH:'F3AC:62380:&#x98BD
CJK UNIFIED IDEOGRAPH:'F3AD:62381:&#x98BB
CJK UNIFIED IDEOGRAPH:'F3AE:62382:&#x98BE
CJK UNIFIED IDEOGRAPH:'F3AF:62383:&#x9948
CJK UNIFIED IDEOGRAPH:'F3B0:62384:&#x9947
CJK UNIFIED IDEOGRAPH:'F3B1:62385:&#x9943
CJK UNIFIED IDEOGRAPH:'F3B2:62386:&#x99A6
CJK UNIFIED IDEOGRAPH:'F3B3:62387:&#x99A7
CJK UNIFIED IDEOGRAPH:'F3B4:62388:&#x9A1A
CJK UNIFIED IDEOGRAPH:'F3B5:62389:&#x9A15
CJK UNIFIED IDEOGRAPH:'F3B6:62390:&#x9A25
CJK UNIFIED IDEOGRAPH:'F3B7:62391:&#x9A1D
CJK UNIFIED IDEOGRAPH:'F3B8:62392:&#x9A24
CJK UNIFIED IDEOGRAPH:'F3B9:62393:&#x9A1B
CJK UNIFIED IDEOGRAPH:'F3BA:62394:&#x9A22
CJK UNIFIED IDEOGRAPH:'F3BB:62395:&#x9A20
CJK UNIFIED IDEOGRAPH:'F3BC:62396:&#x9A27
CJK UNIFIED IDEOGRAPH:'F3BD:62397:&#x9A23
CJK UNIFIED IDEOGRAPH:'F3BE:62398:&#x9A1E
CJK UNIFIED IDEOGRAPH:'F3BF:62399:&#x9A1C
CJK UNIFIED IDEOGRAPH:'F3C0:62400:&#x9A14
CJK UNIFIED IDEOGRAPH:'F3C1:62401:&#x9AC2
CJK UNIFIED IDEOGRAPH:'F3C2:62402:&#x9B0B
CJK UNIFIED IDEOGRAPH:'F3C3:62403:&#x9B0A
CJK UNIFIED IDEOGRAPH:'F3C4:62404:&#x9B0E
CJK UNIFIED IDEOGRAPH:'F3C5:62405:&#x9B0C
CJK UNIFIED IDEOGRAPH:'F3C6:62406:&#x9B37
CJK UNIFIED IDEOGRAPH:'F3C7:62407:&#x9BEA
CJK UNIFIED IDEOGRAPH:'F3C8:62408:&#x9BEB
CJK UNIFIED IDEOGRAPH:'F3C9:62409:&#x9BE0
CJK UNIFIED IDEOGRAPH:'F3CA:62410:&#x9BDE
CJK UNIFIED IDEOGRAPH:'F3CB:62411:&#x9BE4
CJK UNIFIED IDEOGRAPH:'F3CC:62412:&#x9BE6
CJK UNIFIED IDEOGRAPH:'F3CD:62413:&#x9BE2
CJK UNIFIED IDEOGRAPH:'F3CE:62414:&#x9BF0
CJK UNIFIED IDEOGRAPH:'F3CF:62415:&#x9BD4
CJK UNIFIED IDEOGRAPH:'F3D0:62416:&#x9BD7
CJK UNIFIED IDEOGRAPH:'F3D1:62417:&#x9BEC
CJK UNIFIED IDEOGRAPH:'F3D2:62418:&#x9BDC
CJK UNIFIED IDEOGRAPH:'F3D3:62419:&#x9BD9
CJK UNIFIED IDEOGRAPH:'F3D4:62420:&#x9BE5
CJK UNIFIED IDEOGRAPH:'F3D5:62421:&#x9BD5
CJK UNIFIED IDEOGRAPH:'F3D6:62422:&#x9BE1
CJK UNIFIED IDEOGRAPH:'F3D7:62423:&#x9BDA
CJK UNIFIED IDEOGRAPH:'F3D8:62424:&#x9D77
CJK UNIFIED IDEOGRAPH:'F3D9:62425:&#x9D81
CJK UNIFIED IDEOGRAPH:'F3DA:62426:&#x9D8A
CJK UNIFIED IDEOGRAPH:'F3DB:62427:&#x9D84
CJK UNIFIED IDEOGRAPH:'F3DC:62428:&#x9D88
CJK UNIFIED IDEOGRAPH:'F3DD:62429:&#x9D71
CJK UNIFIED IDEOGRAPH:'F3DE:62430:&#x9D80
CJK UNIFIED IDEOGRAPH:'F3DF:62431:&#x9D78
CJK UNIFIED IDEOGRAPH:'F3E0:62432:&#x9D86
CJK UNIFIED IDEOGRAPH:'F3E1:62433:&#x9D8B
CJK UNIFIED IDEOGRAPH:'F3E2:62434:&#x9D8C
CJK UNIFIED IDEOGRAPH:'F3E3:62435:&#x9D7D
CJK UNIFIED IDEOGRAPH:'F3E4:62436:&#x9D6B
CJK UNIFIED IDEOGRAPH:'F3E5:62437:&#x9D74
CJK UNIFIED IDEOGRAPH:'F3E6:62438:&#x9D75
CJK UNIFIED IDEOGRAPH:'F3E7:62439:&#x9D70
CJK UNIFIED IDEOGRAPH:'F3E8:62440:&#x9D69
CJK UNIFIED IDEOGRAPH:'F3E9:62441:&#x9D85
CJK UNIFIED IDEOGRAPH:'F3EA:62442:&#x9D73
CJK UNIFIED IDEOGRAPH:'F3EB:62443:&#x9D7B
CJK UNIFIED IDEOGRAPH:'F3EC:62444:&#x9D82
CJK UNIFIED IDEOGRAPH:'F3ED:62445:&#x9D6F
CJK UNIFIED IDEOGRAPH:'F3EE:62446:&#x9D79
CJK UNIFIED IDEOGRAPH:'F3EF:62447:&#x9D7F
CJK UNIFIED IDEOGRAPH:'F3F0:62448:&#x9D87
CJK UNIFIED IDEOGRAPH:'F3F1:62449:&#x9D68
CJK UNIFIED IDEOGRAPH:'F3F2:62450:&#x9E94
CJK UNIFIED IDEOGRAPH:'F3F3:62451:&#x9E91
CJK UNIFIED IDEOGRAPH:'F3F4:62452:&#x9EC0
CJK UNIFIED IDEOGRAPH:'F3F5:62453:&#x9EFC
CJK UNIFIED IDEOGRAPH:'F3F6:62454:&#x9F2D
CJK UNIFIED IDEOGRAPH:'F3F7:62455:&#x9F40
CJK UNIFIED IDEOGRAPH:'F3F8:62456:&#x9F41
CJK UNIFIED IDEOGRAPH:'F3F9:62457:&#x9F4D
CJK UNIFIED IDEOGRAPH:'F3FA:62458:&#x9F56
CJK UNIFIED IDEOGRAPH:'F3FB:62459:&#x9F57
CJK UNIFIED IDEOGRAPH:'F3FC:62460:&#x9F58
CJK UNIFIED IDEOGRAPH:'F3FD:62461:&#x5337
CJK UNIFIED IDEOGRAPH:'F3FE:62462:&#x56B2
CJK UNIFIED IDEOGRAPH:'F440:62528:&#x56B5
CJK UNIFIED IDEOGRAPH:'F441:62529:&#x56B3
CJK UNIFIED IDEOGRAPH:'F442:62530:&#x58E3
CJK UNIFIED IDEOGRAPH:'F443:62531:&#x5B45
CJK UNIFIED IDEOGRAPH:'F444:62532:&#x5DC6
CJK UNIFIED IDEOGRAPH:'F445:62533:&#x5DC7
CJK UNIFIED IDEOGRAPH:'F446:62534:&#x5EEE
CJK UNIFIED IDEOGRAPH:'F447:62535:&#x5EEF
CJK UNIFIED IDEOGRAPH:'F448:62536:&#x5FC0
CJK UNIFIED IDEOGRAPH:'F449:62537:&#x5FC1
CJK UNIFIED IDEOGRAPH:'F44A:62538:&#x61F9
CJK UNIFIED IDEOGRAPH:'F44B:62539:&#x6517
CJK UNIFIED IDEOGRAPH:'F44C:62540:&#x6516
CJK UNIFIED IDEOGRAPH:'F44D:62541:&#x6515
CJK UNIFIED IDEOGRAPH:'F44E:62542:&#x6513
CJK UNIFIED IDEOGRAPH:'F44F:62543:&#x65DF
CJK UNIFIED IDEOGRAPH:'F450:62544:&#x66E8
CJK UNIFIED IDEOGRAPH:'F451:62545:&#x66E3
CJK UNIFIED IDEOGRAPH:'F452:62546:&#x66E4
CJK UNIFIED IDEOGRAPH:'F453:62547:&#x6AF3
CJK UNIFIED IDEOGRAPH:'F454:62548:&#x6AF0
CJK UNIFIED IDEOGRAPH:'F455:62549:&#x6AEA
CJK UNIFIED IDEOGRAPH:'F456:62550:&#x6AE8
CJK UNIFIED IDEOGRAPH:'F457:62551:&#x6AF9
CJK UNIFIED IDEOGRAPH:'F458:62552:&#x6AF1
CJK UNIFIED IDEOGRAPH:'F459:62553:&#x6AEE
CJK UNIFIED IDEOGRAPH:'F45A:62554:&#x6AEF
CJK UNIFIED IDEOGRAPH:'F45B:62555:&#x703C
CJK UNIFIED IDEOGRAPH:'F45C:62556:&#x7035
CJK UNIFIED IDEOGRAPH:'F45D:62557:&#x702F
CJK UNIFIED IDEOGRAPH:'F45E:62558:&#x7037
CJK UNIFIED IDEOGRAPH:'F45F:62559:&#x7034
CJK UNIFIED IDEOGRAPH:'F460:62560:&#x7031
CJK UNIFIED IDEOGRAPH:'F461:62561:&#x7042
CJK UNIFIED IDEOGRAPH:'F462:62562:&#x7038
CJK UNIFIED IDEOGRAPH:'F463:62563:&#x703F
CJK UNIFIED IDEOGRAPH:'F464:62564:&#x703A
CJK UNIFIED IDEOGRAPH:'F465:62565:&#x7039
CJK UNIFIED IDEOGRAPH:'F466:62566:&#x7040
CJK UNIFIED IDEOGRAPH:'F467:62567:&#x703B
CJK UNIFIED IDEOGRAPH:'F468:62568:&#x7033
CJK UNIFIED IDEOGRAPH:'F469:62569:&#x7041
CJK UNIFIED IDEOGRAPH:'F46A:62570:&#x7213
CJK UNIFIED IDEOGRAPH:'F46B:62571:&#x7214
CJK UNIFIED IDEOGRAPH:'F46C:62572:&#x72A8
CJK UNIFIED IDEOGRAPH:'F46D:62573:&#x737D
CJK UNIFIED IDEOGRAPH:'F46E:62574:&#x737C
CJK UNIFIED IDEOGRAPH:'F46F:62575:&#x74BA
CJK UNIFIED IDEOGRAPH:'F470:62576:&#x76AB
CJK UNIFIED IDEOGRAPH:'F471:62577:&#x76AA
CJK UNIFIED IDEOGRAPH:'F472:62578:&#x76BE
CJK UNIFIED IDEOGRAPH:'F473:62579:&#x76ED
CJK UNIFIED IDEOGRAPH:'F474:62580:&#x77CC
CJK UNIFIED IDEOGRAPH:'F475:62581:&#x77CE
CJK UNIFIED IDEOGRAPH:'F476:62582:&#x77CF
CJK UNIFIED IDEOGRAPH:'F477:62583:&#x77CD
CJK UNIFIED IDEOGRAPH:'F478:62584:&#x77F2
CJK UNIFIED IDEOGRAPH:'F479:62585:&#x7925
CJK UNIFIED IDEOGRAPH:'F47A:62586:&#x7923
CJK UNIFIED IDEOGRAPH:'F47B:62587:&#x7927
CJK UNIFIED IDEOGRAPH:'F47C:62588:&#x7928
CJK UNIFIED IDEOGRAPH:'F47D:62589:&#x7924
CJK UNIFIED IDEOGRAPH:'F47E:62590:&#x7929
CJK UNIFIED IDEOGRAPH:'F4A1:62625:&#x79B2
CJK UNIFIED IDEOGRAPH:'F4A2:62626:&#x7A6E
CJK UNIFIED IDEOGRAPH:'F4A3:62627:&#x7A6C
CJK UNIFIED IDEOGRAPH:'F4A4:62628:&#x7A6D
CJK UNIFIED IDEOGRAPH:'F4A5:62629:&#x7AF7
CJK UNIFIED IDEOGRAPH:'F4A6:62630:&#x7C49
CJK UNIFIED IDEOGRAPH:'F4A7:62631:&#x7C48
CJK UNIFIED IDEOGRAPH:'F4A8:62632:&#x7C4A
CJK UNIFIED IDEOGRAPH:'F4A9:62633:&#x7C47
CJK UNIFIED IDEOGRAPH:'F4AA:62634:&#x7C45
CJK UNIFIED IDEOGRAPH:'F4AB:62635:&#x7CEE
CJK UNIFIED IDEOGRAPH:'F4AC:62636:&#x7E7B
CJK UNIFIED IDEOGRAPH:'F4AD:62637:&#x7E7E
CJK UNIFIED IDEOGRAPH:'F4AE:62638:&#x7E81
CJK UNIFIED IDEOGRAPH:'F4AF:62639:&#x7E80
CJK UNIFIED IDEOGRAPH:'F4B0:62640:&#x7FBA
CJK UNIFIED IDEOGRAPH:'F4B1:62641:&#x7FFF
CJK UNIFIED IDEOGRAPH:'F4B2:62642:&#x8079
CJK UNIFIED IDEOGRAPH:'F4B3:62643:&#x81DB
CJK UNIFIED IDEOGRAPH:'F4B4:62644:&#x81D9
CJK UNIFIED IDEOGRAPH:'F4B5:62645:&#x820B
CJK UNIFIED IDEOGRAPH:'F4B6:62646:&#x8268
CJK UNIFIED IDEOGRAPH:'F4B7:62647:&#x8269
CJK UNIFIED IDEOGRAPH:'F4B8:62648:&#x8622
CJK UNIFIED IDEOGRAPH:'F4B9:62649:&#x85FF
CJK UNIFIED IDEOGRAPH:'F4BA:62650:&#x8601
CJK UNIFIED IDEOGRAPH:'F4BB:62651:&#x85FE
CJK UNIFIED IDEOGRAPH:'F4BC:62652:&#x861B
CJK UNIFIED IDEOGRAPH:'F4BD:62653:&#x8600
CJK UNIFIED IDEOGRAPH:'F4BE:62654:&#x85F6
CJK UNIFIED IDEOGRAPH:'F4BF:62655:&#x8604
CJK UNIFIED IDEOGRAPH:'F4C0:62656:&#x8609
CJK UNIFIED IDEOGRAPH:'F4C1:62657:&#x8605
CJK UNIFIED IDEOGRAPH:'F4C2:62658:&#x860C
CJK UNIFIED IDEOGRAPH:'F4C3:62659:&#x85FD
CJK UNIFIED IDEOGRAPH:'F4C4:62660:&#x8819
CJK UNIFIED IDEOGRAPH:'F4C5:62661:&#x8810
CJK UNIFIED IDEOGRAPH:'F4C6:62662:&#x8811
CJK UNIFIED IDEOGRAPH:'F4C7:62663:&#x8817
CJK UNIFIED IDEOGRAPH:'F4C8:62664:&#x8813
CJK UNIFIED IDEOGRAPH:'F4C9:62665:&#x8816
CJK UNIFIED IDEOGRAPH:'F4CA:62666:&#x8963
CJK UNIFIED IDEOGRAPH:'F4CB:62667:&#x8966
CJK UNIFIED IDEOGRAPH:'F4CC:62668:&#x89B9
CJK UNIFIED IDEOGRAPH:'F4CD:62669:&#x89F7
CJK UNIFIED IDEOGRAPH:'F4CE:62670:&#x8B60
CJK UNIFIED IDEOGRAPH:'F4CF:62671:&#x8B6A
CJK UNIFIED IDEOGRAPH:'F4D0:62672:&#x8B5D
CJK UNIFIED IDEOGRAPH:'F4D1:62673:&#x8B68
CJK UNIFIED IDEOGRAPH:'F4D2:62674:&#x8B63
CJK UNIFIED IDEOGRAPH:'F4D3:62675:&#x8B65
CJK UNIFIED IDEOGRAPH:'F4D4:62676:&#x8B67
CJK UNIFIED IDEOGRAPH:'F4D5:62677:&#x8B6D
CJK UNIFIED IDEOGRAPH:'F4D6:62678:&#x8DAE
CJK UNIFIED IDEOGRAPH:'F4D7:62679:&#x8E86
CJK UNIFIED IDEOGRAPH:'F4D8:62680:&#x8E88
CJK UNIFIED IDEOGRAPH:'F4D9:62681:&#x8E84
CJK UNIFIED IDEOGRAPH:'F4DA:62682:&#x8F59
CJK UNIFIED IDEOGRAPH:'F4DB:62683:&#x8F56
CJK UNIFIED IDEOGRAPH:'F4DC:62684:&#x8F57
CJK UNIFIED IDEOGRAPH:'F4DD:62685:&#x8F55
CJK UNIFIED IDEOGRAPH:'F4DE:62686:&#x8F58
CJK UNIFIED IDEOGRAPH:'F4DF:62687:&#x8F5A
CJK UNIFIED IDEOGRAPH:'F4E0:62688:&#x908D
CJK UNIFIED IDEOGRAPH:'F4E1:62689:&#x9143
CJK UNIFIED IDEOGRAPH:'F4E2:62690:&#x9141
CJK UNIFIED IDEOGRAPH:'F4E3:62691:&#x91B7
CJK UNIFIED IDEOGRAPH:'F4E4:62692:&#x91B5
CJK UNIFIED IDEOGRAPH:'F4E5:62693:&#x91B2
CJK UNIFIED IDEOGRAPH:'F4E6:62694:&#x91B3
CJK UNIFIED IDEOGRAPH:'F4E7:62695:&#x940B
CJK UNIFIED IDEOGRAPH:'F4E8:62696:&#x9413
CJK UNIFIED IDEOGRAPH:'F4E9:62697:&#x93FB
CJK UNIFIED IDEOGRAPH:'F4EA:62698:&#x9420
CJK UNIFIED IDEOGRAPH:'F4EB:62699:&#x940F
CJK UNIFIED IDEOGRAPH:'F4EC:62700:&#x9414
CJK UNIFIED IDEOGRAPH:'F4ED:62701:&#x93FE
CJK UNIFIED IDEOGRAPH:'F4EE:62702:&#x9415
CJK UNIFIED IDEOGRAPH:'F4EF:62703:&#x9410
CJK UNIFIED IDEOGRAPH:'F4F0:62704:&#x9428
CJK UNIFIED IDEOGRAPH:'F4F1:62705:&#x9419
CJK UNIFIED IDEOGRAPH:'F4F2:62706:&#x940D
CJK UNIFIED IDEOGRAPH:'F4F3:62707:&#x93F5
CJK UNIFIED IDEOGRAPH:'F4F4:62708:&#x9400
CJK UNIFIED IDEOGRAPH:'F4F5:62709:&#x93F7
CJK UNIFIED IDEOGRAPH:'F4F6:62710:&#x9407
CJK UNIFIED IDEOGRAPH:'F4F7:62711:&#x940E
CJK UNIFIED IDEOGRAPH:'F4F8:62712:&#x9416
CJK UNIFIED IDEOGRAPH:'F4F9:62713:&#x9412
CJK UNIFIED IDEOGRAPH:'F4FA:62714:&#x93FA
CJK UNIFIED IDEOGRAPH:'F4FB:62715:&#x9409
CJK UNIFIED IDEOGRAPH:'F4FC:62716:&#x93F8
CJK UNIFIED IDEOGRAPH:'F4FD:62717:&#x940A
CJK UNIFIED IDEOGRAPH:'F4FE:62718:&#x93FF
CJK UNIFIED IDEOGRAPH:'F540:62784:&#x93FC
CJK UNIFIED IDEOGRAPH:'F541:62785:&#x940C
CJK UNIFIED IDEOGRAPH:'F542:62786:&#x93F6
CJK UNIFIED IDEOGRAPH:'F543:62787:&#x9411
CJK UNIFIED IDEOGRAPH:'F544:62788:&#x9406
CJK UNIFIED IDEOGRAPH:'F545:62789:&#x95DE
CJK UNIFIED IDEOGRAPH:'F546:62790:&#x95E0
CJK UNIFIED IDEOGRAPH:'F547:62791:&#x95DF
CJK UNIFIED IDEOGRAPH:'F548:62792:&#x972E
CJK UNIFIED IDEOGRAPH:'F549:62793:&#x972F
CJK UNIFIED IDEOGRAPH:'F54A:62794:&#x97B9
CJK UNIFIED IDEOGRAPH:'F54B:62795:&#x97BB
CJK UNIFIED IDEOGRAPH:'F54C:62796:&#x97FD
CJK UNIFIED IDEOGRAPH:'F54D:62797:&#x97FE
CJK UNIFIED IDEOGRAPH:'F54E:62798:&#x9860
CJK UNIFIED IDEOGRAPH:'F54F:62799:&#x9862
CJK UNIFIED IDEOGRAPH:'F550:62800:&#x9863
CJK UNIFIED IDEOGRAPH:'F551:62801:&#x985F
CJK UNIFIED IDEOGRAPH:'F552:62802:&#x98C1
CJK UNIFIED IDEOGRAPH:'F553:62803:&#x98C2
CJK UNIFIED IDEOGRAPH:'F554:62804:&#x9950
CJK UNIFIED IDEOGRAPH:'F555:62805:&#x994E
CJK UNIFIED IDEOGRAPH:'F556:62806:&#x9959
CJK UNIFIED IDEOGRAPH:'F557:62807:&#x994C
CJK UNIFIED IDEOGRAPH:'F558:62808:&#x994B
CJK UNIFIED IDEOGRAPH:'F559:62809:&#x9953
CJK UNIFIED IDEOGRAPH:'F55A:62810:&#x9A32
CJK UNIFIED IDEOGRAPH:'F55B:62811:&#x9A34
CJK UNIFIED IDEOGRAPH:'F55C:62812:&#x9A31
CJK UNIFIED IDEOGRAPH:'F55D:62813:&#x9A2C
CJK UNIFIED IDEOGRAPH:'F55E:62814:&#x9A2A
CJK UNIFIED IDEOGRAPH:'F55F:62815:&#x9A36
CJK UNIFIED IDEOGRAPH:'F560:62816:&#x9A29
CJK UNIFIED IDEOGRAPH:'F561:62817:&#x9A2E
CJK UNIFIED IDEOGRAPH:'F562:62818:&#x9A38
CJK UNIFIED IDEOGRAPH:'F563:62819:&#x9A2D
CJK UNIFIED IDEOGRAPH:'F564:62820:&#x9AC7
CJK UNIFIED IDEOGRAPH:'F565:62821:&#x9ACA
CJK UNIFIED IDEOGRAPH:'F566:62822:&#x9AC6
CJK UNIFIED IDEOGRAPH:'F567:62823:&#x9B10
CJK UNIFIED IDEOGRAPH:'F568:62824:&#x9B12
CJK UNIFIED IDEOGRAPH:'F569:62825:&#x9B11
CJK UNIFIED IDEOGRAPH:'F56A:62826:&#x9C0B
CJK UNIFIED IDEOGRAPH:'F56B:62827:&#x9C08
CJK UNIFIED IDEOGRAPH:'F56C:62828:&#x9BF7
CJK UNIFIED IDEOGRAPH:'F56D:62829:&#x9C05
CJK UNIFIED IDEOGRAPH:'F56E:62830:&#x9C12
CJK UNIFIED IDEOGRAPH:'F56F:62831:&#x9BF8
CJK UNIFIED IDEOGRAPH:'F570:62832:&#x9C40
CJK UNIFIED IDEOGRAPH:'F571:62833:&#x9C07
CJK UNIFIED IDEOGRAPH:'F572:62834:&#x9C0E
CJK UNIFIED IDEOGRAPH:'F573:62835:&#x9C06
CJK UNIFIED IDEOGRAPH:'F574:62836:&#x9C17
CJK UNIFIED IDEOGRAPH:'F575:62837:&#x9C14
CJK UNIFIED IDEOGRAPH:'F576:62838:&#x9C09
CJK UNIFIED IDEOGRAPH:'F577:62839:&#x9D9F
CJK UNIFIED IDEOGRAPH:'F578:62840:&#x9D99
CJK UNIFIED IDEOGRAPH:'F579:62841:&#x9DA4
CJK UNIFIED IDEOGRAPH:'F57A:62842:&#x9D9D
CJK UNIFIED IDEOGRAPH:'F57B:62843:&#x9D92
CJK UNIFIED IDEOGRAPH:'F57C:62844:&#x9D98
CJK UNIFIED IDEOGRAPH:'F57D:62845:&#x9D90
CJK UNIFIED IDEOGRAPH:'F57E:62846:&#x9D9B
CJK UNIFIED IDEOGRAPH:'F5A1:62881:&#x9DA0
CJK UNIFIED IDEOGRAPH:'F5A2:62882:&#x9D94
CJK UNIFIED IDEOGRAPH:'F5A3:62883:&#x9D9C
CJK UNIFIED IDEOGRAPH:'F5A4:62884:&#x9DAA
CJK UNIFIED IDEOGRAPH:'F5A5:62885:&#x9D97
CJK UNIFIED IDEOGRAPH:'F5A6:62886:&#x9DA1
CJK UNIFIED IDEOGRAPH:'F5A7:62887:&#x9D9A
CJK UNIFIED IDEOGRAPH:'F5A8:62888:&#x9DA2
CJK UNIFIED IDEOGRAPH:'F5A9:62889:&#x9DA8
CJK UNIFIED IDEOGRAPH:'F5AA:62890:&#x9D9E
CJK UNIFIED IDEOGRAPH:'F5AB:62891:&#x9DA3
CJK UNIFIED IDEOGRAPH:'F5AC:62892:&#x9DBF
CJK UNIFIED IDEOGRAPH:'F5AD:62893:&#x9DA9
CJK UNIFIED IDEOGRAPH:'F5AE:62894:&#x9D96
CJK UNIFIED IDEOGRAPH:'F5AF:62895:&#x9DA6
CJK UNIFIED IDEOGRAPH:'F5B0:62896:&#x9DA7
CJK UNIFIED IDEOGRAPH:'F5B1:62897:&#x9E99
CJK UNIFIED IDEOGRAPH:'F5B2:62898:&#x9E9B
CJK UNIFIED IDEOGRAPH:'F5B3:62899:&#x9E9A
CJK UNIFIED IDEOGRAPH:'F5B4:62900:&#x9EE5
CJK UNIFIED IDEOGRAPH:'F5B5:62901:&#x9EE4
CJK UNIFIED IDEOGRAPH:'F5B6:62902:&#x9EE7
CJK UNIFIED IDEOGRAPH:'F5B7:62903:&#x9EE6
CJK UNIFIED IDEOGRAPH:'F5B8:62904:&#x9F30
CJK UNIFIED IDEOGRAPH:'F5B9:62905:&#x9F2E
CJK UNIFIED IDEOGRAPH:'F5BA:62906:&#x9F5B
CJK UNIFIED IDEOGRAPH:'F5BB:62907:&#x9F60
CJK UNIFIED IDEOGRAPH:'F5BC:62908:&#x9F5E
CJK UNIFIED IDEOGRAPH:'F5BD:62909:&#x9F5D
CJK UNIFIED IDEOGRAPH:'F5BE:62910:&#x9F59
CJK UNIFIED IDEOGRAPH:'F5BF:62911:&#x9F91
CJK UNIFIED IDEOGRAPH:'F5C0:62912:&#x513A
CJK UNIFIED IDEOGRAPH:'F5C1:62913:&#x5139
CJK UNIFIED IDEOGRAPH:'F5C2:62914:&#x5298
CJK UNIFIED IDEOGRAPH:'F5C3:62915:&#x5297
CJK UNIFIED IDEOGRAPH:'F5C4:62916:&#x56C3
CJK UNIFIED IDEOGRAPH:'F5C5:62917:&#x56BD
CJK UNIFIED IDEOGRAPH:'F5C6:62918:&#x56BE
CJK UNIFIED IDEOGRAPH:'F5C7:62919:&#x5B48
CJK UNIFIED IDEOGRAPH:'F5C8:62920:&#x5B47
CJK UNIFIED IDEOGRAPH:'F5C9:62921:&#x5DCB
CJK UNIFIED IDEOGRAPH:'F5CA:62922:&#x5DCF
CJK UNIFIED IDEOGRAPH:'F5CB:62923:&#x5EF1
CJK UNIFIED IDEOGRAPH:'F5CC:62924:&#x61FD
CJK UNIFIED IDEOGRAPH:'F5CD:62925:&#x651B
CJK UNIFIED IDEOGRAPH:'F5CE:62926:&#x6B02
CJK UNIFIED IDEOGRAPH:'F5CF:62927:&#x6AFC
CJK UNIFIED IDEOGRAPH:'F5D0:62928:&#x6B03
CJK UNIFIED IDEOGRAPH:'F5D1:62929:&#x6AF8
CJK UNIFIED IDEOGRAPH:'F5D2:62930:&#x6B00
CJK UNIFIED IDEOGRAPH:'F5D3:62931:&#x7043
CJK UNIFIED IDEOGRAPH:'F5D4:62932:&#x7044
CJK UNIFIED IDEOGRAPH:'F5D5:62933:&#x704A
CJK UNIFIED IDEOGRAPH:'F5D6:62934:&#x7048
CJK UNIFIED IDEOGRAPH:'F5D7:62935:&#x7049
CJK UNIFIED IDEOGRAPH:'F5D8:62936:&#x7045
CJK UNIFIED IDEOGRAPH:'F5D9:62937:&#x7046
CJK UNIFIED IDEOGRAPH:'F5DA:62938:&#x721D
CJK UNIFIED IDEOGRAPH:'F5DB:62939:&#x721A
CJK UNIFIED IDEOGRAPH:'F5DC:62940:&#x7219
CJK UNIFIED IDEOGRAPH:'F5DD:62941:&#x737E
CJK UNIFIED IDEOGRAPH:'F5DE:62942:&#x7517
CJK UNIFIED IDEOGRAPH:'F5DF:62943:&#x766A
CJK UNIFIED IDEOGRAPH:'F5E0:62944:&#x77D0
CJK UNIFIED IDEOGRAPH:'F5E1:62945:&#x792D
CJK UNIFIED IDEOGRAPH:'F5E2:62946:&#x7931
CJK UNIFIED IDEOGRAPH:'F5E3:62947:&#x792F
CJK UNIFIED IDEOGRAPH:'F5E4:62948:&#x7C54
CJK UNIFIED IDEOGRAPH:'F5E5:62949:&#x7C53
CJK UNIFIED IDEOGRAPH:'F5E6:62950:&#x7CF2
CJK UNIFIED IDEOGRAPH:'F5E7:62951:&#x7E8A
CJK UNIFIED IDEOGRAPH:'F5E8:62952:&#x7E87
CJK UNIFIED IDEOGRAPH:'F5E9:62953:&#x7E88
CJK UNIFIED IDEOGRAPH:'F5EA:62954:&#x7E8B
CJK UNIFIED IDEOGRAPH:'F5EB:62955:&#x7E86
CJK UNIFIED IDEOGRAPH:'F5EC:62956:&#x7E8D
CJK UNIFIED IDEOGRAPH:'F5ED:62957:&#x7F4D
CJK UNIFIED IDEOGRAPH:'F5EE:62958:&#x7FBB
CJK UNIFIED IDEOGRAPH:'F5EF:62959:&#x8030
CJK UNIFIED IDEOGRAPH:'F5F0:62960:&#x81DD
CJK UNIFIED IDEOGRAPH:'F5F1:62961:&#x8618
CJK UNIFIED IDEOGRAPH:'F5F2:62962:&#x862A
CJK UNIFIED IDEOGRAPH:'F5F3:62963:&#x8626
CJK UNIFIED IDEOGRAPH:'F5F4:62964:&#x861F
CJK UNIFIED IDEOGRAPH:'F5F5:62965:&#x8623
CJK UNIFIED IDEOGRAPH:'F5F6:62966:&#x861C
CJK UNIFIED IDEOGRAPH:'F5F7:62967:&#x8619
CJK UNIFIED IDEOGRAPH:'F5F8:62968:&#x8627
CJK UNIFIED IDEOGRAPH:'F5F9:62969:&#x862E
CJK UNIFIED IDEOGRAPH:'F5FA:62970:&#x8621
CJK UNIFIED IDEOGRAPH:'F5FB:62971:&#x8620
CJK UNIFIED IDEOGRAPH:'F5FC:62972:&#x8629
CJK UNIFIED IDEOGRAPH:'F5FD:62973:&#x861E
CJK UNIFIED IDEOGRAPH:'F5FE:62974:&#x8625
CJK UNIFIED IDEOGRAPH:'F640:63040:&#x8829
CJK UNIFIED IDEOGRAPH:'F641:63041:&#x881D
CJK UNIFIED IDEOGRAPH:'F642:63042:&#x881B
CJK UNIFIED IDEOGRAPH:'F643:63043:&#x8820
CJK UNIFIED IDEOGRAPH:'F644:63044:&#x8824
CJK UNIFIED IDEOGRAPH:'F645:63045:&#x881C
CJK UNIFIED IDEOGRAPH:'F646:63046:&#x882B
CJK UNIFIED IDEOGRAPH:'F647:63047:&#x884A
CJK UNIFIED IDEOGRAPH:'F648:63048:&#x896D
CJK UNIFIED IDEOGRAPH:'F649:63049:&#x8969
CJK UNIFIED IDEOGRAPH:'F64A:63050:&#x896E
CJK UNIFIED IDEOGRAPH:'F64B:63051:&#x896B
CJK UNIFIED IDEOGRAPH:'F64C:63052:&#x89FA
CJK UNIFIED IDEOGRAPH:'F64D:63053:&#x8B79
CJK UNIFIED IDEOGRAPH:'F64E:63054:&#x8B78
CJK UNIFIED IDEOGRAPH:'F64F:63055:&#x8B45
CJK UNIFIED IDEOGRAPH:'F650:63056:&#x8B7A
CJK UNIFIED IDEOGRAPH:'F651:63057:&#x8B7B
CJK UNIFIED IDEOGRAPH:'F652:63058:&#x8D10
CJK UNIFIED IDEOGRAPH:'F653:63059:&#x8D14
CJK UNIFIED IDEOGRAPH:'F654:63060:&#x8DAF
CJK UNIFIED IDEOGRAPH:'F655:63061:&#x8E8E
CJK UNIFIED IDEOGRAPH:'F656:63062:&#x8E8C
CJK UNIFIED IDEOGRAPH:'F657:63063:&#x8F5E
CJK UNIFIED IDEOGRAPH:'F658:63064:&#x8F5B
CJK UNIFIED IDEOGRAPH:'F659:63065:&#x8F5D
CJK UNIFIED IDEOGRAPH:'F65A:63066:&#x9146
CJK UNIFIED IDEOGRAPH:'F65B:63067:&#x9144
CJK UNIFIED IDEOGRAPH:'F65C:63068:&#x9145
CJK UNIFIED IDEOGRAPH:'F65D:63069:&#x91B9
CJK UNIFIED IDEOGRAPH:'F65E:63070:&#x943F
CJK UNIFIED IDEOGRAPH:'F65F:63071:&#x943B
CJK UNIFIED IDEOGRAPH:'F660:63072:&#x9436
CJK UNIFIED IDEOGRAPH:'F661:63073:&#x9429
CJK UNIFIED IDEOGRAPH:'F662:63074:&#x943D
CJK UNIFIED IDEOGRAPH:'F663:63075:&#x943C
CJK UNIFIED IDEOGRAPH:'F664:63076:&#x9430
CJK UNIFIED IDEOGRAPH:'F665:63077:&#x9439
CJK UNIFIED IDEOGRAPH:'F666:63078:&#x942A
CJK UNIFIED IDEOGRAPH:'F667:63079:&#x9437
CJK UNIFIED IDEOGRAPH:'F668:63080:&#x942C
CJK UNIFIED IDEOGRAPH:'F669:63081:&#x9440
CJK UNIFIED IDEOGRAPH:'F66A:63082:&#x9431
CJK UNIFIED IDEOGRAPH:'F66B:63083:&#x95E5
CJK UNIFIED IDEOGRAPH:'F66C:63084:&#x95E4
CJK UNIFIED IDEOGRAPH:'F66D:63085:&#x95E3
CJK UNIFIED IDEOGRAPH:'F66E:63086:&#x9735
CJK UNIFIED IDEOGRAPH:'F66F:63087:&#x973A
CJK UNIFIED IDEOGRAPH:'F670:63088:&#x97BF
CJK UNIFIED IDEOGRAPH:'F671:63089:&#x97E1
CJK UNIFIED IDEOGRAPH:'F672:63090:&#x9864
CJK UNIFIED IDEOGRAPH:'F673:63091:&#x98C9
CJK UNIFIED IDEOGRAPH:'F674:63092:&#x98C6
CJK UNIFIED IDEOGRAPH:'F675:63093:&#x98C0
CJK UNIFIED IDEOGRAPH:'F676:63094:&#x9958
CJK UNIFIED IDEOGRAPH:'F677:63095:&#x9956
CJK UNIFIED IDEOGRAPH:'F678:63096:&#x9A39
CJK UNIFIED IDEOGRAPH:'F679:63097:&#x9A3D
CJK UNIFIED IDEOGRAPH:'F67A:63098:&#x9A46
CJK UNIFIED IDEOGRAPH:'F67B:63099:&#x9A44
CJK UNIFIED IDEOGRAPH:'F67C:63100:&#x9A42
CJK UNIFIED IDEOGRAPH:'F67D:63101:&#x9A41
CJK UNIFIED IDEOGRAPH:'F67E:63102:&#x9A3A
CJK UNIFIED IDEOGRAPH:'F6A1:63137:&#x9A3F
CJK UNIFIED IDEOGRAPH:'F6A2:63138:&#x9ACD
CJK UNIFIED IDEOGRAPH:'F6A3:63139:&#x9B15
CJK UNIFIED IDEOGRAPH:'F6A4:63140:&#x9B17
CJK UNIFIED IDEOGRAPH:'F6A5:63141:&#x9B18
CJK UNIFIED IDEOGRAPH:'F6A6:63142:&#x9B16
CJK UNIFIED IDEOGRAPH:'F6A7:63143:&#x9B3A
CJK UNIFIED IDEOGRAPH:'F6A8:63144:&#x9B52
CJK UNIFIED IDEOGRAPH:'F6A9:63145:&#x9C2B
CJK UNIFIED IDEOGRAPH:'F6AA:63146:&#x9C1D
CJK UNIFIED IDEOGRAPH:'F6AB:63147:&#x9C1C
CJK UNIFIED IDEOGRAPH:'F6AC:63148:&#x9C2C
CJK UNIFIED IDEOGRAPH:'F6AD:63149:&#x9C23
CJK UNIFIED IDEOGRAPH:'F6AE:63150:&#x9C28
CJK UNIFIED IDEOGRAPH:'F6AF:63151:&#x9C29
CJK UNIFIED IDEOGRAPH:'F6B0:63152:&#x9C24
CJK UNIFIED IDEOGRAPH:'F6B1:63153:&#x9C21
CJK UNIFIED IDEOGRAPH:'F6B2:63154:&#x9DB7
CJK UNIFIED IDEOGRAPH:'F6B3:63155:&#x9DB6
CJK UNIFIED IDEOGRAPH:'F6B4:63156:&#x9DBC
CJK UNIFIED IDEOGRAPH:'F6B5:63157:&#x9DC1
CJK UNIFIED IDEOGRAPH:'F6B6:63158:&#x9DC7
CJK UNIFIED IDEOGRAPH:'F6B7:63159:&#x9DCA
CJK UNIFIED IDEOGRAPH:'F6B8:63160:&#x9DCF
CJK UNIFIED IDEOGRAPH:'F6B9:63161:&#x9DBE
CJK UNIFIED IDEOGRAPH:'F6BA:63162:&#x9DC5
CJK UNIFIED IDEOGRAPH:'F6BB:63163:&#x9DC3
CJK UNIFIED IDEOGRAPH:'F6BC:63164:&#x9DBB
CJK UNIFIED IDEOGRAPH:'F6BD:63165:&#x9DB5
CJK UNIFIED IDEOGRAPH:'F6BE:63166:&#x9DCE
CJK UNIFIED IDEOGRAPH:'F6BF:63167:&#x9DB9
CJK UNIFIED IDEOGRAPH:'F6C0:63168:&#x9DBA
CJK UNIFIED IDEOGRAPH:'F6C1:63169:&#x9DAC
CJK UNIFIED IDEOGRAPH:'F6C2:63170:&#x9DC8
CJK UNIFIED IDEOGRAPH:'F6C3:63171:&#x9DB1
CJK UNIFIED IDEOGRAPH:'F6C4:63172:&#x9DAD
CJK UNIFIED IDEOGRAPH:'F6C5:63173:&#x9DCC
CJK UNIFIED IDEOGRAPH:'F6C6:63174:&#x9DB3
CJK UNIFIED IDEOGRAPH:'F6C7:63175:&#x9DCD
CJK UNIFIED IDEOGRAPH:'F6C8:63176:&#x9DB2
CJK UNIFIED IDEOGRAPH:'F6C9:63177:&#x9E7A
CJK UNIFIED IDEOGRAPH:'F6CA:63178:&#x9E9C
CJK UNIFIED IDEOGRAPH:'F6CB:63179:&#x9EEB
CJK UNIFIED IDEOGRAPH:'F6CC:63180:&#x9EEE
CJK UNIFIED IDEOGRAPH:'F6CD:63181:&#x9EED
CJK UNIFIED IDEOGRAPH:'F6CE:63182:&#x9F1B
CJK UNIFIED IDEOGRAPH:'F6CF:63183:&#x9F18
CJK UNIFIED IDEOGRAPH:'F6D0:63184:&#x9F1A
CJK UNIFIED IDEOGRAPH:'F6D1:63185:&#x9F31
CJK UNIFIED IDEOGRAPH:'F6D2:63186:&#x9F4E
CJK UNIFIED IDEOGRAPH:'F6D3:63187:&#x9F65
CJK UNIFIED IDEOGRAPH:'F6D4:63188:&#x9F64
CJK UNIFIED IDEOGRAPH:'F6D5:63189:&#x9F92
CJK UNIFIED IDEOGRAPH:'F6D6:63190:&#x4EB9
CJK UNIFIED IDEOGRAPH:'F6D7:63191:&#x56C6
CJK UNIFIED IDEOGRAPH:'F6D8:63192:&#x56C5
CJK UNIFIED IDEOGRAPH:'F6D9:63193:&#x56CB
CJK UNIFIED IDEOGRAPH:'F6DA:63194:&#x5971
CJK UNIFIED IDEOGRAPH:'F6DB:63195:&#x5B4B
CJK UNIFIED IDEOGRAPH:'F6DC:63196:&#x5B4C
CJK UNIFIED IDEOGRAPH:'F6DD:63197:&#x5DD5
CJK UNIFIED IDEOGRAPH:'F6DE:63198:&#x5DD1
CJK UNIFIED IDEOGRAPH:'F6DF:63199:&#x5EF2
CJK UNIFIED IDEOGRAPH:'F6E0:63200:&#x6521
CJK UNIFIED IDEOGRAPH:'F6E1:63201:&#x6520
CJK UNIFIED IDEOGRAPH:'F6E2:63202:&#x6526
CJK UNIFIED IDEOGRAPH:'F6E3:63203:&#x6522
CJK UNIFIED IDEOGRAPH:'F6E4:63204:&#x6B0B
CJK UNIFIED IDEOGRAPH:'F6E5:63205:&#x6B08
CJK UNIFIED IDEOGRAPH:'F6E6:63206:&#x6B09
CJK UNIFIED IDEOGRAPH:'F6E7:63207:&#x6C0D
CJK UNIFIED IDEOGRAPH:'F6E8:63208:&#x7055
CJK UNIFIED IDEOGRAPH:'F6E9:63209:&#x7056
CJK UNIFIED IDEOGRAPH:'F6EA:63210:&#x7057
CJK UNIFIED IDEOGRAPH:'F6EB:63211:&#x7052
CJK UNIFIED IDEOGRAPH:'F6EC:63212:&#x721E
CJK UNIFIED IDEOGRAPH:'F6ED:63213:&#x721F
CJK UNIFIED IDEOGRAPH:'F6EE:63214:&#x72A9
CJK UNIFIED IDEOGRAPH:'F6EF:63215:&#x737F
CJK UNIFIED IDEOGRAPH:'F6F0:63216:&#x74D8
CJK UNIFIED IDEOGRAPH:'F6F1:63217:&#x74D5
CJK UNIFIED IDEOGRAPH:'F6F2:63218:&#x74D9
CJK UNIFIED IDEOGRAPH:'F6F3:63219:&#x74D7
CJK UNIFIED IDEOGRAPH:'F6F4:63220:&#x766D
CJK UNIFIED IDEOGRAPH:'F6F5:63221:&#x76AD
CJK UNIFIED IDEOGRAPH:'F6F6:63222:&#x7935
CJK UNIFIED IDEOGRAPH:'F6F7:63223:&#x79B4
CJK UNIFIED IDEOGRAPH:'F6F8:63224:&#x7A70
CJK UNIFIED IDEOGRAPH:'F6F9:63225:&#x7A71
CJK UNIFIED IDEOGRAPH:'F6FA:63226:&#x7C57
CJK UNIFIED IDEOGRAPH:'F6FB:63227:&#x7C5C
CJK UNIFIED IDEOGRAPH:'F6FC:63228:&#x7C59
CJK UNIFIED IDEOGRAPH:'F6FD:63229:&#x7C5B
CJK UNIFIED IDEOGRAPH:'F6FE:63230:&#x7C5A
CJK UNIFIED IDEOGRAPH:'F740:63296:&#x7CF4
CJK UNIFIED IDEOGRAPH:'F741:63297:&#x7CF1
CJK UNIFIED IDEOGRAPH:'F742:63298:&#x7E91
CJK UNIFIED IDEOGRAPH:'F743:63299:&#x7F4F
CJK UNIFIED IDEOGRAPH:'F744:63300:&#x7F87
CJK UNIFIED IDEOGRAPH:'F745:63301:&#x81DE
CJK UNIFIED IDEOGRAPH:'F746:63302:&#x826B
CJK UNIFIED IDEOGRAPH:'F747:63303:&#x8634
CJK UNIFIED IDEOGRAPH:'F748:63304:&#x8635
CJK UNIFIED IDEOGRAPH:'F749:63305:&#x8633
CJK UNIFIED IDEOGRAPH:'F74A:63306:&#x862C
CJK UNIFIED IDEOGRAPH:'F74B:63307:&#x8632
CJK UNIFIED IDEOGRAPH:'F74C:63308:&#x8636
CJK UNIFIED IDEOGRAPH:'F74D:63309:&#x882C
CJK UNIFIED IDEOGRAPH:'F74E:63310:&#x8828
CJK UNIFIED IDEOGRAPH:'F74F:63311:&#x8826
CJK UNIFIED IDEOGRAPH:'F750:63312:&#x882A
CJK UNIFIED IDEOGRAPH:'F751:63313:&#x8825
CJK UNIFIED IDEOGRAPH:'F752:63314:&#x8971
CJK UNIFIED IDEOGRAPH:'F753:63315:&#x89BF
CJK UNIFIED IDEOGRAPH:'F754:63316:&#x89BE
CJK UNIFIED IDEOGRAPH:'F755:63317:&#x89FB
CJK UNIFIED IDEOGRAPH:'F756:63318:&#x8B7E
CJK UNIFIED IDEOGRAPH:'F757:63319:&#x8B84
CJK UNIFIED IDEOGRAPH:'F758:63320:&#x8B82
CJK UNIFIED IDEOGRAPH:'F759:63321:&#x8B86
CJK UNIFIED IDEOGRAPH:'F75A:63322:&#x8B85
CJK UNIFIED IDEOGRAPH:'F75B:63323:&#x8B7F
CJK UNIFIED IDEOGRAPH:'F75C:63324:&#x8D15
CJK UNIFIED IDEOGRAPH:'F75D:63325:&#x8E95
CJK UNIFIED IDEOGRAPH:'F75E:63326:&#x8E94
CJK UNIFIED IDEOGRAPH:'F75F:63327:&#x8E9A
CJK UNIFIED IDEOGRAPH:'F760:63328:&#x8E92
CJK UNIFIED IDEOGRAPH:'F761:63329:&#x8E90
CJK UNIFIED IDEOGRAPH:'F762:63330:&#x8E96
CJK UNIFIED IDEOGRAPH:'F763:63331:&#x8E97
CJK UNIFIED IDEOGRAPH:'F764:63332:&#x8F60
CJK UNIFIED IDEOGRAPH:'F765:63333:&#x8F62
CJK UNIFIED IDEOGRAPH:'F766:63334:&#x9147
CJK UNIFIED IDEOGRAPH:'F767:63335:&#x944C
CJK UNIFIED IDEOGRAPH:'F768:63336:&#x9450
CJK UNIFIED IDEOGRAPH:'F769:63337:&#x944A
CJK UNIFIED IDEOGRAPH:'F76A:63338:&#x944B
CJK UNIFIED IDEOGRAPH:'F76B:63339:&#x944F
CJK UNIFIED IDEOGRAPH:'F76C:63340:&#x9447
CJK UNIFIED IDEOGRAPH:'F76D:63341:&#x9445
CJK UNIFIED IDEOGRAPH:'F76E:63342:&#x9448
CJK UNIFIED IDEOGRAPH:'F76F:63343:&#x9449
CJK UNIFIED IDEOGRAPH:'F770:63344:&#x9446
CJK UNIFIED IDEOGRAPH:'F771:63345:&#x973F
CJK UNIFIED IDEOGRAPH:'F772:63346:&#x97E3
CJK UNIFIED IDEOGRAPH:'F773:63347:&#x986A
CJK UNIFIED IDEOGRAPH:'F774:63348:&#x9869
CJK UNIFIED IDEOGRAPH:'F775:63349:&#x98CB
CJK UNIFIED IDEOGRAPH:'F776:63350:&#x9954
CJK UNIFIED IDEOGRAPH:'F777:63351:&#x995B
CJK UNIFIED IDEOGRAPH:'F778:63352:&#x9A4E
CJK UNIFIED IDEOGRAPH:'F779:63353:&#x9A53
CJK UNIFIED IDEOGRAPH:'F77A:63354:&#x9A54
CJK UNIFIED IDEOGRAPH:'F77B:63355:&#x9A4C
CJK UNIFIED IDEOGRAPH:'F77C:63356:&#x9A4F
CJK UNIFIED IDEOGRAPH:'F77D:63357:&#x9A48
CJK UNIFIED IDEOGRAPH:'F77E:63358:&#x9A4A
CJK UNIFIED IDEOGRAPH:'F7A1:63393:&#x9A49
CJK UNIFIED IDEOGRAPH:'F7A2:63394:&#x9A52
CJK UNIFIED IDEOGRAPH:'F7A3:63395:&#x9A50
CJK UNIFIED IDEOGRAPH:'F7A4:63396:&#x9AD0
CJK UNIFIED IDEOGRAPH:'F7A5:63397:&#x9B19
CJK UNIFIED IDEOGRAPH:'F7A6:63398:&#x9B2B
CJK UNIFIED IDEOGRAPH:'F7A7:63399:&#x9B3B
CJK UNIFIED IDEOGRAPH:'F7A8:63400:&#x9B56
CJK UNIFIED IDEOGRAPH:'F7A9:63401:&#x9B55
CJK UNIFIED IDEOGRAPH:'F7AA:63402:&#x9C46
CJK UNIFIED IDEOGRAPH:'F7AB:63403:&#x9C48
CJK UNIFIED IDEOGRAPH:'F7AC:63404:&#x9C3F
CJK UNIFIED IDEOGRAPH:'F7AD:63405:&#x9C44
CJK UNIFIED IDEOGRAPH:'F7AE:63406:&#x9C39
CJK UNIFIED IDEOGRAPH:'F7AF:63407:&#x9C33
CJK UNIFIED IDEOGRAPH:'F7B0:63408:&#x9C41
CJK UNIFIED IDEOGRAPH:'F7B1:63409:&#x9C3C
CJK UNIFIED IDEOGRAPH:'F7B2:63410:&#x9C37
CJK UNIFIED IDEOGRAPH:'F7B3:63411:&#x9C34
CJK UNIFIED IDEOGRAPH:'F7B4:63412:&#x9C32
CJK UNIFIED IDEOGRAPH:'F7B5:63413:&#x9C3D
CJK UNIFIED IDEOGRAPH:'F7B6:63414:&#x9C36
CJK UNIFIED IDEOGRAPH:'F7B7:63415:&#x9DDB
CJK UNIFIED IDEOGRAPH:'F7B8:63416:&#x9DD2
CJK UNIFIED IDEOGRAPH:'F7B9:63417:&#x9DDE
CJK UNIFIED IDEOGRAPH:'F7BA:63418:&#x9DDA
CJK UNIFIED IDEOGRAPH:'F7BB:63419:&#x9DCB
CJK UNIFIED IDEOGRAPH:'F7BC:63420:&#x9DD0
CJK UNIFIED IDEOGRAPH:'F7BD:63421:&#x9DDC
CJK UNIFIED IDEOGRAPH:'F7BE:63422:&#x9DD1
CJK UNIFIED IDEOGRAPH:'F7BF:63423:&#x9DDF
CJK UNIFIED IDEOGRAPH:'F7C0:63424:&#x9DE9
CJK UNIFIED IDEOGRAPH:'F7C1:63425:&#x9DD9
CJK UNIFIED IDEOGRAPH:'F7C2:63426:&#x9DD8
CJK UNIFIED IDEOGRAPH:'F7C3:63427:&#x9DD6
CJK UNIFIED IDEOGRAPH:'F7C4:63428:&#x9DF5
CJK UNIFIED IDEOGRAPH:'F7C5:63429:&#x9DD5
CJK UNIFIED IDEOGRAPH:'F7C6:63430:&#x9DDD
CJK UNIFIED IDEOGRAPH:'F7C7:63431:&#x9EB6
CJK UNIFIED IDEOGRAPH:'F7C8:63432:&#x9EF0
CJK UNIFIED IDEOGRAPH:'F7C9:63433:&#x9F35
CJK UNIFIED IDEOGRAPH:'F7CA:63434:&#x9F33
CJK UNIFIED IDEOGRAPH:'F7CB:63435:&#x9F32
CJK UNIFIED IDEOGRAPH:'F7CC:63436:&#x9F42
CJK UNIFIED IDEOGRAPH:'F7CD:63437:&#x9F6B
CJK UNIFIED IDEOGRAPH:'F7CE:63438:&#x9F95
CJK UNIFIED IDEOGRAPH:'F7CF:63439:&#x9FA2
CJK UNIFIED IDEOGRAPH:'F7D0:63440:&#x513D
CJK UNIFIED IDEOGRAPH:'F7D1:63441:&#x5299
CJK UNIFIED IDEOGRAPH:'F7D2:63442:&#x58E8
CJK UNIFIED IDEOGRAPH:'F7D3:63443:&#x58E7
CJK UNIFIED IDEOGRAPH:'F7D4:63444:&#x5972
CJK UNIFIED IDEOGRAPH:'F7D5:63445:&#x5B4D
CJK UNIFIED IDEOGRAPH:'F7D6:63446:&#x5DD8
CJK UNIFIED IDEOGRAPH:'F7D7:63447:&#x882F
CJK UNIFIED IDEOGRAPH:'F7D8:63448:&#x5F4F
CJK UNIFIED IDEOGRAPH:'F7D9:63449:&#x6201
CJK UNIFIED IDEOGRAPH:'F7DA:63450:&#x6203
CJK UNIFIED IDEOGRAPH:'F7DB:63451:&#x6204
CJK UNIFIED IDEOGRAPH:'F7DC:63452:&#x6529
CJK UNIFIED IDEOGRAPH:'F7DD:63453:&#x6525
CJK UNIFIED IDEOGRAPH:'F7DE:63454:&#x6596
CJK UNIFIED IDEOGRAPH:'F7DF:63455:&#x66EB
CJK UNIFIED IDEOGRAPH:'F7E0:63456:&#x6B11
CJK UNIFIED IDEOGRAPH:'F7E1:63457:&#x6B12
CJK UNIFIED IDEOGRAPH:'F7E2:63458:&#x6B0F
CJK UNIFIED IDEOGRAPH:'F7E3:63459:&#x6BCA
CJK UNIFIED IDEOGRAPH:'F7E4:63460:&#x705B
CJK UNIFIED IDEOGRAPH:'F7E5:63461:&#x705A
CJK UNIFIED IDEOGRAPH:'F7E6:63462:&#x7222
CJK UNIFIED IDEOGRAPH:'F7E7:63463:&#x7382
CJK UNIFIED IDEOGRAPH:'F7E8:63464:&#x7381
CJK UNIFIED IDEOGRAPH:'F7E9:63465:&#x7383
CJK UNIFIED IDEOGRAPH:'F7EA:63466:&#x7670
CJK UNIFIED IDEOGRAPH:'F7EB:63467:&#x77D4
CJK UNIFIED IDEOGRAPH:'F7EC:63468:&#x7C67
CJK UNIFIED IDEOGRAPH:'F7ED:63469:&#x7C66
CJK UNIFIED IDEOGRAPH:'F7EE:63470:&#x7E95
CJK UNIFIED IDEOGRAPH:'F7EF:63471:&#x826C
CJK UNIFIED IDEOGRAPH:'F7F0:63472:&#x863A
CJK UNIFIED IDEOGRAPH:'F7F1:63473:&#x8640
CJK UNIFIED IDEOGRAPH:'F7F2:63474:&#x8639
CJK UNIFIED IDEOGRAPH:'F7F3:63475:&#x863C
CJK UNIFIED IDEOGRAPH:'F7F4:63476:&#x8631
CJK UNIFIED IDEOGRAPH:'F7F5:63477:&#x863B
CJK UNIFIED IDEOGRAPH:'F7F6:63478:&#x863E
CJK UNIFIED IDEOGRAPH:'F7F7:63479:&#x8830
CJK UNIFIED IDEOGRAPH:'F7F8:63480:&#x8832
CJK UNIFIED IDEOGRAPH:'F7F9:63481:&#x882E
CJK UNIFIED IDEOGRAPH:'F7FA:63482:&#x8833
CJK UNIFIED IDEOGRAPH:'F7FB:63483:&#x8976
CJK UNIFIED IDEOGRAPH:'F7FC:63484:&#x8974
CJK UNIFIED IDEOGRAPH:'F7FD:63485:&#x8973
CJK UNIFIED IDEOGRAPH:'F7FE:63486:&#x89FE
CJK UNIFIED IDEOGRAPH:'F840:63552:&#x8B8C
CJK UNIFIED IDEOGRAPH:'F841:63553:&#x8B8E
CJK UNIFIED IDEOGRAPH:'F842:63554:&#x8B8B
CJK UNIFIED IDEOGRAPH:'F843:63555:&#x8B88
CJK UNIFIED IDEOGRAPH:'F844:63556:&#x8C45
CJK UNIFIED IDEOGRAPH:'F845:63557:&#x8D19
CJK UNIFIED IDEOGRAPH:'F846:63558:&#x8E98
CJK UNIFIED IDEOGRAPH:'F847:63559:&#x8F64
CJK UNIFIED IDEOGRAPH:'F848:63560:&#x8F63
CJK UNIFIED IDEOGRAPH:'F849:63561:&#x91BC
CJK UNIFIED IDEOGRAPH:'F84A:63562:&#x9462
CJK UNIFIED IDEOGRAPH:'F84B:63563:&#x9455
CJK UNIFIED IDEOGRAPH:'F84C:63564:&#x945D
CJK UNIFIED IDEOGRAPH:'F84D:63565:&#x9457
CJK UNIFIED IDEOGRAPH:'F84E:63566:&#x945E
CJK UNIFIED IDEOGRAPH:'F84F:63567:&#x97C4
CJK UNIFIED IDEOGRAPH:'F850:63568:&#x97C5
CJK UNIFIED IDEOGRAPH:'F851:63569:&#x9800
CJK UNIFIED IDEOGRAPH:'F852:63570:&#x9A56
CJK UNIFIED IDEOGRAPH:'F853:63571:&#x9A59
CJK UNIFIED IDEOGRAPH:'F854:63572:&#x9B1E
CJK UNIFIED IDEOGRAPH:'F855:63573:&#x9B1F
CJK UNIFIED IDEOGRAPH:'F856:63574:&#x9B20
CJK UNIFIED IDEOGRAPH:'F857:63575:&#x9C52
CJK UNIFIED IDEOGRAPH:'F858:63576:&#x9C58
CJK UNIFIED IDEOGRAPH:'F859:63577:&#x9C50
CJK UNIFIED IDEOGRAPH:'F85A:63578:&#x9C4A
CJK UNIFIED IDEOGRAPH:'F85B:63579:&#x9C4D
CJK UNIFIED IDEOGRAPH:'F85C:63580:&#x9C4B
CJK UNIFIED IDEOGRAPH:'F85D:63581:&#x9C55
CJK UNIFIED IDEOGRAPH:'F85E:63582:&#x9C59
CJK UNIFIED IDEOGRAPH:'F85F:63583:&#x9C4C
CJK UNIFIED IDEOGRAPH:'F860:63584:&#x9C4E
CJK UNIFIED IDEOGRAPH:'F861:63585:&#x9DFB
CJK UNIFIED IDEOGRAPH:'F862:63586:&#x9DF7
CJK UNIFIED IDEOGRAPH:'F863:63587:&#x9DEF
CJK UNIFIED IDEOGRAPH:'F864:63588:&#x9DE3
CJK UNIFIED IDEOGRAPH:'F865:63589:&#x9DEB
CJK UNIFIED IDEOGRAPH:'F866:63590:&#x9DF8
CJK UNIFIED IDEOGRAPH:'F867:63591:&#x9DE4
CJK UNIFIED IDEOGRAPH:'F868:63592:&#x9DF6
CJK UNIFIED IDEOGRAPH:'F869:63593:&#x9DE1
CJK UNIFIED IDEOGRAPH:'F86A:63594:&#x9DEE
CJK UNIFIED IDEOGRAPH:'F86B:63595:&#x9DE6
CJK UNIFIED IDEOGRAPH:'F86C:63596:&#x9DF2
CJK UNIFIED IDEOGRAPH:'F86D:63597:&#x9DF0
CJK UNIFIED IDEOGRAPH:'F86E:63598:&#x9DE2
CJK UNIFIED IDEOGRAPH:'F86F:63599:&#x9DEC
CJK UNIFIED IDEOGRAPH:'F870:63600:&#x9DF4
CJK UNIFIED IDEOGRAPH:'F871:63601:&#x9DF3
CJK UNIFIED IDEOGRAPH:'F872:63602:&#x9DE8
CJK UNIFIED IDEOGRAPH:'F873:63603:&#x9DED
CJK UNIFIED IDEOGRAPH:'F874:63604:&#x9EC2
CJK UNIFIED IDEOGRAPH:'F875:63605:&#x9ED0
CJK UNIFIED IDEOGRAPH:'F876:63606:&#x9EF2
CJK UNIFIED IDEOGRAPH:'F877:63607:&#x9EF3
CJK UNIFIED IDEOGRAPH:'F878:63608:&#x9F06
CJK UNIFIED IDEOGRAPH:'F879:63609:&#x9F1C
CJK UNIFIED IDEOGRAPH:'F87A:63610:&#x9F38
CJK UNIFIED IDEOGRAPH:'F87B:63611:&#x9F37
CJK UNIFIED IDEOGRAPH:'F87C:63612:&#x9F36
CJK UNIFIED IDEOGRAPH:'F87D:63613:&#x9F43
CJK UNIFIED IDEOGRAPH:'F87E:63614:&#x9F4F
CJK UNIFIED IDEOGRAPH:'F8A1:63649:&#x9F71
CJK UNIFIED IDEOGRAPH:'F8A2:63650:&#x9F70
CJK UNIFIED IDEOGRAPH:'F8A3:63651:&#x9F6E
CJK UNIFIED IDEOGRAPH:'F8A4:63652:&#x9F6F
CJK UNIFIED IDEOGRAPH:'F8A5:63653:&#x56D3
CJK UNIFIED IDEOGRAPH:'F8A6:63654:&#x56CD
CJK UNIFIED IDEOGRAPH:'F8A7:63655:&#x5B4E
CJK UNIFIED IDEOGRAPH:'F8A8:63656:&#x5C6D
CJK UNIFIED IDEOGRAPH:'F8A9:63657:&#x652D
CJK UNIFIED IDEOGRAPH:'F8AA:63658:&#x66ED
CJK UNIFIED IDEOGRAPH:'F8AB:63659:&#x66EE
CJK UNIFIED IDEOGRAPH:'F8AC:63660:&#x6B13
CJK UNIFIED IDEOGRAPH:'F8AD:63661:&#x705F
CJK UNIFIED IDEOGRAPH:'F8AE:63662:&#x7061
CJK UNIFIED IDEOGRAPH:'F8AF:63663:&#x705D
CJK UNIFIED IDEOGRAPH:'F8B0:63664:&#x7060
CJK UNIFIED IDEOGRAPH:'F8B1:63665:&#x7223
CJK UNIFIED IDEOGRAPH:'F8B2:63666:&#x74DB
CJK UNIFIED IDEOGRAPH:'F8B3:63667:&#x74E5
CJK UNIFIED IDEOGRAPH:'F8B4:63668:&#x77D5
CJK UNIFIED IDEOGRAPH:'F8B5:63669:&#x7938
CJK UNIFIED IDEOGRAPH:'F8B6:63670:&#x79B7
CJK UNIFIED IDEOGRAPH:'F8B7:63671:&#x79B6
CJK UNIFIED IDEOGRAPH:'F8B8:63672:&#x7C6A
CJK UNIFIED IDEOGRAPH:'F8B9:63673:&#x7E97
CJK UNIFIED IDEOGRAPH:'F8BA:63674:&#x7F89
CJK UNIFIED IDEOGRAPH:'F8BB:63675:&#x826D
CJK UNIFIED IDEOGRAPH:'F8BC:63676:&#x8643
CJK UNIFIED IDEOGRAPH:'F8BD:63677:&#x8838
CJK UNIFIED IDEOGRAPH:'F8BE:63678:&#x8837
CJK UNIFIED IDEOGRAPH:'F8BF:63679:&#x8835
CJK UNIFIED IDEOGRAPH:'F8C0:63680:&#x884B
CJK UNIFIED IDEOGRAPH:'F8C1:63681:&#x8B94
CJK UNIFIED IDEOGRAPH:'F8C2:63682:&#x8B95
CJK UNIFIED IDEOGRAPH:'F8C3:63683:&#x8E9E
CJK UNIFIED IDEOGRAPH:'F8C4:63684:&#x8E9F
CJK UNIFIED IDEOGRAPH:'F8C5:63685:&#x8EA0
CJK UNIFIED IDEOGRAPH:'F8C6:63686:&#x8E9D
CJK UNIFIED IDEOGRAPH:'F8C7:63687:&#x91BE
CJK UNIFIED IDEOGRAPH:'F8C8:63688:&#x91BD
CJK UNIFIED IDEOGRAPH:'F8C9:63689:&#x91C2
CJK UNIFIED IDEOGRAPH:'F8CA:63690:&#x946B
CJK UNIFIED IDEOGRAPH:'F8CB:63691:&#x9468
CJK UNIFIED IDEOGRAPH:'F8CC:63692:&#x9469
CJK UNIFIED IDEOGRAPH:'F8CD:63693:&#x96E5
CJK UNIFIED IDEOGRAPH:'F8CE:63694:&#x9746
CJK UNIFIED IDEOGRAPH:'F8CF:63695:&#x9743
CJK UNIFIED IDEOGRAPH:'F8D0:63696:&#x9747
CJK UNIFIED IDEOGRAPH:'F8D1:63697:&#x97C7
CJK UNIFIED IDEOGRAPH:'F8D2:63698:&#x97E5
CJK UNIFIED IDEOGRAPH:'F8D3:63699:&#x9A5E
CJK UNIFIED IDEOGRAPH:'F8D4:63700:&#x9AD5
CJK UNIFIED IDEOGRAPH:'F8D5:63701:&#x9B59
CJK UNIFIED IDEOGRAPH:'F8D6:63702:&#x9C63
CJK UNIFIED IDEOGRAPH:'F8D7:63703:&#x9C67
CJK UNIFIED IDEOGRAPH:'F8D8:63704:&#x9C66
CJK UNIFIED IDEOGRAPH:'F8D9:63705:&#x9C62
CJK UNIFIED IDEOGRAPH:'F8DA:63706:&#x9C5E
CJK UNIFIED IDEOGRAPH:'F8DB:63707:&#x9C60
CJK UNIFIED IDEOGRAPH:'F8DC:63708:&#x9E02
CJK UNIFIED IDEOGRAPH:'F8DD:63709:&#x9DFE
CJK UNIFIED IDEOGRAPH:'F8DE:63710:&#x9E07
CJK UNIFIED IDEOGRAPH:'F8DF:63711:&#x9E03
CJK UNIFIED IDEOGRAPH:'F8E0:63712:&#x9E06
CJK UNIFIED IDEOGRAPH:'F8E1:63713:&#x9E05
CJK UNIFIED IDEOGRAPH:'F8E2:63714:&#x9E00
CJK UNIFIED IDEOGRAPH:'F8E3:63715:&#x9E01
CJK UNIFIED IDEOGRAPH:'F8E4:63716:&#x9E09
CJK UNIFIED IDEOGRAPH:'F8E5:63717:&#x9DFF
CJK UNIFIED IDEOGRAPH:'F8E6:63718:&#x9DFD
CJK UNIFIED IDEOGRAPH:'F8E7:63719:&#x9E04
CJK UNIFIED IDEOGRAPH:'F8E8:63720:&#x9EA0
CJK UNIFIED IDEOGRAPH:'F8E9:63721:&#x9F1E
CJK UNIFIED IDEOGRAPH:'F8EA:63722:&#x9F46
CJK UNIFIED IDEOGRAPH:'F8EB:63723:&#x9F74
CJK UNIFIED IDEOGRAPH:'F8EC:63724:&#x9F75
CJK UNIFIED IDEOGRAPH:'F8ED:63725:&#x9F76
CJK UNIFIED IDEOGRAPH:'F8EE:63726:&#x56D4
CJK UNIFIED IDEOGRAPH:'F8EF:63727:&#x652E
CJK UNIFIED IDEOGRAPH:'F8F0:63728:&#x65B8
CJK UNIFIED IDEOGRAPH:'F8F1:63729:&#x6B18
CJK UNIFIED IDEOGRAPH:'F8F2:63730:&#x6B19
CJK UNIFIED IDEOGRAPH:'F8F3:63731:&#x6B17
CJK UNIFIED IDEOGRAPH:'F8F4:63732:&#x6B1A
CJK UNIFIED IDEOGRAPH:'F8F5:63733:&#x7062
CJK UNIFIED IDEOGRAPH:'F8F6:63734:&#x7226
CJK UNIFIED IDEOGRAPH:'F8F7:63735:&#x72AA
CJK UNIFIED IDEOGRAPH:'F8F8:63736:&#x77D8
CJK UNIFIED IDEOGRAPH:'F8F9:63737:&#x77D9
CJK UNIFIED IDEOGRAPH:'F8FA:63738:&#x7939
CJK UNIFIED IDEOGRAPH:'F8FB:63739:&#x7C69
CJK UNIFIED IDEOGRAPH:'F8FC:63740:&#x7C6B
CJK UNIFIED IDEOGRAPH:'F8FD:63741:&#x7CF6
CJK UNIFIED IDEOGRAPH:'F8FE:63742:&#x7E9A
CJK UNIFIED IDEOGRAPH:'F940:63808:&#x7E98
CJK UNIFIED IDEOGRAPH:'F941:63809:&#x7E9B
CJK UNIFIED IDEOGRAPH:'F942:63810:&#x7E99
CJK UNIFIED IDEOGRAPH:'F943:63811:&#x81E0
CJK UNIFIED IDEOGRAPH:'F944:63812:&#x81E1
CJK UNIFIED IDEOGRAPH:'F945:63813:&#x8646
CJK UNIFIED IDEOGRAPH:'F946:63814:&#x8647
CJK UNIFIED IDEOGRAPH:'F947:63815:&#x8648
CJK UNIFIED IDEOGRAPH:'F948:63816:&#x8979
CJK UNIFIED IDEOGRAPH:'F949:63817:&#x897A
CJK UNIFIED IDEOGRAPH:'F94A:63818:&#x897C
CJK UNIFIED IDEOGRAPH:'F94B:63819:&#x897B
CJK UNIFIED IDEOGRAPH:'F94C:63820:&#x89FF
CJK UNIFIED IDEOGRAPH:'F94D:63821:&#x8B98
CJK UNIFIED IDEOGRAPH:'F94E:63822:&#x8B99
CJK UNIFIED IDEOGRAPH:'F94F:63823:&#x8EA5
CJK UNIFIED IDEOGRAPH:'F950:63824:&#x8EA4
CJK UNIFIED IDEOGRAPH:'F951:63825:&#x8EA3
CJK UNIFIED IDEOGRAPH:'F952:63826:&#x946E
CJK UNIFIED IDEOGRAPH:'F953:63827:&#x946D
CJK UNIFIED IDEOGRAPH:'F954:63828:&#x946F
CJK UNIFIED IDEOGRAPH:'F955:63829:&#x9471
CJK UNIFIED IDEOGRAPH:'F956:63830:&#x9473
CJK UNIFIED IDEOGRAPH:'F957:63831:&#x9749
CJK UNIFIED IDEOGRAPH:'F958:63832:&#x9872
CJK UNIFIED IDEOGRAPH:'F959:63833:&#x995F
CJK UNIFIED IDEOGRAPH:'F95A:63834:&#x9C68
CJK UNIFIED IDEOGRAPH:'F95B:63835:&#x9C6E
CJK UNIFIED IDEOGRAPH:'F95C:63836:&#x9C6D
CJK UNIFIED IDEOGRAPH:'F95D:63837:&#x9E0B
CJK UNIFIED IDEOGRAPH:'F95E:63838:&#x9E0D
CJK UNIFIED IDEOGRAPH:'F95F:63839:&#x9E10
CJK UNIFIED IDEOGRAPH:'F960:63840:&#x9E0F
CJK UNIFIED IDEOGRAPH:'F961:63841:&#x9E12
CJK UNIFIED IDEOGRAPH:'F962:63842:&#x9E11
CJK UNIFIED IDEOGRAPH:'F963:63843:&#x9EA1
CJK UNIFIED IDEOGRAPH:'F964:63844:&#x9EF5
CJK UNIFIED IDEOGRAPH:'F965:63845:&#x9F09
CJK UNIFIED IDEOGRAPH:'F966:63846:&#x9F47
CJK UNIFIED IDEOGRAPH:'F967:63847:&#x9F78
CJK UNIFIED IDEOGRAPH:'F968:63848:&#x9F7B
CJK UNIFIED IDEOGRAPH:'F969:63849:&#x9F7A
CJK UNIFIED IDEOGRAPH:'F96A:63850:&#x9F79
CJK UNIFIED IDEOGRAPH:'F96B:63851:&#x571E
CJK UNIFIED IDEOGRAPH:'F96C:63852:&#x7066
CJK UNIFIED IDEOGRAPH:'F96D:63853:&#x7C6F
CJK UNIFIED IDEOGRAPH:'F96E:63854:&#x883C
CJK UNIFIED IDEOGRAPH:'F96F:63855:&#x8DB2
CJK UNIFIED IDEOGRAPH:'F970:63856:&#x8EA6
CJK UNIFIED IDEOGRAPH:'F971:63857:&#x91C3
CJK UNIFIED IDEOGRAPH:'F972:63858:&#x9474
CJK UNIFIED IDEOGRAPH:'F973:63859:&#x9478
CJK UNIFIED IDEOGRAPH:'F974:63860:&#x9476
CJK UNIFIED IDEOGRAPH:'F975:63861:&#x9475
CJK UNIFIED IDEOGRAPH:'F976:63862:&#x9A60
CJK UNIFIED IDEOGRAPH:'F977:63863:&#x9C74
CJK UNIFIED IDEOGRAPH:'F978:63864:&#x9C73
CJK UNIFIED IDEOGRAPH:'F979:63865:&#x9C71
CJK UNIFIED IDEOGRAPH:'F97A:63866:&#x9C75
CJK UNIFIED IDEOGRAPH:'F97B:63867:&#x9E14
CJK UNIFIED IDEOGRAPH:'F97C:63868:&#x9E13
CJK UNIFIED IDEOGRAPH:'F97D:63869:&#x9EF6
CJK UNIFIED IDEOGRAPH:'F97E:63870:&#x9F0A
CJK UNIFIED IDEOGRAPH:'F9A1:63905:&#x9FA4
CJK UNIFIED IDEOGRAPH:'F9A2:63906:&#x7068
CJK UNIFIED IDEOGRAPH:'F9A3:63907:&#x7065
CJK UNIFIED IDEOGRAPH:'F9A4:63908:&#x7CF7
CJK UNIFIED IDEOGRAPH:'F9A5:63909:&#x866A
CJK UNIFIED IDEOGRAPH:'F9A6:63910:&#x883E
CJK UNIFIED IDEOGRAPH:'F9A7:63911:&#x883D
CJK UNIFIED IDEOGRAPH:'F9A8:63912:&#x883F
CJK UNIFIED IDEOGRAPH:'F9A9:63913:&#x8B9E
CJK UNIFIED IDEOGRAPH:'F9AA:63914:&#x8C9C
CJK UNIFIED IDEOGRAPH:'F9AB:63915:&#x8EA9
CJK UNIFIED IDEOGRAPH:'F9AC:63916:&#x8EC9
CJK UNIFIED IDEOGRAPH:'F9AD:63917:&#x974B
CJK UNIFIED IDEOGRAPH:'F9AE:63918:&#x9873
CJK UNIFIED IDEOGRAPH:'F9AF:63919:&#x9874
CJK UNIFIED IDEOGRAPH:'F9B0:63920:&#x98CC
CJK UNIFIED IDEOGRAPH:'F9B1:63921:&#x9961
CJK UNIFIED IDEOGRAPH:'F9B2:63922:&#x99AB
CJK UNIFIED IDEOGRAPH:'F9B3:63923:&#x9A64
CJK UNIFIED IDEOGRAPH:'F9B4:63924:&#x9A66
CJK UNIFIED IDEOGRAPH:'F9B5:63925:&#x9A67
CJK UNIFIED IDEOGRAPH:'F9B6:63926:&#x9B24
CJK UNIFIED IDEOGRAPH:'F9B7:63927:&#x9E15
CJK UNIFIED IDEOGRAPH:'F9B8:63928:&#x9E17
CJK UNIFIED IDEOGRAPH:'F9B9:63929:&#x9F48
CJK UNIFIED IDEOGRAPH:'F9BA:63930:&#x6207
CJK UNIFIED IDEOGRAPH:'F9BB:63931:&#x6B1E
CJK UNIFIED IDEOGRAPH:'F9BC:63932:&#x7227
CJK UNIFIED IDEOGRAPH:'F9BD:63933:&#x864C
CJK UNIFIED IDEOGRAPH:'F9BE:63934:&#x8EA8
CJK UNIFIED IDEOGRAPH:'F9BF:63935:&#x9482
CJK UNIFIED IDEOGRAPH:'F9C0:63936:&#x9480
CJK UNIFIED IDEOGRAPH:'F9C1:63937:&#x9481
CJK UNIFIED IDEOGRAPH:'F9C2:63938:&#x9A69
CJK UNIFIED IDEOGRAPH:'F9C3:63939:&#x9A68
CJK UNIFIED IDEOGRAPH:'F9C4:63940:&#x9B2E
CJK UNIFIED IDEOGRAPH:'F9C5:63941:&#x9E19
CJK UNIFIED IDEOGRAPH:'F9C6:63942:&#x7229
CJK UNIFIED IDEOGRAPH:'F9C7:63943:&#x864B
CJK UNIFIED IDEOGRAPH:'F9C8:63944:&#x8B9F
CJK UNIFIED IDEOGRAPH:'F9C9:63945:&#x9483
CJK UNIFIED IDEOGRAPH:'F9CA:63946:&#x9C79
CJK UNIFIED IDEOGRAPH:'F9CB:63947:&#x9EB7
CJK UNIFIED IDEOGRAPH:'F9CC:63948:&#x7675
CJK UNIFIED IDEOGRAPH:'F9CD:63949:&#x9A6B
CJK UNIFIED IDEOGRAPH:'F9CE:63950:&#x9C7A
CJK UNIFIED IDEOGRAPH:'F9CF:63951:&#x9E1D
CJK UNIFIED IDEOGRAPH:'F9D0:63952:&#x7069
CJK UNIFIED IDEOGRAPH:'F9D1:63953:&#x706A
CJK UNIFIED IDEOGRAPH:'F9D2:63954:&#x9EA4
CJK UNIFIED IDEOGRAPH:'F9D3:63955:&#x9F7E
CJK UNIFIED IDEOGRAPH:'F9D4:63956:&#x9F49
CJK UNIFIED IDEOGRAPH:'F9D5:63957:&#x9F98
CJK UNIFIED IDEOGRAPH:'F9D6:63958:&#x7881
CJK UNIFIED IDEOGRAPH:'F9D7:63959:&#x92B9
CJK UNIFIED IDEOGRAPH:'F9D8:63960:&#x88CF
CJK UNIFIED IDEOGRAPH:'F9D9:63961:&#x58BB
CJK UNIFIED IDEOGRAPH:'F9DA:63962:&#x6052
CJK UNIFIED IDEOGRAPH:'F9DB:63963:&#x7CA7
CJK UNIFIED IDEOGRAPH:'F9DC:63964:&#x5AFA
BOX DRAWINGS DOUBLE DOWN AND RIGHT:'F9DD:63965:&#x2554
BOX DRAWINGS DOUBLE DOWN AND HORIZONTAL:'F9DE:63966:&#x2566
BOX DRAWINGS DOUBLE DOWN AND LEFT:'F9DF:63967:&#x2557
BOX DRAWINGS DOUBLE VERTICAL AND RIGHT:'F9E0:63968:&#x2560
BOX DRAWINGS DOUBLE VERTICAL AND HORIZONTAL:'F9E1:63969:&#x256C
BOX DRAWINGS DOUBLE VERTICAL AND LEFT:'F9E2:63970:&#x2563
BOX DRAWINGS DOUBLE UP AND RIGHT:'F9E3:63971:&#x255A
BOX DRAWINGS DOUBLE UP AND HORIZONTAL:'F9E4:63972:&#x2569
BOX DRAWINGS DOUBLE UP AND LEFT:'F9E5:63973:&#x255D
BOX DRAWINGS DOWN SINGLE AND RIGHT DOUBLE:'F9E6:63974:&#x2552
BOX DRAWINGS DOWN SINGLE AND HORIZONTAL DOUBLE:'F9E7:63975:&#x2564
BOX DRAWINGS DOWN SINGLE AND LEFT DOUBLE:'F9E8:63976:&#x2555
BOX DRAWINGS VERTICAL SINGLE AND RIGHT DOUBLE:'F9E9:63977:&#x255E
BOX DRAWINGS VERTICAL SINGLE AND HORIZONTAL DOUBLE:'F9EA:63978:&#x256A
BOX DRAWINGS VERTICAL SINGLE AND LEFT DOUBLE:'F9EB:63979:&#x2561
BOX DRAWINGS UP SINGLE AND RIGHT DOUBLE:'F9EC:63980:&#x2558
BOX DRAWINGS UP SINGLE AND HORIZONTAL DOUBLE:'F9ED:63981:&#x2567
BOX DRAWINGS UP SINGLE AND LEFT DOUBLE:'F9EE:63982:&#x255B
BOX DRAWINGS DOWN DOUBLE AND RIGHT SINGLE:'F9EF:63983:&#x2553
BOX DRAWINGS DOWN DOUBLE AND HORIZONTAL SINGLE:'F9F0:63984:&#x2565
BOX DRAWINGS DOWN DOUBLE AND LEFT SINGLE:'F9F1:63985:&#x2556
BOX DRAWINGS VERTICAL DOUBLE AND RIGHT SINGLE:'F9F2:63986:&#x255F
BOX DRAWINGS VERTICAL DOUBLE AND HORIZONTAL SINGLE:'F9F3:63987:&#x256B
BOX DRAWINGS VERTICAL DOUBLE AND LEFT SINGLE:'F9F4:63988:&#x2562
BOX DRAWINGS UP DOUBLE AND RIGHT SINGLE:'F9F5:63989:&#x2559
BOX DRAWINGS UP DOUBLE AND HORIZONTAL SINGLE:'F9F6:63990:&#x2568
BOX DRAWINGS UP DOUBLE AND LEFT SINGLE:'F9F7:63991:&#x255C
BOX DRAWINGS DOUBLE VERTICAL:'F9F8:63992:&#x2551
BOX DRAWINGS DOUBLE HORIZONTAL:'F9F9:63993:&#x2550
BOX DRAWINGS LIGHT ARC DOWN AND RIGHT:'F9FA:63994:&#x256D
BOX DRAWINGS LIGHT ARC DOWN AND LEFT:'F9FB:63995:&#x256E
BOX DRAWINGS LIGHT ARC UP AND RIGHT:'F9FC:63996:&#x2570
BOX DRAWINGS LIGHT ARC UP AND LEFT:'F9FD:63997:&#x256F
DARK SHADE:'F9FE:63998:&#x2593
</ansicpg950>
<ansicpg1250>
SINGLE LOW-9 QUOTATION MARK:'82:8218:&#x201A;
DOUBLE LOW-9 QUOTATION MARK:'84:8222:&#x201E;
HORIZONTAL ELLIPSIS:'85:8230:&#x2026;
SINGLE LOW-9 QUOTATION MARK:'82:8218:&#x201A;
DOUBLE LOW-9 QUOTATION MARK:'84:8222:&#x201E;
HORIZONTAL ELLIPSIS:'85:8230:&#x2026;
DAGGER:'86:8224:&#x2020;
DOUBLE DAGGER:'87:8225:&#x2021;
PER MILLE SIGN:'89:8240:&#x2030;
LATIN CAPITAL LETTER S WITH CARON:'8A:352:&#x0160;
SINGLE LEFT-POINTING ANGLE QUOTATION MARK:'8B:8249:&#x2039;
LATIN CAPITAL LETTER S WITH ACUTE:'8C:346:&#x015A;
LATIN CAPITAL LETTER T WITH CARON:'8D:356:&#x0164;
LATIN CAPITAL LETTER Z WITH CARON:'8E:381:&#x017D;
LATIN CAPITAL LETTER Z WITH ACUTE:'8F:377:&#x0179;
LEFT SINGLE QUOTATION MARK:'91:8216:&#x2018;
RIGHT SINGLE QUOTATION MARK:'92:8217:&#x2019;
LEFT DOUBLE QUOTATION MARK:'93:8220:&#x201C;
RIGHT DOUBLE QUOTATION MARK:'94:8221:&#x201D;
BULLET:'95:8226:&#x2022;
EN DASH:'96:8211:&#x2013;
EM DASH:'97:8212:&#x2014;
TRADE MARK SIGN:'99:8482:&#x2122;
LATIN SMALL LETTER S WITH CARON:'9A:353:&#x0161;
SINGLE RIGHT-POINTING ANGLE QUOTATION MARK:'9B:8250:&#x203A;
LATIN SMALL LETTER S WITH ACUTE:'9C:347:&#x015B;
LATIN SMALL LETTER T WITH CARON:'9D:357:&#x0165;
LATIN SMALL LETTER Z WITH CARON:'9E:382:&#x017E;
LATIN SMALL LETTER Z WITH ACUTE:'9F:378:&#x017A;
NO-BREAK SPACE:'A0:160:&#x00A0;
CARON (MANDARIN CHINESE THIRD TONE):'A1:711:&#x02C7;
BREVE:'A2:728:&#x02D8;
LATIN CAPITAL LETTER L WITH STROKE:'A3:321:&#x0141;
CURRENCY SIGN:'A4:164:&#x00A4;
LATIN CAPITAL LETTER A WITH OGONEK:'A5:260:&#x0104;
BROKEN BAR:'A6:166:&#x00A6;
SECTION SIGN:'A7:167:&#x00A7;
DIAERESIS:'A8:168:&#x00A8;
COPYRIGHT SIGN:'A9:169:&#x00A9;
LATIN CAPITAL LETTER S WITH CEDILLA:'AA:350:&#x015E;
LEFT-POINTING DOUBLE ANGLE QUOTATION MARK:'AB:171:&#x00AB;
NOT SIGN:'AC:172:&#x00AC;
SOFT HYPHEN:'AD:173:&#x00AD;
REGISTERED SIGN:'AE:174:&#x00AE;
LATIN CAPITAL LETTER Z WITH DOT ABOVE:'AF:379:&#x017B;
DEGREE SIGN:'B0:176:&#x00B0;
PLUS-MINUS SIGN:'B1:177:&#x00B1;
OGONEK:'B2:731:&#x02DB;
LATIN SMALL LETTER L WITH STROKE:'B3:322:&#x0142;
ACUTE ACCENT:'B4:180:&#x00B4;
MICRO SIGN:'B5:181:&#x00B5;
PILCROW SIGN:'B6:182:&#x00B6;
MIDDLE DOT:'B7:183:&#x00B7;
CEDILLA:'B8:184:&#x00B8;
LATIN SMALL LETTER A WITH OGONEK:'B9:261:&#x0105;
LATIN SMALL LETTER S WITH CEDILLA:'BA:351:&#x015F;
RIGHT-POINTING DOUBLE ANGLE QUOTATION MARK:'BB:187:&#x00BB;
LATIN CAPITAL LETTER L WITH CARON:'BC:317:&#x013D;
DOUBLE ACUTE ACCENT:'BD:733:&#x02DD;
LATIN SMALL LETTER L WITH CARON:'BE:318:&#x013E;
LATIN SMALL LETTER Z WITH DOT ABOVE:'BF:380:&#x017C;
LATIN CAPITAL LETTER R WITH ACUTE:'C0:340:&#x0154;
LATIN CAPITAL LETTER A WITH ACUTE:'C1:193:&#x00C1;
LATIN CAPITAL LETTER A WITH CIRCUMFLEX:'C2:194:&#x00C2;
LATIN CAPITAL LETTER A WITH BREVE:'C3:258:&#x0102;
LATIN CAPITAL LETTER A WITH DIAERESIS:'C4:196:&#x00C4;
LATIN CAPITAL LETTER L WITH ACUTE:'C5:313:&#x0139;
LATIN CAPITAL LETTER C WITH ACUTE:'C6:262:&#x0106;
LATIN CAPITAL LETTER C WITH CEDILLA:'C7:199:&#x00C7;
LATIN CAPITAL LETTER C WITH CARON:'C8:268:&#x010C;
LATIN CAPITAL LETTER E WITH ACUTE:'C9:201:&#x00C9;
LATIN CAPITAL LETTER E WITH OGONEK:'CA:280:&#x0118;
LATIN CAPITAL LETTER E WITH DIAERESIS:'CB:203:&#x00CB;
LATIN CAPITAL LETTER E WITH CARON:'CC:282:&#x011A;
LATIN CAPITAL LETTER I WITH ACUTE:'CD:205:&#x00CD;
LATIN CAPITAL LETTER I WITH CIRCUMFLEX:'CE:206:&#x00CE;
LATIN CAPITAL LETTER D WITH CARON:'CF:270:&#x010E;
LATIN CAPITAL LETTER D WITH STROKE:'D0:272:&#x0110;
LATIN CAPITAL LETTER N WITH ACUTE:'D1:323:&#x0143;
LATIN CAPITAL LETTER N WITH CARON:'D2:327:&#x0147;
LATIN CAPITAL LETTER O WITH ACUTE:'D3:211:&#x00D3;
LATIN CAPITAL LETTER O WITH CIRCUMFLEX:'D4:212:&#x00D4;
LATIN CAPITAL LETTER O WITH DOUBLE ACUTE:'D5:336:&#x0150;
LATIN CAPITAL LETTER O WITH DIAERESIS:'D6:214:&#x00D6;
MULTIPLICATION SIGN:'D7:215:&#x00D7;
LATIN CAPITAL LETTER R WITH CARON:'D8:344:&#x0158;
LATIN CAPITAL LETTER U WITH RING ABOVE:'D9:366:&#x016E;
LATIN CAPITAL LETTER U WITH ACUTE:'DA:218:&#x00DA;
LATIN CAPITAL LETTER U WITH DOUBLE ACUTE:'DB:368:&#x0170;
LATIN CAPITAL LETTER U WITH DIAERESIS:'DC:220:&#x00DC;
LATIN CAPITAL LETTER Y WITH ACUTE:'DD:221:&#x00DD;
LATIN CAPITAL LETTER T WITH CEDILLA:'DE:354:&#x0162;
LATIN SMALL LETTER SHARP S (GERMAN):'DF:223:&#x00DF;
LATIN SMALL LETTER R WITH ACUTE:'E0:341:&#x0155;
LATIN SMALL LETTER A WITH ACUTE:'E1:225:&#x00E1;
LATIN SMALL LETTER A WITH CIRCUMFLEX:'E2:226:&#x00E2;
LATIN SMALL LETTER A WITH BREVE:'E3:259:&#x0103;
LATIN SMALL LETTER A WITH DIAERESIS:'E4:228:&#x00E4;
LATIN SMALL LETTER L WITH ACUTE:'E5:314:&#x013A;
LATIN SMALL LETTER C WITH ACUTE:'E6:263:&#x0107;
LATIN SMALL LETTER C WITH CEDILLA:'E7:231:&#x00E7;
LATIN SMALL LETTER C WITH CARON:'E8:269:&#x010D;
LATIN SMALL LETTER E WITH ACUTE:'E9:233:&#x00E9;
LATIN SMALL LETTER E WITH OGONEK:'EA:281:&#x0119;
LATIN SMALL LETTER E WITH DIAERESIS:'EB:235:&#x00EB;
LATIN SMALL LETTER E WITH CARON:'EC:283:&#x011B;
LATIN SMALL LETTER I WITH ACUTE:'ED:237:&#x00ED;
LATIN SMALL LETTER I WITH CIRCUMFLEX:'EE:238:&#x00EE;
LATIN SMALL LETTER D WITH CARON:'EF:271:&#x010F;
LATIN SMALL LETTER D WITH STROKE:'F0:273:&#x0111;
LATIN SMALL LETTER N WITH ACUTE:'F1:324:&#x0144;
LATIN SMALL LETTER N WITH CARON:'F2:328:&#x0148;
LATIN SMALL LETTER O WITH ACUTE:'F3:243:&#x00F3;
LATIN SMALL LETTER O WITH CIRCUMFLEX:'F4:244:&#x00F4;
LATIN SMALL LETTER O WITH DOUBLE ACUTE:'F5:337:&#x0151;
LATIN SMALL LETTER O WITH DIAERESIS:'F6:246:&#x00F6;
DIVISION SIGN:'F7:247:&#x00F7;
LATIN SMALL LETTER R WITH CARON:'F8:345:&#x0159;
LATIN SMALL LETTER U WITH RING ABOVE:'F9:367:&#x016F;
LATIN SMALL LETTER U WITH ACUTE:'FA:250:&#x00FA;
LATIN SMALL LETTER U WITH DOUBLE ACUTE:'FB:369:&#x0171;
LATIN SMALL LETTER U WITH DIAERESIS:'FC:252:&#x00FC;
LATIN SMALL LETTER Y WITH ACUTE:'FD:253:&#x00FD;
LATIN SMALL LETTER T WITH CEDILLA:'FE:355:&#x0163;
DOT ABOVE (MANDARIN CHINESE LIGHT TONE):'FF:729:&#x02D9;
</ansicpg1250>
<ansicpg1251>
CYRILLIC CAPITAL LETTER DJE (SERBOCROATIAN):'80:1026:&#x0402;
CYRILLIC CAPITAL LETTER GJE:'81:1027:&#x0403;
SINGLE LOW-9 QUOTATION MARK:'82:8218:&#x201A;
CYRILLIC SMALL LETTER GJE:'83:1107:&#x0453;
DOUBLE LOW-9 QUOTATION MARK:'84:8222:&#x201E;
HORIZONTAL ELLIPSIS:'85:8230:&#x2026;
DAGGER:'86:8224:&#x2020;
DOUBLE DAGGER:'87:8225:&#x2021;
PER MILLE SIGN:'89:8240:&#x2030;
CYRILLIC CAPITAL LETTER LJE:'8A:1033:&#x0409;
SINGLE LEFT-POINTING ANGLE QUOTATION MARK:'8B:8249:&#x2039;
CYRILLIC CAPITAL LETTER NJE:'8C:1034:&#x040A;
CYRILLIC CAPITAL LETTER KJE:'8D:1036:&#x040C;
CYRILLIC CAPITAL LETTER TSHE (SERBOCROATIAN):'8E:1035:&#x040B;
CYRILLIC CAPITAL LETTER DZHE:'8F:1039:&#x040F;
CYRILLIC SMALL LETTER DJE (SERBOCROATIAN):'90:1106:&#x0452;
LEFT SINGLE QUOTATION MARK:'91:8216:&#x2018;
RIGHT SINGLE QUOTATION MARK:'92:8217:&#x2019;
LEFT DOUBLE QUOTATION MARK:'93:8220:&#x201C;
RIGHT DOUBLE QUOTATION MARK:'94:8221:&#x201D;
BULLET:'95:8226:&#x2022;
EN DASH:'96:8211:&#x2013;
EM DASH:'97:8212:&#x2014;
TRADE MARK SIGN:'99:8482:&#x2122;
CYRILLIC SMALL LETTER LJE:'9A:1113:&#x0459;
SINGLE RIGHT-POINTING ANGLE QUOTATION MARK:'9B:8250:&#x203A;
CYRILLIC SMALL LETTER NJE:'9C:1114:&#x045A;
CYRILLIC SMALL LETTER KJE:'9D:1116:&#x045C;
CYRILLIC SMALL LETTER TSHE (SERBOCROATIAN):'9E:1115:&#x045B;
CYRILLIC SMALL LETTER DZHE:'9F:1119:&#x045F;
NO-BREAK SPACE:'A0:160:&#x00A0;
CYRILLIC CAPITAL LETTER SHORT U (BYELORUSSIAN):'A1:1038:&#x040E;
CYRILLIC SMALL LETTER SHORT U (BYELORUSSIAN):'A2:1118:&#x045E;
CYRILLIC CAPITAL LETTER JE:'A3:1032:&#x0408;
CURRENCY SIGN:'A4:164:&#x00A4;
CYRILLIC CAPITAL LETTER GHE WITH UPTURN:'A5:1168:&#x0490;
BROKEN BAR:'A6:166:&#x00A6;
SECTION SIGN:'A7:167:&#x00A7;
CYRILLIC CAPITAL LETTER IO:'A8:1025:&#x0401;
COPYRIGHT SIGN:'A9:169:&#x00A9;
CYRILLIC CAPITAL LETTER UKRAINIAN IE:'AA:1028:&#x0404;
LEFT-POINTING DOUBLE ANGLE QUOTATION MARK:'AB:171:&#x00AB;
NOT SIGN:'AC:172:&#x00AC;
SOFT HYPHEN:'AD:173:&#x00AD;
REGISTERED SIGN:'AE:174:&#x00AE;
CYRILLIC CAPITAL LETTER YI (UKRAINIAN):'AF:1031:&#x0407;
DEGREE SIGN:'B0:176:&#x00B0;
PLUS-MINUS SIGN:'B1:177:&#x00B1;
CYRILLIC CAPITAL LETTER BYELORUSSIAN-UKRAINIAN I:'B2:1030:&#x0406;
CYRILLIC SMALL LETTER BYELORUSSIAN-UKRAINIAN I:'B3:1110:&#x0456;
CYRILLIC SMALL LETTER GHE WITH UPTURN:'B4:1169:&#x0491;
MICRO SIGN:'B5:181:&#x00B5;
PILCROW SIGN:'B6:182:&#x00B6;
MIDDLE DOT:'B7:183:&#x00B7;
CYRILLIC SMALL LETTER IO:'B8:1105:&#x0451;
NUMERO SIGN:'B9:8470:&#x2116;
CYRILLIC SMALL LETTER UKRAINIAN IE:'BA:1108:&#x0454;
RIGHT-POINTING DOUBLE ANGLE QUOTATION MARK:'BB:187:&#x00BB;
CYRILLIC SMALL LETTER JE:'BC:1112:&#x0458;
CYRILLIC CAPITAL LETTER DZE:'BD:1029:&#x0405;
CYRILLIC SMALL LETTER DZE:'BE:1109:&#x0455;
CYRILLIC SMALL LETTER YI (UKRAINIAN):'BF:1111:&#x0457;
CYRILLIC CAPITAL LETTER A:'C0:1040:&#x0410;
CYRILLIC CAPITAL LETTER BE:'C1:1041:&#x0411;
CYRILLIC CAPITAL LETTER VE:'C2:1042:&#x0412;
CYRILLIC CAPITAL LETTER GHE:'C3:1043:&#x0413;
CYRILLIC CAPITAL LETTER DE:'C4:1044:&#x0414;
CYRILLIC CAPITAL LETTER IE:'C5:1045:&#x0415;
CYRILLIC CAPITAL LETTER ZHE:'C6:1046:&#x0416;
CYRILLIC CAPITAL LETTER ZE:'C7:1047:&#x0417;
CYRILLIC CAPITAL LETTER I:'C8:1048:&#x0418;
CYRILLIC CAPITAL LETTER SHORT I:'C9:1049:&#x0419;
CYRILLIC CAPITAL LETTER KA:'CA:1050:&#x041A;
CYRILLIC CAPITAL LETTER EL:'CB:1051:&#x041B;
CYRILLIC CAPITAL LETTER EM:'CC:1052:&#x041C;
CYRILLIC CAPITAL LETTER EN:'CD:1053:&#x041D;
CYRILLIC CAPITAL LETTER O:'CE:1054:&#x041E;
CYRILLIC CAPITAL LETTER PE:'CF:1055:&#x041F;
CYRILLIC CAPITAL LETTER ER:'D0:1056:&#x0420;
CYRILLIC CAPITAL LETTER ES:'D1:1057:&#x0421;
CYRILLIC CAPITAL LETTER TE:'D2:1058:&#x0422;
CYRILLIC CAPITAL LETTER U:'D3:1059:&#x0423;
CYRILLIC CAPITAL LETTER EF:'D4:1060:&#x0424;
CYRILLIC CAPITAL LETTER HA:'D5:1061:&#x0425;
CYRILLIC CAPITAL LETTER TSE:'D6:1062:&#x0426;
CYRILLIC CAPITAL LETTER CHE:'D7:1063:&#x0427;
CYRILLIC CAPITAL LETTER SHA:'D8:1064:&#x0428;
CYRILLIC CAPITAL LETTER SHCHA:'D9:1065:&#x0429;
CYRILLIC CAPITAL LETTER HARD SIGN:'DA:1066:&#x042A;
CYRILLIC CAPITAL LETTER YERU:'DB:1067:&#x042B;
CYRILLIC CAPITAL LETTER SOFT SIGN:'DC:1068:&#x042C;
CYRILLIC CAPITAL LETTER E:'DD:1069:&#x042D;
CYRILLIC CAPITAL LETTER YU:'DE:1070:&#x042E;
CYRILLIC CAPITAL LETTER YA:'DF:1071:&#x042F;
CYRILLIC SMALL LETTER A:'E0:1072:&#x0430;
CYRILLIC SMALL LETTER BE:'E1:1073:&#x0431;
CYRILLIC SMALL LETTER VE:'E2:1074:&#x0432;
CYRILLIC SMALL LETTER GHE:'E3:1075:&#x0433;
CYRILLIC SMALL LETTER DE:'E4:1076:&#x0434;
CYRILLIC SMALL LETTER IE:'E5:1077:&#x0435;
CYRILLIC SMALL LETTER ZHE:'E6:1078:&#x0436;
CYRILLIC SMALL LETTER ZE:'E7:1079:&#x0437;
CYRILLIC SMALL LETTER I:'E8:1080:&#x0438;
CYRILLIC SMALL LETTER SHORT I:'E9:1081:&#x0439;
CYRILLIC SMALL LETTER KA:'EA:1082:&#x043A;
CYRILLIC SMALL LETTER EL:'EB:1083:&#x043B;
CYRILLIC SMALL LETTER EM:'EC:1084:&#x043C;
CYRILLIC SMALL LETTER EN:'ED:1085:&#x043D;
CYRILLIC SMALL LETTER O:'EE:1086:&#x043E;
CYRILLIC SMALL LETTER PE:'EF:1087:&#x043F;
CYRILLIC SMALL LETTER ER:'F0:1088:&#x0440;
CYRILLIC SMALL LETTER ES:'F1:1089:&#x0441;
CYRILLIC SMALL LETTER TE:'F2:1090:&#x0442;
CYRILLIC SMALL LETTER U:'F3:1091:&#x0443;
CYRILLIC SMALL LETTER EF:'F4:1092:&#x0444;
CYRILLIC SMALL LETTER HA:'F5:1093:&#x0445;
CYRILLIC SMALL LETTER TSE:'F6:1094:&#x0446;
CYRILLIC SMALL LETTER CHE:'F7:1095:&#x0447;
CYRILLIC SMALL LETTER SHA:'F8:1096:&#x0448;
CYRILLIC SMALL LETTER SHCHA:'F9:1097:&#x0449;
CYRILLIC SMALL LETTER HARD SIGN:'FA:1098:&#x044A;
CYRILLIC SMALL LETTER YERU:'FB:1099:&#x044B;
CYRILLIC SMALL LETTER SOFT SIGN:'FC:1100:&#x044C;
CYRILLIC SMALL LETTER E:'FD:1101:&#x044D;
CYRILLIC SMALL LETTER YU:'FE:1102:&#x044E;
CYRILLIC SMALL LETTER YA:'FF:1103:&#x044F;
</ansicpg1251>
<ansicpg1252>
LATIN SMALL LETTER Y WITH DIAERESIS:'00:00:&#x00FF;
EURO SIGN:'80:8364:&#x20ac;
SINGLE LOW-9 QUOTATION MARK:'82:8218:&#x201A;
LATIN SMALL LETTER F WITH HOOK:'83:402:&#x0192;
DOUBLE LOW-9 QUOTATION MARK:'84:8222:&#x201E;
HORIZONTAL ELLIPSIS:'85:8230:&#x2026;
DAGGER:'86:8224:&#x2020;
DOUBLE DAGGER:'87:8225:&#x2021;
MODIFIER LETTER CIRCUMFLEX ACCENT:'88:710:&#x02C6;
PER MILLE SIGN:'89:8240:&#x2030;
LATIN CAPITAL LETTER S WITH CARON:'8A:352:&#x0160;
SINGLE LEFT-POINTING ANGLE QUOTATION MARK:'8B:8249:&#x2039;
LATIN CAPITAL LIGATURE OE:'8C:338:&#x0152;
LEFT SINGLE QUOTATION MARK:'91:8216:&#x2018;
RIGHT SINGLE QUOTATION MARK:'92:8217:&#x2019;
LEFT DOUBLE QUOTATION MARK:'93:8220:&#x201C;
RIGHT DOUBLE QUOTATION MARK:'94:8221:&#x201D;
BULLET:'95:8226:&#x2022;
EN DASH:'96:8211:&#x2013;
EM DASH:'97:8212:&#x2014;
SMALL TILDE:'98:732:&#x02DC;
TRADE MARK SIGN:'99:8482:&#x2122;
LATIN SMALL LETTER S WITH CARON:'9A:353:&#x0161;
SINGLE RIGHT-POINTING ANGLE QUOTATION MARK:'9B:8250:&#x203A;
LATIN SMALL LIGATURE OE:'9C:339:&#x0153;
LATIN CAPITAL LETTER Y WITH DIAERESIS:'9F:376:&#x0178;
NO-BREAK SPACE:'A0:160:&#x00A0;
INVERTED EXCLAMATION MARK:'A1:161:&#x00A1;
CENT SIGN:'A2:162:&#x00A2;
POUND SIGN:'A3:163:&#x00A3;
CURRENCY SIGN:'A4:164:&#x00A4;
YEN SIGN:'A5:165:&#x00A5;
BROKEN BAR:'A6:166:&#x00A6;
SECTION SIGN:'A7:167:&#x00A7;
DIAERESIS:'A8:168:&#x00A8;
COPYRIGHT SIGN:'A9:169:&#x00A9;
FEMININE ORDINAL INDICATOR:'AA:170:&#x00AA;
LEFT-POINTING DOUBLE ANGLE QUOTATION MARK:'AB:171:&#x00AB;
NOT SIGN:'AC:172:&#x00AC;
SOFT HYPHEN:'AD:173:&#x00AD;
REGISTERED SIGN:'AE:174:&#x00AE;
MACRON:'AF:175:&#x00AF;
DEGREE SIGN:'B0:176:&#x00B0;
PLUS-MINUS SIGN:'B1:177:&#x00B1;
SUPERSCRIPT TWO:'B2:178:&#x00B2;
SUPERSCRIPT THREE:'B3:179:&#x00B3;
ACUTE ACCENT:'B4:180:&#x00B4;
MICRO SIGN:'B5:181:&#x00B5;
PILCROW SIGN:'B6:182:&#x00B6;
MIDDLE DOT:'B7:183:&#x00B7;
CEDILLA:'B8:184:&#x00B8;
SUPERSCRIPT ONE:'B9:185:&#x00B9;
MASCULINE ORDINAL INDICATOR:'BA:186:&#x00BA;
RIGHT-POINTING DOUBLE ANGLE QUOTATION MARK:'BB:187:&#x00BB;
VULGAR FRACTION ONE QUARTER:'BC:188:&#x00BC;
VULGAR FRACTION ONE HALF:'BD:189:&#x00BD;
VULGAR FRACTION THREE QUARTERS:'BE:190:&#x00BE;
INVERTED QUESTION MARK:'BF:191:&#x00BF;
LATIN CAPITAL LETTER A WITH GRAVE:'C0:192:&#x00C0;
LATIN CAPITAL LETTER A WITH ACUTE:'C1:193:&#x00C1;
LATIN CAPITAL LETTER A WITH CIRCUMFLEX:'C2:194:&#x00C2;
LATIN CAPITAL LETTER A WITH TILDE:'C3:195:&#x00C3;
LATIN CAPITAL LETTER A WITH DIAERESIS:'C4:196:&#x00C4;
LATIN CAPITAL LETTER A WITH RING ABOVE:'C5:197:&#x00C5;
LATIN CAPITAL LETTER AE:'C6:198:&#x00C6;
LATIN CAPITAL LETTER C WITH CEDILLA:'C7:199:&#x00C7;
LATIN CAPITAL LETTER E WITH GRAVE:'C8:200:&#x00C8;
LATIN CAPITAL LETTER E WITH ACUTE:'C9:201:&#x00C9;
LATIN CAPITAL LETTER E WITH CIRCUMFLEX:'CA:202:&#x00CA;
LATIN CAPITAL LETTER E WITH DIAERESIS:'CB:203:&#x00CB;
LATIN CAPITAL LETTER I WITH GRAVE:'CC:204:&#x00CC;
LATIN CAPITAL LETTER I WITH ACUTE:'CD:205:&#x00CD;
LATIN CAPITAL LETTER I WITH CIRCUMFLEX:'CE:206:&#x00CE;
LATIN CAPITAL LETTER I WITH DIAERESIS:'CF:207:&#x00CF;
LATIN CAPITAL LETTER ETH (ICELANDIC):'D0:208:&#x00D0;
LATIN CAPITAL LETTER N WITH TILDE:'D1:209:&#x00D1;
LATIN CAPITAL LETTER O WITH GRAVE:'D2:210:&#x00D2;
LATIN CAPITAL LETTER O WITH ACUTE:'D3:211:&#x00D3;
LATIN CAPITAL LETTER O WITH CIRCUMFLEX:'D4:212:&#x00D4;
LATIN CAPITAL LETTER O WITH TILDE:'D5:213:&#x00D5;
LATIN CAPITAL LETTER O WITH DIAERESIS:'D6:214:&#x00D6;
MULTIPLICATION SIGN:'D7:215:&#x00D7;
LATIN CAPITAL LETTER O WITH STROKE:'D8:216:&#x00D8;
LATIN CAPITAL LETTER U WITH GRAVE:'D9:217:&#x00D9;
LATIN CAPITAL LETTER U WITH ACUTE:'DA:218:&#x00DA;
LATIN CAPITAL LETTER U WITH CIRCUMFLEX:'DB:219:&#x00DB;
LATIN CAPITAL LETTER U WITH DIAERESIS:'DC:220:&#x00DC;
LATIN CAPITAL LETTER Y WITH ACUTE:'DD:221:&#x00DD;
LATIN CAPITAL LETTER THORN (ICELANDIC):'DE:222:&#x00DE;
LATIN SMALL LETTER SHARP S (GERMAN):'DF:223:&#x00DF;
LATIN SMALL LETTER A WITH GRAVE:'E0:224:&#x00E0;
LATIN SMALL LETTER A WITH ACUTE:'E1:225:&#x00E1;
LATIN SMALL LETTER A WITH CIRCUMFLEX:'E2:226:&#x00E2;
LATIN SMALL LETTER A WITH TILDE:'E3:227:&#x00E3;
LATIN SMALL LETTER A WITH DIAERESIS:'E4:228:&#x00E4;
LATIN SMALL LETTER A WITH RING ABOVE:'E5:229:&#x00E5;
LATIN SMALL LETTER AE:'E6:230:&#x00E6;
LATIN SMALL LETTER C WITH CEDILLA:'E7:231:&#x00E7;
LATIN SMALL LETTER E WITH GRAVE:'E8:232:&#x00E8;
LATIN SMALL LETTER E WITH ACUTE:'E9:233:&#x00E9;
LATIN SMALL LETTER E WITH CIRCUMFLEX:'EA:234:&#x00EA;
LATIN SMALL LETTER E WITH DIAERESIS:'EB:235:&#x00EB;
LATIN SMALL LETTER I WITH GRAVE:'EC:236:&#x00EC;
LATIN SMALL LETTER I WITH ACUTE:'ED:237:&#x00ED;
LATIN SMALL LETTER I WITH CIRCUMFLEX:'EE:238:&#x00EE;
LATIN SMALL LETTER I WITH DIAERESIS:'EF:239:&#x00EF;
LATIN SMALL LETTER ETH (ICELANDIC):'F0:240:&#x00F0;
LATIN SMALL LETTER N WITH TILDE:'F1:241:&#x00F1;
LATIN SMALL LETTER O WITH GRAVE:'F2:242:&#x00F2;
LATIN SMALL LETTER O WITH ACUTE:'F3:243:&#x00F3;
LATIN SMALL LETTER O WITH CIRCUMFLEX:'F4:244:&#x00F4;
LATIN SMALL LETTER O WITH TILDE:'F5:245:&#x00F5;
LATIN SMALL LETTER O WITH DIAERESIS:'F6:246:&#x00F6;
DIVISION SIGN:'F7:247:&#x00F7;
LATIN SMALL LETTER O WITH STROKE:'F8:248:&#x00F8;
LATIN SMALL LETTER U WITH GRAVE:'F9:249:&#x00F9;
LATIN SMALL LETTER U WITH ACUTE:'FA:250:&#x00FA;
LATIN SMALL LETTER U WITH CIRCUMFLEX:'FB:251:&#x00FB;
LATIN SMALL LETTER U WITH DIAERESIS:'FC:252:&#x00FC;
LATIN SMALL LETTER Y WITH ACUTE:'FD:253:&#x00FD;
LATIN SMALL LETTER THORN (ICELANDIC):'FE:254:&#x00FE;
LATIN SMALL LETTER Y WITH DIAERESIS:'FF:255:&#x00FF;
MY UNDEFINED SYMBOL:\'8D:141:<udef_symbol num="141"/>
MY UNDEFINED SYMBOL:\'8E:142:<udef_symbol num="142"/>
MY UNDEFINED SYMBOL:\'8F:143:<udef_symbol num="143"/>
MY UNDEFINED SYMBOL:\'90:144:<udef_symbol num="144"/>
MY UNDEFINED SYMBOL:\'9D:157:<udef_symbol num="157"/>
MY UNDEFINED SYMBOL:\'9E:158:<udef_symbol num="158"/>
</ansicpg1252>
<ansicpg1253>
SINGLE LOW-9 QUOTATION MARK:'82:8218:&#x201A;
LATIN SMALL LETTER F WITH HOOK:'83:402:&#x0192;
DOUBLE LOW-9 QUOTATION MARK:'84:8222:&#x201E;
HORIZONTAL ELLIPSIS:'85:8230:&#x2026;
DAGGER:'86:8224:&#x2020;
DOUBLE DAGGER:'87:8225:&#x2021;
PER MILLE SIGN:'89:8240:&#x2030;
SINGLE LEFT-POINTING ANGLE QUOTATION MARK:'8B:8249:&#x2039;
LEFT SINGLE QUOTATION MARK:'91:8216:&#x2018;
RIGHT SINGLE QUOTATION MARK:'92:8217:&#x2019;
LEFT DOUBLE QUOTATION MARK:'93:8220:&#x201C;
RIGHT DOUBLE QUOTATION MARK:'94:8221:&#x201D;
BULLET:'95:8226:&#x2022;
EN DASH:'96:8211:&#x2013;
EM DASH:'97:8212:&#x2014;
TRADE MARK SIGN:'99:8482:&#x2122;
SINGLE RIGHT-POINTING ANGLE QUOTATION MARK:'9B:8250:&#x203A;
NO-BREAK SPACE:'A0:160:&#x00A0;
GREEK DIALYTIKA TONOS:'A1:901:&#x0385;
GREEK CAPITAL LETTER ALPHA WITH TONOS:'A2:902:&#x0386;
POUND SIGN:'A3:163:&#x00A3;
CURRENCY SIGN:'A4:164:&#x00A4;
YEN SIGN:'A5:165:&#x00A5;
BROKEN BAR:'A6:166:&#x00A6;
SECTION SIGN:'A7:167:&#x00A7;
DIAERESIS:'A8:168:&#x00A8;
COPYRIGHT SIGN:'A9:169:&#x00A9;
LEFT-POINTING DOUBLE ANGLE QUOTATION MARK:'AB:171:&#x00AB;
NOT SIGN:'AC:172:&#x00AC;
SOFT HYPHEN:'AD:173:&#x00AD;
REGISTERED SIGN:'AE:174:&#x00AE;
HORIZONTAL BAR:'AF:8213:&#x2015;
DEGREE SIGN:'B0:176:&#x00B0;
PLUS-MINUS SIGN:'B1:177:&#x00B1;
SUPERSCRIPT TWO:'B2:178:&#x00B2;
SUPERSCRIPT THREE:'B3:179:&#x00B3;
GREEK TONOS:'B4:900:&#x0384;
MICRO SIGN:'B5:181:&#x00B5;
PILCROW SIGN:'B6:182:&#x00B6;
MIDDLE DOT:'B7:183:&#x00B7;
GREEK CAPITAL LETTER EPSILON WITH TONOS:'B8:904:&#x0388;
GREEK CAPITAL LETTER ETA WITH TONOS:'B9:905:&#x0389;
GREEK CAPITAL LETTER IOTA WITH TONOS:'BA:906:&#x038A;
RIGHT-POINTING DOUBLE ANGLE QUOTATION MARK:'BB:187:&#x00BB;
GREEK CAPITAL LETTER OMICRON WITH TONOS:'BC:908:&#x038C;
VULGAR FRACTION ONE HALF:'BD:189:&#x00BD;
GREEK CAPITAL LETTER UPSILON WITH TONOS:'BE:910:&#x038E;
GREEK CAPITAL LETTER OMEGA WITH TONOS:'BF:911:&#x038F;
GREEK SMALL LETTER IOTA WITH DIALYTIKA AND TONOS:'C0:912:&#x0390;
GREEK CAPITAL LETTER ALPHA:'C1:913:&#x0391;
GREEK CAPITAL LETTER BETA:'C2:914:&#x0392;
GREEK CAPITAL LETTER GAMMA:'C3:915:&#x0393;
GREEK CAPITAL LETTER DELTA:'C4:916:&#x0394;
GREEK CAPITAL LETTER EPSILON:'C5:917:&#x0395;
GREEK CAPITAL LETTER ZETA:'C6:918:&#x0396;
GREEK CAPITAL LETTER ETA:'C7:919:&#x0397;
GREEK CAPITAL LETTER THETA:'C8:920:&#x0398;
GREEK CAPITAL LETTER IOTA:'C9:921:&#x0399;
GREEK CAPITAL LETTER KAPPA:'CA:922:&#x039A;
GREEK CAPITAL LETTER LAMDA:'CB:923:&#x039B;
GREEK CAPITAL LETTER MU:'CC:924:&#x039C;
GREEK CAPITAL LETTER NU:'CD:925:&#x039D;
GREEK CAPITAL LETTER XI:'CE:926:&#x039E;
GREEK CAPITAL LETTER OMICRON:'CF:927:&#x039F;
GREEK CAPITAL LETTER PI:'D0:928:&#x03A0;
GREEK CAPITAL LETTER RHO:'D1:929:&#x03A1;
GREEK CAPITAL LETTER SIGMA:'D3:931:&#x03A3;
GREEK CAPITAL LETTER TAU:'D4:932:&#x03A4;
GREEK CAPITAL LETTER UPSILON:'D5:933:&#x03A5;
GREEK CAPITAL LETTER PHI:'D6:934:&#x03A6;
GREEK CAPITAL LETTER CHI:'D7:935:&#x03A7;
GREEK CAPITAL LETTER PSI:'D8:936:&#x03A8;
GREEK CAPITAL LETTER OMEGA:'D9:937:&#x03A9;
GREEK CAPITAL LETTER IOTA WITH DIALYTIKA:'DA:938:&#x03AA;
GREEK CAPITAL LETTER UPSILON WITH DIALYTIKA:'DB:939:&#x03AB;
GREEK SMALL LETTER ALPHA WITH TONOS:'DC:940:&#x03AC;
GREEK SMALL LETTER EPSILON WITH TONOS:'DD:941:&#x03AD;
GREEK SMALL LETTER ETA WITH TONOS:'DE:942:&#x03AE;
GREEK SMALL LETTER IOTA WITH TONOS:'DF:943:&#x03AF;
GREEK SMALL LETTER UPSILON WITH DIALYTIKA AND TONOS:'E0:944:&#x03B0;
GREEK SMALL LETTER ALPHA:'E1:945:&#x03B1;
GREEK SMALL LETTER BETA:'E2:946:&#x03B2;
GREEK SMALL LETTER GAMMA:'E3:947:&#x03B3;
GREEK SMALL LETTER DELTA:'E4:948:&#x03B4;
GREEK SMALL LETTER EPSILON:'E5:949:&#x03B5;
GREEK SMALL LETTER ZETA:'E6:950:&#x03B6;
GREEK SMALL LETTER ETA:'E7:951:&#x03B7;
GREEK SMALL LETTER THETA:'E8:952:&#x03B8;
GREEK SMALL LETTER IOTA:'E9:953:&#x03B9;
GREEK SMALL LETTER KAPPA:'EA:954:&#x03BA;
GREEK SMALL LETTER LAMDA:'EB:955:&#x03BB;
GREEK SMALL LETTER MU:'EC:956:&#x03BC;
GREEK SMALL LETTER NU:'ED:957:&#x03BD;
GREEK SMALL LETTER XI:'EE:958:&#x03BE;
GREEK SMALL LETTER OMICRON:'EF:959:&#x03BF;
GREEK SMALL LETTER PI:'F0:960:&#x03C0;
GREEK SMALL LETTER RHO:'F1:961:&#x03C1;
GREEK SMALL LETTER FINAL SIGMA:'F2:962:&#x03C2;
GREEK SMALL LETTER SIGMA:'F3:963:&#x03C3;
GREEK SMALL LETTER TAU:'F4:964:&#x03C4;
GREEK SMALL LETTER UPSILON:'F5:965:&#x03C5;
GREEK SMALL LETTER PHI:'F6:966:&#x03C6;
GREEK SMALL LETTER CHI:'F7:967:&#x03C7;
GREEK SMALL LETTER PSI:'F8:968:&#x03C8;
GREEK SMALL LETTER OMEGA:'F9:969:&#x03C9;
GREEK SMALL LETTER IOTA WITH DIALYTIKA:'FA:970:&#x03CA;
GREEK SMALL LETTER UPSILON WITH DIALYTIKA:'FB:971:&#x03CB;
GREEK SMALL LETTER OMICRON WITH TONOS:'FC:972:&#x03CC;
GREEK SMALL LETTER UPSILON WITH TONOS:'FD:973:&#x03CD;
GREEK SMALL LETTER OMEGA WITH TONOS:'FE:974:&#x03CE;
</ansicpg1253>
<ansicpg1254>
SINGLE LOW-9 QUOTATION MARK:'82:8218:&#x201A;
LATIN SMALL LETTER F WITH HOOK:'83:402:&#x0192;
DOUBLE LOW-9 QUOTATION MARK:'84:8222:&#x201E;
HORIZONTAL ELLIPSIS:'85:8230:&#x2026;
DAGGER:'86:8224:&#x2020;
DOUBLE DAGGER:'87:8225:&#x2021;
MODIFIER LETTER CIRCUMFLEX ACCENT:'88:710:&#x02C6;
PER MILLE SIGN:'89:8240:&#x2030;
LATIN CAPITAL LETTER S WITH CARON:'8A:352:&#x0160;
SINGLE LEFT-POINTING ANGLE QUOTATION MARK:'8B:8249:&#x2039;
LATIN CAPITAL LIGATURE OE:'8C:338:&#x0152;
LEFT SINGLE QUOTATION MARK:'91:8216:&#x2018;
RIGHT SINGLE QUOTATION MARK:'92:8217:&#x2019;
LEFT DOUBLE QUOTATION MARK:'93:8220:&#x201C;
RIGHT DOUBLE QUOTATION MARK:'94:8221:&#x201D;
BULLET:'95:8226:&#x2022;
EN DASH:'96:8211:&#x2013;
EM DASH:'97:8212:&#x2014;
SMALL TILDE:'98:732:&#x02DC;
TRADE MARK SIGN:'99:8482:&#x2122;
LATIN SMALL LETTER S WITH CARON:'9A:353:&#x0161;
SINGLE RIGHT-POINTING ANGLE QUOTATION MARK:'9B:8250:&#x203A;
LATIN SMALL LIGATURE OE:'9C:339:&#x0153;
LATIN CAPITAL LETTER Y WITH DIAERESIS:'9F:376:&#x0178;
NO-BREAK SPACE:'A0:160:&#x00A0;
INVERTED EXCLAMATION MARK:'A1:161:&#x00A1;
CENT SIGN:'A2:162:&#x00A2;
POUND SIGN:'A3:163:&#x00A3;
CURRENCY SIGN:'A4:164:&#x00A4;
YEN SIGN:'A5:165:&#x00A5;
BROKEN BAR:'A6:166:&#x00A6;
SECTION SIGN:'A7:167:&#x00A7;
DIAERESIS:'A8:168:&#x00A8;
COPYRIGHT SIGN:'A9:169:&#x00A9;
FEMININE ORDINAL INDICATOR:'AA:170:&#x00AA;
LEFT-POINTING DOUBLE ANGLE QUOTATION MARK:'AB:171:&#x00AB;
NOT SIGN:'AC:172:&#x00AC;
SOFT HYPHEN:'AD:173:&#x00AD;
REGISTERED SIGN:'AE:174:&#x00AE;
MACRON:'AF:175:&#x00AF;
DEGREE SIGN:'B0:176:&#x00B0;
PLUS-MINUS SIGN:'B1:177:&#x00B1;
SUPERSCRIPT TWO:'B2:178:&#x00B2;
SUPERSCRIPT THREE:'B3:179:&#x00B3;
ACUTE ACCENT:'B4:180:&#x00B4;
MICRO SIGN:'B5:181:&#x00B5;
PILCROW SIGN:'B6:182:&#x00B6;
MIDDLE DOT:'B7:183:&#x00B7;
CEDILLA:'B8:184:&#x00B8;
SUPERSCRIPT ONE:'B9:185:&#x00B9;
MASCULINE ORDINAL INDICATOR:'BA:186:&#x00BA;
RIGHT-POINTING DOUBLE ANGLE QUOTATION MARK:'BB:187:&#x00BB;
VULGAR FRACTION ONE QUARTER:'BC:188:&#x00BC;
VULGAR FRACTION ONE HALF:'BD:189:&#x00BD;
VULGAR FRACTION THREE QUARTERS:'BE:190:&#x00BE;
INVERTED QUESTION MARK:'BF:191:&#x00BF;
LATIN CAPITAL LETTER A WITH GRAVE:'C0:192:&#x00C0;
LATIN CAPITAL LETTER A WITH ACUTE:'C1:193:&#x00C1;
LATIN CAPITAL LETTER A WITH CIRCUMFLEX:'C2:194:&#x00C2;
LATIN CAPITAL LETTER A WITH TILDE:'C3:195:&#x00C3;
LATIN CAPITAL LETTER A WITH DIAERESIS:'C4:196:&#x00C4;
LATIN CAPITAL LETTER A WITH RING ABOVE:'C5:197:&#x00C5;
LATIN CAPITAL LETTER AE:'C6:198:&#x00C6;
LATIN CAPITAL LETTER C WITH CEDILLA:'C7:199:&#x00C7;
LATIN CAPITAL LETTER E WITH GRAVE:'C8:200:&#x00C8;
LATIN CAPITAL LETTER E WITH ACUTE:'C9:201:&#x00C9;
LATIN CAPITAL LETTER E WITH CIRCUMFLEX:'CA:202:&#x00CA;
LATIN CAPITAL LETTER E WITH DIAERESIS:'CB:203:&#x00CB;
LATIN CAPITAL LETTER I WITH GRAVE:'CC:204:&#x00CC;
LATIN CAPITAL LETTER I WITH ACUTE:'CD:205:&#x00CD;
LATIN CAPITAL LETTER I WITH CIRCUMFLEX:'CE:206:&#x00CE;
LATIN CAPITAL LETTER I WITH DIAERESIS:'CF:207:&#x00CF;
LATIN CAPITAL LETTER G WITH BREVE:'D0:286:&#x011E;
LATIN CAPITAL LETTER N WITH TILDE:'D1:209:&#x00D1;
LATIN CAPITAL LETTER O WITH GRAVE:'D2:210:&#x00D2;
LATIN CAPITAL LETTER O WITH ACUTE:'D3:211:&#x00D3;
LATIN CAPITAL LETTER O WITH CIRCUMFLEX:'D4:212:&#x00D4;
LATIN CAPITAL LETTER O WITH TILDE:'D5:213:&#x00D5;
LATIN CAPITAL LETTER O WITH DIAERESIS:'D6:214:&#x00D6;
MULTIPLICATION SIGN:'D7:215:&#x00D7;
LATIN CAPITAL LETTER O WITH STROKE:'D8:216:&#x00D8;
LATIN CAPITAL LETTER U WITH GRAVE:'D9:217:&#x00D9;
LATIN CAPITAL LETTER U WITH ACUTE:'DA:218:&#x00DA;
LATIN CAPITAL LETTER U WITH CIRCUMFLEX:'DB:219:&#x00DB;
LATIN CAPITAL LETTER U WITH DIAERESIS:'DC:220:&#x00DC;
LATIN CAPITAL LETTER I WITH DOT ABOVE:'DD:304:&#x0130;
LATIN CAPITAL LETTER S WITH CEDILLA:'DE:350:&#x015E;
LATIN SMALL LETTER SHARP S (GERMAN):'DF:223:&#x00DF;
LATIN SMALL LETTER A WITH GRAVE:'E0:224:&#x00E0;
LATIN SMALL LETTER A WITH ACUTE:'E1:225:&#x00E1;
LATIN SMALL LETTER A WITH CIRCUMFLEX:'E2:226:&#x00E2;
LATIN SMALL LETTER A WITH TILDE:'E3:227:&#x00E3;
LATIN SMALL LETTER A WITH DIAERESIS:'E4:228:&#x00E4;
LATIN SMALL LETTER A WITH RING ABOVE:'E5:229:&#x00E5;
LATIN SMALL LETTER AE:'E6:230:&#x00E6;
LATIN SMALL LETTER C WITH CEDILLA:'E7:231:&#x00E7;
LATIN SMALL LETTER E WITH GRAVE:'E8:232:&#x00E8;
LATIN SMALL LETTER E WITH ACUTE:'E9:233:&#x00E9;
LATIN SMALL LETTER E WITH OGONEK:'EA:281:&#x0119;
LATIN SMALL LETTER E WITH DIAERESIS:'EB:235:&#x00EB;
LATIN SMALL LETTER E WITH DOT ABOVE:'EC:279:&#x0117;
LATIN SMALL LETTER I WITH ACUTE:'ED:237:&#x00ED;
LATIN SMALL LETTER I WITH CIRCUMFLEX:'EE:238:&#x00EE;
LATIN SMALL LETTER I WITH MACRON:'EF:299:&#x012B;
LATIN SMALL LETTER G WITH BREVE:'F0:287:&#x011F;
LATIN SMALL LETTER N WITH TILDE:'F1:241:&#x00F1;
LATIN SMALL LETTER O WITH GRAVE:'F2:242:&#x00F2;
LATIN SMALL LETTER O WITH ACUTE:'F3:243:&#x00F3;
LATIN SMALL LETTER O WITH CIRCUMFLEX:'F4:244:&#x00F4;
LATIN SMALL LETTER O WITH TILDE:'F5:245:&#x00F5;
LATIN SMALL LETTER O WITH DIAERESIS:'F6:246:&#x00F6;
DIVISION SIGN:'F7:247:&#x00F7;
LATIN SMALL LETTER O WITH STROKE:'F8:248:&#x00F8;
LATIN SMALL LETTER U WITH GRAVE:'F9:249:&#x00F9;
LATIN SMALL LETTER U WITH ACUTE:'FA:250:&#x00FA;
LATIN SMALL LETTER U WITH CIRCUMFLEX:'FB:251:&#x00FB;
LATIN SMALL LETTER U WITH DIAERESIS:'FC:252:&#x00FC;
LATIN SMALL LETTER DOTLESS I:'FD:305:&#x0131;
LATIN SMALL LETTER S WITH CEDILLA:'FE:351:&#x015F;
LATIN SMALL LETTER Y WITH DIAERESIS:'FF:255:&#x00FF;
</ansicpg1254>
<ansicpg1255>
SINGLE LOW-9 QUOTATION MARK:'82:8218:&#x201A;
LATIN SMALL LETTER F WITH HOOK:'83:402:&#x0192;
DOUBLE LOW-9 QUOTATION MARK:'84:8222:&#x201E;
HORIZONTAL ELLIPSIS:'85:8230:&#x2026;
DAGGER:'86:8224:&#x2020;
DOUBLE DAGGER:'87:8225:&#x2021;
PER MILLE SIGN:'89:8240:&#x2030;
SINGLE LEFT-POINTING ANGLE QUOTATION MARK:'8B:8249:&#x2039;
LEFT SINGLE QUOTATION MARK:'91:8216:&#x2018;
RIGHT SINGLE QUOTATION MARK:'92:8217:&#x2019;
LEFT DOUBLE QUOTATION MARK:'93:8220:&#x201C;
RIGHT DOUBLE QUOTATION MARK:'94:8221:&#x201D;
BULLET:'95:8226:&#x2022;
EN DASH:'96:8211:&#x2013;
EM DASH:'97:8212:&#x2014;
TRADE MARK SIGN:'99:8482:&#x2122;
SINGLE RIGHT-POINTING ANGLE QUOTATION MARK:'9B:8250:&#x203A;
NO-BREAK SPACE:'A0:160:&#x00A0;
CENT SIGN:'A2:162:&#x00A2;
POUND SIGN:'A3:163:&#x00A3;
CURRENCY SIGN:'A4:164:&#x00A4;
YEN SIGN:'A5:165:&#x00A5;
BROKEN BAR:'A6:166:&#x00A6;
SECTION SIGN:'A7:167:&#x00A7;
DIAERESIS:'A8:168:&#x00A8;
COPYRIGHT SIGN:'A9:169:&#x00A9;
MULTIPLICATION SIGN:'AA:215:&#x00D7;
LEFT-POINTING DOUBLE ANGLE QUOTATION MARK:'AB:171:&#x00AB;
NOT SIGN:'AC:172:&#x00AC;
SOFT HYPHEN:'AD:173:&#x00AD;
REGISTERED SIGN:'AE:174:&#x00AE;
OVERLINE:'AF:8254:&#x203E;
DEGREE SIGN:'B0:176:&#x00B0;
PLUS-MINUS SIGN:'B1:177:&#x00B1;
SUPERSCRIPT TWO:'B2:178:&#x00B2;
SUPERSCRIPT THREE:'B3:179:&#x00B3;
ACUTE ACCENT:'B4:180:&#x00B4;
MICRO SIGN:'B5:181:&#x00B5;
PILCROW SIGN:'B6:182:&#x00B6;
MIDDLE DOT:'B7:183:&#x00B7;
CEDILLA:'B8:184:&#x00B8;
SUPERSCRIPT ONE:'B9:185:&#x00B9;
DIVISION SIGN:'BA:247:&#x00F7;
RIGHT-POINTING DOUBLE ANGLE QUOTATION MARK:'BB:187:&#x00BB;
VULGAR FRACTION ONE QUARTER:'BC:188:&#x00BC;
VULGAR FRACTION ONE HALF:'BD:189:&#x00BD;
VULGAR FRACTION THREE QUARTERS:'BE:190:&#x00BE;
DOUBLE LOW LINE:'DF:8215:&#x2017;
HEBREW LETTER ALEF:'E0:1488:&#x05D0;
HEBREW LETTER BET:'E1:1489:&#x05D1;
HEBREW LETTER GIMEL:'E2:1490:&#x05D2;
HEBREW LETTER DALET:'E3:1491:&#x05D3;
HEBREW LETTER HE:'E4:1492:&#x05D4;
HEBREW LETTER VAV:'E5:1493:&#x05D5;
HEBREW LETTER ZAYIN:'E6:1494:&#x05D6;
HEBREW LETTER HET:'E7:1495:&#x05D7;
HEBREW LETTER TET:'E8:1496:&#x05D8;
HEBREW LETTER YOD:'E9:1497:&#x05D9;
HEBREW LETTER FINAL KAF:'EA:1498:&#x05DA;
HEBREW LETTER KAF:'EB:1499:&#x05DB;
HEBREW LETTER LAMED:'EC:1500:&#x05DC;
HEBREW LETTER FINAL MEM:'ED:1501:&#x05DD;
HEBREW LETTER MEM:'EE:1502:&#x05DE;
HEBREW LETTER FINAL NUN:'EF:1503:&#x05DF;
HEBREW LETTER NUN:'F0:1504:&#x05E0;
HEBREW LETTER SAMEKH:'F1:1505:&#x05E1;
HEBREW LETTER AYIN:'F2:1506:&#x05E2;
HEBREW LETTER FINAL PE:'F3:1507:&#x05E3;
HEBREW LETTER PE:'F4:1508:&#x05E4;
HEBREW LETTER FINAL TSADI:'F5:1509:&#x05E5;
HEBREW LETTER TSADI:'F6:1510:&#x05E6;
HEBREW LETTER QOF:'F7:1511:&#x05E7;
HEBREW LETTER RESH:'F8:1512:&#x05E8;
HEBREW LETTER SHIN:'F9:1513:&#x05E9;
HEBREW LETTER TAV:'FA:1514:&#x05EA;
LEFT-TO-RIGHT MARK:'FD:8206:&#x200E;
RIGHT-TO-LEFT MARK:'FE:8207:&#x200F;
NUL:'00:0:&#x0000;
</ansicpg1255>
<ansicpg1256>
ARABIC COMMA:'80:1548:&#x060C;
ARABIC-INDIC DIGIT ZERO:'81:1632:&#x0660;
SINGLE LOW-9 QUOTATION MARK:'82:8218:&#x201A;
ARABIC-INDIC DIGIT ONE:'83:1633:&#x0661;
DOUBLE LOW-9 QUOTATION MARK:'84:8222:&#x201E;
HORIZONTAL ELLIPSIS:'85:8230:&#x2026;
DAGGER:'86:8224:&#x2020;
DOUBLE DAGGER:'87:8225:&#x2021;
ARABIC-INDIC DIGIT TWO:'88:1634:&#x0662;
ARABIC-INDIC DIGIT THREE:'89:1635:&#x0663;
ARABIC-INDIC DIGIT FOUR:'8A:1636:&#x0664;
SINGLE LEFT-POINTING ANGLE QUOTATION MARK:'8B:8249:&#x2039;
ARABIC-INDIC DIGIT FIVE:'8C:1637:&#x0665;
ARABIC-INDIC DIGIT SIX:'8D:1638:&#x0666;
ARABIC-INDIC DIGIT SEVEN:'8E:1639:&#x0667;
ARABIC-INDIC DIGIT EIGHT:'8F:1640:&#x0668;
ARABIC-INDIC DIGIT NINE:'90:1641:&#x0669;
LEFT SINGLE QUOTATION MARK:'91:8216:&#x2018;
RIGHT SINGLE QUOTATION MARK:'92:8217:&#x2019;
LEFT DOUBLE QUOTATION MARK:'93:8220:&#x201C;
RIGHT DOUBLE QUOTATION MARK:'94:8221:&#x201D;
BULLET:'95:8226:&#x2022;
EN DASH:'96:8211:&#x2013;
EM DASH:'97:8212:&#x2014;
ARABIC SEMICOLON:'98:1563:&#x061B;
TRADE MARK SIGN:'99:8482:&#x2122;
ARABIC QUESTION MARK:'9A:1567:&#x061F;
SINGLE RIGHT-POINTING ANGLE QUOTATION MARK:'9B:8250:&#x203A;
ARABIC LETTER HAMZA:'9C:1569:&#x0621;
ARABIC LETTER ALEF WITH MADDA ABOVE:'9D:1570:&#x0622;
ARABIC LETTER ALEF WITH HAMZA ABOVE:'9E:1571:&#x0623;
LATIN CAPITAL LETTER Y WITH DIAERESIS:'9F:376:&#x0178;
NO-BREAK SPACE:'A0:160:&#x00A0;
ARABIC LETTER WAW WITH HAMZA ABOVE:'A1:1572:&#x0624;
ARABIC LETTER ALEF WITH HAMZA BELOW:'A2:1573:&#x0625;
POUND SIGN:'A3:163:&#x00A3;
CURRENCY SIGN:'A4:164:&#x00A4;
ARABIC LETTER YEH WITH HAMZA ABOVE:'A5:1574:&#x0626;
BROKEN BAR:'A6:166:&#x00A6;
SECTION SIGN:'A7:167:&#x00A7;
ARABIC LETTER ALEF:'A8:1575:&#x0627;
COPYRIGHT SIGN:'A9:169:&#x00A9;
ARABIC LETTER BEH:'AA:1576:&#x0628;
LEFT-POINTING DOUBLE ANGLE QUOTATION MARK:'AB:171:&#x00AB;
NOT SIGN:'AC:172:&#x00AC;
SOFT HYPHEN:'AD:173:&#x00AD;
REGISTERED SIGN:'AE:174:&#x00AE;
ARABIC LETTER PEH:'AF:1662:&#x067E;
DEGREE SIGN:'B0:176:&#x00B0;
PLUS-MINUS SIGN:'B1:177:&#x00B1;
ARABIC LETTER TEH MARBUTA:'B2:1577:&#x0629;
ARABIC LETTER TEH:'B3:1578:&#x062A;
ARABIC LETTER THEH:'B4:1579:&#x062B;
MICRO SIGN:'B5:181:&#x00B5;
PILCROW SIGN:'B6:182:&#x00B6;
MIDDLE DOT:'B7:183:&#x00B7;
ARABIC LETTER JEEM:'B8:1580:&#x062C;
ARABIC LETTER TCHEH:'B9:1670:&#x0686;
ARABIC LETTER HAH:'BA:1581:&#x062D;
RIGHT-POINTING DOUBLE ANGLE QUOTATION MARK:'BB:187:&#x00BB;
ARABIC LETTER KHAH:'BC:1582:&#x062E;
ARABIC LETTER DAL:'BD:1583:&#x062F;
ARABIC LETTER THAL:'BE:1584:&#x0630;
ARABIC LETTER REH:'BF:1585:&#x0631;
LATIN CAPITAL LETTER A WITH GRAVE:'C0:192:&#x00C0;
ARABIC LETTER ZAIN:'C1:1586:&#x0632;
LATIN CAPITAL LETTER A WITH CIRCUMFLEX:'C2:194:&#x00C2;
ARABIC LETTER JEH:'C3:1688:&#x0698;
ARABIC LETTER SEEN:'C4:1587:&#x0633;
ARABIC LETTER SHEEN:'C5:1588:&#x0634;
ARABIC LETTER SAD:'C6:1589:&#x0635;
LATIN CAPITAL LETTER C WITH CEDILLA:'C7:199:&#x00C7;
LATIN CAPITAL LETTER E WITH GRAVE:'C8:200:&#x00C8;
LATIN CAPITAL LETTER E WITH ACUTE:'C9:201:&#x00C9;
LATIN CAPITAL LETTER E WITH CIRCUMFLEX:'CA:202:&#x00CA;
LATIN CAPITAL LETTER E WITH DIAERESIS:'CB:203:&#x00CB;
ARABIC LETTER DAD:'CC:1590:&#x0636;
ARABIC LETTER TAH:'CD:1591:&#x0637;
LATIN CAPITAL LETTER I WITH CIRCUMFLEX:'CE:206:&#x00CE;
LATIN CAPITAL LETTER I WITH DIAERESIS:'CF:207:&#x00CF;
BOPOMOFO LETTER ZH:'D0:12563:&#x3113;
ARABIC LETTER AIN:'D1:1593:&#x0639;
ARABIC LETTER GHAIN:'D2:1594:&#x063A;
ARABIC TATWEEL:'D3:1600:&#x0640;
LATIN CAPITAL LETTER O WITH CIRCUMFLEX:'D4:212:&#x00D4;
ARABIC LETTER FEH:'D5:1601:&#x0641;
ARABIC LETTER QAF:'D6:1602:&#x0642;
MULTIPLICATION SIGN:'D7:215:&#x00D7;
ARABIC LETTER KAF:'D8:1603:&#x0643;
LATIN CAPITAL LETTER U WITH GRAVE:'D9:217:&#x00D9;
ARABIC LETTER GAF:'DA:1711:&#x06AF;
LATIN CAPITAL LETTER U WITH CIRCUMFLEX:'DB:219:&#x00DB;
LATIN CAPITAL LETTER U WITH DIAERESIS:'DC:220:&#x00DC;
ARABIC LETTER LAM:'DD:1604:&#x0644;
ARABIC LETTER MEEM:'DE:1605:&#x0645;
ARABIC LETTER NOON:'DF:1606:&#x0646;
LATIN SMALL LETTER A WITH GRAVE:'E0:224:&#x00E0;
ARABIC LETTER HEH:'E1:1607:&#x0647;
LATIN SMALL LETTER A WITH CIRCUMFLEX:'E2:226:&#x00E2;
ARABIC LETTER HAH WITH HAMZA ABOVE:'E3:1665:&#x0681;
ARABIC LETTER WAW:'E4:1608:&#x0648;
ARABIC LETTER ALEF MAKSURA:'E5:1609:&#x0649;
ARABIC LETTER YEH:'E6:1610:&#x064A;
LATIN SMALL LETTER C WITH CEDILLA:'E7:231:&#x00E7;
LATIN SMALL LETTER E WITH GRAVE:'E8:232:&#x00E8;
LATIN SMALL LETTER E WITH ACUTE:'E9:233:&#x00E9;
LATIN SMALL LETTER E WITH CIRCUMFLEX:'EA:234:&#x00EA;
LATIN SMALL LETTER E WITH DIAERESIS:'EB:235:&#x00EB;
ARABIC FATHATAN:'EC:1611:&#x064B;
ARABIC DAMMATAN:'ED:1612:&#x064C;
LATIN SMALL LETTER I WITH CIRCUMFLEX:'EE:238:&#x00EE;
LATIN SMALL LETTER I WITH DIAERESIS:'EF:239:&#x00EF;
ARABIC KASRATAN:'F0:1613:&#x064D;
ARABIC FATHA:'F1:1614:&#x064E;
ARABIC DAMMA:'F2:1615:&#x064F;
ARABIC KASRA:'F3:1616:&#x0650;
LATIN SMALL LETTER O WITH CIRCUMFLEX:'F4:244:&#x00F4;
ARABIC SHADDA:'F5:1617:&#x0651;
ARABIC SUKUN:'F6:1618:&#x0652;
DIVISION SIGN:'F7:247:&#x00F7;
LATIN SMALL LETTER U WITH GRAVE:'F9:249:&#x00F9;
LATIN SMALL LETTER U WITH CIRCUMFLEX:'FB:251:&#x00FB;
LATIN SMALL LETTER U WITH DIAERESIS:'FC:252:&#x00FC;
LEFT-TO-RIGHT MARK:'FD:8206:&#x200E;
RIGHT-TO-LEFT MARK:'FE:8207:&#x200F;
LATIN SMALL LETTER Y WITH DIAERESIS:'FF:255:&#x00FF;
</ansicpg1256>
<ansicpg1257>
SINGLE LOW-9 QUOTATION MARK:'82:8218:&#x201A;
DOUBLE LOW-9 QUOTATION MARK:'84:8222:&#x201E;
HORIZONTAL ELLIPSIS:'85:8230:&#x2026;
DAGGER:'86:8224:&#x2020;
DOUBLE DAGGER:'87:8225:&#x2021;
PER MILLE SIGN:'89:8240:&#x2030;
SINGLE LEFT-POINTING ANGLE QUOTATION MARK:'8B:8249:&#x2039;
LEFT SINGLE QUOTATION MARK:'91:8216:&#x2018;
RIGHT SINGLE QUOTATION MARK:'92:8217:&#x2019;
LEFT DOUBLE QUOTATION MARK:'93:8220:&#x201C;
RIGHT DOUBLE QUOTATION MARK:'94:8221:&#x201D;
BULLET:'95:8226:&#x2022;
EN DASH:'96:8211:&#x2013;
EM DASH:'97:8212:&#x2014;
TRADE MARK SIGN:'99:8482:&#x2122;
SINGLE RIGHT-POINTING ANGLE QUOTATION MARK:'9B:8250:&#x203A;
NO-BREAK SPACE:'A0:160:&#x00A0;
CENT SIGN:'A2:162:&#x00A2;
POUND SIGN:'A3:163:&#x00A3;
CURRENCY SIGN:'A4:164:&#x00A4;
BROKEN BAR:'A6:166:&#x00A6;
SECTION SIGN:'A7:167:&#x00A7;
LATIN CAPITAL LETTER O WITH STROKE:'A8:216:&#x00D8;
COPYRIGHT SIGN:'A9:169:&#x00A9;
LATIN CAPITAL LETTER R WITH CEDILLA:'AA:342:&#x0156;
LEFT-POINTING DOUBLE ANGLE QUOTATION MARK:'AB:171:&#x00AB;
NOT SIGN:'AC:172:&#x00AC;
SOFT HYPHEN:'AD:173:&#x00AD;
REGISTERED SIGN:'AE:174:&#x00AE;
LATIN CAPITAL LETTER AE:'AF:198:&#x00C6;
DEGREE SIGN:'B0:176:&#x00B0;
PLUS-MINUS SIGN:'B1:177:&#x00B1;
SUPERSCRIPT TWO:'B2:178:&#x00B2;
SUPERSCRIPT THREE:'B3:179:&#x00B3;
MICRO SIGN:'B5:181:&#x00B5;
PILCROW SIGN:'B6:182:&#x00B6;
MIDDLE DOT:'B7:183:&#x00B7;
LATIN SMALL LETTER O WITH STROKE:'B8:248:&#x00F8;
SUPERSCRIPT ONE:'B9:185:&#x00B9;
LATIN SMALL LETTER R WITH CEDILLA:'BA:343:&#x0157;
RIGHT-POINTING DOUBLE ANGLE QUOTATION MARK:'BB:187:&#x00BB;
VULGAR FRACTION ONE QUARTER:'BC:188:&#x00BC;
VULGAR FRACTION ONE HALF:'BD:189:&#x00BD;
VULGAR FRACTION THREE QUARTERS:'BE:190:&#x00BE;
LATIN SMALL LETTER AE:'BF:230:&#x00E6;
LATIN CAPITAL LETTER A WITH OGONEK:'C0:260:&#x0104;
LATIN CAPITAL LETTER I WITH OGONEK:'C1:302:&#x012E;
LATIN CAPITAL LETTER A WITH MACRON:'C2:256:&#x0100;
LATIN CAPITAL LETTER C WITH ACUTE:'C3:262:&#x0106;
LATIN CAPITAL LETTER A WITH DIAERESIS:'C4:196:&#x00C4;
LATIN CAPITAL LETTER A WITH RING ABOVE:'C5:197:&#x00C5;
LATIN CAPITAL LETTER E WITH OGONEK:'C6:280:&#x0118;
LATIN CAPITAL LETTER E WITH MACRON:'C7:274:&#x0112;
LATIN CAPITAL LETTER C WITH CARON:'C8:268:&#x010C;
LATIN CAPITAL LETTER E WITH ACUTE:'C9:201:&#x00C9;
LATIN CAPITAL LETTER Z WITH ACUTE:'CA:377:&#x0179;
LATIN CAPITAL LETTER E WITH DOT ABOVE:'CB:278:&#x0116;
LATIN CAPITAL LETTER G WITH CEDILLA:'CC:290:&#x0122;
LATIN CAPITAL LETTER K WITH CEDILLA:'CD:310:&#x0136;
LATIN CAPITAL LETTER I WITH MACRON:'CE:298:&#x012A;
LATIN CAPITAL LETTER L WITH CEDILLA:'CF:315:&#x013B;
LATIN CAPITAL LETTER S WITH CARON:'D0:352:&#x0160;
LATIN CAPITAL LETTER N WITH ACUTE:'D1:323:&#x0143;
LATIN CAPITAL LETTER N WITH CEDILLA:'D2:325:&#x0145;
LATIN CAPITAL LETTER O WITH ACUTE:'D3:211:&#x00D3;
LATIN CAPITAL LETTER O WITH MACRON:'D4:332:&#x014C;
LATIN CAPITAL LETTER O WITH TILDE:'D5:213:&#x00D5;
LATIN CAPITAL LETTER O WITH DIAERESIS:'D6:214:&#x00D6;
MULTIPLICATION SIGN:'D7:215:&#x00D7;
LATIN CAPITAL LETTER U WITH OGONEK:'D8:370:&#x0172;
LATIN CAPITAL LETTER L WITH STROKE:'D9:321:&#x0141;
LATIN CAPITAL LETTER S WITH ACUTE:'DA:346:&#x015A;
LATIN CAPITAL LETTER U WITH MACRON:'DB:362:&#x016A;
LATIN CAPITAL LETTER U WITH DIAERESIS:'DC:220:&#x00DC;
LATIN CAPITAL LETTER Z WITH DOT ABOVE:'DD:379:&#x017B;
LATIN CAPITAL LETTER Z WITH CARON:'DE:381:&#x017D;
LATIN SMALL LETTER SHARP S (GERMAN):'DF:223:&#x00DF;
LATIN SMALL LETTER A WITH OGONEK:'E0:261:&#x0105;
LATIN SMALL LETTER I WITH OGONEK:'E1:303:&#x012F;
LATIN SMALL LETTER A WITH MACRON:'E2:257:&#x0101;
LATIN SMALL LETTER C WITH ACUTE:'E3:263:&#x0107;
LATIN SMALL LETTER A WITH DIAERESIS:'E4:228:&#x00E4;
LATIN SMALL LETTER A WITH RING ABOVE:'E5:229:&#x00E5;
LATIN SMALL LETTER E WITH OGONEK:'E6:281:&#x0119;
LATIN SMALL LETTER E WITH MACRON:'E7:275:&#x0113;
LATIN SMALL LETTER C WITH CARON:'E8:269:&#x010D;
LATIN SMALL LETTER E WITH ACUTE:'E9:233:&#x00E9;
LATIN SMALL LETTER Z WITH ACUTE:'EA:378:&#x017A;
LATIN SMALL LETTER E WITH DOT ABOVE:'EB:279:&#x0117;
LATIN SMALL LETTER G WITH CEDILLA:'EC:291:&#x0123;
LATIN SMALL LETTER K WITH CEDILLA:'ED:311:&#x0137;
LATIN SMALL LETTER I WITH MACRON:'EE:299:&#x012B;
LATIN SMALL LETTER L WITH CEDILLA:'EF:316:&#x013C;
LATIN SMALL LETTER S WITH CARON:'F0:353:&#x0161;
LATIN SMALL LETTER N WITH ACUTE:'F1:324:&#x0144;
LATIN SMALL LETTER N WITH CEDILLA:'F2:326:&#x0146;
LATIN SMALL LETTER O WITH ACUTE:'F3:243:&#x00F3;
LATIN SMALL LETTER O WITH MACRON:'F4:333:&#x014D;
LATIN SMALL LETTER O WITH TILDE:'F5:245:&#x00F5;
LATIN SMALL LETTER O WITH DIAERESIS:'F6:246:&#x00F6;
DIVISION SIGN:'F7:247:&#x00F7;
LATIN SMALL LETTER U WITH OGONEK:'F8:371:&#x0173;
LATIN SMALL LETTER L WITH STROKE:'F9:322:&#x0142;
LATIN SMALL LETTER S WITH ACUTE:'FA:347:&#x015B;
LATIN SMALL LETTER U WITH MACRON:'FB:363:&#x016B;
LATIN SMALL LETTER U WITH DIAERESIS:'FC:252:&#x00FC;
LATIN SMALL LETTER Z WITH DOT ABOVE:'FD:380:&#x017C;
LATIN SMALL LETTER Z WITH CARON:'FE:382:&#x017E;
</ansicpg1257>
#mac_roman
<ansicpg10000>
LATIN CAPITAL LETTER A WITH DIAERESIS:'80:196:&#x00C4;
LATIN CAPITAL LETTER A WITH RING ABOVE:'81:197:&#x00C5;
LATIN CAPITAL LETTER C WITH CEDILLA:'82:199:&#x00C7;
LATIN CAPITAL LETTER E WITH ACUTE:'83:201:&#x00C9;
LATIN CAPITAL LETTER N WITH TILDE:'84:209:&#x00D1;
LATIN CAPITAL LETTER O WITH DIAERESIS:'85:214:&#x00D6;
LATIN CAPITAL LETTER U WITH DIAERESIS:'86:220:&#x00DC;
LATIN SMALL LETTER A WITH ACUTE:'87:225:&#x00E1;
LATIN SMALL LETTER A WITH GRAVE:'88:224:&#x00E0;
LATIN SMALL LETTER A WITH CIRCUMFLEX:'89:226:&#x00E2;
LATIN SMALL LETTER A WITH DIAERESIS:'8A:228:&#x00E4;
LATIN SMALL LETTER A WITH TILDE:'8B:227:&#x00E3;
LATIN SMALL LETTER A WITH RING ABOVE:'8C:229:&#x00E5;
LATIN SMALL LETTER C WITH CEDILLA:'8D:231:&#x00E7;
LATIN SMALL LETTER E WITH ACUTE:'8E:233:&#x00E9;
LATIN SMALL LETTER E WITH GRAVE:'8F:232:&#x00E8;
LATIN SMALL LETTER E WITH CIRCUMFLEX:'90:234:&#x00EA;
LATIN SMALL LETTER E WITH DIAERESIS:'91:235:&#x00EB;
LATIN SMALL LETTER I WITH ACUTE:'92:237:&#x00ED;
LATIN SMALL LETTER I WITH GRAVE:'93:236:&#x00EC;
LATIN SMALL LETTER I WITH CIRCUMFLEX:'94:238:&#x00EE;
LATIN SMALL LETTER I WITH DIAERESIS:'95:239:&#x00EF;
LATIN SMALL LETTER N WITH TILDE:'96:241:&#x00F1;
LATIN SMALL LETTER O WITH ACUTE:'97:243:&#x00F3;
LATIN SMALL LETTER O WITH GRAVE:'98:242:&#x00F2;
LATIN SMALL LETTER O WITH CIRCUMFLEX:'99:244:&#x00F4;
LATIN SMALL LETTER O WITH DIAERESIS:'9A:246:&#x00F6;
LATIN SMALL LETTER O WITH TILDE:'9B:245:&#x00F5;
LATIN SMALL LETTER U WITH ACUTE:'9C:250:&#x00FA;
LATIN SMALL LETTER U WITH GRAVE:'9D:249:&#x00F9;
LATIN SMALL LETTER U WITH CIRCUMFLEX:'9E:251:&#x00FB;
LATIN SMALL LETTER U WITH DIAERESIS:'9F:252:&#x00FC;
DAGGER:'A0:8224:&#x2020;
DEGREE SIGN:'A1:176:&#x00B0;
CENT SIGN:'A2:162:&#x00A2;
POUND SIGN:'A3:163:&#x00A3;
SECTION SIGN:'A4:167:&#x00A7;
BULLET:'A5:8226:&#x2022;
PILCROW SIGN:'A6:182:&#x00B6;
LATIN SMALL LETTER SHARP S:'A7:223:&#x00DF;
REGISTERED SIGN:'A8:174:&#x00AE;
COPYRIGHT SIGN:'A9:169:&#x00A9;
TRADE MARK SIGN:'AA:8482:&#x2122;
ACUTE ACCENT:'AB:180:&#x00B4;
DIAERESIS:'AC:168:&#x00A8;
NOT EQUAL TO:'AD:8800:&#x2260;
LATIN CAPITAL LETTER AE:'AE:198:&#x00C6;
LATIN CAPITAL LETTER O WITH STROKE:'AF:216:&#x00D8;
INFINITY:'B0:8734:&#x221E;
PLUS-MINUS SIGN:'B1:177:&#x00B1;
LESS-THAN OR EQUAL TO:'B2:8804:&#x2264;
GREATER-THAN OR EQUAL TO:'B3:8805:&#x2265;
YEN SIGN:'B4:165:&#x00A5;
MICRO SIGN:'B5:181:&#x00B5;
PARTIAL DIFFERENTIAL:'B6:8706:&#x2202;
BULLET:'B7:8226:&#x2022;
N-ARY PRODUCT:'B8:8719:&#x220F;
GREEK SMALL LETTER PI:'B9:960:&#x03C0;
INTEGRAL:'BA:8747:&#x222B;
FEMININE ORDINAL INDICATOR:'BB:170:&#x00AA;
MASCULINE ORDINAL INDICATOR:'BC:186:&#x00BA;
GREEK CAPITAL LETTER OMEGA:'BD:937:&#x03A9;
LATIN SMALL LETTER AE:'BE:230:&#x00E6;
LATIN SMALL LETTER O WITH STROKE:'BF:248:&#x00F8;
INVERTED QUESTION MARK:'C0:191:&#x00BF;
INVERTED EXCLAMATION MARK:'C1:161:&#x00A1;
NOT SIGN:'C2:172:&#x00AC;
SQUARE ROOT:'C3:8730:&#x221A;
LATIN SMALL LETTER F WITH HOOK:'C4:402:&#x0192;
ALMOST EQUAL TO:'C5:8776:&#x2248;
INCREMENT:'C6:8710:&#x2206;
LEFT-POINTING DOUBLE ANGLE QUOTATION MARK:'C7:171:&#x00AB;
RIGHT-POINTING DOUBLE ANGLE QUOTATION MARK:'C8:187:&#x00BB;
HORIZONTAL ELLIPSIS:'C9:8230:&#x2026;
NO-BREAK SPACE:'CA:160:&#x00A0;
LATIN CAPITAL LETTER A WITH GRAVE:'CB:192:&#x00C0;
LATIN CAPITAL LETTER A WITH TILDE:'CC:195:&#x00C3;
LATIN CAPITAL LETTER O WITH TILDE:'CD:213:&#x00D5;
LATIN CAPITAL LIGATURE OE:'CE:338:&#x0152;
LATIN SMALL LIGATURE OE:'CF:339:&#x0153;
EN DASH:'D0:8211:&#x2013;
EM DASH:'D1:8212:&#x2014;
LEFT DOUBLE QUOTATION MARK:'D2:8220:&#x201C;
RIGHT DOUBLE QUOTATION MARK:'D3:8221:&#x201D;
LEFT SINGLE QUOTATION MARK:'D4:8216:&#x2018;
RIGHT SINGLE QUOTATION MARK:'D5:8217:&#x2019;
DIVISION SIGN:'D6:247:&#x00F7;
LOZENGE:'D7:9674:&#x25CA;
LATIN SMALL LETTER Y WITH DIAERESIS:'D8:255:&#x00FF;
LATIN CAPITAL LETTER Y WITH DIAERESIS:'D9:376:&#x0178;
FRACTION SLASH:'DA:8260:&#x2044;
EURO SIGN:'DB:8364:&#x20AC;
SINGLE LEFT-POINTING ANGLE QUOTATION MARK:'DC:8249:&#x2039;
SINGLE RIGHT-POINTING ANGLE QUOTATION MARK:'DD:8250:&#x203A;
LATIN SMALL LIGATURE FI:'DE:64257:&#xFB01;
LATIN SMALL LIGATURE FL:'DF:64258:&#xFB02;
DOUBLE DAGGER:'E0:8225:&#x2021;
MIDDLE DOT:'E1:183:&#x00B7;
SINGLE LOW-9 QUOTATION MARK:'E2:8218:&#x201A;
DOUBLE LOW-9 QUOTATION MARK:'E3:8222:&#x201E;
PER MILLE SIGN:'E4:8240:&#x2030;
LATIN CAPITAL LETTER A WITH CIRCUMFLEX:'E5:194:&#x00C2;
LATIN CAPITAL LETTER E WITH CIRCUMFLEX:'E6:202:&#x00CA;
LATIN CAPITAL LETTER A WITH ACUTE:'E7:193:&#x00C1;
LATIN CAPITAL LETTER E WITH DIAERESIS:'E8:203:&#x00CB;
LATIN CAPITAL LETTER E WITH GRAVE:'E9:200:&#x00C8;
LATIN CAPITAL LETTER I WITH ACUTE:'EA:205:&#x00CD;
LATIN CAPITAL LETTER I WITH CIRCUMFLEX:'EB:206:&#x00CE;
LATIN CAPITAL LETTER I WITH DIAERESIS:'EC:207:&#x00CF;
LATIN CAPITAL LETTER I WITH GRAVE:'ED:204:&#x00CC;
LATIN CAPITAL LETTER O WITH ACUTE:'EE:211:&#x00D3;
LATIN CAPITAL LETTER O WITH CIRCUMFLEX:'EF:212:&#x00D4;
APPLE LOGO:'F0:63743:&#xF8FF;
LATIN CAPITAL LETTER O WITH GRAVE:'F1:210:&#x00D2;
LATIN CAPITAL LETTER U WITH ACUTE:'F2:218:&#x00DA;
LATIN CAPITAL LETTER U WITH CIRCUMFLEX:'F3:219:&#x00DB;
LATIN CAPITAL LETTER U WITH GRAVE:'F4:217:&#x00D9;
LATIN SMALL LETTER DOTLESS I:'F5:305:&#x0131;
MODIFIER LETTER CIRCUMFLEX ACCENT:'F6:710:&#x02C6;
SMALL TILDE:'F7:732:&#x02DC;
MACRON:'F8:175:&#x00AF;
BREVE:'F9:728:&#x02D8;
DOT ABOVE:'FA:729:&#x02D9;
RING ABOVE:'FB:730:&#x02DA;
CEDILLA:'FC:184:&#x00B8;
DOUBLE ACUTE ACCENT:'FD:733:&#x02DD;
OGONEK:'FE:731:&#x02DB;
CARON:'FF:711:&#x02C7;
</ansicpg10000>
<caps_hex>
LATIN SMALL LETTER A:'61:97:'41
LATIN SMALL LETTER B:'62:98:'42
LATIN SMALL LETTER C:'63:99:'43
LATIN SMALL LETTER D:'64:100:'44
LATIN SMALL LETTER E:'65:101:'45
LATIN SMALL LETTER F:'66:102:'46
LATIN SMALL LETTER G:'67:103:'47
LATIN SMALL LETTER H:'68:104:48
LATIN SMALL LETTER I:'69:105:'49
LATIN SMALL LETTER J:'6A:106:'4a
LATIN SMALL LETTER K:'6B:107:'4b
LATIN SMALL LETTER L:'6C:108:'4c
LATIN SMALL LETTER M:'6D:109:'4d
LATIN SMALL LETTER N:'6E:110:'4e
LATIN SMALL LETTER O:'6F:111:'4f
LATIN SMALL LETTER P:'70:112:'50
LATIN SMALL LETTER Q:'71:113:'51
LATIN SMALL LETTER R:'72:114:'52
LATIN SMALL LETTER S:'73:115:'53
LATIN SMALL LETTER T:'74:116:'54
LATIN SMALL LETTER U:'75:117:'55
LATIN SMALL LETTER V:'76:118:'56
LATIN SMALL LETTER W:'77:119:'57
LATIN SMALL LETTER X:'78:120:'58
LATIN SMALL LETTER Y:'79:121:'59
LATIN SMALL LETTER Z:'7A:122:'5a
NO UNICODE VALUE:'87:135:\'E7
NO UNICODE VALUE:'8E:142:\'83
NO UNICODE VALUE:'EA:234:\'92
NO UNICODE VALUE:'97:151:\'EE
NO UNICODE VALUE:'9C:156:\'F2
NO UNICODE VALUE:'88:136:\'CB
NO UNICODE VALUE:'8F:143:\'E9
NO UNICODE VALUE:'93:147:\'ED
NO UNICODE VALUE:'98:152:\'F1
NO UNICODE VALUE:'9D:157:\'F4
NO UNICODE VALUE:'89:137:\'D5
NO UNICODE VALUE:'90:144:\'E6
NO UNICODE VALUE:'94:148:\'EB
NO UNICODE VALUE:'99:153:\'EF
NO UNICODE VALUE:'9E:158:\'F3
NO UNICODE VALUE:'BF:191:\'AF
NO UNICODE VALUE:'96:150:\'84
NO UNICODE VALUE:'9B:155:\'CD
NO UNICODE VALUE:'8B:139:\'CC
NO UNICODE VALUE:'8A:138:\'80
NO UNICODE VALUE:'91:145:\'E8
NO UNICODE VALUE:'95:149:\'EC
NO UNICODE VALUE:'9A:154:\'85
NO UNICODE VALUE:'9F:159:\'86
NO UNICODE VALUE:'8D:141:\'82
NO UNICODE VALUE:'8C:140:\'81
</caps_hex>
<caps_letters>
LATIN SMALL LETTER A:a:97:A
LATIN SMALL LETTER B:b:98:B
LATIN SMALL LETTER C:c:99:C
LATIN SMALL LETTER D:d:100:D
LATIN SMALL LETTER E:e:101:E
LATIN SMALL LETTER F:f:102:F
LATIN SMALL LETTER G:g:103:G
LATIN SMALL LETTER H:h:104:H
LATIN SMALL LETTER I:i:105:I
LATIN SMALL LETTER J:j:106:J
LATIN SMALL LETTER K:k:107:K
LATIN SMALL LETTER L:l:108:L
LATIN SMALL LETTER M:m:109:M
LATIN SMALL LETTER N:n:110:N
LATIN SMALL LETTER O:o:111:O
LATIN SMALL LETTER P:p:112:P
LATIN SMALL LETTER Q:q:113:Q
LATIN SMALL LETTER R:r:114:R
LATIN SMALL LETTER S:s:115:S
LATIN SMALL LETTER T:t:116:T
LATIN SMALL LETTER U:u:117:U
LATIN SMALL LETTER V:v:118:V
LATIN SMALL LETTER W:w:119:W
LATIN SMALL LETTER X:x:120:X
LATIN SMALL LETTER Y:y:121:Y
LATIN SMALL LETTER Z:z:122:Z
</caps_letters>
<SYMBOL>
MY UNDEFINED SYMBOL:'C3:00:<udef_symbol num="195"/>
SPACE:'20:32:
EXCLAMATION MARK:'21:33:!
FOR ALL:'22:8704:&#x2200;
NUMBER SIGN:'23:35:#
THERE EXISTS:'24:8707:&#x2203;
PERCENTAGE SIGN:'25:37:%
AMPERSAND:'26:38:&#x0026;
CONTAINS AS A MEMBER:'27:8715:&#x220B;
LEFT PARENTHESIS:'28:40:(
RIGHT PERENTHESIS:'29:41:)
ASTERISK OPERATOR:'2A:8727:&#x2217;
PLUS SIGN:'2B:43:+
COMMA:'2C:44:,
MINUS SIGN:'2D:8722:&#x2212;
FULL STOP:'2E:46:.
DIVISION SLASH:'2F:8725:&#x2215;
DIGIT ZERO:'30:48:0
DIGIT ONE:'31:49:1
DIGIT TWO:'32:50:2
DIGIT THREE:'33:51:3
DIGIT FOUR:'34:52:4
DIGIT FIVE:'35:53:5
DIGIT SIX:'36:54:6
DIGIT SEVEN:'37:55:7
DIGIT EIGHT:'38:56:8
DIGIT NINE:'39:57:9
RATIO:'3A:8758:&#x2236;
SEMICOLON:'3B:59:;
LESS-THAN SIGN:'3C:60:&lt;
EQUALS SIGN TO:'3D:61:=
GREATER-THAN SIGN:'3E:62:&gt;
QUESTION MARK:'3F:63:?
APPROXTIMATELY EQUAL TO:'40:8773:&#x2245;
GREEK CAPITOL LETTER ALPHA:'41:913:&#x0391;
GREEK CAPAITOL LETTER BETA:'42:914:&#x0392;
GREEK CAPITOL LETTER CHI:'43:935:&#x03A7;
GREEK CAPITOL LETTER DELTA:'44:916:&#x0394;
GREEK CAPITOL LETTER EPSILON:'45:917:&#x0395;
GREEK CAPITOL LETTER PHI:'46:934:&#x03A6;
GREEK CAPITOL LETTER GAMMA:'47:915:&#x0393;
GREEK CAPITOL LETTER ETA:'48:919:&#x0397;
GREEK CAPITOL LETTER ITOA:'49:913:&#x0391;
GREEK THETA SYMBOL:'4A:977:&#x03D1;
GREEK CAPITOL LETTER KAPPA:'4B:922:&#x039A;
GREEK CAPITOL LETTER LAMBDA:'4C:923:&#x039B;
GREEK CAPITOL LETTER MU:'4D:924:&#x039C;
GREEK CAPITOL LETTER NU:'4E:925:&#x039D;
GREEK CAPITOL LETTER OMICRON:'4F:927:&#x039F;
GREEK CAPITAL LETTER PI:'50:928:&#x03A0;
GREEK CAPITOL LETTER THETA:'51:920:&#x0398;
GREEK CAPITOL LETTER RHO:'52:929:&#x03A1;
GREEK CAPITOL LETTER SIGMA:'53:931:&#x03A3;
GREEK CAPITOL LETTER TAU:'54:932:&#x03A4;
GREEK CAPITOL LETTER UPSILON:'55:933:&#x03A5;
GREEK LETTER STIGMA:'56:986:&#x03DA;
GREEK CAPITOL LETTER OMEGA:'57:937:&#x03A9;
GREEK CAPITOL LETTER XI:'58:926:&#x039E;
GREEK CAPITOL LETTER PSI:'59:936:&#x03A8;
GREEK CAPITOL LETTER ZETA:'5A:918:&#x0396;
LEFT SQUARE BRACKET:'5B:91:&#x005B;
THEREFORE:'5C:8756:&#x2234;
RIGHT SQUARE BRACKET:'5D:93:&#x005D;
UP TACK:'5E:8869:&#x22A5;
MODIFIER LETTER LOW MACRON:'5F:717:&#x02CD;
MODIFIER LETTER MACRON:'60:713:&#x02C9;
GREEK SMALL LETTER ALPHA:'61:945:&#x03B1;
GREEK SMALL LETTER BETA:'62:946:&#x03B2;
GREEK SMALL LETTER CHI:'63:967:&#x03C7;
GREEK SMALL LETTER DELTA:'64:948:&#x03B4;
GREEK SMALL LETTER EPSILON:'65:949:&#x03B5;
GREEK PHI SYMBOL:'66:981:&#x03D5;
GREEK MSALL LETTER DELTA:'67:947:&#x03B3;
GREEK SMALL LETTER ETA:'68:951:&#x03B7;
GREEK SMALL LETTER IOTA:'69:953:&#x03B9;
GREEK SMALL LETTER PHI:'6A:966:&#x03C6;
GREEK SMALL LETTER KAPPA:'6B:954:&#x03BA;
GREEK SMALL LETTER LAMDA:'6C:955:&#x03BB;
GREEK SMALL LETTER MU:'6D:956:&#x03BC;
GREEK SMALL LETTER NU:'6E:957:&#x03BD;
GREEK SMALL LETTER OMICRON:'6F:959:&#x03BF;
GREEK SMALL LETTER PI:'70:960:&#x03C0;
GREEK SMALL LETTER THETA:'71:952:&#x03B8;
GREEK SMALL LETTER RHO:'72:961:&#x03C1;
GREEK SMALL LETTER SIGMA:'73:963:&#x03C3;
GREEK SMALL LETTER TAU:'74:964:&#x03C4;
GREEK SMALL LETTER UPSILON:'75:965:&#x03C5;
GREEK PI SYMBOL:'76:982:&#x03D6;
GREEK SMALL LETTER OMEGA:'77:969:&#x03C9;
GREEK SMALL LETTER XI:'78:958:&#x03BE;
GREEK SMALL LETTER PHI:'79:966:&#x03C6;
GREEK SMALL LETTER ZETA:'7A:950:&#x03B6;
LEFT CURLY BRACKET:'7B:123:{
DIVIDES:'7C:8739:&#x2223;
RIGHT CURLY BRACKET:'7D:125:}
TILDE OPERATOR:'7E:8764:&#x223C;
GREEK UPSILON WITH HOOK SYMBOL:'A1:978:&#x03D2;
COMBINING ACUTE TONE MARK:'A2:833:&#x0341;
LESS THAN OR EQUAL TO:'A3:8804:&#x2264;
DIVISION SLASH:'A4:8725:&#x2215;
INFINITY:'A5:8734:&#x221E;
LATIN SMALL LETTER F WITH HOOK:'A6:402:&#x0192;
BLACK CLUB SUIT:'A7:9827:&#x2663;
BLACK DIAMOND SUIT:'A8:9830:&#x2666;
BLACK HEART SUIT:'A9:9829:&#x2665;
BLACK SPADE SUIT:'AA:9824:&#x2660;
LEFT RIGHT ARROW:'AB:8596:&#x2194;
LEFTWARDS ARROW:'AC:8592:&#x2190;
UPWARDS ARROW:'AD:8593:&#x2191;
RIGHTWARDS ARROW:'AE:8594:&#x2192;
DOWNWARDS ARROW:'AF:8595:&#x2193;
DEGREE SIGN:'B0:176:&#x00B0;
PLUS OR MINUS SIGN:'B1:177:&#x00B1;
DOUBLE ACUTE ACCENT:'B2:733:&#x02DD;
GREATER THAN OR EQUAL TO:'B3:8805:&#x2265;
MULTIPLICATION SIGN:'B4:215:&#x00D7;
DON'T KNOW:'B5:8733:&#x221D;
PARTIAL DIFFERENTIAL:'B6:8706:&#x2202;
BULLET:'B7:183:&#x00B7;
DIVISION:'B8:247:&#x00F7;
NOT EQUAL TO:'B9:8800:&#x2260;
IDENTICAL TO:'BA:8801:&#x2261;
ALMOST EQUAL TO:'BB:8776:&#x2248;
MIDLINE HORIZONTAL ELLIPSES:'BC:8943:&#x22EF;
DIVIDES:'BD:8739:&#x2223;
BOX DRAWINGS LIGHT HORIZONTAL:'BE:9472:&#x2500;
DOWNWARDS ARROW WITH TIP LEFTWARDS:'BF:8626:&#x21B2;
CIRCLED TIMES:'C4:8855:&#x2297;
CIRCLED PLUS:'C5:8853:&#x2295;
EMPTY SET:'C6:8709:&#x2205;
INTERSECTION:'C7:8745:&#x2229;
UNION:'C8:8746:&#x222A;
SUPERSET OF:'C9:8835:&#x2283;
SUPERSET OF OR EQUAL TO:'CA:8839:&#x2287;
NIETHER A SUBSET OR EQUAL TO:'CB:8836:&#x2284;
SUBSET OF:'CC:8834:&#x2282;
SUBSET OR EQUAL TO:'CD:8838:&#x2286;
ELEMENT OF:'CE:8712:&#x2208;
NOT AN ELEMENT OF:'CF:8713:&#x2209;
ANGLE:'D0:8736:&#x2220;
WHITE DOWN POINTING TRIANBLE:'D1:9661:&#x25BD;
REGISTERED SIGN:'D2:174:&#x00AE;
COPYRIGHT:'D3:169:&#x00A9;
TRADEMARK SYMBOL:'D4:8482:&#x2122;
NARY OPERATOR:'D5:8719:&#x220F;
SQUARE ROOT:'D6:8730:&#x221A;
BULLET OPERATOR:'D7:8729:&#x2219;
NOT SIGN:'D8:172:&#x00AC;
LOGICAL AND:'D9:8743:&#x2227;
LOGICAL OR:'DA:8744:&#x2228;
LEFT RIGHT DOUBLE ARROW:'DB:8660:&#x21D4;
LEFTWARDS DOUBLE ARROW:'DC:8656:&#x21D0;
UPWARDS DOUBLE ARROW:'DD:8657:&#x21D1;
RIGHTWARDS DOUBLE ARROW:'DE:8658:&#x21D2;
DOWNWARDS DOUBLE ARROW:'DF:8659:&#x21D3;
BETWEEN:'E0:8812:&#x226C;
MATHEMATICAL LEFT ANGELBRACKET:'E1:10216:&#x27E8;
REGISTERED SIGN:'E2:174:&#x00AE;
COPYRIGHT:'E3:169:&#x00A9;
TRADEMARK SYMBOL:'E4:8482:&#x2122;
N-ARY SUMMATION:'E5:8721:&#x2211;
LARGE LEFT PARENTHESIS PART1:'E6:0:<udef_symbol num="0xE6" description="left_paraenthesis part 1"/>
LARGE LEFT PARENTHESIS PART2:'E7:0:<udef_symbol num="0xE7" description="left_parenthesis part 2"/>
LARGE LEFT PARENTHESIS PART3:'E8:0:<udef_symbol num="0xE8" description="left_paranethesis part 3"/>
LARGE LEFT SQUARE BRACKET PART1:'E9:0:<udef_symbol num="0xE9" description="left_square_bracket part 1"/>
LARGE LEFT SQUARE BRACKET PART2:'EA:0:<udef_symbol num="0xEA" description="left_square_bracket part 2"/>
LARGE LEFT SQUARE BRACKET PART3:'EB:0:<udef_symbol num="0xEF" description="left_square_bracket part 3"/>
LARGE LEFT BRACKET PART1:'EC:0:<udef_symbol num="0xEC" description="right_bracket part 1"/>
LARGE LEFT BRACKET PART2:'ED:0:<udef_symbol num="0xED" description="right_bracke tpart 2"/>
LARGE LEFT BRACKET PART3:'EE:0:<udef_symbol num="0xEE" description="right_bracket part 3"/>
DIVIDES:'EF:8739:&#x2223;
MATHEMATICAL RIGHT ANGLE BRACKET:'F1:10217:&#x27E9;
INTEGRAL:'F2:8747:&#x222B;
LARGE INTEGRAL PART 1:'F3:0:<udef_symbol num="0xF3" description="integral part 1"/>
LARGE INTEGRAL PART 2:'F4:0:<udef_symbol num="0xF4" description="integral part 2"/>
LARGE INTEGRAL PART 3:'F5:0:<udef_symbol num="0xF5" description="integral part 3"/>
LARGE RIGHT PARENTHESIS PART1:'F6:0:<udef_symbol num="0xF6" description="right_parenthesis part 1"/>
LARGE RIGHT PARENTHESIS PART2:'F7:0:<udef_symbol num="0xF7" description="right_parenthesis part 2"/>
LARGE RIGHT PARENTHESIS PART3:'F8:0:<udef_symbol num="0xF8" description="right_parenthesis part 3"/>
LARGE RIGHT SQUARE BRACKET PART1:'F9:0:<udef_symbol num="0xF9" description="right_square_bracket part 1"/>
LARGE RIGHT SQUARE BRACKET PART2:'FA:0:<udef_symbol num="0xFA" description="right_square_bracket part 2"/>
LARGE RIGHT SQUARE BRACKETPART3:'FB:0:<udef_symbol num="0xFB" description="right_square_bracket part 3"/>
LARGE RIGHT BRACKET PART1:'FC:0:<udef_symbol num="0xFC" description="right_bracket part 1"/>
LARGE RIGHT BRACKETPART2:'FD:0:<udef_symbol num="0xFD" description="right_bracket part 2"/>
LARGE RIGHT BRACKETPART3:'FE:0:<udef_symbol num="0xFE" description="right_bracket part 3"/>
DOUBLE ACUTE ACCENT:'B2:733:&#x02DD;
MY UNDEFINED SYMBOL:'7F:127:<udef_symbol num="0x7f"/>
MY UNDEFINED SYMBOL:'80:128:<udef_symbol num="0x80"/>
MY UNDEFINED SYMBOL:'81:129:<udef_symbol num="0x81"/>
MY UNDEFINED SYMBOL:'82:130:<udef_symbol num="130"/>
MY UNDEFINED SYMBOL:'83:131:<udef_symbol num="131"/>
MY UNDEFINED SYMBOL:'84:132:<udef_symbol num="132"/>
MY UNDEFINED SYMBOL:'85:133:<udef_symbol num="133"/>
MY UNDEFINED SYMBOL:'86:134:<udef_symbol num="134"/>
MY UNDEFINED SYMBOL:'87:135:<udef_symbol num="135"/>
MY UNDEFINED SYMBOL:'88:136:<udef_symbol num="136"/>
MY UNDEFINED SYMBOL:'89:137:<udef_symbol num="137"/>
MY UNDEFINED SYMBOL:'8A:138:<udef_symbol num="138"/>
MY UNDEFINED SYMBOL:'8B:139:<udef_symbol num="139"/>
MY UNDEFINED SYMBOL:'8C:140:<udef_symbol num="140"/>
MY UNDEFINED SYMBOL:'8D:141:<udef_symbol num="141"/>
MY UNDEFINED SYMBOL:'8E:142:<udef_symbol num="142"/>
MY UNDEFINED SYMBOL:'8F:143:<udef_symbol num="143"/>
MY UNDEFINED SYMBOL:'90:144:<udef_symbol num="144"/>
MY UNDEFINED SYMBOL:'91:145:<udef_symbol num="145"/>
MY UNDEFINED SYMBOL:'92:146:<udef_symbol num="146"/>
MY UNDEFINED SYMBOL:'93:147:<udef_symbol num="147"/>
MY UNDEFINED SYMBOL:'94:148:<udef_symbol num="148"/>
MY UNDEFINED SYMBOL:'95:149:<udef_symbol num="149"/>
MY UNDEFINED SYMBOL:'96:150:<udef_symbol num="150"/>
MY UNDEFINED SYMBOL:'97:151:<udef_symbol num="151"/>
MY UNDEFINED SYMBOL:'98:152:<udef_symbol num="152"/>
MY UNDEFINED SYMBOL:'99:153:<udef_symbol num="153"/>
MY UNDEFINED SYMBOL:'9A:154:<udef_symbol num="154"/>
MY UNDEFINED SYMBOL:'9B:155:<udef_symbol num="155"/>
MY UNDEFINED SYMBOL:'9C:156:<udef_symbol num="156"/>
MY UNDEFINED SYMBOL:'9D:157:<udef_symbol num="157"/>
MY UNDEFINED SYMBOL:'9E:158:<udef_symbol num="158"/>
MY UNDEFINED SYMBOL:'9F:159:<udef_symbol num="159"/>
MY UNDEFINED SYMBOL:'A0:160:<udef_symbol num="160"/>
MY UNDEFINED SYMBOL:'F0:160:<udef_symbol num="240"/>
</SYMBOL>
<ascii_to_hex>
SPACE: :32:\'20
EXCLAMATION MARK:!:33:\'21
QUOTATION MARK:":34:\'22
NUMBER SIGN:#:35:\'23
DOLLAR SIGN:$:36:\'24
PERCENT SIGN:%:37:\'25
AMPERSAND:&amp;:38:\'26
APOSTROPHE:':39:\'27
LEFT PARENTHESIS:(:40:\'28
RIGHT PARENTHESIS:):41:\'29
ASTERISK:*:42:\'2A
PLUS SIGN:+:43:\'2B
COMMA:,:44:\'2C
HYPHEN-MINUS:-:45:\'2D
FULL STOP:.:46:\'2E
SOLIDUS:/:47:\'2F
DIGIT ZERO:0:48:\'30
DIGIT ONE:1:49:\'31
DIGIT TWO:2:50:\'32
DIGIT THREE:3:51:\'33
DIGIT FOUR:4:52:\'34
DIGIT FIVE:5:53:\'35
DIGIT SIX:6:54:\'36
DIGIT SEVEN:7:55:\'37
DIGIT EIGHT:8:56:\'38
DIGIT NINE:9:57:\'39
COLON:\\colon:58:\'3A
SEMICOLON:;:59:\'3B
EQUALS SIGN:=:61:\'3D
QUESTION MARK:?:63:\'3F
LATIN CAPITAL LETTER A:A:65:\'41
LATIN CAPITAL LETTER B:B:66:\'42
LATIN CAPITAL LETTER C:C:67:\'43
LATIN CAPITAL LETTER D:D:68:\'44
LATIN CAPITAL LETTER E:E:69:\'45
LATIN CAPITAL LETTER F:F:70:\'46
LATIN CAPITAL LETTER G:G:71:\'47
LATIN CAPITAL LETTER H:H:72:\'48
LATIN CAPITAL LETTER I:I:73:\'49
LATIN CAPITAL LETTER J:J:74:\'4A
LATIN CAPITAL LETTER K:K:75:\'4B
LATIN CAPITAL LETTER L:L:76:\'4C
LATIN CAPITAL LETTER M:M:77:\'4D
LATIN CAPITAL LETTER N:N:78:\'4E
LATIN CAPITAL LETTER O:O:79:\'4F
LATIN CAPITAL LETTER P:P:80:\'50
LATIN CAPITAL LETTER Q:Q:81:\'51
LATIN CAPITAL LETTER R:R:82:\'52
LATIN CAPITAL LETTER S:S:83:\'53
LATIN CAPITAL LETTER T:T:84:\'54
LATIN CAPITAL LETTER U:U:85:\'55
LATIN CAPITAL LETTER V:V:86:\'56
LATIN CAPITAL LETTER W:W:87:\'57
LATIN CAPITAL LETTER X:X:88:\'58
LATIN CAPITAL LETTER Y:Y:89:\'59
LATIN CAPITAL LETTER Z:Z:90:\'5A
LEFT SQUARE BRACKET:[:91:\'5B
REVERSE SOLIDUS:\\:92:\'5C
RIGHT SQUARE BRACKET:]:93:\'5D
LATIN SMALL LETTER A:a:97:\'61
LATIN SMALL LETTER B:b:98:\'62
LATIN SMALL LETTER C:c:99:\'63
LATIN SMALL LETTER D:d:100:\'64
LATIN SMALL LETTER E:e:101:\'65
LATIN SMALL LETTER F:f:102:\'66
LATIN SMALL LETTER G:g:103:\'67
LATIN SMALL LETTER H:h:104:\'68
LATIN SMALL LETTER I:i:105:\'69
LATIN SMALL LETTER J:j:106:\'6A
LATIN SMALL LETTER K:k:107:\'6B
LATIN SMALL LETTER L:l:108:\'6C
LATIN SMALL LETTER M:m:109:\'6D
LATIN SMALL LETTER N:n:110:\'6E
LATIN SMALL LETTER O:o:111:\'6F
LATIN SMALL LETTER P:p:112:\'70
LATIN SMALL LETTER Q:q:113:\'71
LATIN SMALL LETTER R:r:114:\'72
LATIN SMALL LETTER S:s:115:\'73
LATIN SMALL LETTER T:t:116:\'74
LATIN SMALL LETTER U:u:117:\'75
LATIN SMALL LETTER V:v:118:\'76
LATIN SMALL LETTER W:w:119:\'77
LATIN SMALL LETTER X:x:120:\'78
LATIN SMALL LETTER Y:y:121:\'79
LATIN SMALL LETTER Z:z:122:\'7A
LEFT CURLY BRACKET:{:123:\'7B
VERTICAL LINE:|:124:\'7C
RIGHT CURLY BRACKET:}:125:\'7D
TILDE:~:126:\'7E
</ascii_to_hex>
<wingdings>
SPACE:'20:32:&#x0020;
LOWER RIGHT PENCIL:'21:9998:&#x270E;
BLACK SCISSORS:'22:9986:&#x2702;
UPPER BLADE SCISSORS:'23:9985:&#x2701;
PROPOSE "LOWER LEFT SPECTACLES":'24:none:<udef_symbol num="0x24" description="lower_left_spectacles"/>
PROPOSE "BELL":'25:none:<udef_symbol num="0x25" description="bell"/>
PROPOSE "OPEN BOOK":'26:none:<udef_symbol num="0x26" description="open_book"/>
PROPOSE "LIGHTED CANDLE":'27:none:<udef_symbol num="0x27" description="lighted_candle"/>
BLACK TELEPHONE:'28:9742:&#x260E;
TELEPHONE LOCATION SIGN:'29:9990:&#x2706;
ENVELOPE:'2A:9993:&#x2709;
ENVELOPE:'2B:9993:&#x2709;
PROPOSE "MAIL FLAG DOWN":'2C:none:<udef_symbol num="0x2C" description="mail_flag_down"/>
PROPOSE "MAIL FLAG UP":'2D:none:<udef_symbol num="0x2D" description="mail_flag_up"/>
PROPOSE "MAIL FULL":'2E:none:<udef_symbol num="0x2E" description="mail_full"/>
PROPOSE "MAIL EMPTY":'2F:none:<udef_symbol num="0x2F" description="mail_empty"/>
PROPOSE "FOLDER CLOSE":'30:none:<udef_symbol num="0x30" description="folder_close"/>
PROPOSE "FOLDER OPEN":'31:none:<udef_symbol num="0x31" description="folder_open"/>
PROPOSE "DOCUMENT FOLDED":'32:none:<udef_symbol num="0x32" description="document_folded"/>
PROPOSE "DOCUMENT":'33:none:<udef_symbol num="0x33" description="document"/>
PROPOSE "MULTIPLE DOCUMENTS":'34:none:<udef_symbol num="0x34" description="multiple_documents"/>
PROPOSE "FILE CABINET":'35:none:<udef_symbol num="0x35" description="file_cabinet"/>
HOURGLASS:'36:8987:&#x231B;
KEYBOARD:'37:9000:&#x2328;
PROPOSE "MOUSE":'38:none:<udef_symbol num="0x38" description="mouse"/>
PROPOSE "QUICKCAM CAMERA":'39:none:<udef_symbol num="0x39" description="quickcam_camera"/>
PROPOSE "COMPUTER":'3A:none:<udef_symbol num="0x3A" description="computer"/>
PROPOSE "HARD DRIVE":'3B:none:<udef_symbol num="3B" description="hard_drive"/>
PROPOSE "THREE AND A HALF FLOPPY":'3C:none:<udef_symbol num="0x3c" description = "three_and_a_half_floppy"/>
PROPOSE "FIVE AND A QUARTER FLOPPY":'3D:none:<udef_symbol num="0x3D" description="five_and_a_quarter_floppy"/>
TAPE DRIVE:'3E:9991:&#x2707;
WRITING HAND:'3F:9997:&#x270D;
WRITING HAND:'40:9997:&#x270D;
VICTORY HAND:'41:9996:&#x270C;
PROPOSE "PICKING HAND(OR OMMAT)":'42:none:<udef_symbol num="0x42" description="picking_hand_or_ommat"/>
PROPOSE "WHITE UP POINTING THUMB":'43:none:<udef_symbol num="0x43" description="white_up_pointing_thumb"/>
PROPOSE "WHITE DOWN POINTING THUMB":'44:none:<udef_symbol num="0x44" description="white_down_pointing_thumb"/>
WHITE LEFT POINTING INDEX:'45:9756:&#x261C;
WHITE RIGHT POINTING INDEX:'46:9758:&#x261E;
WHITE UP POINTING INDEX:'47:9757:&#x261D;
WHITE DOWN POINTING INDEX:'48:9759:&#x261F;
PROPOSE "WHITE PALM":'49:none:<udef_symbol num="0x49" description="white_palm"/>
WHITE SMILING FACE:'4A:9786:&#x263A;
WHITE SMILING FACE":'4B:9786:&#x263A;
WHITE FROWNING FACE:'4C:9785:&#x2639;
PROPOSE "BLACK BOMB WITH FUSE":'4D:none:<udef_symbol num="0x4D" description="black_bomb_with_fuse"/>
SKULL AND CROSSBONES:'4E:9760:&#x2620;
PROPOSE "WHITE BILLOWING SQUARE FLAG":'4F:none:<udef_symbol num="0x4F" description="white_billowing_square_flag"/>
PROPOSE "WHITE BILLOWING TRIANGLE FLAG":'50:none:<udef_symbol num="0x50" description="white_billowing_triangle_flag"/>
AIRPLANE:'51:9992:&#x2708;
WHITE SUN WITH RAYS:'52:9788:&#x263C;
PROPOSE "INK BLOT":'53:none:<udef_symbol num="0x53" description="ink_blot"/>
SNOWFLAKE:'54:10052:&#x2744;
SHADOWED WHITE LATIN CROSS:'55:10014:&#x271E;
SHADOWED WHITE LATIN CROSS:'56:10014:&#x271E;
LATIN CROSS:'57:10013:&#x271D;
MALTESE CROSS:'58:10016:&#x2720;
STAR OF DAVID:'59:10017:&#x2721;
STAR AND CRESCENT:'5A:9770:&#x262A;
YIN YANG:'5B:9775:&#x262F;
DEVANGARI OM CORRECT:'5C:2384:&#x0950;
WHEEL OF DHARMA:'5D:9784:&#x2638;
ARIES:'5E:9800:&#x2648;
TAURUS:'5F:9801:&#x2649;
GEMINI:'60:9802:&#x264A;
CANCER:'61:9803:&#x264B;
LEO:'62:9804:&#x264C;
VIRGO:'63:9805:&#x264D;
LIBRA:'64:9806:&#x264E;
SCORPIUS:'65:9807:&#x264F;
SAGITTARIUS:'66:9808:&#x2650;
CAPRICORN:'67:9809:&#x2651;
AQUARIUS:'68:9810:&#x2652;
PISCES:'69:9811:&#x2653;
AMPERSAND:'6A:38:&#x0026;
AMPERSAND:'6B:38:&#x0026;
BLACK CIRCLE:'6C:9679:&#x25CF;
SHADOWED WHITE CIRCLE:'6D:10061:&#x274D;
BLACK SQUARE:'6E:9632:&#x25A0;
WHITE SQUARE:'6F:9633:&#x25A1;
WHITE SQUARE:'70:9633:&#x25A1;
LOWER RIGHT SHADOWED WHITE SQUARE:'71:10065:&#x2751;
UPPER RIGHT SHADOWED WHITE SQUARE:'72:10066:&#x2752;
LOZENGE:'73:9674:&#x25CA;
LOZENGE:'74:9674:&#x25CA;
BLACK DIAMOND:'75:9670:&#x25C6;
BLACK DIAMOND MINUS WHITE X:'76:10070:&#x2756;
BLACK DIAMOND:'77:9670:&#x25C6;
X IN A RECTANGLE BOX:'78:8999:&#x2327;
APL FUNCTIONAL SYMBOL QUAD UP CARET:'79:9043:&#x2353;
PLACE OF INTEREST SIGN:'7A:8984:&#x2318;
WHITE FLORETTE:'7B:10048:&#x2740;
BLACK FLORETTE:'7C:10047:&#x273F;
HEAVY DOUBLE TURNED COMMA QUOTATION MARK ORNAMENT:'7D:10077:&#x275D;
HEAVY DOUBLE COMMA QUOTATION MARK ORNAMENT:'7E:10078:&#x275E;
"UNUSED":'7F:none:udef_symbol/>
CIRCLED DIGIT ZERO:'80:9450:&#x24EA;
CIRCLED DIGIT ONE:'81:9312:&#x2460;
CIRCLED DIGIT TWO:'82:9313:&#x2461;
CIRCLED DIGIT THREE:'83:9314:&#x2462;
CIRCLED DIGIT FOUR:'84:9315:&#x2463;
CIRCLED DIGIT FIVE:'85:9316:&#x2464;
CIRCLED DIGIT SIX:'86:9317:&#x2465;
CIRCLED DIGIT SEVEN:'87:9318:&#x2466;
CIRCLED DIGIT EIGHT:'88:9319:&#x2467;
CIRCLED DIGIT NINE:'89:9320:&#x2468;
CIRCLED NUMBER TEN:'8A:9321:&#x2469;
PROPOSE "DINGBAT NEGATIVE CIRCLED DIGIT ZERO":'8B:none:<udef_symbol num="0x8B" description="dingbat_negative_circled_digit_zero"/>
DINGBAT NEGATIVE CIRCLED DIGIT ONE:'8C:10102:&#x2776;
DINGBAT NEGATIVE CIRCLED DIGIT TWO:'8D:10103:&#x2777;
DINGBAT NEGATIVE CIRCLED DIGIT THREE:'8E:10104:&#x2778;
DINGBAT NEGATIVE CIRCLED DIGIT FOUR:'8F:10105:&#x2779;
DINGBAT NEGATIVE CIRCLED DIGIT FIVE:'90:10106:&#x277A;
DINGBAT NEGATIVE CIRCLED DIGIT SIX:'91:10107:&#x277B;
DINGBAT NEGATIVE CIRCLED DIGIT SEVEN:'92:10108:&#x277C;
DINGBAT NEGATIVE CIRCLED DIGIT EIGHT:'93:10109:&#x277D;
DINGBAT NEGATIVE CIRCLED DIGIT NINE:'94:10110:&#x277E;
DINGBAT NEGATIVE CIRCLED NUMBER TEN:'95:10111:&#x277F;
ROTATED FLORAL HEART BULLET:'96:10087:&#x2767;
REVERSED ROTATED FLORAL HEART BULLET:'97:9753:&#x2619;
REVERSED ROTATED FLORAL HEART BULLET:'98:9753:&#x2619;
ROTATED FLORAL HEART BULLET:'99:10087:&#x2767;
ROTATED FLORAL HEART BULLET:'9A:10087:&#x2767;
REVERSED ROTATED FLORAL HEART BULLET:'9B:9753:&#x2619;
REVERSED ROTATED FLORAL HEART BULLET:'9C:9753:&#x2619;
ROTATED FLORAL HEART BULLET:'9D:10087:&#x2767;
BULLET:'9E:8226:&#x2022;
BLACK CIRCLE:'9F:9679:&#x25CF;
DON'T KNOW:'A0:160:&#x00A0;
WHITE CIRCLE:'A1:9675:&#x25CB;
WHITE CIRCLE:'A2:9675:&#x25CB;
WHITE CIRCLE:'A3:9675:&#x25CB;
SUN:'A4:9737:&#x2609;
SUN:'A5:9737:&#x2609;
SHADOWED WHITE CIRCLE:'A6:10061:&#x274D;
BLACK SMALL SQUARE:'A7:9642:&#x25AA;
WHITE SQUARE:'A8:9633:&#x25A1;
PROPOSE "THEE MIGHT BE IN THERE SOMEWHERE":'A9:none:<udef_symbol num="0xA8" description="thee_might_be_in_there_somewhere"/>
BLACK FOUR POINTED STAR MAYBE:'AA:10022:&#x2726;
BLACK STAR:'AB:9733:&#x2605;
SIX POINTED BLACK STAR:'AC:10038:&#x2736;
EIGHT POINTED RECTILINEAR BLACK STAR:'AD:10039:&#x2737;
TWELVE POINTED BLACK STAR:'AE:10040:&#x2738;
EIGHT POINTED PINWHEEL STAR:'AF:10037:&#x2735;
PROPOSE "CROSSHAIR SQUARE":'B0:none:<udef_symbol num="0xB0" description="crosshair_square"/>
PROPOSE "CROSSHAIR CIRCLE":'B1:none:<udef_symbol num="0xB1" description="crosshair_circle"/>
WHITE FOUR POINTED STAR:'B2:10023:&#x2727;
PROPOSE "THIS HAS TO BE A KNOWN SYMBOL":'B3:none:<udef_symbol num="0xB3" description="this_has_to_be_a_known_symbol"/>
REPLACEMENT CHARACTER:'B4:65533:&#xFFFD;
CIRCLED WHITE STAR:'B5:10026:&#x272A;
SHADOWED WHITE STAR:'B6:10032:&#x2730;
PROPOSE "1 OCLOCK":'B7:none:<udef_symbol num="0xB7" description="one_oclock"/>
PROPOSE "2 OCLOCK":'B8:none:<udef_symbol num="0xB8" description="two_oclock"/>
PROPOSE "3 OCLOCK":'B9:none:<udef_symbol num="0xB9" description="three_oclock"/>
PROPOSE "4 OCLOCK":'BA:none:<udef_symbol num="0xBA" description="four_oclock"/>
PROPOSE "5 OCLOCK":'BB:none:<udef_symbol num="0xBB" description="five_oclock"/>
PROPOSE "6 OCLOCK":'BC:none:<udef_symbol num="0xBC" description="six_oclock"/>
PROPOSE "7 OCLOCK":'BD:none:<udef_symbol num="0xBD" description="seven_oclock"/>
PROPOSE "8 OCLOCK":'BE:none:<udef_symbol num="0xBE" description="eight_oclock"/>
PROPOSE "9 OCLOCK":'BF:none:<udef_symbol num="0xBF" description="nine_oclock"/>
PROPOSE "10 OCLOCK":'C0:none:<udef_symbol num="0xC0" description="ten_oclock"/>
PROPOSE "11 OCLOCK":'C1:none:<udef_symbol num="0xC1" description="eleven_oclock"/>
PROPOSE "12 OCLOCK":'C2:none:<udef_symbol num="0xC2" description="twelve_oclock"/>
PROPOSE "NOTCHED DOWNWARDS DOUBLE ARROW WITH TIP LEFTWARDS":'C3:none:<udef_symbol num="0xC3" description="notched_downwards_double_arrow_with_tip_leftwards"/>
PROPOSE "NOTCHED DOWNWARDS DOUBLE ARROW WITH TIP RIGHTWARDS":'C4:none:<udef_symbol num="0xC4" description="notched_downwards_double_arrow_with_tip_rightwards"/>
PROPOSE "NOTCHED UPWARDS DOUBLE ARROW WITH TIP LEFTWARDS":'C5:none:<udef_symbol num="0xC5" description="notched_upwards_double_arrow_with_tip_leftwards"/>
PROPOSE "NOTCHED UPWARDS DOUBLE ARROW WITH TIP RIGHTWARDS":'C6:none:<udef_symbol num="0xC6" description="notched_upwards_double_arrow_with_tip_rightwards"/>
PROPOSE "NOTCHED LEFTWARDS DOUBLE ARROW WITH TIP UPWARDS":'C7:none:<udef_symbol num="0xC7" description="notched_leftwards_double_arrow_with_tip_upwards"/>
PROPOSE "NOTCHED RIGHTWARDS DOUBLE ARROW WITH TIP UPWARDS":'C8:none:<udef_symbol num="0xC8" description="notched_rightwards_double_arrow_with_tip_upwards"/>
PROPOSE "NOTCHED LEFTWARDS DOUBLE ARROW WITH TIP DOWNWARDS":'C9:none:<udef_symbol num="0xC0" description="notched_leftwards_double_arrow_with_tip_downwards"/>
PROPOSE "NOTCHED RIGHTWARDS DOUBLE ARROW WITH TIP DOWNWARDS":'CA:none:<udef_symbol num="0xCA" description="notched_rightwards_double_arrow_with_tip_downwards"/>
PROPOSE "NO IDEA":'CB:none:<udef_symbol num="0xCB" description="no_idea"/>
PROPOSE "REVERSE OF ABOVE":'CC:none:<udef_symbol num="0xCC" description="reverse_of_above"/>
PROPOSE "HEDERA LOWER LEFT":'CD:none:<udef_symbol num="0xCD" description="hedera_lower_left"/>
PROPOSE "HEDERA UPPER LEFT REVERSED":'CE:none:<udef_symbol num="0xCE" description="hedera_upper_left_reversed"/>
PROPOSE "HEDERA LOWER RIGHT REVERSED":'CF:none:<udef_symbol num="0xCF" description="hedera_lower_right_reversed"/>
PROPOSE "HEDERA UPPER RIGHT":'D0:none:<udef_symbol num="0xD0" description="hedera_upper_right"/>
PROPOSE "HEDERA UPPER LEFT":'D1:none:<udef_symbol num="0xD1" description="hedera_upper_left"/>
PROPOSE "HEDERA LOWER LEFT REVERSED":'D2:none:<udef_symbol num="0xD2" description="hedera_lower_left_reversed"/>
PROPOSE "HEDERA UPPER RIGHT REVERSED":'D3:none:<udef_symbol num="0xD3" description="hedera_upper_right_reversed"/>
PROPOSE "HEDERA LOWER RIGHT":'D4:none:<udef_symbol num="0xD4" description="hedera_lower_right"/>
ERASE TO THE LEFT:'D5:9003:&#x232B;
ERASE TO THE RIGHT:'D6:8998:&#x2326;
PROPOSE "THREE-D TOP-LIGHTED LEFTWARDS ARROWHEAD":'D7:none:<udef_symbol num="0xD7" description="three-d_top-lighted_leftwards_arrowhead"/>
THREE-D TOP-LIGHTED RIGHTWARDS ARROWHEAD:'D8:10146:&#x27A2;
PROPOSE "THREE-D RIGHT-LIGHTED UPWARDS ARROWHEAD":'D9:none:<udef_symbol num="0xD0" description="three-d_right-lighted_upwards_arrowhead"/>
PROPOSE "THREE-D LEFT-LIGHTED DOWNWARDS ARROWHEAD":'DA:none:<udef_symbol num="0xDA" description="three-d_left-lighted_downwards_arrowhead"/>
PROPOSE "CIRCLED HEAVY WHITE LEFTWARDS ARROW":'DB:none:<udef_symbol num="0xDB" description="circled_heavy_white_leftwards_arrow"/>
CIRCLED HEAVY WHITE RIGHTWARDS ARROW:'DC:10162:&#x27B2;
PROPOSE "CIRCLED HEAVY WHITE UPWARDS ARROW":'DD:none:<udef_symbol num="0xDD" description="circled_heavy_white_upwards_arrow"/>
PROPOSE "CIRCLED HEAVY WHITE DOWNWARDS ARROW":'DE:none:<udef_symbol num="0xDE" description="circled_heavy_white_downwards_arrow"/>
PROPOSE "WIDE-HEADED LEFTWARDS ARROW":'DF:none:<udef_symbol num="0xDF" description="wide-headed_leftwards_arrow"/>
PROPOSE "WIDE-HEADED RIGHTWARDS ARROW":'E0:none:<udef_symbol num="0xE0" description="wide-headed_rightwards_arrow"/>
PROPOSE "WIDE-HEADED UPWARDS ARROW":'E1:none:<udef_symbol num="0xE1" description="wide-headed_upwards_arrow"/>
PROPOSE "WIDE-HEADED DOWNWARDS ARROW":'E2:none:<udef_symbol num="0xE2" description="wide-headed_downwards_arrow"/>
PROPOSE "WIDE-HEADED NORTHWEST-WARDS ARROW":'E3:none:<udef_symbol num="0xE3" description="wide-headed_northwest-wards_arrow"/>
PROPOSE "WIDE-HEADED NORTHEAST-WARDS ARROW":'E4:none:<udef_symbol num="0xE4" description="wide-headed_northeast-wards_arrow"/>
PROPOSE "WIDE-HEADED SOUTHWEST-WARDS ARROW":'E5:none:<udef_symbol num="0xE5" description="wide-headed_southwest-wards_arrow"/>
PROPOSE "WIDE-HEADED SOUTHEAST-WARDS ARROW":'E6:none:<udef_symbol num="0xE6" description="wide-headed_southeast-wards_arrow"/>
PROPOSE "HEAVY WIDE-HEADED LEFTWARDS ARROW":'E7:none:<udef_symbol num="0xE7" description="heavy_wide-headed_leftwards_arrow"/>
HEAVY WIDE-HEADED RIGHTWARDS ARROW:'E8:10132:&#x2794;
PROPOSE "HEAVY WIDE-HEADED UPWARDS ARROW":'E9:none:<udef_symbol num="0xE9" description="heavy_wide-headed_upwards_arrow"/>
PROPOSE "HEAVY WIDE-HEADED DOWNWARDS ARROW":'EA:none:<udef_symbol num="0xEA" description="heavy_wide-headed_downwards_arrow"/>
PROPOSE "HEAVY WIDE-HEADED NORTHWEST-WARDS ARROW":'EB:none:<udef_symbol num="0xEB" description="heavy_wide-headed_northwest-wards_arrow"/>
PROPOSE "HEAVY WIDE-HEADED NORTHEAST-WARDS ARROW":'EC:none:<udef_symbol num="0xEC" description="heavy_wide-headed_northeast-wards_arrow"/>
PROPOSE "HEAVY WIDE-HEADED SOUTHWEST-WARDS ARROW":'ED:none:<udef_symbol num="0xED" description="heavy_wide-headed_southwest-wards_arrow"/>
PROPOSE "HEAVY WIDE-HEADED SOUTHEAST-WARDS ARROW":'EE:none:<udef_symbol num="EE" description="heavy_wide-headed_southeast-wards_arrow"/>
LEFTWARDS WHITE ARROW:'EF:8678:&#x21E6;
RIGHTWARDS WHITE ARROW:'F0:8680:&#x21E8;
UPWARDS WHITE ARROW:'F1:8679:&#x21E7;
DOWNWARDS WHITE ARROW:'F2:8681:&#x21E9;
LEFT RIGHT DOUBLE ARROW:'F3:8660:&#x21D4;
UP DOWN DOUBLE ARROW:'F4:8661:&#x21D5;
NORTH WEST DOUBLE ARROW:'F5:8662:&#x21D6;
NORTH EAST DOUBLE ARROW:'F6:8663:&#x21D7;
SOUTH WEST DOUBLE ARROW:'F7:8665:&#x21D9;
SOUTH EAST DOUBLE ARROW:'F8:8664:&#x21D8;
"NO IDEA":'F9:none:<udef_symbol num="0xF9" description="no_idea"/>
"NO IDEA":'FA:none:<udef_symbol num="0xFA" description="no_idea"/>
BALLOT X:'FB:10007:&#x2717;
CHECK MARK:'FC:10003:&#x2713;
BALLOT BOX WITH X:'FD:9746:&#x2612;
BALLOT BOX WITH CHECK:'FE:9745:&#x2611;
PROPOSE "MICROSOFT WINDOWS LOGO":'FF:none:<udef_symbol num="0xFF" description="microsoft_windows_logo"/>
</wingdings>
<dingbats>
SPACE:'20:32:
UPPER BLADE SCISSORS:'21:9985:&#x2701;
BLACK SCISSORS:'22:9986:&#x2702;
LOWER BLADE SCISSORS:'23:9987:&#x2703;
WHITE SCISSORS:'24:9988:&#x2704;
BLACK TELEPHONE:'25:9742:&#x260E;
TELEPHONE LOCATION SIGN:'26:9990:&#x2706;
TAPE DRIVE:'27:9991:&#x2707;
AIRPLANE:'28:9992:&#x2708;
ENVELOPE:'29:9993:&#x2709;
BLACK RIGHT POINTING INDEX:'2A:9755:&#x261B;
WHITE RIGHT POINTING INDEX:'2B:9758:&#x261E;
VICTORY HAND:'2C:9996:&#x270C;
WRITING HAND:'2D:9997:&#x270D;
LOWER RIGHT PENCIL:'2E:9998:&#x270E;
PENCIL:'2F:9999:&#x270F;
UPPER RIGHT PENCIL:'30:10000:&#x2710;
WHITE NIB:'31:10001:&#x2711;
BLACK NIB:'32:10002:&#x2712;
CHECKMARK:'33:10003:&#x2713;
HEAVY CHECKMARK:'34:10004:&#x2714;
MULTIPLICATION X:'35:10005:&#x2715;
HEAVY MULTIPLICATION X:'36:10006:&#x2716;
BALLOT X:'37:10007:&#x2717;
HEAVY BALLOT X:'38:10008:&#x2718;
OUTLINED GREEK CROSS:'39:10009:&#x2719;
HEAVY GREK CROSS:'3A:10010:&#x271A;
OPEN CENTRE CROSS:'3B:10011:&#x271B;
HEAVY OPEN CENTRE CROSS:'3C:10011:&#x271B;
LATIN CROSS:'3D:10013:&#x271D;
SHADOWED WHITE LATIN CROSS:'3E:10014:&#x271E;
OUTLINED LATIN CROSS:'3F:10015:&#x271F;
MALTESE CROSS:'40:10016:&#x2720;
STAR OF DAVID:'41:10017:&#x2721;
FOUR TEARDROP-SPOKED ASTERISK:'42:10018:&#x2722;
FOUR BALLOON-SPOKED ASTERISK:'43:10019:&#x2723;
10019:'43:10019:&#x2723;
HEAVY FOUR BALLOON-SPOKED ASTERISK:'44:10020:&#x2724;
FOUR CLUB-SPOKED ASTERISK:'45:10021:&#x2725;
BLACK FOUR POINTED STAR:'46:10022:&#x2726;
WHITE FOUR POINTED STAR:'47:10023:&#x2727;
BLACK STAR:'48:9989:&#x2705;
STRESS OUTLINED WHITE STAR:'49:10025:&#x2729;
CIRCLED WHITE STAR:'4A:10026:&#x272A;
OPEN CENTRE BLACK STAR:'4B:10027:&#x272B;
BLACK CENTRE WHITE STAR:'4C:10028:&#x272C;
OUTLINED BLACK STAR:'4D:10029:&#x272D;
HEAVY OUTLINED BLACK STAR:'4E:10030:&#x272E;
PINWHEEL STAR:'4F:10031:&#x272F;
SHADOWED WHITE STAR:'50:10032:&#x2730;
HEAVY ASTERISK:'51:10033:&#x2731;
OPEN CENTRE ASTERISK:'52:10034:&#x2732;
EIGHT SPOKED ASTERISK:'53:10035:&#x2733;
EIGHT POINTED BLACK STAR:'54:10036:&#x2734;
EIGHT POINTED PINWHEEL STAR:'55:10037:&#x2735;
SIX POINTED BLACK STAR:'56:10038:&#x2736;
EIGHT  POINTED RECTILINEAR BLACK STAR:'57:10039:&#x2737;
HEAVY EIGHT POINTED RECTILINEAR BLACK STAR:'58:10040:&#x2738;
TWELVE POINTED BLACK STAR:'59:10041:&#x2739;
SIXTEEN POINTED ASTERISK:'5A:10042:&#x273A;
TEARDROP-SPOKED ASTERISK:'5B:10043:&#x273B;
OPEN CENTRE TEARDROP-SPOKED ASTERISK:'5C:10044:&#x273C;
HEAVY TEARDROP-SPOKED ASTERISK:'5D:10045:&#x273D;
SIX PETALLED BLACK AND WHITE FLORETTE:'5E:10046:&#x273E;
BLACK FLORETTE:'5F:10047:&#x273F;
WHITE FLORETTE:'60:10048:&#x2740;
EIGHT PETALLED OUTLINED BLACK FLORETTE:'61:10049:&#x2741;
CIRCLED OPEN CENTRE EIGHT POINTED STAR:'62:10050:&#x2742;
HEAVY TEARDROP-SPOKED PINWHEEL ASTERISK:'63:10051:&#x2743;
SNOWFLAKE:'64:10052:&#x2744;
TIGHT TRIFOLIATE SNOWFLAKE:'65:10053:&#x2745;
HEAVY CHEVRON SNOWFLAKE:'66:10054:&#x2746;
SPARKLE:'67:10055:&#x2747;
HEAVY SPARKLE:'68:10056:&#x2748;
BALLOON-SPOKED ASTERISK:'69:10057:&#x2749;
TEARDROP-SPOKED ASTERISK:'6A:10043:&#x273B;
HEAVY TEARDROP-SPOKED ASTERISK:'6B:10045:&#x273D;
BLACK CIRCLE:'6C:9679:&#x25CF;
SHADOWED WHITE CIRCLE:'6D:10061:&#x274D;
BLACK SQUARE:'6E:9632:&#x25A0;
LOWER RIGHT DROP-SHADOWED SQUARE:'6F:10063:&#x274F;
UPPER RIGHT DROP-SHADOWED WHITE SQUARE:'70:10064:&#x2750;
LOWER RIGHT SHADOWED SQUARE:'71:10065:&#x2751;
UPPER RIGHT SHADOWED WHITE SQUARE:'72:10066:&#x2752;
BLACK UP-POINTING TRIANGLE:'73:9660:&#x25B2;
BLACK DOWN-POINTING TRIANGLE:'74:9651:&#x25BC;
BLACK DIAMOND:'75:9670:&#x25C6;
BLACK DIAMOND MINUS WHITE X:'76:10070:&#x2756;
RIGHT HALF BLACK CIRCLE:'77:9479:&#x2507;
LIGHT VERTICAL BAR:'78:10072:&#x2758;
MEDIUM VERTICAL BAR:'79:10073:&#x2759;
HEAVY VERTICAL BAR:'7A:10074:&#x275A;
HEAVY SINGLE TURNED COMMA QUOTATION MARK ORNAMENT:'7B:10075:&#x275B;
HEAVY SINGLE COMMA QUOTATION MARK ORNAMENT:'7C:10076:&#x275C;
HEAVY DOUBLE TURNED COMMA QUOTATION MARK ORNAMENT:'7D:10077:&#x275D;
HEAVY DOUBLE COMMA QUOTATION MARK ORNAMENT:'7E:10078:&#x275E;
UNUSED:'7F:none:udef_symbol num="7F"/>
MEDIUM LEFT PARENTHESIS ORNAMENT:'80:10088:&#x2768;
MEDIUM RIGHT PARENTHESIS ORNAMENT:'81:10089:&#x2769;
MEDIUM FLATTENED LEFT PARENTHESIS ORNAMENT:'82:10090:&#x276A;
MEDIUM FLATTENED RIGHT PARENTHESIS ORNAMENT:'83:10091:&#x276B;
MEDIUM LEFT-POINTING ANGLE BRACKET ORNAMENT:'84:10092:&#x276C;
MEDIUM RIGHT-POINTING ANGLE BRACKET ORNAMENT:'85:10093:&#x276D;
HEAVY LEFT-POINTING ANGLE QUOTATION MARK ORNAMENT:'86:10094:&#x276E;
HEAVY RIGHT-POITING ANGLE QUOTATION MARK ORNAMENT:'87:10095:&#x276F;
HEAVY LEFT-POINTING ANGLE BRACKET ORNAMENT:'88:10096:&#x2770;
HEAVY RIGHT-POTING ANGLE BRACKET ORNAMENT:'89:10097:&#x2771;
LIGHT LEFT TORTOISE SHELL BRACKET ORNAMENT:'8A:10098:&#x2772;
LIGHT RIGHT TORTOISE SHELL BRACKET ORNAMENT:'8B:10099:&#x2773;
MEDIUM LEFT CURLY BRACKET ORNAMENT:'8C:10100:&#x2774;
MEDIUM RIGHT CURLY BRACKET ORNAMENT:'8D:10101:&#x2775;
UNUSED:'8E:none:<udef_symbol num="8E"/>
UNUSED:'8F:none:udef_symbol num="8F"/>
UNUSED:'90:none:udef_symbol num="90"/>
UNUSED:'91:none:udef_symbol num="91"/>
UNUSED:'92:none:udef_symbol num="92"/>
UNUSED:'93:none:udef_symbol num="93"/>
UNUSED:'94:none:udef_symbol num="94"/>
UNUSED:'95:none:udef_symbol num="95"/>
UNUSED:'96:none:udef_symbol num="96"/>
UNUSED:'97:none:udef_symbol num="97"/>
UNUSED:'98:none:udef_symbol num="98"/>
UNUSED:'99:none:udef_symbol num="99"/>
UNUSED:'9A:none:udef_symbol num="9A"/>
UNUSED:'9B:none:udef_symbol num="9B"/>
UNUSED:'9C:none:udef_symbol num="9C"/>
UNUSED:'9D:none:udef_symbol num="9D"/>
UNUSED:'9E:none:udef_symbol num="9E"/>
UNUSED:'9F:none:udef_symbol num="9F"/>
UNUSED:'A0:none:udef_symbol num="A0"/>
CURVED STEM PARAGRAPH SIGN ORNAMENT:'A1:10081:&#x2761;
HEAVY EXCLAMATION MARK ORNAMENT:'A2:10082:&#x2762;
HEAVY HEART EXCLAMATION MARK ORNAMENT:'A3:10083:&#x2763;
HEAVY BLACK HEART:'A4:10084:&#x2764;
ROTATED HEAVY BLACK HEART BULLET:'A5:10085:&#x2765;
FLORAL HEART:'A6:10086:&#x2766;
ROTATED FLORAL HEART BULLET:'A7:10087:&#x2767;
BLACK CLUB SUIT:'A8:9827:&#x2663;
BLACK DIAMOND SUIT:'A9:9830:&#x2666;
BLACK HEART SUIT:'AA:9829:&#x2665;
BLACK SPADE SUIT:'AB:9824:&#x2660;
DINGBAT CIRCLED SANS SERIF DIGIT ONE:'AC:10112:&#x2780;
DINGBAT CIRCLED SANS SERIF DIGIT TWO:'AD:10113:&#x2781;
DINGBAT CIRCLED SANS SERIF DIGIT THREE:'AE:10114:&#x2782;
DINGBAT CIRCLED SANS SERIF DIGIT FOUR:'AF:10115:&#x2783;
DINGBAT CIRCLED SANS SERIF DIGIT FIVE:'B0:10116:&#x2784;
DINGBAT CIRCLED SANS SERIF DIGIT SIX:'B1:10117:&#x2785;
DINGBAT CIRCLED SANS SERIF DIGIT SEVEN:'B2:10118:&#x2786;
DINGBAT CIRCLED SANS SERIF DIGIT EIGHT:'B3:10119:&#x2787;
DINGBAT CIRCLED SANS SERIF DIGIT NINE:'B4:10120:&#x2788;
DINGBAT CIRCLED SANS SERIF DIGIT TEN:'B5:10121:&#x2789;
DINGBAT NEGATIVE CIRCLED DIGIT ONE:'B6:10102:&#x2776;
DINGBAT NEGATIVE CIRCLED DIGIT TWO:'B7:10103:&#x2777;
DINGBAT NEGATIVE CIRCLED DIGIT THREE:'B8:10104:&#x2778;
DINGBAT NEGATIVE CIRCLED DIGIT FOUR:'B9:10105:&#x2779;
DINGBAT NEGATIVE CIRCLED DIGIT FIVE:'BA:10106:&#x277A;
DINGBAT NEGATIVE CIRCLED DIGIT SIX:'BB:10107:&#x277B;
DINGBAT NEGATIVE CIRCLED DIGIT SEVEN:'BC:10108:&#x277C;
DINGBAT NEGATIVE CIRCLED DIGIT EIGHT:'BD:10109:&#x277D;
DINGBAT NEGATIVE CIRCLED DIGIT:'BE:10110:&#x277E;
DINGBAT NEGATIVE CIRCLED DIGIT:'BF:10111:&#x277F;
DINGBAT CIRCLED SANS-SERIF DIGIT ONE:'C0:10112:&#x2780;
DINGBAT CIRCLED SANS-SERIF DIGIT TWO:'C1:10113:&#x2781;
DINGBAT CIRCLED SANS-SERIF DIGIT THREE:'C2:10114:&#x2782;
DINGBAT CIRCLED SANS-SERIF DIGIT FOUR:'C3:10115:&#x2783;
DINGBAT CIRCLED SANS-SERIF DIGIT FIVE:'C4:10116:&#x2784;
DINGBAT CIRCLED SANS-SERIF DIGIT SIX:'C5:10117:&#x2785;
DINGBAT CIRCLED SANS-SERIF DIGIT SEVEN:'C6:10118:&#x2786;
DINGBAT CIRCLED SANS-SERIF DIGIT EIGHT:'C7:10119:&#x2787;
DINGBAT CIRCLED SANS-SERIF DIGIT NINE:'C8:10120:&#x2788;
DINGBAT CIRCLED SANS-SERIF DIGIT TEN:'C9:10121:&#x2789;
DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT ONE:'CA:10122:&#x278A;
DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT TWO:'CB:10123:&#x278B;
DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT THREE:'CC:10124:&#x278C;
DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT FOUR:'CD:10125:&#x278D;
DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT FIVE:'CE:10126:&#x278E;
DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT SIX:'CF:10127:&#x278F;
DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT SEVEN:'D0:10128:&#x2790;
DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT EIGHT:'D1:10129:&#x2791;
DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT NINE:'D2:10130:&#x2792;
DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT TEN:'D3:10131:&#x2793;
HEAVY WIDE-HEADED RIGHTWARDS ARROW:'D4:10132:&#x2794;
RIGHTWARDS ARROW:'D5:8594:&#x2192;
LEFT RIGHT ARROW:'D6:8596:&#x2194;
UP DOWN ARROW:'D7:8597:&#x2195;
HEAVY SOUTH EAST ARROW:'D8:10136:&#x2798;
HEAVY RIGHTWARDS ARROW:'D9:10137:&#x2799;
HEAVY NORTHEAST ARROW:'DA:10138:&#x279A;
DRAFTING POINT RIGHTWARDS ARROW:'DB:10139:&#x279B;
HEAVY ROUND-TIPPED RIGHTWARDS ARROW:'DC:10140:&#x279C;
TRIANGLE-HEADED RIGHTWARDS ARROW:'DD:10141:&#x279D;
HEAVY TRIANGLE-HEADED RIGHTWARDS ARROW:'DE:10142:&#x279E;
DASHED TRIANGLE-HEADED RIGHTWARDS ARROW:'DF:10143:&#x279F;
HEAVY DASHED TRIANGLE-HEADED RIGHTWARS ARROW:'E0:10144:&#x27A0;
BLACK RIGHTWARDS ARROW:'E1:10145:&#x27A1;
THREE-D TOP-LIGHTED RIGHTWARDS ARROWHEAD:'E2:10146:&#x27A2;
THREE-D BOTTOM-LIGHTED RIGHTWARDS ARROWHEAD:'E3:10147:&#x27A3;
BLACK RIGHTWARDS ARROWHEAD:'E4:10148:&#x27A4;
HEAVY BLACK CURVED DOWNWARDS AND RIGHTWARDS ARROW:'E5:10149:&#x27A5;
HEAVY BLACK CURVED UPWARDS AND RIGHTWARDS ARROW:'E6:10150:&#x27A6;
SQUAT BLACK RIGHTWARDS ARROW:'E7:10151:&#x27A7;
HEAVY CONCAVE-POINTED BLACK RIGHTWARDS ARROW:'E8:10152:&#x27A8;
RIGHT-SHADED WHITE RIGHTWARDS ARROW:'E9:10153:&#x27A9;
LEFT-SHADED WHITE RIGHTWARDS ARROW:'EA:10154:&#x27AA;
BACK-TILTED SHADOWED WHITE RIGHTWARDS ARROW:'EB:10155:&#x27AB;
FRONT-TILTED SHADOWED WHITE RIGHWARDS ARROW:'EC:10156:&#x27AC;
HEAVY LOWER RIGHT-SHADOWED WHITE RIGHTWARDS ARROW:'ED:10157:&#x27AD;
HEAVY UPPER RIGHT-SHADOWED WHITE RIGHTWARDS ARROW:'EE:10157:&#x27AD;
NOTCHED LOWER RIGHT-SHADOWED WHITE RIGHTWARDS ARROW:'EF:10158:&#x27AE;
UNUSED:'F0:none:udef_symbol num="F0"/>
NOTCHED UPPER RIGHT-SHADOWED WHITE RIGHTWARDS ARROW:'F1:10161:&#x27B1;
CIRCLED HEAVY WHITE RIGHTWARDS ARROW:'F2:10162:&#x27B2;
WHITE-FEATHERED RIGHTWARDS ARROW:'F3:10163:&#x27B3;
BLACK-FEATHERED SOUTH EAST ARROW:'F4:10164:&#x27B4;
BLACK-FEATHERED RIGHTWARDS ARROW:'F5:10165:&#x27B5;
BLACK-FEATHERED NORTH EAST ARROW:'F6:10166:&#x27B6;
HEAVY BLACK-FEATHERED SOUTH EAST ARROW:'F7:10167:&#x27B7;
HEAVY BLACK-FEATHERED RIGHTWARDS ARROW:'F8:10168:&#x27B8;
HEAVY BLACK-FEATHERED NORTH EAST ARROW:'F9:10169:&#x27B9;
TEARDROP-BARBED RITGHTWARDS ARROW:'FA:10170:&#x27BA;
HEAVY TEARDROP-SHANKED RIGHTWARDS ARROW:'FB:10171:&#x27BB;
WEDGE-TAILED RIGHTWARDS ARROW:'FC:10172:&#x27BC;
HEAVY WEDGED-TAILED RIGHTWARDS ARROW:'FD:10173:&#x27BD;
OPEN-OUTLINED RIGHTWARDS ARROW:'FE:10174:&#x27BE;
UNUSED:'FF:none:udef_symbol num="FF"/>
</dingbats>
<caps_uni>
LATIN CAPITAL LETTER S WITH CARON:&#x0161;:352:&#x0160;
LATIN CAPITAL LETTER S WITH ACUTE:&#x015B;:346:&#x015A;
LATIN CAPITAL LETTER T WITH CARON:&#x0165;:356:&#x0164;
LATIN CAPITAL LETTER Z WITH CARON:&#x017E;:381:&#x017D;
LATIN CAPITAL LETTER Z WITH ACUTE:&#x017A;:377:&#x0179;
LATIN CAPITAL LETTER L WITH STROKE:&#x0142;:321:&#x0141;
LATIN CAPITAL LETTER A WITH OGONEK:&#x0105;:260:&#x0104;
LATIN CAPITAL LETTER S WITH CEDILLA:&#x015F;:350:&#x015E;
LATIN CAPITAL LETTER Z WITH DOT ABOVE:&#x017C;:379:&#x017B;
LATIN CAPITAL LETTER L WITH CARON:&#x013E;:317:&#x013D;
LATIN CAPITAL LETTER R WITH ACUTE:&#x0155;:340:&#x0154;
LATIN CAPITAL LETTER A WITH ACUTE:&#x00E1;:193:&#x00C1;
LATIN CAPITAL LETTER A WITH CIRCUMFLEX:&#x00E2;:194:&#x00C2;
LATIN CAPITAL LETTER A WITH BREVE:&#x0103;:258:&#x0102;
LATIN CAPITAL LETTER A WITH DIAERESIS:&#x00E4;:196:&#x00C4;
LATIN CAPITAL LETTER L WITH ACUTE:&#x013A;:313:&#x0139;
LATIN CAPITAL LETTER C WITH ACUTE:&#x0107;:262:&#x0106;
LATIN CAPITAL LETTER C WITH CEDILLA:&#x00E7;:199:&#x00C7;
LATIN CAPITAL LETTER C WITH CARON:&#x010D;:268:&#x010C;
LATIN CAPITAL LETTER E WITH ACUTE:&#x00E9;:201:&#x00C9;
LATIN CAPITAL LETTER E WITH OGONEK:&#x0119;:280:&#x0118;
LATIN CAPITAL LETTER E WITH DIAERESIS:&#x00EB;:203:&#x00CB;
LATIN CAPITAL LETTER E WITH CARON:&#x011B;:282:&#x011A;
LATIN CAPITAL LETTER I WITH ACUTE:&#x00ED;:205:&#x00CD;
LATIN CAPITAL LETTER I WITH CIRCUMFLEX:&#x00EE;:206:&#x00CE;
LATIN CAPITAL LETTER D WITH CARON:&#x010F;:270:&#x010E;
LATIN CAPITAL LETTER D WITH STROKE:&#x0111;:272:&#x0110;
LATIN CAPITAL LETTER N WITH ACUTE:&#x0144;:323:&#x0143;
LATIN CAPITAL LETTER N WITH CARON:&#x0148;:327:&#x0147;
LATIN CAPITAL LETTER O WITH ACUTE:&#x00F3;:211:&#x00D3;
LATIN CAPITAL LETTER O WITH CIRCUMFLEX:&#x00F4;:212:&#x00D4;
LATIN CAPITAL LETTER O WITH DOUBLE ACUTE:&#x0151;:336:&#x0150;
LATIN CAPITAL LETTER O WITH DIAERESIS:&#x00F6;:214:&#x00D6;
LATIN CAPITAL LETTER R WITH CARON:&#x0159;:344:&#x0158;
LATIN CAPITAL LETTER U WITH RING ABOVE:&#x016F;:366:&#x016E;
LATIN CAPITAL LETTER U WITH ACUTE:&#x00FA;:218:&#x00DA;
LATIN CAPITAL LETTER U WITH DOUBLE ACUTE:&#x0171;:368:&#x0170;
LATIN CAPITAL LETTER U WITH DIAERESIS:&#x00FC;:220:&#x00DC;
LATIN CAPITAL LETTER Y WITH ACUTE:&#x00FD;:221:&#x00DD;
LATIN CAPITAL LETTER T WITH CEDILLA:&#x0163;:354:&#x0162;
CYRILLIC CAPITAL LETTER DJE (SERBOCROATIAN):&#x0452;:1026:&#x0402;
CYRILLIC CAPITAL LETTER GJE:&#x0453;:1027:&#x0403;
CYRILLIC CAPITAL LETTER LJE:&#x0459;:1033:&#x0409;
CYRILLIC CAPITAL LETTER NJE:&#x045A;:1034:&#x040A;
CYRILLIC CAPITAL LETTER KJE:&#x045C;:1036:&#x040C;
CYRILLIC CAPITAL LETTER TSHE (SERBOCROATIAN):&#x045B;:1035:&#x040B;
CYRILLIC CAPITAL LETTER DZHE:&#x045F;:1039:&#x040F;
CYRILLIC CAPITAL LETTER SHORT U (BYELORUSSIAN):&#x045E;:1038:&#x040E;
CYRILLIC CAPITAL LETTER JE:&#x0458;:1032:&#x0408;
CYRILLIC CAPITAL LETTER GHE WITH UPTURN:&#x0491;:1168:&#x0490;
CYRILLIC CAPITAL LETTER IO:&#x0451;:1025:&#x0401;
CYRILLIC CAPITAL LETTER UKRAINIAN IE:&#x0454;:1028:&#x0404;
CYRILLIC CAPITAL LETTER YI (UKRAINIAN):&#x0457;:1031:&#x0407;
CYRILLIC CAPITAL LETTER BYELORUSSIAN-UKRAINIAN I:&#x0456;:1030:&#x0406;
CYRILLIC CAPITAL LETTER DZE:&#x0455;:1029:&#x0405;
CYRILLIC CAPITAL LETTER A:&#x0430;:1040:&#x0410;
CYRILLIC CAPITAL LETTER BE:&#x0431;:1041:&#x0411;
CYRILLIC CAPITAL LETTER VE:&#x0432;:1042:&#x0412;
CYRILLIC CAPITAL LETTER GHE:&#x0433;:1043:&#x0413;
CYRILLIC CAPITAL LETTER DE:&#x0434;:1044:&#x0414;
CYRILLIC CAPITAL LETTER IE:&#x0435;:1045:&#x0415;
CYRILLIC CAPITAL LETTER ZHE:&#x0436;:1046:&#x0416;
CYRILLIC CAPITAL LETTER ZE:&#x0437;:1047:&#x0417;
CYRILLIC CAPITAL LETTER I:&#x0438;:1048:&#x0418;
CYRILLIC CAPITAL LETTER SHORT I:&#x0439;:1049:&#x0419;
CYRILLIC CAPITAL LETTER KA:&#x043A;:1050:&#x041A;
CYRILLIC CAPITAL LETTER EL:&#x043B;:1051:&#x041B;
CYRILLIC CAPITAL LETTER EM:&#x043C;:1052:&#x041C;
CYRILLIC CAPITAL LETTER EN:&#x043D;:1053:&#x041D;
CYRILLIC CAPITAL LETTER O:&#x043E;:1054:&#x041E;
CYRILLIC CAPITAL LETTER PE:&#x043F;:1055:&#x041F;
CYRILLIC CAPITAL LETTER ER:&#x0440;:1056:&#x0420;
CYRILLIC CAPITAL LETTER ES:&#x0441;:1057:&#x0421;
CYRILLIC CAPITAL LETTER TE:&#x0442;:1058:&#x0422;
CYRILLIC CAPITAL LETTER U:&#x0443;:1059:&#x0423;
CYRILLIC CAPITAL LETTER EF:&#x0444;:1060:&#x0424;
CYRILLIC CAPITAL LETTER HA:&#x0445;:1061:&#x0425;
CYRILLIC CAPITAL LETTER TSE:&#x0446;:1062:&#x0426;
CYRILLIC CAPITAL LETTER CHE:&#x0447;:1063:&#x0427;
CYRILLIC CAPITAL LETTER SHA:&#x0448;:1064:&#x0428;
CYRILLIC CAPITAL LETTER SHCHA:&#x0449;:1065:&#x0429;
CYRILLIC CAPITAL LETTER YERU:&#x044B;:1067:&#x042B;
CYRILLIC CAPITAL LETTER SOFT SIGN:&#x044C;:1068:&#x042C;
CYRILLIC CAPITAL LETTER E:&#x044D;:1069:&#x042D;
CYRILLIC CAPITAL LETTER YU:&#x044E;:1070:&#x042E;
CYRILLIC CAPITAL LETTER YA:&#x044F;:1071:&#x042F;
CYRILLIC CAPITAL LETTER HARD SIGN:&#x044A;:1066:&#x042A;
LATIN CAPITAL LIGATURE OE:&#x0153;:338:&#x0152;
LATIN CAPITAL LETTER Y WITH DIAERESIS:&#x00FF;:376:&#x0178;
LATIN CAPITAL LETTER A WITH GRAVE:&#x00E0;:192:&#x00C0;
LATIN CAPITAL LETTER A WITH TILDE:&#x00E3;:195:&#x00C3;
LATIN CAPITAL LETTER A WITH RING ABOVE:&#x00E5;:197:&#x00C5;
LATIN CAPITAL LETTER AE:&#x00E6;:198:&#x00C6;
LATIN CAPITAL LETTER E WITH GRAVE:&#x00E8;:200:&#x00C8;
LATIN CAPITAL LETTER E WITH CIRCUMFLEX:&#x00EA;:202:&#x00CA;
LATIN CAPITAL LETTER I WITH GRAVE:&#x00EC;:204:&#x00CC;
LATIN CAPITAL LETTER I WITH DIAERESIS:&#x00EF;:207:&#x00CF;
LATIN CAPITAL LETTER ETH (ICELANDIC):&#x00F0;:208:&#x00D0;
LATIN CAPITAL LETTER N WITH TILDE:&#x00F1;:209:&#x00D1;
LATIN CAPITAL LETTER O WITH GRAVE:&#x00F2;:210:&#x00D2;
LATIN CAPITAL LETTER O WITH TILDE:&#x00F5;:213:&#x00D5;
LATIN CAPITAL LETTER O WITH STROKE:&#x00F8;:216:&#x00D8;
LATIN CAPITAL LETTER U WITH GRAVE:&#x00F9;:217:&#x00D9;
LATIN CAPITAL LETTER U WITH CIRCUMFLEX:&#x00FB;:219:&#x00DB;
LATIN CAPITAL LETTER THORN (ICELANDIC):&#x00FE;:222:&#x00DE;
GREEK CAPITAL LETTER EPSILON WITH TONOS:&#x03AD;:904:&#x0388;
GREEK CAPITAL LETTER ETA WITH TONOS:&#x03AE;:905:&#x0389;
GREEK CAPITAL LETTER IOTA WITH TONOS:&#x03AF;:906:&#x038A;
GREEK CAPITAL LETTER OMICRON WITH TONOS:&#x03CC;:908:&#x038C;
GREEK CAPITAL LETTER UPSILON WITH TONOS:&#x03CD;:910:&#x038E;
GREEK CAPITAL LETTER OMEGA WITH TONOS:&#x03CE;:911:&#x038F;
GREEK CAPITAL LETTER IOTA WITH DIALYTIKA:&#x03AA;:938:&#x03AA;
GREEK CAPITAL LETTER ALPHA:&#x03B1;:913:&#x0391;
GREEK CAPITAL LETTER BETA:&#x03B2;:914:&#x0392;
GREEK CAPITAL LETTER GAMMA:&#x03B3;:915:&#x0393;
GREEK CAPITAL LETTER DELTA:&#x03B4;:916:&#x0394;
GREEK CAPITAL LETTER EPSILON:&#x03B5;:917:&#x0395;
GREEK CAPITAL LETTER ZETA:&#x03B6;:918:&#x0396;
GREEK CAPITAL LETTER ETA:&#x03B7;:919:&#x0397;
GREEK CAPITAL LETTER THETA:&#x03B8;:920:&#x0398;
GREEK CAPITAL LETTER IOTA:&#x03B9;:921:&#x0399;
GREEK CAPITAL LETTER KAPPA:&#x03BA;:922:&#x039A;
GREEK CAPITAL LETTER LAMDA:&#x03BB;:923:&#x039B;
GREEK CAPITAL LETTER MU:&#x03BC;:924:&#x039C;
GREEK CAPITAL LETTER NU:&#x03BD;:925:&#x039D;
GREEK CAPITAL LETTER XI:&#x03BE;:926:&#x039E;
GREEK CAPITAL LETTER OMICRON:&#x03BF;:927:&#x039F;
GREEK CAPITAL LETTER PI:&#x03C0;:928:&#x03A0;
GREEK CAPITAL LETTER RHO:&#x03C1;:929:&#x03A1;
GREEK CAPITAL LETTER SIGMA:&#x03C3;:931:&#x03A3;
GREEK CAPITAL LETTER TAU:&#x03C4;:932:&#x03A4;
GREEK CAPITAL LETTER UPSILON:&#x03C5;:933:&#x03A5;
GREEK CAPITAL LETTER PHI:&#x03C6;:934:&#x03A6;
GREEK CAPITAL LETTER CHI:&#x03C7;:935:&#x03A7;
GREEK CAPITAL LETTER OMEGA:&#x03C9;:937:&#x03A9;
GREEK CAPITAL LETTER ALPHA WITH TONOS:&#x03AC;:902:&#x0386;
GREEK CAPITAL LETTER UPSILON WITH DIALYTIKA:&#x03CB;:939:&#x03AB;
GREEK CAPITAL LETTER PSI:&#x03C8;:936:&#x03A8;
GREEK CAPITAL LETTER IOTA WITH DIALYTIKA:&#x03CA;:938:&#x03AA;
LATIN CAPITAL LETTER G WITH BREVE:&#x011F;:286:&#x011E;
LATIN CAPITAL LETTER E WITH DOT ABOVE:&#x0117;:278:&#x0116;
LATIN CAPITAL LETTER R WITH CEDILLA:&#x0157;:342:&#x0156;
LATIN CAPITAL LETTER I WITH OGONEK:&#x012B;:302:&#x012E;
LATIN CAPITAL LETTER A WITH MACRON:&#x0101;:256:&#x0100;
LATIN CAPITAL LETTER E WITH MACRON:&#x0113;:274:&#x0112;
LATIN CAPITAL LETTER G WITH CEDILLA:&#x0123;:290:&#x0122;
LATIN CAPITAL LETTER K WITH CEDILLA:&#x0137;:310:&#x0136;
LATIN CAPITAL LETTER I WITH MACRON:&#x012B;:298:&#x012A;
LATIN CAPITAL LETTER L WITH CEDILLA:&#x013C;:315:&#x013B;
LATIN CAPITAL LETTER N WITH CEDILLA:&#x0146;:325:&#x0145;
LATIN CAPITAL LETTER O WITH MACRON:&#x014D;:332:&#x014C;
LATIN CAPITAL LETTER U WITH OGONEK:&#x0173;:370:&#x0172;
LATIN CAPITAL LETTER U WITH MACRON:&#x016B;:362:&#x016A;
</caps_uni>
#unused character maps
<unused>
<caps_to_lower>
LATIN CAPITAL LETTER A:'41:65:'61
LATIN CAPITAL LETTER B:'42:66:'62
LATIN CAPITAL LETTER C:'43:67:'63
LATIN CAPITAL LETTER D:'44:68:'64
LATIN CAPITAL LETTER E:'45:69:'65
LATIN CAPITAL LETTER F:'46:70:'66
LATIN CAPITAL LETTER G:'47:71:'67
LATIN CAPITAL LETTER H:'48:72:'68
LATIN CAPITAL LETTER I:'49:73:'69
LATIN CAPITAL LETTER J:'4A:74:'6a
LATIN CAPITAL LETTER K:'4B:75:'6b
LATIN CAPITAL LETTER L:'4C:76:'6c
LATIN CAPITAL LETTER M:'4D:77:'6d
LATIN CAPITAL LETTER N:'4E:78:'6e
LATIN CAPITAL LETTER O:'4F:79:'6f
LATIN CAPITAL LETTER P:'50:80:'70
LATIN CAPITAL LETTER Q:'51:81:'71
LATIN CAPITAL LETTER R:'52:82:'72
LATIN CAPITAL LETTER S:'53:83:'73
LATIN CAPITAL LETTER T:'54:84:'74
LATIN CAPITAL LETTER U:'55:85:'75
LATIN CAPITAL LETTER V:'56:86:'76
LATIN CAPITAL LETTER W:'57:87:'77
LATIN CAPITAL LETTER X:'58:88:'78
LATIN CAPITAL LETTER Y:'59:89:'79
LATIN CAPITAL LETTER Z:'5A:90:'7a
LATIN CAPITAL LETTER A:A:65:a
LATIN CAPITAL LETTER B:B:66:b
LATIN CAPITAL LETTER C:C:67:c
LATIN CAPITAL LETTER D:D:68:d
LATIN CAPITAL LETTER E:E:69:e
LATIN CAPITAL LETTER F:F:70:f
LATIN CAPITAL LETTER G:G:71:g
LATIN CAPITAL LETTER H:H:72:h
LATIN CAPITAL LETTER I:I:73:i
LATIN CAPITAL LETTER J:J:74:j
LATIN CAPITAL LETTER K:K:75:K
LATIN CAPITAL LETTER L:L:76:l
LATIN CAPITAL LETTER M:M:77:m
LATIN CAPITAL LETTER N:N:78:n
LATIN CAPITAL LETTER O:O:79:o
LATIN CAPITAL LETTER P:P:80:p
LATIN CAPITAL LETTER Q:Q:81:q
LATIN CAPITAL LETTER R:R:82:r
LATIN CAPITAL LETTER S:S:83:s
LATIN CAPITAL LETTER T:T:84:t
LATIN CAPITAL LETTER U:U:85:u
LATIN CAPITAL LETTER V:V:86:v
LATIN CAPITAL LETTER W:W:87:x
LATIN CAPITAL LETTER X:X:88:x
LATIN CAPITAL LETTER Y:Y:89:y
LATIN CAPITAL LETTER Z:Z:90:z
NO UNICODE VALUE:'E7:231:\'87
NO UNICODE VALUE:'83:131:\'8E
NO UNICODE VALUE:'92:146:\'EA
NO UNICODE VALUE:'EE:238:\'97
NO UNICODE VALUE:'F2:242:\'9C
NO UNICODE VALUE:'CB:203:\'88
NO UNICODE VALUE:'E9:233:\'8F
NO UNICODE VALUE:'ED:237:\'93
NO UNICODE VALUE:'F1:241:\'98
NO UNICODE VALUE:'F4:244:\'9D
NO UNICODE VALUE:'E5:229:\'89
NO UNICODE VALUE:'E6:230:\'90
NO UNICODE VALUE:'EB:235:\'94
NO UNICODE VALUE:'EF:239:\'99
NO UNICODE VALUE:'F3:243:\'9E
NO UNICODE VALUE:'AF:175:\'BF
NO UNICODE VALUE:'84:132:\'96
NO UNICODE VALUE:'CD:205:\'9B
NO UNICODE VALUE:'CC:204:\'8B
NO UNICODE VALUE:'80:128:\'8A
NO UNICODE VALUE:'E8:232:\'91
NO UNICODE VALUE:'EC:236:\'95
NO UNICODE VALUE:'85:133:\'9A
NO UNICODE VALUE:'86:134:\'9F
NO UNICODE VALUE:'82:130:\'8D
NO UNICODE VALUE:'81:129:\'8C
NO UNICODE VALUE:'E1:129:\'C1
</caps_to_lower>
<wingdings_old>
CANCER:a:9803:&#x264B;
LEO:b:9804:&#x264C;
VIRGO:c:9805:&#x264D;
LIBRA:d:9806:&#x264E;
SCORPIOUS:e:9807:&#x264F;
SAGITARRIUS:f:9808:&#x2650;
CAPRICON:g:9809:&#x2651;
AQUARIUS:h:9810:&#x2652;
PISCES:i:9811:&#x2653;
MY LOOPY ET:j:0:<fancy_et/>
AMPERSAND:k:38:&#x0026;
BLACK CIRCLE:l:9679:&#x25CF;
SHADOWED WHITE CIRCLE:m:10061:&#x274D;
BLACK SQUARE:n:9632:&#x25A0;
WHITE SQUARE:o:9633:&#x25A1;
WHITE SQUARE:p:9633:&#x25A1;
LOWER RIGHT SHADOWED SQUARE:q:10065:&#x2751;
UPPER RIGHT SHADOWED WHITE SQUARE:r:10066:&#x2752;
BLACK DIAMOND:s:9670:&#x25C6;
BLACK DIAMOND:t:9670:&#x25C6;
BLACK DIAMOND:u:9670:&#x25C6;
BLACK DIAMOND MINUS WHITE X:v:10070:&#x2756;
BLACK DIAMOND MINUS WHITE X:w:10070:&#x2756;
BALLOT BOX WITH X:x:9746:&#x2612;
MY COMPUTER KEY:y:0:<on_key/>
MY APPLE KEY:z:0:<apple_key/>
VICTORY HAND:A:9996:&#x270C;
OKAY HAND:B:0:<okay_hand/>
MY THUMBS UP:C:0:<thumbs_up/>
MY THUMBS DOWN:D:0:<thumbs_down/>
WHITE LEFT POINTING INDEX:E:9756:&#x261C;
WHITE RIGHT POINTING INDEX:F:9758:&#x261E;
WHITE POINTING UP INDEX:G:9757:&#x261D;
WHITE POINTING DOWN INDEX:H:9759:&#x261F;
MY OPEN HAND:I:0:<open_hand/>
WHITE SMILING FACE:J:9786:&#x263A;
MY STRAIGHT FACE:K:0:<straight_face/>
WHITE FROWNING FACE:L:9785:&#x2639;
MY BOMB:M:0:<bomb/>
SKULL AND CROSSBONES:N:9760:&#x2620;
MY FLAG:O:0:<flag/>
MY PENNANT:P:0:<pennant/>
AIRPLANE:Q:9992:&#x2708;
CIRCLED OPEN CENTRE EIGHT POINTED STAR:R:9794:&#x2642;
MY TEARDROP:S:0:<teardrop/>
SNOWFLAKE:T:10052:&#x2744;
SHADOWED WHITE LATIN CROSS:U:10014:&#x271E;
SHADOWED WHITE LATIN CROSS:V:10014:&#x271E;
MY CELTIC CROSS:W:0:<celtic_cross/>
MALTESE CROSS:X:10016:&#x2720;
STAR OF DAVID:Y:10017:&#x2721;
STAR AND CRESCENT:Z:9770:&#x262A;
MY FOLDER:0:0:<folder/>
MY OPEN FOLDER:1:0:<open_folder/>
MY DOG-EARED DOCUMENT:2:0:<dog_eared_doc/>
MY DOCUMENT:3:0:<document/>
MY PAGES:4:0:<pages/>
MY FILE CABINETS:5:0:<file_cabinets/>
MY HOUR GLASS:6:0:<hour_glass/>
MY KEYBOARD:7:0:<keyboard/>
MY MOUSE:8:0:<mouse/>
MY BOTTOM OF MOUSE:9:0:<bottom_mouse/>
LOWER RIGHT PENCIL:!:9998:&#x270E;
WRITING HAND:@:9996:&#x270C;
UPPER BLADE SCISSORS:#:9985:&#x2701;
MY GLASSES:$:0:<glasses/>
MY BELL:%:0:<bell/>
ARIES:^:9800:&#x2648;
MY BOOK:&:0:<book/>
ENVELOPE:*:9993:&#x2709;
BLACK TELEPHONE:(:9742:&#x260E;
TELEPHONE LOCATION SIGN:):9990:&#x2706;
MY MAILBOX:-:0:<mailbox/>
TAURUS:_:9801:&#x2649;
MY BLACK FLOPPY DISK:=:0:<black_floppy_disk/>
ENVELOPE:+:9993:&#x2709;
HEAVY DOUBLE COMMA QUOTATION MARK ORNAMENT:~:10078:&#x275E;
GEMINI:`:9802:&#x264A;
MY WHITE FLOPPY DISK:<:0:<white_floppy_disk/>
MY TAPE REEL:>:0:<tape_reel/>
MY OPEN MAILBOX:.:0:<open_mailbox/>
WRITING HAND:?:9996:&#x270C;
EIGHT PETALLED OUTLINED BLACK FLORETTE:|:10049:&#x2741;
MY OPEN MAILBOX:/:0:<open_mailbox/>
MY COMPUETR:\\colon:0:<computer/>
MY DOWNWARD LEAF:" :0:<downward_leaf/>
MY UPWARD LEAF:"  :0:<upward_leaf/>
</wingdings_old>
<SYMBOL_decimal>
EXCLAMATION MARK:33:33:!
FOR ALL:34:8704:&#x2200;
NUMBER SIGN:35:35:#
THERE EXISTS:36:8707:&#x2203;
PERCENTAGE SIGN:37:37:%
AMPERSAND:38:38:&#x0026;
CONTAINS AS A MEMBER:39:8715:&#x220B;
LEFT PARENTHESIS:40:40:(
RIGHT PERENTHESIS:41:41:)
ASTERISK OPERATOR:42:8727:&#x2217;
PLUS SIGN:43:43:+
COMMA:44:44:,
MINUS SIGN:45:8722:&#x2212;
FULL STOP:46:46:.
DIVISION SLASH:47:8725:&#x2215;
DIGIT ZERO:48:48:0
DIGIT ONE:49:49:1
DIGIT TWO:50:50:2
DIGIT THREE:51:51:3
DIGIT FOUR:52:52:4
DIGIT FIVE:53:53:5
DIGIT SIX:54:54:6
DIGIT SEVEN:55:55:7
DIGIT EIGHT:56:56:8
DIGIT NINE:57:57:9
RATIO:58:8758:&#x2236;
SEMICOLON:59:59:;
LESS-THAN SIGN:60:60:&lt;
EQUALS SIGN TO:61:61:=
GREATER-THAN SIGN:62:62:&gt;
QUESTION MARK:63:63:?
APPROXTIMATELY EQUAL TO:64:8773:&#x2245;
GREEK CAPITOL LETTER ALPHA:65:913:&#x0391;
GREEK CAPAITOL LETTER BETA:66:914:&#x0392;
GREEK CAPITOL LETTER CHI:67:935:&#x03A7;
GREEK CAPITOL LETTER DELTA:68:916:&#x0394;
GREEK CAPITOL LETTER EPSILON:69:917:&#x0395;
GREEK CAPITOL LETTER PHI:70:934:&#x03A6;
GREEK CAPITOL LETTER GAMMA:71:915:&#x0393;
GREEK CAPITOL LETTER ETA:72:919:&#x0397;
GREEK CAPITOL LETTER ITOA:73:913:&#x0391;
GREEK THETA SYMBOL:74:977:&#x03D1;
GREEK CAPITOL LETTER KAPPA:75:922:&#x039A;
GREEK CAPITOL LETTER LAMBDA:76:923:&#x039B;
GREEK CAPITOL LETTER MU:77:924:&#x039C;
GREEK CAPITOL LETTER NU:78:925:&#x039D;
GREEK CAPITOL LETTER OMICRON:79:927:&#x039F;
GREEK CAPITAL LETTER PI:80:928:&#x03A0;
GREEK CAPITOL LETTER THETA:81:920:&#x0398;
GREEK CAPITOL LETTER RHO:82:929:&#x03A1;
GREEK CAPITOL LETTER SIGMA:83:931:&#x03A3;
GREEK CAPITOL LETTER TAU:84:932:&#x03A4;
GREEK CAPITOL LETTER UPSILON:85:933:&#x03A5;
GREEK LETTER STIGMA:86:986:&#x03DA;
GREEK CAPITOL LETTER OMEGA:87:937:&#x03A9;
GREEK CAPITOL LETTER XI:88:926:&#x039E;
GREEK CAPITOL LETTER PSI:89:936:&#x03A8;
GREEK CAPITOL LETTER ZETA:90:918:&#x0396;
LEFT SQUARE BRACKET:91:91:&#x005B;
THEREFORE:92:8756:&#x2234;
RIGHT SQUARE BRACKET:93:93:&#x005D;
UP TACK:94:8869:&#x22A5;
MODIFIER LETTER LOW MACRON:95:717:&#x02CD;
MODIFIER LETTER MACRON:96:713:&#x02C9;
GREEK SMALL LETTER ALPHA:97:945:&#x03B1;
GREEK SMALL LETTER BETA:98:946:&#x03B2;
GREEK SMALL LETTER CHI:99:967:&#x03C7;
GREEK SMALL LETTER DELTA:100:948:&#x03B4;
GREEK SMALL LETTER EPSILON:101:949:&#x03B5;
GREEK PHI SYMBOL:102:981:&#x03D5;
GREEK MSALL LETTER DELTA:103:947:&#x03B3;
GREEK SMALL LETTER ETA:104:951:&#x03B7;
GREEK SMALL LETTER IOTA:105:953:&#x03B9;
GREEK SMALL LETTER PHI:106:966:&#x03C6;
GREEK SMALL LETTER KAPPA:107:954:&#x03BA;
GREEK SMALL LETTER LAMDA:108:955:&#x03BB;
GREEK SMALL LETTER MU:109:956:&#x03BC;
GREEK SMALL LETTER NU:110:957:&#x03BD;
GREEK SMALL LETTER OMICRON:111:959:&#x03BF;
GREEK SMALL LETTER PI:112:960:&#x03C0;
GREEK SMALL LETTER THETA:113:952:&#x03B8;
GREEK SMALL LETTER RHO:114:961:&#x03C1;
GREEK SMALL LETTER SIGMA:115:963:&#x03C3;
GREEK SMALL LETTER TAU:116:964:&#x03C4;
GREEK SMALL LETTER UPSILON:117:965:&#x03C5;
GREEK PI SYMBOL:118:982:&#x03D6;
GREEK SMALL LETTER OMEGA:119:969:&#x03C9;
GREEK SMALL LETTER XI:120:958:&#x03BE;
GREEK SMALL LETTER PHI:121:966:&#x03C6;
GREEK SMALL LETTER ZETA:122:950:&#x03B6;
LEFT CURLY BRACKET:123:123:{
DIVIDES:124:8739:&#x2223;
RIGHT CURLY BRACKET:125:125:}
TILDE OPERATOR:126:8764:&#x223C;
GREEK UPSILON WITH HOOK SYMBOL:161:978:&#x03D2;
COMBINING ACUTE TONE MARK:162:833:&#x0341;
LESS THAN OR EQUAL TO:163:8804:&#x2264;
DIVISION SLASH:164:8725:&#x2215;
INFINITY:165:8734:&#x221E;
SMALL LETTER F:166:15:f
BLACK CLUB SUIT:167:9827:&#x2663;
BLACK DIAMOND SUIT:168:9830:&#x2666;
BLACK HEART SUIT:169:9829:&#x2665;
BLACK SPADE SUIT:170:9824:&#x2660;
LEFT RIGHT ARROW:171:8596:&#x2194;
LEFTWARDS ARROW:172:8592:&#x2190;
UPWARDS ARROW:173:8593:&#x2191;
RIGHTWARDS ARROW:174:8594:&#x2192;
DOWNWARDS ARROW:175:8595:&#x2193;
DEGREE SIGN:176:176:&#x00B0;
PLUS OR MINUS SIGN:177:177:&#x00B1;
DOUBLE ACUTE ACCENT:178:733:&#x02DD;
GREATER THAN OR EQUAL TO:179:8805:&#x2265;
MULTIPLICATION SIGN:180:215:&#x00D7;
DON'T KNOW:181:8733:&#x221D;
PARTIAL DIFFERENTIAL:182:8706:&#x2202;
BULLET:183:183:&#x00B7;
DIVISION:184:247:&#x00F7;
NOT EQUAL TO:185:8800:&#x2260;
IDENTICAL TO:186:8801:&#x2261;
ALMOST EQUAL TO:187:8776:&#x2248;
MIDLINE HORIZONTAL ELLIPSES:188:8943:&#x22EF;
DIVIDES:189:8739:&#x2223;
BOX DRAWINGS LIGHT HORIZONTAL:190:9472:&#x2500;
DOWNWARDS ARROW WITH TIP LEFTWARDS:191:8626:&#x21B2;
CIRCLED TIMES:196:8855:&#x2297;
CIRCLED PLUS:197:8853:&#x2295;
EMPTY SET:198:8709:&#x2205;
INTERSECTION:199:8745:&#x2229;
UNION:200:8746:&#x222A;
SUPERSET OF:201:8835:&#x2283;
SUPERSET OF OR EQUAL TO:202:8839:&#x2287;
NIETHER A SUBSET OR EQUAL TO:203:8836:&#x2284;
SUBSET OF:204:8834:&#x2282;
SUBSET OR EQUAL TO:205:8838:&#x2286;
ELEMENT OF:206:8712:&#x2208;
NOT AN ELEMENT OF:207:8713:&#x2209;
ANGLE:208:8736:&#x2220;
WHITE DOWN POINTING TRIANBLE:209:9661:&#x25BD;
REGISTERED SIGN:210:174:&#x00AE;
COPYRIGHT:211:169:&#x00A9;
TRADEMARK SYMBOL:212:8482:&#x2122;
NARY OPERATOR:213:8719:&#x220F;
SQUARE ROOT:214:8730:&#x221A;
BULLET OPERATOR:215:8729:&#x2219;
NOT SIGN:216:172:&#x00AC;
LOGICAL AND:217:8743:&#x2227;
LOGICAL OR:218:8744:&#x2228;
LEFT RIGHT DOUBLE ARROW:219:8660:&#x21D4;
LEFTWARDS DOUBLE ARROW:220:8656:&#x21D0;
UPWARDS DOUBLE ARROW:221:8657:&#x21D1;
RIGHTWARDS DOUBLE ARROW:222:8658:&#x21D2;
DOWNWARDS DOUBLE ARROW:223:8659:&#x21D3;
BETWEEN:224:8812:&#x226C;
MATHEMATICAL LEFT ANGELBRACKET:225:10216:&#x27E8;
REGISTERED SIGN:226:174:&#x00AE;
COPYRIGHT:227:169:&#x00A9;
TRADEMARK SYMBOL:228:8482:&#x2122;
N-ARY SUMMATION:229:8721:&#x2211;
LARGE LEFT PARENTHESIS PART1:230:0:<udef_symbol num="0x230" description="left_paraenthesis part 1"/>
LARGE LEFT PARENTHESIS PART2:231:0:<udef_symbol num="0x231" description="left_parenthesis part 2"/>
LARGE LEFT PARENTHESIS PART3:232:0:<udef_symbol num="0x232" description="left_paranethesis part 3"/>
LARGE LEFT SQUARE BRACKET PART1:233:0:<udef_symbol num="0x233" description="left_square_bracket part 1"/>
LARGE LEFT SQUARE BRACKET PART2:234:0:<udef_symbol num="0x234" description="left_square_bracket part 2"/>
LARGE LEFT SQUARE BRACKET PART3:235:0:<udef_symbol num="0x235" description="left_square_bracket part 3"/>
LARGE LEFT BRACKET PART1:236:0:<udef_symbol num="0x236" description="right_bracket part 1"/>
LARGE LEFT BRACKET PART2:237:0:<udef_symbol num="0x237" description="right_bracket part 2"/>
LARGE LEFT BRACKET PART3:238:0:<udef_symbol num="0x238" description="right_bracket part 3"/>
DIVIDES:239:8739:&#x2223;
MATHEMATICAL RIGHT ANGLE BRACKET:241:10217:&#x27E9;
INTEGRAL:242:8747:&#x222B;
LARGE INTEGRAL PART 1:243:0:<udef_symbol num="0x243" description="integral part 1"/>
LARGE INTEGRAL PART 2:244:0:<udef_symbol num="0x244" description="integral part 2"/>
LARGE INTEGRAL PART 3:245:0:<udef_symbol num="0x245" description="integral part 3"/>
LARGE RIGHT PARENTHESIS PART1:246:0:<udef_symbol num="0x246" description="right_parenthesis part 1"/>
LARGE RIGHT PARENTHESIS PART2:247:0:<udef_symbol num="0x247" description="right_parenthesis part 2"/>
LARGE RIGHT PARENTHESIS PART3:248:0:<udef_symbol num="0x248" description="right_parenthesis part 3"/>
LARGE RIGHT SQUARE BRACKET PART1:249:0:<udef_symbol num="0x249" description="right_square_bracket part 1"/>
LARGE RIGHT SQUARE BRACKET PART2:250:0:<udef_symbol num="0x250" description="right_square_bracket part 2"/>
LARGE RIGHT SQUARE BRACKETPART3:251:0:<udef_symbol num="0x251" description="right_square_bracket part 3"/>
LARGE RIGHT BRACKET PART1:252:0:<udef_symbol num="0x252" description="right_bracket part 1"/>
LARGE RIGHT BRACKETPART2:253:0:<udef_symbol num="0x253" description="right_bracket part 2"/>
LARGE RIGHT BRACKETPART3:254:0:<udef_symbol num="0x254" description="right_bracket part 3"/>
DOUBLE ACUTE ACCENT:178:733:&#x02DD;
</SYMBOL_decimal>
<SYMBOL_old>
EXCLMATION POINT:33:unknown:!
FOR ALL:34:8704:&#x2200;
POUND SIGN:35:unknown:#
THERE EXISTS:36:8707:&#x2203;
PERCENTAGE SIGN:37:unknown:%
AMPERSAND:38:38:&#x0026;
CONTAINS AS A MEMBER:39:unknown:&#x220B;
LEFT PARENTHESIS:40:unknown:(
RIGHT PERENTHESIS:41:unknown:)
ASTERISK OPERATOR:42:8727:&#x2217;
PLUS:43:unknown:+
COMMA:44:unknown:,
MINUS SIGN:45:8722:&#x2212;
PERIOD:46:unknown:.
DIVISION SLASH:47:8725:&#x2215;
ZERO:48:0:0
ONE:49:1:1
TWO:50:2:2
THREE:51:3:3
FOUR:52:4:4
FIVE:53:5:5
SIX:54:6:6
SEVEN:55:7:7
EIGHT:56:8:8
NINE:57:9:9
RATIO:58:8758:&#x2236;
SEMICOLON:59:unknown:;
LESS THAN:60:unknown:&lt;
EQAULS TO:61:unknown:=
GREATER THAN:62:unknown:&gt;
QUESTION MARK:63:unknown:?
APPROXTIMATELY EQUAL TO:64:8773:&#x2245;
GREEK CAPITOL LETTER ALPHA:65:913:&#x0391;
GREEK CAPAITOL LETTER BETA:66:914:&#x0392;
GREEK CAPITOL LETTER CHI:67:unknown:&#x03A7;
GREEK CAPITOL LETTER DELTA:68:916:&#x0394;
GREEK CAPITOL LETTER EPSILON:69:917:&#x0395;
GREEK CAPITOL LETTER PHI:70:unknown:&#x03A6;
GREEK CAPITOL LETTER GAMMA:71:915:&#x0393;
GREEK CAPITOL LETTER ETA:72:919:&#x0397;
GREEK CAPITOL LETTER ITOA:73:913:&#x0391;
GREEK THETA SYMBOL:74:unknown:&#x03D1;
GREEK CAPITOL LETTER KAPPA:75:unknown:&#x039A;
GREEK CAPITOL LETTER LAMBDA:76:unknown:&#x039B;
GREEK CAPITOL LETTER MU:77:unknown:&#x039C;
GREEK CAPITOL LETTER NU:78:unknown:&#x039D;
GREEK CAPITOL LETTER OMICRON:79:unknown:&#x039F;
GREEK CAPITAL LETTER PI:80:unknown:&#x03A0;
GREEK CAPITOL LETTER THETA:81:920:&#x0398;
GREEK CAPITOL LETTER RHO:82:unknown:&#x03A1;
GREEK CAPITOL LETTER SIGMA:83:unknown:&#x03A3;
GREEK CAPITOL LETTER TAU:84:unknown:&#x03A4;
GREEK CAPITOL LETTER UPSILON:85:unknown:&#x03A5;
GREEK LETTER STIGMA:86:unknown:&#x03DA;
GREEK CAPITOL LETTEROMEGA:87:unknown:&#x03A9;
GREEK CAPITOL LETTER XI:88:unknown:&#x039E;
GREEK CAPITOL LETTER PSI:89:unknown:&#x03A8;
GREEK CAPITOL LETTER ZETA:90:918:&#x0396;
LEFT BRACKET:91:unknown:[
THEREFORE:92:8756:&#x2234;
LEFT BRACKET:93:unknown:[
UP TACK:94:unknown:&#x22A5;
MODIFIER LETTER LOW MACRON:95:unknown:&#x02CD;
MODIFIER LETTER MACRON:96:unknown:&#x02C9;
GREEK SMALL LETTER ALPHA:97:unknown:&#x03B1;
GREEK SMALL LETTER BETA:98:unknown:&#x03B2;
GREEK SMALL LETTER CHI:99:unknown:&#x03C7;
GREEK SMALL LETTER DELTA:100:unknown:&#x03B4;
GREEK SMALL LETTER EPSILON:101:unknown:&#x03B5;
GREEK PHI SYMBOL:102:unknown:&#x03D5;
GREEK MSALL LETTER DELTA:103:unknown:&#x03B3;
GREEK SMALL LETTER ETA:104:unknown:&#x03B7;
GREEK SMALL LETTER IOTA:105:unknown:&#x03B9;
GREEK SMALL LETTER PHI:106:unknown:&#x03C6;
GREEK SMALL LETTER KAPPA:107:unknown:&#x03BA;
GREEK SMALL LETTER LAMDA:108:unknown:&#x03BB;
GREEK SMALL LETTER MU:109:unknown:&#x03BC;
GREEK SMALL LETTER NU:110:unknown:&#x03BD;
GREEK SMALL LETTER OMICRON:111:unknown:&#x03BF;
GREEK SMALL LETTER PI:112:unknown:&#x03C0;
GREEK SMALL LETTER THETA:113:unknown:&#x03B8;
GREEK SMALL LETTER RHO:114:unknown:&#x03C1;
GREEK SMALL LETTER SIGMA:115:unknown:&#x03C3;
GREEK SMALL LETTER TAU:116:unknown:&#x03C4;
GREEK SMALL LETTER UPSILON:117:unknown:&#x03C5;
GREEK PI SYMBOL:118:unknown:&#x03D6;
GREEK SMALL LETTER OMEGA:119:unknown:&#x03C9;
GREEK SMALL LETTER XI:120:unknown:&#x03BE;
GREEK SMALL LETTER PHI:121:unknown:&#x03C6;
GREEK SMALL LETTER ZETA:122:unknown:&#x03B6;
RIGHT BRACKET:123:unknown:{
DIVIDES:124:8739:&#x2223;
LEFT BRACKET:125:unknown:}
TILDE OPERATOR:126:unknown:&#x223C;
GREEK UPSILON WITH HOOK SYMBOL:161:unknown:&#x03D2;
COMBINING ACUTE TONE MARK:162:833:&#x0341;
LESS THAN OR EQUAL TO:163:8804:&#x2264;
DIVISION SLASH:164:8725:&#x2215;
INFINITY:165:unknown:&#x221E;
SMALL LETTER F:166:unknown:f
BLACK CLUB SUIT:167:9827:&#x2663;
BLACK DIAMOND SUIT:168:9830:&#x2666;
BLACK HEART SUIT:169:9829:&#x2665;
BLACK SPADE SUIT:170:9824:&#x2660;
LEFT RIGHT ARROW:171:8596:&#x2194;
LEFTWARDS ARROW:172:8592:&#x2190;
UPWARDS ARROW:173:8593:&#x2191;
RIGHTWARDS ARROW:174:8594:&#x2192;
DOWNWARDS ARROW:175:8595:&#x2193;
DEGREE SIGN:176:unknown:&#x00B0;
PLUS OR MINUS SIGN:177:unknown:&#x00B1;
DOUBLE ACUTE ACCENT:178:unknown:&#x02DD;
GREATER THAN OR EQUAL TO:179:8805:&#x2265;
MULTIPLICATION SIGN:180:unknown:&#x00D7;
DON' T KNOW:181:unknown:&#x221D;
PARTIAL DIFFERENTIAL:182:8706:&#x2202;
BULLET?:183:unknown:&#x00B7;
DIVISION:184:unknown:&#x00F7;
NOT EQUAL TO:185:8800:&#x2260;
IDENTICAL TO:186:8801:&#x2261;
ALMOST EQUAL TO:187:8776:&#x2248;
MIDLINE HORIZONTAL ELLIPSES:188:unknown:&#x22EF;
DIVIDES:189:8739:&#x2223;
BOX DRAWINGS LIGHT HORIZONTAL:190:9472:&#x2500;
DOWNWARDS ARROW WITH TIP LEFTWARDS:191:unknown:&#x21B2;
CIRCLED TIMES:196:8855:&#x2297;
CIRCLED PLUS:197:8853:&#x2295;
EMPTY SET:198:8709:&#x2205;
INTERSECTION:199:8745:&#x2229;
UNION:200:unknown:&#x222A;
SUPERSET OF:201:8835:&#x2283;
SUPERSET OF OR EQUAL TO:202:8839:&#x2287;
NIETHER A SUBSET OR EQUAL TO:203:8836:&#x2284;
SUBSET OF:204:8834:&#x2282;
SUBSET OR EQUAL TO:205:8838:&#x2286;
ELEMENT OF:206:8712:&#x2208;
NOT AN ELEMENT OF:207:8713:&#x2209;
ANGLE:208:8736:&#x2220;
WHITE DOWN POINTING TRIANBLE:209:unknown:&#x25BD;
REGISTERED SIGN:210:unknown:&#x00AE;
COPYRIGHT:211:unknown:&#x00A9;
NARY OPERATOR:213:unknown:&#x220F;
SQUARE ROOT:214:unknown:&#x221A;
BULLET OPERATOR:215:8729:&#x2219;
NOT SIGN:216:unknown:&#x00AC;
LOGICAL AND:217:8743:&#x2227;
LOGICAL OR:218:8744:&#x2228;
LEFT RIGHT DOUBLE ARROW:219:unknown:&#x21D4;
LEFTWARDS DOUBLE ARROW:220:unknown:&#x21D0;
UPWARDS DOUBLE ARROW:221:unknown:&#x21D1;
RIGHTWARDS DOUBLE ARROW:222:unknown:&#x21D2;
DOWNWARDS DOUBLE ARROW:223:unknown:&#x21D3;
BETWEEN:224:unknown:&#x226C;
MATHEMATICAL LEFT ANGELBRACKET:225:unknown:&#x27E8;
REGISTERED SIGN:226:unknown:&#x00AE;
COPYRIGHT:227:unknown:&#x00A9;
N-ARY SUMMATION:229:8721:&#x2211;
LARGE LEFT PARENTHESIS PART1:230:unknown:<udef_symbol num="0x230" description="left_paraenthesis part 1"/>
LARGE LEFT PARENTHESIS PART2:231:unknown:<udef_symbol num="0x231" description="left_parenthesis part 2"/>
LARGE LEFT PARENTHESIS PART3:232:unknown:<udef_symbol num="0x232" description="left_paranethesis part 3"/>
LARGE LEFT SQUARE BRACKET PART1:233:unknown:<udef_symbol num="0x233" description="right_square_bracket part 1"/>
LARGE LEFT SQUARE BRACKET PART2:234:unknown:<udef_symbol num="0x234" description="right_square_bracket part 2"/>
LARGE LEFT SQUARE BRACKET PART3:235:unknown:<udef_symbol num="0x235" description="right_square_bracket part 3"/>
LARGE LEFT BRACKET PART1:236:unknown:<udef_symbol num="0x236" description="right_bracket part 1"/>
LARGE LEFT BRACKET PART2:237:unknown:<udef_symbol num="0x237" description="right_bracket part 2"/>
LARGE LEFT BRACKET PART3:238:unknown:<udef_symbol num="0x238" description="right_bracket part 3"/>
DIVIDES:239:8739:&#x2223;
MATHEMATICAL RIGHT ANGLE BRACKET:241:unknown:27E9
INTEGRAL:242:unknown:&#x222B;
LARGE INTEGRAL PART 1:243:unknown:<integral part="1"/>
LARGE INTEGRAL PART 2:244:unknown:<integral part="2"/>
LARGE INTEGRAL PART 3:245:unknown:<integral part="3"/>
LARGE RIGHT PARENTHESIS PART1:246:unknown:<right_parenthesis part="1"/>
LARGE RIGHT PARENTHESIS PART2:247:unknown:<right_parenthesis part="2"/>
LARGE RIGHT PARENTHESIS PART3:248:unknown:<right_parenthesis part="3"/>
LARGE RIGHT SQUARE BRACKET PART1:249:unknown:<right_square_bracket part="1"/>
LARGE RIGHT SQUARE BRACKET PART2:250:unknown:<right_square_bracket part="2"/>
LARGE RIGHT SQUARE BRACKETPART3:251:unknown:<right_square_bracket part="3"/>
LARGE RIGHT BRACKET PART1:252:unknown:<right_bracket part="1"/>
LARGE RIGHT BRACKETPART2:253:unknown:<right_bracket part="2"/>
LARGE RIGHT BRACKETPART3:254:unknown:<right_bracket part="3"/>
DOUBLE ACUTE ACCENT:178:unknown:02DD
TRADEMARK SYMBOL:212:unknown:&#x2122;
TRADEMARK SYMBOL:228:unknown:&#x2122;
</SYMBOL_old>
<greek>
GREEK CAPITAL LETTER ALPHA:A:65:&#x0391;
GREEK CAPITAL LETTER BETA:B:66:&#x0392;
GREEK CAPITAL LETTER CHI:C:67:&#x03A7;
GREEK CAPITAL LETTER DELTA:D:68:&#x0394;
GREEK CAPITAL LETTER EPSILON:E:69:&#x0395;
GREEK CAPITAL LETTER PHI:F:70:&#x03A6;
GREEK CAPITAL LETTER GAMMA:G:71:&#x0393;
GREEK CAPITAL LETTER ETA:H:72:&#x0397;
GREEK CAPITAL LETTER IOTA:I:73:&#x0399;
GREEK THETA SYMBOL:J:74:&#x03D1;
GREEK CAPITAL LETTER KAPPA:K:75:&#x039A;
GREEK CAPITAL LETTER LAMDA:L:76:&#x039B;
GREEK CAPITAL LETTER MU:M:77:&#x039C;
GREEK CAPITAL LETTER NU:N:78:&#x039D;
GREEK CAPITAL LETTER OMICRON:O:79:&#x039F;
GREEK CAPITAL LETTER PI:P:80:&#x03A0;
GREEK CAPITAL LETTER THETA:T:81:&#x0398;
GREEK CAPITAL LETTER RHO:R:82:&#x03A1;
GREEK CAPITAL LETTER SIGMA:S:83:&#x03A3;
GREEK CAPITAL LETTER TAU:T:84:&#x03A4;
GREEK CAPITAL LETTER UPSILON:U:85:&#x03A5;
GREEK SMALL LETTER FINAL SIGMA:V:86:&#x03DA;
GREEK CAPITAL LETTER OMEGA:W:87:&#x03A9;
GREEK CAPITAL LETTER XI:X:88:&#x039E;
GREEK CAPITAL LETTER PSI:Y:89:&#x03A8;
GREEK CAPITAL LETTER ZETA:Z:90:&#x0396;
GREEK SMALL LETTER ALPHA:a:97:&#x03B1;
GREEK SMALL LETTER BETA:b:98:&#x03B2;
GREEK SMALL LETTER CHI:c:99:&#x03C7;
GREEK SMALL LETTER DELTA:d:100:&#x03B4;
GREEK SMALL LETTER EPSILON:e:101:&#x03B5;
GREEK SMALL LETTER PHI:f:102:&#x03C6;
GREEK SMALL LETTER GAMMA:g:103:&#x03B3;
GREEK SMALL LETTER ETA:h:104:&#x03B7;
GREEK SMALL LETTER IOTA:i:105:&#x03B9;
GREEK PHI SYMBOL:j:106:&#x03C6;
GREEK SMALL LETTER KAPPA:k:107:&#x03BA;
GREEK SMALL LETTER LAMDA:l:108:&#x03BB;
GREEK SMALL LETTER MU:m:109:&#x03BC;
GREEK SMALL LETTER NU:n:110:&#x03BD;
GREEK SMALL LETTER OMICRON:o:111:&#x03BF;
GREEK SMALL LETTER PI:p:112:&#x03C0;
GREEK SMALL LETTER THETA:q:113:&#x03B8;
GREEK SMALL LETTER RHO:r:114:&#x03C1;
GREEK SMALL LETTER SIGMA:s:115:&#x03C3;
GREEK SMALL LETTER TAU:t:116:&#x03C4;
GREEK SMALL LETTER UPSILON:u:117:&#x03C5;
GREEK PI SYMBOL:v:118:&#x03D6;
GREEK SMALL LETTER OMEGA:w:119:&#x03C9;
GREEK SMALL LETTER XI:x:120:&#x03BE;
GREEK SMALL LETTER PSI:y:121:&#x03C8;
GREEK SMALL LETTER ZETA:z:122:&#x03B6;
APROXTIMATELY EQUAL TO:@:unknown:&#x2245;
THERE EXISTS:$:unknown:2203
UP TACK:^:unknown:&#x22A5;
</greek>
<dingbats_old>
STAR OF DAVID:A:10017:&#x2721;
FOUR TEARDROP-SPOKED ASTERISK:B:10018:&#x2722;
FOUR BALLOON-SPOKED ASTERISK:C:10019:&#x2723;
HEAVY FOUR BALLOON-SPOKED ASTERISK:D:10020:&#x2724;
FOUR CLUB-SPOKED ASTERISK:E:10021:&#x2725;
BLACK FOUR POINTED STAR:F:10022:&#x2726;
WHITE FOUR POINTED STAR:G:10023:&#x2727;
BLACK STAR:H:9989:&#x2705;
STRESS OUTLINED WHITE STAR:I:10025:&#x2729;
CIRCLED WHITE STAR:J:10026:&#x272A;
OPEN CENTRE BLACK STAR:K:10027:&#x272B;
BLACK CENTRE WHITE STAR:L:10028:&#x272C;
OUTLINED BLACK STAR:M:10029:&#x272D;
HEAVY OUTLINED BLACK STAR:N:10030:&#x272E;
PINWHEEL STAR:O:10031:&#x272F;
SHADOWED WHITE STAR:P:10032:&#x2730;
HEAVY ASTERISK:Q:10033:&#x2731;
OPEN CENTRE ASTERISK:R:10034:&#x2732;
EIGHT SPOKED ASTERISK:S:10035:&#x2733;
EIGHT POINTED BLACK STAR:T:10036:&#x2734;
EIGHT POINTED PINWHEEL STAR:U:10037:&#x2735;
SIX POINTED BLACK STAR:V:10038:&#x2736;
EIGHT  POINTED RECTILINEAR BLACK STAR:W:10039:&#x2737;
HEAVY EIGHT POINTED RECTILINEAR BLACK STAR:X:10040:&#x2738;
TWELVE POINTED BLACK STAR:Y:10041:&#x2739;
SIXTEEN POINTED ASTERISK:Z:10042:&#x273A;
EIGHT PETALLED OUTLINED BLACK FLORETTE:a:10049:&#x2741;
CIRCLED OPEN CENTRE EIGHT POINTED STAR:b:10050:&#x2742;
HEAVY TEARDROP-SPOKED PINWHEEL ASTERISK:c:10051:&#x2743;
SNOWFLAKE:d:10052:&#x2744;
TIGHT TRIFOLIATE SNOWFLAKE:e:10053:&#x2745;
HEAVY CHEVRON SNOWFLAKE:f:10054:&#x2746;
SPARKLE:g:10055:&#x2747;
HEAVY SPARKLE:h:10056:&#x2748;
BALLOON-SPOKED ASTERISK:i:10057:&#x2749;
TEARDROP-SPOKED ASTERISK:j:10043:&#x273B;
HEAVY TEARDROP-SPOKED ASTERISK:k:10045:&#x273D;
BLACK CIRCLE:l:9679:&#x25CF;
SHADOWED WHITE CIRCLE:m:10061:&#x274D;
BLACK SQUARE:n:9632:&#x25A0;
LOWER RIGHT DROP-SHADOWED SQUARE:o:10063:&#x274F;
UPPER RIGHT DROP-SHADOWED WHITE SQUARE:p:10064:&#x2750;
LOWER RIGHT SHADOWED SQUARE:q:10065:&#x2751;
UPPER RIGHT SHADOWED WHITE SQUARE:r:10066:&#x2752;
BLACK UP-POINTING TRIANGLE:s:9650:&#x25B2;
BLACK DOWN-POINTING TRIANGLE:t:9660:&#x25BC;
BLACK DIAMOND:u:9670:&#x25C6;
BLACK DIAMOND MINUS WHITE X:v:10070:&#x2756;
RIGHT HALF BLACK CIRCLE:w:9479:&#x2507;
LIGHT VERTICAL BAR:x:10072:&#x2758;
MEDIUM VERTICAL BAR:y:10073:&#x2759;
HEAVY VERTICAL BAR:z:10074:&#x275A;
WHITE NIB:1:10001:&#x2711;
BLACK NIB:2:10002:&#x2712;
CHECKMARK:3:10003:&#x2713;
HEAVY CHECKMARK:4:10004:&#x2714;
MULTIPLICATION X:5:10005:&#x2715;
HEAVY MULTIPLICATION X:6:10006:&#x2716;
BALLOT X:7:10007:&#x2717;
HEAVY BALLOT X:8:10008:&#x2718;
OUTLINED GREEK CROSS:9:10009:&#x2719;
UPPER RIGHT PENCIL:0:10000:&#x2710;
UPPER BLADE SCISSORS:!:9985:&#x2701;
MALTESE CROSS:@:10016:&#x2720;
LOWER BLADE SCISSORS:#:9987:&#x2703;
WHITE SCISSORS:$:9988:&#x2704;
BLACK TELEPHONE:%:9742:&#x260E;
SIX PETALLED BLACK AND WHITE FLORETTE:^:10046:&#x273E;
TELEPHONE LOCATION SIGN:&:9990:&#x2706;
BLACK RIGHT POINTING INDEX:*:9755:&#x261B;
AIRPLANE:(:9992:&#x2708;
ENVELOPE:):9993:&#x2709;
HEAVY GREK CROSS:\\colon:10010:&#x271A;
OUTLINED LATIN CROSS:?:10015:&#x271F;
PENCIL:/:9999:&#x270F;
OPEN CENTRE TEARDROP-SPOKED ASTERISK:\\:10044:&#x273C;
WHITE RIGHT POINTING INDEX:+:9758:&#x261E;
WRITING HAND:-:9997:&#x270D;
LATIN CROSS:=:10013:&#x271D;
HEAVY DOUBLE COMMA QUOTATION MARK ORNAMENT:~:10078:&#x275E;
</dingbats_old>
<ms_set_old>
LEFT DOUBLE QUOTATION MARK:LDBLQUOTE :8220:&#x201C;
RIGHT DOUBLE QUOTATION MARK:RDBLQUOTE :8221:&#x201D;
RIGHT SINGLE QUOTATION MARK:RQUOTE :8217:&#x2019;
LEFT SINGLE QUOTATION MARK:LQUOTE :8216:&#x2018;
EM DASH:EMDASH :8212:&#x2014;
EN DASH:ENDASH :8211:&#x2013;
MIDDLE DOT:BULLET :183:&#x00B7;
NO-BREAK SPACE:~ :167:&#x00A7;
HORIZONTAL TABULATION:TAB :9:&#x0009;
</ms_set_old>
<single_set>
NULL:\xef:0:&#xnull;
</single_set>
</unused>
'''
